
This set of patches orginates here:

  http://elinux.org/Linux_Tiny

More in particular, it is downloadable here:

  http://elinux.org/images/0/0e/Tiny-quilt-2.6.22.1-1.tar.gz

Signed-off-by: Ward Vandewege <ward@gnu.org>

Index: packages/kernel/patches/tiny_m57sli/tiny/nopanic-mips-fix.patch
===================================================================
--- packages/kernel/patches/tiny_m57sli/tiny/nopanic-mips-fix.patch	(revision 0)
+++ packages/kernel/patches/tiny_m57sli/tiny/nopanic-mips-fix.patch	(revision 0)
@@ -0,0 +1,50 @@
+    Although no target in mips seems to use function except_vec0/1_generic
+    that uses PANIC_PIC, PANIC_PIC looks for symbol panic.
+    This patch corrects error when CONFIG_PANIC=n or CONFIG_FULL_PANIC=n.
+  
+
+---
+ arch/mips/kernel/genex.S |   23 	23 +	0 -	0 !
+ 1 files changed, 23 insertions(+)
+
+Index: linux-2.6.22.1/arch/mips/kernel/genex.S
+===================================================================
+--- linux-2.6.22.1.orig/arch/mips/kernel/genex.S
++++ linux-2.6.22.1/arch/mips/kernel/genex.S
+@@ -21,6 +21,16 @@
+ #include <asm/war.h>
+ #include <asm/page.h>
+ 
++#ifndef CONFIG_PANIC
++#define PANIC_PIC(msg)					\
++		.set push;				\
++		.set	reorder;			\
++		PTR_LA	a0,8f;				\
++9:		b	9b;				\
++		.set	pop;				\
++		TEXT(msg)
++#else /* CONFIG_PANIC=y */
++#ifdef CONFIG_FULL_PANIC
+ #define PANIC_PIC(msg)					\
+ 		.set push;				\
+ 		.set	reorder;			\
+@@ -31,6 +41,19 @@
+ 9:		b	9b;				\
+ 		.set	pop;				\
+ 		TEXT(msg)
++#else /* CONFIG_FULL_PANIC=n */
++#define PANIC_PIC(msg)					\
++		.set push;				\
++		.set	reorder;			\
++		PTR_LA	a0,8f;				\
++		.set	noat;				\
++		PTR_LA	AT, tiny_panic;			\
++		jr	AT;				\
++9:		b	9b;				\
++		.set	pop;				\
++		TEXT(msg)
++#endif /* CONFIG_FULL_PANIC=n */
++#endif /* CONFIG_PANIC=y */
+ 
+ 	__INIT
+ 
Index: packages/kernel/patches/tiny_m57sli/tiny/inflate-constdef.patch
===================================================================
--- packages/kernel/patches/tiny_m57sli/tiny/inflate-constdef.patch	(revision 0)
+++ packages/kernel/patches/tiny_m57sli/tiny/inflate-constdef.patch	(revision 0)
@@ -0,0 +1,69 @@
+inflate: small constant tidy-up
+
+Index: linux-2.6.16.19/lib/inflate.c
+===================================================================
+--- linux-2.6.16.19.orig/lib/inflate.c	2006-06-23 17:06:36.000000000 -0700
++++ linux-2.6.16.19/lib/inflate.c	2006-06-23 17:06:36.000000000 -0700
+@@ -52,7 +52,7 @@
+    The Huffman codes themselves are decoded using a multi-level table
+    lookup, in order to maximize the speed of decoding plus the speed of
+    building the decoding tables.  See the comments below that precede the
+-   lbits and dbits tuning parameters.
++   LBITS and dbits tuning parameters.
+  */
+ 
+ /*
+@@ -352,9 +352,9 @@
+    the longer codes.  The time it costs to decode the longer codes is
+    then traded against the time it takes to make longer tables.
+ 
+-   This results of this trade are in the variables lbits and dbits
+-   below.  lbits is the number of bits the first level table for literal/
+-   length codes can decode in one step, and dbits is the same thing for
++   This results of this trade are in the variables LBITS and DBITS
++   below.  LBITS is the number of bits the first level table for literal/
++   length codes can decode in one step, and DBITS is the same thing for
+    the distance codes.  Subsequent tables are also less than or equal to
+    those sizes.  These values may be adjusted either when all of the
+    codes are shorter than that, in which case the longest code length in
+@@ -367,17 +367,15 @@
+    codes 286 possible values, or in a flat code, a little over eight
+    bits.  The distance table codes 30 possible values, or a little less
+    than five bits, flat.  The optimum values for speed end up being
+-   about one bit more than those, so lbits is 8+1 and dbits is 5+1.
++   about one bit more than those, so LBITS is 8+1 and DBITS is 5+1.
+    The optimum values may differ though from machine to machine, and
+    possibly even between compilers.  Your mileage may vary.
+  */
+ 
+-static const int lbits = 9;	/* bits in base literal/length lookup table */
+-static const int dbits = 6;	/* bits in base distance lookup table */
+-
+-/* If BMAX needs to be larger than 16, then h and x[] should be u32. */
+-#define BMAX 16		/* maximum bit length of any code (16 for explode) */
+-#define N_MAX 288	/* maximum number of codes in any set */
++#define LBITS 9 /* bits in base literal/length lookup table */
++#define DBITS 6 /* bits in base distance lookup table */
++#define BMAX 16 /* maximum bit length of any code (16 for explode) */
++#define N_MAX 288 /* maximum number of codes in any set */
+ 
+ /*
+  * huft-build - build a huffman decoding table
+@@ -833,7 +831,7 @@
+ 	huft_free(tl);
+ 
+ 	/* build the decoding tables for literal/length and distance codes */
+-	bl = lbits;
++	bl = LBITS;
+ 	if ((i = huft_build(ll, nl, 257, cplens, cplext, &tl, &bl))) {
+ 		if (i == 1) {
+ 			io->error("incomplete literal tree");
+@@ -842,7 +840,7 @@
+ 		return i;	/* incomplete code set */
+ 	}
+ 
+-	bd = dbits;
++	bd = DBITS;
+ 	if ((i = huft_build(ll + nl, nd, 0, cpdist, cpdext, &td, &bd))) {
+ 		if (i == 1) {
+ 			io->error("incomplete distance tree");
Index: packages/kernel/patches/tiny_m57sli/tiny/inflate-makecrc.patch
===================================================================
--- packages/kernel/patches/tiny_m57sli/tiny/inflate-makecrc.patch	(revision 0)
+++ packages/kernel/patches/tiny_m57sli/tiny/inflate-makecrc.patch	(revision 0)
@@ -0,0 +1,564 @@
+inflate: move CRC calculation
+
+Each inflate user was doing its own open-coded CRC calculation and
+initializing its own CRC table. This is now hidden inside
+lib/inflate.c
+
+Signed-off-by: Matt Mackall <mpm@selenic.com>
+
+Index: linux-2.6.16.19/arch/alpha/boot/misc.c
+===================================================================
+--- linux-2.6.16.19.orig/arch/alpha/boot/misc.c	2006-06-23 17:06:32.000000000 -0700
++++ linux-2.6.16.19/arch/alpha/boot/misc.c	2006-06-23 17:06:32.000000000 -0700
+@@ -80,22 +80,19 @@
+ }
+ 
+ /* ===========================================================================
+- * Write the output window window[0..outcnt-1] and update crc and bytes_out.
++ * Write the output window window[0..outcnt-1] and update bytes_out.
+  * (Used for the decompressed data only.)
+  */
+ void flush_window(void)
+ {
+-	ulg c = crc;
+ 	unsigned n;
+ 	uch *in, *out, ch;
+ 
+ 	in = window;
+ 	out = &output_data[output_ptr];
+-	for (n = 0; n < outcnt; n++) {
++	for (n = 0; n < outcnt; n++)
+ 		ch = *out++ = *in++;
+-		c = crc_32_tab[((int)c ^ ch) & 0xff] ^ (c >> 8);
+-	}
+-	crc = c;
++
+ 	bytes_out += (ulg)outcnt;
+ 	output_ptr += (ulg)outcnt;
+ 	outcnt = 0;
+@@ -129,7 +126,6 @@
+ 	/* put in temp area to reduce initial footprint */
+ 	window = malloc(WSIZE);
+ 
+-	makecrc();
+ /*	puts("Uncompressing Linux..."); */
+ 	gunzip();
+ /*	puts(" done, booting the kernel.\n"); */
+Index: linux-2.6.16.19/arch/arm/boot/compressed/misc.c
+===================================================================
+--- linux-2.6.16.19.orig/arch/arm/boot/compressed/misc.c	2006-06-23 17:06:32.000000000 -0700
++++ linux-2.6.16.19/arch/arm/boot/compressed/misc.c	2006-06-23 17:06:32.000000000 -0700
+@@ -104,22 +104,19 @@
+ }
+ 
+ /* ===========================================================================
+- * Write the output window window[0..outcnt-1] and update crc and bytes_out.
++ * Write the output window window[0..outcnt-1] and update bytes_out.
+  * (Used for the decompressed data only.)
+  */
+ void flush_window(void)
+ {
+-	ulg c = crc;
+ 	unsigned n;
+ 	uch *in, *out, ch;
+ 
+ 	in = window;
+ 	out = &output_data[output_ptr];
+-	for (n = 0; n < outcnt; n++) {
++	for (n = 0; n < outcnt; n++)
+ 		ch = *out++ = *in++;
+-		c = crc_32_tab[((int)c ^ ch) & 0xff] ^ (c >> 8);
+-	}
+-	crc = c;
++
+ 	bytes_out += (ulg)outcnt;
+ 	output_ptr += (ulg)outcnt;
+ 	outcnt = 0;
+@@ -154,7 +151,6 @@
+ 
+ 	arch_decomp_setup();
+ 
+-	makecrc();
+ 	putstr("Uncompressing Linux...");
+ 	gunzip();
+ 	putstr(" done, booting the kernel.\n");
+@@ -168,7 +164,6 @@
+ {
+ 	output_data = output_buffer;
+ 
+-	makecrc();
+ 	putstr("Uncompressing Linux...");
+ 	gunzip();
+ 	putstr("done.\n");
+Index: linux-2.6.16.19/arch/arm26/boot/compressed/misc.c
+===================================================================
+--- linux-2.6.16.19.orig/arch/arm26/boot/compressed/misc.c	2006-06-23 17:06:32.000000000 -0700
++++ linux-2.6.16.19/arch/arm26/boot/compressed/misc.c	2006-06-23 17:06:32.000000000 -0700
+@@ -89,22 +89,19 @@
+ }
+ 
+ /* ===========================================================================
+- * Write the output window window[0..outcnt-1] and update crc and bytes_out.
++ * Write the output window window[0..outcnt-1] and update bytes_out.
+  * (Used for the decompressed data only.)
+  */
+ void flush_window(void)
+ {
+-	ulg c = crc;
+ 	unsigned n;
+ 	uch *in, *out, ch;
+ 
+ 	in = window;
+ 	out = &output_data[output_ptr];
+-	for (n = 0; n < outcnt; n++) {
++	for (n = 0; n < outcnt; n++)
+ 		ch = *out++ = *in++;
+-		c = crc_32_tab[((int)c ^ ch) & 0xff] ^ (c >> 8);
+-	}
+-	crc = c;
++
+ 	bytes_out += (ulg)outcnt;
+ 	output_ptr += (ulg)outcnt;
+ 	outcnt = 0;
+@@ -135,7 +132,6 @@
+ 
+ 	arch_decomp_setup();
+ 
+-	makecrc();
+ 	puts("Uncompressing Linux...");
+ 	gunzip();
+ 	puts(" done, booting the kernel.\n");
+@@ -149,7 +145,6 @@
+ {
+ 	output_data = output_buffer;
+ 
+-	makecrc();
+ 	puts("Uncompressing Linux...");
+ 	gunzip();
+ 	puts("done.\n");
+Index: linux-2.6.16.19/arch/cris/arch-v10/boot/compressed/misc.c
+===================================================================
+--- linux-2.6.16.19.orig/arch/cris/arch-v10/boot/compressed/misc.c	2006-06-23 17:06:32.000000000 -0700
++++ linux-2.6.16.19/arch/cris/arch-v10/boot/compressed/misc.c	2006-06-23 17:06:32.000000000 -0700
+@@ -95,24 +95,21 @@
+ }
+ 
+ /* ===========================================================================
+- * Write the output window window[0..outcnt-1] and update crc and bytes_out.
++ * Write the output window window[0..outcnt-1] and update bytes_out.
+  * (Used for the decompressed data only.)
+  */
+ 
+ static void
+ flush_window()
+ {
+-    ulg c = crc;         /* temporary variable */
+     unsigned n;
+     uch *in, *out, ch;
+-    
++
+     in = window;
+-    out = &output_data[output_ptr]; 
+-    for (n = 0; n < outcnt; n++) {
++    out = &output_data[output_ptr];
++    for (n = 0; n < outcnt; n++)
+ 	    ch = *out++ = *in++;
+-	    c = crc_32_tab[((int)c ^ ch) & 0xff] ^ (c >> 8);
+-    }
+-    crc = c;
++
+     bytes_out += (ulg)outcnt;
+     output_ptr += (ulg)outcnt;
+     outcnt = 0;
+@@ -167,8 +164,6 @@
+ 
+ 	setup_normal_output_buffer();
+ 
+-	makecrc();
+-
+ 	__asm__ volatile ("move vr,%0" : "=rm" (revision));
+ 	if (revision < 10)
+ 	{
+Index: linux-2.6.16.19/arch/cris/arch-v32/boot/compressed/misc.c
+===================================================================
+--- linux-2.6.16.19.orig/arch/cris/arch-v32/boot/compressed/misc.c	2006-06-23 17:06:32.000000000 -0700
++++ linux-2.6.16.19/arch/cris/arch-v32/boot/compressed/misc.c	2006-06-23 17:06:32.000000000 -0700
+@@ -109,24 +109,21 @@
+ }
+ 
+ /* ===========================================================================
+- * Write the output window window[0..outcnt-1] and update crc and bytes_out.
++ * Write the output window window[0..outcnt-1] and update bytes_out.
+  * (Used for the decompressed data only.)
+  */
+ 
+ static void
+ flush_window()
+ {
+-    ulg c = crc;         /* temporary variable */
+     unsigned n;
+     uch *in, *out, ch;
+ 
+     in = window;
+     out = &output_data[output_ptr];
+-    for (n = 0; n < outcnt; n++) {
++    for (n = 0; n < outcnt; n++)
+ 	    ch = *out++ = *in++;
+-	    c = crc_32_tab[((int)c ^ ch) & 0xff] ^ (c >> 8);
+-    }
+-    crc = c;
++
+     bytes_out += (ulg)outcnt;
+     output_ptr += (ulg)outcnt;
+     outcnt = 0;
+@@ -212,8 +209,6 @@
+ 
+ 	setup_normal_output_buffer();
+ 
+-	makecrc();
+-
+ 	__asm__ volatile ("move $vr,%0" : "=rm" (revision));
+ 	if (revision < 32)
+ 	{
+Index: linux-2.6.16.19/arch/i386/boot/compressed/misc.c
+===================================================================
+--- linux-2.6.16.19.orig/arch/i386/boot/compressed/misc.c	2006-06-23 17:06:32.000000000 -0700
++++ linux-2.6.16.19/arch/i386/boot/compressed/misc.c	2006-06-23 17:06:32.000000000 -0700
+@@ -150,22 +150,19 @@
+ }
+ 
+ /* ===========================================================================
+- * Write the output window window[0..outcnt-1] and update crc and bytes_out.
++ * Write the output window window[0..outcnt-1] and update bytes_out.
+  * (Used for the decompressed data only.)
+  */
+ static void flush_window_low(void)
+ {
+-    ulg c = crc;         /* temporary variable */
+     unsigned n;
+     uch *in, *out, ch;
+-    
++
+     in = window;
+-    out = &output_data[output_ptr]; 
+-    for (n = 0; n < outcnt; n++) {
++    out = &output_data[output_ptr];
++    for (n = 0; n < outcnt; n++)
+ 	    ch = *out++ = *in++;
+-	    c = crc_32_tab[((int)c ^ ch) & 0xff] ^ (c >> 8);
+-    }
+-    crc = c;
++
+     bytes_out += (ulg)outcnt;
+     output_ptr += (ulg)outcnt;
+     outcnt = 0;
+@@ -173,16 +170,14 @@
+ 
+ static void flush_window_high(void)
+ {
+-    ulg c = crc;         /* temporary variable */
+     unsigned n;
+     uch *in,  ch;
+     in = window;
+     for (n = 0; n < outcnt; n++) {
+ 	ch = *output_data++ = *in++;
+ 	if ((ulg)output_data == low_buffer_end) output_data=high_buffer_start;
+-	c = crc_32_tab[((int)c ^ ch) & 0xff] ^ (c >> 8);
+     }
+-    crc = c;
++
+     bytes_out += (ulg)outcnt;
+     outcnt = 0;
+ }
+@@ -281,7 +276,6 @@
+ 	if (free_mem_ptr < 0x100000) setup_normal_output_buffer();
+ 	else setup_output_buffer_if_we_run_high(mv);
+ 
+-	makecrc();
+ 	putstr("Uncompressing Linux... ");
+ 	gunzip();
+ 	putstr("Ok, booting the kernel.\n");
+Index: linux-2.6.16.19/arch/m32r/boot/compressed/misc.c
+===================================================================
+--- linux-2.6.16.19.orig/arch/m32r/boot/compressed/misc.c	2006-06-23 17:06:32.000000000 -0700
++++ linux-2.6.16.19/arch/m32r/boot/compressed/misc.c	2006-06-23 17:06:32.000000000 -0700
+@@ -70,22 +70,19 @@
+ }
+ 
+ /* ===========================================================================
+- * Write the output window window[0..outcnt-1] and update crc and bytes_out.
++ * Write the output window window[0..outcnt-1] and update bytes_out.
+  * (Used for the decompressed data only.)
+  */
+ static void flush_window(void)
+ {
+-    ulg c = crc;         /* temporary variable */
+     unsigned n;
+     uch *in, *out, ch;
+ 
+     in = window;
+     out = &output_data[output_ptr];
+-    for (n = 0; n < outcnt; n++) {
++    for (n = 0; n < outcnt; n++)
+ 	    ch = *out++ = *in++;
+-	    c = crc_32_tab[((int)c ^ ch) & 0xff] ^ (c >> 8);
+-    }
+-    crc = c;
++
+     bytes_out += (ulg)outcnt;
+     output_ptr += (ulg)outcnt;
+     outcnt = 0;
+@@ -112,7 +109,6 @@
+ 	input_data = zimage_data;
+ 	input_len = zimage_len;
+ 
+-	makecrc();
+ 	puts("Uncompressing Linux... ");
+ 	gunzip();
+ 	puts("Ok, booting the kernel.\n");
+Index: linux-2.6.16.19/arch/sh/boot/compressed/misc.c
+===================================================================
+--- linux-2.6.16.19.orig/arch/sh/boot/compressed/misc.c	2006-06-23 17:06:32.000000000 -0700
++++ linux-2.6.16.19/arch/sh/boot/compressed/misc.c	2006-06-23 17:06:32.000000000 -0700
+@@ -98,22 +98,19 @@
+ }
+ 
+ /* ===========================================================================
+- * Write the output window window[0..outcnt-1] and update crc and bytes_out.
++ * Write the output window window[0..outcnt-1] and update bytes_out.
+  * (Used for the decompressed data only.)
+  */
+ static void flush_window(void)
+ {
+-    ulg c = crc;         /* temporary variable */
+     unsigned n;
+     uch *in, *out, ch;
+ 
+     in = window;
+     out = &output_data[output_ptr];
+-    for (n = 0; n < outcnt; n++) {
++    for (n = 0; n < outcnt; n++)
+ 	    ch = *out++ = *in++;
+-	    c = crc_32_tab[((int)c ^ ch) & 0xff] ^ (c >> 8);
+-    }
+-    crc = c;
++
+     bytes_out += (ulg)outcnt;
+     output_ptr += (ulg)outcnt;
+     outcnt = 0;
+@@ -139,7 +136,6 @@
+ 	free_mem_ptr = (unsigned long)&_end;
+ 	free_mem_end_ptr = free_mem_ptr + HEAP_SIZE;
+ 
+-	makecrc();
+ 	puts("Uncompressing Linux... ");
+ 	gunzip();
+ 	puts("Ok, booting the kernel.\n");
+Index: linux-2.6.16.19/arch/sh64/boot/compressed/misc.c
+===================================================================
+--- linux-2.6.16.19.orig/arch/sh64/boot/compressed/misc.c	2006-06-23 17:06:32.000000000 -0700
++++ linux-2.6.16.19/arch/sh64/boot/compressed/misc.c	2006-06-23 17:06:32.000000000 -0700
+@@ -78,22 +78,19 @@
+ }
+ 
+ /* ===========================================================================
+- * Write the output window window[0..outcnt-1] and update crc and bytes_out.
++ * Write the output window window[0..outcnt-1] and update bytes_out.
+  * (Used for the decompressed data only.)
+  */
+ static void flush_window(void)
+ {
+-	ulg c = crc;		/* temporary variable */
+ 	unsigned n;
+ 	uch *in, *out, ch;
+ 
+ 	in = window;
+ 	out = &output_data[output_ptr];
+-	for (n = 0; n < outcnt; n++) {
++	for (n = 0; n < outcnt; n++)
+ 		ch = *out++ = *in++;
+-		c = crc_32_tab[((int) c ^ ch) & 0xff] ^ (c >> 8);
+-	}
+-	crc = c;
++
+ 	bytes_out += (ulg) outcnt;
+ 	output_ptr += (ulg) outcnt;
+ 	outcnt = 0;
+@@ -119,7 +116,6 @@
+ 	free_mem_ptr = (unsigned long) &_end;
+ 	free_mem_end_ptr = free_mem_ptr + HEAP_SIZE;
+ 
+-	makecrc();
+ 	puts("Uncompressing Linux... ");
+ 	cache_control(CACHE_ENABLE);
+ 	gunzip();
+Index: linux-2.6.16.19/arch/x86_64/boot/compressed/misc.c
+===================================================================
+--- linux-2.6.16.19.orig/arch/x86_64/boot/compressed/misc.c	2006-06-23 17:06:32.000000000 -0700
++++ linux-2.6.16.19/arch/x86_64/boot/compressed/misc.c	2006-06-23 17:06:32.000000000 -0700
+@@ -140,22 +140,19 @@
+ }
+ 
+ /* ===========================================================================
+- * Write the output window window[0..outcnt-1] and update crc and bytes_out.
++ * Write the output window window[0..outcnt-1] and update bytes_out.
+  * (Used for the decompressed data only.)
+  */
+ static void flush_window_low(void)
+ {
+-    ulg c = crc;         /* temporary variable */
+     unsigned n;
+     uch *in, *out, ch;
+-    
++
+     in = window;
+-    out = &output_data[output_ptr]; 
+-    for (n = 0; n < outcnt; n++) {
++    out = &output_data[output_ptr];
++    for (n = 0; n < outcnt; n++)
+ 	    ch = *out++ = *in++;
+-	    c = crc_32_tab[((int)c ^ ch) & 0xff] ^ (c >> 8);
+-    }
+-    crc = c;
++
+     bytes_out += (ulg)outcnt;
+     output_ptr += (ulg)outcnt;
+     outcnt = 0;
+@@ -163,16 +160,14 @@
+ 
+ static void flush_window_high(void)
+ {
+-    ulg c = crc;         /* temporary variable */
+     unsigned n;
+     uch *in,  ch;
+     in = window;
+     for (n = 0; n < outcnt; n++) {
+ 	ch = *output_data++ = *in++;
+ 	if ((ulg)output_data == low_buffer_end) output_data=high_buffer_start;
+-	c = crc_32_tab[((int)c ^ ch) & 0xff] ^ (c >> 8);
+     }
+-    crc = c;
++
+     bytes_out += (ulg)outcnt;
+     outcnt = 0;
+ }
+@@ -260,7 +255,6 @@
+ 	if (free_mem_ptr < 0x100000) setup_normal_output_buffer();
+ 	else setup_output_buffer_if_we_run_high(mv);
+ 
+-	makecrc();
+ 	putstr(".\nDecompressing Linux...");
+ 	gunzip();
+ 	putstr("done.\nBooting the kernel.\n");
+Index: linux-2.6.16.19/init/do_mounts_rd.c
+===================================================================
+--- linux-2.6.16.19.orig/init/do_mounts_rd.c	2006-06-23 17:06:32.000000000 -0700
++++ linux-2.6.16.19/init/do_mounts_rd.c	2006-06-23 17:06:32.000000000 -0700
+@@ -325,15 +325,14 @@
+ }
+ 
+ /* ===========================================================================
+- * Write the output window window[0..outcnt-1] and update crc and bytes_out.
++ * Write the output window window[0..outcnt-1] and update bytes_out.
+  * (Used for the decompressed data only.)
+  */
+ static void __init flush_window(void)
+ {
+-    ulg c = crc;         /* temporary variable */
+     unsigned n, written;
+     uch *in, ch;
+-    
++
+     written = sys_write(crd_outfd, window, outcnt);
+     if (written != outcnt && unzip_error == 0) {
+ 	printk(KERN_ERR "RAMDISK: incomplete write (%d != %d) %ld\n",
+@@ -341,11 +340,9 @@
+ 	unzip_error = 1;
+     }
+     in = window;
+-    for (n = 0; n < outcnt; n++) {
++    for (n = 0; n < outcnt; n++)
+ 	    ch = *in++;
+-	    c = crc_32_tab[((int)c ^ ch) & 0xff] ^ (c >> 8);
+-    }
+-    crc = c;
++
+     bytes_out += (ulg)outcnt;
+     outcnt = 0;
+ }
+@@ -366,7 +363,6 @@
+ 	outcnt = 0;		/* bytes in output buffer */
+ 	exit_code = 0;
+ 	bytes_out = 0;
+-	crc = (ulg)0xffffffffL; /* shift register contents */
+ 
+ 	crd_infd = in_fd;
+ 	crd_outfd = out_fd;
+@@ -381,7 +377,6 @@
+ 		kfree(inbuf);
+ 		return -1;
+ 	}
+-	makecrc();
+ 	result = gunzip();
+ 	if (unzip_error)
+ 		result = 1;
+Index: linux-2.6.16.19/init/initramfs.c
+===================================================================
+--- linux-2.6.16.19.orig/init/initramfs.c	2006-06-23 17:06:32.000000000 -0700
++++ linux-2.6.16.19/init/initramfs.c	2006-06-23 17:06:32.000000000 -0700
+@@ -360,22 +360,19 @@
+ #include "../lib/inflate.c"
+ 
+ /* ===========================================================================
+- * Write the output window window[0..outcnt-1] and update crc and bytes_out.
++ * Write the output window window[0..outcnt-1] and update bytes_out.
+  * (Used for the decompressed data only.)
+  */
+ static void __init flush_window(void)
+ {
+-	ulg c = crc;         /* temporary variable */
+ 	unsigned n;
+ 	uch *in, ch;
+ 
+ 	flush_buffer(window, outcnt);
+ 	in = window;
+-	for (n = 0; n < outcnt; n++) {
++	for (n = 0; n < outcnt; n++)
+ 		ch = *in++;
+-		c = crc_32_tab[((int)c ^ ch) & 0xff] ^ (c >> 8);
+-	}
+-	crc = c;
++
+ 	bytes_out += (ulg)outcnt;
+ 	outcnt = 0;
+ }
+@@ -414,8 +411,6 @@
+ 		inptr = 0;
+ 		outcnt = 0;		/* bytes in output buffer */
+ 		bytes_out = 0;
+-		crc = (ulg)0xffffffffL; /* shift register contents */
+-		makecrc();
+ 		gunzip();
+ 		if (state != Reset)
+ 			error("junk in gzipped archive");
+Index: linux-2.6.16.19/lib/inflate.c
+===================================================================
+--- linux-2.6.16.19.orig/lib/inflate.c	2006-06-23 17:06:32.000000000 -0700
++++ linux-2.6.16.19/lib/inflate.c	2006-06-23 17:06:32.000000000 -0700
+@@ -150,7 +150,6 @@
+ #endif
+ 
+ static u32 crc_32_tab[256];
+-static u32 crc;		/* dummy var until users get cleaned up */
+ #define CRCPOLY_LE 0xedb88320
+ 
+ /* Huffman code lookup table entry--this entry is four bytes for machines
+@@ -995,6 +994,8 @@
+ 	io.opos = io.bits = io.buf = 0;
+ 	io.crc = 0xffffffffUL;
+ 
++	makecrc(); /* initialize the CRC table */
++
+ 	magic[0] = get_byte();
+ 	magic[1] = get_byte();
+ 	method = get_byte();
Index: packages/kernel/patches/tiny_m57sli/tiny/nopanic.patch
===================================================================
--- packages/kernel/patches/tiny_m57sli/tiny/nopanic.patch	(revision 0)
+++ packages/kernel/patches/tiny_m57sli/tiny/nopanic.patch	(revision 0)
@@ -0,0 +1,100 @@
+
+configurable no-op of panic()
+
+
+ include/linux/kernel.h |    6 	6 +	0 -	0 !
+ init/Kconfig           |    9 	9 +	0 -	0 !
+ init/main.c            |    1 	1 +	0 -	0 !
+ kernel/panic.c         |   10 	6 +	4 -	0 !
+ 4 files changed, 22 insertions(+), 4 deletions(-)
+
+Index: linux-2.6.22.1/include/linux/kernel.h
+===================================================================
+--- linux-2.6.22.1.orig/include/linux/kernel.h
++++ linux-2.6.22.1/include/linux/kernel.h
+@@ -105,6 +105,11 @@ extern int cond_resched(void);
+ 
+ extern struct atomic_notifier_head panic_notifier_list;
+ extern long (*panic_blink)(long time);
++#ifndef CONFIG_PANIC
++NORET_TYPE static inline void panic(const char * fmt, ...)
++	__attribute__ ((NORET_AND format (printf, 1, 2)));
++NORET_TYPE static inline void panic(const char * fmt, ...) {}
++#else
+ #ifdef CONFIG_FULL_PANIC
+ NORET_TYPE void panic(const char * fmt, ...)
+ 	__attribute__ ((NORET_AND format (printf, 1, 2)));
+@@ -112,6 +117,7 @@ NORET_TYPE void panic(const char * fmt, 
+ #define panic(fmt, ...) tiny_panic(0, ## __VA_ARGS__)
+ NORET_TYPE void tiny_panic(int a, ...) ATTRIB_NORET;
+ #endif
++#endif
+ extern void oops_enter(void);
+ extern void oops_exit(void);
+ extern int oops_may_print(void);
+Index: linux-2.6.22.1/init/Kconfig
+===================================================================
+--- linux-2.6.22.1.orig/init/Kconfig
++++ linux-2.6.22.1/init/Kconfig
+@@ -467,7 +467,16 @@ config ELF_CORE
+ 	help
+ 	  Enable support for generating core dumps. Disabling saves about 4k.
+ 
++config PANIC
++	default y
++	bool "Enable panic reporting code" if EMBEDDED
++	help
++	  Disabling this completely removes panic handling code.
++          Warning: this can result in data loss if a panic condition
++          occurs, as the kernel may ignore the condition entirely.
++
+ config FULL_PANIC
++        depends PANIC
+ 	default y
+ 	bool "Full panic reporting data" if EMBEDDED
+ 	help
+Index: linux-2.6.22.1/init/main.c
+===================================================================
+--- linux-2.6.22.1.orig/init/main.c
++++ linux-2.6.22.1/init/main.c
+@@ -782,6 +782,7 @@ static int noinline init_post(void)
+ 	run_init_process("/bin/sh");
+ 
+ 	panic("No init found.  Try passing init= option to kernel.");
++	return 0;
+ }
+ 
+ static int __init kernel_init(void * unused)
+Index: linux-2.6.22.1/kernel/panic.c
+===================================================================
+--- linux-2.6.22.1.orig/kernel/panic.c
++++ linux-2.6.22.1/kernel/panic.c
+@@ -39,15 +39,16 @@ static int __init panic_setup(char *str)
+ }
+ __setup("panic=", panic_setup);
+ 
++/* Returns how long it waited in ms */
++long (*panic_blink)(long time);
++EXPORT_SYMBOL(panic_blink);
++
++#ifdef CONFIG_PANIC
+ static long no_blink(long time)
+ {
+ 	return 0;
+ }
+ 
+-/* Returns how long it waited in ms */
+-long (*panic_blink)(long time);
+-EXPORT_SYMBOL(panic_blink);
+-
+ /**
+  *	panic - halt the system
+  *	@fmt: The text string to print
+@@ -160,6 +161,7 @@ EXPORT_SYMBOL(panic);
+ #else
+ EXPORT_SYMBOL(tiny_panic);
+ #endif
++#endif
+ 
+ /**
+  *	print_tainted - return a string to represent the kernel taint state.
Index: packages/kernel/patches/tiny_m57sli/tiny/tiny-crc.patch
===================================================================
--- packages/kernel/patches/tiny_m57sli/tiny/tiny-crc.patch	(revision 0)
+++ packages/kernel/patches/tiny_m57sli/tiny/tiny-crc.patch	(revision 0)
@@ -0,0 +1,91 @@
+
+Unroll crc32 calculation on small systems
+
+
+ init/Kconfig    |   14 	14 +	0 -	0 !
+ lib/Makefile    |    2 	2 +	0 -	0 !
+ lib/crc32.c     |    2 	2 +	0 -	0 !
+ lib/crc32defs.h |    9 	8 +	1 -	0 !
+ 4 files changed, 26 insertions(+), 1 deletion(-)
+
+Index: linux-2.6.22.1/init/Kconfig
+===================================================================
+--- linux-2.6.22.1.orig/init/Kconfig
++++ linux-2.6.22.1/init/Kconfig
+@@ -566,6 +566,16 @@ config SHMEM
+ 	  option replaces shmem and tmpfs with the much simpler ramfs code,
+ 	  which may be appropriate on small systems without swap.
+ 
++config CRC32_TABLES
++	depends CRC32
++	default y
++	bool "Calculate CRC32 with tables" if EMBEDDED
++	help
++	  This option enables use of tables for CRC calculation. Disabling
++          reduces kernel size by about 2K. This may actually result in
++          faster code on machines with limited memory bandwidth.
++
++
+ config VM_EVENT_COUNTERS
+ 	default y
+ 	bool "Enable VM event counters for /proc/vmstat" if EMBEDDED
+@@ -644,6 +654,10 @@ config RT_MUTEXES
+ 	boolean
+ 	select PLIST
+ 
++config CRC32_CALC
++	default !CRC32_TABLES
++	bool
++
+ config TINY_SHMEM
+ 	default !SHMEM
+ 	bool
+Index: linux-2.6.22.1/lib/Makefile
+===================================================================
+--- linux-2.6.22.1.orig/lib/Makefile
++++ linux-2.6.22.1/lib/Makefile
+@@ -65,7 +65,9 @@ lib-$(CONFIG_GENERIC_BUG) += bug.o
+ hostprogs-y	:= gen_crc32table
+ clean-files	:= crc32table.h
+ 
++ifeq ($(CONFIG_CRC32_TABLES),y)
+ $(obj)/crc32.o: $(obj)/crc32table.h
++endif
+ 
+ quiet_cmd_crc32 = GEN     $@
+       cmd_crc32 = $< > $@
+Index: linux-2.6.22.1/lib/crc32.c
+===================================================================
+--- linux-2.6.22.1.orig/lib/crc32.c
++++ linux-2.6.22.1/lib/crc32.c
+@@ -36,7 +36,9 @@
+ #define tole(x) (x)
+ #define tobe(x) (x)
+ #endif
++#ifdef CONFIG_CRC32_TABLES
+ #include "crc32table.h"
++#endif
+ 
+ MODULE_AUTHOR("Matt Domsch <Matt_Domsch@dell.com>");
+ MODULE_DESCRIPTION("Ethernet CRC32 calculations");
+Index: linux-2.6.22.1/lib/crc32defs.h
+===================================================================
+--- linux-2.6.22.1.orig/lib/crc32defs.h
++++ linux-2.6.22.1/lib/crc32defs.h
+@@ -7,8 +7,15 @@
+ #define CRCPOLY_BE 0x04c11db7
+ 
+ /* How many bits at a time to use.  Requires a table of 4<<CRC_xx_BITS bytes. */
++
++#ifdef CONFIG_CRC32_CALC /* config symbols not visible to gen_crc32table */
++#define CRC_LE_BITS 1
++#define CRC_BE_BITS 1
++#endif
++
+ /* For less performance-sensitive, use 4 */
+-#ifndef CRC_LE_BITS 
++
++#ifndef CRC_LE_BITS
+ # define CRC_LE_BITS 8
+ #endif
+ #ifndef CRC_BE_BITS
Index: packages/kernel/patches/tiny_m57sli/tiny/inflate-iostate.patch
===================================================================
--- packages/kernel/patches/tiny_m57sli/tiny/inflate-iostate.patch	(revision 0)
+++ packages/kernel/patches/tiny_m57sli/tiny/inflate-iostate.patch	(revision 0)
@@ -0,0 +1,522 @@
+inflate  move globals into a state structure
+
+
+ tiny-mpm/lib/inflate.c |  292 ++++++++++++++++++++++---------------------------
+ 1 files changed, 136 insertions(+), 156 deletions(-)
+
+Index: linux-2.6.16.19/lib/inflate.c
+===================================================================
+--- linux-2.6.16.19.orig/lib/inflate.c	2006-06-23 17:06:29.000000000 -0700
++++ linux-2.6.16.19/lib/inflate.c	2006-06-23 17:06:29.000000000 -0700
+@@ -125,16 +125,23 @@
+ 	} v;
+ };
+ 
++struct iostate {
++	u8 *window;
++	int opos, osize, bits;
++	u32 buf;
++};
++
+ /* Function prototypes */
+ static int INIT huft_build(unsigned *, unsigned, unsigned,
+ 			  const u16 *, const u16 *, struct huft **, int *);
+ static int INIT huft_free(struct huft *);
+-static int INIT inflate_codes(struct huft *, struct huft *, int, int);
+-static int INIT inflate_stored(void);
+-static int INIT inflate_fixed(void);
+-static int INIT inflate_dynamic(void);
+-static int INIT inflate_block(int *);
+-static int INIT inflate(void);
++static int INIT inflate_codes(struct iostate *, struct huft *, struct huft *,
++			 int, int);
++static int INIT inflate_stored(struct iostate *);
++static int INIT inflate_fixed(struct iostate *);
++static int INIT inflate_dynamic(struct iostate *);
++static int INIT inflate_block(struct iostate *, int *);
++static int INIT inflate(struct iostate *);
+ 
+ /* The inflate algorithm uses a sliding 32 K byte window on the uncompressed
+    stream to find repeated byte strings.  This is implemented here as a
+@@ -145,7 +152,36 @@
+    "u8 *window;" and then malloc'ed in the latter case.  The definition
+    must be in unzip.h, included above. */
+ 
+-#define flush_output(w) (outcnt=(w),flush_window())
++static void flush_output(struct iostate *io)
++{
++	outcnt = io->opos;
++	flush_window();
++	io->opos = 0;
++}
++
++static void put_byte(struct iostate *io, u8 byte)
++{
++	io->window[io->opos++] = byte;
++	if (io->opos == io->osize)
++		flush_output(io);
++}
++
++static void copy_bytes(struct iostate *io, int len, int dist)
++{
++	int part, pos = io->opos - dist;
++
++	do {
++		pos &= io->osize - 1;
++		part = min(len, io->osize - max(pos, io->opos));
++		len -= part;
++
++		while (part--)
++			io->window[io->opos++] = io->window[pos++];
++
++		if (io->opos == io->osize)
++			flush_output(io);
++	} while (len);
++}
+ 
+ /* Tables for deflate from PKZIP's appnote.txt. */
+ 
+@@ -186,10 +222,10 @@
+ /* Inlines for inflate() bit peeking and grabbing.
+    The usage is:
+ 
+-        x = readbits(&b, &k, j);
+-	dumpbits(&b, &k, j);
++        x = readbits(io, j);
++	dumpbits(io, j);
+ 
+-	x = pullbits(&b, &k, j);
++	x = pullbits(io, j);
+ 
+    where readbits makes sure that b has at least j bits in it, and
+    dumpbits removes the bits from b, while k tracks the number of bits
+@@ -212,29 +248,34 @@
+    the stream.
+  */
+ 
+-static u32 bb;			/* bit buffer */
+-static unsigned bk;		/* bits in bit buffer */
+-
+-static inline u32 readbits(u32 *b, u32 *k, int n)
++static inline u32 readbits(struct iostate *io, int n)
+ {
+-	for( ; *k < n; *k += 8)
+-		*b |= (u32)get_byte() << *k;
+-	return *b & ((1 << n) - 1);
++	for( ; io->bits < n; io->bits += 8)
++		io->buf |= (u32)get_byte() << io->bits;
++	return io->buf & ((1 << n) - 1);
+ }
+ 
+-static inline void dumpbits(u32 *b, u32 *k, int n)
++static inline void dumpbits(struct iostate *io, int n)
+ {
+-	*b >>= n;
+-	*k -= n;
++	io->buf >>= n;
++	io->bits -= n;
+ }
+ 
+-static inline u32 pullbits(u32 *b, u32 *k, int n)
++static inline u32 pullbits(struct iostate *io, int n)
+ {
+-	u32 r = readbits(b, k, n);
+-	dumpbits(b, k, n);
++	u32 r = readbits(io, n);
++	dumpbits(io, n);
+ 	return r;
+ }
+ 
++static inline void popbytes(struct iostate *io)
++{
++	while (io->bits >= 8) {
++		io->bits -= 8;
++		inptr--;
++	}
++}
++
+ /*
+    Huffman code decoding is performed using a multi-level table lookup.
+    The fastest way to decode is to simply build a lookup table whose
+@@ -536,6 +577,7 @@
+ 
+ /*
+  * inflate_codes - decompress the codes in a deflated block
++ * @io: current i/o state
+  * @tl: literal/length decoder tables
+  * @td: distance decoder tables
+  * @bl: number of bits decoded by tl
+@@ -544,129 +586,75 @@
+  * inflate (decompress) the codes in a deflated (compressed) block.
+  * Return an error code or zero if it all goes ok.
+  */
+-static int inflate_codes(struct huft *tl, struct huft *td, int bl, int bd)
++static int INIT inflate_codes(struct iostate *io, struct huft *tl, struct huft *td,
++			 int bl, int bd)
+ {
+ 	unsigned e;	/* table entry flag/number of extra bits */
+-	unsigned n, d;		/* length and index for copy */
+-	unsigned w;		/* current window position */
++	unsigned len, dist;
+ 	struct huft *t;		/* pointer to table entry */
+-	u32 b;		/* bit buffer */
+-	unsigned k;	/* number of bits in bit buffer */
+-
+-	/* make local copies of globals */
+-	b = bb;			/* initialize bit buffer */
+-	k = bk;
+-	w = outcnt;			/* initialize window position */
+ 
+ 	/* inflate the coded data */
+ 	for (;;) {		/* do until end of block */
+-		t = tl + readbits(&b, &k, bl);
++		t = tl + readbits(io, bl);
+ 		e = t->e;
+ 		while (e > 16) {
+ 			if (e == 99)
+ 				return 1;
+-			dumpbits(&b, &k, t->b);
+-			t = t->v.t + readbits(&b, &k, e - 16);
++			dumpbits(io, t->b);
++			t = t->v.t + readbits(io, e - 16);
+ 			e = t->e;
+ 		}
+-		dumpbits(&b, &k, t->b);
++		dumpbits(io, t->b);
+ 		if (e == 16) {	/* then it's a literal */
+-			window[w++] = (u8)t->v.n;
+-			if (w == WSIZE) {
+-				flush_output(w);
+-				w = 0;
+-			}
++			put_byte(io, t->v.n);
+ 		} else {	/* it's an EOB or a length */
+ 			/* exit if end of block */
+ 			if (e == 15)
+ 				break;
+ 
+ 			/* get length of block to copy */
+-			n = t->v.n + pullbits(&b, &k, e);
++			len = t->v.n + pullbits(io, e);
+ 
+ 			/* decode distance of block to copy */
+-			t = td + readbits(&b, &k, bd);
++			t = td + readbits(io, bd);
+ 			e = t->e;
+ 			while (e > 16) {
+ 				if (e == 99)
+ 					return 1;
+-				dumpbits(&b, &k, t->b);
+-				t = t->v.t + readbits(&b, &k, e - 16);
++				dumpbits(io, t->b);
++				t = t->v.t + readbits(io, e - 16);
+ 				e = t->e;
+ 			}
+-			dumpbits(&b, &k, t->b);
+-
+-			d = w - t->v.n - pullbits(&b, &k, e);
++			dumpbits(io, t->b);
+ 
+-			/* do the copy */
+-			do {
+-				n -= (e = (e = WSIZE - ((d &= WSIZE - 1) > w ?
+-							d : w)) > n ? n : e);
+-#if !defined(NOMEMCPY) && !defined(DEBUG)
+-				/* (this test assumes unsigned comparison) */
+-				if (w - d >= e) {
+-					memcpy(window + w, window + d, e);
+-					w += e;
+-					d += e;
+-				} else
+-#endif				/* !NOMEMCPY */
+-					/* avoid memcpy() overlap */
+-					do {
+-						window[w++] = window[d++];
+-					} while (--e);
+-				if (w == WSIZE) {
+-					flush_output(w);
+-					w = 0;
+-				}
+-			} while (n);
++			dist = t->v.n + pullbits(io, e);
++			copy_bytes(io, len, dist);
+ 		}
+ 	}
+ 
+-	/* restore the globals from the locals */
+-	outcnt = w;			/* restore global window pointer */
+-	bb = b;			/* restore global bit buffer */
+-	bk = k;
+-
+-	/* done */
+ 	return 0;
+ }
+ 
+-/* inflate_stored - "decompress" an inflated type 0 (stored) block. */
+-static int INIT inflate_stored(void)
++/* inflate_stored - "decompress" an inflated type 0 (stored) block.
++ * @io: current i/o state
++ */
++static int INIT inflate_stored(struct iostate *io)
+ {
+ 	unsigned n;		/* number of bytes in block */
+-	unsigned w;		/* current window position */
+-	u32 b;		/* bit buffer */
+-	unsigned k;	/* number of bits in bit buffer */
+ 
+ 	DEBG("<stor");
+ 
+-	/* make local copies of globals */
+-	b = bb;			/* initialize bit buffer */
+-	k = bk;
+-	w = outcnt;			/* initialize window position */
+-
+ 	/* go to byte boundary */
+-	dumpbits(&b, &k, k & 7);
++	dumpbits(io, io->bits & 7);
+ 
+ 	/* get the length and its complement */
+-	n = pullbits(&b, &k, 16);
+-	if (n != (~pullbits(&b, &k, 16) & 0xffff))
++	n = pullbits(io, 16);
++	if (n != (~pullbits(io, 16) & 0xffff))
+ 		return 1;	/* error in compressed data */
+ 
+ 	/* read and output the compressed data */
+-	while (n--) {
+-		window[w++] = (u8)get_byte();
+-		if (w == WSIZE) {
+-			flush_output(w);
+-			w = 0;
+-		}
+-	}
+-
+-	/* restore the globals from the locals */
+-	outcnt = w;			/* restore global window pointer */
+-	bb = b;			/* restore global bit buffer */
+-	bk = k;
++	while (n--)
++		put_byte(io, get_byte());
+ 
+ 	DEBG(">");
+ 	return 0;
+@@ -674,6 +662,7 @@
+ 
+ 
+ /* inflate_fixed - decompress a block with fixed Huffman codes
++ * @io: current i/o state
+  *
+  * decompress an inflated type 1 (fixed Huffman codes) block. We
+  * should either replace this with a custom decoder, or at least
+@@ -681,7 +670,7 @@
+  *
+  * We use `noinline' here to prevent gcc-3.5 from using too much stack space
+  */
+-static int noinline INIT inflate_fixed(void)
++static int noinline INIT inflate_fixed(struct iostate *io)
+ {
+ 	int i;			/* temporary variable */
+ 	struct huft *tl;	/* literal/length code table */
+@@ -717,7 +706,7 @@
+ 	}
+ 
+ 	/* decompress until an end-of-block code */
+-	if (inflate_codes(tl, td, bl, bd))
++	if (inflate_codes(io, tl, td, bl, bd))
+ 		return 1;
+ 
+ 	/* free the decoding tables, return */
+@@ -731,7 +720,7 @@
+  *
+  * We use `noinline' here to prevent gcc-3.5 from using too much stack space
+  */
+-static int noinline INIT inflate_dynamic(void)
++static int noinline INIT inflate_dynamic(struct iostate *io)
+ {
+ 	int i;			/* temporary variables */
+ 	unsigned j;
+@@ -745,19 +734,13 @@
+ 	unsigned nl;		/* number of literal/length codes */
+ 	unsigned nd;		/* number of distance codes */
+ 	unsigned ll[286 + 30];	/* literal/length and distance code lengths */
+-	u32 b;		/* bit buffer */
+-	unsigned k;	/* number of bits in bit buffer */
+ 
+ 	DEBG("<dyn");
+ 
+-	/* make local bit buffer */
+-	b = bb;
+-	k = bk;
+-
+ 	/* read in table lengths */
+-	nl = 257 + pullbits(&b, &k, 5); /* number of literal/length codes */
+-	nd = 1 + pullbits(&b, &k, 5); /* number of distance codes */
+-	nb = 4 + pullbits(&b, &k, 4); /* number of bit length codes */
++	nl = 257 + pullbits(io, 5); /* number of literal/length codes */
++	nd = 1 + pullbits(io, 5); /* number of distance codes */
++	nb = 4 + pullbits(io, 4); /* number of bit length codes */
+ 	if (nl > 286 || nd > 30)
+ 		return 1;	/* bad lengths */
+ 
+@@ -765,7 +748,7 @@
+ 
+ 	/* read in bit-length-code lengths */
+ 	for (j = 0; j < nb; j++)
+-		ll[border[j]] = pullbits(&b, &k, 3);
++		ll[border[j]] = pullbits(io, 3);
+ 	for (; j < 19; j++)
+ 		ll[border[j]] = 0;
+ 
+@@ -785,26 +768,26 @@
+ 	n = nl + nd;
+ 	i = l = 0;
+ 	while ((unsigned)i < n) {
+-		td = tl + readbits(&b, &k, bl);
+-		dumpbits(&b, &k, td->b);
++		td = tl + readbits(io, bl);
++		dumpbits(io, td->b);
+ 		j = td->v.n;
+ 		if (j < 16)	/* length of code in bits (0..15) */
+ 			ll[i++] = l = j;	/* save last length in l */
+ 		else if (j == 16) {	/* repeat last length 3 to 6 times */
+-			j = 3 + pullbits(&b, &k, 2);
++			j = 3 + pullbits(io, 2);
+ 			if ((unsigned)i + j > n)
+ 				return 1;
+ 			while (j--)
+ 				ll[i++] = l;
+ 		} else if (j == 17) {	/* 3 to 10 zero length codes */
+-			j = 3 + pullbits(&b, &k, 3);
++			j = 3 + pullbits(io, 3);
+ 			if ((unsigned)i + j > n)
+ 				return 1;
+ 			while (j--)
+ 				ll[i++] = 0;
+ 			l = 0;
+ 		} else {	/* j == 18: 11 to 138 zero length codes */
+-			j = 11 + pullbits(&b, &k, 7);
++			j = 11 + pullbits(io, 7);
+ 			if ((unsigned)i + j > n)
+ 				return 1;
+ 			while (j--)
+@@ -820,10 +803,6 @@
+ 
+ 	DEBG("dyn5 ");
+ 
+-	/* restore the global bit buffer */
+-	bb = b;
+-	bk = k;
+-
+ 	DEBG("dyn5a ");
+ 
+ 	/* build the decoding tables for literal/length and distance codes */
+@@ -851,7 +830,7 @@
+ 	DEBG("dyn6 ");
+ 
+ 	/* decompress until an end-of-block code */
+-	if (inflate_codes(tl, td, bl, bd))
++	if (inflate_codes(io, tl, td, bl, bd))
+ 		return 1;
+ 
+ 	DEBG("dyn7 ");
+@@ -865,24 +844,25 @@
+ }
+ 
+ /* inflate_block - decompress a deflated block
++ * @io: current i/o state
+  * @e: last block flag
+  */
+-static int INIT inflate_block(int *e)
++static int INIT inflate_block(struct iostate *io, int *e)
+ {
+ 	unsigned t;		/* block type */
+ 
+ 	DEBG("<blk");
+ 
+-	*e = pullbits(&bb, &bk, 1); /* read in last block bit */
+-	t = pullbits(&bb, &bk, 2); /* read in block type */
++	*e = pullbits(io, 1); /* read in last block bit */
++	t = pullbits(io, 2); /* read in block type */
+ 
+ 	/* inflate that block type */
+ 	if (t == 2)
+-		return inflate_dynamic();
++		return inflate_dynamic(io);
+ 	if (t == 0)
+-		return inflate_stored();
++		return inflate_stored(io);
+ 	if (t == 1)
+-		return inflate_fixed();
++		return inflate_fixed(io);
+ 
+ 	DEBG(">");
+ 
+@@ -890,40 +870,27 @@
+ 	return 2;
+ }
+ 
+-/* inflate - decompress an inflated entry */
+-static int INIT inflate(void)
++/* inflate - decompress an inflated entry
++ * @io: current i/o state
++ */
++static int INIT inflate(struct iostate *io)
+ {
+ 	int e;			/* last block flag */
+ 	int r;			/* result code */
+ 	void *ptr;
+ 
+-	/* initialize window, bit buffer */
+-	outcnt = 0;
+-	bk = 0;
+-	bb = 0;
+-
+ 	/* decompress until the last block */
+ 	do {
+ 		gzip_mark(&ptr);
+-		if ((r = inflate_block(&e))) {
++		if ((r = inflate_block(io, &e))) {
+ 			gzip_release(&ptr);
+ 			return r;
+ 		}
+ 		gzip_release(&ptr);
+ 	} while (!e);
+ 
+-	/* Undo too much lookahead. The next read will be byte aligned so we
+-	 * can discard unused bits in the last meaningful byte.
+-	 */
+-	while (bk >= 8) {
+-		bk -= 8;
+-		inptr--;
+-	}
+-
+-	/* flush out window */
+-	flush_output(outcnt);
+-
+-	/* return success */
++	popbytes(io);
++	flush_output(io);
+ 	return 0;
+ }
+ 
+@@ -996,6 +963,11 @@
+ 	u32 orig_crc = 0;	/* original crc */
+ 	u32 orig_len = 0;	/* original uncompressed length */
+ 	int res;
++	struct iostate io;
++
++	io.window = window;
++	io.osize = WSIZE;
++	io.opos = io.bits = io.buf = 0;
+ 
+ 	magic[0] = get_byte();
+ 	magic[1] = get_byte();
+@@ -1051,7 +1023,7 @@
+ 			;
+ 
+ 	/* Decompress */
+-	if ((res = inflate())) {
++	if ((res = inflate(&io))) {
+ 		switch (res) {
+ 		case 0:
+ 			break;
Index: packages/kernel/patches/tiny_m57sli/tiny/no-translations.patch
===================================================================
--- packages/kernel/patches/tiny_m57sli/tiny/no-translations.patch	(revision 0)
+++ packages/kernel/patches/tiny_m57sli/tiny/no-translations.patch	(revision 0)
@@ -0,0 +1,129 @@
+
+Configurable support for console charset translation
+
+
+ tiny-mpm/drivers/char/consolemap.c |   70 +++++++++++++++++++++++++++++++++++++
+ tiny-mpm/drivers/char/vt.c         |    4 ++
+ tiny-mpm/init/Kconfig              |    7 +++
+ 3 files changed, 81 insertions(+)
+
+Index: linux-2.6.16.19/drivers/char/consolemap.c
+===================================================================
+--- linux-2.6.16.19.orig/drivers/char/consolemap.c	2006-06-23 17:05:13.000000000 -0700
++++ linux-2.6.16.19/drivers/char/consolemap.c	2006-06-23 17:06:21.000000000 -0700
+@@ -23,6 +23,8 @@
+ #include <linux/consolemap.h>
+ #include <linux/vt_kern.h>
+ 
++#ifdef CONFIG_CONSOLE_TRANSLATIONS
++
+ static unsigned short translations[][256] = {
+   /* 8-bit Latin-1 mapped to Unicode -- trivial mapping */
+   {
+@@ -668,3 +670,72 @@
+ }
+ 
+ EXPORT_SYMBOL(con_copy_unimap);
++
++#else
++
++unsigned char inverse_translate(struct vc_data *conp, int glyph)
++{
++	return glyph;
++}
++
++unsigned short *set_translate(int m, struct vc_data *vc)
++{
++	return NULL;
++}
++
++int con_set_trans_old(unsigned char * arg)
++{
++	return 0;
++}
++
++int con_get_trans_old(unsigned char * arg)
++{
++	return -EINVAL;
++}
++
++int con_set_trans_new(ushort * arg)
++{
++	return 0;
++}
++
++int con_get_trans_new(ushort * arg)
++{
++	return -EINVAL;
++}
++
++int con_clear_unimap(struct vc_data *vc, struct unimapinit *ui)
++{
++	return 0;
++}
++
++int con_set_unimap(struct vc_data *vc, ushort ct, struct unipair *list)
++{
++	return 0;
++}
++
++int con_set_default_unimap(struct vc_data *vc)
++{
++	return 0;
++}
++
++int con_copy_unimap(struct vc_data *d, struct vc_data *s)
++{
++	return 0;
++}
++
++int con_get_unimap(struct vc_data *vc, ushort ct, ushort *uct,
++		   struct unipair *list)
++{
++	return -EINVAL;
++}
++
++void con_free_unimap(struct vc_data *vc) { }
++
++int conv_uni_to_pc(struct vc_data *conp, long ucs)
++{
++	return ucs > 0xff ? -1: ucs;
++}
++
++void __init console_map_init(void) { }
++
++#endif
+Index: linux-2.6.16.19/drivers/char/vt.c
+===================================================================
+--- linux-2.6.16.19.orig/drivers/char/vt.c	2006-06-23 17:05:13.000000000 -0700
++++ linux-2.6.16.19/drivers/char/vt.c	2006-06-23 17:06:21.000000000 -0700
+@@ -2026,7 +2026,11 @@
+ 		      vc->vc_utf_count = 0;
+ 		    }
+ 		} else {	/* no utf */
++#ifdef CONFIG_CONSOLE_TRANSLATIONS
+ 		  tc = vc->vc_translate[vc->vc_toggle_meta ? (c | 0x80) : c];
++#else
++		  tc = c;
++#endif
+ 		}
+ 
+                 /* If the original code was a control character we
+Index: linux-2.6.16.19/init/Kconfig
+===================================================================
+--- linux-2.6.16.19.orig/init/Kconfig	2006-06-23 17:06:21.000000000 -0700
++++ linux-2.6.16.19/init/Kconfig	2006-06-23 17:06:21.000000000 -0700
+@@ -474,6 +474,13 @@
+ 	  will somewhat decrease code size, but with a higher risk of
+ 	  deadlock under some loads.
+ 
++config CONSOLE_TRANSLATIONS
++	default y
++	bool "Enable character translations in console" if EMBEDDED
++	help
++	  This enables support for font mapping and Unicode translation
++          on virtual consoles.
++
+ endmenu		# General setup
+ 
+ config CRC32_CALC
Index: packages/kernel/patches/tiny_m57sli/tiny/inline-kconfig.patch
===================================================================
--- packages/kernel/patches/tiny_m57sli/tiny/inline-kconfig.patch	(revision 0)
+++ packages/kernel/patches/tiny_m57sli/tiny/inline-kconfig.patch	(revision 0)
@@ -0,0 +1,101 @@
+    Added configration to select whether linuxtiny un-inlining patch
+    should take effect.
+
+---
+ fs/buffer.c     |    4 	4 +	0 -	0 !
+ fs/ext2/namei.c |    4 	4 +	0 -	0 !
+ fs/namei.c      |   13 	13 +	0 -	0 !
+ init/Kconfig    |    6 	6 +	0 -	0 !
+ 4 files changed, 27 insertions(+)
+
+Index: linux-2.6.22.1/init/Kconfig
+===================================================================
+--- linux-2.6.22.1.orig/init/Kconfig
++++ linux-2.6.22.1/init/Kconfig
+@@ -606,6 +606,12 @@ config CC_FUNIT_AT_A_TIME
+ 	    Some tests show impressive .text shrinkage from using
+ 	    unit-at-a-time compilation.
+ 
++config LINUXTINY_DO_UNINLINE
++       bool "un-inline several functions for smaller kernel size" if EMBEDDED
++       default n
++       help
++	  Un-inline some function for smaller kernel size.
++
+ endmenu		# General setup
+ 
+ config RT_MUTEXES
+Index: linux-2.6.22.1/fs/buffer.c
+===================================================================
+--- linux-2.6.22.1.orig/fs/buffer.c
++++ linux-2.6.22.1/fs/buffer.c
+@@ -564,7 +564,11 @@ EXPORT_SYMBOL(mark_buffer_async_write);
+ /*
+  * The buffer's backing address_space's private_lock must be held
+  */
++#ifdef CONFIG_LINUXTINY_DO_UNINLINE
+ static void __remove_assoc_queue(struct buffer_head *bh)
++#else
++static inline void __remove_assoc_queue(struct buffer_head *bh)
++#endif
+ {
+ 	list_del_init(&bh->b_assoc_buffers);
+ 	WARN_ON(!bh->b_assoc_map);
+Index: linux-2.6.22.1/fs/namei.c
+===================================================================
+--- linux-2.6.22.1.orig/fs/namei.c
++++ linux-2.6.22.1/fs/namei.c
+@@ -607,7 +607,11 @@ static __always_inline int __do_follow_l
+ 		char *s = nd_get_link(nd);
+ 		error = 0;
+ 		if (s)
++#ifdef CONFIG_LINUXTINY_DO_UNINLINE
+ 			error = vfs_follow_link(nd, s);
++#else
++			error = __vfs_follow_link(nd, s);
++#endif
+ 		if (dentry->d_inode->i_op->put_link)
+ 			dentry->d_inode->i_op->put_link(dentry, nd, cookie);
+ 	}
+@@ -624,7 +628,11 @@ static __always_inline int __do_follow_l
+  * Without that kind of total limit, nasty chains of consecutive
+  * symlinks can cause almost arbitrarily long lookups. 
+  */
++#ifdef CONFIG_LINUXTINY_DO_UNINLINE
+ static int do_follow_link(struct path *path, struct nameidata *nd)
++#else
++static inline int do_follow_link(struct path *path, struct nameidata *nd)
++#endif
+ {
+ 	int err = -ELOOP;
+ 	if (current->link_count >= MAX_NESTED_LINKS)
+@@ -1439,8 +1447,13 @@ static int may_delete(struct inode *dir,
+  *  3. We should have write and exec permissions on dir
+  *  4. We can't do it if dir is immutable (done in permission())
+  */
++#ifdef CONFIG_LINUXTINY_DO_UNINLINE
+ static int may_create(struct inode *dir, struct dentry *child,
+ 			     struct nameidata *nd)
++#else
++static inline int may_create(struct inode *dir, struct dentry *child,
++			     struct nameidata *nd)
++#endif
+ {
+ 	if (child->d_inode)
+ 		return -EEXIST;
+Index: linux-2.6.22.1/fs/ext2/namei.c
+===================================================================
+--- linux-2.6.22.1.orig/fs/ext2/namei.c
++++ linux-2.6.22.1/fs/ext2/namei.c
+@@ -36,7 +36,11 @@
+ #include "acl.h"
+ #include "xip.h"
+ 
++#ifdef CONFIG_LINUXTINY_DO_UNINLINE
+ static int ext2_add_nondir(struct dentry *dentry, struct inode *inode)
++#else
++static inline int ext2_add_nondir(struct dentry *dentry, struct inode *inode)
++#endif
+ {
+ 	int err = ext2_add_link(dentry, inode);
+ 	if (!err) {
Index: packages/kernel/patches/tiny_m57sli/tiny/kgdb-ga.patch
===================================================================
--- packages/kernel/patches/tiny_m57sli/tiny/kgdb-ga.patch	(revision 0)
+++ packages/kernel/patches/tiny_m57sli/tiny/kgdb-ga.patch	(revision 0)
@@ -0,0 +1,6331 @@
+
+
+This kgdb will get called and will trap almost any kernel
+fault WITHOUT BEING ARMED.
+
+It is entered at boot time via "kgdb" in the boot string,
+not "gdb".  This entry occurs when the first setup on the
+boot string is called, not sometime later.  You will not
+find a "waiting for gdb" on your console, as the console has
+not yet been enabled at this time.  (Note, this early stuff
+is a bit fragile as the full trap table has yet to be
+loaded, something I might address, sometime...  So don't try
+to look at memory that can not be reached, for example. 
+Once the full trap table is loaded this restriction goes
+away.)
+
+If you hard code it, you can put a breakpoint() as the FIRST
+LINE OF C CODE.
+
+It does NOT use the serial driver, but if the serial driver
+is loaded, it tells it to release the port to avoid
+conflict.  
+
+The threads stuff is not configurable, does not require
+redirection of schedule() calls and does back track to the
+first non schedule() caller on the info threads command.  If
+you switch to the thread, however, it will show it in the
+switch code (as it should).
+
+It is MUCH more aggressive and paranoid about grabbing the
+other cpus on entry.  It issues a "send_nmi_all_but_self()"
+rather than depending on them to interrupt or hit an NMI
+sometime in the distant future.  If a cpu does not come to
+the party, it will continue without it so all is not lost.
+
+It does not have anything to do with IOCTL calls, but does
+do the control-C thing.
+
+There is a LOT of info in the patch which ends up in
+.../Documentation/i386/kgdb/*
+
+There is a nifty little thing call kgdb_ts() (kgdb time
+stamp) which is a function you can code calls to which puts
+some useful stuff in a circular buffer which can be examined
+with the supplied gdb macros.
+
+It also allows you do to do "p foobar(...)"  i.e. to call a
+function from gdb, just like gdb allows in program
+debugging.
+
+In an SMP system, you can choose to "hold" any given set of
+cpus.  It also defaults to holding other cpus on single step
+(this can be overridden).
+
+This said, you can imagine my consternation when I found it
+"lost it" on continues on 2.5.  I found and fixed this this
+early pm, a hold cpu on exit goof on my part.
+
+Oh, and a final point, the configure options are more
+extensive (the serial port is set up here, for example, (can
+not wait for a command line to do this)).  There is one to
+do system call exit tests.  This is VERY new and causes the
+kernel to hit a hard "int 3" if a system call attempts to
+exit with preempt count other than zero.  This is a fault,
+of course, but the current 2.5 is full of them so I don't
+recommend turning this on.
+
+
+DESC
+kgdbL warning fix
+EDESC
+From: Ingo Molnar <mingo@elte.hu>
+
+this patch fixes a deprecated use of asm input operands. (and shuts up a
+gcc 3.3 warning.)
+
+DESC
+kgdb buffer overflow fix
+EDESC
+From: George Anzinger <george@mvista.com>
+
+
+DESC
+kgdbL warning fix
+EDESC
+From: Ingo Molnar <mingo@elte.hu>
+
+this patch fixes a deprecated use of asm input operands. (and shuts up a
+gcc 3.3 warning.)
+
+DESC
+kgdb: CONFIG_DEBUG_INFO fix
+EDESC
+From: Thomas Schlichter <schlicht@uni-mannheim.de>
+
+that patch sets DEBUG_INFO to y by default, even if whether DEBUG_KERNEL nor 
+KGDB is enabled. The attached patch changes this to enable DEBUG_INFO by 
+default only if KGDB is enabled.
+
+DESC
+x86_64 fixes
+EDESC
+From Andi Kleen
+
+Fix x86_64 for kgdb.  We forget why.
+DESC
+correct kgdb.txt Documentation link (against  2.6.1-rc1-mm2)
+EDESC
+From: Jesper Juhl <juhl-lkml@dif.dk>
+
+The help text for "config KGDB" in arch/i386/Kconfig refers to
+Documentation/i386/kgdb.txt - the actual location is
+Documentation/i386/kgdb/kgdb.txt - patch below to fix that.
+
+DESC
+kgdb: fix for recent gcc
+EDESC
+
+arch/i386/kernel/traps.c:97: error: conflicting types for 'int3'
+arch/i386/kernel/traps.c:77: error: previous declaration of 'int3' was here
+arch/i386/kernel/traps.c:97: error: conflicting types for 'int3'
+arch/i386/kernel/traps.c:77: error: previous declaration of 'int3' was here
+arch/i386/kernel/traps.c:99: error: conflicting types for 'debug'
+arch/i386/kernel/traps.c:75: error: previous declaration of 'debug' was here
+arch/i386/kernel/traps.c:99: error: conflicting types for 'debug'
+arch/i386/kernel/traps.c:75: error: previous declaration of 'debug' was here
+
+DESC
+kgdb warning fixes
+EDESC
+
+arch/i386/kernel/kgdb_stub.c:1306: warning: 'time' might be used uninitialized in this function
+arch/i386/kernel/kgdb_stub.c:1306: warning: 'dum' might be used uninitialized in this function
+DESC
+THREAD_SIZE fixes for kgdb
+EDESC
+From: Matt Mackall <mpm@selenic.com>
+
+Noticed the THREAD_SIZE clean-ups are in -mm now. Here are the missing
+bits for kgdb, tested in -tiny with 4k stacks. 
+DESC
+Fix stack overflow test for non-8k stacks
+EDESC
+From: Matt Mackall <mpm@selenic.com>
+
+This is needed to work properly with 4k and 16k stacks.
+DESC
+kgdb-ga.patch fix for i386 single-step into sysenter
+EDESC
+From: Roland McGrath <roland@redhat.com>
+
+Using kgdb-ga.patch from -mm, if userland single-steps (PTRACE_SINGLESTEP)
+into the `sysenter' instruction, kgdb reports a bogus trap:
+
+	Program received signal SIGTRAP, Trace/breakpoint trap.
+	sysenter_past_esp () at arch/i386/kernel/entry.S:249
+	1: x/i $pc  0xc0106023 <sysenter_past_esp>:	sti    
+	(gdb) 
+
+The hackery in the "FIX_STACK" macro in entry.S changes the saved PC for a
+the spurious kernel-mode debug trap when TF was set on user-mode execution
+of `sysenter', so sysenter_past_esp is where it actually lies in this case.
+ The following patch removes the kgdb hiccup when userland
+PTRACE_SINGLESTEP's into sysenter.
+DESC
+fix TRAP_BAD_SYSCALL_EXITS on i386
+EDESC
+From: Andy Whitcroft <apw@shadowen.org>
+
+We are not using the right offset name, nor the right address when checking
+for a non-zero preempt count.  Move to TI_preempt_count(%ebp).
+
+Signed-off-by: Andy Whitcroft <apw@shadowen.org>
+DESC
+add TRAP_BAD_SYSCALL_EXITS config for i386
+EDESC
+From: Andy Whitcroft <apw@shadowen.org>
+
+There seems to be code recently added to -bk and thereby -mm which supports
+extra debug for preempt on system call exit.  Oddly there doesn't seem to
+be configuration options to enable them.  Below is a possible patch to
+allow enabling this on i386.  Sadly the most obvious menu to add this to is
+the Kernel Hacking menu, but that is defined in architecture specific
+configuration.  If this makes sense I could patch the other arches?
+
+Add a configuration option to allow enabling TRAP_BAD_SYSCALL_EXITS to the
+Kernel Hacking menu.
+
+Signed-off-by: Andy Whitcroft <apw@shadowen.org>
+DESC
+kgdb-is-incompatible-with-kprobes
+EDESC
+DESC
+kgdb-ga-build-fix
+EDESC
+DESC
+kgdb-ga-fixes
+EDESC
+DESC
+kgdb: kill off highmem_start_page
+EDESC
+From: Dave Hansen <haveblue@us.ibm.com>
+
+This patch should fix kgdb to work without highmem_start_page. 
+However, I don't have a kgdb setup, and I'd appreciate someone
+testing this to make sure my selection of a new variable is OK.
+
+Also, should this function possibly be using system_state, instead?
+
+Signed-off-by: Dave Hansen <haveblue@us.ibm.com>
+DESC
+kgdb documentation fix
+EDESC
+From: "Matthias Urlichs" <smurf@smurf.noris.de>
+
+Update Documentation/i386/kgdb/gdbinit-modules to conform to the current
+kernel's module data structure.
+
+Signed-Off-By: Matthias Urlichs <smurf@smurf.noris.de>
+DESC
+kgdb: move config option for BAD_SYSCALL_EXIT
+EDESC
+From: Stas Sergeev <stsp@aknet.ru>
+
+This patch moves the CONFIG_TRAP_BAD_SYSCALL_EXIT from "Executable file
+formats" section to the KGDB section.  I had real problems finding that option
+where it was.
+
+Signed-off-by: Stas Sergeev <stsp@aknet.ru>
+DESC
+kgdb: fix BAD_SYSCALL_EXIT lockup
+EDESC
+From: Stas Sergeev <stsp@aknet.ru>
+
+CONFIG_TRAP_BAD_SYSCALL_EXITS forgets to do GET_THREAD_INFO(%ebp) before
+accessing the TI_preempt_count(%ebp).  This leads to an accesses to the random
+addresses and kills the machine.  Also "int $3" does nothing good by itself
+(it would just Oops AFAICS).  And it is misplaced, too.  Obviously it never
+worked.
+
+The attached patch moves the check to the right place (to the syscall_exit
+path) and replaces the "int $3" by the call to the helper function that only
+prints some debug info and doesn't crash the system.  This fully solves the
+reported problem.
+
+Signed-off-by: Stas Sergeev <stsp@aknet.ru>
+DESC
+kgdb CONFIG_DEBUG_INFO fix
+EDESC
+From: Adrian Bunk <bunk@stusta.de>
+
+Why does everyone think one DEBUG_INFO wasn't enough?
+
+Signed-off-by: Adrian Bunk <bunk@stusta.de>
+Signed-off-by: Andrew Morton <akpm@osdl.org>
+Index: linux-2.6.16.19/Documentation/i386/kgdb/andthen
+===================================================================
+--- linux-2.6.16.19.orig/Documentation/i386/kgdb/andthen	2006-04-21 14:09:39.654033432 -0700
++++ linux-2.6.16.19/Documentation/i386/kgdb/andthen	2006-06-23 17:06:11.000000000 -0700
+@@ -0,0 +1,100 @@
++
++define	set_andthen
++	set var $thp=0
++	set var $thp=(struct kgdb_and_then_struct *)&kgdb_data[0]
++	set var $at_size = (sizeof kgdb_data)/(sizeof *$thp)
++	set var $at_oc=kgdb_and_then_count
++	set var $at_cc=$at_oc
++end
++
++define andthen_next
++	set var $at_cc=$arg0
++end
++
++define andthen
++	andthen_set_edge
++	if ($at_cc >= $at_oc)
++		printf "Outside window.  Window size is %d\n",($at_oc-$at_low)
++	else
++		printf "%d: ",$at_cc
++		output *($thp+($at_cc++ % $at_size ))
++		printf "\n"
++	end
++end
++define andthen_set_edge
++	set var $at_oc=kgdb_and_then_count
++	set var $at_low = $at_oc - $at_size
++	if ($at_low < 0 )
++		set var $at_low = 0
++	end
++	if (( $at_cc > $at_oc) || ($at_cc < $at_low))
++		printf "Count outside of window, setting count to "
++		if ($at_cc >= $at_oc)
++			set var $at_cc = $at_oc
++		else
++			set var $at_cc = $at_low
++		end
++		printf "%d\n",$at_cc
++	end
++end
++
++define beforethat
++	andthen_set_edge
++	if ($at_cc <= $at_low)
++		printf "Outside window.  Window size is %d\n",($at_oc-$at_low)
++	else
++		printf "%d: ",$at_cc-1
++		output *($thp+(--$at_cc % $at_size ))
++		printf "\n"
++	end
++end
++
++document andthen_next
++	andthen_next <count>
++	.	sets the number of the event to display next. If this event
++	.	is not in the event pool, either andthen or beforethat will
++	.	correct it to the nearest event pool edge.  The event pool
++	.	ends at the last event recorded and begins <number of events>
++	.	prior to that.  If beforethat is used next, it will display
++	.	event <count> -1.
++.
++	andthen commands are: set_andthen, andthen_next, andthen and beforethat
++end
++
++
++document andthen
++	andthen
++.	displays the next event in the list.  <set_andthen> sets up to display
++.	the oldest saved event first.
++.	<count> (optional) count of the event to display.
++.	note the number of events saved is specified at configure time.
++.	if events are saved between calls to andthen the index will change
++.	but the displayed event will be the next one (unless the event buffer
++.	is overrun).
++.
++.	andthen commands are: set_andthen, andthen_next, andthen and beforethat
++end
++
++document set_andthen
++	set_andthen
++.	sets up to use the <andthen> and <beforethat> commands.
++.		if you have defined your own struct, use the above and
++.		then enter the following:
++.		p $thp=(struct kgdb_and_then_structX *)&kgdb_data[0]
++.		where <kgdb_and_then_structX> is the name of your structure.
++.
++.	andthen commands are: set_andthen, andthen_next, andthen and beforethat
++end
++
++document beforethat
++	beforethat
++.	displays the next prior event in the list. <set_andthen> sets up to
++.	display the last occuring event first.
++.
++.	note the number of events saved is specified at configure time.
++.	if events are saved between calls to beforethat the index will change
++.	but the displayed event will be the next one (unless the event buffer
++.	is overrun).
++.
++.	andthen commands are: set_andthen, andthen_next, andthen and beforethat
++end
+Index: linux-2.6.16.19/Documentation/i386/kgdb/debug-nmi.txt
+===================================================================
+--- linux-2.6.16.19.orig/Documentation/i386/kgdb/debug-nmi.txt	2006-04-21 14:09:39.654033432 -0700
++++ linux-2.6.16.19/Documentation/i386/kgdb/debug-nmi.txt	2006-06-23 17:06:11.000000000 -0700
+@@ -0,0 +1,37 @@
++Subject: Debugging with NMI
++Date: Mon, 12 Jul 1999 11:28:31 -0500
++From: David Grothe <dave@gcom.com>
++Organization: Gcom, Inc
++To: David Grothe <dave@gcom.com>
++
++Kernel hackers:
++
++Maybe this is old hat, but it is new to me --
++
++On an ISA bus machine, if you short out the A1 and B1 pins of an ISA
++slot you will generate an NMI to the CPU.  This interrupts even a
++machine that is hung in a loop with interrupts disabled.  Used in
++conjunction with kgdb <
++ftp://ftp.gcom.com/pub/linux/src/kgdb-2.3.35/kgdb-2.3.35.tgz > you can
++gain debugger control of a machine that is hung in the kernel!  Even
++without kgdb the kernel will print a stack trace so you can find out
++where it was hung.
++
++The A1/B1 pins are directly opposite one another and the farthest pins
++towards the bracket end of the ISA bus socket.  You can stick a paper
++clip or multi-meter probe between them to short them out.
++
++I had a spare ISA bus to PC104 bus adapter around.  The PC104 end of the
++board consists of two rows of wire wrap pins.  So I wired a push button
++between the A1/B1 pins and now have an ISA board that I can stick into
++any ISA bus slot for debugger entry.
++
++Microsoft has a circuit diagram of a PCI card at
++http://www.microsoft.com/hwdev/DEBUGGING/DMPSW.HTM.  If you want to
++build one you will have to mail them and ask for the PAL equations.
++Nobody makes one comercially.
++
++[THIS TIP COMES WITH NO WARRANTY WHATSOEVER.  It works for me, but if
++your machine catches fire, it is your problem, not mine.]
++
++-- Dave (the kgdb guy)
+Index: linux-2.6.16.19/Documentation/i386/kgdb/gdb-globals.txt
+===================================================================
+--- linux-2.6.16.19.orig/Documentation/i386/kgdb/gdb-globals.txt	2006-04-21 14:09:39.654033432 -0700
++++ linux-2.6.16.19/Documentation/i386/kgdb/gdb-globals.txt	2006-06-23 17:06:11.000000000 -0700
+@@ -0,0 +1,71 @@
++Sender: akale@veritas.com
++Date: Fri, 23 Jun 2000 19:26:35 +0530
++From: "Amit S. Kale" <akale@veritas.com>
++Organization: Veritas Software (India)
++To: Dave Grothe <dave@gcom.com>, linux-kernel@vger.rutgers.edu
++CC: David Milburn <dmilburn@wirespeed.com>,
++        "Edouard G. Parmelan" <Edouard.Parmelan@quadratec.fr>,
++        ezannoni@cygnus.com, Keith Owens <kaos@ocs.com.au>
++Subject: Re: Module debugging using kgdb
++
++Dave Grothe wrote:
++>
++> Amit:
++>
++> There is a 2.4.0 version of kgdb on our ftp site:
++> ftp://ftp.gcom.com/pub/linux/src/kgdb.  I mirrored your version of gdb
++> and loadmodule.sh there.
++>
++> Have a look at the README file and see if I go it right.  If not, send
++> me some corrections and I will update it.
++>
++> Does your version of gdb solve the global variable problem?
++
++Yes.
++Thanks to Elena Zanoni, gdb (developement version) can now calculate
++correctly addresses  of dynamically loaded object files. I have not been
++following gdb developement for sometime and am not sure when symbol
++address calculation fix is going to appear in a gdb stable version.
++
++Elena, any idea when the fix will make it to a prebuilt gdb from a
++redhat release?
++
++For the time being I have built a gdb developement version. It can be
++used for module debugging with loadmodule.sh script.
++
++The problem with calculating of module addresses with previous versions
++of gdb was as follows:
++gdb did not use base address of a section while calculating address of
++a symbol in the section in an object file loaded via 'add-symbol-file'.
++It used address of .text segment instead. Due to this addresses of
++symbols in .data, .bss etc. (e.g. global variables) were calculated incorrectly.
++
++Above mentioned fix allow gdb to use base address of a segment while
++calculating address of a symbol in it. It adds a parameter '-s' to
++'add-symbol-file' command for specifying base address of a segment.
++
++loadmodule.sh script works as follows.
++
++1. Copy a module file to target machine.
++2. Load the module on the target machine using insmod with -m parameter.
++insmod produces a module load map which contains base addresses of all
++sections in the module and addresses of symbols in the module file.
++3. Find all sections and their base addresses in the module from
++the module map.
++4. Generate a script that loads the module file. The script uses
++'add-symbol-file' and specifies address of text segment followed by
++addresses of all segments in the module.
++
++Here is an example gdb script produced by loadmodule.sh script.
++
++add-symbol-file foo 0xd082c060 -s .text.lock 0xd08cbfb5
++-s .fixup 0xd08cfbdf -s .rodata 0xd08cfde0 -s __ex_table 0xd08e3b38
++-s .data 0xd08e3d00 -s .bss 0xd08ec8c0 -s __ksymtab 0xd08ee838
++
++With this command gdb can calculate addresses of symbols in ANY segment
++in a module file.
++
++Regards.
++--
++Amit Kale
++Veritas Software ( http://www.veritas.com )
+Index: linux-2.6.16.19/Documentation/i386/kgdb/gdbinit
+===================================================================
+--- linux-2.6.16.19.orig/Documentation/i386/kgdb/gdbinit	2006-04-21 14:09:39.654033432 -0700
++++ linux-2.6.16.19/Documentation/i386/kgdb/gdbinit	2006-06-23 17:06:11.000000000 -0700
+@@ -0,0 +1,14 @@
++shell echo -e "\003" >/dev/ttyS0
++set remotebaud 38400
++target remote /dev/ttyS0
++define si
++stepi
++printf "EAX=%08x EBX=%08x ECX=%08x EDX=%08x\n", $eax, $ebx, $ecx, $edx
++printf "ESI=%08x EDI=%08x EBP=%08x ESP=%08x\n", $esi, $edi, $ebp, $esp
++x/i $eip
++end
++define ni
++nexti
++printf "EAX=%08x EBX=%08x ECX=%08x EDX=%08x\n", $eax, $ebx, $ecx, $edx
++printf "ESI=%08x EDI=%08x EBP=%08x ESP=%08x\n", $esi, $edi, $ebp, $esp
++x/i $eip
+Index: linux-2.6.16.19/Documentation/i386/kgdb/gdbinit-modules
+===================================================================
+--- linux-2.6.16.19.orig/Documentation/i386/kgdb/gdbinit-modules	2006-04-21 14:09:39.654033432 -0700
++++ linux-2.6.16.19/Documentation/i386/kgdb/gdbinit-modules	2006-06-23 17:06:11.000000000 -0700
+@@ -0,0 +1,149 @@
++#
++# Usefull GDB user-command to debug Linux Kernel Modules with gdbstub.
++#
++# This don't work for Linux-2.0 or older.
++#
++# Author Edouard G. Parmelan <Edouard.Parmelan@quadratec.fr>
++#
++#
++# Fri Apr 30 20:33:29 CEST 1999
++#   First public release.
++#
++#   Major cleanup after experiment Linux-2.0 kernel without success.
++#   Symbols of a module are not in the correct order, I can't explain
++#   why :(
++#
++# Fri Mar 19 15:41:40 CET 1999
++#   Initial version.
++#
++# Thu Jan  6 16:29:03 CST 2000
++#   A little fixing by Dave Grothe <dave@gcom.com>
++#
++# Mon Jun 19 09:33:13 CDT 2000
++#   Alignment changes from Edouard Parmelan
++#
++# The basic idea is to find where insmod load the module and inform
++# GDB to load the symbol table of the module with the GDB command
++# ``add-symbol-file <object> <address>''.
++#
++# The Linux kernel holds the list of all loaded modules in module_list,
++# this list end with &kernel_module (exactly with module->next == NULL,
++# but the last module is not a real module).
++#
++# Insmod allocates the struct module before the object file.  Since
++# Linux-2.1, this structure contain his size.  The real address of
++# the object file is then (char*)module + module->size_of_struct.
++#
++# You can use three user functions ``mod-list'', ``mod-print-symbols''
++# and ``add-module-symbols''.
++#
++# mod-list list all loaded modules with the format:
++#    <module-address> <module-name>
++#
++# As soon as you have found the address of your module, you can
++# print its exported symbols (mod-print-symbols) or inform GDB to add
++# symbols from your module file (mod-add-symbols).
++#
++# The argument that you give to mod-print-symbols or mod-add-symbols
++# is the <module-address> from the mod-list command.
++#
++# When using the mod-add-symbols command you must also give the full
++# pathname of the modules object code file.
++#
++# The command mod-add-lis is an example of how to make this easier.
++# You can edit this macro to contain the path name of your own
++# favorite module and then use it as a shorthand to load it.  You
++# still need the module-address, however.
++#
++# The internal function ``mod-validate'' set the GDB variable $mod
++# as a ``struct module*'' if the kernel known the module otherwise
++# $mod is set to NULL.  This ensure to not add symbols for a wrong
++# address.
++#
++#
++# Sat Feb 12 20:05:47 CET 2005
++#
++# Adapted to the 2.6.* module data structure.
++# (Getting miffed at gdb for not having "offsetof" in the process :-/ )
++#
++# Autogenerate add-symbol-file statements from the module list instead
++# of relying on a no-longer-working loadmodule.sh program.
++#
++#                                   Matthias Urlichs <smurf@debian.org>
++#
++#
++# Have a nice hacking day !
++#
++#
++define mod-list
++    set $lmod = modules->next
++    # This is a circular data structure
++    while $lmod != &modules
++		set $mod = (struct module *)(((char *)$lmod) - ((int)&(((struct module *)0) -> list)))
++        printf "%p\t%s\n", $mod, $mod->name
++		set $lmod = $lmod->next
++    end
++end
++document mod-list
++mod-list
++List all modules in the form: <module-address> <module-name>
++Use the <module-address> as the argument for the other
++mod-commands: mod-print-symbols, mod-add-symbols.
++end
++
++define mod-list-syms
++    set $lmod = modules->next
++    # This is a circular data structure
++    while $lmod != &modules
++		set $mod = (struct module *)(((char *)$lmod) - ((int)&(((struct module *)0) -> list)))
++        printf "add-symbol-file %s.ko %p\n", $mod->name, $mod->module_core
++		set $lmod = $lmod->next
++    end
++end
++document mod-list-syms
++mod-list-syms
++List all modules in the form: add-symbol-file <module-path> <module-core>
++for adding modules' symbol tables without loadmodule.sh.
++end
++
++define mod-validate
++    set $lmod = modules->next
++	set $mod = (struct module *)(((char *)$lmod) - ((int)&(((struct module *)0) -> list)))
++    while ($lmod != &modules) && ($mod != $arg0)
++        set $lmod = $lmod->next
++	    set $mod = (struct module *)(((char *)$lmod) - ((int)&(((struct module *)0) -> list)))
++    end
++    if $lmod == &modules
++    	set $mod = 0
++        printf "%p is not a module\n", $arg0
++    end
++end
++document mod-validate
++mod-validate <module-address>
++Internal user-command used to validate the module parameter.
++If <module> is a real loaded module, set $mod to it, otherwise set $mod
++to 0.
++end
++
++define mod-print-symbols
++    mod-validate $arg0
++    if $mod != 0
++		set $i = 0
++		while $i < $mod->num_syms
++			set $sym = $mod->syms[$i]
++			printf "%p\t%s\n", $sym->value, $sym->name
++			set $i = $i + 1
++		end
++		set $i = 0
++		while $i < $mod->num_gpl_syms
++			set $sym = $mod->gpl_syms[$i]
++			printf "%p\t%s\n", $sym->value, $sym->name
++			set $i = $i + 1
++		end
++    end
++end
++document mod-print-symbols
++mod-print-symbols <module-address>
++Print all exported symbols of the module.  See mod-list
++end
++
+Index: linux-2.6.16.19/Documentation/i386/kgdb/gdbinit.hw
+===================================================================
+--- linux-2.6.16.19.orig/Documentation/i386/kgdb/gdbinit.hw	2006-04-21 14:09:39.654033432 -0700
++++ linux-2.6.16.19/Documentation/i386/kgdb/gdbinit.hw	2006-06-23 17:06:11.000000000 -0700
+@@ -0,0 +1,117 @@
++
++#Using ia-32 hardware breakpoints.
++#
++#4 hardware breakpoints are available in ia-32 processors. These breakpoints
++#do not need code modification. They are set using debug registers.
++#
++#Each hardware breakpoint can be of one of the
++#three types: execution, write, access.
++#1. An Execution breakpoint is triggered when code at the breakpoint address is
++#executed.
++#2. A write breakpoint ( aka watchpoints ) is triggered when memory location
++#at the breakpoint address is written.
++#3. An access breakpoint is triggered when memory location at the breakpoint
++#address is either read or written.
++#
++#As hardware breakpoints are available in limited number, use software
++#breakpoints ( br command in gdb ) instead of execution hardware breakpoints.
++#
++#Length of an access or a write breakpoint defines length of the datatype to
++#be watched. Length is 1 for char, 2 short , 3 int.
++#
++#For placing execution, write and access breakpoints, use commands
++#hwebrk, hwwbrk, hwabrk
++#To remove a breakpoint use hwrmbrk command.
++#
++#These commands take following types of arguments. For arguments associated
++#with each command, use help command.
++#1. breakpointno: 0 to 3
++#2. length: 1 to 3
++#3. address: Memory location in hex ( without 0x ) e.g c015e9bc
++#
++#Use the command exinfo to find which hardware breakpoint occured.
++
++#hwebrk breakpointno address
++define hwebrk
++	maintenance packet Y$arg0,0,0,$arg1
++end
++document hwebrk
++	hwebrk <breakpointno> <address>
++	Places a hardware execution breakpoint
++	<breakpointno> = 0 - 3
++	<address> = Hex digits without leading "0x".
++end
++
++#hwwbrk breakpointno length address
++define hwwbrk
++	maintenance packet Y$arg0,1,$arg1,$arg2
++end
++document hwwbrk
++	hwwbrk <breakpointno> <length> <address>
++	Places a hardware write breakpoint
++	<breakpointno> = 0 - 3
++	<length> = 1 (1 byte), 2 (2 byte), 3 (4 byte)
++	<address> = Hex digits without leading "0x".
++end
++
++#hwabrk breakpointno length address
++define hwabrk
++	maintenance packet Y$arg0,1,$arg1,$arg2
++end
++document hwabrk
++	hwabrk <breakpointno> <length> <address>
++	Places a hardware access breakpoint
++	<breakpointno> = 0 - 3
++	<length> = 1 (1 byte), 2 (2 byte), 3 (4 byte)
++	<address> = Hex digits without leading "0x".
++end
++
++#hwrmbrk breakpointno
++define hwrmbrk
++	maintenance packet y$arg0
++end
++document hwrmbrk
++	hwrmbrk <breakpointno>
++	<breakpointno> = 0 - 3
++	Removes a hardware breakpoint
++end
++
++define reboot
++        maintenance packet r
++end
++#exinfo
++define exinfo
++	maintenance packet qE
++end
++document exinfo
++	exinfo
++	Gives information about a breakpoint.
++end
++define get_th
++	p $th=(struct thread_info *)((int)$esp & ~8191)
++end
++document get_th
++	get_tu
++	Gets and prints the current thread_info pointer, Defines th to be it.
++end
++define get_cu
++	p $cu=((struct thread_info *)((int)$esp & ~8191))->task
++end
++document get_cu
++	get_cu
++	Gets and print the "current" value.  Defines $cu to be it.
++end
++define int_off
++	set var $flags=$eflags
++	set $eflags=$eflags&~0x200
++	end
++define int_on
++	set var $eflags|=$flags&0x200
++	end
++document int_off
++	saves the current interrupt state and clears the processor interrupt
++	flag.  Use int_on to restore the saved flag.
++end
++document int_on
++	Restores the interrupt flag saved by int_off.
++end
+Index: linux-2.6.16.19/Documentation/i386/kgdb/kgdb.txt
+===================================================================
+--- linux-2.6.16.19.orig/Documentation/i386/kgdb/kgdb.txt	2006-04-21 14:09:39.654033432 -0700
++++ linux-2.6.16.19/Documentation/i386/kgdb/kgdb.txt	2006-06-23 17:06:11.000000000 -0700
+@@ -0,0 +1,775 @@
++Last edit: <20030806.1637.12>
++This file has information specific to the i386 kgdb option.  Other
++platforms with the kgdb option may behave in a similar fashion.
++
++New features:
++============
++20030806.1557.37
++This version was made against the 2.6.0-test2 kernel. We have made the
++following changes:
++
++- The getthread() code in the stub calls find_task_by_pid().  It fails
++  if we are early in the bring up such that the pid arrays have yet to
++  be allocated.  We have added a line to kernel/pid.c to make
++  "kgdb_pid_init_done" true once the arrays are allocated.  This way the
++  getthread() code knows not to call.  This is only used by the thread
++  debugging stuff and threads will not yet exist at this point in the
++  boot.
++
++- For some reason, gdb was not asking for a new thread list when the
++  "info thread" command was given.  We changed to the newer version of
++  the thread info command and gdb now seems to ask when needed.  Result,
++  we now get all threads in the thread list.
++
++- We now respond to the ThreadExtraInfo request from gdb with the thread
++  name from task_struct .comm.  This then appears in the thread list.
++  Thoughts on additional options for this are welcome.  Things such as
++  "has BKL" and "Preempted" come to mind.  I think we could have a flag
++  word that could enable different bits of info here.
++
++- We now honor, sort of, the C and S commands.  These are continue and
++  single set after delivering a signal.  We ignore the signal and do the
++  requested action.  This only happens when we told gdb that a signal
++  was the reason for entry, which is only done on memory faults.  The
++  result is that you can now continue into the Oops.
++
++- We changed the -g to -gdwarf-2.  This seems to be the same as -ggdb,
++  but it is more exact on what language to use.
++
++- We added two dwarf2 include files and a bit of code at the end of
++  entry.S.  This does not yet work, so it is disabled.  Still we want to
++  keep track of the code and "maybe" someone out there can fix it.
++
++- Randy Dunlap sent some fix ups for this file which are now merged.
++
++- Hugh Dickins sent a fix to a bit of code in traps.c that prevents a
++  compiler warning if CONFIG_KGDB is off (now who would do that :).
++
++- Andrew Morton sent a fix for the serial driver which is now merged.
++
++- Andrew also sent a change to the stub around the cpu managment code
++  which is also merged.
++
++- Andrew also sent a patch to make "f" as well as "g" work as SysRq
++  commands to enter kgdb, merged.
++
++- If CONFIG_KGDB and CONFIG_DEBUG_SPINLOCKS are both set we added a
++  "who" field to the spinlock data struct.  This is filled with
++  "current" when ever the spinlock suceeds.  Useful if you want to know
++  who has the lock.
++
++_ And last, but not least, we fixed the "get_cu" macro to properly get
++  the current value of "current".
++
++New features:
++============
++20030505.1827.27
++We are starting to align with the sourceforge version, at least in
++commands.  To this end, the boot command string to start kgdb at
++boot time has been changed from "kgdb" to "gdb".
++
++Andrew Morton sent a couple of patches which are now included as follows:
++1.) We now return a flag to the interrupt handler.
++2.) We no longer use smp_num_cpus (a conflict with the lock meter).
++3.) And from William Lee Irwin III <wli@holomorphy.com> code to make
++    sure high-mem is set up before we attempt to register our interrupt
++    handler.
++We now include asm/kgdb.h from config.h so you will most likely never
++have to include it.  It also 'NULLS' the kgdb macros you might have in
++your code when CONFIG_KGDB is not defined.  This allows you to just
++turn off CONFIG_KGDB to turn off all the kgdb_ts() calls and such.
++This include is conditioned on the machine being an x86 so as to not
++mess with other archs.
++
++20020801.1129.03
++This is currently the version for the 2.4.18 (and beyond?) kernel.
++
++We have several new "features" beginning with this version:
++
++1.) Kgdb now syncs the "other" CPUs with a cross-CPU NMI.  No more
++    waiting and it will pull that guy out of an IRQ off spin lock :)
++
++2.) We doctored up the code that tells where a task is waiting and
++    included it so that the "info thread" command will show a bit more
++    than "schedule()".  Try it...
++
++3.) Added the ability to call a function from gdb.  All the standard gdb
++    issues apply, i.e. if you hit a breakpoint in the function, you are
++    not allowed to call another (gdb limitation, not kgdb).  To help
++    this capability we added a memory allocation function.  Gdb does not
++    return this memory (it is used for strings that you pass to that function
++    you are calling from gdb) so we fixed up a way to allow you to
++    manually return the memory (see below).
++
++4.) Kgdb time stamps (kgdb_ts()) are enhanced to expand what was the
++    interrupt flag to now also include the preemption count and the
++    "in_interrupt" info.  The flag is now called "with_pif" to indicate
++    the order, preempt_count, in_interrupt, flag.  The preempt_count is
++    shifted left by 4 bits so you can read the count in hex by dropping
++    the low order digit.  In_interrupt is in bit 1, and the flag is in
++    bit 0.
++
++5.) The command: "p kgdb_info" is now expanded and prints something
++    like:
++(gdb) p kgdb_info
++$2 = {used_malloc = 0, called_from = 0xc0107506, entry_tsc = 67468627259,
++  errcode = 0, vector = 3, print_debug_info = 0, hold_on_sstep = 1,
++  cpus_waiting = {{task = 0xc027a000, pid = 32768, hold = 0,
++      regs = 0xc027bf84}, {task = 0x0, pid = 0, hold = 0, regs = 0x0}}}
++
++    Things to note here: a.) used_malloc is the amount of memory that
++    has been malloc'ed to do calls from gdb.  You can reclaim this
++    memory like this: "p kgdb_info.used_malloc=0" Cool, huh?  b.)
++    cpus_waiting is now "sized" by the number of CPUs you enter at
++    configure time in the kgdb configure section.  This is NOT used
++    anywhere else in the system, but it is "nice" here.  c.)  The task's
++    "pid" is now in the structure.  This is the pid you will need to use
++    to decode to the thread id to get gdb to look at that thread.
++    Remember that the "info thread" command prints a list of threads
++    wherein it numbers each thread with its reference number followed
++    by the thread's pid.  Note that the per-CPU idle threads actually
++    have pids of 0 (yes, there is more than one pid 0 in an SMP system).
++    To avoid confusion, kgdb numbers these threads with numbers beyond
++    the MAX_PID.  That is why you see 32768 and above.
++
++6.) A subtle change, we now provide the complete register set for tasks
++    that are active on the other CPUs.  This allows better trace back on
++    those tasks.
++
++    And, let's mention what we could not fix.  Back-trace from all but the
++    thread that we trapped will, most likely, have a bogus entry in it.
++    The problem is that gdb does not recognize the entry code for
++    functions that use "current" near (at all?) the entry.  The compiler
++    is putting the "current" decode as the first two instructions of the
++    function where gdb expects to find %ebp changing code.  Back trace
++    also has trouble with interrupt frames.  I am talking with Daniel
++    Jacobowitz about some way to fix this, but don't hold your breath.
++
++20011220.0050.35
++Major enhancement with this version is the ability to hold one or more
++CPUs in an SMP system while allowing the others to continue.  Also, by
++default only the current CPU is enabled on single-step commands (please
++note that gdb issues single-step commands at times other than when you
++use the si command).
++
++Another change is to collect some useful information in
++a global structure called "kgdb_info".  You should be able to just:
++
++p kgdb_info
++
++although I have seen cases where the first time this is done gdb just
++prints the first member but prints the whole structure if you then enter
++CR (carriage return or enter).  This also works:
++
++p *&kgdb_info
++
++Here is a sample:
++(gdb) p kgdb_info
++$4 = {called_from = 0xc010732c, entry_tsc = 32804123790856, errcode = 0,
++  vector = 3, print_debug_info = 0}
++
++"Called_from" is the return address from the current entry into kgdb.
++Sometimes it is useful to know why you are in kgdb, for example, was
++it an NMI or a real breakpoint?  The simple way to interrogate this
++return address is:
++
++l *0xc010732c
++
++which will print the surrounding few lines of source code.
++
++"Entry_tsc" is the CPU TSC on entry to kgdb (useful to compare to the
++kgdb_ts entries).
++
++"errcode" and "vector" are other entry parameters which may be helpful on
++some traps.
++
++"print_debug_info" is the internal debugging kgdb print enable flag.  Yes,
++you can modify it.
++
++In SMP systems kgdb_info also includes the "cpus_waiting" structure and
++"hold_on_step":
++
++(gdb) p kgdb_info
++$7 = {called_from = 0xc0112739, entry_tsc = 1034936624074, errcode = 0,
++  vector = 2, print_debug_info = 0, hold_on_sstep = 1, cpus_waiting = {{
++      task = 0x0, hold = 0, regs = 0x0}, {task = 0xc71b8000, hold = 0,
++      regs = 0xc71b9f70}, {task = 0x0, hold = 0, regs = 0x0}, {task = 0x0,
++      hold = 0, regs = 0x0}, {task = 0x0, hold = 0, regs = 0x0}, {task = 0x0,
++      hold = 0, regs = 0x0}, {task = 0x0, hold = 0, regs = 0x0}, {task = 0x0,
++      hold = 0, regs = 0x0}}}
++
++"Cpus_waiting" has an entry for each CPU other than the current one that
++has been stopped.  Each entry contains the task_struct address for that
++CPU, the address of the regs for that task and a hold flag.  All these
++have the proper typing so that, for example:
++
++p *kgdb_info.cpus_waiting[1].regs
++
++will print the registers for CPU 1.
++
++"Hold_on_sstep" is a new feature with this version and comes up set or
++true.  What this means is that whenever kgdb is asked to single-step all
++other CPUs are held (i.e. not allowed to execute).  The flag applies to
++all but the current CPU and, again, can be changed:
++
++p kgdb_info.hold_on_sstep=0
++
++restores the old behavior of letting all CPUs run during single-stepping.
++
++Likewise, each CPU has a "hold" flag, which if set, locks that CPU out
++of execution.  Note that this has some risk in cases where the CPUs need
++to communicate with each other.  If kgdb finds no CPU available on exit,
++it will push a message thru gdb and stay in kgdb.  Note that it is legal
++to hold the current CPU as long as at least one CPU can execute.
++
++20010621.1117.09
++This version implements an event queue.  Events are signaled by calling
++a function in the kgdb stub and may be examined from gdb.  See EVENTS
++below for details.  This version also tightens up the interrupt and SMP
++handling to not allow interrupts on the way to kgdb from a breakpoint
++trap.  It is fine to allow these interrupts for user code, but not
++system debugging.
++
++Version
++=======
++
++This version of the kgdb package was developed and tested on
++kernel version 2.4.16.  It will not install on any earlier kernels.
++It is possible that it will continue to work on later versions
++of 2.4 and then versions of 2.5 (I hope).
++
++
++Debugging Setup
++===============
++
++Designate one machine as the "development" machine.  This is the
++machine on which you run your compiles and which has your source
++code for the kernel.  Designate a second machine as the "target"
++machine.  This is the machine that will run your experimental
++kernel.
++
++The two machines will be connected together via a serial line out
++one or the other of the COM ports of the PC.  You will need the
++appropriate modem eliminator (null modem) cable(s) for this.
++
++Decide on which tty port you want the machines to communicate, then
++connect them up back-to-back using the null modem cable.  COM1 is
++/dev/ttyS0 and COM2 is /dev/ttyS1. You should test this connection
++with the two machines prior to trying to debug a kernel.  Once you
++have it working, on the TARGET machine, enter:
++
++setserial /dev/ttyS0 (or what ever tty you are using)
++
++and record the port address and the IRQ number.
++
++On the DEVELOPMENT machine you need to apply the patch for the kgdb
++hooks.  You have probably already done that if you are reading this
++file.
++
++On your DEVELOPMENT machine, go to your kernel source directory and do
++"make Xconfig" where X is one of "x", "menu", or "".  If you are
++configuring in the standard serial driver, it must not be a module.
++Either yes or no is ok, but making the serial driver a module means it
++will initialize after kgdb has set up the UART interrupt code and may
++cause a failure of the control-C option discussed below.  The configure
++question for the serial driver is under the "Character devices" heading
++and is:
++
++"Standard/generic (8250/16550 and compatible UARTs) serial support"
++
++Go down to the kernel debugging menu item and open it up.  Enable the
++kernel kgdb stub code by selecting that item.  You can also choose to
++turn on the "-ggdb -O1" compile options.  The -ggdb causes the compiler
++to put more debug info (like local symbols) in the object file.  On the
++i386 -g and -ggdb are the same so this option just reduces to "O1".  The
++-O1 reduces the optimization level.  This may be helpful in some cases,
++be aware, however, that this may also mask the problem you are looking
++for.
++
++The baud rate.  Default is 115200.  What ever you choose be sure that
++the host machine is set to the same speed.  I recommend the default.
++
++The port.  This is the I/O address of the serial UART that you should
++have gotten using setserial as described above.  The standard COM1 port
++(3f8) using IRQ 4 is default.  COM2 is 2f8 which by convention uses IRQ
++3.
++
++The port IRQ (see above).
++
++Stack overflow test.  This option makes a minor change in the trap,
++system call and interrupt code to detect stack overflow and transfer
++control to kgdb if it happens.  (Some platforms have this in the
++baseline code, but the i386 does not.)
++
++You can also configure the system to recognize the boot option
++"console=kgdb" which if given will cause all console output during
++booting to be put thru gdb as well as other consoles.  This option
++requires that gdb and kgdb be connected prior to sending console output
++so, if they are not, a breakpoint is executed to force the connection.
++This will happen before any kernel output (it is going thru gdb, right),
++and will stall the boot until the connection is made.
++
++You can also configure in a patch to SysRq to enable the kGdb SysRq.
++This request generates a breakpoint.  Since the serial port IRQ line is
++set up after any serial drivers, it is possible that this command will
++work when the control-C will not.
++
++Save and exit the Xconfig program.  Then do "make clean" , "make dep"
++and "make bzImage" (or whatever target you want to make).  This gets the
++kernel compiled with the "-g" option set -- necessary for debugging.
++
++You have just built the kernel on your DEVELOPMENT machine that you
++intend to run on your TARGET machine.
++
++To install this new kernel, use the following installation procedure.
++Remember, you are on the DEVELOPMENT machine patching the kernel source
++for the kernel that you intend to run on the TARGET machine.
++
++Copy this kernel to your target machine using your usual procedures.  I
++usually arrange to copy development:
++/usr/src/linux/arch/i386/boot/bzImage to /vmlinuz on the TARGET machine
++via a LAN based NFS access.  That is, I run the cp command on the target
++and copy from the development machine via the LAN.  Run Lilo (see "man
++lilo" for details on how to set this up) on the new kernel on the target
++machine so that it will boot!  Then boot the kernel on the target
++machine.
++
++On the DEVELOPMENT machine, create a file called .gdbinit in the
++directory /usr/src/linux.  An example .gdbinit file looks like this:
++
++shell echo -e "\003" >/dev/ttyS0
++set remotebaud 38400 (or what ever speed you have chosen)
++target remote /dev/ttyS0
++
++
++Change the "echo" and "target" definition so that it specifies the tty
++port that you intend to use.  Change the "remotebaud" definition to
++match the data rate that you are going to use for the com line.
++
++You are now ready to try it out.
++
++Boot your target machine with "kgdb" in the boot command i.e. something
++like:
++
++lilo> test kgdb
++
++or if you also want console output thru gdb:
++
++lilo> test kgdb console=kgdb
++
++You should see the lilo message saying it has loaded the kernel and then
++all output stops.  The kgdb stub is trying to connect with gdb.  Start
++gdb something like this:
++
++
++On your DEVELOPMENT machine, cd /usr/src/linux and enter "gdb vmlinux".
++When gdb gets the symbols loaded it will read your .gdbinit file and, if
++everything is working correctly, you should see gdb print out a few
++lines indicating that a breakpoint has been taken.  It will actually
++show a line of code in the target kernel inside the kgdb activation
++code.
++
++The gdb interaction should look something like this:
++
++    linux-dev:/usr/src/linux# gdb vmlinux
++    GDB is free software and you are welcome to distribute copies of it
++     under certain conditions; type "show copying" to see the conditions.
++    There is absolutely no warranty for GDB; type "show warranty" for details.
++    GDB 4.15.1 (i486-slackware-linux),
++    Copyright 1995 Free Software Foundation, Inc...
++    breakpoint () at i386-stub.c:750
++    750     }
++    (gdb)
++
++You can now use whatever gdb commands you like to set breakpoints.
++Enter "continue" to start your target machine executing again.  At this
++point the target system will run at full speed until it encounters
++your breakpoint or gets a segment violation in the kernel, or whatever.
++
++If you have the kgdb console enabled when you continue, gdb will print
++out all the console messages.
++
++The above example caused a breakpoint relatively early in the boot
++process.  For the i386 kgdb it is possible to code a break instruction
++as the first C-language point in init/main.c, i.e. as the first instruction
++in start_kernel().  This could be done as follows:
++
++#include <asm/kgdb.h>
++	 breakpoint();
++
++This breakpoint() is really a function that sets up the breakpoint and
++single-step hardware trap cells and then executes a breakpoint.  Any
++early hard coded breakpoint will need to use this function.  Once the
++trap cells are set up they need not be set again, but doing it again
++does not hurt anything, so you don't need to be concerned about which
++breakpoint is hit first.  Once the trap cells are set up (and the kernel
++sets them up in due course even if breakpoint() is never called) the
++macro:
++
++BREAKPOINT;
++
++will generate an inline breakpoint.  This may be more useful as it stops
++the processor at the instruction instead of in a function a step removed
++from the location of interest.  In either case <asm/kgdb.h> must be
++included to define both breakpoint() and BREAKPOINT.
++
++Triggering kgdbstub at other times
++==================================
++
++Often you don't need to enter the debugger until much later in the boot
++or even after the machine has been running for some time.  Once the
++kernel is booted and interrupts are on, you can force the system to
++enter the debugger by sending a control-C to the debug port. This is
++what the first line of the recommended .gdbinit file does.  This allows
++you to start gdb any time after the system is up as well as when the
++system is already at a breakpoint.  (In the case where the system is
++already at a breakpoint the control-C is not needed, however, it will
++be ignored by the target so no harm is done.  Also note the the echo
++command assumes that the port speed is already set.  This will be true
++once gdb has connected, but it is best to set the port speed before you
++run gdb.)
++
++Another simple way to do this is to put the following file in you ~/bin
++directory:
++
++#!/bin/bash
++echo  -e "\003"  > /dev/ttyS0
++
++Here, the ttyS0 should be replaced with what ever port you are using.
++The "\003" is control-C.  Once you are connected with gdb, you can enter
++control-C at the command prompt.
++
++An alternative way to get control to the debugger is to enable the kGdb
++SysRq command.  Then you would enter Alt-SysRq-g (all three keys at the
++same time, but push them down in the order given).  To refresh your
++memory of the available SysRq commands try Alt-SysRq-=.  Actually any
++undefined command could replace the "=", but I like to KNOW that what I
++am pushing will never be defined.
++
++Debugging hints
++===============
++
++You can break into the target machine at any time from the development
++machine by typing ^C (see above paragraph).  If the target machine has
++interrupts enabled this will stop it in the kernel and enter the
++debugger.
++
++There is unfortunately no way of breaking into the kernel if it is
++in a loop with interrupts disabled, so if this happens to you then
++you need to place exploratory breakpoints or printk's into the kernel
++to find out where it is looping.  The exploratory breakpoints can be
++entered either thru gdb or hard coded into the source.  This is very
++handy if you do something like:
++
++if (<it hurts>) BREAKPOINT;
++
++
++There is a copy of an e-mail in the Documentation/i386/kgdb/ directory
++(debug-nmi.txt) which describes how to create an NMI on an ISA bus
++machine using a paper clip.  I have a sophisticated version of this made
++by wiring a push button switch into a PC104/ISA bus adapter card.  The
++adapter card nicely furnishes wire wrap pins for all the ISA bus
++signals.
++
++When you are done debugging the kernel on the target machine it is a
++good idea to leave it in a running state.  This makes reboots faster,
++bypassing the fsck.  So do a gdb "continue" as the last gdb command if
++this is possible.  To terminate gdb itself on the development machine
++and leave the target machine running, first clear all breakpoints and
++continue, then type ^Z to suspend gdb and then kill it with "kill %1" or
++something similar.
++
++If gdbstub Does Not Work
++========================
++
++If it doesn't work, you will have to troubleshoot it.  Do the easy
++things first like double checking your cabling and data rates.  You
++might try some non-kernel based programs to see if the back-to-back
++connection works properly.  Just something simple like cat /etc/hosts
++>/dev/ttyS0 on one machine and cat /dev/ttyS0 on the other will tell you
++if you can send data from one machine to the other.  Make sure it works
++in both directions.  There is no point in tearing out your hair in the
++kernel if the line doesn't work.
++
++All of the real action takes place in the file
++/usr/src/linux/arch/i386/kernel/kgdb_stub.c.  That is the code on the target
++machine that interacts with gdb on the development machine.  In gdb you can
++turn on a debug switch with the following command:
++
++	set remotedebug
++
++This will print out the protocol messages that gdb is exchanging with
++the target machine.
++
++Another place to look is /usr/src/arch/i386/lib/kgdb_serial.c. This is
++the code that talks to the serial port on the target side.  There might
++be a problem there.  In particular there is a section of this code that
++tests the UART which will tell you what UART you have if you define
++"PRNT" (just remove "_off" from the #define PRNT_off).  To view this
++report you will need to boot the system without any beakpoints.  This
++allows the kernel to run to the point where it calls kgdb to set up
++interrupts.  At this time kgdb will test the UART and print out the type
++it finds.  (You need to wait so that the printks are actually being
++printed.  Early in the boot they are cached, waiting for the console to
++be enabled.  Also, if kgdb is entered thru a breakpoint it is possible
++to cause a dead lock by calling printk when the console is locked.  The
++stub thus avoids doing printks from breakpoints, especially in the
++serial code.)  At this time, if the UART fails to do the expected thing,
++kgdb will print out (using printk) information on what failed.  (These
++messages will be buried in all the other boot up messages.  Look for
++lines that start with "gdb_hook_interrupt:".  You may want to use dmesg
++once the system is up to view the log.  If this fails or if you still
++don't connect, review your answers for the port address.  Use:
++
++setserial /dev/ttyS0
++
++to get the current port and IRQ information.  This command will also
++tell you what the system found for the UART type. The stub recognizes
++the following UART types:
++
++16450, 16550, and 16550A
++
++If you are really desperate you can use printk debugging in the
++kgdbstub code in the target kernel until you get it working.  In particular,
++there is a global variable in /usr/src/linux/arch/i386/kernel/kgdb_stub.c
++named "remote_debug".  Compile your kernel with this set to 1, rather
++than 0 and the debug stub will print out lots of stuff as it does
++what it does.  Likewise there are debug printks in the kgdb_serial.c
++code that can be turned on with simple changes in the macro defines.
++
++
++Debugging Loadable Modules
++==========================
++
++This technique comes courtesy of Edouard Parmelan
++<Edouard.Parmelan@quadratec.fr>
++
++When you run gdb, enter the command
++
++source gdbinit-modules
++
++This will read in a file of gdb macros that was installed in your
++kernel source directory when kgdb was installed.  This file implements
++the following commands:
++
++mod-list
++    Lists the loaded modules in the form <module-address> <module-name>
++
++mod-print-symbols <module-address>
++    Prints all the symbols in the indicated module.
++
++mod-add-symbols <module-address> <object-file-path-name>
++    Loads the symbols from the object file and associates them
++    with the indicated module.
++
++After you have loaded the module that you want to debug, use the command
++mod-list to find the <module-address> of your module.  Then use that
++address in the mod-add-symbols command to load your module's symbols.
++From that point onward you can debug your module as if it were a part
++of the kernel.
++
++The file gdbinit-modules also contains a command named mod-add-lis as
++an example of how to construct a command of your own to load your
++favorite module.  The idea is to "can" the pathname of the module
++in the command so you don't have to type so much.
++
++Threads
++=======
++
++Each process in a target machine is seen as a gdb thread. gdb thread
++related commands (info threads, thread n) can be used.
++
++ia-32 hardware breakpoints
++==========================
++
++kgdb stub contains support for hardware breakpoints using debugging features
++of ia-32(x86) processors. These breakpoints do not need code modification.
++They use debugging registers. 4 hardware breakpoints are available in ia-32
++processors.
++
++Each hardware breakpoint can be of one of the following three types.
++
++1. Execution breakpoint - An Execution breakpoint is triggered when code
++	at the breakpoint address is executed.
++
++	As limited number of hardware breakpoints are available, it is
++	advisable to use software breakpoints ( break command ) instead
++	of execution hardware breakpoints, unless modification of code
++	is to be avoided.
++
++2. Write breakpoint - A write breakpoint is triggered when memory
++	location at the breakpoint address is written.
++
++	A write or can be placed for data of variable length. Length of
++	a write breakpoint indicates length of the datatype to be
++	watched. Length is 1 for 1 byte data , 2 for 2 byte data, 3 for
++	4 byte data.
++
++3. Access breakpoint - An access breakpoint is triggered when memory
++	location at the breakpoint address is either read or written.
++
++	Access breakpoints also have lengths similar to write breakpoints.
++
++IO breakpoints in ia-32 are not supported.
++
++Since gdb stub at present does not use the protocol used by gdb for hardware
++breakpoints, hardware breakpoints are accessed through gdb macros. gdb macros
++for hardware breakpoints are described below.
++
++hwebrk	- Places an execution breakpoint
++	hwebrk breakpointno address
++hwwbrk	- Places a write breakpoint
++	hwwbrk breakpointno length address
++hwabrk	- Places an access breakpoint
++	hwabrk breakpointno length address
++hwrmbrk	- Removes a breakpoint
++	hwrmbrk breakpointno
++exinfo	- Tells whether a software or hardware breakpoint has occurred.
++	Prints number of the hardware breakpoint if a hardware breakpoint has
++	occurred.
++
++Arguments required by these commands are as follows
++breakpointno	- 0 to 3
++length		- 1 to 3
++address		- Memory location in hex digits ( without 0x ) e.g c015e9bc
++
++SMP support
++==========
++
++When a breakpoint occurs or user issues a break ( Ctrl + C ) to gdb
++client, all the processors are forced to enter the debugger. Current
++thread corresponds to the thread running on the processor where
++breakpoint occurred.  Threads running on other processor(s) appear
++similar to other non-running threads in the 'info threads' output.
++Within the kgdb stub there is a structure "waiting_cpus" in which kgdb
++records the values of "current" and "regs" for each CPU other than the
++one that hit the breakpoint.  "current" is a pointer to the task
++structure for the task that CPU is running, while "regs" points to the
++saved registers for the task.  This structure can be examined with the
++gdb "p" command.
++
++ia-32 hardware debugging registers on all processors are set to same
++values.  Hence any hardware breakpoints may occur on any processor.
++
++gdb troubleshooting
++===================
++
++1. gdb hangs
++Kill it. restart gdb. Connect to target machine.
++
++2. gdb cannot connect to target machine (after killing a gdb and
++restarting another) If the target machine was not inside debugger when
++you killed gdb, gdb cannot connect because the target machine won't
++respond.  In this case echo "Ctrl+C"(ASCII 3) to the serial line.
++e.g. echo -e "\003" > /dev/ttyS1
++This forces that target machine into the debugger, after which you
++can connect.
++
++3. gdb cannot connect even after echoing Ctrl+C into serial line
++Try changing serial line settings min to 1 and time to 0
++e.g. stty min 1 time 0 < /dev/ttyS1
++Try echoing again
++
++Check serial line speed and set it to correct value if required
++e.g. stty ispeed 115200 ospeed 115200 < /dev/ttyS1
++
++EVENTS
++======
++
++Ever want to know the order of things happening?  Which CPU did what and
++when?  How did the spinlock get the way it is?  Then events are for
++you.  Events are defined by calls to an event collection interface and
++saved for later examination.  In this case, kgdb events are saved by a
++very fast bit of code in kgdb which is fully SMP and interrupt protected
++and they are examined by using gdb to display them.  Kgdb keeps only
++the last N events, where N must be a power of two and is defined at
++configure time.
++
++
++Events are signaled to kgdb by calling:
++
++kgdb_ts(data0,data1)
++
++For each call kgdb records each call in an array along with other info.
++Here is the array definition:
++
++struct kgdb_and_then_struct {
++#ifdef CONFIG_SMP
++	int	on_cpu;
++#endif
++	long long at_time;
++	int  	from_ln;
++	char	* in_src;
++	void	*from;
++        int     with_if;
++	int	data0;
++	int	data1;
++};
++
++For SMP machines the CPU is recorded, for all machines the TSC is
++recorded (gets a time stamp) as well as the line number and source file
++the call was made from.  The address of the (from), the "if" (interrupt
++flag) and the two data items are also recorded.  The macro kgdb_ts casts
++the types to int, so you can put any 32-bit values here.  There is a
++configure option to select the number of events you want to keep.  A
++nice number might be 128, but you can keep up to 1024 if you want.  The
++number must be a power of two.  An "andthen" macro library is provided
++for gdb to help you look at these events.  It is also possible to define
++a different structure for the event storage and cast the data to this
++structure.  For example the following structure is defined in kgdb:
++
++struct kgdb_and_then_struct2 {
++#ifdef CONFIG_SMP
++	int	on_cpu;
++#endif
++	long long at_time;
++	int  	from_ln;
++	char	* in_src;
++	void	*from;
++        int     with_if;
++	struct task_struct *t1;
++	struct task_struct *t2;
++};
++
++If you use this for display, the data elements will be displayed as
++pointers to task_struct entries.  You may want to define your own
++structure to use in casting.  You should only change the last two items
++and you must keep the structure size the same.  Kgdb will handle these
++as 32-bit ints, but within that constraint you can define a structure to
++cast to any 32-bit quantity.  This need only be available to gdb and is
++only used for casting in the display code.
++
++Final Items
++===========
++
++I picked up this code from Amit S. Kale and enhanced it.
++
++If you make some really cool modification to this stuff, or if you
++fix a bug, please let me know.
++
++George Anzinger
++<george@mvista.com>
++
++Amit S. Kale
++<akale@veritas.com>
++
++(First kgdb by David Grothe <dave@gcom.com>)
++
++(modified by Tigran Aivazian <tigran@sco.com>)
++    Putting gdbstub into the kernel config menu.
++
++(modified by Scott Foehner <sfoehner@engr.sgi.com>)
++    Hooks for entering gdbstub at boot time.
++
++(modified by Amit S. Kale <akale@veritas.com>)
++    Threads, ia-32 hw debugging, mp support, console support,
++    nmi watchdog handling.
++
++(modified by George Anzinger <george@mvista.com>)
++    Extended threads to include the idle threads.
++    Enhancements to allow breakpoint() at first C code.
++    Use of module_init() and __setup() to automate the configure.
++    Enhanced the cpu "collection" code to work in early bring-up.
++    Added ability to call functions from gdb
++    Print info thread stuff without going back to schedule()
++    Now collect the "other" cpus with an IPI/ NMI.
+Index: linux-2.6.16.19/Documentation/i386/kgdb/loadmodule.sh
+===================================================================
+--- linux-2.6.16.19.orig/Documentation/i386/kgdb/loadmodule.sh	2006-04-21 14:09:39.654033432 -0700
++++ linux-2.6.16.19/Documentation/i386/kgdb/loadmodule.sh	2006-06-23 17:06:11.000000000 -0700
+@@ -0,0 +1,78 @@
++#/bin/sh
++# This script loads a module on a target machine and generates a gdb script.
++# source generated gdb script to load the module file at appropriate addresses
++# in gdb.
++#
++# Usage:
++# Loading the module on target machine and generating gdb script)
++#	[foo]$ loadmodule.sh <modulename>
++#
++# Loading the module file into gdb
++#	(gdb) source <gdbscriptpath>
++#
++# Modify following variables according to your setup.
++#	TESTMACHINE - Name of the target machine
++#	GDBSCRIPTS - The directory where a gdb script will be generated
++#
++# Author: Amit S. Kale (akale@veritas.com).
++#
++# If you run into problems, please check files pointed to by following
++# variables.
++#	ERRFILE - /tmp/<modulename>.errs contains stderr output of insmod
++#	MAPFILE - /tmp/<modulename>.map contains stdout output of insmod
++#	GDBSCRIPT - $GDBSCRIPTS/load<modulename> gdb script.
++
++TESTMACHINE=foo
++GDBSCRIPTS=/home/bar
++
++if [ $# -lt 1 ] ; then {
++	echo Usage: $0 modulefile
++	exit
++} ; fi
++
++MODULEFILE=$1
++MODULEFILEBASENAME=`basename $1`
++
++if [ $MODULEFILE = $MODULEFILEBASENAME ] ; then {
++	MODULEFILE=`pwd`/$MODULEFILE
++} fi
++
++ERRFILE=/tmp/$MODULEFILEBASENAME.errs
++MAPFILE=/tmp/$MODULEFILEBASENAME.map
++GDBSCRIPT=$GDBSCRIPTS/load$MODULEFILEBASENAME
++
++function findaddr() {
++	local ADDR=0x$(echo "$SEGMENTS" | \
++		grep "$1" | sed 's/^[^ ]*[ ]*[^ ]*[ ]*//' | \
++		sed 's/[ ]*[^ ]*$//')
++	echo $ADDR
++}
++
++function checkerrs() {
++	if [ "`cat $ERRFILE`" != "" ] ; then {
++		cat $ERRFILE
++		exit
++	} fi
++}
++
++#load the module
++echo Copying $MODULEFILE to $TESTMACHINE
++rcp $MODULEFILE root@${TESTMACHINE}:
++
++echo Loading module $MODULEFILE
++rsh -l root $TESTMACHINE  /sbin/insmod -m ./`basename $MODULEFILE` \
++	> $MAPFILE 2> $ERRFILE
++checkerrs
++
++SEGMENTS=`head -n 11 $MAPFILE | tail -n 10`
++TEXTADDR=$(findaddr "\\.text[^.]")
++LOADSTRING="add-symbol-file $MODULEFILE $TEXTADDR"
++SEGADDRS=`echo "$SEGMENTS" | awk '//{
++	if ($1 != ".text" && $1 != ".this" &&
++	    $1 != ".kstrtab" && $1 != ".kmodtab") {
++		print " -s " $1 " 0x" $3 " "
++	}
++}'`
++LOADSTRING="$LOADSTRING $SEGADDRS"
++echo Generating script $GDBSCRIPT
++echo $LOADSTRING > $GDBSCRIPT
+Index: linux-2.6.16.19/MAINTAINERS
+===================================================================
+--- linux-2.6.16.19.orig/MAINTAINERS	2006-06-23 17:05:14.000000000 -0700
++++ linux-2.6.16.19/MAINTAINERS	2006-06-23 17:06:11.000000000 -0700
+@@ -1494,6 +1494,12 @@
+ W:	http://sf.net/projects/kernel-janitor/
+ S:	Maintained
+ 
++KGDB FOR I386 PLATFORM
++P:	George Anzinger
++M:	george@mvista.com
++L:	linux-net@vger.kernel.org
++S:	Supported
++
+ KERNEL NFSD
+ P:	Neil Brown
+ M:	neilb@cse.unsw.edu.au
+Index: linux-2.6.16.19/arch/i386/Kconfig.debug
+===================================================================
+--- linux-2.6.16.19.orig/arch/i386/Kconfig.debug	2006-06-23 17:05:14.000000000 -0700
++++ linux-2.6.16.19/arch/i386/Kconfig.debug	2006-06-23 17:06:11.000000000 -0700
+@@ -72,4 +72,6 @@
+ 	depends on X86_LOCAL_APIC && !X86_VISWS
+ 	default y
+ 
++source "arch/i386/Kconfig.kgdb"
++
+ endmenu
+Index: linux-2.6.16.19/arch/i386/Kconfig.kgdb
+===================================================================
+--- linux-2.6.16.19.orig/arch/i386/Kconfig.kgdb	2006-04-21 14:09:39.654033432 -0700
++++ linux-2.6.16.19/arch/i386/Kconfig.kgdb	2006-06-23 17:06:11.000000000 -0700
+@@ -0,0 +1,178 @@
++config KGDB
++	bool "Include kgdb kernel debugger"
++	depends on DEBUG_KERNEL && !KPROBES
++	help
++	  If you say Y here, the system will be compiled with the debug
++	  option (-g) and a debugging stub will be included in the
++	  kernel.  This stub communicates with gdb on another (host)
++	  computer via a serial port.  The host computer should have
++	  access to the kernel binary file (vmlinux) and a serial port
++	  that is connected to the target machine.  Gdb can be made to
++	  configure the serial port or you can use stty and setserial to
++	  do this. See the 'target' command in gdb. This option also
++	  configures in the ability to request a breakpoint early in the
++	  boot process.  To request the breakpoint just include 'kgdb'
++	  as a boot option when booting the target machine.  The system
++	  will then break as soon as it looks at the boot options.  This
++	  option also installs a breakpoint in panic and sends any
++	  kernel faults to the debugger. For more information see the
++	  Documentation/i386/kgdb/kgdb.txt file.
++
++choice
++	depends on KGDB
++    	prompt "Debug serial port BAUD"
++	default KGDB_115200BAUD
++	help
++	  Gdb and the kernel stub need to agree on the baud rate to be
++	  used.  Some systems (x86 family at this writing) allow this to
++	  be configured.
++
++config KGDB_9600BAUD
++	bool "9600"
++
++config KGDB_19200BAUD
++	bool "19200"
++
++config KGDB_38400BAUD
++	bool "38400"
++
++config KGDB_57600BAUD
++	bool "57600"
++
++config KGDB_115200BAUD
++	bool "115200"
++endchoice
++
++config KGDB_PORT
++	hex "hex I/O port address of the debug serial port"
++	depends on KGDB
++	default  3f8
++	help
++	  Some systems (x86 family at this writing) allow the port
++	  address to be configured.  The number entered is assumed to be
++	  hex, don't put 0x in front of it.  The standard address are:
++	  COM1 3f8 , irq 4 and COM2 2f8 irq 3.  Setserial /dev/ttySx
++	  will tell you what you have.  It is good to test the serial
++	  connection with a live system before trying to debug.
++
++config KGDB_IRQ
++	int "IRQ of the debug serial port"
++	depends on KGDB
++	default 4
++	help
++	  This is the irq for the debug port.  If everything is working
++	  correctly and the kernel has interrupts on a control C to the
++	  port should cause a break into the kernel debug stub.
++
++config KGDB_MORE
++	bool "Add any additional compile options"
++	depends on KGDB
++	default n
++	help
++	  Saying yes here turns on the ability to enter additional
++	  compile options.
++
++
++config KGDB_OPTIONS
++	depends on KGDB_MORE
++	string "Additional compile arguments"
++	default "-O1"
++	help
++	  This option allows you enter additional compile options for
++	  the whole kernel compile.  Each platform will have a default
++	  that seems right for it.  For example on PPC "-ggdb -O1", and
++	  for i386 "-O1".  Note that by configuring KGDB "-g" is already
++	  turned on.  In addition, on i386 platforms
++	  "-fomit-frame-pointer" is deleted from the standard compile
++	  options.
++
++config NO_KGDB_CPUS
++	int "Number of CPUs"
++	depends on KGDB && SMP
++	default NR_CPUS
++	help
++
++	  This option sets the number of cpus for kgdb ONLY.  It is used
++	  to prune some internal structures so they look "nice" when
++	  displayed with gdb.  This is to overcome possibly larger
++	  numbers that may have been entered above.  Enter the real
++	  number to get nice clean kgdb_info displays.
++
++config KGDB_TS
++	bool "Enable kgdb time stamp macros?"
++	depends on KGDB
++	default n
++	help
++	  Kgdb event macros allow you to instrument your code with calls
++	  to the kgdb event recording function.  The event log may be
++	  examined with gdb at a break point.  Turning on this
++	  capability also allows you to choose how many events to
++	  keep. Kgdb always keeps the lastest events.
++
++choice
++	depends on KGDB_TS
++	prompt "Max number of time stamps to save?"
++	default KGDB_TS_128
++
++config KGDB_TS_64
++	bool "64"
++
++config KGDB_TS_128
++	bool "128"
++
++config KGDB_TS_256
++	bool "256"
++
++config KGDB_TS_512
++	bool "512"
++
++config KGDB_TS_1024
++	bool "1024"
++
++endchoice
++
++config STACK_OVERFLOW_TEST
++	bool "Turn on kernel stack overflow testing?"
++	depends on KGDB
++	default n
++	help
++	  This option enables code in the front line interrupt handlers
++	  to check for kernel stack overflow on interrupts and system
++	  calls.  This is part of the kgdb code on x86 systems.
++
++config TRAP_BAD_SYSCALL_EXITS
++	bool "Debug bad system call exits"
++	depends on KGDB
++	help
++	  If you say Y here the kernel will check for system calls which
++	  return without clearing preempt.
++        default n
++
++config KGDB_CONSOLE
++	bool "Enable serial console thru kgdb port"
++	depends on KGDB
++	default n
++	help
++	  This option enables the command line "console=kgdb" option.
++	  When the system is booted with this option in the command line
++	  all kernel printk output is sent to gdb (as well as to other
++	  consoles).  For this to work gdb must be connected.  For this
++	  reason, this command line option will generate a breakpoint if
++	  gdb has not yet connected.  After the gdb continue command is
++	  given all pent up console output will be printed by gdb on the
++	  host machine.  Neither this option, nor KGDB require the
++	  serial driver to be configured.
++
++config KGDB_SYSRQ
++	bool "Turn on SysRq 'G' command to do a break?"
++	depends on KGDB
++	default y
++	help
++	  This option includes an option in the SysRq code that allows
++	  you to enter SysRq G which generates a breakpoint to the KGDB
++	  stub.  This will work if the keyboard is alive and can
++	  interrupt the system.  Because of constraints on when the
++	  serial port interrupt can be enabled, this code may allow you
++	  to interrupt the system before the serial port control C is
++	  available.  Just say yes here.
++
+Index: linux-2.6.16.19/arch/i386/Makefile
+===================================================================
+--- linux-2.6.16.19.orig/arch/i386/Makefile	2006-06-23 17:05:14.000000000 -0700
++++ linux-2.6.16.19/arch/i386/Makefile	2006-06-23 17:06:11.000000000 -0700
+@@ -81,6 +81,9 @@
+ # default subarch .h files
+ mflags-y += -Iinclude/asm-i386/mach-default
+ 
++mflags-$(CONFIG_KGDB) += -gdwarf-2
++mflags-$(CONFIG_KGDB_MORE) += $(shell echo $(CONFIG_KGDB_OPTIONS) | sed -e 's/"//g')
++
+ head-y := arch/i386/kernel/head.o arch/i386/kernel/init_task.o
+ 
+ libs-y 					+= arch/i386/lib/
+Index: linux-2.6.16.19/arch/i386/kernel/Makefile
+===================================================================
+--- linux-2.6.16.19.orig/arch/i386/kernel/Makefile	2006-06-23 17:05:14.000000000 -0700
++++ linux-2.6.16.19/arch/i386/kernel/Makefile	2006-06-23 17:06:11.000000000 -0700
+@@ -14,6 +14,7 @@
+ obj-y				+= acpi/
+ obj-$(CONFIG_X86_BIOS_REBOOT)	+= reboot.o
+ obj-$(CONFIG_MCA)		+= mca.o
++obj-$(CONFIG_KGDB)		+= kgdb_stub.o
+ obj-$(CONFIG_X86_MSR)		+= msr.o
+ obj-$(CONFIG_X86_CPUID)		+= cpuid.o
+ obj-$(CONFIG_MICROCODE)		+= microcode.o
+Index: linux-2.6.16.19/arch/i386/kernel/entry.S
+===================================================================
+--- linux-2.6.16.19.orig/arch/i386/kernel/entry.S	2006-06-23 17:05:14.000000000 -0700
++++ linux-2.6.16.19/arch/i386/kernel/entry.S	2006-06-23 17:06:11.000000000 -0700
+@@ -49,6 +49,18 @@
+ #include <asm/page.h>
+ #include <asm/desc.h>
+ #include "irq_vectors.h"
++        /* We do not recover from a stack overflow, but at least
++         * we know it happened and should be able to track it down.
++         */
++#ifdef CONFIG_STACK_OVERFLOW_TEST
++#define STACK_OVERFLOW_TEST \
++        testl $(THREAD_SIZE - 512),%esp;    \
++        jnz   10f;            \
++        call  stack_overflow; \
++10:
++#else
++#define STACK_OVERFLOW_TEST
++#endif
+ 
+ #define nr_syscalls ((syscall_table_size)/4)
+ 
+@@ -95,7 +107,8 @@
+ 	pushl %ebx; \
+ 	movl $(__USER_DS), %edx; \
+ 	movl %edx, %ds; \
+-	movl %edx, %es;
++	movl %edx, %es; \
++        STACK_OVERFLOW_TEST
+ 
+ #define RESTORE_INT_REGS \
+ 	popl %ebx;	\
+@@ -178,6 +191,7 @@
+ 	# sysenter call handler stub
+ ENTRY(sysenter_entry)
+ 	movl TSS_sysenter_esp0(%esp),%esp
++	.globl sysenter_past_esp
+ sysenter_past_esp:
+ 	sti
+ 	pushl $(__USER_DS)
+@@ -239,6 +253,10 @@
+ 	cli				# make sure we don't miss an interrupt
+ 					# setting need_resched or sigpending
+ 					# between sampling and the iret
++#ifdef CONFIG_TRAP_BAD_SYSCALL_EXITS
++	movl %esp, %eax			# pt_regs pointer
++	call sys_call_exit
++#endif
+ 	movl TI_flags(%ebp), %ecx
+ 	testw $_TIF_ALLWORK_MASK, %cx	# current->work
+ 	jne syscall_exit_work
+Index: linux-2.6.16.19/arch/i386/kernel/kgdb_stub.c
+===================================================================
+--- linux-2.6.16.19.orig/arch/i386/kernel/kgdb_stub.c	2006-04-21 14:09:39.654033432 -0700
++++ linux-2.6.16.19/arch/i386/kernel/kgdb_stub.c	2006-06-23 17:06:11.000000000 -0700
+@@ -0,0 +1,2328 @@
++/*
++ *
++ * This program is free software; you can redistribute it and/or modify it
++ * under the terms of the GNU General Public License as published by the
++ * Free Software Foundation; either version 2, or (at your option) any
++ * later version.
++ *
++ * This program is distributed in the hope that it will be useful, but
++ * WITHOUT ANY WARRANTY; without even the implied warranty of
++ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.	 See the GNU
++ * General Public License for more details.
++ *
++ */
++
++/*
++ * Copyright (c) 2000 VERITAS Software Corporation.
++ *
++ */
++/****************************************************************************
++ *  Header: remcom.c,v 1.34 91/03/09 12:29:49 glenne Exp $
++ *
++ *  Module name: remcom.c $
++ *  Revision: 1.34 $
++ *  Date: 91/03/09 12:29:49 $
++ *  Contributor:     Lake Stevens Instrument Division$
++ *
++ *  Description:     low level support for gdb debugger. $
++ *
++ *  Considerations:  only works on target hardware $
++ *
++ *  Written by:	     Glenn Engel $
++ *  Updated by:	     David Grothe <dave@gcom.com>
++ *  ModuleState:     Experimental $
++ *
++ *  NOTES:	     See Below $
++ *
++ *  Modified for 386 by Jim Kingdon, Cygnus Support.
++ *  Compatibility with 2.1.xx kernel by David Grothe <dave@gcom.com>
++ *
++ *  Changes to allow auto initilization.  All that is needed is that it
++ *  be linked with the kernel and a break point (int 3) be executed.
++ *  The header file <asm/kgdb.h> defines BREAKPOINT to allow one to do
++ *  this. It should also be possible, once the interrupt system is up, to
++ *  call putDebugChar("+").  Once this is done, the remote debugger should
++ *  get our attention by sending a ^C in a packet. George Anzinger
++ *  <george@mvista.com>
++ *  Integrated into 2.2.5 kernel by Tigran Aivazian <tigran@sco.com>
++ *  Added thread support, support for multiple processors,
++ *	support for ia-32(x86) hardware debugging.
++ *	Amit S. Kale ( akale@veritas.com )
++ *
++ *
++ *  To enable debugger support, two things need to happen.  One, a
++ *  call to set_debug_traps() is necessary in order to allow any breakpoints
++ *  or error conditions to be properly intercepted and reported to gdb.
++ *  Two, a breakpoint needs to be generated to begin communication.  This
++ *  is most easily accomplished by a call to breakpoint().  Breakpoint()
++ *  simulates a breakpoint by executing an int 3.
++ *
++ *************
++ *
++ *    The following gdb commands are supported:
++ *
++ * command	    function				   Return value
++ *
++ *    g		    return the value of the CPU registers  hex data or ENN
++ *    G		    set the value of the CPU registers	   OK or ENN
++ *
++ *    mAA..AA,LLLL  Read LLLL bytes at address AA..AA	   hex data or ENN
++ *    MAA..AA,LLLL: Write LLLL bytes at address AA.AA	   OK or ENN
++ *
++ *    c		    Resume at current address		   SNN	 ( signal NN)
++ *    cAA..AA	    Continue at address AA..AA		   SNN
++ *
++ *    s		    Step one instruction		   SNN
++ *    sAA..AA	    Step one instruction from AA..AA	   SNN
++ *
++ *    k		    kill
++ *
++ *    ?		    What was the last sigval ?		   SNN	 (signal NN)
++ *
++ * All commands and responses are sent with a packet which includes a
++ * checksum.  A packet consists of
++ *
++ * $<packet info>#<checksum>.
++ *
++ * where
++ * <packet info> :: <characters representing the command or response>
++ * <checksum>	 :: < two hex digits computed as modulo 256 sum of <packetinfo>>
++ *
++ * When a packet is received, it is first acknowledged with either '+' or '-'.
++ * '+' indicates a successful transfer.	 '-' indicates a failed transfer.
++ *
++ * Example:
++ *
++ * Host:		  Reply:
++ * $m0,10#2a		   +$00010203040506070809101112131415#42
++ *
++ ****************************************************************************/
++#define KGDB_VERSION "<20030915.1651.33>"
++#include <linux/config.h>
++#include <linux/types.h>
++#include <asm/string.h>		/* for strcpy */
++#include <linux/kernel.h>
++#include <linux/sched.h>
++#include <asm/vm86.h>
++#include <asm/system.h>
++#include <asm/ptrace.h>		/* for linux pt_regs struct */
++#include <asm/kgdb_local.h>
++#include <linux/list.h>
++#include <asm/atomic.h>
++#include <asm/processor.h>
++#include <linux/irq.h>
++#include <asm/desc.h>
++
++/************************************************************************
++ *
++ * external low-level support routines
++ */
++typedef void (*Function) (void);	/* pointer to a function */
++
++/* Thread reference */
++typedef unsigned char threadref[8];
++
++extern void putDebugChar(int);	/* write a single character	 */
++extern int getDebugChar(void);	/* read and return a single char */
++
++/************************************************************************/
++/* BUFMAX defines the maximum number of characters in inbound/outbound buffers*/
++/* at least NUMREGBYTES*2 are needed for register packets */
++/* Longer buffer is needed to list all threads */
++#define BUFMAX 400
++
++char *kgdb_version = KGDB_VERSION;
++
++/*  debug >  0 prints ill-formed commands in valid packets & checksum errors */
++int debug_regs = 0;		/* set to non-zero to print registers */
++
++/* filled in by an external module */
++char *gdb_module_offsets;
++
++static const char hexchars[] = "0123456789abcdef";
++
++/* Number of bytes of registers.  */
++#define NUMREGBYTES 64
++/*
++ * Note that this register image is in a different order than
++ * the register image that Linux produces at interrupt time.
++ *
++ * Linux's register image is defined by struct pt_regs in ptrace.h.
++ * Just why GDB uses a different order is a historical mystery.
++ */
++enum regnames { _EAX,		/* 0 */
++	_ECX,			/* 1 */
++	_EDX,			/* 2 */
++	_EBX,			/* 3 */
++	_ESP,			/* 4 */
++	_EBP,			/* 5 */
++	_ESI,			/* 6 */
++	_EDI,			/* 7 */
++	_PC /* 8 also known as eip */ ,
++	_PS /* 9 also known as eflags */ ,
++	_CS,			/* 10 */
++	_SS,			/* 11 */
++	_DS,			/* 12 */
++	_ES,			/* 13 */
++	_FS,			/* 14 */
++	_GS			/* 15 */
++};
++
++/***************************  ASSEMBLY CODE MACROS *************************/
++/*
++ * Put the error code here just in case the user cares.
++ * Likewise, the vector number here (since GDB only gets the signal
++ * number through the usual means, and that's not very specific).
++ * The called_from is the return address so he can tell how we entered kgdb.
++ * This will allow him to seperate out the various possible entries.
++ */
++#define REMOTE_DEBUG 0		/* set != to turn on printing (also available in info) */
++
++#define PID_MAX PID_MAX_DEFAULT
++
++#ifdef CONFIG_SMP
++void smp_send_nmi_allbutself(void);
++#define IF_SMP(x) x
++#undef MAX_NO_CPUS
++#ifndef CONFIG_NO_KGDB_CPUS
++#define CONFIG_NO_KGDB_CPUS 2
++#endif
++#if CONFIG_NO_KGDB_CPUS > NR_CPUS
++#define MAX_NO_CPUS NR_CPUS
++#else
++#define MAX_NO_CPUS CONFIG_NO_KGDB_CPUS
++#endif
++#define hold_init hold_on_sstep: 1,
++#define MAX_CPU_MASK (unsigned long)((1LL << MAX_NO_CPUS) - 1LL)
++#define NUM_CPUS num_online_cpus()
++#else
++#define IF_SMP(x)
++#define hold_init
++#undef MAX_NO_CPUS
++#define MAX_NO_CPUS 1
++#define NUM_CPUS 1
++#endif
++#define NOCPU (struct task_struct *)0xbad1fbad
++/* *INDENT-OFF*	 */
++struct kgdb_info {
++	int used_malloc;
++	void *called_from;
++	long long entry_tsc;
++	int errcode;
++	int vector;
++	int print_debug_info;
++#ifdef CONFIG_SMP
++	int hold_on_sstep;
++	struct {
++		volatile struct task_struct *task;
++		int pid;
++		int hold;
++		struct pt_regs *regs;
++	} cpus_waiting[MAX_NO_CPUS];
++#endif
++} kgdb_info = {hold_init print_debug_info:REMOTE_DEBUG, vector:-1};
++
++/* *INDENT-ON*	*/
++
++#define used_m kgdb_info.used_malloc
++/*
++ * This is little area we set aside to contain the stack we
++ * need to build to allow gdb to call functions.  We use one
++ * per cpu to avoid locking issues.  We will do all this work
++ * with interrupts off so that should take care of the protection
++ * issues.
++ */
++#define LOOKASIDE_SIZE 200	/* should be more than enough */
++#define MALLOC_MAX   200	/* Max malloc size */
++struct {
++	unsigned int esp;
++	int array[LOOKASIDE_SIZE];
++} fn_call_lookaside[MAX_NO_CPUS];
++
++static int trap_cpu;
++static unsigned int OLD_esp;
++
++#define END_OF_LOOKASIDE  &fn_call_lookaside[trap_cpu].array[LOOKASIDE_SIZE]
++#define IF_BIT 0x200
++#define TF_BIT 0x100
++
++#define MALLOC_ROUND 8-1
++
++static char malloc_array[MALLOC_MAX];
++IF_SMP(static void to_gdb(const char *mess));
++void *
++malloc(int size)
++{
++
++	if (size <= (MALLOC_MAX - used_m)) {
++		int old_used = used_m;
++		used_m += ((size + MALLOC_ROUND) & (~MALLOC_ROUND));
++		return &malloc_array[old_used];
++	} else {
++		return NULL;
++	}
++}
++
++/*
++ * Gdb calls functions by pushing agruments, including a return address
++ * on the stack and the adjusting EIP to point to the function.	 The
++ * whole assumption in GDB is that we are on a different stack than the
++ * one the "user" i.e. code that hit the break point, is on.  This, of
++ * course is not true in the kernel.  Thus various dodges are needed to
++ * do the call without directly messing with EIP (which we can not change
++ * as it is just a location and not a register.	 To adjust it would then
++ * require that we move every thing below EIP up or down as needed.  This
++ * will not work as we may well have stack relative pointer on the stack
++ * (such as the pointer to regs, for example).
++
++ * So here is what we do:
++ * We detect gdb attempting to store into the stack area and instead, store
++ * into the fn_call_lookaside.array at the same relative location as if it
++ * were the area ESP pointed at.  We also trap ESP modifications
++ * and uses these to adjust fn_call_lookaside.esp.  On entry
++ * fn_call_lookaside.esp will be set to point at the last entry in
++ * fn_call_lookaside.array.  This allows us to check if it has changed, and
++ * if so, on exit, we add the registers we will use to do the move and a
++ * trap/ interrupt return exit sequence.  We then adjust the eflags in the
++ * regs array (remember we now have a copy in the fn_call_lookaside.array) to
++ * kill the interrupt bit, AND we change EIP to point at our set up stub.
++ * As part of the register set up we preset the registers to point at the
++ * begining and end of the fn_call_lookaside.array, so all the stub needs to
++ * do is move words from the array to the stack until ESP= the desired value
++ * then do the rti.  This will then transfer to the desired function with
++ * all the correct registers.  Nifty huh?
++ */
++extern asmlinkage void fn_call_stub(void);
++extern asmlinkage void fn_rtn_stub(void);
++/*					   *INDENT-OFF*	 */
++__asm__("fn_rtn_stub:\n\t"
++	"movl %eax,%esp\n\t"
++	"fn_call_stub:\n\t"
++	"1:\n\t"
++	"addl $-4,%ebx\n\t"
++	"movl (%ebx), %eax\n\t"
++	"pushl %eax\n\t"
++	"cmpl %esp,%ecx\n\t"
++	"jne  1b\n\t"
++	"popl %eax\n\t"
++	"popl %ebx\n\t"
++	"popl %ecx\n\t"
++	"iret \n\t");
++/*					     *INDENT-ON*  */
++#define gdb_i386vector	kgdb_info.vector
++#define gdb_i386errcode kgdb_info.errcode
++#define waiting_cpus	kgdb_info.cpus_waiting
++#define remote_debug	kgdb_info.print_debug_info
++#define hold_cpu(cpu)	kgdb_info.cpus_waiting[cpu].hold
++/* gdb locks */
++
++#ifdef CONFIG_SMP
++static int in_kgdb_called;
++static spinlock_t waitlocks[MAX_NO_CPUS] =
++    {[0 ... MAX_NO_CPUS - 1] = SPIN_LOCK_UNLOCKED };
++/*
++ * The following array has the thread pointer of each of the "other"
++ * cpus.  We make it global so it can be seen by gdb.
++ */
++volatile int in_kgdb_entry_log[MAX_NO_CPUS];
++volatile struct pt_regs *in_kgdb_here_log[MAX_NO_CPUS];
++/*
++static spinlock_t continuelocks[MAX_NO_CPUS];
++*/
++spinlock_t kgdb_spinlock = SPIN_LOCK_UNLOCKED;
++/* waiters on our spinlock plus us */
++static atomic_t spinlock_waiters = ATOMIC_INIT(1);
++static int spinlock_count = 0;
++static int spinlock_cpu = 0;
++/*
++ * Note we use nested spin locks to account for the case where a break
++ * point is encountered when calling a function by user direction from
++ * kgdb. Also there is the memory exception recursion to account for.
++ * Well, yes, but this lets other cpus thru too.  Lets add a
++ * cpu id to the lock.
++ */
++#define KGDB_SPIN_LOCK(x) if( spinlock_count == 0 || \
++			      spinlock_cpu != smp_processor_id()){\
++				      atomic_inc(&spinlock_waiters); \
++				      while (! spin_trylock(x)) {\
++					    in_kgdb(&regs);\
++				      }\
++				      atomic_dec(&spinlock_waiters); \
++				      spinlock_count = 1; \
++				      spinlock_cpu = smp_processor_id(); \
++			  }else{  \
++				      spinlock_count++; \
++			  }
++#define KGDB_SPIN_UNLOCK(x) if( --spinlock_count == 0) spin_unlock(x)
++#else
++unsigned kgdb_spinlock = 0;
++#define KGDB_SPIN_LOCK(x) --*x
++#define KGDB_SPIN_UNLOCK(x) ++*x
++#endif
++
++int
++hex(char ch)
++{
++	if ((ch >= 'a') && (ch <= 'f'))
++		return (ch - 'a' + 10);
++	if ((ch >= '0') && (ch <= '9'))
++		return (ch - '0');
++	if ((ch >= 'A') && (ch <= 'F'))
++		return (ch - 'A' + 10);
++	return (-1);
++}
++
++/* scan for the sequence $<data>#<checksum>	*/
++void
++getpacket(char *buffer)
++{
++	unsigned char checksum;
++	unsigned char xmitcsum;
++	int i;
++	int count;
++	char ch;
++
++	do {
++		/* wait around for the start character, ignore all other characters */
++		while ((ch = (getDebugChar() & 0x7f)) != '$') ;
++		checksum = 0;
++		xmitcsum = -1;
++
++		count = 0;
++
++		/* now, read until a # or end of buffer is found */
++		while (count < BUFMAX) {
++			ch = getDebugChar() & 0x7f;
++			if (ch == '#')
++				break;
++			checksum = checksum + ch;
++			buffer[count] = ch;
++			count = count + 1;
++		}
++		buffer[count] = 0;
++
++		if (ch == '#') {
++			xmitcsum = hex(getDebugChar() & 0x7f) << 4;
++			xmitcsum += hex(getDebugChar() & 0x7f);
++			if ((remote_debug) && (checksum != xmitcsum)) {
++				printk
++				    ("bad checksum.	My count = 0x%x, sent=0x%x. buf=%s\n",
++				     checksum, xmitcsum, buffer);
++			}
++
++			if (checksum != xmitcsum)
++				putDebugChar('-');	/* failed checksum */
++			else {
++				putDebugChar('+');	/* successful transfer */
++				/* if a sequence char is present, reply the sequence ID */
++				if (buffer[2] == ':') {
++					putDebugChar(buffer[0]);
++					putDebugChar(buffer[1]);
++					/* remove sequence chars from buffer */
++					count = strlen(buffer);
++					for (i = 3; i <= count; i++)
++						buffer[i - 3] = buffer[i];
++				}
++			}
++		}
++	} while (checksum != xmitcsum);
++
++	if (remote_debug)
++		printk("R:%s\n", buffer);
++}
++
++/* send the packet in buffer.  */
++
++void
++putpacket(char *buffer)
++{
++	unsigned char checksum;
++	int count;
++	char ch;
++
++	/*  $<packet info>#<checksum>. */
++	do {
++		if (remote_debug)
++			printk("T:%s\n", buffer);
++		putDebugChar('$');
++		checksum = 0;
++		count = 0;
++
++		while ((ch = buffer[count])) {
++			putDebugChar(ch);
++			checksum += ch;
++			count += 1;
++		}
++
++		putDebugChar('#');
++		putDebugChar(hexchars[checksum >> 4]);
++		putDebugChar(hexchars[checksum % 16]);
++
++	} while ((getDebugChar() & 0x7f) != '+');
++
++}
++
++static char remcomInBuffer[BUFMAX];
++static char remcomOutBuffer[BUFMAX];
++static short error;
++
++void
++debug_error(char *format, char *parm)
++{
++	if (remote_debug)
++		printk(format, parm);
++}
++
++static void
++print_regs(struct pt_regs *regs)
++{
++	printk("EAX=%08lx ", regs->eax);
++	printk("EBX=%08lx ", regs->ebx);
++	printk("ECX=%08lx ", regs->ecx);
++	printk("EDX=%08lx ", regs->edx);
++	printk("\n");
++	printk("ESI=%08lx ", regs->esi);
++	printk("EDI=%08lx ", regs->edi);
++	printk("EBP=%08lx ", regs->ebp);
++	printk("ESP=%08lx ", (long) &regs->esp);
++	printk("\n");
++	printk(" DS=%08x ", regs->xds);
++	printk(" ES=%08x ", regs->xes);
++	printk(" SS=%08x ", __KERNEL_DS);
++	printk(" FL=%08lx ", regs->eflags);
++	printk("\n");
++	printk(" CS=%08x ", regs->xcs);
++	printk(" IP=%08lx ", regs->eip);
++#if 0
++	printk(" FS=%08x ", regs->fs);
++	printk(" GS=%08x ", regs->gs);
++#endif
++	printk("\n");
++
++}				/* print_regs */
++
++#define NEW_esp fn_call_lookaside[trap_cpu].esp
++
++static void
++regs_to_gdb_regs(int *gdb_regs, struct pt_regs *regs)
++{
++	gdb_regs[_EAX] = regs->eax;
++	gdb_regs[_EBX] = regs->ebx;
++	gdb_regs[_ECX] = regs->ecx;
++	gdb_regs[_EDX] = regs->edx;
++	gdb_regs[_ESI] = regs->esi;
++	gdb_regs[_EDI] = regs->edi;
++	gdb_regs[_EBP] = regs->ebp;
++	gdb_regs[_DS] = regs->xds;
++	gdb_regs[_ES] = regs->xes;
++	gdb_regs[_PS] = regs->eflags;
++	gdb_regs[_CS] = regs->xcs;
++	gdb_regs[_PC] = regs->eip;
++	/* Note, as we are a debugging the kernel, we will always
++	 * trap in kernel code, this means no priviledge change,
++	 * and so the pt_regs structure is not completely valid.  In a non
++	 * privilege change trap, only EFLAGS, CS and EIP are put on the stack,
++	 * SS and ESP are not stacked, this means that the last 2 elements of
++	 * pt_regs is not valid (they would normally refer to the user stack)
++	 * also, using regs+1 is no good because you end up will a value that is
++	 * 2 longs (8) too high.  This used to cause stepping over functions
++	 * to fail, so my fix is to use the address of regs->esp, which
++	 * should point at the end of the stack frame.	Note I have ignored
++	 * completely exceptions that cause an error code to be stacked, such
++	 * as double fault.  Stuart Hughes, Zentropix.
++	 * original code: gdb_regs[_ESP] =  (int) (regs + 1) ;
++
++	 * this is now done on entry and moved to OLD_esp (as well as NEW_esp).
++	 */
++	gdb_regs[_ESP] = NEW_esp;
++	gdb_regs[_SS] = __KERNEL_DS;
++	gdb_regs[_FS] = 0xFFFF;
++	gdb_regs[_GS] = 0xFFFF;
++}				/* regs_to_gdb_regs */
++
++static void
++gdb_regs_to_regs(int *gdb_regs, struct pt_regs *regs)
++{
++	regs->eax = gdb_regs[_EAX];
++	regs->ebx = gdb_regs[_EBX];
++	regs->ecx = gdb_regs[_ECX];
++	regs->edx = gdb_regs[_EDX];
++	regs->esi = gdb_regs[_ESI];
++	regs->edi = gdb_regs[_EDI];
++	regs->ebp = gdb_regs[_EBP];
++	regs->xds = gdb_regs[_DS];
++	regs->xes = gdb_regs[_ES];
++	regs->eflags = gdb_regs[_PS];
++	regs->xcs = gdb_regs[_CS];
++	regs->eip = gdb_regs[_PC];
++	NEW_esp = gdb_regs[_ESP];	/* keep the value */
++#if 0				/* can't change these */
++	regs->esp = gdb_regs[_ESP];
++	regs->xss = gdb_regs[_SS];
++	regs->fs = gdb_regs[_FS];
++	regs->gs = gdb_regs[_GS];
++#endif
++
++}				/* gdb_regs_to_regs */
++
++int thread_list = 0;
++
++void
++get_gdb_regs(struct task_struct *p, struct pt_regs *regs, int *gdb_regs)
++{
++	unsigned long stack_page;
++	int count = 0;
++	IF_SMP(int i);
++	if (!p || p == current) {
++		regs_to_gdb_regs(gdb_regs, regs);
++		return;
++	}
++#ifdef CONFIG_SMP
++	for (i = 0; i < MAX_NO_CPUS; i++) {
++		if (p == kgdb_info.cpus_waiting[i].task) {
++			regs_to_gdb_regs(gdb_regs,
++					 kgdb_info.cpus_waiting[i].regs);
++			gdb_regs[_ESP] =
++			    (int) &kgdb_info.cpus_waiting[i].regs->esp;
++
++			return;
++		}
++	}
++#endif
++	memset(gdb_regs, 0, NUMREGBYTES);
++	gdb_regs[_ESP] = p->thread.esp;
++	gdb_regs[_PC] = p->thread.eip;
++	gdb_regs[_EBP] = *(int *) gdb_regs[_ESP];
++	gdb_regs[_EDI] = *(int *) (gdb_regs[_ESP] + 4);
++	gdb_regs[_ESI] = *(int *) (gdb_regs[_ESP] + 8);
++
++/*
++ * This code is to give a more informative notion of where a process
++ * is waiting.	It is used only when the user asks for a thread info
++ * list.  If he then switches to the thread, s/he will find the task
++ * is in schedule, but a back trace should show the same info we come
++ * up with.  This code was shamelessly purloined from process.c.  It was
++ * then enhanced to provide more registers than simply the program
++ * counter.
++ */
++
++	if (!thread_list) {
++		return;
++	}
++
++	if (p->state == TASK_RUNNING)
++		return;
++	stack_page = (unsigned long) p->thread_info;
++	if (gdb_regs[_ESP] < stack_page || gdb_regs[_ESP] >
++	    THREAD_SIZE - sizeof(long) + stack_page)
++		return;
++	/* include/asm-i386/system.h:switch_to() pushes ebp last. */
++	do {
++		if (gdb_regs[_EBP] < stack_page ||
++		    gdb_regs[_EBP] > THREAD_SIZE - 2*sizeof(long) + stack_page)
++			return;
++		gdb_regs[_PC] = *(unsigned long *) (gdb_regs[_EBP] + 4);
++		gdb_regs[_ESP] = gdb_regs[_EBP] + 8;
++		gdb_regs[_EBP] = *(unsigned long *) gdb_regs[_EBP];
++		if (!in_sched_functions(gdb_regs[_PC]))
++			return;
++	} while (count++ < 16);
++	return;
++}
++
++/* Indicate to caller of mem2hex or hex2mem that there has been an
++   error.  */
++static volatile int mem_err = 0;
++static volatile int mem_err_expected = 0;
++static volatile int mem_err_cnt = 0;
++static int garbage_loc = -1;
++
++int
++get_char(char *addr)
++{
++	return *addr;
++}
++
++void
++set_char(char *addr, int val, int may_fault)
++{
++	/*
++	 * This code traps references to the area mapped to the kernel
++	 * stack as given by the regs and, instead, stores to the
++	 * fn_call_lookaside[cpu].array
++	 */
++	if (may_fault &&
++	    (unsigned int) addr < OLD_esp &&
++	    ((unsigned int) addr > (OLD_esp - (unsigned int) LOOKASIDE_SIZE))) {
++		addr = (char *) END_OF_LOOKASIDE - ((char *) OLD_esp - addr);
++	}
++	*addr = val;
++}
++
++/* convert the memory pointed to by mem into hex, placing result in buf */
++/* return a pointer to the last char put in buf (null) */
++/* If MAY_FAULT is non-zero, then we should set mem_err in response to
++   a fault; if zero treat a fault like any other fault in the stub.  */
++char *
++mem2hex(char *mem, char *buf, int count, int may_fault)
++{
++	int i;
++	unsigned char ch;
++
++	if (may_fault) {
++		mem_err_expected = 1;
++		mem_err = 0;
++	}
++	for (i = 0; i < count; i++) {
++		/* printk("%lx = ", mem) ; */
++
++		ch = get_char(mem++);
++
++		/* printk("%02x\n", ch & 0xFF) ; */
++		if (may_fault && mem_err) {
++			if (remote_debug)
++				printk("Mem fault fetching from addr %lx\n",
++				       (long) (mem - 1));
++			*buf = 0;	/* truncate buffer */
++			return (buf);
++		}
++		*buf++ = hexchars[ch >> 4];
++		*buf++ = hexchars[ch % 16];
++	}
++	*buf = 0;
++	if (may_fault)
++		mem_err_expected = 0;
++	return (buf);
++}
++
++/* convert the hex array pointed to by buf into binary to be placed in mem */
++/* return a pointer to the character AFTER the last byte written */
++/* NOTE: We use the may fault flag to also indicate if the write is to
++ * the registers (0) or "other" memory (!=0)
++ */
++char *
++hex2mem(char *buf, char *mem, int count, int may_fault)
++{
++	int i;
++	unsigned char ch;
++
++	if (may_fault) {
++		mem_err_expected = 1;
++		mem_err = 0;
++	}
++	for (i = 0; i < count; i++) {
++		ch = hex(*buf++) << 4;
++		ch = ch + hex(*buf++);
++		set_char(mem++, ch, may_fault);
++
++		if (may_fault && mem_err) {
++			if (remote_debug)
++				printk("Mem fault storing to addr %lx\n",
++				       (long) (mem - 1));
++			return (mem);
++		}
++	}
++	if (may_fault)
++		mem_err_expected = 0;
++	return (mem);
++}
++
++/**********************************************/
++/* WHILE WE FIND NICE HEX CHARS, BUILD AN INT */
++/* RETURN NUMBER OF CHARS PROCESSED	      */
++/**********************************************/
++int
++hexToInt(char **ptr, int *intValue)
++{
++	int numChars = 0;
++	int hexValue;
++
++	*intValue = 0;
++
++	while (**ptr) {
++		hexValue = hex(**ptr);
++		if (hexValue >= 0) {
++			*intValue = (*intValue << 4) | hexValue;
++			numChars++;
++		} else
++			break;
++
++		(*ptr)++;
++	}
++
++	return (numChars);
++}
++
++#define stubhex(h) hex(h)
++#ifdef old_thread_list
++
++static int
++stub_unpack_int(char *buff, int fieldlength)
++{
++	int nibble;
++	int retval = 0;
++
++	while (fieldlength) {
++		nibble = stubhex(*buff++);
++		retval |= nibble;
++		fieldlength--;
++		if (fieldlength)
++			retval = retval << 4;
++	}
++	return retval;
++}
++#endif
++static char *
++pack_hex_byte(char *pkt, int byte)
++{
++	*pkt++ = hexchars[(byte >> 4) & 0xf];
++	*pkt++ = hexchars[(byte & 0xf)];
++	return pkt;
++}
++
++#define BUF_THREAD_ID_SIZE 16
++
++static char *
++pack_threadid(char *pkt, threadref * id)
++{
++	char *limit;
++	unsigned char *altid;
++
++	altid = (unsigned char *) id;
++	limit = pkt + BUF_THREAD_ID_SIZE;
++	while (pkt < limit)
++		pkt = pack_hex_byte(pkt, *altid++);
++	return pkt;
++}
++
++#ifdef old_thread_list
++static char *
++unpack_byte(char *buf, int *value)
++{
++	*value = stub_unpack_int(buf, 2);
++	return buf + 2;
++}
++
++static char *
++unpack_threadid(char *inbuf, threadref * id)
++{
++	char *altref;
++	char *limit = inbuf + BUF_THREAD_ID_SIZE;
++	int x, y;
++
++	altref = (char *) id;
++
++	while (inbuf < limit) {
++		x = stubhex(*inbuf++);
++		y = stubhex(*inbuf++);
++		*altref++ = (x << 4) | y;
++	}
++	return inbuf;
++}
++#endif
++void
++int_to_threadref(threadref * id, int value)
++{
++	unsigned char *scan;
++
++	scan = (unsigned char *) id;
++	{
++		int i = 4;
++		while (i--)
++			*scan++ = 0;
++	}
++	*scan++ = (value >> 24) & 0xff;
++	*scan++ = (value >> 16) & 0xff;
++	*scan++ = (value >> 8) & 0xff;
++	*scan++ = (value & 0xff);
++}
++int
++int_to_hex_v(unsigned char * id, int value)
++{
++	unsigned char *start = id;
++	int shift;
++	int ch;
++
++	for (shift = 28; shift >= 0; shift -= 4) {
++		if ((ch = (value >> shift) & 0xf) || (id != start)) {
++			*id = hexchars[ch];
++			id++;
++		}
++	}
++	if (id == start)
++		*id++ = '0';
++	return id - start;
++}
++#ifdef old_thread_list
++
++static int
++threadref_to_int(threadref * ref)
++{
++	int i, value = 0;
++	unsigned char *scan;
++
++	scan = (char *) ref;
++	scan += 4;
++	i = 4;
++	while (i-- > 0)
++		value = (value << 8) | ((*scan++) & 0xff);
++	return value;
++}
++#endif
++static int
++cmp_str(char *s1, char *s2, int count)
++{
++	while (count--) {
++		if (*s1++ != *s2++)
++			return 0;
++	}
++	return 1;
++}
++
++#if 1				/* this is a hold over from 2.4 where O(1) was "sometimes" */
++extern struct task_struct *kgdb_get_idle(int cpu);
++#define idle_task(cpu) kgdb_get_idle(cpu)
++#else
++#define idle_task(cpu) init_tasks[cpu]
++#endif
++
++extern int kgdb_pid_init_done;
++
++struct task_struct *
++getthread(int pid)
++{
++	struct task_struct *thread;
++	if (pid >= PID_MAX && pid <= (PID_MAX + MAX_NO_CPUS)) {
++
++		return idle_task(pid - PID_MAX);
++	} else {
++		/*
++		 * find_task_by_pid is relatively safe all the time
++		 * Other pid functions require lock downs which imply
++		 * that we may be interrupting them (as we get here
++		 * in the middle of most any lock down).
++		 * Still we don't want to call until the table exists!
++		 */
++		if (kgdb_pid_init_done){
++			thread = find_task_by_pid(pid);
++			if (thread) {
++				return thread;
++			}
++		}
++	}
++	return NULL;
++}
++/* *INDENT-OFF*	 */
++struct hw_breakpoint {
++	unsigned enabled;
++	unsigned type;
++	unsigned len;
++	unsigned addr;
++} breakinfo[4] = { {enabled:0},
++		   {enabled:0},
++		   {enabled:0},
++		   {enabled:0}};
++/* *INDENT-ON*	*/
++unsigned hw_breakpoint_status;
++void
++correct_hw_break(void)
++{
++	int breakno;
++	int correctit;
++	int breakbit;
++	unsigned dr7;
++
++	asm volatile ("movl %%db7, %0\n":"=r" (dr7)
++		      :);
++	/* *INDENT-OFF*	 */
++	do {
++		unsigned addr0, addr1, addr2, addr3;
++		asm volatile ("movl %%db0, %0\n"
++			      "movl %%db1, %1\n"
++			      "movl %%db2, %2\n"
++			      "movl %%db3, %3\n"
++			      :"=r" (addr0), "=r"(addr1),
++			      "=r"(addr2), "=r"(addr3)
++			      :);
++	} while (0);
++	/* *INDENT-ON*	*/
++	correctit = 0;
++	for (breakno = 0; breakno < 3; breakno++) {
++		breakbit = 2 << (breakno << 1);
++		if (!(dr7 & breakbit) && breakinfo[breakno].enabled) {
++			correctit = 1;
++			dr7 |= breakbit;
++			dr7 &= ~(0xf0000 << (breakno << 2));
++			dr7 |= (((breakinfo[breakno].len << 2) |
++				 breakinfo[breakno].type) << 16) <<
++			    (breakno << 2);
++			switch (breakno) {
++			case 0:
++				asm volatile ("movl %0, %%dr0\n"::"r"
++					      (breakinfo[breakno].addr));
++				break;
++
++			case 1:
++				asm volatile ("movl %0, %%dr1\n"::"r"
++					      (breakinfo[breakno].addr));
++				break;
++
++			case 2:
++				asm volatile ("movl %0, %%dr2\n"::"r"
++					      (breakinfo[breakno].addr));
++				break;
++
++			case 3:
++				asm volatile ("movl %0, %%dr3\n"::"r"
++					      (breakinfo[breakno].addr));
++				break;
++			}
++		} else if ((dr7 & breakbit) && !breakinfo[breakno].enabled) {
++			correctit = 1;
++			dr7 &= ~breakbit;
++			dr7 &= ~(0xf0000 << (breakno << 2));
++		}
++	}
++	if (correctit) {
++		asm volatile ("movl %0, %%db7\n"::"r" (dr7));
++	}
++}
++
++int
++remove_hw_break(unsigned breakno)
++{
++	if (!breakinfo[breakno].enabled) {
++		return -1;
++	}
++	breakinfo[breakno].enabled = 0;
++	return 0;
++}
++
++int
++set_hw_break(unsigned breakno, unsigned type, unsigned len, unsigned addr)
++{
++	if (breakinfo[breakno].enabled) {
++		return -1;
++	}
++	breakinfo[breakno].enabled = 1;
++	breakinfo[breakno].type = type;
++	breakinfo[breakno].len = len;
++	breakinfo[breakno].addr = addr;
++	return 0;
++}
++
++#ifdef CONFIG_SMP
++static int in_kgdb_console = 0;
++
++int
++in_kgdb(struct pt_regs *regs)
++{
++	unsigned flags;
++	int cpu = smp_processor_id();
++	in_kgdb_called = 1;
++	if (!spin_is_locked(&kgdb_spinlock)) {
++		if (in_kgdb_here_log[cpu] ||	/* we are holding this cpu */
++		    in_kgdb_console) {	/* or we are doing slow i/o */
++			return 1;
++		}
++		return 0;
++	}
++
++	/* As I see it the only reason not to let all cpus spin on
++	 * the same spin_lock is to allow selected ones to proceed.
++	 * This would be a good thing, so we leave it this way.
++	 * Maybe someday....  Done !
++
++	 * in_kgdb() is called from an NMI so we don't pretend
++	 * to have any resources, like printk() for example.
++	 */
++
++	kgdb_local_irq_save(flags);	/* only local here, to avoid hanging */
++	/*
++	 * log arival of this cpu
++	 * The NMI keeps on ticking.  Protect against recurring more
++	 * than once, and ignor the cpu that has the kgdb lock
++	 */
++	in_kgdb_entry_log[cpu]++;
++	in_kgdb_here_log[cpu] = regs;
++	if (cpu == spinlock_cpu || waiting_cpus[cpu].task) {
++		goto exit_in_kgdb;
++	}
++	/*
++	 * For protection of the initilization of the spin locks by kgdb
++	 * it locks the kgdb spinlock before it gets the wait locks set
++	 * up.	We wait here for the wait lock to be taken.  If the
++	 * kgdb lock goes away first??	Well, it could be a slow exit
++	 * sequence where the wait lock is removed prior to the kgdb lock
++	 * so if kgdb gets unlocked, we just exit.
++	 */
++	while (spin_is_locked(&kgdb_spinlock) &&
++	       !spin_is_locked(waitlocks + cpu)) ;
++	if (!spin_is_locked(&kgdb_spinlock)) {
++		goto exit_in_kgdb;
++	}
++	waiting_cpus[cpu].task = current;
++	waiting_cpus[cpu].pid = (current->pid) ? : (PID_MAX + cpu);
++	waiting_cpus[cpu].regs = regs;
++
++	spin_unlock_wait(waitlocks + cpu);
++	/*
++	 * log departure of this cpu
++	 */
++	waiting_cpus[cpu].task = 0;
++	waiting_cpus[cpu].pid = 0;
++	waiting_cpus[cpu].regs = 0;
++	correct_hw_break();
++      exit_in_kgdb:
++	in_kgdb_here_log[cpu] = 0;
++	kgdb_local_irq_restore(flags);
++	return 1;
++	/*
++	   spin_unlock(continuelocks + smp_processor_id());
++	 */
++}
++
++void
++smp__in_kgdb(struct pt_regs regs)
++{
++	ack_APIC_irq();
++	in_kgdb(&regs);
++}
++#else
++int
++in_kgdb(struct pt_regs *regs)
++{
++	return (kgdb_spinlock);
++}
++#endif
++
++void
++printexceptioninfo(int exceptionNo, int errorcode, char *buffer)
++{
++	unsigned dr6;
++	int i;
++	switch (exceptionNo) {
++	case 1:		/* debug exception */
++		break;
++	case 3:		/* breakpoint */
++		sprintf(buffer, "Software breakpoint");
++		return;
++	default:
++		sprintf(buffer, "Details not available");
++		return;
++	}
++	asm volatile ("movl %%db6, %0\n":"=r" (dr6)
++		      :);
++	if (dr6 & 0x4000) {
++		sprintf(buffer, "Single step");
++		return;
++	}
++	for (i = 0; i < 4; ++i) {
++		if (dr6 & (1 << i)) {
++			sprintf(buffer, "Hardware breakpoint %d", i);
++			return;
++		}
++	}
++	sprintf(buffer, "Unknown trap");
++	return;
++}
++
++/*
++ * This function does all command procesing for interfacing to gdb.
++ *
++ * NOTE:  The INT nn instruction leaves the state of the interrupt
++ *	  enable flag UNCHANGED.  That means that when this routine
++ *	  is entered via a breakpoint (INT 3) instruction from code
++ *	  that has interrupts enabled, then interrupts will STILL BE
++ *	  enabled when this routine is entered.	 The first thing that
++ *	  we do here is disable interrupts so as to prevent recursive
++ *	  entries and bothersome serial interrupts while we are
++ *	  trying to run the serial port in polled mode.
++ *
++ * For kernel version 2.1.xx the kgdb_cli() actually gets a spin lock so
++ * it is always necessary to do a restore_flags before returning
++ * so as to let go of that lock.
++ */
++int
++kgdb_handle_exception(int exceptionVector,
++		      int signo, int err_code, struct pt_regs *linux_regs)
++{
++	struct task_struct *usethread = NULL;
++	struct task_struct *thread_list_start = 0, *thread = NULL;
++	int addr, length;
++	int breakno, breaktype;
++	char *ptr;
++	int newPC;
++	threadref thref;
++	int threadid;
++	int thread_min = PID_MAX + MAX_NO_CPUS;
++#ifdef old_thread_list
++	int maxthreads;
++#endif
++	int nothreads;
++	unsigned long flags;
++	int gdb_regs[NUMREGBYTES / 4];
++	int dr6;
++	IF_SMP(int entry_state = 0);	/* 0, ok, 1, no nmi, 2 sync failed */
++#define NO_NMI 1
++#define NO_SYNC 2
++#define	regs	(*linux_regs)
++#define NUMREGS NUMREGBYTES/4
++	/*
++	 * If the entry is not from the kernel then return to the Linux
++	 * trap handler and let it process the interrupt normally.
++	 */
++	if ((linux_regs->eflags & VM_MASK) || (3 & linux_regs->xcs)) {
++		printk("ignoring non-kernel exception\n");
++		print_regs(&regs);
++		return (0);
++	}
++
++	kgdb_local_irq_save(flags);
++
++	/* Get kgdb spinlock */
++
++	KGDB_SPIN_LOCK(&kgdb_spinlock);
++	rdtscll(kgdb_info.entry_tsc);
++	/*
++	 * We depend on this spinlock and the NMI watch dog to control the
++	 * other cpus.	They will arrive at "in_kgdb()" as a result of the
++	 * NMI and will wait there for the following spin locks to be
++	 * released.
++	 */
++#ifdef CONFIG_SMP
++
++#if 0
++	if (cpu_callout_map & ~MAX_CPU_MASK) {
++		printk("kgdb : too many cpus, possibly not mapped"
++		       " in contiguous space, change MAX_NO_CPUS"
++		       " in kgdb_stub and make new kernel.\n"
++		       " cpu_callout_map is %lx\n", cpu_callout_map);
++		goto exit_just_unlock;
++	}
++#endif
++	if (spinlock_count == 1) {
++		int time = 0, end_time, dum = 0;
++		int i;
++		int cpu_logged_in[MAX_NO_CPUS] = {[0 ... MAX_NO_CPUS - 1] = (0)
++		};
++		if (remote_debug) {
++			printk("kgdb : cpu %d entry, syncing others\n",
++			       smp_processor_id());
++		}
++		for (i = 0; i < MAX_NO_CPUS; i++) {
++			/*
++			 * Use trylock as we may already hold the lock if
++			 * we are holding the cpu.  Net result is all
++			 * locked.
++			 */
++			spin_trylock(&waitlocks[i]);
++		}
++		for (i = 0; i < MAX_NO_CPUS; i++)
++			cpu_logged_in[i] = 0;
++		/*
++		 * Wait for their arrival.  We know the watch dog is active if
++		 * in_kgdb() has ever been called, as it is always called on a
++		 * watchdog tick.
++		 */
++		rdtsc(dum, time);
++		end_time = time + 2;	/* Note: we use the High order bits! */
++		i = 1;
++		if (num_online_cpus() > 1) {
++			int me_in_kgdb = in_kgdb_entry_log[smp_processor_id()];
++			smp_send_nmi_allbutself();
++			while (i < num_online_cpus() && time != end_time) {
++				int j;
++				for (j = 0; j < MAX_NO_CPUS; j++) {
++					if (waiting_cpus[j].task &&
++					    !cpu_logged_in[j]) {
++						i++;
++						cpu_logged_in[j] = 1;
++						if (remote_debug) {
++							printk
++							    ("kgdb : cpu %d arrived at kgdb\n",
++							     j);
++						}
++						break;
++					} else if (!waiting_cpus[j].task &&
++						   !cpu_online(j)) {
++						waiting_cpus[j].task = NOCPU;
++						cpu_logged_in[j] = 1;
++						waiting_cpus[j].hold = 1;
++						break;
++					}
++					if (!waiting_cpus[j].task &&
++					    in_kgdb_here_log[j]) {
++
++						int wait = 100000;
++						while (wait--) ;
++						if (!waiting_cpus[j].task &&
++						    in_kgdb_here_log[j]) {
++							printk
++							    ("kgdb : cpu %d stall"
++							     " in in_kgdb\n",
++							     j);
++							i++;
++							cpu_logged_in[j] = 1;
++							waiting_cpus[j].task =
++							    (struct task_struct
++							     *) 1;
++						}
++					}
++				}
++
++				if (in_kgdb_entry_log[smp_processor_id()] >
++				    (me_in_kgdb + 10)) {
++					break;
++				}
++
++				rdtsc(dum, time);
++			}
++			if (i < num_online_cpus()) {
++				printk
++				    ("kgdb : time out, proceeding without sync\n");
++#if 0
++				printk("kgdb : Waiting_cpus: 0 = %d, 1 = %d\n",
++				       waiting_cpus[0].task != 0,
++				       waiting_cpus[1].task != 0);
++				printk("kgdb : Cpu_logged in: 0 = %d, 1 = %d\n",
++				       cpu_logged_in[0], cpu_logged_in[1]);
++				printk
++				    ("kgdb : in_kgdb_here_log in: 0 = %d, 1 = %d\n",
++				     in_kgdb_here_log[0] != 0,
++				     in_kgdb_here_log[1] != 0);
++#endif
++				entry_state = NO_SYNC;
++			} else {
++#if 0
++				int ent =
++				    in_kgdb_entry_log[smp_processor_id()] -
++				    me_in_kgdb;
++				printk("kgdb : sync after %d entries\n", ent);
++#endif
++			}
++		} else {
++			if (remote_debug) {
++				printk
++				    ("kgdb : %d cpus, but watchdog not active\n"
++				     "proceeding without locking down other cpus\n",
++				     num_online_cpus());
++				entry_state = NO_NMI;
++			}
++		}
++	}
++#endif
++
++	if (remote_debug) {
++		unsigned long *lp = (unsigned long *) &linux_regs;
++
++		printk("handle_exception(exceptionVector=%d, "
++		       "signo=%d, err_code=%d, linux_regs=%p)\n",
++		       exceptionVector, signo, err_code, linux_regs);
++		if (debug_regs) {
++			print_regs(&regs);
++			printk("Stk: %8lx %8lx %8lx %8lx"
++			       "  %8lx %8lx %8lx %8lx\n",
++			       lp[0], lp[1], lp[2], lp[3],
++			       lp[4], lp[5], lp[6], lp[7]);
++			printk("     %8lx %8lx %8lx %8lx"
++			       "  %8lx %8lx %8lx %8lx\n",
++			       lp[8], lp[9], lp[10], lp[11],
++			       lp[12], lp[13], lp[14], lp[15]);
++			printk("     %8lx %8lx %8lx %8lx  "
++			       "%8lx %8lx %8lx %8lx\n",
++			       lp[16], lp[17], lp[18], lp[19],
++			       lp[20], lp[21], lp[22], lp[23]);
++			printk("     %8lx %8lx %8lx %8lx  "
++			       "%8lx %8lx %8lx %8lx\n",
++			       lp[24], lp[25], lp[26], lp[27],
++			       lp[28], lp[29], lp[30], lp[31]);
++		}
++	}
++
++	/* Disable hardware debugging while we are in kgdb */
++	/* Get the debug register status register */
++/*				       *INDENT-OFF*  */
++      __asm__("movl %0,%%db7"
++	      :	/* no output */
++	      :"r"(0));
++
++	asm volatile ("movl %%db6, %0\n"
++		      :"=r" (hw_breakpoint_status)
++		      :);
++
++/*				       *INDENT-ON*  */
++	switch (exceptionVector) {
++	case 0:		/* divide error */
++	case 1:		/* debug exception */
++	case 2:		/* NMI */
++	case 3:		/* breakpoint */
++	case 4:		/* overflow */
++	case 5:		/* bounds check */
++	case 6:		/* invalid opcode */
++	case 7:		/* device not available */
++	case 8:		/* double fault (errcode) */
++	case 10:		/* invalid TSS (errcode) */
++	case 12:		/* stack fault (errcode) */
++	case 16:		/* floating point error */
++	case 17:		/* alignment check (errcode) */
++	default:		/* any undocumented */
++		break;
++	case 11:		/* segment not present (errcode) */
++	case 13:		/* general protection (errcode) */
++	case 14:		/* page fault (special errcode) */
++	case 19:		/* cache flush denied */
++		if (mem_err_expected) {
++			/*
++			 * This fault occured because of the
++			 * get_char or set_char routines.  These
++			 * two routines use either eax of edx to
++			 * indirectly reference the location in
++			 * memory that they are working with.
++			 * For a page fault, when we return the
++			 * instruction will be retried, so we
++			 * have to make sure that these
++			 * registers point to valid memory.
++			 */
++			mem_err = 1;	/* set mem error flag */
++			mem_err_expected = 0;
++			mem_err_cnt++;	/* helps in debugging */
++			/* make valid address */
++			regs.eax = (long) &garbage_loc;
++			/* make valid address */
++			regs.edx = (long) &garbage_loc;
++			if (remote_debug)
++				printk("Return after memory error: "
++				       "mem_err_cnt=%d\n", mem_err_cnt);
++			if (debug_regs)
++				print_regs(&regs);
++			goto exit_kgdb;
++		}
++		break;
++	}
++	if (remote_debug)
++		printk("kgdb : entered kgdb on cpu %d\n", smp_processor_id());
++
++	gdb_i386vector = exceptionVector;
++	gdb_i386errcode = err_code;
++	kgdb_info.called_from = __builtin_return_address(0);
++#ifdef CONFIG_SMP
++	/*
++	 * OK, we can now communicate, lets tell gdb about the sync.
++	 * but only if we had a problem.
++	 */
++	switch (entry_state) {
++	case NO_NMI:
++		to_gdb("NMI not active, other cpus not stopped\n");
++		break;
++	case NO_SYNC:
++		to_gdb("Some cpus not stopped, see 'kgdb_info' for details\n");
++	default:;
++	}
++
++#endif
++/*
++ * Set up the gdb function call area.
++ */
++	trap_cpu = smp_processor_id();
++	OLD_esp = NEW_esp = (int) (&linux_regs->esp);
++
++      IF_SMP(once_again:)
++	    /* reply to host that an exception has occurred */
++	    remcomOutBuffer[0] = 'S';
++	remcomOutBuffer[1] = hexchars[signo >> 4];
++	remcomOutBuffer[2] = hexchars[signo % 16];
++	remcomOutBuffer[3] = 0;
++
++	putpacket(remcomOutBuffer);
++
++	while (1 == 1) {
++		error = 0;
++		remcomOutBuffer[0] = 0;
++		getpacket(remcomInBuffer);
++		switch (remcomInBuffer[0]) {
++		case '?':
++			remcomOutBuffer[0] = 'S';
++			remcomOutBuffer[1] = hexchars[signo >> 4];
++			remcomOutBuffer[2] = hexchars[signo % 16];
++			remcomOutBuffer[3] = 0;
++			break;
++		case 'd':
++			remote_debug = !(remote_debug);	/* toggle debug flag */
++			printk("Remote debug %s\n",
++			       remote_debug ? "on" : "off");
++			break;
++		case 'g':	/* return the value of the CPU registers */
++			get_gdb_regs(usethread, &regs, gdb_regs);
++			mem2hex((char *) gdb_regs,
++				remcomOutBuffer, NUMREGBYTES, 0);
++			break;
++		case 'G':	/* set the value of the CPU registers - return OK */
++			hex2mem(&remcomInBuffer[1],
++				(char *) gdb_regs, NUMREGBYTES, 0);
++			if (!usethread || usethread == current) {
++				gdb_regs_to_regs(gdb_regs, &regs);
++				strcpy(remcomOutBuffer, "OK");
++			} else {
++				strcpy(remcomOutBuffer, "E00");
++			}
++			break;
++
++		case 'P':{	/* set the value of a single CPU register -
++				   return OK */
++				/*
++				 * For some reason, gdb wants to talk about psudo
++				 * registers (greater than 15).	 These may have
++				 * meaning for ptrace, but for us it is safe to
++				 * ignor them.	We do this by dumping them into
++				 * _GS which we also ignor, but do have memory for.
++				 */
++				int regno;
++
++				ptr = &remcomInBuffer[1];
++				regs_to_gdb_regs(gdb_regs, &regs);
++				if ((!usethread || usethread == current) &&
++				    hexToInt(&ptr, &regno) &&
++				    *ptr++ == '=' && (regno >= 0)) {
++					regno =
++					    (regno >= NUMREGS ? _GS : regno);
++					hex2mem(ptr, (char *) &gdb_regs[regno],
++						4, 0);
++					gdb_regs_to_regs(gdb_regs, &regs);
++					strcpy(remcomOutBuffer, "OK");
++					break;
++				}
++				strcpy(remcomOutBuffer, "E01");
++				break;
++			}
++
++			/* mAA..AA,LLLL	 Read LLLL bytes at address AA..AA */
++		case 'm':
++			/* TRY TO READ %x,%x.  IF SUCCEED, SET PTR = 0 */
++			ptr = &remcomInBuffer[1];
++			if (hexToInt(&ptr, &addr) &&
++			    (*(ptr++) == ',') && (hexToInt(&ptr, &length))) {
++				ptr = 0;
++				/*
++				 * hex doubles the byte count
++				 */
++				if (length > (BUFMAX / 2))
++					length = BUFMAX / 2;
++				mem2hex((char *) addr,
++					remcomOutBuffer, length, 1);
++				if (mem_err) {
++					strcpy(remcomOutBuffer, "E03");
++					debug_error("memory fault\n", NULL);
++				}
++			}
++
++			if (ptr) {
++				strcpy(remcomOutBuffer, "E01");
++				debug_error
++				    ("malformed read memory command: %s\n",
++				     remcomInBuffer);
++			}
++			break;
++
++			/* MAA..AA,LLLL:
++			   Write LLLL bytes at address AA.AA return OK */
++		case 'M':
++			/* TRY TO READ '%x,%x:'.  IF SUCCEED, SET PTR = 0 */
++			ptr = &remcomInBuffer[1];
++			if (hexToInt(&ptr, &addr) &&
++			    (*(ptr++) == ',') &&
++			    (hexToInt(&ptr, &length)) && (*(ptr++) == ':')) {
++				hex2mem(ptr, (char *) addr, length, 1);
++
++				if (mem_err) {
++					strcpy(remcomOutBuffer, "E03");
++					debug_error("memory fault\n", NULL);
++				} else {
++					strcpy(remcomOutBuffer, "OK");
++				}
++
++				ptr = 0;
++			}
++			if (ptr) {
++				strcpy(remcomOutBuffer, "E02");
++				debug_error
++				    ("malformed write memory command: %s\n",
++				     remcomInBuffer);
++			}
++			break;
++		case 'S':
++			remcomInBuffer[0] = 's';
++		case 'C':
++			/* Csig;AA..AA where ;AA..AA is optional
++			 * continue with signal
++			 * Since signals are meaning less to us, delete that
++			 * part and then fall into the 'c' code.
++			 */
++			ptr = &remcomInBuffer[1];
++			length = 2;
++			while (*ptr && *ptr != ';') {
++				length++;
++				ptr++;
++			}
++			if (*ptr) {
++				do {
++					ptr++;
++					*(ptr - length++) = *ptr;
++				} while (*ptr);
++			} else {
++				remcomInBuffer[1] = 0;
++			}
++
++			/* cAA..AA  Continue at address AA..AA(optional) */
++			/* sAA..AA  Step one instruction from AA..AA(optional) */
++			/* D	    detach, reply OK and then continue */
++		case 'c':
++		case 's':
++		case 'D':
++
++			/* try to read optional parameter,
++			   pc unchanged if no parm */
++			ptr = &remcomInBuffer[1];
++			if (hexToInt(&ptr, &addr)) {
++				if (remote_debug)
++					printk("Changing EIP to 0x%x\n", addr);
++
++				regs.eip = addr;
++			}
++
++			newPC = regs.eip;
++
++			/* clear the trace bit */
++			regs.eflags &= 0xfffffeff;
++
++			/* set the trace bit if we're stepping */
++			if (remcomInBuffer[0] == 's')
++				regs.eflags |= 0x100;
++
++			/* detach is a friendly version of continue. Note that
++			   debugging is still enabled (e.g hit control C)
++			 */
++			if (remcomInBuffer[0] == 'D') {
++				strcpy(remcomOutBuffer, "OK");
++				putpacket(remcomOutBuffer);
++			}
++
++			if (remote_debug) {
++				printk("Resuming execution\n");
++				print_regs(&regs);
++			}
++			asm volatile ("movl %%db6, %0\n":"=r" (dr6)
++				      :);
++			if (!(dr6 & 0x4000)) {
++				for (breakno = 0; breakno < 4; ++breakno) {
++					if (dr6 & (1 << breakno) &&
++					    (breakinfo[breakno].type == 0)) {
++						/* Set restore flag */
++						regs.eflags |= 0x10000;
++						break;
++					}
++				}
++			}
++			correct_hw_break();
++			asm volatile ("movl %0, %%db6\n"::"r" (0));
++			goto exit_kgdb;
++
++			/* kill the program */
++		case 'k':	/* do nothing */
++			break;
++
++			/* query */
++		case 'q':
++			nothreads = 0;
++			switch (remcomInBuffer[1]) {
++			case 'f':
++				threadid = 1;
++				thread_list = 2;
++				thread_list_start = (usethread ? : current);
++			case 's':
++				if (!cmp_str(&remcomInBuffer[2],
++					     "ThreadInfo", 10))
++					break;
++
++				remcomOutBuffer[nothreads++] = 'm';
++				for (; threadid < PID_MAX + MAX_NO_CPUS;
++				     threadid++) {
++					thread = getthread(threadid);
++					if (thread) {
++						nothreads += int_to_hex_v(
++							&remcomOutBuffer[
++								nothreads],
++							threadid);
++						if (thread_min > threadid)
++							thread_min = threadid;
++						remcomOutBuffer[
++							nothreads] = ',';
++						nothreads++;
++						if (nothreads > BUFMAX - 10)
++							break;
++					}
++				}
++				if (remcomOutBuffer[nothreads - 1] == 'm') {
++					remcomOutBuffer[nothreads - 1] = 'l';
++				} else {
++					nothreads--;
++				}
++				remcomOutBuffer[nothreads] = 0;
++				break;
++
++#ifdef old_thread_list /* Old thread info request */
++			case 'L':
++				/* List threads */
++				thread_list = 2;
++				thread_list_start = (usethread ? : current);
++				unpack_byte(remcomInBuffer + 3, &maxthreads);
++				unpack_threadid(remcomInBuffer + 5, &thref);
++				do {
++					int buf_thread_limit =
++					    (BUFMAX - 22) / BUF_THREAD_ID_SIZE;
++					if (maxthreads > buf_thread_limit) {
++						maxthreads = buf_thread_limit;
++					}
++				} while (0);
++				remcomOutBuffer[0] = 'q';
++				remcomOutBuffer[1] = 'M';
++				remcomOutBuffer[4] = '0';
++				pack_threadid(remcomOutBuffer + 5, &thref);
++
++				threadid = threadref_to_int(&thref);
++				for (nothreads = 0;
++				     nothreads < maxthreads &&
++				     threadid < PID_MAX + MAX_NO_CPUS;
++				     threadid++) {
++					thread = getthread(threadid);
++					if (thread) {
++						int_to_threadref(&thref,
++								 threadid);
++						pack_threadid(remcomOutBuffer +
++							      21 +
++							      nothreads * 16,
++							      &thref);
++						nothreads++;
++						if (thread_min > threadid)
++							thread_min = threadid;
++					}
++				}
++
++				if (threadid == PID_MAX + MAX_NO_CPUS) {
++					remcomOutBuffer[4] = '1';
++				}
++				pack_hex_byte(remcomOutBuffer + 2, nothreads);
++				remcomOutBuffer[21 + nothreads * 16] = '\0';
++				break;
++#endif
++			case 'C':
++				/* Current thread id */
++				remcomOutBuffer[0] = 'Q';
++				remcomOutBuffer[1] = 'C';
++				threadid = current->pid;
++				if (!threadid) {
++					/*
++					 * idle thread
++					 */
++					for (threadid = PID_MAX;
++					     threadid < PID_MAX + MAX_NO_CPUS;
++					     threadid++) {
++						if (current ==
++						    idle_task(threadid -
++							      PID_MAX))
++							break;
++					}
++				}
++				int_to_threadref(&thref, threadid);
++				pack_threadid(remcomOutBuffer + 2, &thref);
++				remcomOutBuffer[18] = '\0';
++				break;
++
++			case 'E':
++				/* Print exception info */
++				printexceptioninfo(exceptionVector,
++						   err_code, remcomOutBuffer);
++				break;
++			case 'T':{
++				char * nptr;
++				/* Thread extra info */
++				if (!cmp_str(&remcomInBuffer[2],
++					    "hreadExtraInfo,", 15)) {
++					break;
++				}
++				ptr = &remcomInBuffer[17];
++				hexToInt(&ptr, &threadid);
++				thread = getthread(threadid);
++				nptr = &thread->comm[0];
++				length = 0;
++				ptr = &remcomOutBuffer[0];
++				do {
++					length++;
++					ptr = pack_hex_byte(ptr, *nptr++);
++				 } while (*nptr && length < 16);
++				/*
++				 * would like that 16 to be the size of
++				 * task_struct.comm but don't know the
++				 * syntax..
++				 */
++				*ptr = 0;
++			}
++			}
++			break;
++
++			/* task related */
++		case 'H':
++			switch (remcomInBuffer[1]) {
++			case 'g':
++				ptr = &remcomInBuffer[2];
++				hexToInt(&ptr, &threadid);
++				thread = getthread(threadid);
++				if (!thread) {
++					remcomOutBuffer[0] = 'E';
++					remcomOutBuffer[1] = '\0';
++					break;
++				}
++				/*
++				 * Just in case I forget what this is all about,
++				 * the "thread info" command to gdb causes it
++				 * to ask for a thread list.  It then switches
++				 * to each thread and asks for the registers.
++				 * For this (and only this) usage, we want to
++				 * fudge the registers of tasks not on the run
++				 * list (i.e. waiting) to show the routine that
++				 * called schedule. Also, gdb, is a minimalist
++				 * in that if the current thread is the last
++				 * it will not re-read the info when done.
++				 * This means that in this case we must show
++				 * the real registers. So here is how we do it:
++				 * Each entry we keep track of the min
++				 * thread in the list (the last that gdb will)
++				 * get info for.  We also keep track of the
++				 * starting thread.
++				 * "thread_list" is cleared when switching back
++				 * to the min thread if it is was current, or
++				 * if it was not current, thread_list is set
++				 * to 1.  When the switch to current comes,
++				 * if thread_list is 1, clear it, else do
++				 * nothing.
++				 */
++				usethread = thread;
++				if ((thread_list == 1) &&
++				    (thread == thread_list_start)) {
++					thread_list = 0;
++				}
++				if (thread_list && (threadid == thread_min)) {
++					if (thread == thread_list_start) {
++						thread_list = 0;
++					} else {
++						thread_list = 1;
++					}
++				}
++				/* follow through */
++			case 'c':
++				remcomOutBuffer[0] = 'O';
++				remcomOutBuffer[1] = 'K';
++				remcomOutBuffer[2] = '\0';
++				break;
++			}
++			break;
++
++			/* Query thread status */
++		case 'T':
++			ptr = &remcomInBuffer[1];
++			hexToInt(&ptr, &threadid);
++			thread = getthread(threadid);
++			if (thread) {
++				remcomOutBuffer[0] = 'O';
++				remcomOutBuffer[1] = 'K';
++				remcomOutBuffer[2] = '\0';
++				if (thread_min > threadid)
++					thread_min = threadid;
++			} else {
++				remcomOutBuffer[0] = 'E';
++				remcomOutBuffer[1] = '\0';
++			}
++			break;
++
++		case 'Y': /* set up a hardware breakpoint */
++			ptr = &remcomInBuffer[1];
++			hexToInt(&ptr, &breakno);
++			ptr++;
++			hexToInt(&ptr, &breaktype);
++			ptr++;
++			hexToInt(&ptr, &length);
++			ptr++;
++			hexToInt(&ptr, &addr);
++			if (set_hw_break(breakno & 0x3,
++					 breaktype & 0x3,
++					 length & 0x3, addr) == 0) {
++				strcpy(remcomOutBuffer, "OK");
++			} else {
++				strcpy(remcomOutBuffer, "ERROR");
++			}
++			break;
++
++			/* Remove hardware breakpoint */
++		case 'y':
++			ptr = &remcomInBuffer[1];
++			hexToInt(&ptr, &breakno);
++			if (remove_hw_break(breakno & 0x3) == 0) {
++				strcpy(remcomOutBuffer, "OK");
++			} else {
++				strcpy(remcomOutBuffer, "ERROR");
++			}
++			break;
++
++		case 'r':	/* reboot */
++			strcpy(remcomOutBuffer, "OK");
++			putpacket(remcomOutBuffer);
++			/*to_gdb("Rebooting\n"); */
++			/* triplefault	 no return from here */
++			{
++				static long no_idt[2];
++				__asm__ __volatile__("lidt %0"::"m"(no_idt[0]));
++				BREAKPOINT;
++			}
++
++		}		/* switch */
++
++		/* reply to the request */
++		putpacket(remcomOutBuffer);
++	}			/* while(1==1) */
++	/*
++	 *  reached by goto only.
++	 */
++      exit_kgdb:
++	/*
++	 * Here is where we set up to trap a gdb function call.	 NEW_esp
++	 * will be changed if we are trying to do this.	 We handle both
++	 * adding and subtracting, thus allowing gdb to put grung on
++	 * the stack which it removes later.
++	 */
++	if (NEW_esp != OLD_esp) {
++		int *ptr = END_OF_LOOKASIDE;
++		if (NEW_esp < OLD_esp)
++			ptr -= (OLD_esp - NEW_esp) / sizeof (int);
++		*--ptr = linux_regs->eflags;
++		*--ptr = linux_regs->xcs;
++		*--ptr = linux_regs->eip;
++		*--ptr = linux_regs->ecx;
++		*--ptr = linux_regs->ebx;
++		*--ptr = linux_regs->eax;
++		linux_regs->ecx = NEW_esp - (sizeof (int) * 6);
++		linux_regs->ebx = (unsigned int) END_OF_LOOKASIDE;
++		if (NEW_esp < OLD_esp) {
++			linux_regs->eip = (unsigned int) fn_call_stub;
++		} else {
++			linux_regs->eip = (unsigned int) fn_rtn_stub;
++			linux_regs->eax = NEW_esp;
++		}
++		linux_regs->eflags &= ~(IF_BIT | TF_BIT);
++	}
++#ifdef CONFIG_SMP
++	/*
++	 * Release gdb wait locks
++	 * Sanity check time.  Must have at least one cpu to run.  Also single
++	 * step must not be done if the current cpu is on hold.
++	 */
++	if (spinlock_count == 1) {
++		int ss_hold = (regs.eflags & 0x100) && kgdb_info.hold_on_sstep;
++		int cpu_avail = 0;
++		int i;
++
++		for (i = 0; i < MAX_NO_CPUS; i++) {
++			if (!cpu_online(i))
++				break;
++			if (!hold_cpu(i)) {
++				cpu_avail = 1;
++			}
++		}
++		/*
++		 * Early in the bring up there will be NO cpus on line...
++		 */
++		if (!cpu_avail && !cpus_empty(cpu_online_map)) {
++			to_gdb("No cpus unblocked, see 'kgdb_info.hold_cpu'\n");
++			goto once_again;
++		}
++		if (hold_cpu(smp_processor_id()) && (regs.eflags & 0x100)) {
++			to_gdb
++			    ("Current cpu must be unblocked to single step\n");
++			goto once_again;
++		}
++		if (!(ss_hold)) {
++			int i;
++			for (i = 0; i < MAX_NO_CPUS; i++) {
++				if (!hold_cpu(i)) {
++					spin_unlock(&waitlocks[i]);
++				}
++			}
++		} else {
++			spin_unlock(&waitlocks[smp_processor_id()]);
++		}
++		/* Release kgdb spinlock */
++		KGDB_SPIN_UNLOCK(&kgdb_spinlock);
++		/*
++		 * If this cpu is on hold, this is where we
++		 * do it.  Note, the NMI will pull us out of here,
++		 * but will return as the above lock is not held.
++		 * We will stay here till another cpu releases the lock for us.
++		 */
++		spin_unlock_wait(waitlocks + smp_processor_id());
++		kgdb_local_irq_restore(flags);
++		return (0);
++	}
++#if 0
++exit_just_unlock:
++#endif
++#endif
++	/* Release kgdb spinlock */
++	KGDB_SPIN_UNLOCK(&kgdb_spinlock);
++	kgdb_local_irq_restore(flags);
++	return (0);
++}
++
++/* this function is used to set up exception handlers for tracing and
++ * breakpoints.
++ * This function is not needed as the above line does all that is needed.
++ * We leave it for backward compatitability...
++ */
++void
++set_debug_traps(void)
++{
++	/*
++	 * linux_debug_hook is defined in traps.c.  We store a pointer
++	 * to our own exception handler into it.
++
++	 * But really folks, every hear of labeled common, an old Fortran
++	 * concept.  Lots of folks can reference it and it is define if
++	 * anyone does.	 Only one can initialize it at link time.  We do
++	 * this with the hook.	See the statement above.  No need for any
++	 * executable code and it is ready as soon as the kernel is
++	 * loaded.  Very desirable in kernel debugging.
++
++	 linux_debug_hook = handle_exception ;
++	 */
++
++	/* In case GDB is started before us, ack any packets (presumably
++	   "$?#xx") sitting there.
++	   putDebugChar ('+');
++
++	   initialized = 1;
++	 */
++}
++
++/* This function will generate a breakpoint exception.	It is used at the
++   beginning of a program to sync up with a debugger and can be used
++   otherwise as a quick means to stop program execution and "break" into
++   the debugger. */
++/* But really, just use the BREAKPOINT macro.  We will handle the int stuff
++ */
++
++#ifdef later
++/*
++ * possibly we should not go thru the traps.c code at all?  Someday.
++ */
++void
++do_kgdb_int3(struct pt_regs *regs, long error_code)
++{
++	kgdb_handle_exception(3, 5, error_code, regs);
++	return;
++}
++#endif
++#undef regs
++#ifdef CONFIG_TRAP_BAD_SYSCALL_EXITS
++fastcall void sys_call_exit(struct pt_regs *regs)
++{
++	if (preempt_count())
++		printk("Sys call %d return with %x preempt_count\n",
++			(int) regs->orig_eax, preempt_count());
++}
++#endif
++#ifdef CONFIG_STACK_OVERFLOW_TEST
++#include <asm/kgdb.h>
++fastcall void stack_overflow(void)
++{
++#ifdef BREAKPOINT
++	BREAKPOINT;
++#else
++	printk("Kernel stack overflow, looping forever\n");
++#endif
++	while (1) {
++	}
++}
++#endif
++
++#if defined(CONFIG_SMP) || defined(CONFIG_KGDB_CONSOLE)
++char gdbconbuf[BUFMAX];
++
++static void
++kgdb_gdb_message(const char *s, unsigned count)
++{
++	int i;
++	int wcount;
++	char *bufptr;
++	/*
++	 * This takes care of NMI while spining out chars to gdb
++	 */
++	IF_SMP(in_kgdb_console = 1);
++	gdbconbuf[0] = 'O';
++	bufptr = gdbconbuf + 1;
++	while (count > 0) {
++		if ((count << 1) > (BUFMAX - 2)) {
++			wcount = (BUFMAX - 2) >> 1;
++		} else {
++			wcount = count;
++		}
++		count -= wcount;
++		for (i = 0; i < wcount; i++) {
++			bufptr = pack_hex_byte(bufptr, s[i]);
++		}
++		*bufptr = '\0';
++		s += wcount;
++
++		putpacket(gdbconbuf);
++
++	}
++	IF_SMP(in_kgdb_console = 0);
++}
++#endif
++#ifdef CONFIG_SMP
++static void
++to_gdb(const char *s)
++{
++	int count = 0;
++	while (s[count] && (count++ < BUFMAX)) ;
++	kgdb_gdb_message(s, count);
++}
++#endif
++#ifdef CONFIG_KGDB_CONSOLE
++#include <linux/console.h>
++#include <linux/init.h>
++#include <linux/fs.h>
++#include <asm/uaccess.h>
++#include <asm/semaphore.h>
++
++void
++kgdb_console_write(struct console *co, const char *s, unsigned count)
++{
++
++	if (gdb_i386vector == -1) {
++		/*
++		 * We have not yet talked to gdb.  What to do...
++		 * lets break, on continue we can do the write.
++		 * But first tell him whats up. Uh, well no can do,
++		 * as this IS the console.  Oh well...
++		 * We do need to wait or the messages will be lost.
++		 * Other option would be to tell the above code to
++		 * ignore this breakpoint and do an auto return,
++		 * but that might confuse gdb.	Also this happens
++		 * early enough in boot up that we don't have the traps
++		 * set up yet, so...
++		 */
++		breakpoint();
++	}
++	kgdb_gdb_message(s, count);
++}
++
++/*
++ * ------------------------------------------------------------
++ * Serial KGDB driver
++ * ------------------------------------------------------------
++ */
++
++static struct console kgdbcons = {
++	name:"kgdb",
++	write:kgdb_console_write,
++#ifdef CONFIG_KGDB_USER_CONSOLE
++	device:kgdb_console_device,
++#endif
++	flags:CON_PRINTBUFFER | CON_ENABLED,
++	index:-1,
++};
++
++/*
++ * The trick here is that this file gets linked before printk.o
++ * That means we get to peer at the console info in the command
++ * line before it does.	 If we are up, we register, otherwise,
++ * do nothing.	By returning 0, we allow printk to look also.
++ */
++static int kgdb_console_enabled;
++
++int __init
++kgdb_console_init(char *str)
++{
++	if ((strncmp(str, "kgdb", 4) == 0) || (strncmp(str, "gdb", 3) == 0)) {
++		register_console(&kgdbcons);
++		kgdb_console_enabled = 1;
++	}
++	return 0;		/* let others look at the string */
++}
++
++__setup("console=", kgdb_console_init);
++
++#ifdef CONFIG_KGDB_USER_CONSOLE
++static kdev_t kgdb_console_device(struct console *c);
++/* This stuff sort of works, but it knocks out telnet devices
++ * we are leaving it here in case we (or you) find time to figure it out
++ * better..
++ */
++
++/*
++ * We need a real char device as well for when the console is opened for user
++ * space activities.
++ */
++
++static int
++kgdb_consdev_open(struct inode *inode, struct file *file)
++{
++	return 0;
++}
++
++static ssize_t
++kgdb_consdev_write(struct file *file, const char *buf,
++		   size_t count, loff_t * ppos)
++{
++	int size, ret = 0;
++	static char kbuf[128];
++	static DECLARE_MUTEX(sem);
++
++	/* We are not reentrant... */
++	if (down_interruptible(&sem))
++		return -ERESTARTSYS;
++
++	while (count > 0) {
++		/* need to copy the data from user space */
++		size = count;
++		if (size > sizeof (kbuf))
++			size = sizeof (kbuf);
++		if (copy_from_user(kbuf, buf, size)) {
++			ret = -EFAULT;
++			break;;
++		}
++		kgdb_console_write(&kgdbcons, kbuf, size);
++		count -= size;
++		ret += size;
++		buf += size;
++	}
++
++	up(&sem);
++
++	return ret;
++}
++
++struct file_operations kgdb_consdev_fops = {
++	open:kgdb_consdev_open,
++	write:kgdb_consdev_write
++};
++static kdev_t
++kgdb_console_device(struct console *c)
++{
++	return MKDEV(TTYAUX_MAJOR, 1);
++}
++
++/*
++ * This routine gets called from the serial stub in the i386/lib
++ * This is so it is done late in bring up (just before the console open).
++ */
++void
++kgdb_console_finit(void)
++{
++	if (kgdb_console_enabled) {
++		char *cptr = cdevname(MKDEV(TTYAUX_MAJOR, 1));
++		char *cp = cptr;
++		while (*cptr && *cptr != '(')
++			cptr++;
++		*cptr = 0;
++		unregister_chrdev(TTYAUX_MAJOR, cp);
++		register_chrdev(TTYAUX_MAJOR, "kgdb", &kgdb_consdev_fops);
++	}
++}
++#endif
++#endif
++#ifdef CONFIG_KGDB_TS
++#include <asm/msr.h>		/* time stamp code */
++#include <asm/hardirq.h>	/* in_interrupt */
++#ifdef CONFIG_KGDB_TS_64
++#define DATA_POINTS 64
++#endif
++#ifdef CONFIG_KGDB_TS_128
++#define DATA_POINTS 128
++#endif
++#ifdef CONFIG_KGDB_TS_256
++#define DATA_POINTS 256
++#endif
++#ifdef CONFIG_KGDB_TS_512
++#define DATA_POINTS 512
++#endif
++#ifdef CONFIG_KGDB_TS_1024
++#define DATA_POINTS 1024
++#endif
++#ifndef DATA_POINTS
++#define DATA_POINTS 128		/* must be a power of two */
++#endif
++#define INDEX_MASK (DATA_POINTS - 1)
++#if (INDEX_MASK & DATA_POINTS)
++#error "CONFIG_KGDB_TS_COUNT must be a power of 2"
++#endif
++struct kgdb_and_then_struct {
++#ifdef CONFIG_SMP
++	int on_cpu;
++#endif
++	struct task_struct *task;
++	long long at_time;
++	int from_ln;
++	char *in_src;
++	void *from;
++	int *with_shpf;
++	int data0;
++	int data1;
++};
++struct kgdb_and_then_struct2 {
++#ifdef CONFIG_SMP
++	int on_cpu;
++#endif
++	struct task_struct *task;
++	long long at_time;
++	int from_ln;
++	char *in_src;
++	void *from;
++	int *with_shpf;
++	struct task_struct *t1;
++	struct task_struct *t2;
++};
++struct kgdb_and_then_struct kgdb_data[DATA_POINTS];
++
++struct kgdb_and_then_struct *kgdb_and_then = &kgdb_data[0];
++int kgdb_and_then_count;
++
++void
++kgdb_tstamp(int line, char *source, int data0, int data1)
++{
++	static spinlock_t ts_spin = SPIN_LOCK_UNLOCKED;
++	int flags;
++	kgdb_local_irq_save(flags);
++	spin_lock(&ts_spin);
++	rdtscll(kgdb_and_then->at_time);
++#ifdef CONFIG_SMP
++	kgdb_and_then->on_cpu = smp_processor_id();
++#endif
++	kgdb_and_then->task = current;
++	kgdb_and_then->from_ln = line;
++	kgdb_and_then->in_src = source;
++	kgdb_and_then->from = __builtin_return_address(0);
++	kgdb_and_then->with_shpf = (int *) (((flags & IF_BIT) >> 9) |
++					    (preempt_count() << 8));
++	kgdb_and_then->data0 = data0;
++	kgdb_and_then->data1 = data1;
++	kgdb_and_then = &kgdb_data[++kgdb_and_then_count & INDEX_MASK];
++	spin_unlock(&ts_spin);
++	kgdb_local_irq_restore(flags);
++#ifdef CONFIG_PREEMPT
++
++#endif
++	return;
++}
++#endif
++typedef int gdb_debug_hook(int exceptionVector,
++			   int signo, int err_code, struct pt_regs *linux_regs);
++gdb_debug_hook *linux_debug_hook = &kgdb_handle_exception;	/* histerical reasons... */
+Index: linux-2.6.16.19/arch/i386/kernel/nmi.c
+===================================================================
+--- linux-2.6.16.19.orig/arch/i386/kernel/nmi.c	2006-06-23 17:05:14.000000000 -0700
++++ linux-2.6.16.19/arch/i386/kernel/nmi.c	2006-06-23 17:06:11.000000000 -0700
+@@ -32,7 +32,17 @@
+ 
+ #include "mach_traps.h"
+ 
++#ifdef CONFIG_KGDB
++#include <asm/kgdb.h>
++#ifdef CONFIG_SMP
++unsigned int nmi_watchdog = NMI_IO_APIC;
++#else
++unsigned int nmi_watchdog = NMI_LOCAL_APIC;
++#endif
++#else
+ unsigned int nmi_watchdog = NMI_NONE;
++#endif
++
+ extern int unknown_nmi_panic;
+ static unsigned int nmi_hz = HZ;
+ static unsigned int nmi_perfctr_msr;	/* the MSR to reset in NMI handler */
+@@ -518,6 +528,9 @@
+ 	 */
+ 	touch_softlockup_watchdog();
+ }
++#ifdef CONFIG_KGDB
++int tune_watchdog = 5*HZ;
++#endif
+ 
+ extern void die_nmi(struct pt_regs *, const char *msg);
+ 
+@@ -533,12 +546,24 @@
+ 
+ 	sum = per_cpu(irq_stat, cpu).apic_timer_irqs;
+ 
++#ifdef CONFIG_KGDB
++	if (!in_kgdb(regs) && last_irq_sums[cpu] == sum) {
++
++#else
+ 	if (last_irq_sums[cpu] == sum) {
++#endif
+ 		/*
+ 		 * Ayiee, looks like this CPU is stuck ...
+ 		 * wait a few IRQs (5 seconds) before doing the oops ...
+ 		 */
+ 		alert_counter[cpu]++;
++#ifdef CONFIG_KGDB
++		if (alert_counter[cpu] == tune_watchdog) {
++			kgdb_handle_exception(2, SIGPWR, 0, regs);
++			last_irq_sums[cpu] = sum;
++			alert_counter[cpu] = 0;
++		}
++#endif
+ 		if (alert_counter[cpu] == 5*nmi_hz)
+ 			/*
+ 			 * die_nmi will return ONLY if NOTIFY_STOP happens..
+Index: linux-2.6.16.19/arch/i386/kernel/smp.c
+===================================================================
+--- linux-2.6.16.19.orig/arch/i386/kernel/smp.c	2006-06-23 17:05:14.000000000 -0700
++++ linux-2.6.16.19/arch/i386/kernel/smp.c	2006-06-23 17:06:11.000000000 -0700
+@@ -468,7 +468,17 @@
+ {
+ 	on_each_cpu(do_flush_tlb_all, NULL, 1, 1);
+ }
+-
++#ifdef CONFIG_KGDB
++/*
++ * By using the NMI code instead of a vector we just sneak thru the
++ * word generator coming out with just what we want.  AND it does
++ * not matter if clustered_apic_mode is set or not.
++ */
++void smp_send_nmi_allbutself(void)
++{
++	send_IPI_allbutself(APIC_DM_NMI);
++}
++#endif
+ /*
+  * this function sends a 'reschedule' IPI to another CPU.
+  * it goes straight through and wastes no time serializing
+Index: linux-2.6.16.19/arch/i386/kernel/traps.c
+===================================================================
+--- linux-2.6.16.19.orig/arch/i386/kernel/traps.c	2006-06-23 17:05:14.000000000 -0700
++++ linux-2.6.16.19/arch/i386/kernel/traps.c	2006-06-23 17:06:11.000000000 -0700
+@@ -106,6 +106,39 @@
+ }
+ EXPORT_SYMBOL(register_die_notifier);
+ 
++#ifdef CONFIG_KGDB
++extern void sysenter_past_esp(void);
++#include <asm/kgdb.h>
++#include <linux/init.h>
++void set_intr_gate(unsigned int n, void *addr);
++static void set_intr_usr_gate(unsigned int n, void *addr);
++/*
++ * Should be able to call this breakpoint() very early in
++ * bring up.  Just hard code the call where needed.
++ * The breakpoint() code is here because set_?_gate() functions
++ * are local (static) to trap.c.  They need be done only once,
++ * but it does not hurt to do them over.
++ */
++void breakpoint(void)
++{
++	set_intr_usr_gate(3,&int3); /* disable ints on trap */
++	set_intr_gate(1,&debug);
++	set_intr_gate(14,&page_fault);
++
++        BREAKPOINT;
++}
++#define	CHK_REMOTE_DEBUG(trapnr,signr,error_code,regs,after)		\
++    {									\
++	if (!user_mode(regs)  ) \
++	{								\
++		kgdb_handle_exception(trapnr, signr, error_code, regs);	\
++		after;							\
++	} else if ((trapnr == 3) && (regs->eflags &0x200)) local_irq_enable(); \
++    }
++#else
++#define	CHK_REMOTE_DEBUG(trapnr,signr,error_code,regs,after)
++#endif
++
+ static inline int valid_stack_ptr(struct thread_info *tinfo, void *p)
+ {
+ 	return	p > (void *)tinfo &&
+@@ -365,6 +398,15 @@
+ #endif
+ 		if (nl)
+ 			printk("\n");
++#ifdef CONFIG_KGDB
++	/* This is about the only place we want to go to kgdb even if in
++	 * user mode.  But we must go in via a trap so within kgdb we will
++	 * always be in kernel mode.
++	 */
++		if (user_mode(regs))
++			BREAKPOINT;
++#endif
++	 	CHK_REMOTE_DEBUG(0,SIGTRAP,err,regs,)
+ 	notify_die(DIE_OOPS, (char *)str, regs, err, 255, SIGSEGV);
+ 		show_registers(regs);
+   	} else
+@@ -435,6 +477,7 @@
+ #define DO_ERROR(trapnr, signr, str, name) \
+ fastcall void do_##name(struct pt_regs * regs, long error_code) \
+ { \
++	CHK_REMOTE_DEBUG(trapnr,signr,error_code,regs,) \
+ 	if (notify_die(DIE_TRAP, str, regs, error_code, trapnr, signr) \
+ 						== NOTIFY_STOP) \
+ 		return; \
+@@ -458,6 +501,7 @@
+ #define DO_VM86_ERROR(trapnr, signr, str, name) \
+ fastcall void do_##name(struct pt_regs * regs, long error_code) \
+ { \
++	CHK_REMOTE_DEBUG(trapnr, signr, error_code,regs, return) \
+ 	if (notify_die(DIE_TRAP, str, regs, error_code, trapnr, signr) \
+ 						== NOTIFY_STOP) \
+ 		return; \
+@@ -547,6 +591,7 @@
+ 
+ gp_in_kernel:
+ 	if (!fixup_exception(regs)) {
++ 		CHK_REMOTE_DEBUG(13,SIGSEGV,error_code,regs,)
+ 		if (notify_die(DIE_GPF, "general protection fault", regs,
+ 				error_code, 13, SIGSEGV) == NOTIFY_STOP)
+ 			return;
+@@ -775,10 +820,33 @@
+ 		 * check for kernel mode by just checking the CPL
+ 		 * of CS.
+ 		 */
++#ifdef CONFIG_KGDB
++		/*
++		 * I think this is the only "real" case of a TF in the kernel
++		 * that really belongs to user space.  Others are
++		 * "Ours all ours!"
++		 */
++		if (!user_mode(regs) && (void *)regs->eip == sysenter_past_esp)
++			goto clear_TF_reenable;
++#else
+ 		if (!user_mode(regs))
+ 			goto clear_TF_reenable;
++#endif
+ 	}
+ 
++#ifdef CONFIG_KGDB
++	/*
++	 * If this is a kernel mode trap, we need to reset db7 to allow us
++	 * to continue sanely ALSO skip the signal delivery
++	 */
++	if (!user_mode(regs))
++		goto clear_dr7;
++
++	/* if not kernel, allow ints but only if they were on */
++	if (regs->eflags & 0x200)
++		local_irq_enable();
++#endif
++
+ 	/* Ok, finally something we can handle */
+ 	send_sigtrap(tsk, regs, error_code);
+ 
+@@ -787,6 +855,7 @@
+ 	 */
+ clear_dr7:
+ 	set_debugreg(0, 7);
++	CHK_REMOTE_DEBUG(1,SIGTRAP,error_code,regs,)
+ 	return;
+ 
+ debug_vm86:
+@@ -1090,6 +1159,12 @@
+ {
+ 	_set_gate(idt_table+n,5,0,0,(gdt_entry<<3));
+ }
++#ifdef CONFIG_KGDB
++void set_intr_usr_gate(unsigned int n, void *addr)
++{
++	_set_gate(idt_table+n,14,3,addr,__KERNEL_CS);
++}
++#endif
+ 
+ 
+ void __init trap_init(void)
+@@ -1109,7 +1184,11 @@
+ 	set_trap_gate(0,&divide_error);
+ 	set_intr_gate(1,&debug);
+ 	set_intr_gate(2,&nmi);
++#ifndef CONFIG_KGDB
+ 	set_system_intr_gate(3, &int3); /* int3/4 can be called from all */
++#else
++	set_intr_usr_gate(3, &int3); /* int3/4 can be called from all */
++#endif
+ 	set_system_gate(4,&overflow);
+ 	set_trap_gate(5,&bounds);
+ 	set_trap_gate(6,&invalid_op);
+Index: linux-2.6.16.19/arch/i386/lib/Makefile
+===================================================================
+--- linux-2.6.16.19.orig/arch/i386/lib/Makefile	2006-06-23 17:05:14.000000000 -0700
++++ linux-2.6.16.19/arch/i386/lib/Makefile	2006-06-23 17:06:11.000000000 -0700
+@@ -7,3 +7,4 @@
+ 	bitops.o
+ 
+ lib-$(CONFIG_X86_USE_3DNOW) += mmx.o
++lib-$(CONFIG_KGDB) += kgdb_serial.o
+Index: linux-2.6.16.19/arch/i386/lib/kgdb_serial.c
+===================================================================
+--- linux-2.6.16.19.orig/arch/i386/lib/kgdb_serial.c	2006-04-21 14:09:39.654033432 -0700
++++ linux-2.6.16.19/arch/i386/lib/kgdb_serial.c	2006-06-23 17:06:11.000000000 -0700
+@@ -0,0 +1,485 @@
++/*
++ * Serial interface GDB stub
++ *
++ * Written (hacked together) by David Grothe (dave@gcom.com)
++ * Modified to allow invokation early in boot see also
++ * kgdb.h for instructions by George Anzinger(george@mvista.com)
++ *
++ */
++
++#include <linux/module.h>
++#include <linux/errno.h>
++#include <linux/signal.h>
++#include <linux/sched.h>
++#include <linux/timer.h>
++#include <linux/interrupt.h>
++#include <linux/tty.h>
++#include <linux/tty_flip.h>
++#include <linux/serial.h>
++#include <linux/serial_reg.h>
++#include <linux/config.h>
++#include <linux/major.h>
++#include <linux/string.h>
++#include <linux/fcntl.h>
++#include <linux/ptrace.h>
++#include <linux/ioport.h>
++#include <linux/mm.h>
++#include <linux/init.h>
++#include <linux/highmem.h>
++#include <asm/system.h>
++#include <asm/io.h>
++#include <asm/segment.h>
++#include <asm/bitops.h>
++#include <asm/system.h>
++#include <asm/kgdb_local.h>
++#ifdef CONFIG_KGDB_USER_CONSOLE
++extern void kgdb_console_finit(void);
++#endif
++#define PRNT_off
++#define TEST_EXISTANCE
++#ifdef PRNT
++#define dbprintk(s) printk s
++#else
++#define dbprintk(s)
++#endif
++#define TEST_INTERRUPT_off
++#ifdef TEST_INTERRUPT
++#define intprintk(s) printk s
++#else
++#define intprintk(s)
++#endif
++
++#define IRQ_T(info) ((info->flags & ASYNC_SHARE_IRQ) ? SA_SHIRQ : SA_INTERRUPT)
++
++#define	GDB_BUF_SIZE	512	/* power of 2, please */
++
++static char gdb_buf[GDB_BUF_SIZE];
++static int gdb_buf_in_inx;
++static atomic_t gdb_buf_in_cnt;
++static int gdb_buf_out_inx;
++
++struct async_struct *gdb_async_info;
++static int gdb_async_irq;
++
++#define outb_px(a,b) outb_p(b,a)
++
++static void program_uart(struct async_struct *info);
++static void write_char(struct async_struct *info, int chr);
++/*
++ * Get a byte from the hardware data buffer and return it
++ */
++static int
++read_data_bfr(struct async_struct *info)
++{
++	char it = inb_p(info->port + UART_LSR);
++
++	if (it & UART_LSR_DR)
++		return (inb_p(info->port + UART_RX));
++	/*
++	 * If we have a framing error assume somebody messed with
++	 * our uart.  Reprogram it and send '-' both ways...
++	 */
++	if (it & 0xc) {
++		program_uart(info);
++		write_char(info, '-');
++		return ('-');
++	}
++	return (-1);
++
++}				/* read_data_bfr */
++
++/*
++ * Get a char if available, return -1 if nothing available.
++ * Empty the receive buffer first, then look at the interface hardware.
++
++ * Locking here is a bit of a problem.	We MUST not lock out communication
++ * if we are trying to talk to gdb about a kgdb entry.	ON the other hand
++ * we can loose chars in the console pass thru if we don't lock.  It is also
++ * possible that we could hold the lock or be waiting for it when kgdb
++ * NEEDS to talk.  Since kgdb locks down the world, it does not need locks.
++ * We do, of course have possible issues with interrupting a uart operation,
++ * but we will just depend on the uart status to help keep that straight.
++
++ */
++static spinlock_t uart_interrupt_lock = SPIN_LOCK_UNLOCKED;
++#ifdef CONFIG_SMP
++extern spinlock_t kgdb_spinlock;
++#endif
++
++static int
++read_char(struct async_struct *info)
++{
++	int chr;
++	unsigned long flags;
++	local_irq_save(flags);
++#ifdef CONFIG_SMP
++	if (!spin_is_locked(&kgdb_spinlock)) {
++		spin_lock(&uart_interrupt_lock);
++	}
++#endif
++	if (atomic_read(&gdb_buf_in_cnt) != 0) {	/* intr routine has q'd chars */
++		chr = gdb_buf[gdb_buf_out_inx++];
++		gdb_buf_out_inx &= (GDB_BUF_SIZE - 1);
++		atomic_dec(&gdb_buf_in_cnt);
++	} else {
++		chr = read_data_bfr(info);
++	}
++#ifdef CONFIG_SMP
++	if (!spin_is_locked(&kgdb_spinlock)) {
++		spin_unlock(&uart_interrupt_lock);
++	}
++#endif
++	local_irq_restore(flags);
++	return (chr);
++}
++
++/*
++ * Wait until the interface can accept a char, then write it.
++ */
++static void
++write_char(struct async_struct *info, int chr)
++{
++	while (!(inb_p(info->port + UART_LSR) & UART_LSR_THRE)) ;
++
++	outb_p(chr, info->port + UART_TX);
++
++}				/* write_char */
++
++/*
++ * Mostly we don't need a spinlock, but since the console goes
++ * thru here with interrutps on, well, we need to catch those
++ * chars.
++ */
++/*
++ * This is the receiver interrupt routine for the GDB stub.
++ * It will receive a limited number of characters of input
++ * from the gdb  host machine and save them up in a buffer.
++ *
++ * When the gdb stub routine getDebugChar() is called it
++ * draws characters out of the buffer until it is empty and
++ * then reads directly from the serial port.
++ *
++ * We do not attempt to write chars from the interrupt routine
++ * since the stubs do all of that via putDebugChar() which
++ * writes one byte after waiting for the interface to become
++ * ready.
++ *
++ * The debug stubs like to run with interrupts disabled since,
++ * after all, they run as a consequence of a breakpoint in
++ * the kernel.
++ *
++ * Perhaps someone who knows more about the tty driver than I
++ * care to learn can make this work for any low level serial
++ * driver.
++ */
++static irqreturn_t
++gdb_interrupt(int irq, void *dev_id, struct pt_regs *regs)
++{
++	struct async_struct *info;
++	unsigned long flags;
++
++	info = gdb_async_info;
++	if (!info || !info->tty || irq != gdb_async_irq)
++		return IRQ_NONE;
++
++	local_irq_save(flags);
++	spin_lock(&uart_interrupt_lock);
++	do {
++		int chr = read_data_bfr(info);
++		intprintk(("Debug char on int: %x hex\n", chr));
++		if (chr < 0)
++			continue;
++
++		if (chr == 3) {	/* Ctrl-C means remote interrupt */
++			BREAKPOINT;
++			continue;
++		}
++
++		if (atomic_read(&gdb_buf_in_cnt) >= GDB_BUF_SIZE) {
++			/* buffer overflow tosses early char */
++			read_char(info);
++		}
++		gdb_buf[gdb_buf_in_inx++] = chr;
++		gdb_buf_in_inx &= (GDB_BUF_SIZE - 1);
++	} while (inb_p(info->port + UART_IIR) & UART_IIR_RDI);
++	spin_unlock(&uart_interrupt_lock);
++	local_irq_restore(flags);
++	return IRQ_HANDLED;
++}				/* gdb_interrupt */
++
++/*
++ * Just a NULL routine for testing.
++ */
++void
++gdb_null(void)
++{
++}				/* gdb_null */
++
++/* These structure are filled in with values defined in asm/kgdb_local.h
++ */
++static struct serial_state state = SB_STATE;
++static struct async_struct local_info = SB_INFO;
++static int ok_to_enable_ints = 0;
++static void kgdb_enable_ints_now(void);
++
++extern char *kgdb_version;
++/*
++ * Hook an IRQ for KGDB.
++ *
++ * This routine is called from putDebugChar, below.
++ */
++static int ints_disabled = 1;
++int
++gdb_hook_interrupt(struct async_struct *info, int verb)
++{
++	struct serial_state *state = info->state;
++	unsigned long flags;
++	int port;
++#ifdef TEST_EXISTANCE
++	int scratch, scratch2;
++#endif
++
++	/* The above fails if memory managment is not set up yet.
++	 * Rather than fail the set up, just keep track of the fact
++	 * and pick up the interrupt thing later.
++	 */
++	gdb_async_info = info;
++	port = gdb_async_info->port;
++	gdb_async_irq = state->irq;
++	if (verb) {
++		printk("kgdb %s : port =%x, IRQ=%d, divisor =%d\n",
++		       kgdb_version,
++		       port,
++		       gdb_async_irq, gdb_async_info->state->custom_divisor);
++	}
++	local_irq_save(flags);
++#ifdef TEST_EXISTANCE
++	/* Existance test */
++	/* Should not need all this, but just in case.... */
++
++	scratch = inb_p(port + UART_IER);
++	outb_px(port + UART_IER, 0);
++	outb_px(0xff, 0x080);
++	scratch2 = inb_p(port + UART_IER);
++	outb_px(port + UART_IER, scratch);
++	if (scratch2) {
++		printk
++		    ("gdb_hook_interrupt: Could not clear IER, not a UART!\n");
++		local_irq_restore(flags);
++		return 1;	/* We failed; there's nothing here */
++	}
++	scratch2 = inb_p(port + UART_LCR);
++	outb_px(port + UART_LCR, 0xBF);	/* set up for StarTech test */
++	outb_px(port + UART_EFR, 0);	/* EFR is the same as FCR */
++	outb_px(port + UART_LCR, 0);
++	outb_px(port + UART_FCR, UART_FCR_ENABLE_FIFO);
++	scratch = inb_p(port + UART_IIR) >> 6;
++	if (scratch == 1) {
++		printk("gdb_hook_interrupt: Undefined UART type!"
++		       "  Not a UART! \n");
++		local_irq_restore(flags);
++		return 1;
++	} else {
++		dbprintk(("gdb_hook_interrupt: UART type "
++			  "is %d where 0=16450, 2=16550 3=16550A\n", scratch));
++	}
++	scratch = inb_p(port + UART_MCR);
++	outb_px(port + UART_MCR, UART_MCR_LOOP | scratch);
++	outb_px(port + UART_MCR, UART_MCR_LOOP | 0x0A);
++	scratch2 = inb_p(port + UART_MSR) & 0xF0;
++	outb_px(port + UART_MCR, scratch);
++	if (scratch2 != 0x90) {
++		printk("gdb_hook_interrupt: "
++		       "Loop back test failed! Not a UART!\n");
++		local_irq_restore(flags);
++		return scratch2 + 1000;	/* force 0 to fail */
++	}
++#endif				/* test existance */
++	program_uart(info);
++	local_irq_restore(flags);
++
++	return (0);
++
++}				/* gdb_hook_interrupt */
++
++static void
++program_uart(struct async_struct *info)
++{
++	int port = info->port;
++
++	(void) inb_p(port + UART_RX);
++	outb_px(port + UART_IER, 0);
++
++	(void) inb_p(port + UART_RX);	/* serial driver comments say */
++	(void) inb_p(port + UART_IIR);	/* this clears the interrupt regs */
++	(void) inb_p(port + UART_MSR);
++	outb_px(port + UART_LCR, UART_LCR_WLEN8 | UART_LCR_DLAB);
++	outb_px(port + UART_DLL, info->state->custom_divisor & 0xff);	/* LS */
++	outb_px(port + UART_DLM, info->state->custom_divisor >> 8);	/* MS  */
++	outb_px(port + UART_MCR, info->MCR);
++
++	outb_px(port + UART_FCR, UART_FCR_ENABLE_FIFO | UART_FCR_TRIGGER_1 | UART_FCR_CLEAR_XMIT | UART_FCR_CLEAR_RCVR);	/* set fcr */
++	outb_px(port + UART_LCR, UART_LCR_WLEN8);	/* reset DLAB */
++	outb_px(port + UART_FCR, UART_FCR_ENABLE_FIFO | UART_FCR_TRIGGER_1);	/* set fcr */
++	if (!ints_disabled) {
++		intprintk(("KGDB: Sending %d to port %x offset %d\n",
++			   gdb_async_info->IER,
++			   (int) gdb_async_info->port, UART_IER));
++		outb_px(gdb_async_info->port + UART_IER, gdb_async_info->IER);
++	}
++	return;
++}
++
++/*
++ * getDebugChar
++ *
++ * This is a GDB stub routine.	It waits for a character from the
++ * serial interface and then returns it.  If there is no serial
++ * interface connection then it returns a bogus value which will
++ * almost certainly cause the system to hang.  In the
++ */
++int kgdb_in_isr = 0;
++int kgdb_in_lsr = 0;
++extern spinlock_t kgdb_spinlock;
++
++/* Caller takes needed protections */
++
++int
++getDebugChar(void)
++{
++	volatile int chr, dum, time, end_time;
++
++	dbprintk(("getDebugChar(port %x): ", gdb_async_info->port));
++
++	if (gdb_async_info == NULL) {
++		gdb_hook_interrupt(&local_info, 0);
++	}
++	/*
++	 * This trick says if we wait a very long time and get
++	 * no char, return the -1 and let the upper level deal
++	 * with it.
++	 */
++	rdtsc(dum, time);
++	end_time = time + 2;
++	while (((chr = read_char(gdb_async_info)) == -1) &&
++	       (end_time - time) > 0) {
++		rdtsc(dum, time);
++	};
++	/*
++	 * This covers our butts if some other code messes with
++	 * our uart, hay, it happens :o)
++	 */
++	if (chr == -1)
++		program_uart(gdb_async_info);
++
++	dbprintk(("%c\n", chr > ' ' && chr < 0x7F ? chr : ' '));
++	return (chr);
++
++}				/* getDebugChar */
++
++static int count = 3;
++static spinlock_t one_at_atime = SPIN_LOCK_UNLOCKED;
++
++static int __init
++kgdb_enable_ints(void)
++{
++	if (gdb_async_info == NULL) {
++		gdb_hook_interrupt(&local_info, 1);
++	}
++	ok_to_enable_ints = 1;
++	kgdb_enable_ints_now();
++#ifdef CONFIG_KGDB_USER_CONSOLE
++	kgdb_console_finit();
++#endif
++	return 0;
++}
++
++#ifdef CONFIG_SERIAL_8250
++void shutdown_for_kgdb(struct async_struct *gdb_async_info);
++#endif
++
++#ifdef CONFIG_DISCONTIGMEM
++static inline int kgdb_mem_init_done(void)
++{
++	return totalram_pages != 0;
++}
++#else
++static inline int kgdb_mem_init_done(void)
++{
++	return max_mapnr != 0;
++}
++#endif
++
++static void
++kgdb_enable_ints_now(void)
++{
++	if (!spin_trylock(&one_at_atime))
++		return;
++	if (!ints_disabled)
++		goto exit;
++	if (kgdb_mem_init_done() &&
++			ints_disabled) {	/* don't try till mem init */
++#ifdef CONFIG_SERIAL_8250
++		/*
++		 * The ifdef here allows the system to be configured
++		 * without the serial driver.
++		 * Don't make it a module, however, it will steal the port
++		 */
++		shutdown_for_kgdb(gdb_async_info);
++#endif
++		ints_disabled = request_irq(gdb_async_info->state->irq,
++					    gdb_interrupt,
++					    IRQ_T(gdb_async_info),
++					    "KGDB-stub", NULL);
++		intprintk(("KGDB: request_irq returned %d\n", ints_disabled));
++	}
++	if (!ints_disabled) {
++		intprintk(("KGDB: Sending %d to port %x offset %d\n",
++			   gdb_async_info->IER,
++			   (int) gdb_async_info->port, UART_IER));
++		outb_px(gdb_async_info->port + UART_IER, gdb_async_info->IER);
++	}
++      exit:
++	spin_unlock(&one_at_atime);
++}
++
++/*
++ * putDebugChar
++ *
++ * This is a GDB stub routine.	It waits until the interface is ready
++ * to transmit a char and then sends it.  If there is no serial
++ * interface connection then it simply returns to its caller, having
++ * pretended to send the char.	Caller takes needed protections.
++ */
++void
++putDebugChar(int chr)
++{
++	dbprintk(("putDebugChar(port %x): chr=%02x '%c', ints_on=%d\n",
++		  gdb_async_info->port,
++		  chr,
++		  chr > ' ' && chr < 0x7F ? chr : ' ', ints_disabled ? 0 : 1));
++
++	if (gdb_async_info == NULL) {
++		gdb_hook_interrupt(&local_info, 0);
++	}
++
++	write_char(gdb_async_info, chr);	/* this routine will wait */
++	count = (chr == '#') ? 0 : count + 1;
++	if ((count == 2)) {	/* try to enable after */
++		if (ints_disabled & ok_to_enable_ints)
++			kgdb_enable_ints_now();	/* try to enable after */
++
++		/* We do this a lot because, well we really want to get these
++		 * interrupts.	The serial driver will clear these bits when it
++		 * initializes the chip.  Every thing else it does is ok,
++		 * but this.
++		 */
++		if (!ints_disabled) {
++			outb_px(gdb_async_info->port + UART_IER,
++				gdb_async_info->IER);
++		}
++	}
++
++}				/* putDebugChar */
++
++module_init(kgdb_enable_ints);
+Index: linux-2.6.16.19/arch/i386/mm/fault.c
+===================================================================
+--- linux-2.6.16.19.orig/arch/i386/mm/fault.c	2006-06-23 17:05:14.000000000 -0700
++++ linux-2.6.16.19/arch/i386/mm/fault.c	2006-06-23 17:06:11.000000000 -0700
+@@ -437,6 +437,12 @@
+  * Oops. The kernel tried to access some bad page. We'll have to
+  * terminate things with extreme prejudice.
+  */
++#ifdef CONFIG_KGDB
++        if (!user_mode(regs)){
++                kgdb_handle_exception(14,SIGBUS, error_code, regs);
++                return;
++        }
++#endif
+ 
+ 	bust_spinlocks(1);
+ 
+Index: linux-2.6.16.19/arch/x86_64/boot/compressed/head.S
+===================================================================
+--- linux-2.6.16.19.orig/arch/x86_64/boot/compressed/head.S	2006-06-23 17:05:14.000000000 -0700
++++ linux-2.6.16.19/arch/x86_64/boot/compressed/head.S	2006-06-23 17:06:11.000000000 -0700
+@@ -24,6 +24,7 @@
+ .code32
+ .text
+ 
++#define IN_BOOTLOADER
+ #include <linux/linkage.h>
+ #include <asm/segment.h>
+ #include <asm/page.h>
+Index: linux-2.6.16.19/arch/x86_64/boot/compressed/misc.c
+===================================================================
+--- linux-2.6.16.19.orig/arch/x86_64/boot/compressed/misc.c	2006-06-23 17:05:14.000000000 -0700
++++ linux-2.6.16.19/arch/x86_64/boot/compressed/misc.c	2006-06-23 17:06:11.000000000 -0700
+@@ -9,6 +9,7 @@
+  * High loaded stuff by Hans Lermen & Werner Almesberger, Feb. 1996
+  */
+ 
++#define IN_BOOTLOADER
+ #include <linux/screen_info.h>
+ #include <asm/io.h>
+ #include <asm/page.h>
+Index: linux-2.6.16.19/drivers/char/keyboard.c
+===================================================================
+--- linux-2.6.16.19.orig/drivers/char/keyboard.c	2006-06-23 17:05:14.000000000 -0700
++++ linux-2.6.16.19/drivers/char/keyboard.c	2006-06-23 17:06:11.000000000 -0700
+@@ -1069,6 +1069,9 @@
+ 	}
+ 	if (sysrq_down && down && !rep) {
+ 		handle_sysrq(kbd_sysrq_xlate[keycode], regs, tty);
++#ifdef CONFIG_KGDB_SYSRQ
++                sysrq_down = 0;        /* in case we miss the "up" event */
++#endif
+ 		return;
+ 	}
+ #endif
+Index: linux-2.6.16.19/drivers/char/sysrq.c
+===================================================================
+--- linux-2.6.16.19.orig/drivers/char/sysrq.c	2006-06-23 17:05:14.000000000 -0700
++++ linux-2.6.16.19/drivers/char/sysrq.c	2006-06-23 17:06:11.000000000 -0700
+@@ -38,6 +38,25 @@
+ #include <linux/kexec.h>
+ 
+ #include <asm/ptrace.h>
++#ifdef CONFIG_KGDB_SYSRQ
++
++#define  GDB_OP &kgdb_op
++static void kgdb_sysrq(int key, struct pt_regs *pt_regs, struct tty_struct *tty)
++{
++	printk("kgdb sysrq\n");
++	breakpoint();
++}
++
++static struct sysrq_key_op kgdb_op = {
++	.handler	= kgdb_sysrq,
++	.help_msg	= "kGdb|Fgdb",
++	.action_msg	= "Debug breakpoint\n",
++};
++
++#else
++#define  GDB_OP NULL
++#endif
++
+ 
+ /* Whether we react on sysrq keys or just ignore them */
+ int sysrq_enabled = 1;
+@@ -316,7 +335,7 @@
+ #endif
+ /* e */	&sysrq_term_op,
+ /* f */	&sysrq_moom_op,
+-/* g */	NULL,
++/* g */	GDB_OP,
+ /* h */	NULL,
+ /* i */	&sysrq_kill_op,
+ /* j */	NULL,
+Index: linux-2.6.16.19/drivers/serial/8250.c
+===================================================================
+--- linux-2.6.16.19.orig/drivers/serial/8250.c	2006-06-23 17:05:14.000000000 -0700
++++ linux-2.6.16.19/drivers/serial/8250.c	2006-06-23 17:06:11.000000000 -0700
+@@ -1495,6 +1495,11 @@
+ 	spin_unlock_irqrestore(&up->port.lock, flags);
+ }
+ 
++#ifdef CONFIG_KGDB
++int kgdb_irq = -1;
++EXPORT_SYMBOL(kgdb_irq);
++#endif
++
+ static int serial8250_startup(struct uart_port *port)
+ {
+ 	struct uart_8250_port *up = (struct uart_8250_port *)port;
+@@ -1502,6 +1507,11 @@
+ 	unsigned char lsr, iir;
+ 	int retval;
+ 
++#ifdef CONFIG_KGDB
++	if (up->port.irq == kgdb_irq)
++		return -EBUSY;
++#endif
++
+ 	up->capabilities = uart_config[up->port.type].flags;
+ 	up->mcr = 0;
+ 
+@@ -2577,6 +2587,33 @@
+ }
+ EXPORT_SYMBOL(serial8250_unregister_port);
+ 
++#ifdef CONFIG_KGDB
++#include <linux/serialP.h>
++
++/*
++ * Find all the ports using the given irq and shut them down.
++ * Result should be that the irq will be released.
++ */
++void shutdown_for_kgdb(struct async_struct * info)
++{
++        int irq = info->state->irq;
++        struct uart_8250_port *up;
++	int ttyS;
++
++	kgdb_irq = irq;			/* save for later init */
++	for (ttyS = 0; ttyS < UART_NR; ttyS++){
++		up =  &serial8250_ports[ttyS];
++		if (up->port.irq == irq && (irq_lists + irq)->head) {
++#ifdef CONFIG_DEBUG_SPINLOCK   /* ugly business... */
++			if(up->port.lock.magic != SPINLOCK_MAGIC)
++				spin_lock_init(&up->port.lock);
++#endif
++			serial8250_shutdown(&up->port);
++		}
++        }
++}
++#endif	/* CONFIG_KGDB */
++
+ static int __init serial8250_init(void)
+ {
+ 	int ret, i;
+Index: linux-2.6.16.19/drivers/serial/serial_core.c
+===================================================================
+--- linux-2.6.16.19.orig/drivers/serial/serial_core.c	2006-06-23 17:05:14.000000000 -0700
++++ linux-2.6.16.19/drivers/serial/serial_core.c	2006-06-23 17:06:11.000000000 -0700
+@@ -2024,6 +2024,15 @@
+ {
+ 	unsigned int flags;
+ 
++#ifdef CONFIG_KGDB
++	{
++		extern int kgdb_irq;
++
++		if (port->irq == kgdb_irq)
++			return;
++	}
++#endif
++
+ 	/*
+ 	 * If there isn't a port here, don't do anything further.
+ 	 */
+Index: linux-2.6.16.19/include/asm-i386/bugs.h
+===================================================================
+--- linux-2.6.16.19.orig/include/asm-i386/bugs.h	2006-06-23 17:05:14.000000000 -0700
++++ linux-2.6.16.19/include/asm-i386/bugs.h	2006-06-23 17:06:11.000000000 -0700
+@@ -1,11 +1,11 @@
+ /*
+  *  include/asm-i386/bugs.h
+  *
+- *  Copyright (C) 1994  Linus Torvalds
++ *  Copyright (C) 1994	Linus Torvalds
+  *
+  *  Cyrix stuff, June 1998 by:
+  *	- Rafael R. Reilova (moved everything from head.S),
+- *        <rreilova@ececs.uc.edu>
++ *	  <rreilova@ececs.uc.edu>
+  *	- Channing Corn (tests & fixes),
+  *	- Andrew D. Balsa (code cleanup).
+  */
+@@ -22,7 +22,20 @@
+ #include <asm/processor.h>
+ #include <asm/i387.h>
+ #include <asm/msr.h>
+-
++#ifdef CONFIG_KGDB
++/*
++ * Provied the command line "gdb" initial break
++ */
++int __init kgdb_initial_break(char * str)
++{
++	if (*str == '\0'){
++		breakpoint();
++		return 1;
++	}
++	return 0;
++}
++__setup("gdb",kgdb_initial_break);
++#endif
+ static int __init no_halt(char *s)
+ {
+ 	boot_cpu_data.hlt_works_ok = 0;
+@@ -122,7 +135,7 @@
+ 	  : "ecx", "edi" );
+ 	/* If this fails, it means that any user program may lock the CPU hard. Too bad. */
+ 	if (res != 12345678) printk( "Buggy.\n" );
+-		        else printk( "OK.\n" );
++			else printk( "OK.\n" );
+ #endif
+ }
+ 
+Index: linux-2.6.16.19/include/asm-i386/kgdb.h
+===================================================================
+--- linux-2.6.16.19.orig/include/asm-i386/kgdb.h	2006-04-21 14:09:39.654033432 -0700
++++ linux-2.6.16.19/include/asm-i386/kgdb.h	2006-06-23 17:06:11.000000000 -0700
+@@ -0,0 +1,59 @@
++#ifndef __KGDB
++#define __KGDB
++
++/*
++ * This file should not include ANY others.  This makes it usable
++ * most anywhere without the fear of include order or inclusion.
++ * Make it so!
++ *
++ * This file may be included all the time.  It is only active if
++ * CONFIG_KGDB is defined, otherwise it stubs out all the macros
++ * and entry points.
++ */
++#if defined(CONFIG_KGDB) && !defined(__ASSEMBLY__)
++
++extern void breakpoint(void);
++#define INIT_KGDB_INTS kgdb_enable_ints()
++
++#ifndef BREAKPOINT
++#define BREAKPOINT   asm("   int $3")
++#endif
++/*
++ * GDB debug stub (or any debug stub) can point the 'linux_debug_hook'
++ * pointer to its routine and it will be entered as the first thing
++ * when a trap occurs.
++ *
++ * Return values are, at present, undefined.
++ *
++ * The debug hook routine does not necessarily return to its caller.
++ * It has the register image and thus may choose to resume execution
++ * anywhere it pleases.
++ */
++struct pt_regs;
++
++extern int kgdb_handle_exception(int trapno,
++				 int signo, int err_code, struct pt_regs *regs);
++extern int in_kgdb(struct pt_regs *regs);
++
++#ifdef CONFIG_KGDB_TS
++void kgdb_tstamp(int line, char *source, int data0, int data1);
++/*
++ * This is the time stamp function.  The macro adds the source info and
++ * does a cast on the data to allow most any 32-bit value.
++ */
++
++#define kgdb_ts(data0,data1) kgdb_tstamp(__LINE__,__FILE__,(int)data0,(int)data1)
++#else
++#define kgdb_ts(data0,data1)
++#endif
++#else				/* CONFIG_KGDB  && ! __ASSEMBLY__ ,stubs follow... */
++#ifndef BREAKPOINT
++#define BREAKPOINT
++#endif
++#define kgdb_ts(data0,data1)
++#define in_kgdb
++#define kgdb_handle_exception
++#define breakpoint
++#define INIT_KGDB_INTS
++#endif
++#endif				/* __KGDB */
+Index: linux-2.6.16.19/include/asm-i386/kgdb_local.h
+===================================================================
+--- linux-2.6.16.19.orig/include/asm-i386/kgdb_local.h	2006-04-21 14:09:39.654033432 -0700
++++ linux-2.6.16.19/include/asm-i386/kgdb_local.h	2006-06-23 17:06:11.000000000 -0700
+@@ -0,0 +1,102 @@
++#ifndef __KGDB_LOCAL
++#define ___KGDB_LOCAL
++#include <linux/config.h>
++#include <linux/types.h>
++#include <linux/serial.h>
++#include <linux/serialP.h>
++#include <linux/spinlock.h>
++#include <asm/processor.h>
++#include <asm/msr.h>
++#include <asm/kgdb.h>
++
++#define PORT 0x3f8
++#ifdef CONFIG_KGDB_PORT
++#undef PORT
++#define PORT CONFIG_KGDB_PORT
++#endif
++#define IRQ 4
++#ifdef CONFIG_KGDB_IRQ
++#undef IRQ
++#define IRQ CONFIG_KGDB_IRQ
++#endif
++#define SB_CLOCK 1843200
++#define SB_BASE (SB_CLOCK/16)
++#define SB_BAUD9600 SB_BASE/9600
++#define SB_BAUD192  SB_BASE/19200
++#define SB_BAUD384  SB_BASE/38400
++#define SB_BAUD576  SB_BASE/57600
++#define SB_BAUD1152 SB_BASE/115200
++#ifdef CONFIG_KGDB_9600BAUD
++#define SB_BAUD SB_BAUD9600
++#endif
++#ifdef CONFIG_KGDB_19200BAUD
++#define SB_BAUD SB_BAUD192
++#endif
++#ifdef CONFIG_KGDB_38400BAUD
++#define SB_BAUD SB_BAUD384
++#endif
++#ifdef CONFIG_KGDB_57600BAUD
++#define SB_BAUD SB_BAUD576
++#endif
++#ifdef CONFIG_KGDB_115200BAUD
++#define SB_BAUD SB_BAUD1152
++#endif
++#ifndef SB_BAUD
++#define SB_BAUD SB_BAUD1152	/* Start with this if not given */
++#endif
++
++#ifndef CONFIG_X86_TSC
++#undef rdtsc
++#define rdtsc(a,b) if (a++ > 10000){a = 0; b++;}
++#undef rdtscll
++#define rdtscll(s) s++
++#endif
++
++#ifdef _raw_read_unlock		/* must use a name that is "define"ed, not an inline */
++#undef spin_lock
++#undef spin_trylock
++#undef spin_unlock
++#define spin_lock	 _raw_spin_lock
++#define spin_trylock	 _raw_spin_trylock
++#define spin_unlock	 _raw_spin_unlock
++#else
++#endif
++#undef spin_unlock_wait
++#define spin_unlock_wait(x)  do { cpu_relax(); barrier();} \
++                                     while(spin_is_locked(x))
++
++#define SB_IER 1
++#define SB_MCR UART_MCR_OUT2 | UART_MCR_DTR | UART_MCR_RTS
++
++#define FLAGS 0
++#define SB_STATE { \
++     magic: SSTATE_MAGIC, \
++     baud_base: SB_BASE,  \
++     port:      PORT,     \
++     irq:       IRQ,      \
++     flags:     FLAGS,    \
++     custom_divisor:SB_BAUD}
++#define SB_INFO  { \
++      magic: SERIAL_MAGIC, \
++      port:  PORT,0,FLAGS, \
++      state: &state,       \
++      tty:   (struct tty_struct *)&state, \
++      IER:   SB_IER,       \
++      MCR:   SB_MCR}
++extern void putDebugChar(int);
++/* RTAI support needs us to really stop/start interrupts */
++
++#define kgdb_sti() __asm__ __volatile__("sti": : :"memory")
++#define kgdb_cli() __asm__ __volatile__("cli": : :"memory")
++#define kgdb_local_save_flags(x) __asm__ __volatile__(\
++                                   "pushfl ; popl %0":"=g" (x): /* no input */)
++#define kgdb_local_irq_restore(x) __asm__ __volatile__(\
++                                   "pushl %0 ; popfl": \
++                                     /* no output */ :"g" (x):"memory", "cc")
++#define kgdb_local_irq_save(x) kgdb_local_save_flags(x); kgdb_cli()
++
++#ifdef CONFIG_SERIAL
++extern void shutdown_for_kgdb(struct async_struct *info);
++#endif
++#define INIT_KDEBUG putDebugChar("+");
++#endif				/* __KGDB_LOCAL */
+Index: linux-2.6.16.19/include/linux/config.h
+===================================================================
+--- linux-2.6.16.19.orig/include/linux/config.h	2006-06-23 17:05:14.000000000 -0700
++++ linux-2.6.16.19/include/linux/config.h	2006-06-23 17:06:11.000000000 -0700
+@@ -4,5 +4,8 @@
+  * autoconf.h is now included via -imacros on the commandline
+  */
+ #include <linux/autoconf.h>
++#if defined(__i386__) && !defined(IN_BOOTLOADER)
++#include <asm/kgdb.h>
++#endif
+ 
+ #endif
+Index: linux-2.6.16.19/include/linux/dwarf2-lang.h
+===================================================================
+--- linux-2.6.16.19.orig/include/linux/dwarf2-lang.h	2006-04-21 14:09:39.654033432 -0700
++++ linux-2.6.16.19/include/linux/dwarf2-lang.h	2006-06-23 17:06:11.000000000 -0700
+@@ -0,0 +1,132 @@
++#ifndef DWARF2_LANG
++#define DWARF2_LANG
++#include <linux/dwarf2.h>
++
++/*
++ * This is free software; you can redistribute it and/or modify it under
++ * the terms of the GNU General Public License as published by the Free
++ * Software Foundation; either version 2, or (at your option) any later
++ * version.
++ */
++/*
++ * This file defines macros that allow generation of DWARF debug records
++ * for asm files.  This file is platform independent.  Register numbers
++ * (which are about the only thing that is platform dependent) are to be
++ * supplied by a platform defined file.
++ */
++#define DWARF_preamble()	.section	.debug_frame,"",@progbits
++/*
++ * This macro starts a debug frame section.  The debug_frame describes
++ * where to find the registers that the enclosing function saved on
++ * entry.
++ *
++ * ORD is use by the label generator and should be the same as what is
++ * passed to CFI_postamble.
++ *
++ * pc,	pc register gdb ordinal.
++ *
++ * code_align this is the factor used to define locations or regions
++ * where the given definitions apply.  If you use labels to define these
++ * this should be 1.
++ *
++ * data_align this is the factor used to define register offsets.  If
++ * you use struct offset, this should be the size of the register in
++ * bytes or the negative of that.  This is how it is used: you will
++ * define a register as the reference register, say the stack pointer,
++ * then you will say where a register is located relative to this
++ * reference registers value, say 40 for register 3 (the gdb register
++ * number).  The <40> will be multiplied by <data_align> to define the
++ * byte offset of the given register (3, in this example).  So if your
++ * <40> is the byte offset and the reference register points at the
++ * begining, you would want 1 for the data_offset.  If <40> was the 40th
++ * 4-byte element in that structure you would want 4.  And if your
++ * reference register points at the end of the structure you would want
++ * a negative data_align value(and you would have to do other math as
++ * well).
++ */
++
++#define CFI_preamble(ORD, pc, code_align, data_align)	\
++.section	.debug_frame,"",@progbits ;		\
++frame/**/_/**/ORD:						\
++	.long end/**/_/**/ORD-start/**/_/**/ORD;			\
++start/**/_/**/ORD:						\
++	.long	DW_CIE_ID;				\
++	.byte	DW_CIE_VERSION;			\
++	.byte 0	 ;				\
++	.uleb128 code_align;				\
++	.sleb128 data_align;				\
++	.byte pc;
++
++/*
++ * After the above macro and prior to the CFI_postamble, you need to
++ * define the initial state.  This starts with defining the reference
++ * register and, usually the pc.  Here are some helper macros:
++ */
++
++#define CFA_define_reference(reg, offset)	\
++	.byte DW_CFA_def_cfa;			\
++	.uleb128 reg;				\
++	.uleb128 (offset);
++
++#define CFA_define_offset(reg, offset)		\
++	.byte (DW_CFA_offset + reg);		\
++	.uleb128 (offset);
++
++#define CFI_postamble(ORD)			\
++	.align 4;				\
++end/**/_/**/ORD:
++/*
++ * So now your code pushs stuff on the stack, you need a new location
++ * and the rules for what to do.  This starts a running description of
++ * the call frame.  You need to describe what changes with respect to
++ * the call registers as the location of the pc moves through the code.
++ * The following builds an FDE (fram descriptor entry?).  Like the
++ * above, it has a preamble and a postamble.  It also is tied to the CFI
++ * above.
++ * The first entry after the preamble must be the location in the code
++ * that the call frame is being described for.
++ */
++#define FDE_preamble(ORD, fde_no, initial_address, length)	\
++	.long FDE_end/**/_/**/fde_no-FDE_start/**/_/**/fde_no;		\
++FDE_start/**/_/**/fde_no:						\
++	.long frame/**/_/**/ORD;					\
++	.long initial_address;					\
++	.long length;
++
++#define FDE_postamble(fde_no)			\
++	.align 4;				\
++FDE_end/**/_/**/fde_no:
++/*
++ * That done, you can now add registers, subtract registers, move the
++ * reference and even change the reference.  You can also define a new
++ * area of code the info applies to.  For discontinuous bits you should
++ * start a new FDE.  You may have as many as you like.
++ */
++
++/*
++ * To advance the address by <bytes>
++ */
++
++#define FDE_advance(bytes)			\
++	.byte DW_CFA_advance_loc4		\
++	.long bytes
++
++
++
++/*
++ * With the above you can define all the register locations.  But
++ * suppose the reference register moves... Takes the new offset NOT an
++ * increment.  This is how esp is tracked if it is not saved.
++ */
++
++#define CFA_define_cfa_offset(offset) \
++	.byte $DW_CFA_def_cfa_offset; \
++	.uleb128 (offset);
++/*
++ * Or suppose you want to use a different reference register...
++ */
++#define CFA_define_cfa_register(reg)		\
++	.byte DW_CFA_def_cfa_register;		\
++	.uleb128 reg;
++
++#endif
+Index: linux-2.6.16.19/include/linux/dwarf2.h
+===================================================================
+--- linux-2.6.16.19.orig/include/linux/dwarf2.h	2006-04-21 14:09:39.654033432 -0700
++++ linux-2.6.16.19/include/linux/dwarf2.h	2006-06-23 17:06:11.000000000 -0700
+@@ -0,0 +1,738 @@
++/* Declarations and definitions of codes relating to the DWARF2 symbolic
++   debugging information format.
++   Copyright (C) 1992, 1993, 1995, 1996, 1997, 1999, 2000, 2001, 2002
++   Free Software Foundation, Inc.
++
++   Written by Gary Funck (gary@intrepid.com) The Ada Joint Program
++   Office (AJPO), Florida State Unviversity and Silicon Graphics Inc.
++   provided support for this effort -- June 21, 1995.
++
++   Derived from the DWARF 1 implementation written by Ron Guilmette
++   (rfg@netcom.com), November 1990.
++
++   This file is part of GCC.
++
++   GCC is free software; you can redistribute it and/or modify it under
++   the terms of the GNU General Public License as published by the Free
++   Software Foundation; either version 2, or (at your option) any later
++   version.
++
++   GCC is distributed in the hope that it will be useful, but WITHOUT
++   ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
++   or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public
++   License for more details.
++
++   You should have received a copy of the GNU General Public License
++   along with GCC; see the file COPYING.  If not, write to the Free
++   Software Foundation, 59 Temple Place - Suite 330, Boston, MA
++   02111-1307, USA.  */
++
++/* This file is derived from the DWARF specification (a public document)
++   Revision 2.0.0 (July 27, 1993) developed by the UNIX International
++   Programming Languages Special Interest Group (UI/PLSIG) and distributed
++   by UNIX International.  Copies of this specification are available from
++   UNIX International, 20 Waterview Boulevard, Parsippany, NJ, 07054.
++
++   This file also now contains definitions from the DWARF 3 specification.  */
++
++/* This file is shared between GCC and GDB, and should not contain
++   prototypes.	*/
++
++#ifndef _ELF_DWARF2_H
++#define _ELF_DWARF2_H
++
++/* Structure found in the .debug_line section.	*/
++#ifndef __ASSEMBLY__
++typedef struct
++{
++  unsigned char li_length	   [4];
++  unsigned char li_version	   [2];
++  unsigned char li_prologue_length [4];
++  unsigned char li_min_insn_length [1];
++  unsigned char li_default_is_stmt [1];
++  unsigned char li_line_base	   [1];
++  unsigned char li_line_range	   [1];
++  unsigned char li_opcode_base	   [1];
++}
++DWARF2_External_LineInfo;
++
++typedef struct
++{
++  unsigned long  li_length;
++  unsigned short li_version;
++  unsigned int	 li_prologue_length;
++  unsigned char  li_min_insn_length;
++  unsigned char  li_default_is_stmt;
++  int		 li_line_base;
++  unsigned char  li_line_range;
++  unsigned char  li_opcode_base;
++}
++DWARF2_Internal_LineInfo;
++
++/* Structure found in .debug_pubnames section.	*/
++typedef struct
++{
++  unsigned char pn_length  [4];
++  unsigned char pn_version [2];
++  unsigned char pn_offset  [4];
++  unsigned char pn_size    [4];
++}
++DWARF2_External_PubNames;
++
++typedef struct
++{
++  unsigned long  pn_length;
++  unsigned short pn_version;
++  unsigned long  pn_offset;
++  unsigned long  pn_size;
++}
++DWARF2_Internal_PubNames;
++
++/* Structure found in .debug_info section.  */
++typedef struct
++{
++  unsigned char  cu_length	  [4];
++  unsigned char  cu_version	  [2];
++  unsigned char  cu_abbrev_offset [4];
++  unsigned char  cu_pointer_size  [1];
++}
++DWARF2_External_CompUnit;
++
++typedef struct
++{
++  unsigned long  cu_length;
++  unsigned short cu_version;
++  unsigned long  cu_abbrev_offset;
++  unsigned char  cu_pointer_size;
++}
++DWARF2_Internal_CompUnit;
++
++typedef struct
++{
++  unsigned char  ar_length	 [4];
++  unsigned char  ar_version	 [2];
++  unsigned char  ar_info_offset  [4];
++  unsigned char  ar_pointer_size [1];
++  unsigned char  ar_segment_size [1];
++}
++DWARF2_External_ARange;
++
++typedef struct
++{
++  unsigned long  ar_length;
++  unsigned short ar_version;
++  unsigned long  ar_info_offset;
++  unsigned char  ar_pointer_size;
++  unsigned char  ar_segment_size;
++}
++DWARF2_Internal_ARange;
++
++#define ENUM(name) enum name {
++#define IF_NOT_ASM(a) a
++#define COMMA ,
++#else
++#define ENUM(name)
++#define IF_NOT_ASM(a)
++#define COMMA
++
++#endif
++
++/* Tag names and codes.  */
++ENUM(dwarf_tag)
++
++    DW_TAG_padding = 0x00 COMMA
++    DW_TAG_array_type = 0x01 COMMA
++    DW_TAG_class_type = 0x02 COMMA
++    DW_TAG_entry_point = 0x03 COMMA
++    DW_TAG_enumeration_type = 0x04 COMMA
++    DW_TAG_formal_parameter = 0x05 COMMA
++    DW_TAG_imported_declaration = 0x08 COMMA
++    DW_TAG_label = 0x0a COMMA
++    DW_TAG_lexical_block = 0x0b COMMA
++    DW_TAG_member = 0x0d COMMA
++    DW_TAG_pointer_type = 0x0f COMMA
++    DW_TAG_reference_type = 0x10 COMMA
++    DW_TAG_compile_unit = 0x11 COMMA
++    DW_TAG_string_type = 0x12 COMMA
++    DW_TAG_structure_type = 0x13 COMMA
++    DW_TAG_subroutine_type = 0x15 COMMA
++    DW_TAG_typedef = 0x16 COMMA
++    DW_TAG_union_type = 0x17 COMMA
++    DW_TAG_unspecified_parameters = 0x18 COMMA
++    DW_TAG_variant = 0x19 COMMA
++    DW_TAG_common_block = 0x1a COMMA
++    DW_TAG_common_inclusion = 0x1b COMMA
++    DW_TAG_inheritance = 0x1c COMMA
++    DW_TAG_inlined_subroutine = 0x1d COMMA
++    DW_TAG_module = 0x1e COMMA
++    DW_TAG_ptr_to_member_type = 0x1f COMMA
++    DW_TAG_set_type = 0x20 COMMA
++    DW_TAG_subrange_type = 0x21 COMMA
++    DW_TAG_with_stmt = 0x22 COMMA
++    DW_TAG_access_declaration = 0x23 COMMA
++    DW_TAG_base_type = 0x24 COMMA
++    DW_TAG_catch_block = 0x25 COMMA
++    DW_TAG_const_type = 0x26 COMMA
++    DW_TAG_constant = 0x27 COMMA
++    DW_TAG_enumerator = 0x28 COMMA
++    DW_TAG_file_type = 0x29 COMMA
++    DW_TAG_friend = 0x2a COMMA
++    DW_TAG_namelist = 0x2b COMMA
++    DW_TAG_namelist_item = 0x2c COMMA
++    DW_TAG_packed_type = 0x2d COMMA
++    DW_TAG_subprogram = 0x2e COMMA
++    DW_TAG_template_type_param = 0x2f COMMA
++    DW_TAG_template_value_param = 0x30 COMMA
++    DW_TAG_thrown_type = 0x31 COMMA
++    DW_TAG_try_block = 0x32 COMMA
++    DW_TAG_variant_part = 0x33 COMMA
++    DW_TAG_variable = 0x34 COMMA
++    DW_TAG_volatile_type = 0x35 COMMA
++    /* DWARF 3.  */
++    DW_TAG_dwarf_procedure = 0x36 COMMA
++    DW_TAG_restrict_type = 0x37 COMMA
++    DW_TAG_interface_type = 0x38 COMMA
++    DW_TAG_namespace = 0x39 COMMA
++    DW_TAG_imported_module = 0x3a COMMA
++    DW_TAG_unspecified_type = 0x3b COMMA
++    DW_TAG_partial_unit = 0x3c COMMA
++    DW_TAG_imported_unit = 0x3d COMMA
++    /* SGI/MIPS Extensions.  */
++    DW_TAG_MIPS_loop = 0x4081 COMMA
++    /* GNU extensions.	*/
++    DW_TAG_format_label = 0x4101 COMMA	/* For FORTRAN 77 and Fortran 90.  */
++    DW_TAG_function_template = 0x4102 COMMA	/* For C++.  */
++    DW_TAG_class_template = 0x4103 COMMA	/* For C++.  */
++    DW_TAG_GNU_BINCL = 0x4104 COMMA
++    DW_TAG_GNU_EINCL = 0x4105 COMMA
++    /* Extensions for UPC.  See: http://upc.gwu.edu/~upc.  */
++    DW_TAG_upc_shared_type = 0x8765 COMMA
++    DW_TAG_upc_strict_type = 0x8766 COMMA
++    DW_TAG_upc_relaxed_type = 0x8767
++IF_NOT_ASM(};)
++
++#define DW_TAG_lo_user	0x4080
++#define DW_TAG_hi_user	0xffff
++
++/* Flag that tells whether entry has a child or not.  */
++#define DW_children_no	 0
++#define	DW_children_yes  1
++
++/* Form names and codes.  */
++ENUM(dwarf_form)
++
++    DW_FORM_addr = 0x01 COMMA
++    DW_FORM_block2 = 0x03 COMMA
++    DW_FORM_block4 = 0x04 COMMA
++    DW_FORM_data2 = 0x05 COMMA
++    DW_FORM_data4 = 0x06 COMMA
++    DW_FORM_data8 = 0x07 COMMA
++    DW_FORM_string = 0x08 COMMA
++    DW_FORM_block = 0x09 COMMA
++    DW_FORM_block1 = 0x0a COMMA
++    DW_FORM_data1 = 0x0b COMMA
++    DW_FORM_flag = 0x0c COMMA
++    DW_FORM_sdata = 0x0d COMMA
++    DW_FORM_strp = 0x0e COMMA
++    DW_FORM_udata = 0x0f COMMA
++    DW_FORM_ref_addr = 0x10 COMMA
++    DW_FORM_ref1 = 0x11 COMMA
++    DW_FORM_ref2 = 0x12 COMMA
++    DW_FORM_ref4 = 0x13 COMMA
++    DW_FORM_ref8 = 0x14 COMMA
++    DW_FORM_ref_udata = 0x15 COMMA
++    DW_FORM_indirect = 0x16
++IF_NOT_ASM(};)
++
++/* Attribute names and codes.  */
++
++ENUM(dwarf_attribute)
++
++    DW_AT_sibling = 0x01 COMMA
++    DW_AT_location = 0x02 COMMA
++    DW_AT_name = 0x03 COMMA
++    DW_AT_ordering = 0x09 COMMA
++    DW_AT_subscr_data = 0x0a COMMA
++    DW_AT_byte_size = 0x0b COMMA
++    DW_AT_bit_offset = 0x0c COMMA
++    DW_AT_bit_size = 0x0d COMMA
++    DW_AT_element_list = 0x0f COMMA
++    DW_AT_stmt_list = 0x10 COMMA
++    DW_AT_low_pc = 0x11 COMMA
++    DW_AT_high_pc = 0x12 COMMA
++    DW_AT_language = 0x13 COMMA
++    DW_AT_member = 0x14 COMMA
++    DW_AT_discr = 0x15 COMMA
++    DW_AT_discr_value = 0x16 COMMA
++    DW_AT_visibility = 0x17 COMMA
++    DW_AT_import = 0x18 COMMA
++    DW_AT_string_length = 0x19 COMMA
++    DW_AT_common_reference = 0x1a COMMA
++    DW_AT_comp_dir = 0x1b COMMA
++    DW_AT_const_value = 0x1c COMMA
++    DW_AT_containing_type = 0x1d COMMA
++    DW_AT_default_value = 0x1e COMMA
++    DW_AT_inline = 0x20 COMMA
++    DW_AT_is_optional = 0x21 COMMA
++    DW_AT_lower_bound = 0x22 COMMA
++    DW_AT_producer = 0x25 COMMA
++    DW_AT_prototyped = 0x27 COMMA
++    DW_AT_return_addr = 0x2a COMMA
++    DW_AT_start_scope = 0x2c COMMA
++    DW_AT_stride_size = 0x2e COMMA
++    DW_AT_upper_bound = 0x2f COMMA
++    DW_AT_abstract_origin = 0x31 COMMA
++    DW_AT_accessibility = 0x32 COMMA
++    DW_AT_address_class = 0x33 COMMA
++    DW_AT_artificial = 0x34 COMMA
++    DW_AT_base_types = 0x35 COMMA
++    DW_AT_calling_convention = 0x36 COMMA
++    DW_AT_count = 0x37 COMMA
++    DW_AT_data_member_location = 0x38 COMMA
++    DW_AT_decl_column = 0x39 COMMA
++    DW_AT_decl_file = 0x3a COMMA
++    DW_AT_decl_line = 0x3b COMMA
++    DW_AT_declaration = 0x3c COMMA
++    DW_AT_discr_list = 0x3d COMMA
++    DW_AT_encoding = 0x3e COMMA
++    DW_AT_external = 0x3f COMMA
++    DW_AT_frame_base = 0x40 COMMA
++    DW_AT_friend = 0x41 COMMA
++    DW_AT_identifier_case = 0x42 COMMA
++    DW_AT_macro_info = 0x43 COMMA
++    DW_AT_namelist_items = 0x44 COMMA
++    DW_AT_priority = 0x45 COMMA
++    DW_AT_segment = 0x46 COMMA
++    DW_AT_specification = 0x47 COMMA
++    DW_AT_static_link = 0x48 COMMA
++    DW_AT_type = 0x49 COMMA
++    DW_AT_use_location = 0x4a COMMA
++    DW_AT_variable_parameter = 0x4b COMMA
++    DW_AT_virtuality = 0x4c COMMA
++    DW_AT_vtable_elem_location = 0x4d COMMA
++    /* DWARF 3 values.	*/
++    DW_AT_allocated	= 0x4e COMMA
++    DW_AT_associated	= 0x4f COMMA
++    DW_AT_data_location = 0x50 COMMA
++    DW_AT_stride	= 0x51 COMMA
++    DW_AT_entry_pc	= 0x52 COMMA
++    DW_AT_use_UTF8	= 0x53 COMMA
++    DW_AT_extension	= 0x54 COMMA
++    DW_AT_ranges	= 0x55 COMMA
++    DW_AT_trampoline	= 0x56 COMMA
++    DW_AT_call_column	= 0x57 COMMA
++    DW_AT_call_file	= 0x58 COMMA
++    DW_AT_call_line	= 0x59 COMMA
++    /* SGI/MIPS extensions.  */
++    DW_AT_MIPS_fde = 0x2001 COMMA
++    DW_AT_MIPS_loop_begin = 0x2002 COMMA
++    DW_AT_MIPS_tail_loop_begin = 0x2003 COMMA
++    DW_AT_MIPS_epilog_begin = 0x2004 COMMA
++    DW_AT_MIPS_loop_unroll_factor = 0x2005 COMMA
++    DW_AT_MIPS_software_pipeline_depth = 0x2006 COMMA
++    DW_AT_MIPS_linkage_name = 0x2007 COMMA
++    DW_AT_MIPS_stride = 0x2008 COMMA
++    DW_AT_MIPS_abstract_name = 0x2009 COMMA
++    DW_AT_MIPS_clone_origin = 0x200a COMMA
++    DW_AT_MIPS_has_inlines = 0x200b COMMA
++    /* GNU extensions.	*/
++    DW_AT_sf_names   = 0x2101 COMMA
++    DW_AT_src_info   = 0x2102 COMMA
++    DW_AT_mac_info   = 0x2103 COMMA
++    DW_AT_src_coords = 0x2104 COMMA
++    DW_AT_body_begin = 0x2105 COMMA
++    DW_AT_body_end   = 0x2106 COMMA
++    DW_AT_GNU_vector = 0x2107 COMMA
++    /* VMS extensions.	*/
++    DW_AT_VMS_rtnbeg_pd_address = 0x2201 COMMA
++    /* UPC extension.  */
++    DW_AT_upc_threads_scaled = 0x3210
++IF_NOT_ASM(};)
++
++#define DW_AT_lo_user	0x2000	/* Implementation-defined range start.	*/
++#define DW_AT_hi_user	0x3ff0	/* Implementation-defined range end.  */
++
++/* Location atom names and codes.  */
++ENUM(dwarf_location_atom)
++
++    DW_OP_addr = 0x03 COMMA
++    DW_OP_deref = 0x06 COMMA
++    DW_OP_const1u = 0x08 COMMA
++    DW_OP_const1s = 0x09 COMMA
++    DW_OP_const2u = 0x0a COMMA
++    DW_OP_const2s = 0x0b COMMA
++    DW_OP_const4u = 0x0c COMMA
++    DW_OP_const4s = 0x0d COMMA
++    DW_OP_const8u = 0x0e COMMA
++    DW_OP_const8s = 0x0f COMMA
++    DW_OP_constu = 0x10 COMMA
++    DW_OP_consts = 0x11 COMMA
++    DW_OP_dup = 0x12 COMMA
++    DW_OP_drop = 0x13 COMMA
++    DW_OP_over = 0x14 COMMA
++    DW_OP_pick = 0x15 COMMA
++    DW_OP_swap = 0x16 COMMA
++    DW_OP_rot = 0x17 COMMA
++    DW_OP_xderef = 0x18 COMMA
++    DW_OP_abs = 0x19 COMMA
++    DW_OP_and = 0x1a COMMA
++    DW_OP_div = 0x1b COMMA
++    DW_OP_minus = 0x1c COMMA
++    DW_OP_mod = 0x1d COMMA
++    DW_OP_mul = 0x1e COMMA
++    DW_OP_neg = 0x1f COMMA
++    DW_OP_not = 0x20 COMMA
++    DW_OP_or = 0x21 COMMA
++    DW_OP_plus = 0x22 COMMA
++    DW_OP_plus_uconst = 0x23 COMMA
++    DW_OP_shl = 0x24 COMMA
++    DW_OP_shr = 0x25 COMMA
++    DW_OP_shra = 0x26 COMMA
++    DW_OP_xor = 0x27 COMMA
++    DW_OP_bra = 0x28 COMMA
++    DW_OP_eq = 0x29 COMMA
++    DW_OP_ge = 0x2a COMMA
++    DW_OP_gt = 0x2b COMMA
++    DW_OP_le = 0x2c COMMA
++    DW_OP_lt = 0x2d COMMA
++    DW_OP_ne = 0x2e COMMA
++    DW_OP_skip = 0x2f COMMA
++    DW_OP_lit0 = 0x30 COMMA
++    DW_OP_lit1 = 0x31 COMMA
++    DW_OP_lit2 = 0x32 COMMA
++    DW_OP_lit3 = 0x33 COMMA
++    DW_OP_lit4 = 0x34 COMMA
++    DW_OP_lit5 = 0x35 COMMA
++    DW_OP_lit6 = 0x36 COMMA
++    DW_OP_lit7 = 0x37 COMMA
++    DW_OP_lit8 = 0x38 COMMA
++    DW_OP_lit9 = 0x39 COMMA
++    DW_OP_lit10 = 0x3a COMMA
++    DW_OP_lit11 = 0x3b COMMA
++    DW_OP_lit12 = 0x3c COMMA
++    DW_OP_lit13 = 0x3d COMMA
++    DW_OP_lit14 = 0x3e COMMA
++    DW_OP_lit15 = 0x3f COMMA
++    DW_OP_lit16 = 0x40 COMMA
++    DW_OP_lit17 = 0x41 COMMA
++    DW_OP_lit18 = 0x42 COMMA
++    DW_OP_lit19 = 0x43 COMMA
++    DW_OP_lit20 = 0x44 COMMA
++    DW_OP_lit21 = 0x45 COMMA
++    DW_OP_lit22 = 0x46 COMMA
++    DW_OP_lit23 = 0x47 COMMA
++    DW_OP_lit24 = 0x48 COMMA
++    DW_OP_lit25 = 0x49 COMMA
++    DW_OP_lit26 = 0x4a COMMA
++    DW_OP_lit27 = 0x4b COMMA
++    DW_OP_lit28 = 0x4c COMMA
++    DW_OP_lit29 = 0x4d COMMA
++    DW_OP_lit30 = 0x4e COMMA
++    DW_OP_lit31 = 0x4f COMMA
++    DW_OP_reg0 = 0x50 COMMA
++    DW_OP_reg1 = 0x51 COMMA
++    DW_OP_reg2 = 0x52 COMMA
++    DW_OP_reg3 = 0x53 COMMA
++    DW_OP_reg4 = 0x54 COMMA
++    DW_OP_reg5 = 0x55 COMMA
++    DW_OP_reg6 = 0x56 COMMA
++    DW_OP_reg7 = 0x57 COMMA
++    DW_OP_reg8 = 0x58 COMMA
++    DW_OP_reg9 = 0x59 COMMA
++    DW_OP_reg10 = 0x5a COMMA
++    DW_OP_reg11 = 0x5b COMMA
++    DW_OP_reg12 = 0x5c COMMA
++    DW_OP_reg13 = 0x5d COMMA
++    DW_OP_reg14 = 0x5e COMMA
++    DW_OP_reg15 = 0x5f COMMA
++    DW_OP_reg16 = 0x60 COMMA
++    DW_OP_reg17 = 0x61 COMMA
++    DW_OP_reg18 = 0x62 COMMA
++    DW_OP_reg19 = 0x63 COMMA
++    DW_OP_reg20 = 0x64 COMMA
++    DW_OP_reg21 = 0x65 COMMA
++    DW_OP_reg22 = 0x66 COMMA
++    DW_OP_reg23 = 0x67 COMMA
++    DW_OP_reg24 = 0x68 COMMA
++    DW_OP_reg25 = 0x69 COMMA
++    DW_OP_reg26 = 0x6a COMMA
++    DW_OP_reg27 = 0x6b COMMA
++    DW_OP_reg28 = 0x6c COMMA
++    DW_OP_reg29 = 0x6d COMMA
++    DW_OP_reg30 = 0x6e COMMA
++    DW_OP_reg31 = 0x6f COMMA
++    DW_OP_breg0 = 0x70 COMMA
++    DW_OP_breg1 = 0x71 COMMA
++    DW_OP_breg2 = 0x72 COMMA
++    DW_OP_breg3 = 0x73 COMMA
++    DW_OP_breg4 = 0x74 COMMA
++    DW_OP_breg5 = 0x75 COMMA
++    DW_OP_breg6 = 0x76 COMMA
++    DW_OP_breg7 = 0x77 COMMA
++    DW_OP_breg8 = 0x78 COMMA
++    DW_OP_breg9 = 0x79 COMMA
++    DW_OP_breg10 = 0x7a COMMA
++    DW_OP_breg11 = 0x7b COMMA
++    DW_OP_breg12 = 0x7c COMMA
++    DW_OP_breg13 = 0x7d COMMA
++    DW_OP_breg14 = 0x7e COMMA
++    DW_OP_breg15 = 0x7f COMMA
++    DW_OP_breg16 = 0x80 COMMA
++    DW_OP_breg17 = 0x81 COMMA
++    DW_OP_breg18 = 0x82 COMMA
++    DW_OP_breg19 = 0x83 COMMA
++    DW_OP_breg20 = 0x84 COMMA
++    DW_OP_breg21 = 0x85 COMMA
++    DW_OP_breg22 = 0x86 COMMA
++    DW_OP_breg23 = 0x87 COMMA
++    DW_OP_breg24 = 0x88 COMMA
++    DW_OP_breg25 = 0x89 COMMA
++    DW_OP_breg26 = 0x8a COMMA
++    DW_OP_breg27 = 0x8b COMMA
++    DW_OP_breg28 = 0x8c COMMA
++    DW_OP_breg29 = 0x8d COMMA
++    DW_OP_breg30 = 0x8e COMMA
++    DW_OP_breg31 = 0x8f COMMA
++    DW_OP_regx = 0x90 COMMA
++    DW_OP_fbreg = 0x91 COMMA
++    DW_OP_bregx = 0x92 COMMA
++    DW_OP_piece = 0x93 COMMA
++    DW_OP_deref_size = 0x94 COMMA
++    DW_OP_xderef_size = 0x95 COMMA
++    DW_OP_nop = 0x96 COMMA
++    /* DWARF 3 extensions.  */
++    DW_OP_push_object_address = 0x97 COMMA
++    DW_OP_call2 = 0x98 COMMA
++    DW_OP_call4 = 0x99 COMMA
++    DW_OP_call_ref = 0x9a COMMA
++    /* GNU extensions.	*/
++    DW_OP_GNU_push_tls_address = 0xe0
++IF_NOT_ASM(};)
++
++#define DW_OP_lo_user	0xe0	/* Implementation-defined range start.	*/
++#define DW_OP_hi_user	0xff	/* Implementation-defined range end.  */
++
++/* Type encodings.  */
++ENUM(dwarf_type)
++
++    DW_ATE_void = 0x0 COMMA
++    DW_ATE_address = 0x1 COMMA
++    DW_ATE_boolean = 0x2 COMMA
++    DW_ATE_complex_float = 0x3 COMMA
++    DW_ATE_float = 0x4 COMMA
++    DW_ATE_signed = 0x5 COMMA
++    DW_ATE_signed_char = 0x6 COMMA
++    DW_ATE_unsigned = 0x7 COMMA
++    DW_ATE_unsigned_char = 0x8 COMMA
++    /* DWARF 3.  */
++    DW_ATE_imaginary_float = 0x9
++IF_NOT_ASM(};)
++
++#define	DW_ATE_lo_user 0x80
++#define	DW_ATE_hi_user 0xff
++
++/* Array ordering names and codes.  */
++ENUM(dwarf_array_dim_ordering)
++
++    DW_ORD_row_major = 0 COMMA
++    DW_ORD_col_major = 1
++IF_NOT_ASM(};)
++
++/* Access attribute.  */
++ENUM(dwarf_access_attribute)
++
++    DW_ACCESS_public = 1 COMMA
++    DW_ACCESS_protected = 2 COMMA
++    DW_ACCESS_private = 3
++IF_NOT_ASM(};)
++
++/* Visibility.	*/
++ENUM(dwarf_visibility_attribute)
++
++    DW_VIS_local = 1 COMMA
++    DW_VIS_exported = 2 COMMA
++    DW_VIS_qualified = 3
++IF_NOT_ASM(};)
++
++/* Virtuality.	*/
++ENUM(dwarf_virtuality_attribute)
++
++    DW_VIRTUALITY_none = 0 COMMA
++    DW_VIRTUALITY_virtual = 1 COMMA
++    DW_VIRTUALITY_pure_virtual = 2
++IF_NOT_ASM(};)
++
++/* Case sensitivity.  */
++ENUM(dwarf_id_case)
++
++    DW_ID_case_sensitive = 0 COMMA
++    DW_ID_up_case = 1 COMMA
++    DW_ID_down_case = 2 COMMA
++    DW_ID_case_insensitive = 3
++IF_NOT_ASM(};)
++
++/* Calling convention.	*/
++ENUM(dwarf_calling_convention)
++
++    DW_CC_normal = 0x1 COMMA
++    DW_CC_program = 0x2 COMMA
++    DW_CC_nocall = 0x3
++IF_NOT_ASM(};)
++
++#define DW_CC_lo_user 0x40
++#define DW_CC_hi_user 0xff
++
++/* Inline attribute.  */
++ENUM(dwarf_inline_attribute)
++
++    DW_INL_not_inlined = 0 COMMA
++    DW_INL_inlined = 1 COMMA
++    DW_INL_declared_not_inlined = 2 COMMA
++    DW_INL_declared_inlined = 3
++IF_NOT_ASM(};)
++
++/* Discriminant lists.	*/
++ENUM(dwarf_discrim_list)
++
++    DW_DSC_label = 0 COMMA
++    DW_DSC_range = 1
++IF_NOT_ASM(};)
++
++/* Line number opcodes.  */
++ENUM(dwarf_line_number_ops)
++
++    DW_LNS_extended_op = 0 COMMA
++    DW_LNS_copy = 1 COMMA
++    DW_LNS_advance_pc = 2 COMMA
++    DW_LNS_advance_line = 3 COMMA
++    DW_LNS_set_file = 4 COMMA
++    DW_LNS_set_column = 5 COMMA
++    DW_LNS_negate_stmt = 6 COMMA
++    DW_LNS_set_basic_block = 7 COMMA
++    DW_LNS_const_add_pc = 8 COMMA
++    DW_LNS_fixed_advance_pc = 9 COMMA
++    /* DWARF 3.  */
++    DW_LNS_set_prologue_end = 10 COMMA
++    DW_LNS_set_epilogue_begin = 11 COMMA
++    DW_LNS_set_isa = 12
++IF_NOT_ASM(};)
++
++/* Line number extended opcodes.  */
++ENUM(dwarf_line_number_x_ops)
++
++    DW_LNE_end_sequence = 1 COMMA
++    DW_LNE_set_address = 2 COMMA
++    DW_LNE_define_file = 3
++IF_NOT_ASM(};)
++
++/* Call frame information.  */
++ENUM(dwarf_call_frame_info)
++
++    DW_CFA_advance_loc = 0x40 COMMA
++    DW_CFA_offset = 0x80 COMMA
++    DW_CFA_restore = 0xc0 COMMA
++    DW_CFA_nop = 0x00 COMMA
++    DW_CFA_set_loc = 0x01 COMMA
++    DW_CFA_advance_loc1 = 0x02 COMMA
++    DW_CFA_advance_loc2 = 0x03 COMMA
++    DW_CFA_advance_loc4 = 0x04 COMMA
++    DW_CFA_offset_extended = 0x05 COMMA
++    DW_CFA_restore_extended = 0x06 COMMA
++    DW_CFA_undefined = 0x07 COMMA
++    DW_CFA_same_value = 0x08 COMMA
++    DW_CFA_register = 0x09 COMMA
++    DW_CFA_remember_state = 0x0a COMMA
++    DW_CFA_restore_state = 0x0b COMMA
++    DW_CFA_def_cfa = 0x0c COMMA
++    DW_CFA_def_cfa_register = 0x0d COMMA
++    DW_CFA_def_cfa_offset = 0x0e COMMA
++
++    /* DWARF 3.  */
++    DW_CFA_def_cfa_expression = 0x0f COMMA
++    DW_CFA_expression = 0x10 COMMA
++    DW_CFA_offset_extended_sf = 0x11 COMMA
++    DW_CFA_def_cfa_sf = 0x12 COMMA
++    DW_CFA_def_cfa_offset_sf = 0x13 COMMA
++
++    /* SGI/MIPS specific.  */
++    DW_CFA_MIPS_advance_loc8 = 0x1d COMMA
++
++    /* GNU extensions.	*/
++    DW_CFA_GNU_window_save = 0x2d COMMA
++    DW_CFA_GNU_args_size = 0x2e COMMA
++    DW_CFA_GNU_negative_offset_extended = 0x2f
++IF_NOT_ASM(};)
++
++#define DW_CIE_ID	  0xffffffff
++#define DW_CIE_VERSION	  1
++
++#define DW_CFA_extended   0
++#define DW_CFA_lo_user	  0x1c
++#define DW_CFA_hi_user	  0x3f
++
++#define DW_CHILDREN_no		     0x00
++#define DW_CHILDREN_yes		     0x01
++
++#define DW_ADDR_none		0
++
++/* Source language names and codes.  */
++ENUM(dwarf_source_language)
++
++    DW_LANG_C89 = 0x0001 COMMA
++    DW_LANG_C = 0x0002 COMMA
++    DW_LANG_Ada83 = 0x0003 COMMA
++    DW_LANG_C_plus_plus = 0x0004 COMMA
++    DW_LANG_Cobol74 = 0x0005 COMMA
++    DW_LANG_Cobol85 = 0x0006 COMMA
++    DW_LANG_Fortran77 = 0x0007 COMMA
++    DW_LANG_Fortran90 = 0x0008 COMMA
++    DW_LANG_Pascal83 = 0x0009 COMMA
++    DW_LANG_Modula2 = 0x000a COMMA
++    DW_LANG_Java = 0x000b COMMA
++    /* DWARF 3.  */
++    DW_LANG_C99 = 0x000c COMMA
++    DW_LANG_Ada95 = 0x000d COMMA
++    DW_LANG_Fortran95 = 0x000e COMMA
++    /* MIPS.  */
++    DW_LANG_Mips_Assembler = 0x8001 COMMA
++    /* UPC.  */
++    DW_LANG_Upc = 0x8765
++IF_NOT_ASM(};)
++
++#define DW_LANG_lo_user 0x8000	/* Implementation-defined range start.	*/
++#define DW_LANG_hi_user 0xffff	/* Implementation-defined range start.	*/
++
++/* Names and codes for macro information.  */
++ENUM(dwarf_macinfo_record_type)
++
++    DW_MACINFO_define = 1 COMMA
++    DW_MACINFO_undef = 2 COMMA
++    DW_MACINFO_start_file = 3 COMMA
++    DW_MACINFO_end_file = 4 COMMA
++    DW_MACINFO_vendor_ext = 255
++IF_NOT_ASM(};)
++
++/* @@@ For use with GNU frame unwind information.  */
++
++#define DW_EH_PE_absptr		0x00
++#define DW_EH_PE_omit		0xff
++
++#define DW_EH_PE_uleb128	0x01
++#define DW_EH_PE_udata2		0x02
++#define DW_EH_PE_udata4		0x03
++#define DW_EH_PE_udata8		0x04
++#define DW_EH_PE_sleb128	0x09
++#define DW_EH_PE_sdata2		0x0A
++#define DW_EH_PE_sdata4		0x0B
++#define DW_EH_PE_sdata8		0x0C
++#define DW_EH_PE_signed		0x08
++
++#define DW_EH_PE_pcrel		0x10
++#define DW_EH_PE_textrel	0x20
++#define DW_EH_PE_datarel	0x30
++#define DW_EH_PE_funcrel	0x40
++#define DW_EH_PE_aligned	0x50
++
++#define DW_EH_PE_indirect	0x80
++
++#endif /* _ELF_DWARF2_H */
+Index: linux-2.6.16.19/include/linux/serial_core.h
+===================================================================
+--- linux-2.6.16.19.orig/include/linux/serial_core.h	2006-06-23 17:05:14.000000000 -0700
++++ linux-2.6.16.19/include/linux/serial_core.h	2006-06-23 17:06:11.000000000 -0700
+@@ -215,7 +215,6 @@
+ 	unsigned char		x_char;			/* xon/xoff char */
+ 	unsigned char		regshift;		/* reg offset shift */
+ 	unsigned char		iotype;			/* io access style */
+-
+ #define UPIO_PORT		(0)
+ #define UPIO_HUB6		(1)
+ #define UPIO_MEM		(2)
+Index: linux-2.6.16.19/kernel/pid.c
+===================================================================
+--- linux-2.6.16.19.orig/kernel/pid.c	2006-06-23 17:05:14.000000000 -0700
++++ linux-2.6.16.19/kernel/pid.c	2006-06-23 17:06:11.000000000 -0700
+@@ -252,6 +252,9 @@
+  * machine.  From a minimum of 16 slots up to 4096 slots at one gigabyte or
+  * more.
+  */
++#ifdef CONFIG_KGDB
++int kgdb_pid_init_done; /* so we don't call prior to... */
++#endif
+ void __init pidhash_init(void)
+ {
+ 	int i, j, pidhash_size;
+@@ -273,6 +276,9 @@
+ 		for (j = 0; j < pidhash_size; j++)
+ 			INIT_HLIST_HEAD(&pid_hash[i][j]);
+ 	}
++#ifdef CONFIG_KGDB
++	kgdb_pid_init_done++;
++#endif
+ }
+ 
+ void __init pidmap_init(void)
+Index: linux-2.6.16.19/kernel/sched.c
+===================================================================
+--- linux-2.6.16.19.orig/kernel/sched.c	2006-06-23 17:05:14.000000000 -0700
++++ linux-2.6.16.19/kernel/sched.c	2006-06-23 17:06:11.000000000 -0700
+@@ -3507,6 +3507,13 @@
+ 		capable(CAP_SYS_NICE));
+ }
+ 
++#ifdef CONFIG_KGDB
++struct task_struct *kgdb_get_idle(int this_cpu)
++{
++        return cpu_rq(this_cpu)->idle;
++}
++#endif
++
+ #ifdef __ARCH_WANT_SYS_NICE
+ 
+ /*
Index: packages/kernel/patches/tiny_m57sli/tiny/kgdboe-netpoll.patch
===================================================================
--- packages/kernel/patches/tiny_m57sli/tiny/kgdboe-netpoll.patch	(revision 0)
+++ packages/kernel/patches/tiny_m57sli/tiny/kgdboe-netpoll.patch	(revision 0)
@@ -0,0 +1,745 @@
+
+From: Matt Mackall <mpm@selenic.com>
+
+Ok, here's kgdboe via netpoll, on top of 2.6.0, the above rolled kgdb
+bits and Jeff's netdrvr-1.
+
+DESC
+kgdboe: fix configuration of MAC address
+EDESC
+From: Matt Mackall <mpm@selenic.com>
+
+This seems to have gotten dropped.  Without it, kgdboe can get into a
+half-configured state.
+
+Signed-off-by: Andrew Morton <akpm@osdl.org>
+Index: linux-2.6.16.19/Documentation/i386/kgdb/kgdbeth.txt
+===================================================================
+--- linux-2.6.16.19.orig/Documentation/i386/kgdb/kgdbeth.txt	2006-04-21 14:09:39.654033432 -0700
++++ linux-2.6.16.19/Documentation/i386/kgdb/kgdbeth.txt	2006-06-23 17:06:12.000000000 -0700
+@@ -0,0 +1,92 @@
++KGDB over ethernet
++==================
++
++Authors
++-------
++
++Robert Walsh <rjwalsh@durables.org>  (2.6 port)
++wangdi <wangdi@clusterfs.com>        (2.6 port)
++Matt Mackall <mpm@selenic.com>       (netpoll api)
++San Mehat                            (original 2.4 code)
++
++
++Introduction
++------------
++
++KGDB supports debugging over ethernet (kgdboe) via polling of a given
++network interface. Most cards should be supported automatically.
++Debugging facilities are available as soon as the network driver and
++kgdboe have initialized. Unfortunately, this is too late in the boot
++process for debugging some issues, but works quite well for many
++others. This should not interfere with normal network usage and
++doesn't require a dedicated NIC.
++
++Terminology
++-----------
++
++This document uses the following terms:
++
++  TARGET: the machine being debugged.
++  HOST:   the machine running gdb.
++
++
++Usage
++-----
++
++You need to use the following command-line option on the TARGET kernel:
++
++  kgdboe=[tgt-port]@<tgt-ip>/[dev],[host-port]@<host-ip>/[host-macaddr]
++
++    where
++        tgt-port      source for UDP packets (defaults to 6443)
++        tgt-ip        source IP to use (interface address)
++        dev           network interface (eth0)
++        host-port     HOST UDP port (6442) (not really used)
++        host-ip       IP address for HOST machine
++        host-macaddr  ethernet MAC address for HOST (ff:ff:ff:ff:ff:ff)
++
++  examples:
++
++    kgdboe=7000@192.168.0.1/eth1,7001@192.168.0.2/00:05:3C:04:47:5D
++        this machine is 192.168.0.1 on eth1
++        remote machine is 192.168.0.2 with MAC address 00:05:3C:04:47:5D
++        listen for gdb packets on port 7000
++        send unsolicited gdb packets to port 7001
++
++    kgdboe=@192.168.0.1/,@192.168.0.2/
++        this machine is 192.168.0.1 on default interface eth0
++        remote machine is 192.168.0.2, use default broadcast MAC address
++        listen for gdb packets on default port 6443
++        send unsolicited gdb packets to port 6442
++
++Only packets originating from the configured HOST IP address will be
++accepted by the debugger.
++
++On the HOST side, run gdb as normal and use a remote UDP host as the
++target:
++
++   % gdb ./vmlinux
++   GNU gdb Red Hat Linux (5.3post-0.20021129.18rh)
++   Copyright 2003 Free Software Foundation, Inc.
++   GDB is free software, covered by the GNU General Public License, and you are
++   welcome to change it and/or distribute copies of it under certain conditions.
++   Type "show copying" to see the conditions.
++   There is absolutely no warranty for GDB.  Type "show warranty" for details.
++   This GDB was configured as "i386-redhat-linux-gnu"...
++   (gdb) target remote udp:HOSTNAME:6443
++
++You can now continue as if you were debugging over a serial line.
++
++Limitations
++-----------
++
++The current release of this code is exclusive of using kgdb on a
++serial interface, so you must boot without the kgdboe option to use
++serial debugging. Trying to debug the network driver while using it
++will prove interesting.
++
++Bug reports
++-----------
++
++Send bug reports to Robert Walsh <rjwalsh@durables.org> and Matt
++Mackall <mpm@selenic.com>.
+Index: linux-2.6.16.19/arch/i386/kernel/irq.c
+===================================================================
+--- linux-2.6.16.19.orig/arch/i386/kernel/irq.c	2006-06-23 17:05:14.000000000 -0700
++++ linux-2.6.16.19/arch/i386/kernel/irq.c	2006-06-23 17:06:12.000000000 -0700
+@@ -109,6 +109,8 @@
+ 
+ 	irq_exit();
+ 
++	kgdb_process_breakpoint();
++
+ 	return 1;
+ }
+ 
+Index: linux-2.6.16.19/arch/i386/kernel/kgdb_stub.c
+===================================================================
+--- linux-2.6.16.19.orig/arch/i386/kernel/kgdb_stub.c	2006-06-23 17:06:11.000000000 -0700
++++ linux-2.6.16.19/arch/i386/kernel/kgdb_stub.c	2006-06-23 17:06:12.000000000 -0700
+@@ -30,6 +30,8 @@
+  *
+  *  Written by:	     Glenn Engel $
+  *  Updated by:	     David Grothe <dave@gcom.com>
++ *  Updated by:	     Robert Walsh <rjwalsh@durables.org>
++ *  Updated by:	     wangdi <wangdi@clusterfs.com>
+  *  ModuleState:     Experimental $
+  *
+  *  NOTES:	     See Below $
+@@ -49,6 +51,10 @@
+  *	support for ia-32(x86) hardware debugging.
+  *	Amit S. Kale ( akale@veritas.com )
+  *
++ *  Modified to support debugging over ethernet by Robert Walsh
++ *  <rjwalsh@durables.org> and wangdi <wangdi@clusterfs.com>, based on
++ *  code by San Mehat.
++ *
+  *
+  *  To enable debugger support, two things need to happen.  One, a
+  *  call to set_debug_traps() is necessary in order to allow any breakpoints
+@@ -112,6 +118,8 @@
+ #include <asm/processor.h>
+ #include <linux/irq.h>
+ #include <asm/desc.h>
++#include <linux/inet.h>
++#include <linux/netpoll.h>
+ 
+ /************************************************************************
+  *
+@@ -122,8 +130,12 @@
+ /* Thread reference */
+ typedef unsigned char threadref[8];
+ 
+-extern void putDebugChar(int);	/* write a single character	 */
+-extern int getDebugChar(void);	/* read and return a single char */
++extern int tty_putDebugChar(int);     /* write a single character      */
++extern int tty_getDebugChar(void);    /* read and return a single char */
++extern void tty_flushDebugChar(void); /* flush pending characters      */
++extern int eth_putDebugChar(int);     /* write a single character      */
++extern int eth_getDebugChar(void);    /* read and return a single char */
++extern void eth_flushDebugChar(void); /* flush pending characters      */
+ 
+ /************************************************************************/
+ /* BUFMAX defines the maximum number of characters in inbound/outbound buffers*/
+@@ -264,6 +276,41 @@
+ }
+ 
+ /*
++ * I/O dispatch functions...
++ * Based upon kgdboe, either call the ethernet
++ * handler or the serial one..
++ */
++void
++putDebugChar(int c)
++{
++	if (!kgdboe) {
++		tty_putDebugChar(c);
++	} else {
++		eth_putDebugChar(c);
++	}
++}
++
++int
++getDebugChar(void)
++{
++	if (!kgdboe) {
++		return tty_getDebugChar();
++	} else {
++		return eth_getDebugChar();
++	}
++}
++
++void
++flushDebugChar(void)
++{
++	if (!kgdboe) {
++		tty_flushDebugChar();
++	} else {
++		eth_flushDebugChar();
++	}
++}
++
++/*
+  * Gdb calls functions by pushing agruments, including a return address
+  * on the stack and the adjusting EIP to point to the function.	 The
+  * whole assumption in GDB is that we are on a different stack than the
+@@ -429,6 +476,7 @@
+ 
+ 	if (remote_debug)
+ 		printk("R:%s\n", buffer);
++	flushDebugChar();
+ }
+ 
+ /* send the packet in buffer.  */
+@@ -441,25 +489,67 @@
+ 	char ch;
+ 
+ 	/*  $<packet info>#<checksum>. */
+-	do {
+-		if (remote_debug)
+-			printk("T:%s\n", buffer);
+-		putDebugChar('$');
+-		checksum = 0;
+-		count = 0;
+ 
+-		while ((ch = buffer[count])) {
+-			putDebugChar(ch);
+-			checksum += ch;
+-			count += 1;
+-		}
++	if (!kgdboe) {
++		do {
++			if (remote_debug)
++				printk("T:%s\n", buffer);
++			putDebugChar('$');
++			checksum = 0;
++			count = 0;
++
++			while ((ch = buffer[count])) {
++				putDebugChar(ch);
++				checksum += ch;
++				count += 1;
++			}
++
++			putDebugChar('#');
++			putDebugChar(hexchars[checksum >> 4]);
++			putDebugChar(hexchars[checksum % 16]);
++			flushDebugChar();
++
++		} while ((getDebugChar() & 0x7f) != '+');
++	} else {
++		/*
++		 * For udp, we can not transfer too much bytes once.
++		 * We only transfer MAX_SEND_COUNT size bytes each time
++		 */
+ 
+-		putDebugChar('#');
+-		putDebugChar(hexchars[checksum >> 4]);
+-		putDebugChar(hexchars[checksum % 16]);
++#define MAX_SEND_COUNT 30
+ 
+-	} while ((getDebugChar() & 0x7f) != '+');
++		int send_count = 0, i = 0;
++		char send_buf[MAX_SEND_COUNT];
+ 
++		do {
++			if (remote_debug)
++				printk("T:%s\n", buffer);
++			putDebugChar('$');
++			checksum = 0;
++			count = 0;
++			send_count = 0;
++			while ((ch = buffer[count])) {
++				if (send_count >= MAX_SEND_COUNT) {
++					for(i = 0; i < MAX_SEND_COUNT; i++) {
++						putDebugChar(send_buf[i]);
++					}
++					flushDebugChar();
++					send_count = 0;
++				} else {
++					send_buf[send_count] = ch;
++					checksum += ch;
++					count ++;
++					send_count++;
++				}
++			}
++			for(i = 0; i < send_count; i++)
++				putDebugChar(send_buf[i]);
++			putDebugChar('#');
++			putDebugChar(hexchars[checksum >> 4]);
++			putDebugChar(hexchars[checksum % 16]);
++			flushDebugChar();
++		} while ((getDebugChar() & 0x7f) != '+');
++	}
+ }
+ 
+ static char remcomInBuffer[BUFMAX];
+@@ -1045,9 +1135,9 @@
+ 	 */
+ 	in_kgdb_entry_log[cpu]++;
+ 	in_kgdb_here_log[cpu] = regs;
+-	if (cpu == spinlock_cpu || waiting_cpus[cpu].task) {
++	if (cpu == spinlock_cpu || waiting_cpus[cpu].task)
+ 		goto exit_in_kgdb;
+-	}
++
+ 	/*
+ 	 * For protection of the initilization of the spin locks by kgdb
+ 	 * it locks the kgdb spinlock before it gets the wait locks set
+@@ -1056,16 +1146,18 @@
+ 	 * sequence where the wait lock is removed prior to the kgdb lock
+ 	 * so if kgdb gets unlocked, we just exit.
+ 	 */
++
+ 	while (spin_is_locked(&kgdb_spinlock) &&
+ 	       !spin_is_locked(waitlocks + cpu)) ;
+-	if (!spin_is_locked(&kgdb_spinlock)) {
++	if (!spin_is_locked(&kgdb_spinlock))
+ 		goto exit_in_kgdb;
+-	}
++
+ 	waiting_cpus[cpu].task = current;
+ 	waiting_cpus[cpu].pid = (current->pid) ? : (PID_MAX + cpu);
+ 	waiting_cpus[cpu].regs = regs;
+ 
+ 	spin_unlock_wait(waitlocks + cpu);
++
+ 	/*
+ 	 * log departure of this cpu
+ 	 */
+@@ -1177,6 +1269,12 @@
+ 		print_regs(&regs);
+ 		return (0);
+ 	}
++	/*
++	 * If we're using eth mode, set the 'mode' in the netdevice.
++	 */
++
++	if (kgdboe)
++		netpoll_set_trap(1);
+ 
+ 	kgdb_local_irq_save(flags);
+ 
+@@ -1231,10 +1329,12 @@
+ 		if (num_online_cpus() > 1) {
+ 			int me_in_kgdb = in_kgdb_entry_log[smp_processor_id()];
+ 			smp_send_nmi_allbutself();
++
+ 			while (i < num_online_cpus() && time != end_time) {
+ 				int j;
+ 				for (j = 0; j < MAX_NO_CPUS; j++) {
+ 					if (waiting_cpus[j].task &&
++					    waiting_cpus[j].task != NOCPU &&
+ 					    !cpu_logged_in[j]) {
+ 						i++;
+ 						cpu_logged_in[j] = 1;
+@@ -1623,6 +1723,10 @@
+ 					}
+ 				}
+ 			}
++
++			if (kgdboe)
++				netpoll_set_trap(0);
++
+ 			correct_hw_break();
+ 			asm volatile ("movl %0, %%db6\n"::"r" (0));
+ 			goto exit_kgdb;
+@@ -2326,3 +2430,23 @@
+ typedef int gdb_debug_hook(int exceptionVector,
+ 			   int signo, int err_code, struct pt_regs *linux_regs);
+ gdb_debug_hook *linux_debug_hook = &kgdb_handle_exception;	/* histerical reasons... */
++
++static int kgdb_need_breakpoint[NR_CPUS];
++
++void kgdb_schedule_breakpoint(void)
++{
++	kgdb_need_breakpoint[smp_processor_id()] = 1;
++}
++
++void kgdb_process_breakpoint(void)
++{
++	/*
++	 * Handle a breakpoint queued from inside network driver code
++         * to avoid reentrancy issues
++	 */
++	if (kgdb_need_breakpoint[smp_processor_id()]) {
++		kgdb_need_breakpoint[smp_processor_id()] = 0;
++		BREAKPOINT;
++	}
++}
++
+Index: linux-2.6.16.19/arch/i386/lib/kgdb_serial.c
+===================================================================
+--- linux-2.6.16.19.orig/arch/i386/lib/kgdb_serial.c	2006-06-23 17:06:11.000000000 -0700
++++ linux-2.6.16.19/arch/i386/lib/kgdb_serial.c	2006-06-23 17:06:12.000000000 -0700
+@@ -4,6 +4,9 @@
+  * Written (hacked together) by David Grothe (dave@gcom.com)
+  * Modified to allow invokation early in boot see also
+  * kgdb.h for instructions by George Anzinger(george@mvista.com)
++ * Modified to handle debugging over ethernet by Robert Walsh
++ * <rjwalsh@durables.org> and wangdi <wangdi@clusterfs.com>, based on
++ * code by San Mehat.
+  *
+  */
+ 
+@@ -155,12 +158,12 @@
+  * It will receive a limited number of characters of input
+  * from the gdb  host machine and save them up in a buffer.
+  *
+- * When the gdb stub routine getDebugChar() is called it
++ * When the gdb stub routine tty_getDebugChar() is called it
+  * draws characters out of the buffer until it is empty and
+  * then reads directly from the serial port.
+  *
+  * We do not attempt to write chars from the interrupt routine
+- * since the stubs do all of that via putDebugChar() which
++ * since the stubs do all of that via tty_putDebugChar() which
+  * writes one byte after waiting for the interface to become
+  * ready.
+  *
+@@ -226,7 +229,7 @@
+ /*
+  * Hook an IRQ for KGDB.
+  *
+- * This routine is called from putDebugChar, below.
++ * This routine is called from tty_putDebugChar, below.
+  */
+ static int ints_disabled = 1;
+ int
+@@ -331,7 +334,7 @@
+ }
+ 
+ /*
+- * getDebugChar
++ * tty_getDebugChar
+  *
+  * This is a GDB stub routine.	It waits for a character from the
+  * serial interface and then returns it.  If there is no serial
+@@ -345,11 +348,11 @@
+ /* Caller takes needed protections */
+ 
+ int
+-getDebugChar(void)
++tty_getDebugChar(void)
+ {
+ 	volatile int chr, dum, time, end_time;
+ 
+-	dbprintk(("getDebugChar(port %x): ", gdb_async_info->port));
++	dbprintk(("tty_getDebugChar(port %x): ", gdb_async_info->port));
+ 
+ 	if (gdb_async_info == NULL) {
+ 		gdb_hook_interrupt(&local_info, 0);
+@@ -375,7 +378,7 @@
+ 	dbprintk(("%c\n", chr > ' ' && chr < 0x7F ? chr : ' '));
+ 	return (chr);
+ 
+-}				/* getDebugChar */
++}				/* tty_getDebugChar */
+ 
+ static int count = 3;
+ static spinlock_t one_at_atime = SPIN_LOCK_UNLOCKED;
+@@ -383,6 +386,9 @@
+ static int __init
+ kgdb_enable_ints(void)
+ {
++	if (kgdboe) {
++		return 0;
++	}
+ 	if (gdb_async_info == NULL) {
+ 		gdb_hook_interrupt(&local_info, 1);
+ 	}
+@@ -444,7 +450,7 @@
+ }
+ 
+ /*
+- * putDebugChar
++ * tty_putDebugChar
+  *
+  * This is a GDB stub routine.	It waits until the interface is ready
+  * to transmit a char and then sends it.  If there is no serial
+@@ -452,9 +458,9 @@
+  * pretended to send the char.	Caller takes needed protections.
+  */
+ void
+-putDebugChar(int chr)
++tty_putDebugChar(int chr)
+ {
+-	dbprintk(("putDebugChar(port %x): chr=%02x '%c', ints_on=%d\n",
++	dbprintk(("tty_putDebugChar(port %x): chr=%02x '%c', ints_on=%d\n",
+ 		  gdb_async_info->port,
+ 		  chr,
+ 		  chr > ' ' && chr < 0x7F ? chr : ' ', ints_disabled ? 0 : 1));
+@@ -480,6 +486,14 @@
+ 		}
+ 	}
+ 
+-}				/* putDebugChar */
++}				/* tty_putDebugChar */
++
++/*
++ * This does nothing for the serial port, since it doesn't buffer.
++ */
++
++void tty_flushDebugChar(void)
++{
++}
+ 
+ module_init(kgdb_enable_ints);
+Index: linux-2.6.16.19/drivers/net/Kconfig
+===================================================================
+--- linux-2.6.16.19.orig/drivers/net/Kconfig	2006-06-23 17:05:14.000000000 -0700
++++ linux-2.6.16.19/drivers/net/Kconfig	2006-06-23 17:06:12.000000000 -0700
+@@ -2715,18 +2715,17 @@
+ 	If you want to log kernel messages over the network, enable this.
+ 	See <file:Documentation/networking/netconsole.txt> for details.
+ 
++config KGDBOE
++       def_bool X86 && KGDB
++
+ config NETPOLL
+-	def_bool NETCONSOLE
++	def_bool NETCONSOLE || KGDBOE
+ 
+ config NETPOLL_RX
+-	bool "Netpoll support for trapping incoming packets"
+-	default n
+-	depends on NETPOLL
++	def_bool KGDBOE
+ 
+ config NETPOLL_TRAP
+-	bool "Netpoll traffic trapping"
+-	default n
+-	depends on NETPOLL
++	def_bool KGDBOE
+ 
+ config NET_POLL_CONTROLLER
+ 	def_bool NETPOLL
+Index: linux-2.6.16.19/drivers/net/Makefile
+===================================================================
+--- linux-2.6.16.19.orig/drivers/net/Makefile	2006-06-23 17:05:14.000000000 -0700
++++ linux-2.6.16.19/drivers/net/Makefile	2006-06-23 17:06:12.000000000 -0700
+@@ -209,6 +209,7 @@
+ obj-$(CONFIG_ETRAX_ETHERNET) += cris/
+ obj-$(CONFIG_ENP2611_MSF_NET) += ixp2000/
+ 
++obj-$(CONFIG_KGDB) += kgdb_eth.o
+ obj-$(CONFIG_NETCONSOLE) += netconsole.o
+ 
+ obj-$(CONFIG_FS_ENET) += fs_enet/
+Index: linux-2.6.16.19/drivers/net/kgdb_eth.c
+===================================================================
+--- linux-2.6.16.19.orig/drivers/net/kgdb_eth.c	2006-04-21 14:09:39.654033432 -0700
++++ linux-2.6.16.19/drivers/net/kgdb_eth.c	2006-06-23 17:06:12.000000000 -0700
+@@ -0,0 +1,132 @@
++/*
++ * Network interface GDB stub
++ *
++ * Written by San Mehat (nettwerk@biodome.org)
++ * Based upon 'gdbserial' by David Grothe (dave@gcom.com)
++ * and Scott Foehner (sfoehner@engr.sgi.com)
++ *
++ * Twiddled for 2.6 by Robert Walsh <rjwalsh@durables.org>
++ * and wangdi <wangdi@clusterfs.com>.
++ *
++ * Refactored for netpoll API by Matt Mackall <mpm@selenic.com>
++ *
++ */
++
++#include <linux/sched.h>
++#include <linux/interrupt.h>
++#include <linux/config.h>
++#include <linux/string.h>
++#include <linux/netpoll.h>
++
++#include <asm/system.h>
++#include <asm/kgdb.h>
++#include <asm/io.h>
++#include <asm/bitops.h>
++#include <asm/system.h>
++#include <asm/irq.h>
++#include <asm/atomic.h>
++
++#define IN_BUF_SIZE 512 /* power of 2, please */
++#define OUT_BUF_SIZE 256
++
++static char in_buf[IN_BUF_SIZE], out_buf[OUT_BUF_SIZE];
++static int in_head, in_tail, out_count;
++static atomic_t in_count;
++int kgdboe = 0; /* Default to tty mode */
++
++extern void set_debug_traps(void);
++extern void breakpoint(void);
++static void rx_hook(struct netpoll *np, int port, char *msg, int len);
++
++static struct netpoll np = {
++	.name = "kgdboe",
++	.dev_name = "eth0",
++	.rx_hook = rx_hook,
++	.local_port = 6443,
++	.remote_port = 6442,
++	.remote_mac = {0xff, 0xff, 0xff, 0xff, 0xff, 0xff},
++};
++static int configured;
++
++int eth_getDebugChar(void)
++{
++	int chr;
++
++	while (atomic_read(&in_count) == 0)
++		netpoll_poll(&np);
++
++	chr = in_buf[in_tail++];
++	in_tail &= (IN_BUF_SIZE - 1);
++	atomic_dec(&in_count);
++	return chr;
++}
++
++void eth_flushDebugChar(void)
++{
++	if(out_count && np.dev) {
++		netpoll_send_udp(&np, out_buf, out_count);
++		out_count = 0;
++	}
++}
++
++void eth_putDebugChar(int chr)
++{
++	out_buf[out_count++] = chr;
++	if(out_count == OUT_BUF_SIZE)
++		eth_flushDebugChar();
++}
++
++static void rx_hook(struct netpoll *np, int port, char *msg, int len)
++{
++	int i;
++
++	np->remote_port = port;
++
++	/* Is this gdb trying to attach? */
++	if (!netpoll_trap() && len == 8 && !strncmp(msg, "$Hc-1#09", 8))
++		kgdb_schedule_breakpoint();
++
++	for (i = 0; i < len; i++) {
++		if (msg[i] == 3)
++			kgdb_schedule_breakpoint();
++
++		if (atomic_read(&in_count) >= IN_BUF_SIZE) {
++			/* buffer overflow, clear it */
++			in_head = in_tail = 0;
++			atomic_set(&in_count, 0);
++			break;
++		}
++		in_buf[in_head++] = msg[i];
++		in_head &= (IN_BUF_SIZE - 1);
++		atomic_inc(&in_count);
++	}
++}
++
++static int option_setup(char *opt)
++{
++	configured = !netpoll_parse_options(&np, opt);
++	return 0;
++}
++__setup("kgdboe=", option_setup);
++
++static int init_kgdboe(void)
++{
++#ifdef CONFIG_SMP
++	if (num_online_cpus() > CONFIG_NO_KGDB_CPUS) {
++		printk("kgdb: too manu cpus. Cannot enable debugger with more than %d cpus\n", CONFIG_NO_KGDB_CPUS);
++		return -1;
++	}
++#endif
++
++	set_debug_traps();
++
++	if(!configured || netpoll_setup(&np))
++		return 1;
++
++	kgdboe = 1;
++	printk(KERN_INFO "kgdb: debugging over ethernet enabled\n");
++
++	return 0;
++}
++
++module_init(init_kgdboe);
+Index: linux-2.6.16.19/include/asm-i386/kgdb.h
+===================================================================
+--- linux-2.6.16.19.orig/include/asm-i386/kgdb.h	2006-06-23 17:06:11.000000000 -0700
++++ linux-2.6.16.19/include/asm-i386/kgdb.h	2006-06-23 17:06:12.000000000 -0700
+@@ -18,6 +18,14 @@
+ #ifndef BREAKPOINT
+ #define BREAKPOINT   asm("   int $3")
+ #endif
++
++extern void kgdb_schedule_breakpoint(void);
++extern void kgdb_process_breakpoint(void);
++
++extern int kgdb_tty_hook(void);
++extern int kgdb_eth_hook(void);
++extern int kgdboe;
++
+ /*
+  * GDB debug stub (or any debug stub) can point the 'linux_debug_hook'
+  * pointer to its routine and it will be entered as the first thing
+@@ -55,5 +63,7 @@
+ #define kgdb_handle_exception
+ #define breakpoint
+ #define INIT_KGDB_INTS
++#define kgdb_process_breakpoint() do {} while(0)
++
+ #endif
+ #endif				/* __KGDB */
+Index: linux-2.6.16.19/net/core/dev.c
+===================================================================
+--- linux-2.6.16.19.orig/net/core/dev.c	2006-06-23 17:05:14.000000000 -0700
++++ linux-2.6.16.19/net/core/dev.c	2006-06-23 17:06:12.000000000 -0700
+@@ -1370,7 +1370,6 @@
+ 
+ DEFINE_PER_CPU(struct netif_rx_stats, netdev_rx_stat) = { 0, };
+ 
+-
+ /**
+  *	netif_rx	-	post buffer to the network code
+  *	@skb: buffer to post
+@@ -1748,6 +1747,10 @@
+ 			dev_put(dev);
+ 			local_irq_disable();
+ 		}
++
++#ifdef CONFIG_KGDBOE
++		kgdb_process_breakpoint();
++#endif
+ 	}
+ out:
+ 	local_irq_enable();
Index: packages/kernel/patches/tiny_m57sli/tiny/do-printk.patch
===================================================================
--- packages/kernel/patches/tiny_m57sli/tiny/do-printk.patch	(revision 0)
+++ packages/kernel/patches/tiny_m57sli/tiny/do-printk.patch	(revision 0)
@@ -0,0 +1,116 @@
+---
+ include/linux/kernel.h |    2 	1 +	1 -	0 !
+ init/Kconfig           |   24 	21 +	3 -	0 !
+ kernel/printk.c        |   23 	12 +	11 -	0 !
+ 3 files changed, 34 insertions(+), 15 deletions(-)
+
+Index: linux-2.6.22.1/include/linux/kernel.h
+===================================================================
+--- linux-2.6.22.1.orig/include/linux/kernel.h
++++ linux-2.6.22.1/include/linux/kernel.h
+@@ -162,7 +162,7 @@ extern struct pid *session_of_pgrp(struc
+ 
+ extern void dump_thread(struct pt_regs *regs, struct user *dump);
+ 
+-#ifdef CONFIG_PRINTK
++#if defined(CONFIG_PRINTK) || (defined(CONFIG_PRINTK_FUNC) && defined(DO_PRINTK))
+ asmlinkage int vprintk(const char *fmt, va_list args)
+ 	__attribute__ ((format (printf, 1, 0)));
+ asmlinkage int printk(const char * fmt, ...)
+Index: linux-2.6.22.1/init/Kconfig
+===================================================================
+--- linux-2.6.22.1.orig/init/Kconfig
++++ linux-2.6.22.1/init/Kconfig
+@@ -491,15 +491,33 @@ config HOTPLUG
+ 	  disabling this option for embedded systems that do not use modules, a
+ 	  dynamic /dev tree, or dynamic device discovery.  Just say Y.
+ 
++config PRINTK_FUNC
++	default y
++	bool "Enable printk function in the kernel" if EMBEDDED
++	help
++	  This option controls whether the printk function is compiled
++	  into the kernel or not. Removing it eliminates a small amount of
++	  code size from the kernel, which is OK if it is not being used.
++	  See CONFIG_PRINTK to disable the function calls to printk(),
++	  which is where the big win comes in reducing kernel size.
++
+ config PRINTK
+ 	default y
+-	bool "Enable support for printk" if EMBEDDED
++	bool "Enable support for printk messages" if EMBEDDED
++	depends on PRINTK_FUNC
+ 	help
+ 	  This option enables normal printk support. Removing it
+ 	  eliminates most of the message strings from the kernel image
+ 	  and makes the kernel more or less silent. As this makes it
+-	  very difficult to diagnose system problems, saying N here is
+-	  strongly discouraged.
++	  very difficult to diagnose system problems, saying N here on
++	  non-embedded systems is strongly discouraged.
++
++	  With this set to 'N', if the printk function is still enabled,
++	  (see CONFIG_PRINTK_FUNC) you can enable printk messages from
++	  a single file, by putting "#define DO_PRINTK 1" at the top of
++	  the file.  This must appear before any #include lines.
++ 	  This makes it possible to have fine-grained control over
++	  which printk messages to include in the kernel.
+ 
+ config BUG
+ 	bool "BUG() support" if EMBEDDED
+Index: linux-2.6.22.1/kernel/printk.c
+===================================================================
+--- linux-2.6.22.1.orig/kernel/printk.c
++++ linux-2.6.22.1/kernel/printk.c
+@@ -16,6 +16,11 @@
+  *	01Mar01 Andrew Morton <andrewm@uow.edu.au>
+  */
+ 
++/*
++ * have kernel.h produce real declarations, depending on configuration
++ */
++#define DO_PRINTK 1
++
+ #include <linux/kernel.h>
+ #include <linux/mm.h>
+ #include <linux/tty.h>
+@@ -114,13 +119,14 @@ static int preferred_console = -1;
+ /* Flag: console code may call schedule() */
+ static int console_may_schedule;
+ 
+-#ifdef CONFIG_PRINTK
++#ifdef CONFIG_PRINTK_FUNC
+ 
+ static char __log_buf[__LOG_BUF_LEN];
+ static char *log_buf = __log_buf;
+ static int log_buf_len = __LOG_BUF_LEN;
+ static unsigned long logged_chars; /* Number of chars produced since last read+clear operation */
+ 
++
+ static int __init log_buf_len_setup(char *str)
+ {
+ 	unsigned long size = memparse(str, &str);
+@@ -636,18 +642,13 @@ asmlinkage int vprintk(const char *fmt, 
+ EXPORT_SYMBOL(printk);
+ EXPORT_SYMBOL(vprintk);
+ 
+-#else
++#else /* !CONFIG_PRINTK_FUNC */
+ 
+-asmlinkage long sys_syslog(int type, char __user *buf, int len)
+-{
+-	return -ENOSYS;
+-}
+-
+-static void call_console_drivers(unsigned long start, unsigned long end)
+-{
+-}
++asmlinkage long sys_syslog(int type, char __user *buf, int len) { return 0; }
++int do_syslog(int type, char __user *buf, int len) { return 0; }
++static void call_console_drivers(unsigned long start, unsigned long end) {}
+ 
+-#endif
++#endif /* !CONFIG_PRINTK_FUNC */
+ 
+ /*
+  * Set up a list of consoles.  Called from init/main.c
Index: packages/kernel/patches/tiny_m57sli/tiny/buffer-inlines.patch
===================================================================
--- packages/kernel/patches/tiny_m57sli/tiny/buffer-inlines.patch	(revision 0)
+++ packages/kernel/patches/tiny_m57sli/tiny/buffer-inlines.patch	(revision 0)
@@ -0,0 +1,20 @@
+
+uninline a buffer.c function
+
+
+ fs/buffer.c |    2 	1 +	1 -	0 !
+ 1 files changed, 1 insertion(+), 1 deletion(-)
+
+Index: linux-2.6.22.1/fs/buffer.c
+===================================================================
+--- linux-2.6.22.1.orig/fs/buffer.c
++++ linux-2.6.22.1/fs/buffer.c
+@@ -564,7 +564,7 @@ EXPORT_SYMBOL(mark_buffer_async_write);
+ /*
+  * The buffer's backing address_space's private_lock must be held
+  */
+-static inline void __remove_assoc_queue(struct buffer_head *bh)
++static void __remove_assoc_queue(struct buffer_head *bh)
+ {
+ 	list_del_init(&bh->b_assoc_buffers);
+ 	WARN_ON(!bh->b_assoc_map);
Index: packages/kernel/patches/tiny_m57sli/tiny/deprecate-inline.patch
===================================================================
--- packages/kernel/patches/tiny_m57sli/tiny/deprecate-inline.patch	(revision 0)
+++ packages/kernel/patches/tiny_m57sli/tiny/deprecate-inline.patch	(revision 0)
@@ -0,0 +1,182 @@
+
+System for counting inline usage by generating deprecation warnings.
+
+
+ include/linux/compiler-gcc.h  |   12 	9 +	3 -	0 !
+ include/linux/compiler-gcc3.h |    2 	1 +	1 -	0 !
+ include/linux/compiler-gcc4.h |    8 	4 +	4 -	0 !
+ init/Kconfig                  |    9 	9 +	0 -	0 !
+ scripts/count-inlines         |   88 	88 +	0 -	0 !
+ 5 files changed, 111 insertions(+), 8 deletions(-)
+
+Index: alp-linux/include/linux/compiler-gcc.h
+===================================================================
+--- alp-linux.orig/include/linux/compiler-gcc.h
++++ alp-linux/include/linux/compiler-gcc.h
+@@ -26,9 +26,15 @@
+ #define __must_be_array(a) \
+   BUILD_BUG_ON_ZERO(__builtin_types_compatible_p(typeof(a), typeof(&a[0])))
+ 
+-#define inline		inline		__attribute__((always_inline))
+-#define __inline__	__inline__	__attribute__((always_inline))
+-#define __inline	__inline	__attribute__((always_inline))
++#ifdef CONFIG_MEASURE_INLINES
++#define SHOWINLINE __attribute__((deprecated))
++#else
++#define SHOWINLINE
++#endif
++
++#define inline		inline		__attribute__((always_inline)) SHOWINLINE
++#define __inline__	__inline__	__attribute__((always_inline)) SHOWINLINE
++#define __inline	__inline	__attribute__((always_inline)) SHOWINLINE
+ #define __deprecated			__attribute__((deprecated))
+ #define __packed			__attribute__((packed))
+ #define __weak				__attribute__((weak))
+Index: alp-linux/include/linux/compiler-gcc3.h
+===================================================================
+--- alp-linux.orig/include/linux/compiler-gcc3.h
++++ alp-linux/include/linux/compiler-gcc3.h
+@@ -21,4 +21,4 @@
+  */
+ #define uninitialized_var(x) x = x
+ 
+-#define __always_inline		inline __attribute__((always_inline))
++#define __always_inline		inline __attribute__((always_inline)) SHOWINLINE
+Index: alp-linux/init/Kconfig
+===================================================================
+--- alp-linux.orig/init/Kconfig
++++ alp-linux/init/Kconfig
+@@ -345,6 +345,15 @@ source "usr/Kconfig"
+ 
+ endif
+ 
++config MEASURE_INLINES
++	default n
++	bool "Enable inline measurement" if EMBEDDED
++	help
++          This marks inline functions deprecated, causing a kernel compile
++          to spew information about the location of each compilation.
++          With some post-processing, this can be used to generate counts
++          and footprints. Say N unless you want a very noisy build.
++
+ config CC_OPTIMIZE_FOR_SIZE
+ 	bool "Optimize for size (Look out for broken compilers!)"
+ 	default y
+Index: alp-linux/scripts/count-inlines
+===================================================================
+--- /dev/null
++++ alp-linux/scripts/count-inlines
+@@ -0,0 +1,88 @@
++#!/usr/bin/python
++
++# This reads the output of a kernel compile with
++# CONFIG_MEASURE_INLINES enabled and counts the number of code
++# instantiations. This is slightly tricky as gcc reports
++# instantiations of inlines inside of other inlines in header files,
++# so we need to build a call tree and credit only instantiations in C
++# files.
++#
++# Usage:
++#
++#  make 2> inline-data
++#  scripts/count-inlines inline-data | less
++#
++
++import sys, re
++
++showtree=1
++seen={}
++defined={}
++calls={}
++callers={}
++file={}
++uses={}
++usecount={}
++used=[]
++findcache={}
++bytesused={}
++total={}
++in_c={}
++
++inlines = open(sys.argv[1])
++
++# Credit nested inlines
++def credit(func, nest):
++    if nest>10: sys.exit(-1)
++    usecount[func] = usecount.get(func, 0) + 1
++    if not calls.has_key(func): return
++    for f in calls[func]: credit(f, nest+1)
++
++fr = re.compile(r"In function `(\S+)'")
++dr = re.compile(r"(\S+):(\d+): warning: `(\S+)' is deprecated \(declared at (\S+):(\d+)\)")
++current = "unknown"
++
++for l in inlines.xreadlines():
++    g = dr.match(l)
++    if not g:
++        try: current = fr.search(l).group(1)
++        except: pass
++        continue
++
++    use, useline, name, src, srcline = g.groups()
++    srcline, useline = int(srcline), int(useline)
++
++    total[name]= total.get(name, 0) + 1
++    if use[-1] == 'c': in_c[name] = in_c.get(name, 0) + 1
++
++    file.setdefault(src, []).append((name, srcline))
++    if src != use or (srcline != useline and srcline != useline-1):
++        uses.setdefault(name, []).append((use,useline))
++        if use[-1] == 'c': used.append(name)
++
++    if not name in calls.setdefault(current, []):
++        calls[current].append(name)
++    if not current in callers.setdefault(name, []):
++        callers[name].append(current)
++
++for func in used: credit(func, 0)
++
++l = [ (usecount[f], f) for f in usecount.keys() ]
++l.sort()
++l.reverse()
++
++for (c,f) in l:
++    def list(l):
++        a = [ (usecount.get(f),f) for f in l ]
++        a = filter(lambda x: x[0], a)
++        n = len(l) - len(a)
++        if n:
++            a.append((n,"<other>"))
++        a.sort()
++        a.reverse()
++        return " ".join(["%s(%d)" % (f,c) for c,f in a])
++    print "%d  %s (%d in *.c)" % (c, f, in_c.get(f, 0))
++    if showtree:
++        print "calls:", list(calls.get(f, []))
++        print "callers:", list(callers.get(f, []))
++        print
+Index: alp-linux/include/linux/compiler-gcc4.h
+===================================================================
+--- alp-linux.orig/include/linux/compiler-gcc4.h
++++ alp-linux/include/linux/compiler-gcc4.h
+@@ -7,16 +7,16 @@
+ # undef inline
+ # undef __inline__
+ # undef __inline
+-# define inline			inline		__attribute__((always_inline))
+-# define __inline__		__inline__	__attribute__((always_inline))
+-# define __inline		__inline	__attribute__((always_inline))
++# define inline			inline		__attribute__((always_inline)) SHOWINLINE
++# define __inline__		__inline__	__attribute__((always_inline)) SHOWINLINE
++# define __inline		__inline	__attribute__((always_inline)) SHOWINLINE
+ #endif
+ 
+ #define __used			__attribute__((__used__))
+ #define __attribute_used__	__used			/* deprecated */
+ #define __must_check 		__attribute__((warn_unused_result))
+ #define __compiler_offsetof(a,b) __builtin_offsetof(a,b)
+-#define __always_inline		inline __attribute__((always_inline))
++#define __always_inline		inline __attribute__((always_inline)) SHOWINLINE
+ 
+ /*
+  * A trick to suppress uninitialized variable warning without generating any
Index: packages/kernel/patches/tiny_m57sli/tiny/use-funit-at-a-time-kconfig.patch
===================================================================
--- packages/kernel/patches/tiny_m57sli/tiny/use-funit-at-a-time-kconfig.patch	(revision 0)
+++ packages/kernel/patches/tiny_m57sli/tiny/use-funit-at-a-time-kconfig.patch	(revision 0)
@@ -0,0 +1,45 @@
+   Adds a configration to select whether to add -funit-at-a-time to
+   compiler option.
+
+---
+ Makefile     |    2 	2 +	0 -	0 !
+ init/Kconfig |   12 	12 +	0 -	0 !
+ 2 files changed, 14 insertions(+)
+
+Index: linux-2.6.22.1/init/Kconfig
+===================================================================
+--- linux-2.6.22.1.orig/init/Kconfig
++++ linux-2.6.22.1/init/Kconfig
+@@ -587,6 +587,18 @@ config SLOB
+ 
+ endchoice
+ 
++config CC_FUNIT_AT_A_TIME
++       bool "add -funit-at-a-time to gcc option" if EMBEDDED
++       default n
++       help
++	    Enabling this option will pass "-funit-at-a-time" to gcc.
++	    What it does is to first load the whole file into memory
++	    and then generate the output. This allows it to use a
++	    better inlining strategy, drop unused static functions and
++	    use -mregparm automatically for static functions.
++	    Some tests show impressive .text shrinkage from using
++	    unit-at-a-time compilation.
++
+ endmenu		# General setup
+ 
+ config RT_MUTEXES
+Index: linux-2.6.22.1/Makefile
+===================================================================
+--- linux-2.6.22.1.orig/Makefile
++++ linux-2.6.22.1/Makefile
+@@ -509,7 +509,9 @@ CHECKFLAGS     += $(NOSTDINC_FLAGS)
+ 
+ # Enable unit-at-a-time mode when possible. It shrinks the
+ # kernel considerably.
++ifdef CONFIG_CC_FUNIT_AT_A_TIME
+ CFLAGS += $(call cc-option,-funit-at-a-time,)
++endif
+ 
+ # warn about C99 declaration after statement
+ CFLAGS += $(call cc-option,-Wdeclaration-after-statement,)
Index: packages/kernel/patches/tiny_m57sli/tiny/net-filter.patch
===================================================================
--- packages/kernel/patches/tiny_m57sli/tiny/net-filter.patch	(revision 0)
+++ packages/kernel/patches/tiny_m57sli/tiny/net-filter.patch	(revision 0)
@@ -0,0 +1,131 @@
+
+configurable old-style packet filtering support
+
+
+ tiny-mpm/include/linux/filter.h |    6 ++++++
+ tiny-mpm/include/net/sock.h     |   10 +++++++++-
+ tiny-mpm/init/Kconfig           |    8 ++++++++
+ tiny-mpm/net/core/Makefile      |    6 ++++--
+ tiny-mpm/net/core/sock.c        |    6 ++++--
+ 5 files changed, 31 insertions(+), 5 deletions(-)
+
+Index: new-linux/include/linux/filter.h
+===================================================================
+--- new-linux.orig/include/linux/filter.h	2007-04-25 20:08:32.000000000 -0700
++++ new-linux/include/linux/filter.h	2007-04-30 17:13:51.000000000 -0700
+@@ -141,12 +141,18 @@
+ #define SKF_LL_OFF    (-0x200000)
+ 
+ #ifdef __KERNEL__
++#ifdef CONFIG_NET_SK_FILTER
+ struct sk_buff;
+ struct sock;
+ 
+ extern unsigned int sk_run_filter(struct sk_buff *skb, struct sock_filter *filter, int flen);
+ extern int sk_attach_filter(struct sock_fprog *fprog, struct sock *sk);
+ extern int sk_chk_filter(struct sock_filter *filter, int flen);
++#else
++#define sk_run_filter(a, b, c) (0)
++#define sk_attach_filter(a, b) (-EINVAL)
++#define sk_chk_filter(a, b) (-EINVAL)
++#endif /* NET_SK_FILTER */
+ #endif /* __KERNEL__ */
+ 
+ #endif /* __LINUX_FILTER_H__ */
+Index: new-linux/include/net/sock.h
+===================================================================
+--- new-linux.orig/include/net/sock.h	2007-04-27 10:51:05.000000000 -0700
++++ new-linux/include/net/sock.h	2007-04-30 17:14:54.000000000 -0700
+@@ -886,7 +886,7 @@
+  * be accepted or -EPERM if the packet should be tossed.
+  *
+  */
+-
++#ifdef CONFIG_NET_SK_FILTER
+ static inline int sk_filter(struct sock *sk, struct sk_buff *skb)
+ {
+ 	int err;
+@@ -941,6 +941,14 @@
+ 	atomic_inc(&fp->refcnt);
+ 	atomic_add(sk_filter_len(fp), &sk->sk_omem_alloc);
+ }
++#else
++
++#define sk_filter(a, b, c) (0)
++#define sk_filter_release(a, fp)
++#define sk_filter_charge(a, b)
++
++#endif
++
+ 
+ /*
+  * Socket reference counting postulates.
+Index: new-linux/init/Kconfig
+===================================================================
+--- new-linux.orig/init/Kconfig	2007-04-30 17:13:32.000000000 -0700
++++ new-linux/init/Kconfig	2007-04-30 17:13:51.000000000 -0700
+@@ -341,6 +341,14 @@
+ 	  This allows storing various information about network peers.
+ 	  Disabling saves about 2k.
+ 
++config NET_SK_FILTER
++        default y
++	depends NET
++	bool "Enable network socket filter" if EMBEDDED
++	help
++	  This enables the basic kernel level socket filtering, not to
++	  be confused with netfilter. Required by PPP.
++
+ config MEASURE_INLINES
+ 	default n
+ 	bool "Enable inline measurement" if EMBEDDED
+Index: new-linux/net/core/Makefile
+===================================================================
+--- new-linux.orig/net/core/Makefile	2007-04-30 17:12:54.000000000 -0700
++++ new-linux/net/core/Makefile	2007-04-30 17:15:37.000000000 -0700
+@@ -8,7 +8,7 @@
+ obj-$(CONFIG_SYSCTL) += sysctl_net_core.o
+ 
+ obj-y		     += dev.o dev_mcast.o dst.o netevent.o \
+-			neighbour.o rtnetlink.o utils.o link_watch.o filter.o
++			neighbour.o rtnetlink.o utils.o link_watch.o
+ 
+ obj-$(CONFIG_XFRM) += flow.o
+ obj-$(CONFIG_ETHTOOL) += ethtool.o
+@@ -18,3 +18,4 @@
+ obj-$(CONFIG_NETPOLL) += netpoll.o
+ obj-$(CONFIG_NET_DMA) += user_dma.o
+ obj-$(CONFIG_FIB_RULES) += fib_rules.o
++obj-$(CONFIG_NET_SK_FILTER) += filter.o
+Index: new-linux/net/core/sock.c
+===================================================================
+--- new-linux.orig/net/core/sock.c	2007-04-27 10:51:05.000000000 -0700
++++ new-linux/net/core/sock.c	2007-04-30 17:13:51.000000000 -0700
+@@ -342,7 +342,9 @@
+ 		    char __user *optval, int optlen)
+ {
+ 	struct sock *sk=sock->sk;
++#ifdef CONFIG_NET_SK_FILTER
+ 	struct sk_filter *filter;
++#endif
+ 	int val;
+ 	int valbool;
+ 	struct linger ling;
+@@ -591,7 +593,7 @@
+ 		}
+ #endif
+ 
+-
++#ifdef CONFIG_NET_SK_FILTER
+ 		case SO_ATTACH_FILTER:
+ 			ret = -EINVAL;
+ 			if (optlen == sizeof(struct sock_fprog)) {
+@@ -624,7 +626,7 @@
+ 			else
+ 				clear_bit(SOCK_PASSSEC, &sock->flags);
+ 			break;
+-
++#endif
+ 		/* We implement the SO_SNDLOWAT etc to
+ 		   not be settable (1003.1g 5.3) */
+ 		default:
Index: packages/kernel/patches/tiny_m57sli/tiny/slob-accounting-fix.patch
===================================================================
--- packages/kernel/patches/tiny_m57sli/tiny/slob-accounting-fix.patch	(revision 0)
+++ packages/kernel/patches/tiny_m57sli/tiny/slob-accounting-fix.patch	(revision 0)
@@ -0,0 +1,29 @@
+    This patch fixes compilation error when CONFIG_SLOB=y and 
+    CONFIG_KMALLOC_ACCOUNTING is not set.
+
+---
+ include/linux/slab.h |    4 	2 +	2 -	0 !
+ 1 file changed, 2 insertions(+), 2 deletions(-)
+
+Index: linux-2.6.16-ALP-BRANCH_SS-mips.git/include/linux/slab.h
+===================================================================
+--- linux-2.6.16-ALP-BRANCH_SS-mips.git.orig/include/linux/slab.h
++++ linux-2.6.16-ALP-BRANCH_SS-mips.git/include/linux/slab.h
+@@ -53,8 +53,6 @@ typedef struct kmem_cache kmem_cache_t;
+ #define SLAB_CTOR_ATOMIC	0x002UL		/* tell constructor it can't sleep */
+ #define	SLAB_CTOR_VERIFY	0x004UL		/* tell constructor it's a verify call */
+ 
+-#ifndef CONFIG_SLOB
+-
+ #ifdef CONFIG_KMALLOC_ACCOUNTING
+ void __kmalloc_account(const void *, const void *, int, int);
+ 
+@@ -72,6 +70,8 @@ static void inline kfree_account(const v
+ #define kfree_account(a, b)
+ #endif
+ 
++#ifndef CONFIG_SLOB
++
+ /* prototypes */
+ extern void __init kmem_cache_init(void);
+ 
Index: packages/kernel/patches/tiny_m57sli/tiny/sysenter.patch
===================================================================
--- packages/kernel/patches/tiny_m57sli/tiny/sysenter.patch	(revision 0)
+++ packages/kernel/patches/tiny_m57sli/tiny/sysenter.patch	(revision 0)
@@ -0,0 +1,184 @@
+
+configurable sysenter support
+
+
+ arch/i386/kernel/Makefile     |    3 	2 +	1 -	0 !
+ arch/i386/kernel/cpu/common.c |    4 	4 +	0 -	0 !
+ arch/i386/kernel/entry.S      |    8 	8 +	0 -	0 !
+ arch/i386/kernel/sysenter.c   |   12 	12 +	0 -	0 !
+ arch/i386/power/cpu.c         |    2 	2 +	0 -	0 !
+ include/asm-i386/elf.h        |    2 	2 +	0 -	0 !
+ init/Kconfig                  |    8 	8 +	0 -	0 !
+ 7 files changed, 38 insertions(+), 1 deletion(-)
+
+Index: linux-2.6.22.1/arch/i386/kernel/Makefile
+===================================================================
+--- linux-2.6.22.1.orig/arch/i386/kernel/Makefile
++++ linux-2.6.22.1/arch/i386/kernel/Makefile
+@@ -31,7 +31,8 @@ obj-$(CONFIG_X86_NUMAQ)		+= numaq.o
+ obj-$(CONFIG_X86_SUMMIT_NUMA)	+= summit.o
+ obj-$(CONFIG_KPROBES)		+= kprobes.o
+ obj-$(CONFIG_MODULES)		+= module.o
+-obj-y				+= sysenter.o vsyscall.o
++obj-y				+= sysenter.o
++obj-$(CONFIG_SYSENTER)		+= vsyscall.o
+ obj-$(CONFIG_ACPI_SRAT) 	+= srat.o
+ obj-$(CONFIG_EFI) 		+= efi.o efi_stub.o
+ obj-$(CONFIG_DOUBLEFAULT) 	+= doublefault.o
+Index: linux-2.6.22.1/arch/i386/kernel/cpu/common.c
+===================================================================
+--- linux-2.6.22.1.orig/arch/i386/kernel/cpu/common.c
++++ linux-2.6.22.1/arch/i386/kernel/cpu/common.c
+@@ -506,8 +506,10 @@ static void __cpuinit identify_cpu(struc
+ void __init identify_boot_cpu(void)
+ {
+ 	identify_cpu(&boot_cpu_data);
++#ifdef CONFIG_SYSENTER
+ 	sysenter_setup();
+ 	enable_sep_cpu();
++#endif
+ 	mtrr_bp_init();
+ }
+ 
+@@ -515,7 +517,9 @@ void __cpuinit identify_secondary_cpu(st
+ {
+ 	BUG_ON(c == &boot_cpu_data);
+ 	identify_cpu(c);
++#ifdef CONFIG_SYSENTER
+ 	enable_sep_cpu();
++#endif
+ 	mtrr_ap_init();
+ }
+ 
+Index: linux-2.6.22.1/arch/i386/kernel/entry.S
+===================================================================
+--- linux-2.6.22.1.orig/arch/i386/kernel/entry.S
++++ linux-2.6.22.1/arch/i386/kernel/entry.S
+@@ -283,6 +283,7 @@ END(resume_kernel)
+ 
+ 	# sysenter call handler stub
+ ENTRY(sysenter_entry)
++#ifdef CONFIG_SYSENTER
+ 	CFI_STARTPROC simple
+ 	CFI_SIGNAL_FRAME
+ 	CFI_DEF_CFA esp, 0
+@@ -359,6 +360,7 @@ sysenter_past_esp:
+ 	.long 1b,2b
+ .popsection
+ ENDPROC(sysenter_entry)
++#endif
+ 
+ 	# system call handler stub
+ ENTRY(system_call)
+@@ -749,6 +751,8 @@ END(device_not_available)
+  * by hand onto the new stack - while updating the return eip past
+  * the instruction that would have done it for sysenter.
+  */
++
++#ifdef CONFIG_SYSENTER
+ #define FIX_STACK(offset, ok, label)		\
+ 	cmpw $__KERNEL_CS,4(%esp);		\
+ 	jne ok;					\
+@@ -763,6 +767,10 @@ label:						\
+ 	pushl $sysenter_past_esp;		\
+ 	CFI_ADJUST_CFA_OFFSET 4;		\
+ 	CFI_REL_OFFSET eip, 0
++#else
++#define FIX_STACK(offset, ok, label) \
++label:
++#endif
+ 
+ KPROBE_ENTRY(debug)
+ 	RING0_INT_FRAME
+Index: linux-2.6.22.1/arch/i386/power/cpu.c
+===================================================================
+--- linux-2.6.22.1.orig/arch/i386/power/cpu.c
++++ linux-2.6.22.1/arch/i386/power/cpu.c
+@@ -111,11 +111,13 @@ void __restore_processor_state(struct sa
+  	loadsegment(gs, ctxt->gs);
+  	loadsegment(ss, ctxt->ss);
+ 
++#ifdef CONFIG_SYSENTER
+ 	/*
+ 	 * sysenter MSRs
+ 	 */
+ 	if (boot_cpu_has(X86_FEATURE_SEP))
+ 		enable_sep_cpu();
++#endif
+ 
+ 	fix_processor_context();
+ 	do_fpu_end();
+Index: linux-2.6.22.1/include/asm-i386/elf.h
+===================================================================
+--- linux-2.6.22.1.orig/include/asm-i386/elf.h
++++ linux-2.6.22.1/include/asm-i386/elf.h
+@@ -152,11 +152,13 @@ extern int arch_setup_additional_pages(s
+ 
+ extern unsigned int vdso_enabled;
+ 
++#ifdef CONFIG_SYSENTER
+ #define ARCH_DLINFO							\
+ do if (vdso_enabled) {							\
+ 		NEW_AUX_ENT(AT_SYSINFO,	VDSO_ENTRY);			\
+ 		NEW_AUX_ENT(AT_SYSINFO_EHDR, VDSO_CURRENT_BASE);	\
+ } while (0)
++#endif
+ 
+ #endif
+ 
+Index: linux-2.6.22.1/init/Kconfig
+===================================================================
+--- linux-2.6.22.1.orig/init/Kconfig
++++ linux-2.6.22.1/init/Kconfig
+@@ -345,6 +345,14 @@ source "usr/Kconfig"
+ 
+ endif
+ 
++config SYSENTER
++	depends X86
++	default y
++	bool "Enable syscalls via sysenter" if EMBEDDED
++	help
++	  Disabling this feature removes sysenter handling as well as
++	  vsyscall fixmaps.
++
+ config MEASURE_INLINES
+ 	default n
+ 	bool "Enable inline measurement" if EMBEDDED
+Index: linux-2.6.22.1/arch/i386/kernel/sysenter.c
+===================================================================
+--- linux-2.6.22.1.orig/arch/i386/kernel/sysenter.c
++++ linux-2.6.22.1/arch/i386/kernel/sysenter.c
+@@ -37,6 +37,9 @@ enum {
+ #define VDSO_DEFAULT	VDSO_ENABLED
+ #endif
+ 
++
++#ifdef CONFIG_SYSENTER
++
+ /*
+  * Should the kernel map a VDSO page into processes and pass its
+  * address down to glibc upon exec()?
+@@ -317,6 +320,7 @@ int arch_setup_additional_pages(struct l
+ 	return ret;
+ }
+ 
++
+ const char *arch_vma_name(struct vm_area_struct *vma)
+ {
+ 	if (vma->vm_mm && vma->vm_start == (long)vma->vm_mm->context.vdso)
+@@ -334,6 +338,14 @@ struct vm_area_struct *get_gate_vma(stru
+ 	return NULL;
+ }
+ 
++#else
++unsigned int __read_mostly vdso_enabled = 0;
++void enable_sep_cpu(void) { }
++int arch_setup_additional_pages(struct linux_binprm *bprm, int exstack) { return 0; }
++const char *arch_vma_name(struct vm_area_struct *vma) { return NULL; }
++struct vm_area_struct *get_gate_vma(struct task_struct *tsk) { return NULL; }
++#endif
++
+ int in_gate_area(struct task_struct *task, unsigned long addr)
+ {
+ 	return 0;
Index: packages/kernel/patches/tiny_m57sli/tiny/no-doublefault.patch
===================================================================
--- packages/kernel/patches/tiny_m57sli/tiny/no-doublefault.patch	(revision 0)
+++ packages/kernel/patches/tiny_m57sli/tiny/no-doublefault.patch	(revision 0)
@@ -0,0 +1,65 @@
+
+configurable support for doublefault reporting
+
+
+ tiny-mpm/arch/i386/kernel/Makefile     |    7 ++++---
+ tiny-mpm/arch/i386/kernel/cpu/common.c |    2 ++
+ tiny-mpm/init/Kconfig                  |    9 +++++++++
+ 3 files changed, 15 insertions(+), 3 deletions(-)
+
+Index: tiny/arch/i386/kernel/Makefile
+===================================================================
+--- tiny.orig/arch/i386/kernel/Makefile	2005-10-10 17:39:10.000000000 -0700
++++ tiny/arch/i386/kernel/Makefile	2005-10-10 17:42:15.000000000 -0700
+@@ -7,7 +7,7 @@ extra-y := head.o init_task.o vmlinux.ld
+ obj-y	:= process.o semaphore.o signal.o entry.o traps.o irq.o vm86.o \
+ 		ptrace.o time.o ioport.o ldt.o setup.o i8259.o sys_i386.o \
+ 		pci-dma.o i386_ksyms.o i387.o dmi_scan.o bootflag.o \
+-		doublefault.o quirks.o i8237.o
++		quirks.o i8237.o
+ 
+ obj-y				+= cpu/
+ obj-y				+= timers/
+@@ -34,6 +34,7 @@ obj-y				+= sysenter.o vsyscall.o
+ obj-$(CONFIG_ACPI_SRAT) 	+= srat.o
+ obj-$(CONFIG_HPET_TIMER) 	+= time_hpet.o
+ obj-$(CONFIG_EFI) 		+= efi.o efi_stub.o
++obj-$(CONFIG_DOUBLEFAULT) 	+= doublefault.o
+ obj-$(CONFIG_EARLY_PRINTK)	+= early_printk.o
+ 
+ EXTRA_AFLAGS   := -traditional
+Index: tiny/arch/i386/kernel/cpu/common.c
+===================================================================
+--- tiny.orig/arch/i386/kernel/cpu/common.c	2005-10-10 17:32:31.000000000 -0700
++++ tiny/arch/i386/kernel/cpu/common.c	2005-10-10 17:41:59.000000000 -0700
+@@ -635,8 +635,10 @@ void __devinit cpu_init(void)
+ 	load_TR_desc();
+ 	load_LDT(&init_mm.context);
+ 
++#ifdef CONFIG_DOUBLEFAULT
+ 	/* Set up doublefault TSS pointer in the GDT */
+ 	__set_tss_desc(cpu, GDT_ENTRY_DOUBLEFAULT_TSS, &doublefault_tss);
++#endif
+ 
+ 	/* Clear %fs and %gs. */
+ 	asm volatile ("xorl %eax, %eax; movl %eax, %fs; movl %eax, %gs");
+Index: tiny/init/Kconfig
+===================================================================
+--- tiny.orig/init/Kconfig	2005-10-10 17:41:53.000000000 -0700
++++ tiny/init/Kconfig	2005-10-10 17:41:59.000000000 -0700
+@@ -329,6 +329,15 @@ config AUDIT_BOOTMEM
+ 	  This option causes the bootmem allocator to print caller and size
+           for each allocation.
+ 
++config DOUBLEFAULT
++	depends X86
++	default y if X86
++	bool "Enable doublefault exception handler" if EMBEDDED
++	help
++          This option allows trapping of rare doublefault exceptions that
++          would otherwise cause a system to silently reboot. Disabling this
++          option saves about 4k.
++
+ config BASE_FULL
+ 	default y
+ 	bool "Enable full-sized data structures for core" if EMBEDDED
Index: packages/kernel/patches/tiny_m57sli/tiny/max-swapfiles.patch
===================================================================
--- packages/kernel/patches/tiny_m57sli/tiny/max-swapfiles.patch	(revision 0)
+++ packages/kernel/patches/tiny_m57sli/tiny/max-swapfiles.patch	(revision 0)
@@ -0,0 +1,82 @@
+
+configurable number of swapfiles
+
+
+ include/linux/swap.h    |    2 	1 +	1 -	0 !
+ include/linux/swapops.h |   24 	24 +	0 -	0 !
+ init/Kconfig            |    7 	7 +	0 -	0 !
+ 3 files changed, 32 insertions(+), 1 deletion(-)
+
+Index: linux-2.6.22.1/include/linux/swap.h
+===================================================================
+--- linux-2.6.22.1.orig/include/linux/swap.h
++++ linux-2.6.22.1/include/linux/swap.h
+@@ -31,7 +31,7 @@ static inline int current_is_kswapd(void
+  * on 32-bit-pgoff_t architectures.  And that assumes that the architecture packs
+  * the type/offset into the pte as 5/27 as well.
+  */
+-#define MAX_SWAPFILES_SHIFT	5
++#define MAX_SWAPFILES_SHIFT	CONFIG_MAX_SWAPFILES_SHIFT
+ #ifndef CONFIG_MIGRATION
+ #define MAX_SWAPFILES		(1 << MAX_SWAPFILES_SHIFT)
+ #else
+Index: linux-2.6.22.1/include/linux/swapops.h
+===================================================================
+--- linux-2.6.22.1.orig/include/linux/swapops.h
++++ linux-2.6.22.1/include/linux/swapops.h
+@@ -15,6 +15,9 @@
+ /*
+  * Store a type+offset into a swp_entry_t in an arch-independent format
+  */
++
++#if MAX_SWAPFILES_SHIFT > 0
++
+ static inline swp_entry_t swp_entry(unsigned long type, pgoff_t offset)
+ {
+ 	swp_entry_t ret;
+@@ -42,6 +45,27 @@ static inline pgoff_t swp_offset(swp_ent
+ 	return entry.val & SWP_OFFSET_MASK(entry);
+ }
+ 
++#else /* avoid undefined shift operations */
++
++static inline swp_entry_t swp_entry(unsigned type, pgoff_t offset)
++{
++	swp_entry_t ret;
++	ret.val = offset;
++	return ret;
++}
++
++static inline unsigned swp_type(swp_entry_t entry)
++{
++	return 0;
++}
++
++static inline pgoff_t swp_offset(swp_entry_t entry)
++{
++	return entry.val;
++}
++
++#endif
++
+ /*
+  * Convert the arch-dependent pte representation of a swp_entry_t into an
+  * arch-independent swp_entry_t.
+Index: linux-2.6.22.1/init/Kconfig
+===================================================================
+--- linux-2.6.22.1.orig/init/Kconfig
++++ linux-2.6.22.1/init/Kconfig
+@@ -704,6 +704,13 @@ config BINFMT_SCRIPT
+ 	help
+ 	  Enable support for standard #!-style shell scripts.
+ 
++config MAX_SWAPFILES_SHIFT
++	int "Number of swap files log2 (0 => 1, 5 => 32)" if EMBEDDED
++	range 0 5
++	default 5
++	help
++	  Select the maximum number of swapfiles (5 for default).
++
+ endmenu		# General setup
+ 
+ config RT_MUTEXES
Index: packages/kernel/patches/tiny_m57sli/tiny/tiny-cflags.patch
===================================================================
--- packages/kernel/patches/tiny_m57sli/tiny/tiny-cflags.patch	(revision 0)
+++ packages/kernel/patches/tiny_m57sli/tiny/tiny-cflags.patch	(revision 0)
@@ -0,0 +1,57 @@
+
+configurable CFLAGS override
+
+
+ tiny-mpm/arch/i386/Makefile |    4 ++++
+ tiny-mpm/init/Kconfig       |   14 ++++++++++++++
+ 2 files changed, 18 insertions(+)
+
+Index: tiny/arch/i386/Makefile
+===================================================================
+--- tiny.orig/arch/i386/Makefile	2005-10-10 17:39:10.000000000 -0700
++++ tiny/arch/i386/Makefile	2005-10-10 17:41:44.000000000 -0700
+@@ -35,6 +35,10 @@ CFLAGS += -pipe -msoft-float
+ CFLAGS += $(call cc-option,-mpreferred-stack-boundary=2)
+ 
+ align := $(cc-option-align)
++
++ifdef CONFIG_TINY_CFLAGS
++cflags-y += $(shell echo $(CONFIG_TINY_CFLAGS_VAL))
++else
+ cflags-$(CONFIG_M386)		+= -march=i386
+ cflags-$(CONFIG_M486)		+= -march=i486
+ cflags-$(CONFIG_M586)		+= -march=i586
+@@ -64,6 +68,8 @@ cflags-$(CONFIG_X86_ELAN)	+= -march=i486
+ # Geode GX1 support
+ cflags-$(CONFIG_MGEODEGX1)		+= $(call cc-option,-march=pentium-mmx,-march=i486)
+ 
++endif
++
+ # -mregparm=3 works ok on gcc-3.0 and later
+ #
+ GCC_VERSION			:= $(call cc-version)
+Index: tiny/init/Kconfig
+===================================================================
+--- tiny.orig/init/Kconfig	2005-10-10 17:41:17.000000000 -0700
++++ tiny/init/Kconfig	2005-10-10 17:41:44.000000000 -0700
+@@ -405,6 +405,20 @@ config CC_ALIGN_JUMPS
+ 	  no dummy operations need be executed.
+ 	  Zero means use compiler's default.
+ 
++config TINY_CFLAGS
++	default n
++	bool "Set compiler arch flags for small 386 code" if EMBEDDED
++	help
++	  This allows user to replace the architecture CFLAGS despite the arch
++	  setting in an attempt to build smaller code.
++
++config TINY_CFLAGS_VAL
++	depends TINY_CFLAGS
++	default "-march=i386"
++	string "Arch CFLAGS"
++	help
++	  Enter CFLAGS to override ARCH defaults.
++
+ endmenu		# General setup
+ 
+ config TINY_SHMEM
Index: packages/kernel/patches/tiny_m57sli/tiny/max_user_rt_prio.patch
===================================================================
--- packages/kernel/patches/tiny_m57sli/tiny/max_user_rt_prio.patch	(revision 0)
+++ packages/kernel/patches/tiny_m57sli/tiny/max_user_rt_prio.patch	(revision 0)
@@ -0,0 +1,39 @@
+
+Configurable number of RT priority O(1) scheduling queues
+
+
+ include/linux/sched.h |    2 	1 +	1 -	0 !
+ init/Kconfig          |    7 	7 +	0 -	0 !
+ 2 files changed, 8 insertions(+), 1 deletion(-)
+
+Index: linux-2.6.22.1/include/linux/sched.h
+===================================================================
+--- linux-2.6.22.1.orig/include/linux/sched.h
++++ linux-2.6.22.1/include/linux/sched.h
+@@ -535,7 +535,7 @@ struct signal_struct {
+  * MAX_RT_PRIO must not be smaller than MAX_USER_RT_PRIO.
+  */
+ 
+-#define MAX_USER_RT_PRIO	100
++#define MAX_USER_RT_PRIO	CONFIG_MAX_USER_RT_PRIO
+ #define MAX_RT_PRIO		MAX_USER_RT_PRIO
+ 
+ #define MAX_PRIO		(MAX_RT_PRIO + 40)
+Index: linux-2.6.22.1/init/Kconfig
+===================================================================
+--- linux-2.6.22.1.orig/init/Kconfig
++++ linux-2.6.22.1/init/Kconfig
+@@ -719,6 +719,13 @@ config NR_LDISCS
+ 	  Select the maximum number of tty line disciplines. Embedded systems
+           can probably get by with only a few of these.
+ 
++config MAX_USER_RT_PRIO
++	int "Number of realtime priority levels" if EMBEDDED
++	range 5 100
++	default 100
++	help
++	  This option allows reducing the size of the O(1) scheduler arrays.
++
+ endmenu		# General setup
+ 
+ config RT_MUTEXES
Index: packages/kernel/patches/tiny_m57sli/tiny/fslock.patch
===================================================================
--- packages/kernel/patches/tiny_m57sli/tiny/fslock.patch	(revision 0)
+++ packages/kernel/patches/tiny_m57sli/tiny/fslock.patch	(revision 0)
@@ -0,0 +1,225 @@
+
+configurable POSIX file locking API
+
+
+ fs/Kconfig          |    2 	1 +	1 -	0 !
+ fs/Makefile         |    3 	2 +	1 -	0 !
+ fs/proc/proc_misc.c |    4 	4 +	0 -	0 !
+ include/linux/fs.h  |   43 	42 +	1 -	0 !
+ init/Kconfig        |    6 	6 +	0 -	0 !
+ kernel/sys_ni.c     |    1 	1 +	0 -	0 !
+ kernel/sysctl.c     |    5 	5 +	0 -	0 !
+ 7 files changed, 61 insertions(+), 3 deletions(-)
+
+Index: linux-2.6.22.1/fs/Kconfig
+===================================================================
+--- linux-2.6.22.1.orig/fs/Kconfig
++++ linux-2.6.22.1/fs/Kconfig
+@@ -1525,7 +1525,7 @@ menu "Network File Systems"
+ 
+ config NFS_FS
+ 	tristate "NFS file system support"
+-	depends on INET
++	depends on INET && FILE_LOCKING
+ 	select LOCKD
+ 	select SUNRPC
+ 	select NFS_ACL_SUPPORT if NFS_V3_ACL
+Index: linux-2.6.22.1/fs/Makefile
+===================================================================
+--- linux-2.6.22.1.orig/fs/Makefile
++++ linux-2.6.22.1/fs/Makefile
+@@ -7,7 +7,7 @@
+ 
+ obj-y :=	open.o read_write.o file_table.o super.o \
+ 		char_dev.o stat.o exec.o pipe.o namei.o fcntl.o \
+-		ioctl.o readdir.o select.o fifo.o locks.o dcache.o inode.o \
++		ioctl.o readdir.o select.o fifo.o dcache.o inode.o \
+ 		attr.o bad_inode.o file.o filesystems.o namespace.o aio.o \
+ 		seq_file.o libfs.o fs-writeback.o \
+ 		pnode.o drop_caches.o splice.o sync.o utimes.o \
+@@ -21,6 +21,7 @@ endif
+ 
+ obj-$(CONFIG_INOTIFY)		+= inotify.o
+ obj-$(CONFIG_INOTIFY_USER)	+= inotify_user.o
++obj-$(CONFIG_FILE_LOCKING)	+= locks.o
+ obj-$(CONFIG_XATTR)		+= xattr.o
+ obj-$(CONFIG_EPOLL)		+= eventpoll.o
+ obj-$(CONFIG_ANON_INODES)	+= anon_inodes.o
+Index: linux-2.6.22.1/fs/proc/proc_misc.c
+===================================================================
+--- linux-2.6.22.1.orig/fs/proc/proc_misc.c
++++ linux-2.6.22.1/fs/proc/proc_misc.c
+@@ -606,6 +606,7 @@ static int cmdline_read_proc(char *page,
+ 	return proc_calc_metrics(page, start, off, count, eof, len);
+ }
+ 
++#ifdef CONFIG_FILE_LOCKING
+ static int locks_read_proc(char *page, char **start, off_t off,
+ 				 int count, int *eof, void *data)
+ {
+@@ -615,6 +616,7 @@ static int locks_read_proc(char *page, c
+ 		*eof = 1;
+ 	return len;
+ }
++#endif
+ 
+ static int execdomains_read_proc(char *page, char **start, off_t off,
+ 				 int count, int *eof, void *data)
+@@ -673,7 +675,9 @@ void __init proc_misc_init(void)
+ #endif
+ 		{"filesystems",	filesystems_read_proc},
+ 		{"cmdline",	cmdline_read_proc},
++#ifdef CONFIG_FILE_LOCKING
+ 		{"locks",	locks_read_proc},
++#endif
+ 		{"execdomains",	execdomains_read_proc},
+ 		{NULL,}
+ 	};
+Index: linux-2.6.22.1/include/linux/fs.h
+===================================================================
+--- linux-2.6.22.1.orig/include/linux/fs.h
++++ linux-2.6.22.1/include/linux/fs.h
+@@ -832,6 +832,9 @@ struct file_lock {
+ 
+ #include <linux/fcntl.h>
+ 
++extern void send_sigio(struct fown_struct *fown, int fd, int band);
++
++#ifdef CONFIG_FILE_LOCKING
+ extern int fcntl_getlk(struct file *, struct flock __user *);
+ extern int fcntl_setlk(unsigned int, struct file *, unsigned int,
+ 			struct flock __user *);
+@@ -842,7 +845,6 @@ extern int fcntl_setlk64(unsigned int, s
+ 			struct flock64 __user *);
+ #endif
+ 
+-extern void send_sigio(struct fown_struct *fown, int fd, int band);
+ extern int fcntl_setlease(unsigned int fd, struct file *filp, long arg);
+ extern int fcntl_getlease(struct file *filp);
+ 
+@@ -870,6 +872,32 @@ extern int lease_modify(struct file_lock
+ extern int lock_may_read(struct inode *, loff_t start, unsigned long count);
+ extern int lock_may_write(struct inode *, loff_t start, unsigned long count);
+ 
++#else /* !CONFIG_FILE_LOCKING */
++#define fcntl_getlk(a, b) (-EINVAL)
++#define fcntl_setlk(a, b, c, d) (-EACCES)
++#if BITS_PER_LONG == 32
++#define fcntl_getlk64(a, b) (-EINVAL)
++#define fcntl_setlk64(a, b, c, d) (-EACCES)
++#endif
++#define fcntl_setlease(a, b, c) (0)
++#define fcntl_getlease(a) (0)
++#define locks_init_lock(a)
++#define locks_copy_lock(a, b)
++#define locks_remove_posix(a, b)
++#define locks_remove_flock(a)
++#define posix_test_lock(a, b) (0)
++#define posix_lock_file(a, b) (-ENOLCK)
++#define posix_lock_file_wait(a, b) (-ENOLCK)
++#define posix_block_lock(a, b)
++#define posix_unblock_lock(a, b)
++#define posix_locks_deadlock(a, b) (0)
++#define __break_lease(a, b) (0)
++#define lease_get_mtime(a, b)
++#define lock_may_read(a, b, c) (1)
++#define lock_may_write(a, b, c) (1)
++#define steal_locks(a)
++#endif /* !CONFIG_FILE_LOCKING */
++
+ struct fasync_struct {
+ 	int	magic;
+ 	int	fa_fd;
+@@ -1456,6 +1484,7 @@ extern struct kset fs_subsys;
+ #define FLOCK_VERIFY_READ  1
+ #define FLOCK_VERIFY_WRITE 2
+ 
++#ifdef CONFIG_FILE_LOCKING
+ extern int locks_mandatory_locked(struct inode *);
+ extern int locks_mandatory_area(int, struct inode *, struct file *, loff_t, size_t);
+ 
+@@ -1496,6 +1525,18 @@ static inline int break_lease(struct ino
+ 	return 0;
+ }
+ 
++#else /* !CONFIG_FILE_LOCKING */
++
++#define locks_mandatory_locked(a) (0)
++#define locks_mandatory_area(a, b, c, d, e) (0)
++#define MANDATORY_LOCK(inode) (0)
++#define locks_verify_locked(a) (0)
++#define locks_verify_area(a, b, c, d, e) (0 && (b))
++#define locks_verify_truncate(a, b, c) (0)
++#define break_lease(a, b) (0)
++
++#endif /* !CONFIG_FILE_LOCKING */
++
+ /* fs/open.c */
+ 
+ extern int do_truncate(struct dentry *, loff_t start, unsigned int time_attrs,
+Index: linux-2.6.22.1/init/Kconfig
+===================================================================
+--- linux-2.6.22.1.orig/init/Kconfig
++++ linux-2.6.22.1/init/Kconfig
+@@ -367,6 +367,12 @@ config XATTR
+ 	help
+ 	  This enables the filesystem extended attribute syscalls.
+ 
++config FILE_LOCKING
++	default y
++	bool "Enable POSIX file locking API" if EMBEDDED
++	help
++	  This enables standard file locking support.
++
+ config MEASURE_INLINES
+ 	default n
+ 	bool "Enable inline measurement" if EMBEDDED
+Index: linux-2.6.22.1/kernel/sys_ni.c
+===================================================================
+--- linux-2.6.22.1.orig/kernel/sys_ni.c
++++ linux-2.6.22.1/kernel/sys_ni.c
+@@ -129,6 +129,7 @@ cond_syscall(sys_flistxattr);
+ cond_syscall(sys_removexattr);
+ cond_syscall(sys_lremovexattr);
+ cond_syscall(sys_fremovexattr);
++cond_syscall(sys_flock);
+ 
+ /* arch-specific weak syscall entries */
+ cond_syscall(sys_pciconfig_read);
+Index: linux-2.6.22.1/kernel/sysctl.c
+===================================================================
+--- linux-2.6.22.1.orig/kernel/sysctl.c
++++ linux-2.6.22.1/kernel/sysctl.c
+@@ -954,6 +954,7 @@ static ctl_table fs_table[] = {
+ 		.extra1		= &minolduid,
+ 		.extra2		= &maxolduid,
+ 	},
++#ifdef CONFIG_FILE_LOCKING
+ 	{
+ 		.ctl_name	= FS_LEASES,
+ 		.procname	= "leases-enable",
+@@ -962,6 +963,7 @@ static ctl_table fs_table[] = {
+ 		.mode		= 0644,
+ 		.proc_handler	= &proc_dointvec,
+ 	},
++#endif
+ #ifdef CONFIG_DNOTIFY
+ 	{
+ 		.ctl_name	= FS_DIR_NOTIFY,
+@@ -973,6 +975,8 @@ static ctl_table fs_table[] = {
+ 	},
+ #endif
+ #ifdef CONFIG_MMU
++#ifdef CONFIG_FILE_LOCKING
++
+ 	{
+ 		.ctl_name	= FS_LEASE_TIME,
+ 		.procname	= "lease-break-time",
+@@ -981,6 +985,7 @@ static ctl_table fs_table[] = {
+ 		.mode		= 0644,
+ 		.proc_handler	= &proc_dointvec,
+ 	},
++#endif
+ #ifdef CONFIG_AIO
+ 	{
+ 		.ctl_name	= FS_AIO_NR,
Index: packages/kernel/patches/tiny_m57sli/tiny/inflate-killmarkrelease.patch
===================================================================
--- packages/kernel/patches/tiny_m57sli/tiny/inflate-killmarkrelease.patch	(revision 0)
+++ packages/kernel/patches/tiny_m57sli/tiny/inflate-killmarkrelease.patch	(revision 0)
@@ -0,0 +1,971 @@
+inflate: refactor inflate malloc code
+
+Inflate requires some dynamic memory allocation very early in the boot
+process and this is provided with a set of four functions:
+malloc/free/gzip_mark/gzip_release.
+
+The old inflate code used a mark/release strategy rather than
+implement free. This new version instead keeps a count on the number
+of outstanding allocations and when it hits zero, it resets the malloc
+arena.
+
+This allows removing all the mark and release implementations and
+unifying all the malloc/free implementations.
+
+This also fixes bogus usage of malloc/free rather than kmalloc/kfree
+in initramfs.c.
+
+Signed-off-by: Matt Mackall <mpm@selenic.com>
+
+Index: linux-2.6.16.19/arch/alpha/boot/misc.c
+===================================================================
+--- linux-2.6.16.19.orig/arch/alpha/boot/misc.c	2006-06-23 17:06:31.000000000 -0700
++++ linux-2.6.16.19/arch/alpha/boot/misc.c	2006-06-23 17:06:32.000000000 -0700
+@@ -4,8 +4,6 @@
+  * This is a collection of several routines from gzip-1.0.3 
+  * adapted for Linux.
+  *
+- * malloc by Hannu Savolainen 1993 and Matthias Urlichs 1994
+- *
+  * Modified for ARM Linux by Russell King
+  *
+  * Nicolas Pitre <nico@visuaide.com>  1999/04/14 :
+@@ -49,8 +47,6 @@
+ static int  fill_inbuf(void);
+ static void flush_window(void);
+ static void error(char *m);
+-static void gzip_mark(void **);
+-static void gzip_release(void **);
+ 
+ static char *input_data;
+ static int  input_data_size;
+@@ -59,12 +55,6 @@
+ static ulg output_ptr;
+ static ulg bytes_out;
+ 
+-static void *malloc(int size);
+-static void free(void *where);
+-static void error(char *m);
+-static void gzip_mark(void **);
+-static void gzip_release(void **);
+-
+ extern int end;
+ static ulg free_mem_ptr;
+ static ulg free_mem_ptr_end;
+@@ -73,37 +63,6 @@
+ 
+ #include "../../../lib/inflate.c"
+ 
+-static void *malloc(int size)
+-{
+-	void *p;
+-
+-	if (size <0) error("Malloc error");
+-	if (free_mem_ptr <= 0) error("Memory error");
+-
+-	free_mem_ptr = (free_mem_ptr + 3) & ~3;	/* Align */
+-
+-	p = (void *)free_mem_ptr;
+-	free_mem_ptr += size;
+-
+-	if (free_mem_ptr >= free_mem_ptr_end)
+-		error("Out of memory");
+-	return p;
+-}
+-
+-static void free(void *where)
+-{ /* gzip_mark & gzip_release do the free */
+-}
+-
+-static void gzip_mark(void **ptr)
+-{
+-	*ptr = (void *) free_mem_ptr;
+-}
+-
+-static void gzip_release(void **ptr)
+-{
+-	free_mem_ptr = (long) *ptr;
+-}
+-
+ /* ===========================================================================
+  * Fill the input buffer. This is called only when the buffer is empty
+  * and at least one byte is really needed.
+Index: linux-2.6.16.19/arch/arm/boot/compressed/misc.c
+===================================================================
+--- linux-2.6.16.19.orig/arch/arm/boot/compressed/misc.c	2006-06-23 17:06:31.000000000 -0700
++++ linux-2.6.16.19/arch/arm/boot/compressed/misc.c	2006-06-23 17:06:32.000000000 -0700
+@@ -4,8 +4,6 @@
+  * This is a collection of several routines from gzip-1.0.3 
+  * adapted for Linux.
+  *
+- * malloc by Hannu Savolainen 1993 and Matthias Urlichs 1994
+- *
+  * Modified for ARM Linux by Russell King
+  *
+  * Nicolas Pitre <nico@visuaide.com>  1999/04/14 :
+@@ -67,8 +65,6 @@
+ static int  fill_inbuf(void);
+ static void flush_window(void);
+ static void error(char *m);
+-static void gzip_mark(void **);
+-static void gzip_release(void **);
+ 
+ extern char input_data[];
+ extern char input_data_end[];
+@@ -77,12 +73,6 @@
+ static ulg output_ptr;
+ static ulg bytes_out;
+ 
+-static void *malloc(int size);
+-static void free(void *where);
+-static void error(char *m);
+-static void gzip_mark(void **);
+-static void gzip_release(void **);
+-
+ static void putstr(const char *);
+ 
+ extern int end;
+@@ -93,47 +83,8 @@
+ 
+ #include "../../../../lib/inflate.c"
+ 
+-#ifndef STANDALONE_DEBUG
+-static void *malloc(int size)
+-{
+-	void *p;
+-
+-	if (size <0) error("Malloc error");
+-	if (free_mem_ptr <= 0) error("Memory error");
+-
+-	free_mem_ptr = (free_mem_ptr + 3) & ~3;	/* Align */
+-
+-	p = (void *)free_mem_ptr;
+-	free_mem_ptr += size;
+-
+-	if (free_mem_ptr >= free_mem_ptr_end)
+-		error("Out of memory");
+-	return p;
+-}
+-
+-static void free(void *where)
+-{ /* gzip_mark & gzip_release do the free */
+-}
+-
+-static void gzip_mark(void **ptr)
+-{
+-	arch_decomp_wdog();
+-	*ptr = (void *) free_mem_ptr;
+-}
+-
+-static void gzip_release(void **ptr)
+-{
+-	arch_decomp_wdog();
+-	free_mem_ptr = (long) *ptr;
+-}
+-#else
+-static void gzip_mark(void **ptr)
+-{
+-}
+-
+-static void gzip_release(void **ptr)
+-{
+-}
++#ifdef STANDALONE_DEBUG
++#define NO_INFLATE_MALLOC
+ #endif
+ 
+ /* ===========================================================================
+Index: linux-2.6.16.19/arch/arm26/boot/compressed/misc.c
+===================================================================
+--- linux-2.6.16.19.orig/arch/arm26/boot/compressed/misc.c	2006-06-23 17:06:31.000000000 -0700
++++ linux-2.6.16.19/arch/arm26/boot/compressed/misc.c	2006-06-23 17:06:32.000000000 -0700
+@@ -4,8 +4,6 @@
+  * This is a collection of several routines from gzip-1.0.3 
+  * adapted for Linux.
+  *
+- * malloc by Hannu Savolainen 1993 and Matthias Urlichs 1994
+- *
+  * Modified for ARM Linux by Russell King
+  *
+  * Nicolas Pitre <nico@visuaide.com>  1999/04/14 :
+@@ -52,8 +50,6 @@
+ static int  fill_inbuf(void);
+ static void flush_window(void);
+ static void error(char *m);
+-static void gzip_mark(void **);
+-static void gzip_release(void **);
+ 
+ extern char input_data[];
+ extern char input_data_end[];
+@@ -62,12 +58,6 @@
+ static ulg output_ptr;
+ static ulg bytes_out;
+ 
+-static void *malloc(int size);
+-static void free(void *where);
+-static void error(char *m);
+-static void gzip_mark(void **);
+-static void gzip_release(void **);
+-
+ static void puts(const char *);
+ 
+ extern int end;
+@@ -78,47 +68,8 @@
+ 
+ #include "../../../../lib/inflate.c"
+ 
+-#ifndef STANDALONE_DEBUG
+-static void *malloc(int size)
+-{
+-	void *p;
+-
+-	if (size <0) error("Malloc error");
+-	if (free_mem_ptr <= 0) error("Memory error");
+-
+-	free_mem_ptr = (free_mem_ptr + 3) & ~3;	/* Align */
+-
+-	p = (void *)free_mem_ptr;
+-	free_mem_ptr += size;
+-
+-	if (free_mem_ptr >= free_mem_ptr_end)
+-		error("Out of memory");
+-	return p;
+-}
+-
+-static void free(void *where)
+-{ /* gzip_mark & gzip_release do the free */
+-}
+-
+-static void gzip_mark(void **ptr)
+-{
+-	arch_decomp_wdog();
+-	*ptr = (void *) free_mem_ptr;
+-}
+-
+-static void gzip_release(void **ptr)
+-{
+-	arch_decomp_wdog();
+-	free_mem_ptr = (long) *ptr;
+-}
+-#else
+-static void gzip_mark(void **ptr)
+-{
+-}
+-
+-static void gzip_release(void **ptr)
+-{
+-}
++#ifdef STANDALONE_DEBUG
++#define NO_INFLATE_MALLOC
+ #endif
+ 
+ /* ===========================================================================
+Index: linux-2.6.16.19/arch/cris/arch-v10/boot/compressed/misc.c
+===================================================================
+--- linux-2.6.16.19.orig/arch/cris/arch-v10/boot/compressed/misc.c	2006-06-23 17:06:31.000000000 -0700
++++ linux-2.6.16.19/arch/cris/arch-v10/boot/compressed/misc.c	2006-06-23 17:06:32.000000000 -0700
+@@ -6,7 +6,6 @@
+  * This is a collection of several routines from gzip-1.0.3 
+  * adapted for Linux.
+  *
+- * malloc by Hannu Savolainen 1993 and Matthias Urlichs 1994
+  * puts by Nick Holloway 1993, better puts by Martin Mares 1995
+  * adoptation for Linux/CRIS Axis Communications AB, 1999
+  * 
+@@ -51,57 +50,22 @@
+ static int  fill_inbuf(void);
+ static void flush_window(void);
+ static void error(char *m);
+-static void gzip_mark(void **);
+-static void gzip_release(void **);
+ 
+ extern char *input_data;  /* lives in head.S */
+ 
+ static long bytes_out = 0;
+ static uch *output_data;
+ static unsigned long output_ptr = 0;
+- 
+-static void *malloc(int size);
+-static void free(void *where);
+-static void error(char *m);
+-static void gzip_mark(void **);
+-static void gzip_release(void **);
+- 
++
+ static void puts(const char *);
+ 
+ /* the "heap" is put directly after the BSS ends, at end */
+   
+ extern int end;
+ static long free_mem_ptr = (long)&end;
+- 
+-#include "../../../../../lib/inflate.c"
+-
+-static void *malloc(int size)
+-{
+-	void *p;
+-
+-	if (size <0) error("Malloc error");
+-
+-	free_mem_ptr = (free_mem_ptr + 3) & ~3;	/* Align */
+-
+-	p = (void *)free_mem_ptr;
+-	free_mem_ptr += size;
+-
+-	return p;
+-}
+-
+-static void free(void *where)
+-{	/* Don't care */
+-}
++static long free_mem_end_ptr = 0xffffffff;
+ 
+-static void gzip_mark(void **ptr)
+-{
+-	*ptr = (void *) free_mem_ptr;
+-}
+-
+-static void gzip_release(void **ptr)
+-{
+-	free_mem_ptr = (long) *ptr;
+-}
++#include "../../../../../lib/inflate.c"
+ 
+ /* decompressor info and error messages to serial console */
+ 
+Index: linux-2.6.16.19/arch/cris/arch-v32/boot/compressed/misc.c
+===================================================================
+--- linux-2.6.16.19.orig/arch/cris/arch-v32/boot/compressed/misc.c	2006-06-23 17:06:31.000000000 -0700
++++ linux-2.6.16.19/arch/cris/arch-v32/boot/compressed/misc.c	2006-06-23 17:06:32.000000000 -0700
+@@ -6,7 +6,6 @@
+  * This is a collection of several routines from gzip-1.0.3
+  * adapted for Linux.
+  *
+- * malloc by Hannu Savolainen 1993 and Matthias Urlichs 1994
+  * puts by Nick Holloway 1993, better puts by Martin Mares 1995
+  * adoptation for Linux/CRIS Axis Communications AB, 1999
+  *
+@@ -53,8 +52,6 @@
+ static int  fill_inbuf(void);
+ static void flush_window(void);
+ static void error(char *m);
+-static void gzip_mark(void **);
+-static void gzip_release(void **);
+ 
+ extern char *input_data;  /* lives in head.S */
+ 
+@@ -62,49 +59,16 @@
+ static uch *output_data;
+ static unsigned long output_ptr = 0;
+ 
+-static void *malloc(int size);
+-static void free(void *where);
+-static void error(char *m);
+-static void gzip_mark(void **);
+-static void gzip_release(void **);
+-
+ static void puts(const char *);
+ 
+ /* the "heap" is put directly after the BSS ends, at end */
+ 
+ extern int _end;
+ static long free_mem_ptr = (long)&_end;
++static long free_mem_end_ptr = 0xffffffff;
+ 
+ #include "../../../../../lib/inflate.c"
+ 
+-static void *malloc(int size)
+-{
+-	void *p;
+-
+-	if (size <0) error("Malloc error");
+-
+-	free_mem_ptr = (free_mem_ptr + 3) & ~3;	/* Align */
+-
+-	p = (void *)free_mem_ptr;
+-	free_mem_ptr += size;
+-
+-	return p;
+-}
+-
+-static void free(void *where)
+-{	/* Don't care */
+-}
+-
+-static void gzip_mark(void **ptr)
+-{
+-	*ptr = (void *) free_mem_ptr;
+-}
+-
+-static void gzip_release(void **ptr)
+-{
+-	free_mem_ptr = (long) *ptr;
+-}
+-
+ /* decompressor info and error messages to serial console */
+ 
+ static inline void
+Index: linux-2.6.16.19/arch/i386/boot/compressed/misc.c
+===================================================================
+--- linux-2.6.16.19.orig/arch/i386/boot/compressed/misc.c	2006-06-23 17:06:31.000000000 -0700
++++ linux-2.6.16.19/arch/i386/boot/compressed/misc.c	2006-06-23 17:06:32.000000000 -0700
+@@ -4,7 +4,6 @@
+  * This is a collection of several routines from gzip-1.0.3 
+  * adapted for Linux.
+  *
+- * malloc by Hannu Savolainen 1993 and Matthias Urlichs 1994
+  * puts by Nick Holloway 1993, better puts by Martin Mares 1995
+  * High loaded stuff by Hans Lermen & Werner Almesberger, Feb. 1996
+  */
+@@ -43,9 +42,7 @@
+ static int  fill_inbuf(void);
+ static void flush_window(void);
+ static void error(char *m);
+-static void gzip_mark(void **);
+-static void gzip_release(void **);
+-  
++
+ /*
+  * This is set up by the setup-routine at boot-time
+  */
+@@ -64,9 +61,6 @@
+ static uch *output_data;
+ static unsigned long output_ptr = 0;
+ 
+-static void *malloc(int size);
+-static void free(void *where);
+-
+ static void putstr(const char *);
+ 
+ extern int end;
+@@ -91,38 +85,6 @@
+ 
+ #include "../../../../lib/inflate.c"
+ 
+-static void *malloc(int size)
+-{
+-	void *p;
+-
+-	if (size <0) error("Malloc error");
+-	if (free_mem_ptr <= 0) error("Memory error");
+-
+-	free_mem_ptr = (free_mem_ptr + 3) & ~3;	/* Align */
+-
+-	p = (void *)free_mem_ptr;
+-	free_mem_ptr += size;
+-
+-	if (free_mem_ptr >= free_mem_end_ptr)
+-		error("Out of memory");
+-
+-	return p;
+-}
+-
+-static void free(void *where)
+-{	/* Don't care */
+-}
+-
+-static void gzip_mark(void **ptr)
+-{
+-	*ptr = (void *) free_mem_ptr;
+-}
+-
+-static void gzip_release(void **ptr)
+-{
+-	free_mem_ptr = (long) *ptr;
+-}
+- 
+ static void scroll(void)
+ {
+ 	int i;
+Index: linux-2.6.16.19/arch/m32r/boot/compressed/misc.c
+===================================================================
+--- linux-2.6.16.19.orig/arch/m32r/boot/compressed/misc.c	2006-06-23 17:06:31.000000000 -0700
++++ linux-2.6.16.19/arch/m32r/boot/compressed/misc.c	2006-06-23 17:06:32.000000000 -0700
+@@ -4,8 +4,6 @@
+  * This is a collection of several routines from gzip-1.0.3
+  * adapted for Linux.
+  *
+- * malloc by Hannu Savolainen 1993 and Matthias Urlichs 1994
+- *
+  * Adapted for SH by Stuart Menefy, Aug 1999
+  *
+  * 2003-02-12:	Support M32R by Takeo Takahashi
+@@ -38,8 +36,6 @@
+ static int  fill_inbuf(void);
+ static void flush_window(void);
+ static void error(char *m);
+-static void gzip_mark(void **);
+-static void gzip_release(void **);
+ 
+ static unsigned char *input_data;
+ static int input_len;
+@@ -50,9 +46,6 @@
+ 
+ #include "m32r_sio.c"
+ 
+-static void *malloc(int size);
+-static void free(void *where);
+-
+ static unsigned long free_mem_ptr;
+ static unsigned long free_mem_end_ptr;
+ 
+@@ -60,38 +53,6 @@
+ 
+ #include "../../../../lib/inflate.c"
+ 
+-static void *malloc(int size)
+-{
+-	void *p;
+-
+-	if (size <0) error("Malloc error");
+-	if (free_mem_ptr == 0) error("Memory error");
+-
+-	free_mem_ptr = (free_mem_ptr + 3) & ~3;	/* Align */
+-
+-	p = (void *)free_mem_ptr;
+-	free_mem_ptr += size;
+-
+-	if (free_mem_ptr >= free_mem_end_ptr)
+-		error("Out of memory");
+-
+-	return p;
+-}
+-
+-static void free(void *where)
+-{	/* Don't care */
+-}
+-
+-static void gzip_mark(void **ptr)
+-{
+-	*ptr = (void *) free_mem_ptr;
+-}
+-
+-static void gzip_release(void **ptr)
+-{
+-	free_mem_ptr = (long) *ptr;
+-}
+-
+ /* ===========================================================================
+  * Fill the input buffer. This is called only when the buffer is empty
+  * and at least one byte is really needed.
+Index: linux-2.6.16.19/arch/sh/boot/compressed/misc.c
+===================================================================
+--- linux-2.6.16.19.orig/arch/sh/boot/compressed/misc.c	2006-06-23 17:06:31.000000000 -0700
++++ linux-2.6.16.19/arch/sh/boot/compressed/misc.c	2006-06-23 17:06:32.000000000 -0700
+@@ -4,8 +4,6 @@
+  * This is a collection of several routines from gzip-1.0.3
+  * adapted for Linux.
+  *
+- * malloc by Hannu Savolainen 1993 and Matthias Urlichs 1994
+- *
+  * Adapted for SH by Stuart Menefy, Aug 1999
+  *
+  * Modified to use standard LinuxSH BIOS by Greg Banks 7Jul2000
+@@ -40,8 +38,6 @@
+ static int  fill_inbuf(void);
+ static void flush_window(void);
+ static void error(char *m);
+-static void gzip_mark(void **);
+-static void gzip_release(void **);
+ 
+ extern char input_data[];
+ extern int input_len;
+@@ -50,12 +46,6 @@
+ static uch *output_data;
+ static unsigned long output_ptr = 0;
+ 
+-static void *malloc(int size);
+-static void free(void *where);
+-static void error(char *m);
+-static void gzip_mark(void **);
+-static void gzip_release(void **);
+-
+ int puts(const char *);
+ 
+ extern int _text;		/* Defined in vmlinux.lds.S */
+@@ -67,38 +57,6 @@
+ 
+ #include "../../../../lib/inflate.c"
+ 
+-static void *malloc(int size)
+-{
+-	void *p;
+-
+-	if (size <0) error("Malloc error");
+-	if (free_mem_ptr == 0) error("Memory error");
+-
+-	free_mem_ptr = (free_mem_ptr + 3) & ~3;	/* Align */
+-
+-	p = (void *)free_mem_ptr;
+-	free_mem_ptr += size;
+-
+-	if (free_mem_ptr >= free_mem_end_ptr)
+-		error("Out of memory");
+-
+-	return p;
+-}
+-
+-static void free(void *where)
+-{	/* Don't care */
+-}
+-
+-static void gzip_mark(void **ptr)
+-{
+-	*ptr = (void *) free_mem_ptr;
+-}
+-
+-static void gzip_release(void **ptr)
+-{
+-	free_mem_ptr = (long) *ptr;
+-}
+-
+ #ifdef CONFIG_SH_STANDARD_BIOS
+ size_t strlen(const char *s)
+ {
+Index: linux-2.6.16.19/arch/sh64/boot/compressed/misc.c
+===================================================================
+--- linux-2.6.16.19.orig/arch/sh64/boot/compressed/misc.c	2006-06-23 17:06:31.000000000 -0700
++++ linux-2.6.16.19/arch/sh64/boot/compressed/misc.c	2006-06-23 17:06:32.000000000 -0700
+@@ -4,8 +4,6 @@
+  * This is a collection of several routines from gzip-1.0.3
+  * adapted for Linux.
+  *
+- * malloc by Hannu Savolainen 1993 and Matthias Urlichs 1994
+- *
+  * Adapted for SHmedia from sh by Stuart Menefy, May 2002
+  */
+ 
+@@ -40,8 +38,6 @@
+ static int fill_inbuf(void);
+ static void flush_window(void);
+ static void error(char *m);
+-static void gzip_mark(void **);
+-static void gzip_release(void **);
+ 
+ extern char input_data[];
+ extern int input_len;
+@@ -50,12 +46,6 @@
+ static uch *output_data;
+ static unsigned long output_ptr = 0;
+ 
+-static void *malloc(int size);
+-static void free(void *where);
+-static void error(char *m);
+-static void gzip_mark(void **);
+-static void gzip_release(void **);
+-
+ static void puts(const char *);
+ 
+ extern int _text;		/* Defined in vmlinux.lds.S */
+@@ -67,40 +57,6 @@
+ 
+ #include "../../../../lib/inflate.c"
+ 
+-static void *malloc(int size)
+-{
+-	void *p;
+-
+-	if (size < 0)
+-		error("Malloc error\n");
+-	if (free_mem_ptr == 0)
+-		error("Memory error\n");
+-
+-	free_mem_ptr = (free_mem_ptr + 3) & ~3;	/* Align */
+-
+-	p = (void *) free_mem_ptr;
+-	free_mem_ptr += size;
+-
+-	if (free_mem_ptr >= free_mem_end_ptr)
+-		error("\nOut of memory\n");
+-
+-	return p;
+-}
+-
+-static void free(void *where)
+-{				/* Don't care */
+-}
+-
+-static void gzip_mark(void **ptr)
+-{
+-	*ptr = (void *) free_mem_ptr;
+-}
+-
+-static void gzip_release(void **ptr)
+-{
+-	free_mem_ptr = (long) *ptr;
+-}
+-
+ void puts(const char *s)
+ {
+ }
+Index: linux-2.6.16.19/arch/x86_64/boot/compressed/misc.c
+===================================================================
+--- linux-2.6.16.19.orig/arch/x86_64/boot/compressed/misc.c	2006-06-23 17:06:31.000000000 -0700
++++ linux-2.6.16.19/arch/x86_64/boot/compressed/misc.c	2006-06-23 17:06:32.000000000 -0700
+@@ -4,7 +4,6 @@
+  * This is a collection of several routines from gzip-1.0.3 
+  * adapted for Linux.
+  *
+- * malloc by Hannu Savolainen 1993 and Matthias Urlichs 1994
+  * puts by Nick Holloway 1993, better puts by Martin Mares 1995
+  * High loaded stuff by Hans Lermen & Werner Almesberger, Feb. 1996
+  */
+@@ -37,9 +36,7 @@
+ static int  fill_inbuf(void);
+ static void flush_window(void);
+ static void error(char *m);
+-static void gzip_mark(void **);
+-static void gzip_release(void **);
+-  
++
+ /*
+  * This is set up by the setup-routine at boot-time
+  */
+@@ -58,9 +55,6 @@
+ static uch *output_data;
+ static unsigned long output_ptr = 0;
+ 
+-static void *malloc(int size);
+-static void free(void *where);
+-
+ static void putstr(const char *);
+ 
+ extern int end;
+@@ -81,38 +75,6 @@
+ 
+ #include "../../../../lib/inflate.c"
+ 
+-static void *malloc(int size)
+-{
+-	void *p;
+-
+-	if (size <0) error("Malloc error");
+-	if (free_mem_ptr <= 0) error("Memory error");
+-
+-	free_mem_ptr = (free_mem_ptr + 3) & ~3;	/* Align */
+-
+-	p = (void *)free_mem_ptr;
+-	free_mem_ptr += size;
+-
+-	if (free_mem_ptr >= free_mem_end_ptr)
+-		error("Out of memory");
+-
+-	return p;
+-}
+-
+-static void free(void *where)
+-{	/* Don't care */
+-}
+-
+-static void gzip_mark(void **ptr)
+-{
+-	*ptr = (void *) free_mem_ptr;
+-}
+-
+-static void gzip_release(void **ptr)
+-{
+-	free_mem_ptr = (long) *ptr;
+-}
+- 
+ static void scroll(void)
+ {
+ 	int i;
+Index: linux-2.6.16.19/init/do_mounts_rd.c
+===================================================================
+--- linux-2.6.16.19.orig/init/do_mounts_rd.c	2006-06-23 17:06:31.000000000 -0700
++++ linux-2.6.16.19/init/do_mounts_rd.c	2006-06-23 17:06:32.000000000 -0700
+@@ -298,32 +298,11 @@
+ 
+ static int  __init fill_inbuf(void);
+ static void __init flush_window(void);
+-static void __init *malloc(size_t size);
+-static void __init free(void *where);
+ static void __init error(char *m);
+-static void __init gzip_mark(void **);
+-static void __init gzip_release(void **);
+ 
+-#include "../lib/inflate.c"
+-
+-static void __init *malloc(size_t size)
+-{
+-	return kmalloc(size, GFP_KERNEL);
+-}
+-
+-static void __init free(void *where)
+-{
+-	kfree(where);
+-}
+-
+-static void __init gzip_mark(void **ptr)
+-{
+-}
+-
+-static void __init gzip_release(void **ptr)
+-{
+-}
++#define NO_INFLATE_MALLOC
+ 
++#include "../lib/inflate.c"
+ 
+ /* ===========================================================================
+  * Fill the input buffer. This is called only when the buffer is empty
+Index: linux-2.6.16.19/init/initramfs.c
+===================================================================
+--- linux-2.6.16.19.orig/init/initramfs.c	2006-06-23 17:06:31.000000000 -0700
++++ linux-2.6.16.19/init/initramfs.c	2006-06-23 17:06:32.000000000 -0700
+@@ -14,16 +14,6 @@
+ 		message = x;
+ }
+ 
+-static void __init *malloc(size_t size)
+-{
+-	return kmalloc(size, GFP_KERNEL);
+-}
+-
+-static void __init free(void *where)
+-{
+-	kfree(where);
+-}
+-
+ /* link hash */
+ 
+ static __initdata struct hash {
+@@ -51,7 +41,7 @@
+ 			continue;
+ 		return (*p)->name;
+ 	}
+-	q = (struct hash *)malloc(sizeof(struct hash));
++	q = kmalloc(sizeof(struct hash), GFP_KERNEL);
+ 	if (!q)
+ 		panic("can't allocate link hash entry");
+ 	q->ino = ino;
+@@ -70,7 +60,7 @@
+ 		while (*p) {
+ 			q = *p;
+ 			*p = q->next;
+-			free(q);
++			kfree(q);
+ 		}
+ 	}
+ }
+@@ -364,18 +354,10 @@
+ 
+ static void __init flush_window(void);
+ static void __init error(char *m);
+-static void __init gzip_mark(void **);
+-static void __init gzip_release(void **);
+ 
+-#include "../lib/inflate.c"
++#define NO_INFLATE_MALLOC
+ 
+-static void __init gzip_mark(void **ptr)
+-{
+-}
+-
+-static void __init gzip_release(void **ptr)
+-{
+-}
++#include "../lib/inflate.c"
+ 
+ /* ===========================================================================
+  * Write the output window window[0..outcnt-1] and update crc and bytes_out.
+@@ -402,10 +384,10 @@
+ {
+ 	int written;
+ 	dry_run = check_only;
+-	header_buf = malloc(110);
+-	symlink_buf = malloc(PATH_MAX + N_ALIGN(PATH_MAX) + 1);
+-	name_buf = malloc(N_ALIGN(PATH_MAX));
+-	window = malloc(WSIZE);
++	header_buf = kmalloc(110, GFP_KERNEL);
++	symlink_buf = kmalloc(PATH_MAX + N_ALIGN(PATH_MAX) + 1, GFP_KERNEL);
++	name_buf = kmalloc(N_ALIGN(PATH_MAX), GFP_KERNEL);
++	window = kmalloc(WSIZE, GFP_KERNEL);
+ 	if (!window || !header_buf || !symlink_buf || !name_buf)
+ 		panic("can't allocate buffers");
+ 	state = Start;
+@@ -441,10 +423,10 @@
+ 		buf += inptr;
+ 		len -= inptr;
+ 	}
+-	free(window);
+-	free(name_buf);
+-	free(symlink_buf);
+-	free(header_buf);
++	kfree(window);
++	kfree(name_buf);
++	kfree(symlink_buf);
++	kfree(header_buf);
+ 	return message;
+ }
+ 
+Index: linux-2.6.16.19/lib/inflate.c
+===================================================================
+--- linux-2.6.16.19.orig/lib/inflate.c	2006-06-23 17:06:31.000000000 -0700
++++ linux-2.6.16.19/lib/inflate.c	2006-06-23 17:06:32.000000000 -0700
+@@ -109,6 +109,46 @@
+ 
+ #include <asm/types.h>
+ 
++#ifndef NO_INFLATE_MALLOC
++/* A trivial malloc implementation, adapted from
++ *  malloc by Hannu Savolainen 1993 and Matthias Urlichs 1994
++ */
++
++static unsigned long malloc_ptr;
++static int malloc_count;
++
++static void *malloc(int size)
++{
++	void *p;
++
++	if (size <0)
++		error("Malloc error");
++	if (!malloc_ptr)
++		malloc_ptr = free_mem_ptr;
++
++	malloc_ptr = (malloc_ptr + 3) & ~3;	/* Align */
++
++	p = (void *)malloc_ptr;
++	malloc_ptr += size;
++
++	if (malloc_ptr >= free_mem_end_ptr)
++		error("Out of memory");
++
++	malloc_count++;
++	return p;
++}
++
++static void free(void *where)
++{
++	malloc_count--;
++	if (!malloc_count)
++		malloc_ptr = free_mem_ptr;
++}
++#else
++#define malloc(a) kmalloc(a, GFP_KERNEL)
++#define free(a) kfree(a)
++#endif
++
+ static u32 crc_32_tab[256];
+ static u32 crc;		/* dummy var until users get cleaned up */
+ #define CRCPOLY_LE 0xedb88320
+@@ -897,16 +937,12 @@
+ {
+ 	int e;			/* last block flag */
+ 	int r;			/* result code */
+-	void *ptr;
+ 
+ 	/* decompress until the last block */
+ 	do {
+-		gzip_mark(&ptr);
+-		if ((r = inflate_block(io, &e))) {
+-			gzip_release(&ptr);
++		r = inflate_block(io, &e);
++		if (r)
+ 			return r;
+-		}
+-		gzip_release(&ptr);
+ 	} while (!e);
+ 
+ 	popbytes(io);
Index: packages/kernel/patches/tiny_m57sli/tiny/inflate-crc.patch
===================================================================
--- packages/kernel/patches/tiny_m57sli/tiny/inflate-crc.patch	(revision 0)
+++ packages/kernel/patches/tiny_m57sli/tiny/inflate-crc.patch	(revision 0)
@@ -0,0 +1,138 @@
+
+inflate  cleanup CRC calculation
+
+Move CRC calculation into inflate code
+Cleanup table calculation code
+
+
+ tiny-mpm/lib/inflate.c |   65 ++++++++++++++++---------------------------------
+ 1 files changed, 22 insertions(+), 43 deletions(-)
+
+Index: linux-2.6.16.19/lib/inflate.c
+===================================================================
+--- linux-2.6.16.19.orig/lib/inflate.c	2006-06-23 17:06:29.000000000 -0700
++++ linux-2.6.16.19/lib/inflate.c	2006-06-23 17:06:31.000000000 -0700
+@@ -109,6 +109,10 @@
+ 
+ #include <asm/types.h>
+ 
++static u32 crc_32_tab[256];
++static u32 crc;		/* dummy var until users get cleaned up */
++#define CRCPOLY_LE 0xedb88320
++
+ /* Huffman code lookup table entry--this entry is four bytes for machines
+    that have 16-bit pointers (e.g. PC's in the small or medium model).
+    Valid extra bits are 0..13.  e == 15 is EOB (end of block), e == 16
+@@ -128,7 +132,7 @@
+ struct iostate {
+ 	u8 *window;
+ 	int opos, osize, bits;
+-	u32 buf;
++	u32 buf, crc;
+ };
+ 
+ /* Function prototypes */
+@@ -154,6 +158,12 @@
+ 
+ static void flush_output(struct iostate *io)
+ {
++	int i;
++
++	for (i = 0; i < io->opos; i++)
++		io->crc = crc_32_tab[(io->window[i] ^ (int)io->crc) & 0xff]
++			^ (io->crc >> 8);
++
+ 	outcnt = io->opos;
+ 	flush_window();
+ 	io->opos = 0;
+@@ -908,47 +918,16 @@
+  *
+  **********************************************************************/
+ 
+-static u32 crc_32_tab[256];
+-static u32 crc;		/* initialized in makecrc() so it'll reside in bss */
+-#define CRC_VALUE (crc ^ 0xffffffffUL)
+-
+-/*
+- * Code to compute the CRC-32 table. Borrowed from
+- * gzip-1.0.3/makecrc.c.
+- * Not copyrighted 1990 Mark Adler
+- */
+-
+ static void INIT makecrc(void)
+ {
++	unsigned i, j;
++	u32 c = 1;
+ 
+-	unsigned long c;	/* crc shift register */
+-	unsigned long e;	/* polynomial exclusive-or pattern */
+-	int i;			/* counter for all possible eight bit values */
+-	int k;			/* byte being shifted into crc apparatus */
+-
+-	/* terms of polynomial defining this crc (except x^32): */
+-	static const int p[] =
+-	    { 0, 1, 2, 4, 5, 7, 8, 10, 11, 12, 16, 22, 23, 26 };
+-
+-	/* Make exclusive-or pattern from polynomial */
+-	e = 0;
+-	for (i = 0; i < sizeof(p) / sizeof(int); i++)
+-		e |= 1L << (31 - p[i]);
+-
+-	crc_32_tab[0] = 0;
+-
+-	for (i = 1; i < 256; i++) {
+-		c = 0;
+-		for (k = i | 256; k != 1; k >>= 1) {
+-			c = c & 1 ? (c >> 1) ^ e : c >> 1;
+-			if (k & 1)
+-				c ^= e;
+-		}
+-		crc_32_tab[i] = c;
++	for (i = 128; i; i >>= 1) {
++		c = (c >> 1) ^ ((c & 1) ? CRCPOLY_LE : 0);
++		for (j = 0; j < 256; j += 2 * i)
++			crc_32_tab[i + j] = c ^ crc_32_tab[j];
+ 	}
+-
+-	/* this is initialized here so this code could reside in ROM */
+-	crc = 0xffffffffUL;	/* shift register contents */
+ }
+ 
+ /* gzip flag byte */
+@@ -968,14 +947,15 @@
+ 	u8 flags;
+ 	unsigned char magic[2];	/* magic header */
+ 	char method;
+-	u32 orig_crc = 0;	/* original crc */
+-	u32 orig_len = 0;	/* original uncompressed length */
++	u32 orig_crc;
++	u32 orig_len;
+ 	int res;
+ 	struct iostate io;
+ 
+ 	io.window = window;
+ 	io.osize = WSIZE;
+ 	io.opos = io.bits = io.buf = 0;
++	io.crc = 0xffffffffUL;
+ 
+ 	magic[0] = get_byte();
+ 	magic[1] = get_byte();
+@@ -1053,8 +1033,7 @@
+ 		return -1;
+ 	}
+ 
+-	/* Get the crc and original length */
+-	/* crc32  (see algorithm.doc)
++	/* Get the crc and original length
+ 	 * uncompressed input size modulo 2^32
+ 	 */
+ 	orig_crc = (u32)get_byte();
+@@ -1068,7 +1047,7 @@
+ 	orig_len |= (u32)get_byte() << 24;
+ 
+ 	/* Validate decompression */
+-	if (orig_crc != CRC_VALUE) {
++	if (orig_crc != ~io.crc) {
+ 		error("crc error");
+ 		return -1;
+ 	}
Index: packages/kernel/patches/tiny_m57sli/tiny/slob-accounting.patch
===================================================================
--- packages/kernel/patches/tiny_m57sli/tiny/slob-accounting.patch	(revision 0)
+++ packages/kernel/patches/tiny_m57sli/tiny/slob-accounting.patch	(revision 0)
@@ -0,0 +1,159 @@
+
+kmalloc accounting support for slob
+
+
+
+ tiny-mpm/mm/kmallocacct.c |   10 ++++++++++
+ tiny-mpm/mm/slob.c        |   46 ++++++++++++++++++++++++++++++++++++++++------
+ 2 files changed, 50 insertions(+), 6 deletions(-)
+
+Index: new-linux/mm/slob.c
+===================================================================
+--- new-linux.orig/mm/slob.c	2007-04-27 10:51:05.000000000 -0700
++++ new-linux/mm/slob.c	2007-04-30 17:05:11.000000000 -0700
+@@ -45,6 +45,9 @@
+ #define SLOB_UNIT sizeof(slob_t)
+ #define SLOB_UNITS(size) (((size) + SLOB_UNIT - 1)/SLOB_UNIT)
+ #define SLOB_ALIGN L1_CACHE_BYTES
++#define SLOB_ACCT(size) ALIGN(size, SLOB_UNIT)
++#define SLOB_ACCT_S(obj) SLOB_ACCT(sizeof(obj))
++#define PAGE_ACCT(order) (PAGE_SIZE<<order)
+ 
+ struct bigblock {
+ 	int order;
+@@ -58,6 +61,16 @@
+ static bigblock_t *bigblocks;
+ static DEFINE_SPINLOCK(slob_lock);
+ static DEFINE_SPINLOCK(block_lock);
++#ifdef CONFIG_KMALLOC_ACCOUNTING
++int slob_total, slob_req;
++
++static void slob_acct(int real, int req) {
++	slob_total += real;
++	slob_req += req;
++}
++#else
++#define slob_acct(a, b)
++#endif
+ 
+ static void slob_free(void *b, int size);
+ static void slob_timer_cbk(void);
+@@ -96,6 +109,7 @@
+ 			}
+ 
+ 			slobfree = prev;
++			slob_acct(0, units * SLOB_UNIT);
+ 			spin_unlock_irqrestore(&slob_lock, flags);
+ 			return cur;
+ 		}
+@@ -109,6 +123,7 @@
+ 			if (!cur)
+ 				return 0;
+ 
++			slob_acct(PAGE_SIZE, PAGE_SIZE);
+ 			slob_free(cur, PAGE_SIZE);
+ 			spin_lock_irqsave(&slob_lock, flags);
+ 			cur = slobfree;
+@@ -129,6 +144,7 @@
+ 
+ 	/* Find reinsertion point */
+ 	spin_lock_irqsave(&slob_lock, flags);
++	slob_acct(0, b->units * -SLOB_UNIT);
+ 	for (cur = slobfree; !(b > cur && b < cur->next); cur = cur->next)
+ 		if (cur >= cur->next && (b > cur || b < cur->next))
+ 			break;
+@@ -167,6 +183,7 @@
+ 
+ 	if (size < PAGE_SIZE - SLOB_UNIT) {
+ 		m = slob_alloc(size + SLOB_UNIT, gfp, 0);
++		kmalloc_account(m, SLOB_ACCT(size + SLOB_UNIT), size);
+ 		return m ? (void *)(m + 1) : 0;
+ 	}
+ 
+@@ -182,6 +199,8 @@
+ 		bb->next = bigblocks;
+ 		bigblocks = bb;
+ 		spin_unlock_irqrestore(&block_lock, flags);
++		kmalloc_account(bb->pages, PAGE_ACCT(bb->order)
++				+SLOB_ACCT_S(bigblock_t), size);
+ 		return bb->pages;
+ 	}
+ 
+@@ -205,6 +224,8 @@
+ 			if (bb->pages == block) {
+ 				*last = bb->next;
+ 				spin_unlock_irqrestore(&block_lock, flags);
++				kfree_account(block, PAGE_ACCT(bb->order) +
++					      SLOB_ACCT_S(bigblock_t));
+ 				free_pages((unsigned long)block, bb->order);
+ 				slob_free(bb, sizeof(bigblock_t));
+ 				return;
+@@ -213,6 +234,7 @@
+ 		spin_unlock_irqrestore(&block_lock, flags);
+ 	}
+ 
++	kfree_account((slob_t *)block-1, ksize(block));
+ 	slob_free((slob_t *)block - 1, 0);
+ 	return;
+ }
+@@ -255,6 +277,7 @@
+ 	struct kmem_cache *c;
+ 
+ 	c = slob_alloc(sizeof(struct kmem_cache), flags, 0);
++	kmalloc_account(c, SLOB_ACCT_S(kmem_cache_t), sizeof(kmem_cache_t));
+ 
+ 	if (c) {
+ 		c->name = name;
+@@ -274,6 +297,7 @@
+ void kmem_cache_destroy(struct kmem_cache *c)
+ {
+ 	slob_free(c, sizeof(struct kmem_cache));
++	kfree_account(c, SLOT_ACCT_S(kmem_cache_t));
+ }
+ EXPORT_SYMBOL(kmem_cache_destroy);
+ 
+@@ -281,11 +305,13 @@
+ {
+ 	void *b;
+ 
+-	if (c->size < PAGE_SIZE)
++	if (c->size < PAGE_SIZE) {
+ 		b = slob_alloc(c->size, flags, c->align);
+-	else
++		kmalloc_account(b, SLOB_ACCT(c->size), c->size);
++	} else {
+ 		b = (void *)__get_free_pages(flags, find_order(c->size));
+-
++		kmalloc_account(b, PAGE_ACCT(find_order(c->size)), c->size);
++	}
+ 	if (c->ctor)
+ 		c->ctor(b, c, SLAB_CTOR_CONSTRUCTOR);
+ 
+@@ -308,10 +334,13 @@
+ 	if (c->dtor)
+ 		c->dtor(b, c, 0);
+ 
+-	if (c->size < PAGE_SIZE)
++	if (c->size < PAGE_SIZE) {
+ 		slob_free(b, c->size);
+-	else
++		kfree_account(b, SLOB_ACCT(c->size));
++	} else {
+ 		free_pages((unsigned long)b, find_order(c->size));
++		kfree_account(b, PAGE_ACCT(find_order(c->size)));
++	}
+ }
+ EXPORT_SYMBOL(kmem_cache_free);
+ 
+@@ -350,8 +379,10 @@
+ {
+ 	void *p = slob_alloc(PAGE_SIZE, 0, PAGE_SIZE-1);
+ 
+-	if (p)
++	if (p) {
+ 		free_page((unsigned long)p);
++		slob_acct(-PAGE_SIZE, -PAGE_SIZE);
++	}
+ 
+ 	mod_timer(&slob_timer, jiffies + HZ);
+ }
Index: packages/kernel/patches/tiny_m57sli/tiny/serial-pci.patch
===================================================================
--- packages/kernel/patches/tiny_m57sli/tiny/serial-pci.patch	(revision 0)
+++ packages/kernel/patches/tiny_m57sli/tiny/serial-pci.patch	(revision 0)
@@ -0,0 +1,41 @@
+
+Configurable support for PCI serial devices
+
+
+ tiny-mpm/drivers/serial/Makefile |    2 +-
+ tiny-mpm/init/Kconfig            |    9 +++++++++
+ 2 files changed, 10 insertions(+), 1 deletion(-)
+
+Index: linux-2.6.16.19/drivers/serial/Makefile
+===================================================================
+--- linux-2.6.16.19.orig/drivers/serial/Makefile	2006-06-23 17:05:11.000000000 -0700
++++ linux-2.6.16.19/drivers/serial/Makefile	2006-06-23 17:06:26.000000000 -0700
+@@ -8,7 +8,7 @@
+ serial-8250-$(CONFIG_SERIAL_8250_ACPI) += 8250_acpi.o
+ serial-8250-$(CONFIG_PNP) += 8250_pnp.o
+ serial-8250-$(CONFIG_GSC) += 8250_gsc.o
+-serial-8250-$(CONFIG_PCI) += 8250_pci.o
++serial-8250-$(CONFIG_SERIAL_PCI) += 8250_pci.o
+ serial-8250-$(CONFIG_HP300) += 8250_hp300.o
+ 
+ obj-$(CONFIG_SERIAL_CORE) += serial_core.o
+Index: linux-2.6.16.19/init/Kconfig
+===================================================================
+--- linux-2.6.16.19.orig/init/Kconfig	2006-06-23 17:06:26.000000000 -0700
++++ linux-2.6.16.19/init/Kconfig	2006-06-23 17:06:26.000000000 -0700
+@@ -600,6 +600,15 @@
+ 	help
+ 	  This enables support for the Simple Bootflag Specification.
+ 
++config SERIAL_PCI
++	depends PCI && SERIAL_8250
++	default y
++	bool "Enable standard PCI serial support" if EMBEDDED
++	help
++	  This builds standard PCI serial support. You may be able to disable
++          this feature if you are only need legacy serial support.
++	  Saves about 9K.
++
+ endmenu		# General setup
+ 
+ config CRC32_CALC
Index: packages/kernel/patches/tiny_m57sli/tiny/deprecate-inline-kconfig-move.patch
===================================================================
--- packages/kernel/patches/tiny_m57sli/tiny/deprecate-inline-kconfig-move.patch	(revision 0)
+++ packages/kernel/patches/tiny_m57sli/tiny/deprecate-inline-kconfig-move.patch	(revision 0)
@@ -0,0 +1,77 @@
+   This paches moves isolated configrations that should be place in
+   EMBEDDED menu.
+
+---
+ init/Kconfig |   52 	26 +	26 -	0 !
+ 1 file changed, 26 insertions(+), 26 deletions(-)
+
+Index: linux-2.6.16-ALP-BRANCH_SS-mips.git/init/Kconfig
+===================================================================
+--- linux-2.6.16-ALP-BRANCH_SS-mips.git.orig/init/Kconfig
++++ linux-2.6.16-ALP-BRANCH_SS-mips.git/init/Kconfig
+@@ -216,32 +216,6 @@ config CPUSETS
+ 
+ source "usr/Kconfig"
+ 
+-config UID16
+-	bool "Enable 16-bit UID system calls" if EMBEDDED
+-	depends on ARM || CRIS || FRV || H8300 || X86_32 || M68K || (S390 && !64BIT) || SUPERH || SPARC32 || (SPARC64 && SPARC32_COMPAT) || UML || (X86_64 && IA32_EMULATION)
+-	default y
+-	help
+-	  This enables the legacy 16-bit UID syscall wrappers.
+-
+-config VM86
+-	depends X86
+-	default y
+-	bool "Enable VM86 support" if EMBEDDED
+-	help
+-          This option is required by programs like DOSEMU to run 16-bit legacy
+-	  code on X86 processors. It also may be needed by software like
+-          XFree86 to initialize some video cards via BIOS. Disabling this
+-          option saves about 6k.
+-
+-config MEASURE_INLINES
+-	default n
+-	bool "Enable inline measurement" if EMBEDDED
+-	help
+-          This marks inline functions deprecated, causing a kernel compile
+-          to spew information about the location of each compilation.
+-          With some post-processing, this can be used to generate counts
+-          and footprints. Say N unless you want a very noisy build.
+-
+ config CC_OPTIMIZE_FOR_SIZE
+ 	bool "Optimize for size (Look out for broken compilers!)"
+ 	default y
+@@ -291,6 +265,32 @@ menuconfig EMBEDDED
+           environments which can tolerate a "non-standard" kernel.
+           Only use this if you really know what you are doing.
+ 
++config UID16
++	bool "Enable 16-bit UID system calls" if EMBEDDED
++	depends on ARM || CRIS || FRV || H8300 || X86_32 || M68K || (S390 && !64BIT) || SUPERH || SPARC32 || (SPARC64 && SPARC32_COMPAT) || UML || (X86_64 && IA32_EMULATION)
++	default y
++	help
++	  This enables the legacy 16-bit UID syscall wrappers.
++
++config VM86
++	depends X86
++	default y
++	bool "Enable VM86 support" if EMBEDDED
++	help
++          This option is required by programs like DOSEMU to run 16-bit legacy
++	  code on X86 processors. It also may be needed by software like
++          XFree86 to initialize some video cards via BIOS. Disabling this
++          option saves about 6k.
++
++config MEASURE_INLINES
++	default n
++	bool "Enable inline measurement" if EMBEDDED
++	help
++          This marks inline functions deprecated, causing a kernel compile
++          to spew information about the location of each compilation.
++          With some post-processing, this can be used to generate counts
++          and footprints. Say N unless you want a very noisy build.
++
+ config KALLSYMS
+ 	 bool "Load all symbols for debugging/kksymoops" if EMBEDDED
+ 	 default y
Index: packages/kernel/patches/tiny_m57sli/tiny/use-funit-at-a-time.patch
===================================================================
--- packages/kernel/patches/tiny_m57sli/tiny/use-funit-at-a-time.patch	(revision 0)
+++ packages/kernel/patches/tiny_m57sli/tiny/use-funit-at-a-time.patch	(revision 0)
@@ -0,0 +1,65 @@
+
+enable -funit-at-a-time for gcc3.4
+
+From: Andi Kleen <ak@muc.de>
+
+The upcomming gcc 3.4 has a new compilation mode called unit-at-a-time.
+What it does is to first load the whole file into memory and then generate
+the output. This allows it to use a better inlining strategy, drop unused
+static functions and use -mregparm automatically for static functions.
+
+It does not seem to compile significantly slower.
+
+This is also available in some of the 3.3 based "hammer branch"
+compilers used in distributions (at least in SuSE and Mandrake)
+
+Some tests show impressive .text shrinkage from unit-at-a-time.
+
+e.g. here is the same kernel compiled with -fno-unit-at-a-time and
+-funit-at-a-time with a gcc 3.4 snapshot. The gains are really
+impressive:
+
+   text    data     bss     dec     hex filename
+4129346  708629  207240 5045215  4cfbdf vmlinux-nounitatatime
+3999250  674853  207208 4881311  4a7b9f vmlinux-unitatatime
+
+.text shrinks by over 130KB!. And .data shrinks too. 
+
+At first look the numbers look nearly too good to be true, but they have been 
+verified with several configurations and seem to be real. It looks like
+we have a lot of stupid inlines or dead functions. I'm really not 
+sure why it is that much better. But it's hard to argue with hard
+numbers.
+
+[A bloat-o-meter comparision between the two vmlinuxes can be found in 
+http://www.firstfloor.org/~andi/unit-vs-no-unit.gz . It doesn't show
+any obvious candidates unfortunately, just lots of small changes]
+
+With the gcc 3.3-hammer from SuSE 9.0 the gains are a bit smaller, but
+still noticeable (>100KB on .text)
+
+Enabling -funit-at-a-time requires at least one kernel patch (the
+"noinline" patch I sent earlier), on top of the no always inline patch required
+to make it compile on gcc 3.4.
+
+This patch enables -funit-at-a-time by default when it is available.
+
+---
+ Makefile |    4 	4 +	0 -	0 !
+ 1 files changed, 4 insertions(+)
+
+Index: linux-2.6.22.1/Makefile
+===================================================================
+--- linux-2.6.22.1.orig/Makefile
++++ linux-2.6.22.1/Makefile
+@@ -507,6 +507,10 @@ CFLAGS          += $(call cc-option, -fn
+ NOSTDINC_FLAGS += -nostdinc -isystem $(shell $(CC) -print-file-name=include)
+ CHECKFLAGS     += $(NOSTDINC_FLAGS)
+ 
++# Enable unit-at-a-time mode when possible. It shrinks the
++# kernel considerably.
++CFLAGS += $(call cc-option,-funit-at-a-time,)
++
+ # warn about C99 declaration after statement
+ CFLAGS += $(call cc-option,-Wdeclaration-after-statement,)
+ 
Index: packages/kernel/patches/tiny_m57sli/tiny/inflate-flush.patch
===================================================================
--- packages/kernel/patches/tiny_m57sli/tiny/inflate-flush.patch	(revision 0)
+++ packages/kernel/patches/tiny_m57sli/tiny/inflate-flush.patch	(revision 0)
@@ -0,0 +1,1031 @@
+inflate: internalize most of the output window handling
+
+This moves the inflate window definition and handling into
+lib/inflate.c and rather than calling a global flush_window function,
+now calls a flush callback. The callback in most callers is also
+greatly simplified.
+
+Signed-off-by: Matt Mackall <mpm@selenic.com>
+
+Index: alp-linux/arch/alpha/boot/misc.c
+===================================================================
+--- alp-linux.orig/arch/alpha/boot/misc.c	2006-07-12 14:22:54.000000000 -0700
++++ alp-linux/arch/alpha/boot/misc.c	2006-07-12 15:20:35.000000000 -0700
+@@ -32,21 +32,12 @@
+ typedef unsigned short ush;
+ typedef unsigned long  ulg;
+ 
+-#define WSIZE 0x8000		/* Window size must be at least 32k, */
+-				/* and a power of two */
+-
+-static uch *window;		/* Sliding window buffer */
+-static unsigned outcnt;		/* bytes in output buffer */
+-
+-static void flush_window(void);
+ static void error(char *m);
+ 
+ static char *input_data;
+ static int  input_data_size;
+ 
+ static uch *output_data;
+-static ulg output_ptr;
+-static ulg bytes_out;
+ 
+ extern int end;
+ static ulg free_mem_ptr;
+@@ -56,24 +47,11 @@
+ 
+ #include "../../../lib/inflate.c"
+ 
+-/* ===========================================================================
+- * Write the output window window[0..outcnt-1] and update bytes_out.
+- * (Used for the decompressed data only.)
+- */
+-void flush_window(void)
++/* flush gunzip output window */
++static void flush_window(const u8 *buf, int len)
+ {
+-	unsigned n;
+-	uch *in, *out, ch;
+-
+-	in = window;
+-	out = &output_data[output_ptr];
+-	for (n = 0; n < outcnt; n++)
+-		ch = *out++ = *in++;
+-
+-	bytes_out += (ulg)outcnt;
+-	output_ptr += (ulg)outcnt;
+-	outcnt = 0;
+-/*	puts("."); */
++	while(len--)
++		*output_data++ = *buf++;
+ }
+ 
+ static void error(char *x)
+@@ -91,6 +69,7 @@
+ 		  size_t ksize,
+ 		  size_t kzsize)
+ {
++	int count;
+ 	output_data		= (uch *)output_start;
+ 	input_data		= (uch *)input_start;
+ 	input_data_size		= kzsize; /* use compressed size */
+@@ -100,11 +79,8 @@
+ 	free_mem_ptr_end	= (ulg)output_start + ksize + 0x200000;
+ 	/* FIXME FIXME FIXME */
+ 
+-	/* put in temp area to reduce initial footprint */
+-	window = malloc(WSIZE);
+-
+ /*	puts("Uncompressing Linux..."); */
+-	gunzip(input_data, input_data_size, NULL);
++	count = gunzip(input_data, input_data_size, NULL, flush_window);
+ /*	puts(" done, booting the kernel.\n"); */
+-	return output_ptr;
++	return count;
+ }
+Index: alp-linux/arch/arm/boot/compressed/misc.c
+===================================================================
+--- alp-linux.orig/arch/arm/boot/compressed/misc.c	2006-07-12 14:22:54.000000000 -0700
++++ alp-linux/arch/arm/boot/compressed/misc.c	2006-07-12 15:22:12.000000000 -0700
+@@ -50,21 +50,12 @@
+ typedef unsigned short ush;
+ typedef unsigned long  ulg;
+ 
+-#define WSIZE 0x8000		/* Window size must be at least 32k, */
+-				/* and a power of two */
+-
+-static uch window[WSIZE];	/* Sliding window buffer */
+-static unsigned outcnt;		/* bytes in output buffer */
+-
+-static void flush_window(void);
+ static void error(char *m);
+ 
+ extern char input_data[];
+ extern char input_data_end[];
+ 
+ static uch *output_data;
+-static ulg output_ptr;
+-static ulg bytes_out;
+ 
+ static void putstr(const char *);
+ 
+@@ -80,23 +71,11 @@
+ #define NO_INFLATE_MALLOC
+ #endif
+ 
+-/* ===========================================================================
+- * Write the output window window[0..outcnt-1] and update bytes_out.
+- * (Used for the decompressed data only.)
+- */
+-void flush_window(void)
++/* flush gunzip output window */
++static void flush_window(const u8 *buf, int len)
+ {
+-	unsigned n;
+-	uch *in, *out, ch;
+-
+-	in = window;
+-	out = &output_data[output_ptr];
+-	for (n = 0; n < outcnt; n++)
+-		ch = *out++ = *in++;
+-
+-	bytes_out += (ulg)outcnt;
+-	output_ptr += (ulg)outcnt;
+-	outcnt = 0;
++	while(len--)
++		*output_data++ = *buf++;
+ 	putstr(".");
+ }
+ 
+@@ -121,6 +100,8 @@
+ decompress_kernel(ulg output_start, ulg free_mem_ptr_p, ulg free_mem_ptr_end_p,
+ 		  int arch_id)
+ {
++	int count;
++
+ 	output_data		= (uch *)output_start;	/* Points to kernel start */
+ 	free_mem_ptr		= free_mem_ptr_p;
+ 	free_mem_ptr_end	= free_mem_ptr_end_p;
+@@ -129,9 +110,9 @@
+ 	arch_decomp_setup();
+ 
+ 	putstr("Uncompressing Linux...");
+-	gunzip(input_data, input_data_end - input_data, NULL);
++	count = gunzip(input_data, input_data_end - input_data, NULL, flush_window);
+ 	putstr(" done, booting the kernel.\n");
+-	return output_ptr;
++	return count;
+ }
+ #else
+ 
+@@ -142,7 +123,7 @@
+ 	output_data = output_buffer;
+ 
+ 	putstr("Uncompressing Linux...");
+-	gunzip(input_data, input_data_end - input_data, NULL);
++	gunzip(input_data, input_data_end - input_data, NULL, flush_window);
+ 	putstr("done.\n");
+ 	return 0;
+ }
+Index: alp-linux/arch/arm26/boot/compressed/misc.c
+===================================================================
+--- alp-linux.orig/arch/arm26/boot/compressed/misc.c	2006-07-12 14:22:54.000000000 -0700
++++ alp-linux/arch/arm26/boot/compressed/misc.c	2006-07-12 15:20:35.000000000 -0700
+@@ -35,21 +35,11 @@
+ typedef unsigned short ush;
+ typedef unsigned long  ulg;
+ 
+-#define WSIZE 0x8000		/* Window size must be at least 32k, */
+-				/* and a power of two */
+-
+-static uch window[WSIZE];	/* Sliding window buffer */
+-static unsigned outcnt;		/* bytes in output buffer */
+-
+-static void flush_window(void);
+ static void error(char *m);
+ 
+ extern char input_data[];
+ extern char input_data_end[];
+-
+ static uch *output_data;
+-static ulg output_ptr;
+-static ulg bytes_out;
+ 
+ static void puts(const char *);
+ 
+@@ -65,23 +55,11 @@
+ #define NO_INFLATE_MALLOC
+ #endif
+ 
+-/* ===========================================================================
+- * Write the output window window[0..outcnt-1] and update bytes_out.
+- * (Used for the decompressed data only.)
+- */
+-void flush_window(void)
++/* flush the gunzip output window */
++static void flush_window(const u8 *buf, int len)
+ {
+-	unsigned n;
+-	uch *in, *out, ch;
+-
+-	in = window;
+-	out = &output_data[output_ptr];
+-	for (n = 0; n < outcnt; n++)
+-		ch = *out++ = *in++;
+-
+-	bytes_out += (ulg)outcnt;
+-	output_ptr += (ulg)outcnt;
+-	outcnt = 0;
++	while (len--)
++		*output_data++ = *buf++;
+ 	puts(".");
+ }
+ 
+@@ -110,7 +88,7 @@
+ 	arch_decomp_setup();
+ 
+ 	puts("Uncompressing Linux...");
+-	gunzip(input_data, input_data_end - input_data, NULL);
++	gunzip(input_data, input_data_end - input_data, NULL, flush_window);
+ 	puts(" done, booting the kernel.\n");
+ 	return output_ptr;
+ }
+@@ -123,7 +101,7 @@
+ 	output_data = output_buffer;
+ 
+ 	puts("Uncompressing Linux...");
+-	gunzip(input_data, input_data_end - input_data, NULL);
++	gunzip(input_data, input_data_end - input_data, NULL, flush_window);
+ 	puts("done.\n");
+ 	return 0;
+ }
+Index: alp-linux/arch/cris/arch-v10/boot/compressed/misc.c
+===================================================================
+--- alp-linux.orig/arch/cris/arch-v10/boot/compressed/misc.c	2006-07-12 14:22:54.000000000 -0700
++++ alp-linux/arch/cris/arch-v10/boot/compressed/misc.c	2006-07-12 15:20:35.000000000 -0700
+@@ -32,21 +32,12 @@
+ typedef unsigned short ush;
+ typedef unsigned long  ulg;
+ 
+-#define WSIZE 0x8000		/* Window size must be at least 32k, */
+-				/* and a power of two */
+-
+-static uch window[WSIZE];    /* Sliding window buffer */
+ unsigned compsize; /* compressed size, used by head.S */
+-static unsigned outcnt = 0;  /* bytes in output buffer */
+ 
+-static void flush_window(void);
+ static void error(char *m);
+ 
+ extern char *input_data;  /* lives in head.S */
+-
+-static long bytes_out = 0;
+ static uch *output_data;
+-static unsigned long output_ptr = 0;
+ 
+ static void puts(const char *);
+ 
+@@ -85,25 +76,11 @@
+ #endif
+ }
+ 
+-/* ===========================================================================
+- * Write the output window window[0..outcnt-1] and update bytes_out.
+- * (Used for the decompressed data only.)
+- */
+-
+-static void
+-flush_window()
++/* flush the gunzip output window */
++static void flush_window(const u8 *buf, int len)
+ {
+-    unsigned n;
+-    uch *in, *out, ch;
+-
+-    in = window;
+-    out = &output_data[output_ptr];
+-    for (n = 0; n < outcnt; n++)
+-	    ch = *out++ = *in++;
+-
+-    bytes_out += (ulg)outcnt;
+-    output_ptr += (ulg)outcnt;
+-    outcnt = 0;
++	while (len--)
++		*output_data++ = *buf++;
+ }
+ 
+ static void
+@@ -163,6 +140,6 @@
+ 	}
+ 
+ 	puts("Uncompressing Linux...\n");
+-	compsize = gunzip(input_data, 0x7fffffff, NULL);
++	compsize = gunzip(input_data, 0x7fffffff, NULL, flush_window);
+ 	puts("Done. Now booting the kernel.\n");
+ }
+Index: alp-linux/arch/cris/arch-v32/boot/compressed/misc.c
+===================================================================
+--- alp-linux.orig/arch/cris/arch-v32/boot/compressed/misc.c	2006-07-12 14:22:54.000000000 -0700
++++ alp-linux/arch/cris/arch-v32/boot/compressed/misc.c	2006-07-12 15:20:35.000000000 -0700
+@@ -34,20 +34,10 @@
+ typedef unsigned short ush;
+ typedef unsigned long  ulg;
+ 
+-#define WSIZE 0x8000		/* Window size must be at least 32k, */
+-				/* and a power of two */
+-
+-static uch window[WSIZE];    /* Sliding window buffer */
+-static unsigned outcnt = 0;  /* bytes in output buffer */
+-
+-static void flush_window(void);
+ static void error(char *m);
+ 
+ extern char *input_data;  /* lives in head.S */
+-
+-static long bytes_out = 0;
+ static uch *output_data;
+-static unsigned long output_ptr = 0;
+ 
+ static void puts(const char *);
+ 
+@@ -98,25 +88,11 @@
+ #endif
+ }
+ 
+-/* ===========================================================================
+- * Write the output window window[0..outcnt-1] and update bytes_out.
+- * (Used for the decompressed data only.)
+- */
+-
+-static void
+-flush_window()
++/* flush the gunzip output window */
++static void flush_window(const u8 *buf, int len)
+ {
+-    unsigned n;
+-    uch *in, *out, ch;
+-
+-    in = window;
+-    out = &output_data[output_ptr];
+-    for (n = 0; n < outcnt; n++)
+-	    ch = *out++ = *in++;
+-
+-    bytes_out += (ulg)outcnt;
+-    output_ptr += (ulg)outcnt;
+-    outcnt = 0;
++	while (len--)
++		*output_data++ = *buf++;
+ }
+ 
+ static void
+@@ -207,6 +183,6 @@
+ 	}
+ 
+ 	puts("Uncompressing Linux...\n");
+-	gunzip(input_data, 0x7fffffff, NULL);
++	gunzip(input_data, 0x7fffffff, NULL, flush_window);
+ 	puts("Done. Now booting the kernel.\n");
+ }
+Index: alp-linux/arch/i386/boot/compressed/misc.c
+===================================================================
+--- alp-linux.orig/arch/i386/boot/compressed/misc.c	2006-07-12 14:22:54.000000000 -0700
++++ alp-linux/arch/i386/boot/compressed/misc.c	2006-07-12 15:20:35.000000000 -0700
+@@ -27,13 +27,6 @@
+ typedef unsigned short ush;
+ typedef unsigned long  ulg;
+ 
+-#define WSIZE 0x8000		/* Window size must be at least 32k, */
+-				/* and a power of two */
+-
+-static uch window[WSIZE];    /* Sliding window buffer */
+-static unsigned outcnt = 0;  /* bytes in output buffer */
+-
+-static void flush_window(void);
+ static void error(char *m);
+ 
+ /*
+@@ -49,10 +42,8 @@
+ 
+ extern char input_data[];
+ extern int input_len;
+-
+-static long bytes_out = 0;
++static long bytes_out;
+ static uch *output_data;
+-static unsigned long output_ptr = 0;
+ 
+ static void putstr(const char *);
+ 
+@@ -126,43 +117,15 @@
+ 	outb_p(0xff & (pos >> 1), vidport+1);
+ }
+ 
+-/* ===========================================================================
+- * Write the output window window[0..outcnt-1] and update bytes_out.
+- * (Used for the decompressed data only.)
+- */
+-static void flush_window_low(void)
+-{
+-    unsigned n;
+-    uch *in, *out, ch;
+-
+-    in = window;
+-    out = &output_data[output_ptr];
+-    for (n = 0; n < outcnt; n++)
+-	    ch = *out++ = *in++;
+-
+-    bytes_out += (ulg)outcnt;
+-    output_ptr += (ulg)outcnt;
+-    outcnt = 0;
+-}
+-
+-static void flush_window_high(void)
++/* flush the gunzip output buffer */
++static void flush_window(const u8 *buf, int len)
+ {
+-    unsigned n;
+-    uch *in,  ch;
+-    in = window;
+-    for (n = 0; n < outcnt; n++) {
+-	ch = *output_data++ = *in++;
+-	if ((ulg)output_data == low_buffer_end) output_data=high_buffer_start;
+-    }
+-
+-    bytes_out += (ulg)outcnt;
+-    outcnt = 0;
+-}
+-
+-static void flush_window(void)
+-{
+-	if (high_loaded) flush_window_high();
+-	else flush_window_low();
++	bytes_out += len;
++	while (len--) {
++		*output_data++ = *buf++;
++		if (high_loaded && (u32)output_data == low_buffer_end)
++			output_data = high_buffer_start;
++	}
+ }
+ 
+ static void error(char *x)
+@@ -254,7 +217,7 @@
+ 	else setup_output_buffer_if_we_run_high(mv);
+ 
+ 	putstr("Uncompressing Linux... ");
+-	gunzip(input_data, input_len, NULL);
++	gunzip(input_data, input_len, NULL, flush_window);
+ 	putstr("Ok, booting the kernel.\n");
+ 	if (high_loaded) close_output_buffer_if_we_run_high(mv);
+ 	return high_loaded;
+Index: alp-linux/arch/m32r/boot/compressed/misc.c
+===================================================================
+--- alp-linux.orig/arch/m32r/boot/compressed/misc.c	2006-07-12 14:22:54.000000000 -0700
++++ alp-linux/arch/m32r/boot/compressed/misc.c	2006-07-12 15:20:35.000000000 -0700
+@@ -21,21 +21,11 @@
+ typedef unsigned short ush;
+ typedef unsigned long  ulg;
+ 
+-#define WSIZE 0x8000		/* Window size must be at least 32k, */
+-				/* and a power of two */
+-
+-static uch window[WSIZE];    /* Sliding window buffer */
+-static unsigned outcnt = 0;  /* bytes in output buffer */
+-
+-static void flush_window(void);
+ static void error(char *m);
+ 
+ static unsigned char *input_data;
+ static int input_len;
+-
+-static long bytes_out = 0;
+ static uch *output_data;
+-static unsigned long output_ptr = 0;
+ 
+ #include "m32r_sio.c"
+ 
+@@ -46,23 +36,11 @@
+ 
+ #include "../../../../lib/inflate.c"
+ 
+-/* ===========================================================================
+- * Write the output window window[0..outcnt-1] and update bytes_out.
+- * (Used for the decompressed data only.)
+- */
+-static void flush_window(void)
++/* flush the gunzip output buffer */
++static void flush_window(const u8 *buf, int len)
+ {
+-    unsigned n;
+-    uch *in, *out, ch;
+-
+-    in = window;
+-    out = &output_data[output_ptr];
+-    for (n = 0; n < outcnt; n++)
+-	    ch = *out++ = *in++;
+-
+-    bytes_out += (ulg)outcnt;
+-    output_ptr += (ulg)outcnt;
+-    outcnt = 0;
++	while (len--)
++		*output_data++ = *buf++;
+ }
+ 
+ static void error(char *x)
+@@ -87,6 +65,6 @@
+ 	input_len = zimage_len;
+ 
+ 	puts("Uncompressing Linux... ");
+-	gunzip(input_data, input_len, NULL);
++	gunzip(input_data, input_len, NULL, flush_window);
+ 	puts("Ok, booting the kernel.\n");
+ }
+Index: alp-linux/arch/sh/boot/compressed/misc.c
+===================================================================
+--- alp-linux.orig/arch/sh/boot/compressed/misc.c	2006-07-12 14:22:54.000000000 -0700
++++ alp-linux/arch/sh/boot/compressed/misc.c	2006-07-12 15:20:35.000000000 -0700
+@@ -23,21 +23,11 @@
+ typedef unsigned short ush;
+ typedef unsigned long  ulg;
+ 
+-#define WSIZE 0x8000		/* Window size must be at least 32k, */
+-				/* and a power of two */
+-
+-static uch window[WSIZE];    /* Sliding window buffer */
+-static unsigned outcnt = 0;  /* bytes in output buffer */
+-
+-static void flush_window(void);
+ static void error(char *m);
+ 
+ extern char input_data[];
+ extern int input_len;
+-
+-static long bytes_out = 0;
+ static uch *output_data;
+-static unsigned long output_ptr = 0;
+ 
+ int puts(const char *);
+ 
+@@ -74,23 +64,11 @@
+ }
+ #endif
+ 
+-/* ===========================================================================
+- * Write the output window window[0..outcnt-1] and update bytes_out.
+- * (Used for the decompressed data only.)
+- */
+-static void flush_window(void)
+-{
+-    unsigned n;
+-    uch *in, *out, ch;
+-
+-    in = window;
+-    out = &output_data[output_ptr];
+-    for (n = 0; n < outcnt; n++)
+-	    ch = *out++ = *in++;
+-
+-    bytes_out += (ulg)outcnt;
+-    output_ptr += (ulg)outcnt;
+-    outcnt = 0;
++/* flush the gunzip output window */
++static void flush_window(const u8 *buf, int len)
++{
++	while (len--)
++		*output_data++ = *buf++;
+ }
+ 
+ static void error(char *x)
+@@ -114,6 +92,6 @@
+ 	free_mem_end_ptr = free_mem_ptr + HEAP_SIZE;
+ 
+ 	puts("Uncompressing Linux... ");
+-	gunzip(input_data, input_len, NULL);
++	gunzip(input_data, input_len, NULL, flush_window);
+ 	puts("Ok, booting the kernel.\n");
+ }
+Index: alp-linux/arch/sh64/boot/compressed/misc.c
+===================================================================
+--- alp-linux.orig/arch/sh64/boot/compressed/misc.c	2006-07-12 14:22:54.000000000 -0700
++++ alp-linux/arch/sh64/boot/compressed/misc.c	2006-07-12 15:20:35.000000000 -0700
+@@ -23,21 +23,11 @@
+ typedef unsigned short ush;
+ typedef unsigned long ulg;
+ 
+-#define WSIZE 0x8000		/* Window size must be at least 32k, */
+-				/* and a power of two */
+-
+-static uch window[WSIZE];	/* Sliding window buffer */
+-static unsigned outcnt = 0;	/* bytes in output buffer */
+-
+-static void flush_window(void);
+ static void error(char *m);
+ 
+ extern char input_data[];
+ extern int input_len;
+-
+-static long bytes_out = 0;
+ static uch *output_data;
+-static unsigned long output_ptr = 0;
+ 
+ static void puts(const char *);
+ 
+@@ -54,23 +44,12 @@
+ {
+ }
+ 
+-/* ===========================================================================
+- * Write the output window window[0..outcnt-1] and update bytes_out.
+- * (Used for the decompressed data only.)
+- */
+-static void flush_window(void)
++/* flush the gunzip window */
++static void flush_window(const u8 *buf, int len)
+ {
+-	unsigned n;
+-	uch *in, *out, ch;
++	while(len--)
++		*output_data++ = *buf++;
+ 
+-	in = window;
+-	out = &output_data[output_ptr];
+-	for (n = 0; n < outcnt; n++)
+-		ch = *out++ = *in++;
+-
+-	bytes_out += (ulg) outcnt;
+-	output_ptr += (ulg) outcnt;
+-	outcnt = 0;
+ 	puts(".");
+ }
+ 
+@@ -95,7 +74,7 @@
+ 
+ 	puts("Uncompressing Linux... ");
+ 	cache_control(CACHE_ENABLE);
+-	gunzip(input_data, input_len, NULL);
++	gunzip(input_data, input_len, NULL, flush_window);
+ 	puts("\n");
+ 
+ #if 0
+Index: alp-linux/arch/x86_64/boot/compressed/misc.c
+===================================================================
+--- alp-linux.orig/arch/x86_64/boot/compressed/misc.c	2006-07-12 14:22:54.000000000 -0700
++++ alp-linux/arch/x86_64/boot/compressed/misc.c	2006-07-12 15:20:35.000000000 -0700
+@@ -20,13 +20,6 @@
+ typedef unsigned short ush;
+ typedef unsigned long  ulg;
+ 
+-#define WSIZE 0x8000		/* Window size must be at least 32k, */
+-				/* and a power of two */
+-
+-static uch window[WSIZE];    /* Sliding window buffer */
+-static unsigned outcnt = 0;  /* bytes in output buffer */
+-
+-static void flush_window(void);
+ static void error(char *m);
+ 
+ /*
+@@ -43,9 +36,8 @@
+ extern unsigned char input_data[];
+ extern int input_len;
+ 
+-static long bytes_out = 0;
++static long bytes_out;
+ static uch *output_data;
+-static unsigned long output_ptr = 0;
+ 
+ static void putstr(const char *);
+ 
+@@ -115,43 +107,15 @@
+ 	outb_p(0xff & (pos >> 1), vidport+1);
+ }
+ 
+-/* ===========================================================================
+- * Write the output window window[0..outcnt-1] and update bytes_out.
+- * (Used for the decompressed data only.)
+- */
+-static void flush_window_low(void)
++/* flush the gunzip output window */
++static void flush_window(const u8 *buf, int len)
+ {
+-    unsigned n;
+-    uch *in, *out, ch;
+-
+-    in = window;
+-    out = &output_data[output_ptr];
+-    for (n = 0; n < outcnt; n++)
+-	    ch = *out++ = *in++;
+-
+-    bytes_out += (ulg)outcnt;
+-    output_ptr += (ulg)outcnt;
+-    outcnt = 0;
+-}
+-
+-static void flush_window_high(void)
+-{
+-    unsigned n;
+-    uch *in,  ch;
+-    in = window;
+-    for (n = 0; n < outcnt; n++) {
+-	ch = *output_data++ = *in++;
+-	if ((ulg)output_data == low_buffer_end) output_data=high_buffer_start;
+-    }
+-
+-    bytes_out += (ulg)outcnt;
+-    outcnt = 0;
+-}
+-
+-static void flush_window(void)
+-{
+-	if (high_loaded) flush_window_high();
+-	else flush_window_low();
++	bytes_out += len;
++	while (len--) {
++		*output_data++ = *buf++;
++		if (high_loaded && output_data == (u8 *)low_buffer_end)
++			output_data = high_buffer_start;
++	}
+ }
+ 
+ static void error(char *x)
+@@ -201,7 +165,7 @@
+ 	mv->high_buffer_start = high_buffer_start;
+ }
+ 
+-void close_output_buffer_if_we_run_high(struct moveparams *mv)
++void close_output_buffer_if_we_run_high(struct moveparams *mv, int bytes_out)
+ {
+ 	if (bytes_out > low_buffer_size) {
+ 		mv->lcount = low_buffer_size;
+@@ -232,7 +196,7 @@
+ 	else setup_output_buffer_if_we_run_high(mv);
+ 
+ 	putstr(".\nDecompressing Linux...");
+-	gunzip(input_data, input_len, NULL);
++	gunzip(input_data, input_len, NULL, flush_window);
+ 	putstr("done.\nBooting the kernel.\n");
+ 	if (high_loaded) close_output_buffer_if_we_run_high(mv);
+ 	return high_loaded;
+Index: alp-linux/init/do_mounts_rd.c
+===================================================================
+--- alp-linux.orig/init/do_mounts_rd.c	2006-07-12 14:22:54.000000000 -0700
++++ alp-linux/init/do_mounts_rd.c	2006-07-12 15:20:35.000000000 -0700
+@@ -291,20 +291,13 @@
+ typedef unsigned long  ulg;
+ 
+ #define INBUFSIZ 4096
+-#define WSIZE 0x8000    /* window size--must be a power of two, and */
+-			/*  at least 32K for zip's deflate method */
+-
+ static uch *inbuf;
+-static uch *window;
+-static unsigned outcnt;  /* bytes in output buffer */
+ static int exit_code;
+ static int unzip_error;
+-static long bytes_out;
+ static int crd_infd, crd_outfd;
+ 
+ #define INIT __init
+ 
+-static void __init flush_window(void);
+ static void __init error(char *m);
+ 
+ #define NO_INFLATE_MALLOC
+@@ -321,27 +314,17 @@
+ 		error("RAMDISK: ran out of compressed data");
+ }
+ 
+-/* ===========================================================================
+- * Write the output window window[0..outcnt-1] and update bytes_out.
+- * (Used for the decompressed data only.)
+- */
+-static void __init flush_window(void)
++/* Write the output window from gunzip */
++static void __init flush_buffer(const u8 *buf, int len)
+ {
+-    unsigned n, written;
+-    uch *in, ch;
+-
+-    written = sys_write(crd_outfd, window, outcnt);
+-    if (written != outcnt && unzip_error == 0) {
+-	printk(KERN_ERR "RAMDISK: incomplete write (%d != %d) %ld\n",
+-	       written, outcnt, bytes_out);
+-	unzip_error = 1;
+-    }
+-    in = window;
+-    for (n = 0; n < outcnt; n++)
+-	    ch = *in++;
++	unsigned written;
+ 
+-    bytes_out += (ulg)outcnt;
+-    outcnt = 0;
++	written = sys_write(crd_outfd, buf, len);
++	if (written != len && unzip_error == 0) {
++		printk(KERN_ERR "RAMDISK: incomplete write (%d != %d)\n",
++		       written, len);
++		unzip_error = 1;
++	}
+ }
+ 
+ static void __init error(char *x)
+@@ -355,9 +338,7 @@
+ {
+ 	int result;
+ 
+-	outcnt = 0;		/* bytes in output buffer */
+ 	exit_code = 0;
+-	bytes_out = 0;
+ 
+ 	crd_infd = in_fd;
+ 	crd_outfd = out_fd;
+@@ -366,17 +347,10 @@
+ 		printk(KERN_ERR "RAMDISK: Couldn't allocate gzip buffer\n");
+ 		return -1;
+ 	}
+-	window = kmalloc(WSIZE, GFP_KERNEL);
+-	if (window == 0) {
+-		printk(KERN_ERR "RAMDISK: Couldn't allocate gzip window\n");
+-		kfree(inbuf);
+-		return -1;
+-	}
+-	result = gunzip(inbuf, INBUFSIZ, fill_inbuf);
++	result = gunzip(inbuf, INBUFSIZ, fill_inbuf, flush_buffer);
+ 	if (unzip_error)
+ 		result = 1;
+ 	kfree(inbuf);
+-	kfree(window);
+ 	return result;
+ }
+ 
+Index: alp-linux/init/initramfs.c
+===================================================================
+--- alp-linux.orig/init/initramfs.c	2006-07-12 14:22:54.000000000 -0700
++++ alp-linux/init/initramfs.c	2006-07-12 15:20:35.000000000 -0700
+@@ -309,12 +309,12 @@
+ 	return len - count;
+ }
+ 
+-static void __init flush_buffer(char *buf, unsigned len)
++static void __init flush_buffer(const u8 *buf, int len)
+ {
+ 	int written;
+ 	if (message)
+ 		return;
+-	while ((written = write_buffer(buf, len)) < len && !message) {
++	while ((written = write_buffer((char *)buf, len)) < len && !message) {
+ 		char c = buf[written];
+ 		if (c == '0') {
+ 			buf += written;
+@@ -337,41 +337,14 @@
+ typedef unsigned short ush;
+ typedef unsigned long  ulg;
+ 
+-#define WSIZE 0x8000    /* window size--must be a power of two, and */
+-			/*  at least 32K for zip's deflate method */
+-
+-static uch *window;
+-
+-static unsigned outcnt;  /* bytes in output buffer */
+-static long bytes_out;
+-
+ #define INIT __init
+ 
+-static void __init flush_window(void);
+ static void __init error(char *m);
+ 
+ #define NO_INFLATE_MALLOC
+ 
+ #include "../lib/inflate.c"
+ 
+-/* ===========================================================================
+- * Write the output window window[0..outcnt-1] and update bytes_out.
+- * (Used for the decompressed data only.)
+- */
+-static void __init flush_window(void)
+-{
+-	unsigned n;
+-	uch *in, ch;
+-
+-	flush_buffer(window, outcnt);
+-	in = window;
+-	for (n = 0; n < outcnt; n++)
+-		ch = *in++;
+-
+-	bytes_out += (ulg)outcnt;
+-	outcnt = 0;
+-}
+-
+ static char * __init unpack_to_rootfs(char *buf, unsigned len, int check_only)
+ {
+ 	int written, cnt;
+@@ -379,8 +352,7 @@
+ 	header_buf = kmalloc(110, GFP_KERNEL);
+ 	symlink_buf = kmalloc(PATH_MAX + N_ALIGN(PATH_MAX) + 1, GFP_KERNEL);
+ 	name_buf = kmalloc(N_ALIGN(PATH_MAX), GFP_KERNEL);
+-	window = kmalloc(WSIZE, GFP_KERNEL);
+-	if (!window || !header_buf || !symlink_buf || !name_buf)
++	if (!header_buf || !symlink_buf || !name_buf)
+ 		panic("can't allocate buffers");
+ 	state = Start;
+ 	this_header = 0;
+@@ -401,16 +373,13 @@
+ 			continue;
+ 		}
+ 		this_header = 0;
+-		outcnt = 0;		/* bytes in output buffer */
+-		bytes_out = 0;
+-		cnt = gunzip(buf, len, NULL);
++		cnt = gunzip(buf, len, NULL, flush_buffer);
+ 		if (state != Reset)
+ 			error("junk in gzipped archive");
+ 		this_header = saved_offset + cnt;
+ 		buf += cnt;
+ 		len -= cnt;
+ 	}
+-	kfree(window);
+ 	kfree(name_buf);
+ 	kfree(symlink_buf);
+ 	kfree(header_buf);
+Index: alp-linux/lib/inflate.c
+===================================================================
+--- alp-linux.orig/lib/inflate.c	2006-07-12 14:22:54.000000000 -0700
++++ alp-linux/lib/inflate.c	2006-07-12 15:20:35.000000000 -0700
+@@ -144,7 +144,10 @@
+ 	if (!malloc_count)
+ 		malloc_ptr = free_mem_ptr;
+ }
++
++static u8 window[0x8000]; /* use a statically allocated window */
+ #else
++static u8 *window; /* dynamically allocate */
+ #define malloc(a) kmalloc(a, GFP_KERNEL)
+ #define free(a) kfree(a)
+ #endif
+@@ -170,8 +173,9 @@
+ 
+ struct iostate {
+ 	u8 *window, *ibuf;
+-	int ipos, isize, itotal, opos, osize, bits;
++	int ipos, isize, itotal, opos, osize, ototal, bits;
+ 	void (*fill)(u8 *ibuf, int len);
++	void (*flush)(const u8 *obuf, int len);
+ 	u32 buf, crc;
+ };
+ 
+@@ -191,10 +195,6 @@
+    stream to find repeated byte strings.  This is implemented here as a
+    circular buffer.  The index is updated simply by incrementing and then
+    ANDing with 0x7fff (32K-1). */
+-/* It is left to other modules to supply the 32 K area.  It is assumed
+-   to be usable as if it were declared "u8 window[32768];" or as just
+-   "u8 *window;" and then malloc'ed in the latter case.  The definition
+-   must be in unzip.h, included above. */
+ 
+ static void flush_output(struct iostate *io)
+ {
+@@ -204,8 +204,8 @@
+ 		io->crc = crc_32_tab[(io->window[i] ^ (int)io->crc) & 0xff]
+ 			^ (io->crc >> 8);
+ 
+-	outcnt = io->opos;
+-	flush_window();
++	io->flush(io->window, io->opos);
++	io->ototal += io->opos;
+ 	io->opos = 0;
+ }
+ 
+@@ -993,8 +993,10 @@
+  * @ibuf: input character pool
+  * @isize: size of pool
+  * @fill: function to fill the input pool
++ * @flush: function to flush the output pool
+  */
+-static int INIT gunzip(u8 *ibuf, int isize, void (*fill)(u8 *buf, int size))
++static int INIT gunzip(u8 *ibuf, int isize, void (*fill)(u8 *buf, int size),
++		       void (*flush)(const u8 *buf, int size))
+ {
+ 	u8 flags;
+ 	unsigned char magic[2];	/* magic header */
+@@ -1004,13 +1006,13 @@
+ 	int res;
+ 	struct iostate io;
+ 
+-	io.window = window;
+-	io.osize = WSIZE;
+-	io.opos = io.bits = io.buf = io.ipos = io.itotal = 0;
++	io.osize = 0x8000; /* 32k for inflate */
++	io.opos = io.bits = io.buf = io.ipos = io.itotal = io.ototal = 0;
+ 	io.crc = 0xffffffffUL;
+ 	io.isize = isize;
+ 	io.ibuf = ibuf;
+ 	io.fill = fill;
++	io.flush = flush;
+ 
+ 	makecrc(); /* initialize the CRC table */
+ 
+@@ -1068,7 +1070,16 @@
+ 			;
+ 
+ 	/* Decompress */
+-	if ((res = inflate(&io))) {
++	io.window = window ? window : malloc(io.osize);
++	if (!io.window) {
++		error("couldn't allocate gunzip window\n");
++		return -1;
++	}
++	res = inflate(&io);
++	if (!window)
++		free(io.window);
++
++	if (res) {
+ 		switch (res) {
+ 		case 0:
+ 			break;
+@@ -1108,7 +1119,7 @@
+ 		error("crc error");
+ 		return -1;
+ 	}
+-	if (orig_len != bytes_out) {
++	if (orig_len != io.ototal) {
+ 		error("length error");
+ 		return -1;
+ 	}
Index: packages/kernel/patches/tiny_m57sli/tiny/func-size.patch
===================================================================
--- packages/kernel/patches/tiny_m57sli/tiny/func-size.patch	(revision 0)
+++ packages/kernel/patches/tiny_m57sli/tiny/func-size.patch	(revision 0)
@@ -0,0 +1,175 @@
+
+start of inline function size measurement
+
+
+ tiny-mpm/scripts/func-size |  163 +++++++++++++++++++++++++++++++++++++++++++++
+ 1 files changed, 163 insertions(+)
+
+Index: linux-2.6.16.19/scripts/func-size
+===================================================================
+--- linux-2.6.16.19.orig/scripts/func-size	2006-04-21 14:09:39.654033432 -0700
++++ linux-2.6.16.19/scripts/func-size	2006-06-23 17:06:19.000000000 -0700
+@@ -0,0 +1,163 @@
++#!/usr/bin/python
++
++# This reads the output of a kernel compile with
++# CONFIG_MEASURE_INLINES enabled and counts the number of code
++# instantiations. This is slightly tricky as gcc reports
++# instantiations of inlines inside of other inlines in header files,
++# so we need to build a call tree and credit only instantiations in C
++# files.
++#
++# Usage:
++#
++#  make 2> inline-data
++#  scripts/count-inlines inline-data | less
++#
++
++import sys, re, os
++
++showtree = 1
++seen = {}
++defined = {}
++calls = {}
++callers = {}
++file = {}
++uses = {}
++usecount = {}
++used = []
++findcache = {}
++bytesused = {}
++total = {}
++in_c = {}
++fsize = {}
++fr = re.compile(r"In function `(\S+)'")
++dr = re.compile(r"(\S+):(\d+): warning: `(\S+)' is deprecated \(declared at (\S+):(\d+)\)")
++current = "unknown"
++linecache = {}
++fusers = {}
++
++kernel = sys.argv[1]
++inlines = open(sys.argv[2])
++
++# Credit nested inlines
++def credit(func, nest):
++    if nest>10: sys.exit(-1)
++    usecount[func] = usecount.get(func, 0) + 1
++    if not calls.has_key(func): return
++    for f in calls[func]: credit(f, nest+1)
++
++def line2func(pos):
++    try:
++        return linecache[pos]
++    except KeyError:
++        c = pos.index(":")
++        f = pos[:c]
++        l = int(pos[c+1:])
++        best = "unknown(%s)" % pos[:-1]
++
++        try:
++            for func, line in file[f]:
++                if line <= l: best = func
++        except KeyError: pass
++
++        linecache[pos] = best
++        return best
++
++def locate_section(file, section):
++    tsec = re.compile(" %s\s+(\S+)\s+(\S+)" % section)
++    for l in os.popen("objdump -h %s" % file).xreadlines():
++        try: size, start = map(lambda x: int(x, 16), tsec.search(l).groups())
++        except: pass
++
++    return (start, size)
++
++def scan_range(file, start, size):
++    r,w = os.pipe()
++    if os.fork():
++        os.close(w)
++        f = "unknown"
++        for l in os.fdopen(r).xreadlines():
++            l = l[:-1]
++            print l
++            if ":" in l:
++                f = line2func(l)
++                try: fsize[f] += 1
++                except: fsize[f] = 1
++            else:
++                try: fusers[f][l] += 1
++                except: fusers[f] = { l: 1 }
++
++    else: # child
++        os.close(r)
++        os.dup2(w, 1)
++        a2l = os.popen("addr2line -f -e %s" % file, "w")
++        for b in range(start, start+size):
++            a2l.write(hex(b)+"\n")
++        sys.exit(0)
++
++print >>sys.stderr, "Scanning inlining info...";
++
++for l in inlines.xreadlines():
++    g = dr.match(l)
++    if not g:
++        try: current = fr.search(l).group(1)
++        except: pass
++        continue
++
++    use, useline, name, src, srcline = g.groups()
++    srcline, useline = int(srcline), int(useline)
++
++    total[name]= total.get(name, 0) + 1
++    if use[-1] == 'c': in_c[name] = in_c.get(name, 0) + 1
++
++    file.setdefault(src, []).append((name, srcline))
++    if src != use or (srcline != useline and srcline != useline-1):
++        uses.setdefault(name, []).append((use,useline))
++        if use[-1] == 'c': used.append(name)
++
++    if not name in calls.setdefault(current, []):
++        calls[current].append(name)
++    if not current in callers.setdefault(name, []):
++        callers[name].append(current)
++
++for func in used: credit(func, 0)
++
++for sec in (".text", ".init.text", ".exit.text"):
++    print >>sys.stderr, "Scanning %s..." % sec;
++    start, size = locate_section(kernel, sec)
++    scan_range(kernel, start, size)
++
++d = zip(fsize.values(), fsize.keys())
++d.sort()
++d.reverse()
++
++for c, f in d:
++    print "%d %s" % (c, f)
++    e = zip(fusers[f].values(), fusers[f].keys())
++    e.sort()
++    e.reverse()
++    for g, c in e:
++        print " %s(%d)" % (c, g),
++    print
++
++
++l = [ (usecount[f], f) for f in usecount.keys() ]
++l.sort()
++l.reverse()
++
++for (c,f) in l:
++    def list(l):
++        a = [ (usecount.get(f),f) for f in l ]
++        a = filter(lambda x: x[0], a)
++        n = len(l) - len(a)
++        if n:
++            a.append((n,"<other>"))
++        a.sort()
++        a.reverse()
++        return " ".join(["%s(%d)" % (f,c) for c,f in a])
++    print "%d  %s (%d in *.c)" % (c, f, in_c.get(f, 0))
++    if showtree:
++        print "calls:", list(calls.get(f, []))
++        print "callers:", list(callers.get(f, []))
++        print
++
++
Index: packages/kernel/patches/tiny_m57sli/tiny/inet_protos.patch
===================================================================
--- packages/kernel/patches/tiny_m57sli/tiny/inet_protos.patch	(revision 0)
+++ packages/kernel/patches/tiny_m57sli/tiny/inet_protos.patch	(revision 0)
@@ -0,0 +1,24 @@
+
+CONFIG_NET_SMALL reduce MAX_INET_PROTOS
+
+
+ include/net/protocol.h |    5 	4 +	1 -	0 !
+ 1 files changed, 4 insertions(+), 1 deletion(-)
+
+Index: linux-2.6.22.1/include/net/protocol.h
+===================================================================
+--- linux-2.6.22.1.orig/include/net/protocol.h
++++ linux-2.6.22.1/include/net/protocol.h
+@@ -29,8 +29,11 @@
+ #include <linux/ipv6.h>
+ #endif
+ 
++#ifdef CONFIG_NET_SMALL
++#define MAX_INET_PROTOS 32
++#else
+ #define MAX_INET_PROTOS	256		/* Must be a power of 2		*/
+-
++#endif
+ 
+ /* This is used to register protocols. */
+ struct net_protocol {
Index: packages/kernel/patches/tiny_m57sli/tiny/no-elf-core.patch
===================================================================
--- packages/kernel/patches/tiny_m57sli/tiny/no-elf-core.patch	(revision 0)
+++ packages/kernel/patches/tiny_m57sli/tiny/no-elf-core.patch	(revision 0)
@@ -0,0 +1,47 @@
+
+configurable support for ELF core dumps
+
+
+ tiny-mpm/fs/binfmt_elf.c |    4 ++--
+ tiny-mpm/init/Kconfig    |    6 ++++++
+ 2 files changed, 8 insertions(+), 2 deletions(-)
+
+Index: tiny/fs/binfmt_elf.c
+===================================================================
+--- tiny.orig/fs/binfmt_elf.c	2005-10-10 17:30:45.000000000 -0700
++++ tiny/fs/binfmt_elf.c	2005-10-10 17:42:36.000000000 -0700
+@@ -58,7 +58,7 @@ extern int dump_fpu (struct pt_regs *, e
+  * If we don't support core dumping, then supply a NULL so we
+  * don't even try.
+  */
+-#ifdef USE_ELF_CORE_DUMP
++#if defined(USE_ELF_CORE_DUMP) && defined(CONFIG_ELF_CORE)
+ static int elf_core_dump(long signr, struct pt_regs * regs, struct file * file);
+ #else
+ #define elf_core_dump	NULL
+@@ -1109,7 +1109,7 @@ out:
+  * Note that some platforms still use traditional core dumps and not
+  * the ELF core dump.  Each platform can select it as appropriate.
+  */
+-#ifdef USE_ELF_CORE_DUMP
++#if defined(USE_ELF_CORE_DUMP) && defined(CONFIG_ELF_CORE)
+ 
+ /*
+  * ELF core dumper
+Index: tiny/init/Kconfig
+===================================================================
+--- tiny.orig/init/Kconfig	2005-10-10 17:42:34.000000000 -0700
++++ tiny/init/Kconfig	2005-10-10 17:42:36.000000000 -0700
+@@ -355,6 +355,12 @@ config FULL_PANIC
+           Disabling compiles out the explanations for panics, saving
+ 	  string space. Use with caution.
+ 
++config ELF_CORE
++	default y
++	bool "Enable ELF core dumps" if EMBEDDED
++	help
++	  Enable support for generating core dumps. Disabling saves about 4k.
++
+ config BASE_FULL
+ 	default y
+ 	bool "Enable full-sized data structures for core" if EMBEDDED
Index: packages/kernel/patches/tiny_m57sli/tiny/inflate-error.patch
===================================================================
--- packages/kernel/patches/tiny_m57sli/tiny/inflate-error.patch	(revision 0)
+++ packages/kernel/patches/tiny_m57sli/tiny/inflate-error.patch	(revision 0)
@@ -0,0 +1,500 @@
+inflate: error handling cleanup
+
+This passes the error function as a callback rather than using a
+global symbol.
+
+Signed-off-by: Matt Mackall <mpm@selenic.com>
+
+Index: alp-linux/arch/alpha/boot/misc.c
+===================================================================
+--- alp-linux.orig/arch/alpha/boot/misc.c	2006-07-12 15:22:40.000000000 -0700
++++ alp-linux/arch/alpha/boot/misc.c	2006-07-12 15:24:21.000000000 -0700
+@@ -28,8 +28,6 @@
+  * gzip delarations
+  */
+ 
+-static void error(char *m);
+-
+ static char *input_data;
+ static int  input_data_size;
+ 
+@@ -50,7 +48,7 @@
+ 		*output_data++ = *buf++;
+ }
+ 
+-static void error(char *x)
++static void error(const char *x)
+ {
+ 	puts("\n\n");
+ 	puts(x);
+@@ -76,7 +74,7 @@
+ 	/* FIXME FIXME FIXME */
+ 
+ /*	puts("Uncompressing Linux..."); */
+-	count = gunzip(input_data, input_data_size, NULL, flush_window);
++	count = gunzip(input_data, input_data_size, NULL, flush_window, error);
+ /*	puts(" done, booting the kernel.\n"); */
+ 	return count;
+ }
+Index: alp-linux/arch/arm/boot/compressed/misc.c
+===================================================================
+--- alp-linux.orig/arch/arm/boot/compressed/misc.c	2006-07-12 15:23:57.000000000 -0700
++++ alp-linux/arch/arm/boot/compressed/misc.c	2006-07-12 15:25:03.000000000 -0700
+@@ -46,8 +46,6 @@
+  * gzip declarations
+  */
+ 
+-static void error(char *m);
+-
+ extern char input_data[];
+ extern char input_data_end[];
+ 
+@@ -79,7 +77,7 @@
+ #define arch_error(x)
+ #endif
+ 
+-static void error(char *x)
++static void error(const char *x)
+ {
+ 	arch_error(x);
+ 
+@@ -106,7 +104,8 @@
+ 	arch_decomp_setup();
+ 
+ 	putstr("Uncompressing Linux...");
+-	count = gunzip(input_data, input_data_end - input_data, NULL, flush_window);
++	count = gunzip(input_data, input_data_end - input_data, NULL, flush_window,
++ 	       error);
+ 	putstr(" done, booting the kernel.\n");
+ 	return count;
+ }
+@@ -119,7 +118,8 @@
+ 	output_data = output_buffer;
+ 
+ 	putstr("Uncompressing Linux...");
+-	gunzip(input_data, input_data_end - input_data, NULL, flush_window);
++	gunzip(input_data, input_data_end - input_data, NULL, flush_window,
++	       error);
+ 	putstr("done.\n");
+ 	return 0;
+ }
+Index: alp-linux/arch/arm26/boot/compressed/misc.c
+===================================================================
+--- alp-linux.orig/arch/arm26/boot/compressed/misc.c	2006-07-12 15:22:40.000000000 -0700
++++ alp-linux/arch/arm26/boot/compressed/misc.c	2006-07-12 15:24:21.000000000 -0700
+@@ -31,8 +31,6 @@
+  * gzip delarations
+  */
+ 
+-static void error(char *m);
+-
+ extern char input_data[];
+ extern char input_data_end[];
+ static u8 *output_data;
+@@ -59,7 +57,7 @@
+ 	puts(".");
+ }
+ 
+-static void error(char *x)
++static void error(const char *x)
+ {
+ 	int ptr;
+ 
+@@ -84,7 +82,8 @@
+ 	arch_decomp_setup();
+ 
+ 	puts("Uncompressing Linux...");
+-	gunzip(input_data, input_data_end - input_data, NULL, flush_window);
++	gunzip(input_data, input_data_end - input_data, NULL, flush_window,
++	       error);
+ 	puts(" done, booting the kernel.\n");
+ 	return output_ptr;
+ }
+@@ -97,7 +96,8 @@
+ 	output_data = output_buffer;
+ 
+ 	puts("Uncompressing Linux...");
+-	gunzip(input_data, input_data_end - input_data, NULL, flush_window);
++	gunzip(input_data, input_data_end - input_data, NULL, flush_window,
++	       error);
+ 	puts("done.\n");
+ 	return 0;
+ }
+Index: alp-linux/arch/cris/arch-v10/boot/compressed/misc.c
+===================================================================
+--- alp-linux.orig/arch/cris/arch-v10/boot/compressed/misc.c	2006-07-12 15:22:40.000000000 -0700
++++ alp-linux/arch/cris/arch-v10/boot/compressed/misc.c	2006-07-12 15:24:21.000000000 -0700
+@@ -30,8 +30,6 @@
+ 
+ unsigned compsize; /* compressed size, used by head.S */
+ 
+-static void error(char *m);
+-
+ extern char *input_data;  /* lives in head.S */
+ static u8 *output_data;
+ 
+@@ -79,8 +77,7 @@
+ 		*output_data++ = *buf++;
+ }
+ 
+-static void
+-error(char *x)
++static void error(const char *x)
+ {
+ 	puts("\n\n");
+ 	puts(x);
+@@ -136,6 +133,6 @@
+ 	}
+ 
+ 	puts("Uncompressing Linux...\n");
+-	compsize = gunzip(input_data, 0x7fffffff, NULL, flush_window);
++	compsize = gunzip(input_data, 0x7fffffff, NULL, flush_window, error);
+ 	puts("Done. Now booting the kernel.\n");
+ }
+Index: alp-linux/arch/cris/arch-v32/boot/compressed/misc.c
+===================================================================
+--- alp-linux.orig/arch/cris/arch-v32/boot/compressed/misc.c	2006-07-12 15:22:40.000000000 -0700
++++ alp-linux/arch/cris/arch-v32/boot/compressed/misc.c	2006-07-12 15:24:21.000000000 -0700
+@@ -30,8 +30,6 @@
+  * gzip declarations
+  */
+ 
+-static void error(char *m);
+-
+ extern char *input_data;  /* lives in head.S */
+ static u8 *output_data;
+ 
+@@ -91,8 +89,7 @@
+ 		*output_data++ = *buf++;
+ }
+ 
+-static void
+-error(char *x)
++static void error(const char *x)
+ {
+ 	puts("\n\n");
+ 	puts(x);
+@@ -179,6 +176,6 @@
+ 	}
+ 
+ 	puts("Uncompressing Linux...\n");
+-	gunzip(input_data, 0x7fffffff, NULL, flush_window);
++	gunzip(input_data, 0x7fffffff, NULL, flush_window, error);
+ 	puts("Done. Now booting the kernel.\n");
+ }
+Index: alp-linux/arch/i386/boot/compressed/misc.c
+===================================================================
+--- alp-linux.orig/arch/i386/boot/compressed/misc.c	2006-07-12 15:22:40.000000000 -0700
++++ alp-linux/arch/i386/boot/compressed/misc.c	2006-07-12 15:24:21.000000000 -0700
+@@ -24,8 +24,6 @@
+  * Incomprehensible are the ways of bootloaders.
+  */
+ 
+-static void error(char *m);
+-
+ /*
+  * This is set up by the setup-routine at boot-time
+  */
+@@ -125,7 +123,7 @@
+ 	}
+ }
+ 
+-static void error(char *x)
++static void error(const char *x)
+ {
+ 	putstr("\n\n");
+ 	putstr(x);
+@@ -214,7 +212,7 @@
+ 	else setup_output_buffer_if_we_run_high(mv);
+ 
+ 	putstr("Uncompressing Linux... ");
+-	gunzip(input_data, input_len, NULL, flush_window);
++	gunzip(input_data, input_len, NULL, flush_window, error);
+ 	putstr("Ok, booting the kernel.\n");
+ 	if (high_loaded) close_output_buffer_if_we_run_high(mv);
+ 	return high_loaded;
+Index: alp-linux/arch/m32r/boot/compressed/misc.c
+===================================================================
+--- alp-linux.orig/arch/m32r/boot/compressed/misc.c	2006-07-12 15:22:40.000000000 -0700
++++ alp-linux/arch/m32r/boot/compressed/misc.c	2006-07-12 15:24:21.000000000 -0700
+@@ -17,8 +17,6 @@
+  * gzip declarations
+  */
+ 
+-static void error(char *m);
+-
+ static unsigned char *input_data;
+ static int input_len;
+ static u8 *output_data;
+@@ -39,7 +37,7 @@
+ 		*output_data++ = *buf++;
+ }
+ 
+-static void error(char *x)
++static void error(const char *x)
+ {
+ 	puts("\n\n");
+ 	puts(x);
+@@ -61,6 +59,6 @@
+ 	input_len = zimage_len;
+ 
+ 	puts("Uncompressing Linux... ");
+-	gunzip(input_data, input_len, NULL, flush_window);
++	gunzip(input_data, input_len, NULL, flush_window, error);
+ 	puts("Ok, booting the kernel.\n");
+ }
+Index: alp-linux/arch/sh/boot/compressed/misc.c
+===================================================================
+--- alp-linux.orig/arch/sh/boot/compressed/misc.c	2006-07-12 15:22:40.000000000 -0700
++++ alp-linux/arch/sh/boot/compressed/misc.c	2006-07-12 15:24:21.000000000 -0700
+@@ -19,8 +19,6 @@
+  * gzip declarations
+  */
+ 
+-static void error(char *m);
+-
+ extern char input_data[];
+ extern int input_len;
+ static u8 *output_data;
+@@ -67,7 +65,7 @@
+ 		*output_data++ = *buf++;
+ }
+ 
+-static void error(char *x)
++static void error(const char *x)
+ {
+ 	puts("\n\n");
+ 	puts(x);
+@@ -88,6 +86,6 @@
+ 	free_mem_end_ptr = free_mem_ptr + HEAP_SIZE;
+ 
+ 	puts("Uncompressing Linux... ");
+-	gunzip(input_data, input_len, NULL, flush_window);
++	gunzip(input_data, input_len, NULL, flush_window, error);
+ 	puts("Ok, booting the kernel.\n");
+ }
+Index: alp-linux/arch/sh64/boot/compressed/misc.c
+===================================================================
+--- alp-linux.orig/arch/sh64/boot/compressed/misc.c	2006-07-12 15:22:40.000000000 -0700
++++ alp-linux/arch/sh64/boot/compressed/misc.c	2006-07-12 15:24:21.000000000 -0700
+@@ -19,8 +19,6 @@
+  * gzip declarations
+  */
+ 
+-static void error(char *m);
+-
+ extern char input_data[];
+ extern int input_len;
+ static u8 *output_data;
+@@ -49,7 +47,7 @@
+ 	puts(".");
+ }
+ 
+-static void error(char *x)
++static void error(const char *x)
+ {
+ 	puts("\n\n");
+ 	puts(x);
+@@ -70,7 +68,7 @@
+ 
+ 	puts("Uncompressing Linux... ");
+ 	cache_control(CACHE_ENABLE);
+-	gunzip(input_data, input_len, NULL, flush_window);
++	gunzip(input_data, input_len, NULL, flush_window, error);
+ 	puts("\n");
+ 
+ #if 0
+Index: alp-linux/arch/x86_64/boot/compressed/misc.c
+===================================================================
+--- alp-linux.orig/arch/x86_64/boot/compressed/misc.c	2006-07-12 15:22:40.000000000 -0700
++++ alp-linux/arch/x86_64/boot/compressed/misc.c	2006-07-12 15:24:21.000000000 -0700
+@@ -16,8 +16,6 @@
+  * gzip declarations
+  */
+ 
+-static void error(char *m);
+-
+ /*
+  * This is set up by the setup-routine at boot-time
+  */
+@@ -114,7 +112,7 @@
+ 	}
+ }
+ 
+-static void error(char *x)
++static void error(const char *x)
+ {
+ 	putstr("\n\n");
+ 	putstr(x);
+@@ -192,7 +190,7 @@
+ 	else setup_output_buffer_if_we_run_high(mv);
+ 
+ 	putstr(".\nDecompressing Linux...");
+-	gunzip(input_data, input_len, NULL, flush_window);
++	gunzip(input_data, input_len, NULL, flush_window, error);
+ 	putstr("done.\nBooting the kernel.\n");
+ 	if (high_loaded) close_output_buffer_if_we_run_high(mv);
+ 	return high_loaded;
+Index: alp-linux/init/do_mounts_rd.c
+===================================================================
+--- alp-linux.orig/init/do_mounts_rd.c	2006-07-12 15:22:40.000000000 -0700
++++ alp-linux/init/do_mounts_rd.c	2006-07-12 15:24:21.000000000 -0700
+@@ -294,12 +294,17 @@
+ 
+ #define INIT __init
+ 
+-static void __init error(char *m);
+-
+ #define NO_INFLATE_MALLOC
+ 
+ #include "../lib/inflate.c"
+ 
++static void __init error(const char *x)
++{
++	printk(KERN_ERR "%s\n", x);
++	exit_code = 1;
++	unzip_error = 1;
++}
++
+ /*
+  * Fill the input buffer. This is called only when the buffer is empty
+  * and at least one byte is really needed.
+@@ -323,13 +328,6 @@
+ 	}
+ }
+ 
+-static void __init error(char *x)
+-{
+-	printk(KERN_ERR "%s\n", x);
+-	exit_code = 1;
+-	unzip_error = 1;
+-}
+-
+ static int __init crd_load(int in_fd, int out_fd)
+ {
+ 	int result;
+@@ -343,7 +341,7 @@
+ 		printk(KERN_ERR "RAMDISK: Couldn't allocate gzip buffer\n");
+ 		return -1;
+ 	}
+-	result = gunzip(inbuf, INBUFSIZ, fill_inbuf, flush_buffer);
++	result = gunzip(inbuf, INBUFSIZ, fill_inbuf, flush_buffer, error);
+ 	if (unzip_error)
+ 		result = 1;
+ 	kfree(inbuf);
+Index: alp-linux/init/initramfs.c
+===================================================================
+--- alp-linux.orig/init/initramfs.c	2006-07-12 15:22:40.000000000 -0700
++++ alp-linux/init/initramfs.c	2006-07-12 15:24:21.000000000 -0700
+@@ -7,8 +7,8 @@
+ #include <linux/string.h>
+ #include <linux/syscalls.h>
+ 
+-static __initdata char *message;
+-static void __init error(char *x)
++static __initdata const char *message;
++static void __init error(const char *x)
+ {
+ 	if (!message)
+ 		message = x;
+@@ -335,13 +335,12 @@
+ 
+ #define INIT __init
+ 
+-static void __init error(char *m);
+-
+ #define NO_INFLATE_MALLOC
+ 
+ #include "../lib/inflate.c"
+ 
+-static char * __init unpack_to_rootfs(char *buf, unsigned len, int check_only)
++static const char * __init unpack_to_rootfs(char *buf, unsigned len,
++					    int check_only)
+ {
+ 	int written, cnt;
+ 	dry_run = check_only;
+@@ -369,7 +368,7 @@
+ 			continue;
+ 		}
+ 		this_header = 0;
+-		cnt = gunzip(buf, len, NULL, flush_buffer);
++		cnt = gunzip(buf, len, NULL, flush_buffer, error);
+ 		if (state != Reset)
+ 			error("junk in gzipped archive");
+ 		this_header = saved_offset + cnt;
+@@ -419,8 +418,9 @@
+ 
+ void __init populate_rootfs(void)
+ {
+-	char *err = unpack_to_rootfs(__initramfs_start,
+-			 __initramfs_end - __initramfs_start, 0);
++	const char *err =
++		unpack_to_rootfs(__initramfs_start,
++				 __initramfs_end - __initramfs_start, 0);
+ 	if (err)
+ 		panic(err);
+ #ifdef CONFIG_BLK_DEV_INITRD
+Index: alp-linux/lib/inflate.c
+===================================================================
+--- alp-linux.orig/lib/inflate.c	2006-07-12 15:20:35.000000000 -0700
++++ alp-linux/lib/inflate.c	2006-07-12 15:24:21.000000000 -0700
+@@ -121,8 +121,9 @@
+ {
+ 	void *p;
+ 
+-	if (size <0)
+-		error("Malloc error");
++	if (size < 0)
++		return NULL;
++
+ 	if (!malloc_ptr)
+ 		malloc_ptr = free_mem_ptr;
+ 
+@@ -132,7 +133,7 @@
+ 	malloc_ptr += size;
+ 
+ 	if (malloc_ptr >= free_mem_end_ptr)
+-		error("Out of memory");
++		return NULL;
+ 
+ 	malloc_count++;
+ 	return p;
+@@ -176,6 +177,7 @@
+ 	int ipos, isize, itotal, opos, osize, ototal, bits;
+ 	void (*fill)(u8 *ibuf, int len);
+ 	void (*flush)(const u8 *obuf, int len);
++	void (*error)(const char *msg);
+ 	u32 buf, crc;
+ };
+ 
+@@ -881,7 +883,7 @@
+ 	if ((i = huft_build(ll, nl, 257, cplens, cplext, &tl, &bl))) {
+ 		DEBG("dyn5b ");
+ 		if (i == 1) {
+-			error("incomplete literal tree");
++			io->error("incomplete literal tree");
+ 			huft_free(tl);
+ 		}
+ 		return i;	/* incomplete code set */
+@@ -891,7 +893,7 @@
+ 	if ((i = huft_build(ll + nl, nd, 0, cpdist, cpdext, &td, &bd))) {
+ 		DEBG("dyn5d ");
+ 		if (i == 1) {
+-			error("incomplete distance tree");
++			io->error("incomplete distance tree");
+ 			huft_free(td);
+ 		}
+ 		huft_free(tl);
+@@ -994,9 +996,11 @@
+  * @isize: size of pool
+  * @fill: function to fill the input pool
+  * @flush: function to flush the output pool
++ * @error: function to report an error
+  */
+ static int INIT gunzip(u8 *ibuf, int isize, void (*fill)(u8 *buf, int size),
+-		       void (*flush)(const u8 *buf, int size))
++		       void (*flush)(const u8 *buf, int size),
++		       void (*error)(const char *msg))
+ {
+ 	u8 flags;
+ 	unsigned char magic[2];	/* magic header */
Index: packages/kernel/patches/tiny_m57sli/tiny/tiny-extraversion.patch
===================================================================
--- packages/kernel/patches/tiny_m57sli/tiny/tiny-extraversion.patch	(revision 0)
+++ packages/kernel/patches/tiny_m57sli/tiny/tiny-extraversion.patch	(revision 0)
@@ -0,0 +1,22 @@
+
+Add -tiny to extraversion
+
+
+ tiny-mpm/Makefile |    4 ++--
+ 1 files changed, 2 insertions(+), 2 deletions(-)
+
+Index: linux-2.6.16.19/Makefile
+===================================================================
+--- linux-2.6.16.19.orig/Makefile	2006-06-23 17:06:12.000000000 -0700
++++ linux-2.6.16.19/Makefile	2006-06-23 17:06:37.000000000 -0700
+@@ -1,8 +1,8 @@
+ VERSION = 2
+ PATCHLEVEL = 6
+ SUBLEVEL = 16
+-EXTRAVERSION = .19
+-NAME=Sliding Snow Leopard
++EXTRAVERSION = .19-tiny
++NAME=Skinny Snow Leopard
+ 
+ # *DOCUMENTATION*
+ # To see a list of typical targets execute "make help"
Index: packages/kernel/patches/tiny_m57sli/tiny/sbf.patch
===================================================================
--- packages/kernel/patches/tiny_m57sli/tiny/sbf.patch	(revision 0)
+++ packages/kernel/patches/tiny_m57sli/tiny/sbf.patch	(revision 0)
@@ -0,0 +1,51 @@
+
+configurable simple bootflag support
+
+From: Zwane Mwaikambo <zwane@arm.linux.org.uk>
+Subject: Re: 2.6.1-rc1-tiny2
+
+Make SBF optional
+
+zwane@montezuma linux-2.6.1-rc1-tiny2 {0:0} size arch/i386/kernel/bootflag.o
+   text    data     bss     dec     hex filename
+   1645       8       0    1653     675 arch/i386/kernel/bootflag.o
+
+Index: alp-linux/arch/i386/kernel/Makefile
+===================================================================
+--- alp-linux.orig/arch/i386/kernel/Makefile	2006-07-12 14:18:59.000000000 -0700
++++ alp-linux/arch/i386/kernel/Makefile	2006-07-12 14:22:24.000000000 -0700
+@@ -6,7 +6,7 @@
+ 
+ obj-y	:= process.o semaphore.o signal.o entry.o traps.o irq.o \
+ 		ptrace.o time.o ioport.o ldt.o setup.o i8259.o sys_i386.o \
+-		pci-dma.o i386_ksyms.o i387.o dmi_scan.o bootflag.o \
++		pci-dma.o i386_ksyms.o i387.o dmi_scan.o \
+ 		quirks.o i8237.o topology.o
+ 
+ obj-y				+= cpu/
+@@ -39,6 +39,7 @@
+ obj-$(CONFIG_EARLY_PRINTK)	+= early_printk.o
+ obj-$(CONFIG_KGDB)		+= kgdb.o kgdb-jmp.o
+ obj-$(CONFIG_LTT)               += ltt.o
++obj-$(CONFIG_BOOTFLAG)		+= bootflag.o
+ 
+ EXTRA_AFLAGS   := -traditional
+ 
+Index: alp-linux/init/Kconfig
+===================================================================
+--- alp-linux.orig/init/Kconfig	2006-07-12 14:21:22.000000000 -0700
++++ alp-linux/init/Kconfig	2006-07-12 14:21:30.000000000 -0700
+@@ -631,6 +631,13 @@
+ 	  Select the maximum number of IDE interfaces (0 for default).
+           Saves up to 14k.
+ 
++config BOOTFLAG
++	depends X86
++	default y
++	bool "Enable Simple Bootflag" if EMBEDDED
++	help
++	  This enables support for the Simple Bootflag Specification.
++
+ endmenu		# General setup
+ 
+ config CRC32_CALC
Index: packages/kernel/patches/tiny_m57sli/tiny/devlist.patch
===================================================================
--- packages/kernel/patches/tiny_m57sli/tiny/devlist.patch	(revision 0)
+++ packages/kernel/patches/tiny_m57sli/tiny/devlist.patch	(revision 0)
@@ -0,0 +1,265 @@
+This patch introduces a simple allocator for tracking reservations of
+block and character device ranges. After poking around, I came to the
+conclusion that we can't avoid having a separate data structure for
+reservations vs currently available devices so I tidied up the current
+scheme.
+
+The new allocator is a simple linked list of dev_t range start and end
+points. This lets us allocate regions of multiple majors (as is done
+by SCSI) or just a portion of a major (as is done by several types of
+character device). Dynamic allocation is handled by passing in a start
+and end base address.
+
+This unifies a lot of the code for registering block and character. It
+also does away with the hash tables, which were complicating the code
+and made allocating more devices than available hash table entries
+difficult. As these tables are now only used at driver load, unload,
+and for /proc/devices, and the number of entries is relatively
+limited, the hash tables are no longer necessary.
+
+Index: linux-2.6.16.19/block/genhd.c
+===================================================================
+--- linux-2.6.16.19.orig/block/genhd.c	2006-05-30 17:31:44.000000000 -0700
++++ linux-2.6.16.19/block/genhd.c	2006-06-23 13:47:08.000000000 -0700
+@@ -54,54 +54,23 @@
+ 
+ int register_blkdev(unsigned int major, const char *name)
+ {
+-	struct blk_major_name **n, *p;
+-	int index, ret = 0;
++	dev_t r;
++	int ret;
+ 
+ 	down(&block_subsys_sem);
+ 
+-	/* temporary */
+-	if (major == 0) {
+-		for (index = ARRAY_SIZE(major_names)-1; index > 0; index--) {
+-			if (major_names[index] == NULL)
+-				break;
+-		}
+-
+-		if (index == 0) {
+-			printk("register_blkdev: failed to get major for %s\n",
+-			       name);
+-			ret = -EBUSY;
+-			goto out;
+-		}
+-		major = index;
+-		ret = major;
+-	}
+-
+-	p = kmalloc(sizeof(struct blk_major_name), GFP_KERNEL);
+-	if (p == NULL) {
+-		ret = -ENOMEM;
+-		goto out;
+-	}
+-
+-	p->major = major;
+-	strlcpy(p->name, name, sizeof(p->name));
+-	p->next = NULL;
+-	index = major_to_index(major);
+-
+-	for (n = &major_names[index]; *n; n = &(*n)->next) {
+-		if ((*n)->major == major)
+-			break;
+-	}
+-	if (!*n)
+-		*n = p;
+-	else
+-		ret = -EBUSY;
++	if (!major) {
++		ret = register_dev(MKDEV(1, 0), MKDEV(254, 0),
++				   MKDEV(1, 0), name, &blkdev_names, &r);
++		ret = ret ? ret : MAJOR(r);
++	} else
++		ret = register_dev(MKDEV(major, 0), MKDEV(major, 0),
++				   MKDEV(1, 0), name, &blkdev_names, NULL);
+ 
+-	if (ret < 0) {
++	if (ret == -EBUSY)
+ 		printk("register_blkdev: cannot get major %d for %s\n",
+ 		       major, name);
+-		kfree(p);
+-	}
+-out:
++
+ 	up(&block_subsys_sem);
+ 	return ret;
+ }
+@@ -111,24 +80,11 @@
+ /* todo: make void - error printk here */
+ int unregister_blkdev(unsigned int major, const char *name)
+ {
+-	struct blk_major_name **n;
+-	struct blk_major_name *p = NULL;
+-	int index = major_to_index(major);
+-	int ret = 0;
++	int ret;
+ 
+ 	down(&block_subsys_sem);
+-	for (n = &major_names[index]; *n; n = &(*n)->next)
+-		if ((*n)->major == major)
+-			break;
+-	if (!*n || strcmp((*n)->name, name))
+-		ret = -EINVAL;
+-	else {
+-		p = *n;
+-		*n = p->next;
+-	}
++	ret = unregister_dev(MKDEV(major, 0), &blkdev_names);
+ 	up(&block_subsys_sem);
+-	kfree(p);
+-
+ 	return ret;
+ }
+ 
+Index: linux-2.6.16.19/fs/char_dev.c
+===================================================================
+--- linux-2.6.16.19.orig/fs/char_dev.c	2006-05-30 17:31:44.000000000 -0700
++++ linux-2.6.16.19/fs/char_dev.c	2006-06-23 13:47:08.000000000 -0700
+@@ -20,6 +20,8 @@
+ #include <linux/kobject.h>
+ #include <linux/kobj_map.h>
+ #include <linux/cdev.h>
++#include <linux/list.h>
++#include <linux/devlist.h>
+ 
+ #ifdef CONFIG_KMOD
+ #include <linux/kmod.h>
+Index: linux-2.6.16.19/fs/devlist.c
+===================================================================
+--- linux-2.6.16.19.orig/fs/devlist.c	2006-04-21 14:09:39.654033432 -0700
++++ linux-2.6.16.19/fs/devlist.c	2006-06-23 13:47:08.000000000 -0700
+@@ -0,0 +1,115 @@
++/*
++ * A simple device node space allocator for the Linux kernel
++ *
++ * This uses a simple linked list of dev_t ranges for reserving device
++ * areas. The io resource tree scheme would almost work here except we
++ * have backwards compatibility requirements for dynamically allocated
++ * ranges from the top of the legacy device space downwards.
++ *
++ * Feb 27 2005  Matt Mackall <mpm@selenic.com>
++ */
++
++#include <linux/kernel.h>
++#include <linux/list.h>
++#include <linux/slab.h>
++#include <linux/kdev_t.h>
++#include <linux/errno.h>
++
++struct device_name {
++	struct list_head list;
++	dev_t begin, end;
++	char name[16];
++};
++
++/*
++ * register_dev - reserve a range of devices for a given device name
++ * @a: first allowed base address
++ * @b: last allowed base address
++ * @size: size of minor devices device range to reserve
++ * @name: name to associate with the range
++ * @list: device name list to add to
++ * @r: allocated base address
++ *
++ * This function reserves a range of %size minor devices starting
++ * somewhere between a and b of the given size. To allocate at a fixed
++ * base address, use a = b. For dynamic allocation in a range,
++ * allocation occurs from the top down for backward compatibility.
++ * There are no restrictions on the size of the range allocated, so
++ * multiple contiguous majors can be allocated with a single call.
++ *
++ * This function returns 0 on success, -EBUSY if the requested range
++ * could not be allocated, and -ENOMEM if the device name node could
++ * not be allocated.
++ */
++
++int register_dev(dev_t a, dev_t b, int size, const char * name,
++		 struct list_head *list, dev_t *r)
++{
++	struct device_name *d;
++	struct list_head *l;
++
++	/* search for insertion point in reverse for dynamic allocation */
++	list_for_each_prev(l, list) {
++		d = list_entry(l, struct device_name, list);
++		/* have we found an insertion point? */
++		if (b >= d->end)
++			break;
++		/* does the current entry force us to shrink the range? */
++		if (b + size > d->begin)
++			b = d->begin - size;
++		/* have we searched outside of or collapsed the range? */
++		if (a > d->begin || a > b)
++			return -EBUSY;
++	}
++
++	d = kmalloc(sizeof(struct device_name), GFP_KERNEL);
++	if (!d)
++		return -ENOMEM;
++
++	strlcpy(d->name, name, sizeof(d->name));
++	d->begin = b;
++	d->end = b + size;
++	list_add(&d->list, l);
++
++	if (r)
++		*r = b;
++
++	return 0;
++}
++
++/*
++ * unregister_dev - drop a device reservation range
++ * @begin: beginning of allocated range
++ * @l: device name list
++ *
++ * This function removes the range starting with begin from the given
++ * list and returns 0, or returns -EINVAL if the range is not found.
++ */
++
++int unregister_dev(dev_t begin, struct list_head *l)
++{
++	struct device_name *n;
++
++	list_for_each_entry(n, l, list)
++		if (n->begin == begin) {
++			list_del(&n->list);
++			kfree(n);
++			return 0;
++		}
++
++	return -EINVAL;
++}
++
++#ifdef CONFIG_PROC_FS
++int get_dev_list(char *p, struct list_head *l)
++{
++	struct device_name *n;
++	int len = 0, m;
++
++	list_for_each_entry(n, l, list)
++		for (m = MAJOR(n->begin); m <= MAJOR(n->end - 1); m++)
++			len += sprintf(p + len, "%3d %s\n", m, n->name);
++
++	return len;
++}
++#endif
+Index: linux-2.6.16.19/include/linux/devlist.h
+===================================================================
+--- linux-2.6.16.19.orig/include/linux/devlist.h	2006-04-21 14:09:39.654033432 -0700
++++ linux-2.6.16.19/include/linux/devlist.h	2006-06-23 13:47:08.000000000 -0700
+@@ -0,0 +1,11 @@
++#ifndef _LINUX_DEVLIST_H
++#define _LINUX_DEVLIST_H
++
++int register_dev(dev_t a, dev_t b, int size, const char * name,
++		 struct list_head *list, dev_t *r);
++
++int unregister_dev(dev_t begin, struct list_head *l);
++
++int get_dev_list(char *p, struct list_head *l);
++
++#endif
Index: packages/kernel/patches/tiny_m57sli/tiny/no-xattr.patch
===================================================================
--- packages/kernel/patches/tiny_m57sli/tiny/no-xattr.patch	(revision 0)
+++ packages/kernel/patches/tiny_m57sli/tiny/no-xattr.patch	(revision 0)
@@ -0,0 +1,83 @@
+
+configurable xattr support
+
+---
+ fs/Kconfig      |    1 	1 +	0 -	0 !
+ fs/Makefile     |    3 	2 +	1 -	0 !
+ init/Kconfig    |    6 	6 +	0 -	0 !
+ kernel/sys_ni.c |   12 	12 +	0 -	0 !
+ 4 files changed, 21 insertions(+), 1 deletion(-)
+
+Index: linux-2.6.22.1/fs/Kconfig
+===================================================================
+--- linux-2.6.22.1.orig/fs/Kconfig
++++ linux-2.6.22.1/fs/Kconfig
+@@ -19,6 +19,7 @@ config EXT2_FS
+ config EXT2_FS_XATTR
+ 	bool "Ext2 extended attributes"
+ 	depends on EXT2_FS
++	depends on XATTR
+ 	help
+ 	  Extended attributes are name:value pairs associated with inodes by
+ 	  the kernel or by users (see the attr(5) manual page, or visit
+Index: linux-2.6.22.1/fs/Makefile
+===================================================================
+--- linux-2.6.22.1.orig/fs/Makefile
++++ linux-2.6.22.1/fs/Makefile
+@@ -9,7 +9,7 @@ obj-y :=	open.o read_write.o file_table.
+ 		char_dev.o stat.o exec.o pipe.o namei.o fcntl.o \
+ 		ioctl.o readdir.o select.o fifo.o locks.o dcache.o inode.o \
+ 		attr.o bad_inode.o file.o filesystems.o namespace.o aio.o \
+-		seq_file.o xattr.o libfs.o fs-writeback.o \
++		seq_file.o libfs.o fs-writeback.o \
+ 		pnode.o drop_caches.o splice.o sync.o utimes.o \
+ 		stack.o
+ 
+@@ -21,6 +21,7 @@ endif
+ 
+ obj-$(CONFIG_INOTIFY)		+= inotify.o
+ obj-$(CONFIG_INOTIFY_USER)	+= inotify_user.o
++obj-$(CONFIG_XATTR)		+= xattr.o
+ obj-$(CONFIG_EPOLL)		+= eventpoll.o
+ obj-$(CONFIG_ANON_INODES)	+= anon_inodes.o
+ obj-$(CONFIG_SIGNALFD)		+= signalfd.o
+Index: linux-2.6.22.1/init/Kconfig
+===================================================================
+--- linux-2.6.22.1.orig/init/Kconfig
++++ linux-2.6.22.1/init/Kconfig
+@@ -361,6 +361,12 @@ config AIO
+ 	  some high performance threaded applications. Disabling
+ 	  this option saves about 5k.
+ 
++config XATTR
++	default y
++	bool "Enable filesystem extended attribute support" if EMBEDDED
++	help
++	  This enables the filesystem extended attribute syscalls.
++
+ config MEASURE_INLINES
+ 	default n
+ 	bool "Enable inline measurement" if EMBEDDED
+Index: linux-2.6.22.1/kernel/sys_ni.c
+===================================================================
+--- linux-2.6.22.1.orig/kernel/sys_ni.c
++++ linux-2.6.22.1/kernel/sys_ni.c
+@@ -117,6 +117,18 @@ cond_syscall(sys_io_destroy);
+ cond_syscall(sys_io_submit);
+ cond_syscall(sys_io_cancel);
+ cond_syscall(sys_io_getevents);
++cond_syscall(sys_setxattr);
++cond_syscall(sys_lsetxattr);
++cond_syscall(sys_fsetxattr);
++cond_syscall(sys_getxattr);
++cond_syscall(sys_lgetxattr);
++cond_syscall(sys_fgetxattr);
++cond_syscall(sys_listxattr);
++cond_syscall(sys_llistxattr);
++cond_syscall(sys_flistxattr);
++cond_syscall(sys_removexattr);
++cond_syscall(sys_lremovexattr);
++cond_syscall(sys_fremovexattr);
+ 
+ /* arch-specific weak syscall entries */
+ cond_syscall(sys_pciconfig_read);
Index: packages/kernel/patches/tiny_m57sli/tiny/binfmt-script.patch
===================================================================
--- packages/kernel/patches/tiny_m57sli/tiny/binfmt-script.patch	(revision 0)
+++ packages/kernel/patches/tiny_m57sli/tiny/binfmt-script.patch	(revision 0)
@@ -0,0 +1,41 @@
+
+configurable shell script support
+
+
+ fs/Makefile  |    5 	1 +	4 -	0 !
+ init/Kconfig |    6 	6 +	0 -	0 !
+ 2 files changed, 7 insertions(+), 4 deletions(-)
+
+Index: linux-2.6.22.1/fs/Makefile
+===================================================================
+--- linux-2.6.22.1.orig/fs/Makefile
++++ linux-2.6.22.1/fs/Makefile
+@@ -36,10 +36,7 @@ obj-y				+= $(nfsd-y) $(nfsd-m)
+ obj-$(CONFIG_BINFMT_AOUT)	+= binfmt_aout.o
+ obj-$(CONFIG_BINFMT_EM86)	+= binfmt_em86.o
+ obj-$(CONFIG_BINFMT_MISC)	+= binfmt_misc.o
+-
+-# binfmt_script is always there
+-obj-y				+= binfmt_script.o
+-
++obj-$(CONFIG_BINFMT_SCRIPT)	+= binfmt_script.o
+ obj-$(CONFIG_BINFMT_ELF)	+= binfmt_elf.o
+ obj-$(CONFIG_BINFMT_ELF_FDPIC)	+= binfmt_elf_fdpic.o
+ obj-$(CONFIG_BINFMT_SOM)	+= binfmt_som.o
+Index: linux-2.6.22.1/init/Kconfig
+===================================================================
+--- linux-2.6.22.1.orig/init/Kconfig
++++ linux-2.6.22.1/init/Kconfig
+@@ -698,6 +698,12 @@ config LINUXTINY_DO_UNINLINE
+        help
+ 	  Un-inline some function for smaller kernel size.
+ 
++config BINFMT_SCRIPT
++	default y
++	bool "Enable support for executable shell scripts" if EMBEDDED
++	help
++	  Enable support for standard #!-style shell scripts.
++
+ endmenu		# General setup
+ 
+ config RT_MUTEXES
Index: packages/kernel/patches/tiny_m57sli/tiny/ldiscs.patch
===================================================================
--- packages/kernel/patches/tiny_m57sli/tiny/ldiscs.patch	(revision 0)
+++ packages/kernel/patches/tiny_m57sli/tiny/ldiscs.patch	(revision 0)
@@ -0,0 +1,40 @@
+
+configurable number of tty line disciplines
+
+
+ include/linux/tty.h |    2 	1 +	1 -	0 !
+ init/Kconfig        |    8 	8 +	0 -	0 !
+ 2 files changed, 9 insertions(+), 1 deletion(-)
+
+Index: linux-2.6.22.1/include/linux/tty.h
+===================================================================
+--- linux-2.6.22.1.orig/include/linux/tty.h
++++ linux-2.6.22.1/include/linux/tty.h
+@@ -24,7 +24,7 @@
+ #define NR_PTYS	CONFIG_LEGACY_PTY_COUNT   /* Number of legacy ptys */
+ #define NR_UNIX98_PTY_DEFAULT	4096      /* Default maximum for Unix98 ptys */
+ #define NR_UNIX98_PTY_MAX	(1 << MINORBITS) /* Absolute limit */
+-#define NR_LDISCS		17
++#define NR_LDISCS		CONFIG_NR_LDISCS
+ 
+ /* line disciplines */
+ #define N_TTY		0
+Index: linux-2.6.22.1/init/Kconfig
+===================================================================
+--- linux-2.6.22.1.orig/init/Kconfig
++++ linux-2.6.22.1/init/Kconfig
+@@ -711,6 +711,14 @@ config MAX_SWAPFILES_SHIFT
+ 	help
+ 	  Select the maximum number of swapfiles (5 for default).
+ 
++config NR_LDISCS
++	int "Number of tty line disciplines" if EMBEDDED
++	range 0 16
++	default 16
++	help
++	  Select the maximum number of tty line disciplines. Embedded systems
++          can probably get by with only a few of these.
++
+ endmenu		# General setup
+ 
+ config RT_MUTEXES
Index: packages/kernel/patches/tiny_m57sli/tiny/inflate-killdebug.patch
===================================================================
--- packages/kernel/patches/tiny_m57sli/tiny/inflate-killdebug.patch	(revision 0)
+++ packages/kernel/patches/tiny_m57sli/tiny/inflate-killdebug.patch	(revision 0)
@@ -0,0 +1,287 @@
+inflate: remove legacy DEBG macros
+
+Signed-off-by: Matt Mackall <mpm@selenic.com>
+
+Index: linux-2.6.16.19/lib/inflate.c
+===================================================================
+--- linux-2.6.16.19.orig/lib/inflate.c	2006-06-23 17:06:33.000000000 -0700
++++ linux-2.6.16.19/lib/inflate.c	2006-06-23 17:06:36.000000000 -0700
+@@ -1,5 +1,3 @@
+-#define DEBG(x)
+-#define DEBG1(x)
+ /* inflate.c -- Not copyrighted 1992 by Mark Adler
+  * version c10p1, 10 January 1993
+  *
+@@ -418,8 +416,6 @@
+ 	int y;			/* number of dummy codes added */
+ 	unsigned z;		/* number of entries in current table */
+ 
+-	DEBG("huft1 ");
+-
+ 	for (i = 0; i < BMAX + 1; i++)
+ 		c[i] = 0;
+ 
+@@ -437,8 +433,6 @@
+ 		return 2;
+ 	}
+ 
+-	DEBG("huft2 ");
+-
+ 	/* Find minimum and maximum length, bound *m by those */
+ 	l = *m;
+ 	for (j = 1; j <= BMAX; j++)
+@@ -457,8 +451,6 @@
+ 		l = i;
+ 	*m = l;
+ 
+-	DEBG("huft3 ");
+-
+ 	/* Adjust last length count to fill out codes, if needed */
+ 	for (y = 1 << j; j < i; j++, y <<= 1) {
+ 		y -= c[j];
+@@ -471,8 +463,6 @@
+ 		return 2;
+ 	c[i] += y;
+ 
+-	DEBG("huft4 ");
+-
+ 	/* Generate starting offsets into the value table for each length */
+ 	x[1] = j = 0;
+ 	p = c + 1;
+@@ -485,8 +475,6 @@
+ 		xp++;
+ 	}
+ 
+-	DEBG("huft5 ");
+-
+ 	/* Make a table of values in order of bit lengths */
+ 	p = b;
+ 	i = 0;
+@@ -498,8 +486,6 @@
+ 
+ 	n = x[g];                   /* set n to length of v */
+ 
+-	DEBG("h6 ");
+-
+ 	/* Generate the Huffman codes and for each, make the table entries */
+ 	x[0] = i = 0; /* first Huffman code is zero */
+ 	p = v; /* grab values in bit order */
+@@ -508,18 +494,14 @@
+ 	u[0] = NULL; /* just to keep compilers happy */
+ 	q = NULL; /* ditto */
+ 	z = 0; /* ditto */
+-	DEBG("h6a ");
+ 
+ 	/* go through the bit lengths (k already is bits in shortest code) */
+ 	for (; k <= g; k++) {
+-		DEBG("h6b ");
+ 		a = c[k];
+ 		while (a--) {
+-			DEBG("h6b1 ");
+ 			/* i is the Huffman code of length k for value *p */
+ 			/* make tables up to required level */
+ 			while (k > w + l) {
+-				DEBG1("1 ");
+ 				h++;
+ 				w += l;	/* previous table always l bits */
+ 
+@@ -532,7 +514,6 @@
+ 				f = 1 << j;
+ 				if (f > a + 1) {
+ 					/* too few codes for k-w bit table */
+-					DEBG1("2 ");
+ 					/* deduct codes from patterns left */
+ 					f -= a + 1;
+ 					xp = c + k;
+@@ -549,7 +530,6 @@
+ 					}
+     				}
+ 
+-				DEBG1("3 ");
+ 				/* table entries for j-bit table */
+ 				z = 1 << j;
+ 
+@@ -561,13 +541,11 @@
+ 					return 3;	/* not enough memory */
+ 				}
+ 
+-				DEBG1("4 ");
+ 				*t = q + 1; /* link to list for huft_free */
+ 				t = &q->next;
+ 				*t = NULL;
+ 				u[h] = ++q;	/* table starts after link */
+ 
+-				DEBG1("5 ");
+ 				/* connect to last table, if there is one */
+ 				if (h) {
+ 					/* save pattern for backing up */
+@@ -583,9 +561,7 @@
+ 					/* connect to last table */
+ 					u[h - 1][j] = r;
+ 				}
+-				DEBG1("6 ");
+ 			}
+-			DEBG("h6c ");
+ 
+ 			/* set up table entry in r */
+ 			r.bits = (u8)(k - w);
+@@ -603,7 +579,6 @@
+ 				r.extra = (u8)e[*p - s];
+ 				r.val = d[*p++ - s];
+ 			}
+-			DEBG("h6d ");
+ 
+ 			/* fill code-like entries with r */
+ 			f = 1 << (k - w);
+@@ -620,13 +595,9 @@
+ 				h--;	/* don't need to update q */
+ 				w -= l;
+ 			}
+-			DEBG("h6e ");
+ 		}
+-		DEBG("h6f ");
+ 	}
+ 
+-	DEBG("huft7 ");
+-
+ 	/* Return true (1) if we were given an incomplete table */
+ 	return y && g != 1;
+ }
+@@ -715,8 +686,6 @@
+ {
+ 	unsigned n;		/* number of bytes in block */
+ 
+-	DEBG("<stor");
+-
+ 	/* go to byte boundary */
+ 	dumpbits(io, io->bits & 7);
+ 
+@@ -729,7 +698,6 @@
+ 	while (n--)
+ 		put_byte(io, readbyte(io));
+ 
+-	DEBG(">");
+ 	return 0;
+ }
+ 
+@@ -752,8 +720,6 @@
+ 	int bd;			/* lookup bits for td */
+ 	unsigned l[N_MAX];	/* length list for huft_build */
+ 
+-	DEBG("<fix");
+-
+ 	/* set up literal table */
+ 	for (i = 0; i < 144; i++)
+ 		l[i] = 8;
+@@ -774,7 +740,6 @@
+ 	if ((i = huft_build(l, 30, 0, cpdist, cpdext, &td, &bd)) > 1) {
+ 		huft_free(tl);
+ 
+-		DEBG(">");
+ 		return i;
+ 	}
+ 
+@@ -808,8 +773,6 @@
+ 	unsigned nd;		/* number of distance codes */
+ 	unsigned ll[286 + 30];	/* literal/length and distance code lengths */
+ 
+-	DEBG("<dyn");
+-
+ 	/* read in table lengths */
+ 	nl = 257 + pullbits(io, 5); /* number of literal/length codes */
+ 	nd = 1 + pullbits(io, 5); /* number of distance codes */
+@@ -817,16 +780,12 @@
+ 	if (nl > 286 || nd > 30)
+ 		return 1;	/* bad lengths */
+ 
+-	DEBG("dyn1 ");
+-
+ 	/* read in bit-length-code lengths */
+ 	for (j = 0; j < nb; j++)
+ 		ll[border[j]] = pullbits(io, 3);
+ 	for (; j < 19; j++)
+ 		ll[border[j]] = 0;
+ 
+-	DEBG("dyn2 ");
+-
+ 	/* build decoding table for trees--single level, 7 bit lookup */
+ 	bl = 7;
+ 	if ((i = huft_build(ll, 19, 19, 0, 0, &tl, &bl))) {
+@@ -835,8 +794,6 @@
+ 		return i;	/* incomplete code set */
+ 	}
+ 
+-	DEBG("dyn3 ");
+-
+ 	/* read in literal and distance code lengths */
+ 	n = nl + nd;
+ 	i = l = 0;
+@@ -869,29 +826,21 @@
+ 		}
+ 	}
+ 
+-	DEBG("dyn4 ");
+-
+ 	/* free decoding table for trees */
+ 	huft_free(tl);
+ 
+-	DEBG("dyn5 ");
+-
+-	DEBG("dyn5a ");
+-
+ 	/* build the decoding tables for literal/length and distance codes */
+ 	bl = lbits;
+ 	if ((i = huft_build(ll, nl, 257, cplens, cplext, &tl, &bl))) {
+-		DEBG("dyn5b ");
+ 		if (i == 1) {
+ 			io->error("incomplete literal tree");
+ 			huft_free(tl);
+ 		}
+ 		return i;	/* incomplete code set */
+ 	}
+-	DEBG("dyn5c ");
++
+ 	bd = dbits;
+ 	if ((i = huft_build(ll + nl, nd, 0, cpdist, cpdext, &td, &bd))) {
+-		DEBG("dyn5d ");
+ 		if (i == 1) {
+ 			io->error("incomplete distance tree");
+ 			huft_free(td);
+@@ -900,19 +849,14 @@
+ 		return i;	/* incomplete code set */
+ 	}
+ 
+-	DEBG("dyn6 ");
+-
+ 	/* decompress until an end-of-block code */
+ 	if (inflate_codes(io, tl, td, bl, bd))
+ 		return 1;
+ 
+-	DEBG("dyn7 ");
+-
+ 	/* free the decoding tables, return */
+ 	huft_free(tl);
+ 	huft_free(td);
+ 
+-	DEBG(">");
+ 	return 0;
+ }
+ 
+@@ -924,8 +868,6 @@
+ {
+ 	unsigned t;		/* block type */
+ 
+-	DEBG("<blk");
+-
+ 	*e = pullbits(io, 1); /* read in last block bit */
+ 	t = pullbits(io, 2); /* read in block type */
+ 
+@@ -937,8 +879,6 @@
+ 	if (t == 1)
+ 		return inflate_fixed(io);
+ 
+-	DEBG(">");
+-
+ 	/* bad block type */
+ 	return 2;
+ }
Index: packages/kernel/patches/tiny_m57sli/tiny/no-aio.patch
===================================================================
--- packages/kernel/patches/tiny_m57sli/tiny/no-aio.patch	(revision 0)
+++ packages/kernel/patches/tiny_m57sli/tiny/no-aio.patch	(revision 0)
@@ -0,0 +1,133 @@
+
+Configurable AIO support
+
+---
+ fs/aio.c        |   53 	53 +	0 -	0 !
+ init/Kconfig    |    8 	8 +	0 -	0 !
+ kernel/sys_ni.c |    5 	5 +	0 -	0 !
+ kernel/sysctl.c |    2 	2 +	0 -	0 !
+ 4 files changed, 68 insertions(+)
+
+Index: linux-2.6.22.1/fs/aio.c
+===================================================================
+--- linux-2.6.22.1.orig/fs/aio.c
++++ linux-2.6.22.1/fs/aio.c
+@@ -36,6 +36,8 @@
+ #include <asm/uaccess.h>
+ #include <asm/mmu_context.h>
+ 
++#ifdef CONFIG_AIO
++
+ #if DEBUG > 1
+ #define dprintk		printk
+ #else
+@@ -1787,3 +1789,54 @@ __initcall(aio_setup);
+ EXPORT_SYMBOL(aio_complete);
+ EXPORT_SYMBOL(aio_put_req);
+ EXPORT_SYMBOL(wait_on_sync_kiocb);
++
++#else
++
++ssize_t fastcall wait_on_sync_kiocb(struct kiocb *iocb)
++{
++	return 0;
++}
++
++void fastcall exit_aio(struct mm_struct *mm)
++{
++}
++
++void fastcall __put_ioctx(struct kioctx *ctx)
++{
++}
++
++int fastcall aio_put_req(struct kiocb *req)
++{
++	return 0;
++}
++
++struct kioctx *lookup_ioctx(unsigned long ctx_id)
++{
++	return 0;
++}
++
++void fastcall kick_iocb(struct kiocb *iocb)
++{
++}
++
++int fastcall aio_complete(struct kiocb *iocb, long res, long res2)
++{
++	return 0;
++}
++
++int fastcall io_submit_one(struct kioctx *ctx, struct iocb __user *user_iocb,
++			 struct iocb *iocb)
++{
++	return -EINVAL;
++}
++
++struct kiocb *lookup_kiocb(struct kioctx *ctx, struct iocb *iocb, u32 key)
++{
++	return 0;
++}
++
++EXPORT_SYMBOL(aio_complete);
++EXPORT_SYMBOL(aio_put_req);
++EXPORT_SYMBOL(wait_on_sync_kiocb);
++
++#endif
+Index: linux-2.6.22.1/init/Kconfig
+===================================================================
+--- linux-2.6.22.1.orig/init/Kconfig
++++ linux-2.6.22.1/init/Kconfig
+@@ -353,6 +353,14 @@ config SYSENTER
+ 	  Disabling this feature removes sysenter handling as well as
+ 	  vsyscall fixmaps.
+ 
++config AIO
++	default y
++	bool "Enable AIO support" if EMBEDDED
++	help
++	  This option enables POSIX async IO which may be used by
++	  some high performance threaded applications. Disabling
++	  this option saves about 5k.
++
+ config MEASURE_INLINES
+ 	default n
+ 	bool "Enable inline measurement" if EMBEDDED
+Index: linux-2.6.22.1/kernel/sys_ni.c
+===================================================================
+--- linux-2.6.22.1.orig/kernel/sys_ni.c
++++ linux-2.6.22.1/kernel/sys_ni.c
+@@ -112,6 +112,11 @@ cond_syscall(sys_vm86old);
+ cond_syscall(sys_vm86);
+ cond_syscall(compat_sys_ipc);
+ cond_syscall(compat_sys_sysctl);
++cond_syscall(sys_io_setup);
++cond_syscall(sys_io_destroy);
++cond_syscall(sys_io_submit);
++cond_syscall(sys_io_cancel);
++cond_syscall(sys_io_getevents);
+ 
+ /* arch-specific weak syscall entries */
+ cond_syscall(sys_pciconfig_read);
+Index: linux-2.6.22.1/kernel/sysctl.c
+===================================================================
+--- linux-2.6.22.1.orig/kernel/sysctl.c
++++ linux-2.6.22.1/kernel/sysctl.c
+@@ -981,6 +981,7 @@ static ctl_table fs_table[] = {
+ 		.mode		= 0644,
+ 		.proc_handler	= &proc_dointvec,
+ 	},
++#ifdef CONFIG_AIO
+ 	{
+ 		.ctl_name	= FS_AIO_NR,
+ 		.procname	= "aio-nr",
+@@ -997,6 +998,7 @@ static ctl_table fs_table[] = {
+ 		.mode		= 0644,
+ 		.proc_handler	= &proc_doulongvec_minmax,
+ 	},
++#endif
+ #ifdef CONFIG_INOTIFY_USER
+ 	{
+ 		.ctl_name	= FS_INOTIFY,
Index: packages/kernel/patches/tiny_m57sli/tiny/inflate-share-crc.patch
===================================================================
--- packages/kernel/patches/tiny_m57sli/tiny/inflate-share-crc.patch	(revision 0)
+++ packages/kernel/patches/tiny_m57sli/tiny/inflate-share-crc.patch	(revision 0)
@@ -0,0 +1,98 @@
+inflate: make in-core inflate share common CRC code
+
+Signed-off-by: Matt Mackall <mpm@selenic.com>
+
+Index: linux-2.6.16.19/lib/inflate.c
+===================================================================
+--- linux-2.6.16.19.orig/lib/inflate.c	2006-06-23 17:06:36.000000000 -0700
++++ linux-2.6.16.19/lib/inflate.c	2006-06-23 17:06:37.000000000 -0700
+@@ -144,23 +144,46 @@
+ 		malloc_ptr = free_mem_ptr;
+ }
+ 
+-static u8 INITDATA window[0x8000]; /* use a statically allocated window */
++static u8 window[0x8000]; /* use a statically allocated window */
++
++/* simple CRC calculation */
++static u32 crc_32_tab[256];
++#define CRCPOLY_LE 0xedb88320
++
++static void makecrc(void)
++{
++	unsigned i, j;
++	u32 c = 1;
++
++	for (i = 128; i; i >>= 1) {
++		c = (c >> 1) ^ ((c & 1) ? CRCPOLY_LE : 0);
++		for (j = 0; j < 256; j += 2 * i)
++			crc_32_tab[i + j] = c ^ crc_32_tab[j];
++	}
++}
++
++static u32 crc32_le(u32 crc, unsigned char const *p, size_t len)
++{
++	while (len--)
++		crc = crc_32_tab[(*p++ ^ crc) & 0xff] ^ (crc >> 8);
++
++	return crc;
++}
+ 
+ #else
+ 
+ #include <linux/module.h>
++#include <linux/crc32.h>
+ 
+ static u8 *window; /* dynamically allocate */
+ #define malloc(a) kmalloc(a, GFP_KERNEL)
+ #define free(a) kfree(a)
++#define makecrc()
+ #define INIT __init
+ #define INITDATA __initdata
+ 
+ #endif
+ 
+-static u32 crc_32_tab[256];
+-#define CRCPOLY_LE 0xedb88320
+-
+ /* Huffman code lookup table entry--this entry is four bytes for machines
+    that have 16-bit pointers (e.g. PC's in the small or medium model).
+    Valid extra bits are 0..13.  e == 15 is EOB (end of block), e == 16
+@@ -205,12 +228,7 @@
+ 
+ static void flush_output(struct iostate *io)
+ {
+-	int i;
+-
+-	for (i = 0; i < io->opos; i++)
+-		io->crc = crc_32_tab[(io->window[i] ^ (int)io->crc) & 0xff]
+-			^ (io->crc >> 8);
+-
++	io->crc = crc32_le(io->crc, io->window, io->opos);
+ 	io->flush(io->window, io->opos);
+ 	io->ototal += io->opos;
+ 	io->opos = 0;
+@@ -908,24 +926,6 @@
+ 	return 0;
+ }
+ 
+-/**********************************************************************
+- *
+- * The following are support routines for inflate.c
+- *
+- **********************************************************************/
+-
+-static void INIT makecrc(void)
+-{
+-	unsigned i, j;
+-	u32 c = 1;
+-
+-	for (i = 128; i; i >>= 1) {
+-		c = (c >> 1) ^ ((c & 1) ? CRCPOLY_LE : 0);
+-		for (j = 0; j < 256; j += 2 * i)
+-			crc_32_tab[i + j] = c ^ crc_32_tab[j];
+-	}
+-}
+-
+ /* gzip flag byte */
+ #define ASCII_FLAG   0x01 /* bit 0 set: file probably ASCII text */
+ #define CONTINUATION 0x02 /* bit 1 set: continuation of multi-part gzip file */
Index: packages/kernel/patches/tiny_m57sli/tiny/pci-quirks.patch
===================================================================
--- packages/kernel/patches/tiny_m57sli/tiny/pci-quirks.patch	(revision 0)
+++ packages/kernel/patches/tiny_m57sli/tiny/pci-quirks.patch	(revision 0)
@@ -0,0 +1,91 @@
+
+configurable support for PCI quirks
+
+From: Zwane Mwaikambo <zwane@arm.linux.org.uk>
+Subject: [PATCH][2.6-tiny] remove PCI quirks
+
+Index: linux-2.6.1-rc1-tiny2/init/Kconfig===================================================================
+RCS file: /home/cvsroot/linux-2.6.1-rc1-tiny2/init/Kconfig,v
+retrieving revision 1.2
+diff -u -p -B -r1.2 Kconfig
+
+
+ tiny-mpm/drivers/pci/quirks.c |   26 +++++++++++++++-----------
+ tiny-mpm/init/Kconfig         |    8 ++++++++
+ 2 files changed, 23 insertions(+), 11 deletions(-)
+
+Index: linux-2.6.16.19/drivers/pci/quirks.c
+===================================================================
+--- linux-2.6.16.19.orig/drivers/pci/quirks.c	2006-06-23 17:05:11.000000000 -0700
++++ linux-2.6.16.19/drivers/pci/quirks.c	2006-06-23 17:06:28.000000000 -0700
+@@ -24,6 +24,10 @@
+ #include <linux/acpi.h>
+ #include "pci.h"
+ 
++int isa_dma_bridge_buggy;		/* Exported */
++int pci_pci_problems;
++
++#ifdef CONFIG_PCI_QUIRKS
+ /* Deal with broken BIOS'es that neglect to enable passive release,
+    which can cause problems in combination with the 82441FX/PPro MTRRs */
+ static void __devinit quirk_passive_release(struct pci_dev *dev)
+@@ -44,14 +48,13 @@
+ }
+ DECLARE_PCI_FIXUP_FINAL(PCI_VENDOR_ID_INTEL,	PCI_DEVICE_ID_INTEL_82441,	quirk_passive_release );
+ 
+-/*  The VIA VP2/VP3/MVP3 seem to have some 'features'. There may be a workaround
+-    but VIA don't answer queries. If you happen to have good contacts at VIA
+-    ask them for me please -- Alan 
+-    
+-    This appears to be BIOS not version dependent. So presumably there is a 
+-    chipset level fix */
+-int isa_dma_bridge_buggy;		/* Exported */
+-    
++/*  The VIA VP2/VP3/MVP3 seem to have some 'features'. There may be a
++    workaround but VIA don't answer queries. If you happen to have
++    good contacts at VIA ask them for me please -- Alan
++
++    This appears to be BIOS not version dependent. So presumably there
++    is a chipset level fix */
++
+ static void __devinit quirk_isa_dma_hangs(struct pci_dev *dev)
+ {
+ 	if (!isa_dma_bridge_buggy) {
+@@ -71,8 +74,6 @@
+ DECLARE_PCI_FIXUP_FINAL(PCI_VENDOR_ID_NEC,	PCI_DEVICE_ID_NEC_CBUS_2,	quirk_isa_dma_hangs );
+ DECLARE_PCI_FIXUP_FINAL(PCI_VENDOR_ID_NEC,	PCI_DEVICE_ID_NEC_CBUS_3,	quirk_isa_dma_hangs );
+ 
+-int pci_pci_problems;
+-
+ /*
+  *	Chipsets where PCI->PCI transfers vanish or hang
+  */
+@@ -1391,3 +1392,9 @@
+ #ifdef CONFIG_HOTPLUG
+ EXPORT_SYMBOL(pci_fixup_device);
+ #endif
++
++#else
++void pci_fixup_device(enum pci_fixup_pass pass, struct pci_dev *dev)
++{
++}
++#endif /* CONFIG_PCI_QUIRKS */
+Index: linux-2.6.16.19/init/Kconfig
+===================================================================
+--- linux-2.6.16.19.orig/init/Kconfig	2006-06-23 17:06:28.000000000 -0700
++++ linux-2.6.16.19/init/Kconfig	2006-06-23 17:06:28.000000000 -0700
+@@ -620,6 +620,14 @@
+           BIOS code. Disabling scanning saves about 5k of
+           initialization code.
+ 
++config PCI_QUIRKS
++	default y
++	bool "Enable PCI quirk workarounds" if EMBEDDED
++	help
++	  This enables workarounds for various PCI chipset bugs/quirks.
++	  Disable this only if you know your target machine is unaffected
++	  by PCI quirks.
++
+ endmenu		# General setup
+ 
+ config CRC32_CALC
Index: packages/kernel/patches/tiny_m57sli/tiny/namei-inlines.patch
===================================================================
--- packages/kernel/patches/tiny_m57sli/tiny/namei-inlines.patch	(revision 0)
+++ packages/kernel/patches/tiny_m57sli/tiny/namei-inlines.patch	(revision 0)
@@ -0,0 +1,38 @@
+
+uninline various namei.c functions
+
+
+ fs/namei.c |    6 	3 +	3 -	0 !
+ 1 files changed, 3 insertions(+), 3 deletions(-)
+
+Index: linux-2.6.22.1/fs/namei.c
+===================================================================
+--- linux-2.6.22.1.orig/fs/namei.c
++++ linux-2.6.22.1/fs/namei.c
+@@ -607,7 +607,7 @@ static __always_inline int __do_follow_l
+ 		char *s = nd_get_link(nd);
+ 		error = 0;
+ 		if (s)
+-			error = __vfs_follow_link(nd, s);
++			error = vfs_follow_link(nd, s);
+ 		if (dentry->d_inode->i_op->put_link)
+ 			dentry->d_inode->i_op->put_link(dentry, nd, cookie);
+ 	}
+@@ -624,7 +624,7 @@ static __always_inline int __do_follow_l
+  * Without that kind of total limit, nasty chains of consecutive
+  * symlinks can cause almost arbitrarily long lookups. 
+  */
+-static inline int do_follow_link(struct path *path, struct nameidata *nd)
++static int do_follow_link(struct path *path, struct nameidata *nd)
+ {
+ 	int err = -ELOOP;
+ 	if (current->link_count >= MAX_NESTED_LINKS)
+@@ -1439,7 +1439,7 @@ static int may_delete(struct inode *dir,
+  *  3. We should have write and exec permissions on dir
+  *  4. We can't do it if dir is immutable (done in permission())
+  */
+-static inline int may_create(struct inode *dir, struct dentry *child,
++static int may_create(struct inode *dir, struct dentry *child,
+ 			     struct nameidata *nd)
+ {
+ 	if (child->d_inode)
Index: packages/kernel/patches/tiny_m57sli/tiny/inflate-formatting.patch
===================================================================
--- packages/kernel/patches/tiny_m57sli/tiny/inflate-formatting.patch	(revision 0)
+++ packages/kernel/patches/tiny_m57sli/tiny/inflate-formatting.patch	(revision 0)
@@ -0,0 +1,1947 @@
+
+inflate  Lindent and general reformatting
+
+
+ tiny-mpm/lib/inflate.c | 1749 ++++++++++++++++++++++++-------------------------
+ 1 files changed, 866 insertions(+), 883 deletions(-)
+
+Index: linux-2.6.16.19/lib/inflate.c
+===================================================================
+--- linux-2.6.16.19.orig/lib/inflate.c	2006-06-23 17:05:11.000000000 -0700
++++ linux-2.6.16.19/lib/inflate.c	2006-06-23 17:06:29.000000000 -0700
+@@ -1,17 +1,17 @@
+ #define DEBG(x)
+ #define DEBG1(x)
+ /* inflate.c -- Not copyrighted 1992 by Mark Adler
+-   version c10p1, 10 January 1993 */
+-
+-/* 
++ * version c10p1, 10 January 1993
++ *
+  * Adapted for booting Linux by Hannu Savolainen 1993
+- * based on gzip-1.0.3 
++ * based on gzip-1.0.3
+  *
+  * Nicolas Pitre <nico@cam.org>, 1999/04/14 :
+- *   Little mods for all variable to reside either into rodata or bss segments
+- *   by marking constant variables with 'const' and initializing all the others
+- *   at run-time only.  This allows for the kernel uncompressor to run
+- *   directly from Flash or ROM memory on embedded systems.
++ *   Little mods for all variable to reside either into rodata or bss
++ *   segments by marking constant variables with 'const' and
++ *   initializing all the others at run-time only. This allows for the
++ *   kernel uncompressor to run directly from Flash or ROM memory on
++ *   embedded systems.
+  */
+ 
+ /*
+@@ -50,14 +50,13 @@
+    chunks), otherwise the dynamic method is used.  In the latter case, the
+    codes are customized to the probabilities in the current block, and so
+    can code it much better than the pre-determined fixed codes.
+- 
++
+    The Huffman codes themselves are decoded using a multi-level table
+    lookup, in order to maximize the speed of decoding plus the speed of
+    building the decoding tables.  See the comments below that precede the
+    lbits and dbits tuning parameters.
+  */
+ 
+-
+ /*
+    Notes beyond the 1.93a appnote.txt:
+ 
+@@ -122,7 +121,7 @@
+ #ifndef INIT
+ #define INIT
+ #endif
+-	
++
+ #define slide window
+ 
+ /* Huffman code lookup table entry--this entry is four bytes for machines
+@@ -133,17 +132,16 @@
+    an unused code.  If a code with e == 99 is looked up, this implies an
+    error in the data. */
+ struct huft {
+-  uch e;                /* number of extra bits or operation */
+-  uch b;                /* number of bits in this code or subcode */
+-  union {
+-    ush n;              /* literal, length base, or distance base */
+-    struct huft *t;     /* pointer to next level of table */
+-  } v;
++	uch e;			/* number of extra bits or operation */
++	uch b;			/* number of bits in this code or subcode */
++	union {
++		ush n;		/* literal, length base, or distance base */
++		struct huft *t;	/* pointer to next level of table */
++	} v;
+ };
+ 
+-
+ /* Function prototypes */
+-STATIC int INIT huft_build OF((unsigned *, unsigned, unsigned, 
++STATIC int INIT huft_build OF((unsigned *, unsigned, unsigned,
+ 		const ush *, const ush *, struct huft **, int *));
+ STATIC int INIT huft_free OF((struct huft *));
+ STATIC int INIT inflate_codes OF((struct huft *, struct huft *, int, int));
+@@ -153,7 +151,6 @@
+ STATIC int INIT inflate_block OF((int *));
+ STATIC int INIT inflate OF((void));
+ 
+-
+ /* The inflate algorithm uses a sliding 32 K byte window on the uncompressed
+    stream to find repeated byte strings.  This is implemented here as a
+    circular buffer.  The index is updated simply by incrementing and then
+@@ -167,29 +164,44 @@
+ #define flush_output(w) (wp=(w),flush_window())
+ 
+ /* Tables for deflate from PKZIP's appnote.txt. */
+-static const unsigned border[] = {    /* Order of the bit length code lengths */
+-        16, 17, 18, 0, 8, 7, 9, 6, 10, 5, 11, 4, 12, 3, 13, 2, 14, 1, 15};
+-static const ush cplens[] = {         /* Copy lengths for literal codes 257..285 */
+-        3, 4, 5, 6, 7, 8, 9, 10, 11, 13, 15, 17, 19, 23, 27, 31,
+-        35, 43, 51, 59, 67, 83, 99, 115, 131, 163, 195, 227, 258, 0, 0};
+-        /* note: see note #13 above about the 258 in this list. */
+-static const ush cplext[] = {         /* Extra bits for literal codes 257..285 */
+-        0, 0, 0, 0, 0, 0, 0, 0, 1, 1, 1, 1, 2, 2, 2, 2,
+-        3, 3, 3, 3, 4, 4, 4, 4, 5, 5, 5, 5, 0, 99, 99}; /* 99==invalid */
+-static const ush cpdist[] = {         /* Copy offsets for distance codes 0..29 */
+-        1, 2, 3, 4, 5, 7, 9, 13, 17, 25, 33, 49, 65, 97, 129, 193,
+-        257, 385, 513, 769, 1025, 1537, 2049, 3073, 4097, 6145,
+-        8193, 12289, 16385, 24577};
+-static const ush cpdext[] = {         /* Extra bits for distance codes */
+-        0, 0, 0, 0, 1, 1, 2, 2, 3, 3, 4, 4, 5, 5, 6, 6,
+-        7, 7, 8, 8, 9, 9, 10, 10, 11, 11,
+-        12, 12, 13, 13};
+ 
++/* Order of the bit length code lengths */
++static const unsigned border[] = {
++	16, 17, 18, 0, 8, 7, 9, 6, 10, 5, 11, 4, 12, 3, 13, 2, 14, 1, 15
++};
++
++/* Copy lengths for literal codes 257..285 */
++static const ush cplens[] = {
++	3, 4, 5, 6, 7, 8, 9, 10, 11, 13, 15, 17, 19, 23, 27, 31,
++	35, 43, 51, 59, 67, 83, 99, 115, 131, 163, 195, 227, 258, 0, 0
++};
++
++/* Extra bits for literal codes 257..285
++ * note: see note #13 above about the 258 in this list.
++ * 99==invalid
++ */
++static const ush cplext[] = {
++	0, 0, 0, 0, 0, 0, 0, 0, 1, 1, 1, 1, 2, 2, 2, 2,
++	3, 3, 3, 3, 4, 4, 4, 4, 5, 5, 5, 5, 0, 99, 99
++};
++
++/* Copy offsets for distance codes 0..29 */
++static const ush cpdist[] = {
++	1, 2, 3, 4, 5, 7, 9, 13, 17, 25, 33, 49, 65, 97, 129, 193,
++	257, 385, 513, 769, 1025, 1537, 2049, 3073, 4097, 6145,
++	8193, 12289, 16385, 24577
++};
+ 
++/* Extra bits for distance codes */
++static const ush cpdext[] = {
++	0, 0, 0, 0, 1, 1, 2, 2, 3, 3, 4, 4, 5, 5, 6, 6,
++	7, 7, 8, 8, 9, 9, 10, 10, 11, 11,
++	12, 12, 13, 13
++};
+ 
+ /* Macros for inflate() bit peeking and grabbing.
+    The usage is:
+-   
++
+         NEEDBITS(j)
+         x = b & mask_bits[j];
+         DUMPBITS(j)
+@@ -217,19 +229,18 @@
+    the stream.
+  */
+ 
+-STATIC ulg bb;                         /* bit buffer */
+-STATIC unsigned bk;                    /* bits in bit buffer */
++STATIC ulg bb;			/* bit buffer */
++STATIC unsigned bk;		/* bits in bit buffer */
+ 
+ STATIC const ush mask_bits[] = {
+-    0x0000,
+-    0x0001, 0x0003, 0x0007, 0x000f, 0x001f, 0x003f, 0x007f, 0x00ff,
+-    0x01ff, 0x03ff, 0x07ff, 0x0fff, 0x1fff, 0x3fff, 0x7fff, 0xffff
++	0x0000,
++	0x0001, 0x0003, 0x0007, 0x000f, 0x001f, 0x003f, 0x007f, 0x00ff,
++	0x01ff, 0x03ff, 0x07ff, 0x0fff, 0x1fff, 0x3fff, 0x7fff, 0xffff
+ };
+ 
+ #define NEXTBYTE()  ({ int v = get_byte(); if (v < 0) goto underrun; (uch)v; })
+-#define NEEDBITS(n) {while(k<(n)){b|=((ulg)NEXTBYTE())<<k;k+=8;}}
+-#define DUMPBITS(n) {b>>=(n);k-=(n);}
+-
++#define NEEDBITS(n) do {while(k<(n)){b|=((ulg)NEXTBYTE())<<k;k+=8;}} while(0)
++#define DUMPBITS(n) do {b>>=(n);k-=(n);} while(0)
+ 
+ /*
+    Huffman code decoding is performed using a multi-level table lookup.
+@@ -263,771 +274,757 @@
+    possibly even between compilers.  Your mileage may vary.
+  */
+ 
+-
+-STATIC const int lbits = 9;          /* bits in base literal/length lookup table */
+-STATIC const int dbits = 6;          /* bits in base distance lookup table */
+-
++STATIC const int lbits = 9;	/* bits in base literal/length lookup table */
++STATIC const int dbits = 6;	/* bits in base distance lookup table */
+ 
+ /* If BMAX needs to be larger than 16, then h and x[] should be ulg. */
+-#define BMAX 16         /* maximum bit length of any code (16 for explode) */
+-#define N_MAX 288       /* maximum number of codes in any set */
+-
++#define BMAX 16		/* maximum bit length of any code (16 for explode) */
++#define N_MAX 288	/* maximum number of codes in any set */
+ 
+-STATIC unsigned hufts;         /* track memory usage */
++STATIC unsigned hufts;		/* track memory usage */
+ 
+-
+-STATIC int INIT huft_build(
+-	unsigned *b,            /* code lengths in bits (all assumed <= BMAX) */
+-	unsigned n,             /* number of codes (assumed <= N_MAX) */
+-	unsigned s,             /* number of simple-valued codes (0..s-1) */
+-	const ush *d,           /* list of base values for non-simple codes */
+-	const ush *e,           /* list of extra bits for non-simple codes */
+-	struct huft **t,        /* result: starting table */
+-	int *m                  /* maximum lookup bits, returns actual */
+-	)
+-/* Given a list of code lengths and a maximum table size, make a set of
+-   tables to decode that set of codes.  Return zero on success, one if
+-   the given code set is incomplete (the tables are still built in this
+-   case), two if the input is invalid (all zero length codes or an
+-   oversubscribed set of lengths), and three if not enough memory. */
++/*
++ * huft-build - build a huffman decoding table
++ * @b: code lengths in bits (all assumed <= BMAX)
++ * @n: number of codes (assumed <= N_MAX)
++ * @s: number of simple-valued codes (0..s-1)
++ * @d: list of base values for non-simple codes
++ * @e: list of extra bits for non-simple codes
++ * @t: returns pointer to starting table
++ * @m: maximum lookup bits, returns actual
++ *
++ * Given a list of code lengths and a maximum table size, make a set
++ * of tables to decode that set of codes. Return zero on success, one
++ * if the given code set is incomplete (the tables are still built in
++ * this case), two if the input is invalid (all zero length codes or
++ * an oversubscribed set of lengths), and three if not enough
++ * memory.
++ */
++STATIC int INIT huft_build(unsigned *b, unsigned n, unsigned s, const ush * d,
++		      const ush * e, struct huft **t, int *m)
+ {
+-  unsigned a;                   /* counter for codes of length k */
+-  unsigned c[BMAX+1];           /* bit length count table */
+-  unsigned f;                   /* i repeats in table every f entries */
+-  int g;                        /* maximum code length */
+-  int h;                        /* table level */
+-  register unsigned i;          /* counter, current code */
+-  register unsigned j;          /* counter */
+-  register int k;               /* number of bits in current code */
+-  int l;                        /* bits per table (returned in m) */
+-  register unsigned *p;         /* pointer into c[], b[], or v[] */
+-  register struct huft *q;      /* points to current table */
+-  struct huft r;                /* table entry for structure assignment */
+-  struct huft *u[BMAX];         /* table stack */
+-  unsigned v[N_MAX];            /* values in order of bit length */
+-  register int w;               /* bits before this table == (l * h) */
+-  unsigned x[BMAX+1];           /* bit offsets, then code stack */
+-  unsigned *xp;                 /* pointer into x */
+-  int y;                        /* number of dummy codes added */
+-  unsigned z;                   /* number of entries in current table */
+-
+-DEBG("huft1 ");
+-
+-  /* Generate counts for each bit length */
+-  memzero(c, sizeof(c));
+-  p = b;  i = n;
+-  do {
+-    Tracecv(*p, (stderr, (n-i >= ' ' && n-i <= '~' ? "%c %d\n" : "0x%x %d\n"), 
+-	    n-i, *p));
+-    c[*p]++;                    /* assume all entries <= BMAX */
+-    p++;                      /* Can't combine with above line (Solaris bug) */
+-  } while (--i);
+-  if (c[0] == n)                /* null input--all zero length codes */
+-  {
+-    *t = (struct huft *)NULL;
+-    *m = 0;
+-    return 2;
+-  }
+-
+-DEBG("huft2 ");
+-
+-  /* Find minimum and maximum length, bound *m by those */
+-  l = *m;
+-  for (j = 1; j <= BMAX; j++)
+-    if (c[j])
+-      break;
+-  k = j;                        /* minimum code length */
+-  if ((unsigned)l < j)
+-    l = j;
+-  for (i = BMAX; i; i--)
+-    if (c[i])
+-      break;
+-  g = i;                        /* maximum code length */
+-  if ((unsigned)l > i)
+-    l = i;
+-  *m = l;
+-
+-DEBG("huft3 ");
+-
+-  /* Adjust last length count to fill out codes, if needed */
+-  for (y = 1 << j; j < i; j++, y <<= 1)
+-    if ((y -= c[j]) < 0)
+-      return 2;                 /* bad input: more codes than bits */
+-  if ((y -= c[i]) < 0)
+-    return 2;
+-  c[i] += y;
+-
+-DEBG("huft4 ");
+-
+-  /* Generate starting offsets into the value table for each length */
+-  x[1] = j = 0;
+-  p = c + 1;  xp = x + 2;
+-  while (--i) {                 /* note that i == g from above */
+-    *xp++ = (j += *p++);
+-  }
+-
+-DEBG("huft5 ");
+-
+-  /* Make a table of values in order of bit lengths */
+-  p = b;  i = 0;
+-  do {
+-    if ((j = *p++) != 0)
+-      v[x[j]++] = i;
+-  } while (++i < n);
+-  n = x[g];                   /* set n to length of v */
+-
+-DEBG("h6 ");
+-
+-  /* Generate the Huffman codes and for each, make the table entries */
+-  x[0] = i = 0;                 /* first Huffman code is zero */
+-  p = v;                        /* grab values in bit order */
+-  h = -1;                       /* no tables yet--level -1 */
+-  w = -l;                       /* bits decoded == (l * h) */
+-  u[0] = (struct huft *)NULL;   /* just to keep compilers happy */
+-  q = (struct huft *)NULL;      /* ditto */
+-  z = 0;                        /* ditto */
+-DEBG("h6a ");
+-
+-  /* go through the bit lengths (k already is bits in shortest code) */
+-  for (; k <= g; k++)
+-  {
+-DEBG("h6b ");
+-    a = c[k];
+-    while (a--)
+-    {
+-DEBG("h6b1 ");
+-      /* here i is the Huffman code of length k bits for value *p */
+-      /* make tables up to required level */
+-      while (k > w + l)
+-      {
+-DEBG1("1 ");
+-        h++;
+-        w += l;                 /* previous table always l bits */
+-
+-        /* compute minimum size table less than or equal to l bits */
+-        z = (z = g - w) > (unsigned)l ? l : z;  /* upper limit on table size */
+-        if ((f = 1 << (j = k - w)) > a + 1)     /* try a k-w bit table */
+-        {                       /* too few codes for k-w bit table */
+-DEBG1("2 ");
+-          f -= a + 1;           /* deduct codes from patterns left */
+-          xp = c + k;
+-          if (j < z)
+-            while (++j < z)       /* try smaller tables up to z bits */
+-            {
+-              if ((f <<= 1) <= *++xp)
+-                break;            /* enough codes to use up j bits */
+-              f -= *xp;           /* else deduct codes from patterns */
+-            }
+-        }
+-DEBG1("3 ");
+-        z = 1 << j;             /* table entries for j-bit table */
+-
+-        /* allocate and link in new table */
+-        if ((q = (struct huft *)malloc((z + 1)*sizeof(struct huft))) ==
+-            (struct huft *)NULL)
+-        {
+-          if (h)
+-            huft_free(u[0]);
+-          return 3;             /* not enough memory */
+-        }
+-DEBG1("4 ");
+-        hufts += z + 1;         /* track memory usage */
+-        *t = q + 1;             /* link to list for huft_free() */
+-        *(t = &(q->v.t)) = (struct huft *)NULL;
+-        u[h] = ++q;             /* table starts after link */
+-
+-DEBG1("5 ");
+-        /* connect to last table, if there is one */
+-        if (h)
+-        {
+-          x[h] = i;             /* save pattern for backing up */
+-          r.b = (uch)l;         /* bits to dump before this table */
+-          r.e = (uch)(16 + j);  /* bits in this table */
+-          r.v.t = q;            /* pointer to this table */
+-          j = i >> (w - l);     /* (get around Turbo C bug) */
+-          u[h-1][j] = r;        /* connect to last table */
+-        }
+-DEBG1("6 ");
+-      }
+-DEBG("h6c ");
+-
+-      /* set up table entry in r */
+-      r.b = (uch)(k - w);
+-      if (p >= v + n)
+-        r.e = 99;               /* out of values--invalid code */
+-      else if (*p < s)
+-      {
+-        r.e = (uch)(*p < 256 ? 16 : 15);    /* 256 is end-of-block code */
+-        r.v.n = (ush)(*p);             /* simple code is just the value */
+-	p++;                           /* one compiler does not like *p++ */
+-      }
+-      else
+-      {
+-        r.e = (uch)e[*p - s];   /* non-simple--look up in lists */
+-        r.v.n = d[*p++ - s];
+-      }
+-DEBG("h6d ");
+-
+-      /* fill code-like entries with r */
+-      f = 1 << (k - w);
+-      for (j = i >> w; j < z; j += f)
+-        q[j] = r;
+-
+-      /* backwards increment the k-bit code i */
+-      for (j = 1 << (k - 1); i & j; j >>= 1)
+-        i ^= j;
+-      i ^= j;
+-
+-      /* backup over finished tables */
+-      while ((i & ((1 << w) - 1)) != x[h])
+-      {
+-        h--;                    /* don't need to update q */
+-        w -= l;
+-      }
+-DEBG("h6e ");
+-    }
+-DEBG("h6f ");
+-  }
++	unsigned a;		/* counter for codes of length k */
++	unsigned c[BMAX + 1];	/* bit length count table */
++	unsigned f;		/* i repeats in table every f entries */
++	int g;			/* maximum code length */
++	int h;			/* table level */
++	register unsigned i;	/* counter, current code */
++	register unsigned j;	/* counter */
++	register int k;		/* number of bits in current code */
++	int l;			/* bits per table (returned in m) */
++	register unsigned *p;	/* pointer into c[], b[], or v[] */
++	register struct huft *q;	/* points to current table */
++	struct huft r;		/* table entry for structure assignment */
++	struct huft *u[BMAX];	/* table stack */
++	unsigned v[N_MAX];	/* values in order of bit length */
++	register int w;		/* bits before this table == (l * h) */
++	unsigned x[BMAX + 1];	/* bit offsets, then code stack */
++	unsigned *xp;		/* pointer into x */
++	int y;			/* number of dummy codes added */
++	unsigned z;		/* number of entries in current table */
++
++	DEBG("huft1 ");
++
++	/* Generate counts for each bit length */
++	memzero(c, sizeof(c));
++	p = b;
++	i = n;
++	do {
++		Tracecv(*p, (stderr, (n - i >= ' ' && n - i <= '~' ?
++				      "%c %d\n" : "0x%x %d\n"), n - i, *p));
++		c[*p]++;	/* assume all entries <= BMAX */
++		p++;
++	} while (--i);
++
++	if (c[0] == n) {	/* null input--all zero length codes */
++		*t = (struct huft *)NULL;
++		*m = 0;
++		return 2;
++	}
++
++	DEBG("huft2 ");
++
++	/* Find minimum and maximum length, bound *m by those */
++	l = *m;
++	for (j = 1; j <= BMAX; j++)
++		if (c[j])
++			break;
++	k = j; /* minimum code length */
++
++	if ((unsigned)l < j)
++		l = j;
++	for (i = BMAX; i; i--)
++		if (c[i])
++			break;
++	g = i; /* maximum code length */
++
++	if ((unsigned)l > i)
++		l = i;
++	*m = l;
++
++	DEBG("huft3 ");
++
++	/* Adjust last length count to fill out codes, if needed */
++	for (y = 1 << j; j < i; j++, y <<= 1)
++		if ((y -= c[j]) < 0)
++			return 2; /* bad input: more codes than bits */
++
++	if ((y -= c[i]) < 0)
++		return 2;
++	c[i] += y;
++
++	DEBG("huft4 ");
++
++	/* Generate starting offsets into the value table for each length */
++	x[1] = j = 0;
++	p = c + 1;
++	xp = x + 2;
++	/* note that i == g from above */
++	while (--i)
++		*xp++ = (j += *p++);
++
++	DEBG("huft5 ");
++
++	/* Make a table of values in order of bit lengths */
++	p = b;
++	i = 0;
++	do {
++		if ((j = *p++) != 0)
++			v[x[j]++] = i;
++	} while (++i < n);
++
++	n = x[g];                   /* set n to length of v */
++
++	DEBG("h6 ");
++
++	/* Generate the Huffman codes and for each, make the table entries */
++	x[0] = i = 0; /* first Huffman code is zero */
++	p = v; /* grab values in bit order */
++	h = -1; /* no tables yet--level -1 */
++	w = -l;	/* bits decoded == (l * h) */
++	u[0] = (struct huft *)NULL; /* just to keep compilers happy */
++	q = (struct huft *)NULL; /* ditto */
++	z = 0; /* ditto */
++	DEBG("h6a ");
++
++	/* go through the bit lengths (k already is bits in shortest code) */
++	for (; k <= g; k++) {
++		DEBG("h6b ");
++		a = c[k];
++		while (a--) {
++			DEBG("h6b1 ");
++			/* i is the Huffman code of length k for value *p */
++			/* make tables up to required level */
++			while (k > w + l) {
++				DEBG1("1 ");
++				h++;
++				w += l;	/* previous table always l bits */
++
++				/* compute min size <= l bits */
++				/* upper limit on table size */
++				z = (z = g - w) > (unsigned)l ? l : z;
++
++				/* try a k-w bit table */
++				if ((f = 1 << (j = k - w)) > a + 1) {
++					/* too few codes for k-w bit table */
++					DEBG1("2 ");
++					/* deduct codes from patterns left */
++					f -= a + 1;
++					xp = c + k;
++					/* try smaller tables up to z bits */
++					if (j < z) {
++						/* enough codes for j bits? */
++						while (++j < z) {
++							if ((f <<= 1) <= *++xp)
++								break;
++							/* deduct from pats */
++							f -= *xp;
++						}
++					}
++    				}
++
++				DEBG1("3 ");
++				/* table entries for j-bit table */
++				z = 1 << j;
++
++				/* allocate and link in new table */
++				if ((q = (struct huft *)malloc(
++					     (z + 1) * sizeof(struct huft)))
++				    == (struct huft *)NULL) {
++					if (h)
++						huft_free(u[0]);
++					return 3;	/* not enough memory */
++				}
++				DEBG1("4 ");
++				hufts += z + 1;	/* track memory usage */
++				*t = q + 1; /* link to list for huft_free */
++				*(t = &(q->v.t)) = (struct huft *)NULL;
++				u[h] = ++q;	/* table starts after link */
++
++				DEBG1("5 ");
++				/* connect to last table, if there is one */
++				if (h) {
++					/* save pattern for backing up */
++					x[h] = i;
++					/* bits to dump before this table */
++					r.b = (uch)l;
++					/* bits in this table */
++					r.e = (uch)(16 + j);
++					/* pointer to this table */
++					r.v.t = q;
++					/* (get around Turbo C bug) */
++					j = i >> (w - l);
++					/* connect to last table */
++					u[h - 1][j] = r;
++				}
++				DEBG1("6 ");
++			}
++			DEBG("h6c ");
++
++			/* set up table entry in r */
++			r.b = (uch) (k - w);
++			if (p >= v + n)
++				r.e = 99; /* out of values--invalid code */
++			else if (*p < s) {
++				/* 256 is end-of-block code */
++				r.e = (uch) (*p < 256 ? 16 : 15);
++				/* simple code is just the value */
++				r.v.n = (ush) (*p);
++				/* one compiler does not like *p++ */
++				p++;
++			} else {
++				/* non-simple--look up in lists */
++				r.e = (uch)e[*p - s];
++				r.v.n = d[*p++ - s];
++			}
++			DEBG("h6d ");
++
++			/* fill code-like entries with r */
++			f = 1 << (k - w);
++			for (j = i >> w; j < z; j += f)
++				q[j] = r;
++
++			/* backwards increment the k-bit code i */
++			for (j = 1 << (k - 1); i & j; j >>= 1)
++				i ^= j;
++			i ^= j;
++
++			/* backup over finished tables */
++			while ((i & ((1 << w) - 1)) != x[h]) {
++				h--;	/* don't need to update q */
++				w -= l;
++			}
++			DEBG("h6e ");
++		}
++		DEBG("h6f ");
++	}
+ 
+-DEBG("huft7 ");
++	DEBG("huft7 ");
+ 
+-  /* Return true (1) if we were given an incomplete table */
+-  return y != 0 && g != 1;
++	/* Return true (1) if we were given an incomplete table */
++	return y != 0 && g != 1;
+ }
+ 
+-
+-
+-STATIC int INIT huft_free(
+-	struct huft *t         /* table to free */
+-	)
+-/* Free the malloc'ed tables built by huft_build(), which makes a linked
+-   list of the tables it made, with the links in a dummy first entry of
+-   each table. */
++/*
++ * huft_free - free a huffman table
++ * @t: table to free
++ *
++ * Free the malloc'ed tables built by huft_build(), which makes a
++ * linked list of the tables it made, with the links in a dummy first
++ * entry of each table.
++ */
++STATIC int INIT huft_free(struct huft *t)
+ {
+-  register struct huft *p, *q;
++	register struct huft *p, *q;
+ 
+-
+-  /* Go through linked list, freeing from the malloced (t[-1]) address. */
+-  p = t;
+-  while (p != (struct huft *)NULL)
+-  {
+-    q = (--p)->v.t;
+-    free((char*)p);
+-    p = q;
+-  } 
+-  return 0;
++	/* Go through list, freeing from the malloced (t[-1]) address. */
++	p = t;
++	while (p != (struct huft *)NULL) {
++		q = (--p)->v.t;
++		free((char *)p);
++		p = q;
++	}
++	return 0;
+ }
+ 
+-
+-STATIC int INIT inflate_codes(
+-	struct huft *tl,    /* literal/length decoder tables */
+-	struct huft *td,    /* distance decoder tables */
+-	int bl,             /* number of bits decoded by tl[] */
+-	int bd              /* number of bits decoded by td[] */
+-	)
+-/* inflate (decompress) the codes in a deflated (compressed) block.
+-   Return an error code or zero if it all goes ok. */
++/*
++ * inflate_codes - decompress the codes in a deflated block
++ * @tl: literal/length decoder tables
++ * @td: distance decoder tables
++ * @bl: number of bits decoded by tl
++ * @bd: number of bits decoded by td
++ *
++ * inflate (decompress) the codes in a deflated (compressed) block.
++ * Return an error code or zero if it all goes ok.
++ */
++STATIC int inflate_codes(struct huft *tl, struct huft *td, int bl, int bd)
+ {
+-  register unsigned e;  /* table entry flag/number of extra bits */
+-  unsigned n, d;        /* length and index for copy */
+-  unsigned w;           /* current window position */
+-  struct huft *t;       /* pointer to table entry */
+-  unsigned ml, md;      /* masks for bl and bd bits */
+-  register ulg b;       /* bit buffer */
+-  register unsigned k;  /* number of bits in bit buffer */
+-
+-
+-  /* make local copies of globals */
+-  b = bb;                       /* initialize bit buffer */
+-  k = bk;
+-  w = wp;                       /* initialize window position */
+-
+-  /* inflate the coded data */
+-  ml = mask_bits[bl];           /* precompute masks for speed */
+-  md = mask_bits[bd];
+-  for (;;)                      /* do until end of block */
+-  {
+-    NEEDBITS((unsigned)bl)
+-    if ((e = (t = tl + ((unsigned)b & ml))->e) > 16)
+-      do {
+-        if (e == 99)
+-          return 1;
+-        DUMPBITS(t->b)
+-        e -= 16;
+-        NEEDBITS(e)
+-      } while ((e = (t = t->v.t + ((unsigned)b & mask_bits[e]))->e) > 16);
+-    DUMPBITS(t->b)
+-    if (e == 16)                /* then it's a literal */
+-    {
+-      slide[w++] = (uch)t->v.n;
+-      Tracevv((stderr, "%c", slide[w-1]));
+-      if (w == WSIZE)
+-      {
+-        flush_output(w);
+-        w = 0;
+-      }
+-    }
+-    else                        /* it's an EOB or a length */
+-    {
+-      /* exit if end of block */
+-      if (e == 15)
+-        break;
+-
+-      /* get length of block to copy */
+-      NEEDBITS(e)
+-      n = t->v.n + ((unsigned)b & mask_bits[e]);
+-      DUMPBITS(e);
+-
+-      /* decode distance of block to copy */
+-      NEEDBITS((unsigned)bd)
+-      if ((e = (t = td + ((unsigned)b & md))->e) > 16)
+-        do {
+-          if (e == 99)
+-            return 1;
+-          DUMPBITS(t->b)
+-          e -= 16;
+-          NEEDBITS(e)
+-        } while ((e = (t = t->v.t + ((unsigned)b & mask_bits[e]))->e) > 16);
+-      DUMPBITS(t->b)
+-      NEEDBITS(e)
+-      d = w - t->v.n - ((unsigned)b & mask_bits[e]);
+-      DUMPBITS(e)
+-      Tracevv((stderr,"\\[%d,%d]", w-d, n));
+-
+-      /* do the copy */
+-      do {
+-        n -= (e = (e = WSIZE - ((d &= WSIZE-1) > w ? d : w)) > n ? n : e);
++	register unsigned e;	/* table entry flag/number of extra bits */
++	unsigned n, d;		/* length and index for copy */
++	unsigned w;		/* current window position */
++	struct huft *t;		/* pointer to table entry */
++	unsigned ml, md;	/* masks for bl and bd bits */
++	register ulg b;		/* bit buffer */
++	register unsigned k;	/* number of bits in bit buffer */
++
++	/* make local copies of globals */
++	b = bb;			/* initialize bit buffer */
++	k = bk;
++	w = wp;			/* initialize window position */
++
++	/* inflate the coded data */
++	ml = mask_bits[bl];	/* precompute masks for speed */
++	md = mask_bits[bd];
++	for (;;) {		/* do until end of block */
++		NEEDBITS((unsigned)bl);
++		if ((e = (t = tl + ((unsigned)b & ml))->e) > 16)
++			do {
++				if (e == 99)
++					return 1;
++				DUMPBITS(t->b);
++				e -= 16;
++				NEEDBITS(e);
++			} while ((e = (t = t->v.t + ((unsigned)b &
++						     mask_bits[e]))->e) > 16);
++		DUMPBITS(t->b);
++		if (e == 16) {	/* then it's a literal */
++			slide[w++] = (uch)t->v.n;
++			Tracevv((stderr, "%c", slide[w - 1]));
++			if (w == WSIZE) {
++				flush_output(w);
++				w = 0;
++			}
++		} else {	/* it's an EOB or a length */
++			/* exit if end of block */
++			if (e == 15)
++				break;
++
++			/* get length of block to copy */
++			NEEDBITS(e);
++			n = t->v.n + ((unsigned)b & mask_bits[e]);
++			DUMPBITS(e);
++
++			/* decode distance of block to copy */
++			NEEDBITS((unsigned)bd);
++			if ((e = (t = td + ((unsigned)b & md))->e) > 16)
++				do {
++					if (e == 99)
++						return 1;
++					DUMPBITS(t->b);
++					e -= 16;
++					NEEDBITS(e);
++				} while ((e = (t = t->v.t + ((unsigned)b
++					   & mask_bits[e]))->e) > 16);
++			DUMPBITS(t->b);
++			NEEDBITS(e);
++			d = w - t->v.n - ((unsigned)b & mask_bits[e]);
++			DUMPBITS(e)
++			    Tracevv((stderr, "\\[%d,%d]", w - d, n));
++
++			/* do the copy */
++			do {
++				n -= (e = (e = WSIZE - ((d &= WSIZE - 1) > w ?
++							d : w)) > n ? n : e);
+ #if !defined(NOMEMCPY) && !defined(DEBUG)
+-        if (w - d >= e)         /* (this test assumes unsigned comparison) */
+-        {
+-          memcpy(slide + w, slide + d, e);
+-          w += e;
+-          d += e;
+-        }
+-        else                      /* do it slow to avoid memcpy() overlap */
+-#endif /* !NOMEMCPY */
+-          do {
+-            slide[w++] = slide[d++];
+-	    Tracevv((stderr, "%c", slide[w-1]));
+-          } while (--e);
+-        if (w == WSIZE)
+-        {
+-          flush_output(w);
+-          w = 0;
+-        }
+-      } while (n);
+-    }
+-  }
+-
+-
+-  /* restore the globals from the locals */
+-  wp = w;                       /* restore global window pointer */
+-  bb = b;                       /* restore global bit buffer */
+-  bk = k;
++				/* (this test assumes unsigned comparison) */
++				if (w - d >= e) {
++					memcpy(slide + w, slide + d, e);
++					w += e;
++					d += e;
++				} else
++#endif				/* !NOMEMCPY */
++					/* avoid memcpy() overlap */
++					do {
++						slide[w++] = slide[d++];
++						Tracevv((stderr, "%c",
++							 slide[w - 1]));
++					} while (--e);
++				if (w == WSIZE) {
++					flush_output(w);
++					w = 0;
++				}
++			} while (n);
++		}
++	}
++
++	/* restore the globals from the locals */
++	wp = w;			/* restore global window pointer */
++	bb = b;			/* restore global bit buffer */
++	bk = k;
+ 
+-  /* done */
+-  return 0;
++	/* done */
++	return 0;
+ 
+- underrun:
+-  return 4;			/* Input underrun */
++      underrun:
++	return 4;		/* Input underrun */
+ }
+ 
+-
+-
++/* inflate_stored - "decompress" an inflated type 0 (stored) block. */
+ STATIC int INIT inflate_stored(void)
+-/* "decompress" an inflated type 0 (stored) block. */
+ {
+-  unsigned n;           /* number of bytes in block */
+-  unsigned w;           /* current window position */
+-  register ulg b;       /* bit buffer */
+-  register unsigned k;  /* number of bits in bit buffer */
+-
+-DEBG("<stor");
+-
+-  /* make local copies of globals */
+-  b = bb;                       /* initialize bit buffer */
+-  k = bk;
+-  w = wp;                       /* initialize window position */
+-
+-
+-  /* go to byte boundary */
+-  n = k & 7;
+-  DUMPBITS(n);
+-
+-
+-  /* get the length and its complement */
+-  NEEDBITS(16)
+-  n = ((unsigned)b & 0xffff);
+-  DUMPBITS(16)
+-  NEEDBITS(16)
+-  if (n != (unsigned)((~b) & 0xffff))
+-    return 1;                   /* error in compressed data */
+-  DUMPBITS(16)
+-
+-
+-  /* read and output the compressed data */
+-  while (n--)
+-  {
+-    NEEDBITS(8)
+-    slide[w++] = (uch)b;
+-    if (w == WSIZE)
+-    {
+-      flush_output(w);
+-      w = 0;
+-    }
+-    DUMPBITS(8)
+-  }
+-
+-
+-  /* restore the globals from the locals */
+-  wp = w;                       /* restore global window pointer */
+-  bb = b;                       /* restore global bit buffer */
+-  bk = k;
++	unsigned n;		/* number of bytes in block */
++	unsigned w;		/* current window position */
++	register ulg b;		/* bit buffer */
++	register unsigned k;	/* number of bits in bit buffer */
++
++	DEBG("<stor");
++
++	/* make local copies of globals */
++	b = bb;			/* initialize bit buffer */
++	k = bk;
++	w = wp;			/* initialize window position */
++
++	/* go to byte boundary */
++	n = k & 7;
++	DUMPBITS(n);
++
++	/* get the length and its complement */
++	NEEDBITS(16);
++	n = ((unsigned)b & 0xffff);
++	DUMPBITS(16);
++	NEEDBITS(16);
++	if (n != (unsigned)((~b) & 0xffff))
++		return 1;	/* error in compressed data */
++	DUMPBITS(16);
++
++	/* read and output the compressed data */
++	while (n--) {
++		NEEDBITS(8);
++		slide[w++] = (uch)b;
++		if (w == WSIZE) {
++			flush_output(w);
++			w = 0;
++		}
++		DUMPBITS(8);
++	}
++
++	/* restore the globals from the locals */
++	wp = w;			/* restore global window pointer */
++	bb = b;			/* restore global bit buffer */
++	bk = k;
+ 
+-  DEBG(">");
+-  return 0;
++	DEBG(">");
++	return 0;
+ 
+- underrun:
+-  return 4;			/* Input underrun */
++      underrun:
++	return 4;		/* Input underrun */
+ }
+ 
+ 
+-/*
++/* inflate_fixed - decompress a block with fixed Huffman codes
++ *
++ * decompress an inflated type 1 (fixed Huffman codes) block. We
++ * should either replace this with a custom decoder, or at least
++ * precompute the Huffman tables.
++ *
+  * We use `noinline' here to prevent gcc-3.5 from using too much stack space
+  */
+ STATIC int noinline INIT inflate_fixed(void)
+-/* decompress an inflated type 1 (fixed Huffman codes) block.  We should
+-   either replace this with a custom decoder, or at least precompute the
+-   Huffman tables. */
+ {
+-  int i;                /* temporary variable */
+-  struct huft *tl;      /* literal/length code table */
+-  struct huft *td;      /* distance code table */
+-  int bl;               /* lookup bits for tl */
+-  int bd;               /* lookup bits for td */
+-  unsigned l[288];      /* length list for huft_build */
+-
+-DEBG("<fix");
+-
+-  /* set up literal table */
+-  for (i = 0; i < 144; i++)
+-    l[i] = 8;
+-  for (; i < 256; i++)
+-    l[i] = 9;
+-  for (; i < 280; i++)
+-    l[i] = 7;
+-  for (; i < 288; i++)          /* make a complete, but wrong code set */
+-    l[i] = 8;
+-  bl = 7;
+-  if ((i = huft_build(l, 288, 257, cplens, cplext, &tl, &bl)) != 0)
+-    return i;
+-
+-
+-  /* set up distance table */
+-  for (i = 0; i < 30; i++)      /* make an incomplete code set */
+-    l[i] = 5;
+-  bd = 5;
+-  if ((i = huft_build(l, 30, 0, cpdist, cpdext, &td, &bd)) > 1)
+-  {
+-    huft_free(tl);
+-
+-    DEBG(">");
+-    return i;
+-  }
+-
+-
+-  /* decompress until an end-of-block code */
+-  if (inflate_codes(tl, td, bl, bd))
+-    return 1;
+-
+-
+-  /* free the decoding tables, return */
+-  huft_free(tl);
+-  huft_free(td);
+-  return 0;
++	int i;			/* temporary variable */
++	struct huft *tl;	/* literal/length code table */
++	struct huft *td;	/* distance code table */
++	int bl;			/* lookup bits for tl */
++	int bd;			/* lookup bits for td */
++	unsigned l[288];	/* length list for huft_build */
++
++	DEBG("<fix");
++
++	/* set up literal table */
++	for (i = 0; i < 144; i++)
++		l[i] = 8;
++	for (; i < 256; i++)
++		l[i] = 9;
++	for (; i < 280; i++)
++		l[i] = 7;
++	for (; i < 288; i++)	/* make a complete, but wrong code set */
++		l[i] = 8;
++	bl = 7;
++	if ((i = huft_build(l, 288, 257, cplens, cplext, &tl, &bl)) != 0)
++		return i;
++
++	/* set up distance table */
++	for (i = 0; i < 30; i++)	/* make an incomplete code set */
++		l[i] = 5;
++	bd = 5;
++	if ((i = huft_build(l, 30, 0, cpdist, cpdext, &td, &bd)) > 1) {
++		huft_free(tl);
++
++		DEBG(">");
++		return i;
++	}
++
++	/* decompress until an end-of-block code */
++	if (inflate_codes(tl, td, bl, bd))
++		return 1;
++
++	/* free the decoding tables, return */
++	huft_free(tl);
++	huft_free(td);
++	return 0;
+ }
+ 
+ 
+-/*
++/* inflate_dynamic - decompress a type 2 (dynamic Huffman codes) block.
++ *
+  * We use `noinline' here to prevent gcc-3.5 from using too much stack space
+  */
+ STATIC int noinline INIT inflate_dynamic(void)
+-/* decompress an inflated type 2 (dynamic Huffman codes) block. */
+ {
+-  int i;                /* temporary variables */
+-  unsigned j;
+-  unsigned l;           /* last length */
+-  unsigned m;           /* mask for bit lengths table */
+-  unsigned n;           /* number of lengths to get */
+-  struct huft *tl;      /* literal/length code table */
+-  struct huft *td;      /* distance code table */
+-  int bl;               /* lookup bits for tl */
+-  int bd;               /* lookup bits for td */
+-  unsigned nb;          /* number of bit length codes */
+-  unsigned nl;          /* number of literal/length codes */
+-  unsigned nd;          /* number of distance codes */
++	int i;			/* temporary variables */
++	unsigned j;
++	unsigned l;		/* last length */
++	unsigned m;		/* mask for bit lengths table */
++	unsigned n;		/* number of lengths to get */
++	struct huft *tl;	/* literal/length code table */
++	struct huft *td;	/* distance code table */
++	int bl;			/* lookup bits for tl */
++	int bd;			/* lookup bits for td */
++	unsigned nb;		/* number of bit length codes */
++	unsigned nl;		/* number of literal/length codes */
++	unsigned nd;		/* number of distance codes */
+ #ifdef PKZIP_BUG_WORKAROUND
+-  unsigned ll[288+32];  /* literal/length and distance code lengths */
++	unsigned ll[288 + 32];	/* literal/length and distance code lengths */
+ #else
+-  unsigned ll[286+30];  /* literal/length and distance code lengths */
++	unsigned ll[286 + 30];	/* literal/length and distance code lengths */
+ #endif
+-  register ulg b;       /* bit buffer */
+-  register unsigned k;  /* number of bits in bit buffer */
++	register ulg b;		/* bit buffer */
++	register unsigned k;	/* number of bits in bit buffer */
+ 
+-DEBG("<dyn");
++	DEBG("<dyn");
+ 
+-  /* make local bit buffer */
+-  b = bb;
+-  k = bk;
+-
+-
+-  /* read in table lengths */
+-  NEEDBITS(5)
+-  nl = 257 + ((unsigned)b & 0x1f);      /* number of literal/length codes */
+-  DUMPBITS(5)
+-  NEEDBITS(5)
+-  nd = 1 + ((unsigned)b & 0x1f);        /* number of distance codes */
+-  DUMPBITS(5)
+-  NEEDBITS(4)
+-  nb = 4 + ((unsigned)b & 0xf);         /* number of bit length codes */
+-  DUMPBITS(4)
++	/* make local bit buffer */
++	b = bb;
++	k = bk;
++
++	/* read in table lengths */
++	NEEDBITS(5);
++	nl = 257 + ((unsigned)b & 0x1f); /* number of literal/length codes */
++	DUMPBITS(5);
++	NEEDBITS(5);
++	nd = 1 + ((unsigned)b & 0x1f);	/* number of distance codes */
++	DUMPBITS(5);
++	NEEDBITS(4);
++	nb = 4 + ((unsigned)b & 0xf);	/* number of bit length codes */
++	DUMPBITS(4);
+ #ifdef PKZIP_BUG_WORKAROUND
+-  if (nl > 288 || nd > 32)
++	if (nl > 288 || nd > 32)
+ #else
+-  if (nl > 286 || nd > 30)
++	if (nl > 286 || nd > 30)
+ #endif
+-    return 1;                   /* bad lengths */
++		return 1;	/* bad lengths */
+ 
+-DEBG("dyn1 ");
++	DEBG("dyn1 ");
+ 
+-  /* read in bit-length-code lengths */
+-  for (j = 0; j < nb; j++)
+-  {
+-    NEEDBITS(3)
+-    ll[border[j]] = (unsigned)b & 7;
+-    DUMPBITS(3)
+-  }
+-  for (; j < 19; j++)
+-    ll[border[j]] = 0;
+-
+-DEBG("dyn2 ");
+-
+-  /* build decoding table for trees--single level, 7 bit lookup */
+-  bl = 7;
+-  if ((i = huft_build(ll, 19, 19, NULL, NULL, &tl, &bl)) != 0)
+-  {
+-    if (i == 1)
+-      huft_free(tl);
+-    return i;                   /* incomplete code set */
+-  }
+-
+-DEBG("dyn3 ");
+-
+-  /* read in literal and distance code lengths */
+-  n = nl + nd;
+-  m = mask_bits[bl];
+-  i = l = 0;
+-  while ((unsigned)i < n)
+-  {
+-    NEEDBITS((unsigned)bl)
+-    j = (td = tl + ((unsigned)b & m))->b;
+-    DUMPBITS(j)
+-    j = td->v.n;
+-    if (j < 16)                 /* length of code in bits (0..15) */
+-      ll[i++] = l = j;          /* save last length in l */
+-    else if (j == 16)           /* repeat last length 3 to 6 times */
+-    {
+-      NEEDBITS(2)
+-      j = 3 + ((unsigned)b & 3);
+-      DUMPBITS(2)
+-      if ((unsigned)i + j > n)
+-        return 1;
+-      while (j--)
+-        ll[i++] = l;
+-    }
+-    else if (j == 17)           /* 3 to 10 zero length codes */
+-    {
+-      NEEDBITS(3)
+-      j = 3 + ((unsigned)b & 7);
+-      DUMPBITS(3)
+-      if ((unsigned)i + j > n)
+-        return 1;
+-      while (j--)
+-        ll[i++] = 0;
+-      l = 0;
+-    }
+-    else                        /* j == 18: 11 to 138 zero length codes */
+-    {
+-      NEEDBITS(7)
+-      j = 11 + ((unsigned)b & 0x7f);
+-      DUMPBITS(7)
+-      if ((unsigned)i + j > n)
+-        return 1;
+-      while (j--)
+-        ll[i++] = 0;
+-      l = 0;
+-    }
+-  }
+-
+-DEBG("dyn4 ");
+-
+-  /* free decoding table for trees */
+-  huft_free(tl);
+-
+-DEBG("dyn5 ");
+-
+-  /* restore the global bit buffer */
+-  bb = b;
+-  bk = k;
+-
+-DEBG("dyn5a ");
+-
+-  /* build the decoding tables for literal/length and distance codes */
+-  bl = lbits;
+-  if ((i = huft_build(ll, nl, 257, cplens, cplext, &tl, &bl)) != 0)
+-  {
+-DEBG("dyn5b ");
+-    if (i == 1) {
+-      error("incomplete literal tree");
+-      huft_free(tl);
+-    }
+-    return i;                   /* incomplete code set */
+-  }
+-DEBG("dyn5c ");
+-  bd = dbits;
+-  if ((i = huft_build(ll + nl, nd, 0, cpdist, cpdext, &td, &bd)) != 0)
+-  {
+-DEBG("dyn5d ");
+-    if (i == 1) {
+-      error("incomplete distance tree");
++	/* read in bit-length-code lengths */
++	for (j = 0; j < nb; j++) {
++		NEEDBITS(3);
++		ll[border[j]] = (unsigned)b & 7;
++		DUMPBITS(3);
++	}
++	for (; j < 19; j++)
++		ll[border[j]] = 0;
++
++	DEBG("dyn2 ");
++
++	/* build decoding table for trees--single level, 7 bit lookup */
++	bl = 7;
++	if ((i = huft_build(ll, 19, 19, NULL, NULL, &tl, &bl)) != 0) {
++		if (i == 1)
++			huft_free(tl);
++		return i;	/* incomplete code set */
++	}
++
++	DEBG("dyn3 ");
++
++	/* read in literal and distance code lengths */
++	n = nl + nd;
++	m = mask_bits[bl];
++	i = l = 0;
++	while ((unsigned)i < n) {
++		NEEDBITS((unsigned)bl);
++		j = (td = tl + ((unsigned)b & m))->b;
++		DUMPBITS(j);
++		j = td->v.n;
++		if (j < 16)	/* length of code in bits (0..15) */
++			ll[i++] = l = j;	/* save last length in l */
++		else if (j == 16) {	/* repeat last length 3 to 6 times */
++			NEEDBITS(2);
++			j = 3 + ((unsigned)b & 3);
++			DUMPBITS(2);
++			    if ((unsigned)i + j > n)
++				return 1;
++			while (j--)
++				ll[i++] = l;
++		} else if (j == 17) {	/* 3 to 10 zero length codes */
++			NEEDBITS(3);
++			j = 3 + ((unsigned)b & 7);
++			DUMPBITS(3);
++			if ((unsigned)i + j > n)
++				return 1;
++			while (j--)
++				ll[i++] = 0;
++			l = 0;
++		} else {	/* j == 18: 11 to 138 zero length codes */
++			NEEDBITS(7);
++			j = 11 + ((unsigned)b & 0x7f);
++			DUMPBITS(7);
++			if ((unsigned)i + j > n)
++				return 1;
++			while (j--)
++				ll[i++] = 0;
++			l = 0;
++		}
++	}
++
++	DEBG("dyn4 ");
++
++	/* free decoding table for trees */
++	huft_free(tl);
++
++	DEBG("dyn5 ");
++
++	/* restore the global bit buffer */
++	bb = b;
++	bk = k;
++
++	DEBG("dyn5a ");
++
++	/* build the decoding tables for literal/length and distance codes */
++	bl = lbits;
++	if ((i = huft_build(ll, nl, 257, cplens, cplext, &tl, &bl)) != 0) {
++		DEBG("dyn5b ");
++		if (i == 1) {
++			error("incomplete literal tree");
++			huft_free(tl);
++		}
++		return i;	/* incomplete code set */
++	}
++	DEBG("dyn5c ");
++	bd = dbits;
++	if ((i = huft_build(ll + nl, nd, 0, cpdist, cpdext, &td, &bd)) != 0) {
++		DEBG("dyn5d ");
++		if (i == 1) {
++			error("incomplete distance tree");
+ #ifdef PKZIP_BUG_WORKAROUND
+-      i = 0;
+-    }
++			i = 0;
++		}
+ #else
+-      huft_free(td);
+-    }
+-    huft_free(tl);
+-    return i;                   /* incomplete code set */
++			huft_free(td);
++		}
++		huft_free(tl);
++		return i;	/* incomplete code set */
+ #endif
+-  }
++	}
+ 
+-DEBG("dyn6 ");
++	DEBG("dyn6 ");
+ 
+-  /* decompress until an end-of-block code */
+-  if (inflate_codes(tl, td, bl, bd))
+-    return 1;
++	/* decompress until an end-of-block code */
++	if (inflate_codes(tl, td, bl, bd))
++		return 1;
+ 
+-DEBG("dyn7 ");
++	DEBG("dyn7 ");
+ 
+-  /* free the decoding tables, return */
+-  huft_free(tl);
+-  huft_free(td);
++	/* free the decoding tables, return */
++	huft_free(tl);
++	huft_free(td);
+ 
+-  DEBG(">");
+-  return 0;
++	DEBG(">");
++	return 0;
+ 
+- underrun:
+-  return 4;			/* Input underrun */
++      underrun:
++	return 4;		/* Input underrun */
+ }
+ 
+-
+-
+-STATIC int INIT inflate_block(
+-	int *e                  /* last block flag */
+-	)
+-/* decompress an inflated block */
++/* inflate_block - decompress a deflated block
++ * @e: last block flag
++ */
++STATIC int INIT inflate_block(int *e)
+ {
+-  unsigned t;           /* block type */
+-  register ulg b;       /* bit buffer */
+-  register unsigned k;  /* number of bits in bit buffer */
++	unsigned t;		/* block type */
++	register ulg b;		/* bit buffer */
++	register unsigned k;	/* number of bits in bit buffer */
++
++	DEBG("<blk");
++
++	/* make local bit buffer */
++	b = bb;
++	k = bk;
++
++	/* read in last block bit */
++	NEEDBITS(1);
++	*e = (int)b & 1;
++	DUMPBITS(1);
++
++	/* read in block type */
++	NEEDBITS(2);
++	t = (unsigned)b & 3;
++	DUMPBITS(2);
++
++	/* restore the global bit buffer */
++	bb = b;
++	bk = k;
++
++	/* inflate that block type */
++	if (t == 2)
++		return inflate_dynamic();
++	if (t == 0)
++		return inflate_stored();
++	if (t == 1)
++		return inflate_fixed();
+ 
+-  DEBG("<blk");
++	DEBG(">");
+ 
+-  /* make local bit buffer */
+-  b = bb;
+-  k = bk;
++	/* bad block type */
++	return 2;
+ 
+-
+-  /* read in last block bit */
+-  NEEDBITS(1)
+-  *e = (int)b & 1;
+-  DUMPBITS(1)
+-
+-
+-  /* read in block type */
+-  NEEDBITS(2)
+-  t = (unsigned)b & 3;
+-  DUMPBITS(2)
+-
+-
+-  /* restore the global bit buffer */
+-  bb = b;
+-  bk = k;
+-
+-  /* inflate that block type */
+-  if (t == 2)
+-    return inflate_dynamic();
+-  if (t == 0)
+-    return inflate_stored();
+-  if (t == 1)
+-    return inflate_fixed();
+-
+-  DEBG(">");
+-
+-  /* bad block type */
+-  return 2;
+-
+- underrun:
+-  return 4;			/* Input underrun */
++      underrun:
++	return 4;		/* Input underrun */
+ }
+ 
+-
+-
++/* inflate - decompress an inflated entry */
+ STATIC int INIT inflate(void)
+-/* decompress an inflated entry */
+ {
+-  int e;                /* last block flag */
+-  int r;                /* result code */
+-  unsigned h;           /* maximum struct huft's malloc'ed */
+-  void *ptr;
+-
+-  /* initialize window, bit buffer */
+-  wp = 0;
+-  bk = 0;
+-  bb = 0;
+-
+-
+-  /* decompress until the last block */
+-  h = 0;
+-  do {
+-    hufts = 0;
+-    gzip_mark(&ptr);
+-    if ((r = inflate_block(&e)) != 0) {
+-      gzip_release(&ptr);	    
+-      return r;
+-    }
+-    gzip_release(&ptr);
+-    if (hufts > h)
+-      h = hufts;
+-  } while (!e);
+-
+-  /* Undo too much lookahead. The next read will be byte aligned so we
+-   * can discard unused bits in the last meaningful byte.
+-   */
+-  while (bk >= 8) {
+-    bk -= 8;
+-    inptr--;
+-  }
++	int e;			/* last block flag */
++	int r;			/* result code */
++	unsigned h;		/* maximum struct huft's malloc'ed */
++	void *ptr;
++
++	/* initialize window, bit buffer */
++	wp = 0;
++	bk = 0;
++	bb = 0;
++
++	/* decompress until the last block */
++	h = 0;
++	do {
++		hufts = 0;
++		gzip_mark(&ptr);
++		if ((r = inflate_block(&e)) != 0) {
++			gzip_release(&ptr);
++			return r;
++		}
++		gzip_release(&ptr);
++		if (hufts > h)
++			h = hufts;
++	} while (!e);
++
++	/* Undo too much lookahead. The next read will be byte aligned so we
++	 * can discard unused bits in the last meaningful byte.
++	 */
++	while (bk >= 8) {
++		bk -= 8;
++		inptr--;
++	}
+ 
+-  /* flush out slide */
+-  flush_output(wp);
++	/* flush out slide */
++	flush_output(wp);
+ 
+-
+-  /* return success */
++	/* return success */
+ #ifdef DEBUG
+-  fprintf(stderr, "<%u> ", h);
+-#endif /* DEBUG */
+-  return 0;
++	fprintf(stderr, "<%u> ", h);
++#endif				/* DEBUG */
++	return 0;
+ }
+ 
+ /**********************************************************************
+@@ -1041,44 +1038,42 @@
+ #define CRC_VALUE (crc ^ 0xffffffffUL)
+ 
+ /*
+- * Code to compute the CRC-32 table. Borrowed from 
++ * Code to compute the CRC-32 table. Borrowed from
+  * gzip-1.0.3/makecrc.c.
++ * Not copyrighted 1990 Mark Adler
+  */
+ 
+-static void INIT
+-makecrc(void)
++static void INIT makecrc(void)
+ {
+-/* Not copyrighted 1990 Mark Adler	*/
+ 
+-  unsigned long c;      /* crc shift register */
+-  unsigned long e;      /* polynomial exclusive-or pattern */
+-  int i;                /* counter for all possible eight bit values */
+-  int k;                /* byte being shifted into crc apparatus */
+-
+-  /* terms of polynomial defining this crc (except x^32): */
+-  static const int p[] = {0,1,2,4,5,7,8,10,11,12,16,22,23,26};
+-
+-  /* Make exclusive-or pattern from polynomial */
+-  e = 0;
+-  for (i = 0; i < sizeof(p)/sizeof(int); i++)
+-    e |= 1L << (31 - p[i]);
+-
+-  crc_32_tab[0] = 0;
+-
+-  for (i = 1; i < 256; i++)
+-  {
+-    c = 0;
+-    for (k = i | 256; k != 1; k >>= 1)
+-    {
+-      c = c & 1 ? (c >> 1) ^ e : c >> 1;
+-      if (k & 1)
+-        c ^= e;
+-    }
+-    crc_32_tab[i] = c;
+-  }
++	unsigned long c;	/* crc shift register */
++	unsigned long e;	/* polynomial exclusive-or pattern */
++	int i;			/* counter for all possible eight bit values */
++	int k;			/* byte being shifted into crc apparatus */
++
++	/* terms of polynomial defining this crc (except x^32): */
++	static const int p[] =
++	    { 0, 1, 2, 4, 5, 7, 8, 10, 11, 12, 16, 22, 23, 26 };
++
++	/* Make exclusive-or pattern from polynomial */
++	e = 0;
++	for (i = 0; i < sizeof(p) / sizeof(int); i++)
++		e |= 1L << (31 - p[i]);
++
++	crc_32_tab[0] = 0;
++
++	for (i = 1; i < 256; i++) {
++		c = 0;
++		for (k = i | 256; k != 1; k >>= 1) {
++			c = c & 1 ? (c >> 1) ^ e : c >> 1;
++			if (k & 1)
++				c ^= e;
++		}
++		crc_32_tab[i] = c;
++	}
+ 
+-  /* this is initialized here so this code could reside in ROM */
+-  crc = (ulg)0xffffffffUL; /* shift register contents */
++	/* this is initialized here so this code could reside in ROM */
++	crc = (ulg)0xffffffffUL;	/* shift register contents */
+ }
+ 
+ /* gzip flag byte */
+@@ -1095,118 +1090,118 @@
+  */
+ static int INIT gunzip(void)
+ {
+-    uch flags;
+-    unsigned char magic[2]; /* magic header */
+-    char method;
+-    ulg orig_crc = 0;       /* original crc */
+-    ulg orig_len = 0;       /* original uncompressed length */
+-    int res;
+-
+-    magic[0] = NEXTBYTE();
+-    magic[1] = NEXTBYTE();
+-    method   = NEXTBYTE();
+-
+-    if (magic[0] != 037 ||
+-	((magic[1] != 0213) && (magic[1] != 0236))) {
+-	    error("bad gzip magic numbers");
+-	    return -1;
+-    }
+-
+-    /* We only support method #8, DEFLATED */
+-    if (method != 8)  {
+-	    error("internal error, invalid method");
+-	    return -1;
+-    }
+-
+-    flags  = (uch)get_byte();
+-    if ((flags & ENCRYPTED) != 0) {
+-	    error("Input is encrypted");
+-	    return -1;
+-    }
+-    if ((flags & CONTINUATION) != 0) {
+-	    error("Multi part input");
+-	    return -1;
+-    }
+-    if ((flags & RESERVED) != 0) {
+-	    error("Input has invalid flags");
+-	    return -1;
+-    }
+-    NEXTBYTE();	/* Get timestamp */
+-    NEXTBYTE();
+-    NEXTBYTE();
+-    NEXTBYTE();
+-
+-    (void)NEXTBYTE();  /* Ignore extra flags for the moment */
+-    (void)NEXTBYTE();  /* Ignore OS type for the moment */
+-
+-    if ((flags & EXTRA_FIELD) != 0) {
+-	    unsigned len = (unsigned)NEXTBYTE();
+-	    len |= ((unsigned)NEXTBYTE())<<8;
+-	    while (len--) (void)NEXTBYTE();
+-    }
+-
+-    /* Get original file name if it was truncated */
+-    if ((flags & ORIG_NAME) != 0) {
+-	    /* Discard the old name */
+-	    while (NEXTBYTE() != 0) /* null */ ;
+-    } 
+-
+-    /* Discard file comment if any */
+-    if ((flags & COMMENT) != 0) {
+-	    while (NEXTBYTE() != 0) /* null */ ;
+-    }
+-
+-    /* Decompress */
+-    if ((res = inflate())) {
+-	    switch (res) {
+-	    case 0:
+-		    break;
+-	    case 1:
+-		    error("invalid compressed format (err=1)");
+-		    break;
+-	    case 2:
+-		    error("invalid compressed format (err=2)");
+-		    break;
+-	    case 3:
+-		    error("out of memory");
+-		    break;
+-	    case 4:
+-		    error("out of input data");
+-		    break;
+-	    default:
+-		    error("invalid compressed format (other)");
+-	    }
+-	    return -1;
+-    }
+-	    
+-    /* Get the crc and original length */
+-    /* crc32  (see algorithm.doc)
+-     * uncompressed input size modulo 2^32
+-     */
+-    orig_crc = (ulg) NEXTBYTE();
+-    orig_crc |= (ulg) NEXTBYTE() << 8;
+-    orig_crc |= (ulg) NEXTBYTE() << 16;
+-    orig_crc |= (ulg) NEXTBYTE() << 24;
+-    
+-    orig_len = (ulg) NEXTBYTE();
+-    orig_len |= (ulg) NEXTBYTE() << 8;
+-    orig_len |= (ulg) NEXTBYTE() << 16;
+-    orig_len |= (ulg) NEXTBYTE() << 24;
+-    
+-    /* Validate decompression */
+-    if (orig_crc != CRC_VALUE) {
+-	    error("crc error");
+-	    return -1;
+-    }
+-    if (orig_len != bytes_out) {
+-	    error("length error");
+-	    return -1;
+-    }
+-    return 0;
+-
+- underrun:			/* NEXTBYTE() goto's here if needed */
+-    error("out of input data");
+-    return -1;
++	uch flags;
++	unsigned char magic[2];	/* magic header */
++	char method;
++	ulg orig_crc = 0;	/* original crc */
++	ulg orig_len = 0;	/* original uncompressed length */
++	int res;
++
++	magic[0] = NEXTBYTE();
++	magic[1] = NEXTBYTE();
++	method = NEXTBYTE();
++
++	if (magic[0] != 037 || ((magic[1] != 0213) && (magic[1] != 0236))) {
++		error("bad gzip magic numbers");
++		return -1;
++	}
++
++	/* We only support method #8, DEFLATED */
++	if (method != 8) {
++		error("internal error, invalid method");
++		return -1;
++	}
++
++	flags = (uch)get_byte();
++	if ((flags & ENCRYPTED) != 0) {
++		error("Input is encrypted");
++		return -1;
++	}
++	if ((flags & CONTINUATION) != 0) {
++		error("Multi part input");
++		return -1;
++	}
++	if ((flags & RESERVED) != 0) {
++		error("Input has invalid flags");
++		return -1;
++	}
++	NEXTBYTE();		/* Get timestamp */
++	NEXTBYTE();
++	NEXTBYTE();
++	NEXTBYTE();
++
++	(void)NEXTBYTE();	/* Ignore extra flags for the moment */
++	(void)NEXTBYTE();	/* Ignore OS type for the moment */
++
++	if ((flags & EXTRA_FIELD) != 0) {
++		unsigned len = (unsigned)NEXTBYTE();
++		len |= ((unsigned)NEXTBYTE()) << 8;
++		while (len--)
++			(void)NEXTBYTE();
++	}
++
++	/* Get original file name if it was truncated */
++	if ((flags & ORIG_NAME) != 0) {
++		/* Discard the old name */
++		while (NEXTBYTE() != 0)	/* null */
++			;
++	}
++
++	/* Discard file comment if any */
++	if ((flags & COMMENT) != 0) {
++		while (NEXTBYTE() != 0)	/* null */
++			;
++	}
++
++	/* Decompress */
++	if ((res = inflate())) {
++		switch (res) {
++		case 0:
++			break;
++		case 1:
++			error("invalid compressed format (err=1)");
++			break;
++		case 2:
++			error("invalid compressed format (err=2)");
++			break;
++		case 3:
++			error("out of memory");
++			break;
++		case 4:
++			error("out of input data");
++			break;
++		default:
++			error("invalid compressed format (other)");
++		}
++		return -1;
++	}
++
++	/* Get the crc and original length */
++	/* crc32  (see algorithm.doc)
++	 * uncompressed input size modulo 2^32
++	 */
++	orig_crc = (ulg)NEXTBYTE();
++	orig_crc |= (ulg)NEXTBYTE() << 8;
++	orig_crc |= (ulg)NEXTBYTE() << 16;
++	orig_crc |= (ulg)NEXTBYTE() << 24;
++
++	orig_len = (ulg)NEXTBYTE();
++	orig_len |= (ulg)NEXTBYTE() << 8;
++	orig_len |= (ulg)NEXTBYTE() << 16;
++	orig_len |= (ulg)NEXTBYTE() << 24;
++
++	/* Validate decompression */
++	if (orig_crc != CRC_VALUE) {
++		error("crc error");
++		return -1;
++	}
++	if (orig_len != bytes_out) {
++		error("length error");
++		return -1;
++	}
++	return 0;
++
++      underrun:		/* NEXTBYTE() goto's here if needed */
++	error("out of input data");
++	return -1;
+ }
+-
+-
Index: packages/kernel/patches/tiny_m57sli/tiny/ext2namei-inlines.patch
===================================================================
--- packages/kernel/patches/tiny_m57sli/tiny/ext2namei-inlines.patch	(revision 0)
+++ packages/kernel/patches/tiny_m57sli/tiny/ext2namei-inlines.patch	(revision 0)
@@ -0,0 +1,20 @@
+
+uninline ext2_add_nondir
+
+
+ tiny-mpm/fs/ext2/namei.c |    2 +-
+ 1 files changed, 1 insertion(+), 1 deletion(-)
+
+Index: new-linux/fs/ext2/namei.c
+===================================================================
+--- new-linux.orig/fs/ext2/namei.c	2007-04-25 20:08:32.000000000 -0700
++++ new-linux/fs/ext2/namei.c	2007-04-30 16:52:38.000000000 -0700
+@@ -36,7 +36,7 @@
+ #include "acl.h"
+ #include "xip.h"
+ 
+-static inline int ext2_add_nondir(struct dentry *dentry, struct inode *inode)
++static int ext2_add_nondir(struct dentry *dentry, struct inode *inode)
+ {
+ 	int err = ext2_add_link(dentry, inode);
+ 	if (!err) {
Index: packages/kernel/patches/tiny_m57sli/tiny/cpu-support.patch
===================================================================
--- packages/kernel/patches/tiny_m57sli/tiny/cpu-support.patch	(revision 0)
+++ packages/kernel/patches/tiny_m57sli/tiny/cpu-support.patch	(revision 0)
@@ -0,0 +1,210 @@
+
+configurable support for vendor-specific x86 CPU features
+
+
+ tiny-mpm/arch/i386/kernel/cpu/Makefile |   16 ++++----
+ tiny-mpm/arch/i386/kernel/cpu/common.c |   18 +++++++++
+ tiny-mpm/arch/i386/mm/init.c           |   12 +++++-
+ tiny-mpm/init/Kconfig                  |   62 +++++++++++++++++++++++++++++++++
+ 4 files changed, 99 insertions(+), 9 deletions(-)
+
+Index: linux-2.6.16.19/arch/i386/kernel/cpu/Makefile
+===================================================================
+--- linux-2.6.16.19.orig/arch/i386/kernel/cpu/Makefile	2006-06-23 17:05:11.000000000 -0700
++++ linux-2.6.16.19/arch/i386/kernel/cpu/Makefile	2006-06-23 17:06:28.000000000 -0700
+@@ -4,14 +4,14 @@
+ 
+ obj-y	:=	common.o proc.o
+ 
+-obj-y	+=	amd.o
+-obj-y	+=	cyrix.o
+-obj-y	+=	centaur.o
+-obj-y	+=	transmeta.o
+-obj-y	+=	intel.o intel_cacheinfo.o
+-obj-y	+=	rise.o
+-obj-y	+=	nexgen.o
+-obj-y	+=	umc.o
++obj-$(CONFIG_CPU_SUP_AMD)	+=	amd.o
++obj-$(CONFIG_CPU_SUP_CYRIX)	+=	cyrix.o
++obj-$(CONFIG_CPU_SUP_CENTAUR)	+=	centaur.o
++obj-$(CONFIG_CPU_SUP_TRANSMETA)	+=	transmeta.o
++obj-$(CONFIG_CPU_SUP_INTEL)	+=	intel.o intel_cacheinfo.o
++obj-$(CONFIG_CPU_SUP_RISE)	+=	rise.o
++obj-$(CONFIG_CPU_SUP_NEXGEN)	+=	nexgen.o
++obj-$(CONFIG_CPU_SUP_UMC)	+=	umc.o
+ 
+ obj-$(CONFIG_X86_MCE)	+=	mcheck/
+ 
+Index: linux-2.6.16.19/arch/i386/kernel/cpu/common.c
+===================================================================
+--- linux-2.6.16.19.orig/arch/i386/kernel/cpu/common.c	2006-06-23 17:06:21.000000000 -0700
++++ linux-2.6.16.19/arch/i386/kernel/cpu/common.c	2006-06-23 17:06:28.000000000 -0700
+@@ -11,6 +11,7 @@
+ #include <asm/msr.h>
+ #include <asm/io.h>
+ #include <asm/mmu_context.h>
++#include <asm/uaccess.h>
+ #ifdef CONFIG_X86_LOCAL_APIC
+ #include <asm/mpspec.h>
+ #include <asm/apic.h>
+@@ -300,7 +301,9 @@
+ 		}
+ 	}
+ 
++#ifdef CONFIG_CPU_SUP_INTEL
+ 	early_intel_workaround(c);
++#endif
+ 
+ #ifdef CONFIG_X86_HT
+ 	phys_proc_id[smp_processor_id()] = (cpuid_ebx(1) >> 24) & 0xff;
+@@ -547,15 +550,33 @@
+ 
+ void __init early_cpu_init(void)
+ {
++#ifdef CONFIG_CPU_SUP_INTEL
+ 	intel_cpu_init();
++#endif
++#ifdef CONFIG_CPU_SUP_CYRIX
+ 	cyrix_init_cpu();
++#endif
++#ifdef CONFIG_CPU_SUP_NSC
+ 	nsc_init_cpu();
++#endif
++#ifdef CONFIG_CPU_SUP_AMD
+ 	amd_init_cpu();
++#endif
++#ifdef CONFIG_CPU_SUP_CENTAUR
+ 	centaur_init_cpu();
++#endif
++#ifdef CONFIG_CPU_SUP_TRANSMETA
+ 	transmeta_init_cpu();
++#endif
++#ifdef CONFIG_CPU_SUP_RISE
+ 	rise_init_cpu();
++#endif
++#ifdef CONFIG_CPU_SUP_NEXGEN
+ 	nexgen_init_cpu();
++#endif
++#ifdef CONFIG_CPU_SUP_UMC
+ 	umc_init_cpu();
++#endif
+ 	early_cpu_detect();
+ 
+ #ifdef CONFIG_DEBUG_PAGEALLOC
+Index: linux-2.6.16.19/arch/i386/mm/init.c
+===================================================================
+--- linux-2.6.16.19.orig/arch/i386/mm/init.c	2006-06-23 17:05:11.000000000 -0700
++++ linux-2.6.16.19/arch/i386/mm/init.c	2006-06-23 17:06:28.000000000 -0700
+@@ -184,12 +184,14 @@
+ 	}
+ }
+ 
++#ifdef CONFIG_CPU_SUP_INTEL
+ static inline int page_kills_ppro(unsigned long pagenr)
+ {
+ 	if (pagenr >= 0x70000 && pagenr <= 0x7003F)
+ 		return 1;
+ 	return 0;
+ }
++#endif
+ 
+ extern int is_available_memory(efi_memory_desc_t *);
+ 
+@@ -277,7 +279,11 @@
+ 
+ void __init add_one_highpage_init(struct page *page, int pfn, int bad_ppro)
+ {
+-	if (page_is_ram(pfn) && !(bad_ppro && page_kills_ppro(pfn))) {
++	if (page_is_ram(pfn)
++#ifdef CONFIG_CPU_SUP_INTEL
++	    && !(bad_ppro && page_kills_ppro(pfn))
++#endif
++		) {
+ 		ClearPageReserved(page);
+ 		free_new_highpage(page);
+ 	} else
+@@ -577,7 +583,11 @@
+ 		BUG();
+ #endif
+ 	
++#ifdef CONFIG_CPU_SUP_INTEL
+ 	bad_ppro = ppro_with_ram_bug();
++#else
++	bad_ppro = 0;
++#endif
+ 
+ #ifdef CONFIG_HIGHMEM
+ 	/* check that fixmap and pkmap do not overlap */
+Index: linux-2.6.16.19/init/Kconfig
+===================================================================
+--- linux-2.6.16.19.orig/init/Kconfig	2006-06-23 17:06:28.000000000 -0700
++++ linux-2.6.16.19/init/Kconfig	2006-06-23 17:06:28.000000000 -0700
+@@ -635,6 +635,68 @@
+ 	help
+ 	  This enables support for the Time Stamp Counter timer.
+ 
++menuconfig PROCESSOR_SELECT
++	depends X86
++	default y
++	bool "Supported processor vendors" if EMBEDDED
++	help
++	  This lets you choose what x86 vendor support code your kernel
++	  will include.
++
++config CPU_SUP_INTEL
++       default y
++       bool "Support Intel processors" if PROCESSOR_SELECT
++       help
++         This enables extended support for Intel processors
++
++config CPU_SUP_CYRIX
++       default y
++       bool "Support Cyrix processors" if PROCESSOR_SELECT
++       help
++         This enables extended support for Cyrix processors
++
++config CPU_SUP_NSC
++       default y
++       bool "Support NSC processors" if PROCESSOR_SELECT
++       help
++         This enables extended support for NSC processors
++
++config CPU_SUP_AMD
++       default y
++       bool "Support AMD processors" if PROCESSOR_SELECT
++       help
++         This enables extended support for AMD processors
++
++config CPU_SUP_CENTAUR
++       default y
++       bool "Support Centaur processors" if PROCESSOR_SELECT
++       help
++         This enables extended support for Centaur processors
++
++config CPU_SUP_TRANSMETA
++       default y
++       bool "Support Transmeta processors" if PROCESSOR_SELECT
++       help
++         This enables extended support for Transmeta processors
++
++config CPU_SUP_RISE
++       default y
++       bool "Support Rise processors" if PROCESSOR_SELECT
++       help
++         This enables extended support for Rise processors
++
++config CPU_SUP_NEXGEN
++       default y
++       bool "Support NexGen processors" if PROCESSOR_SELECT
++       help
++         This enables extended support for NexGen processors
++
++config CPU_SUP_UMC
++       default y
++       bool "Support UMC processors" if PROCESSOR_SELECT
++       help
++         This enables extended support for UMC processors
++
+ endmenu		# General setup
+ 
+ config CRC32_CALC
Index: packages/kernel/patches/tiny_m57sli/tiny/dmi_blacklist.patch
===================================================================
--- packages/kernel/patches/tiny_m57sli/tiny/dmi_blacklist.patch	(revision 0)
+++ packages/kernel/patches/tiny_m57sli/tiny/dmi_blacklist.patch	(revision 0)
@@ -0,0 +1,95 @@
+
+configurable dmi blacklist support
+
+
+ tiny-mpm/arch/i386/kernel/Makefile     |    3 ++-
+ tiny-mpm/arch/i386/kernel/i386_ksyms.c |    2 ++
+ tiny-mpm/arch/i386/kernel/setup.c      |    2 ++
+ tiny-mpm/init/Kconfig                  |   11 +++++++++++
+ 4 files changed, 17 insertions(+), 1 deletion(-)
+
+Index: linux-2.6.16.19/arch/i386/kernel/Makefile
+===================================================================
+--- linux-2.6.16.19.orig/arch/i386/kernel/Makefile	2006-06-23 17:06:26.000000000 -0700
++++ linux-2.6.16.19/arch/i386/kernel/Makefile	2006-06-23 17:06:28.000000000 -0700
+@@ -6,7 +6,7 @@
+ 
+ obj-y	:= process.o semaphore.o signal.o entry.o traps.o irq.o \
+ 		ptrace.o time.o ioport.o ldt.o setup.o i8259.o sys_i386.o \
+-		pci-dma.o i386_ksyms.o i387.o dmi_scan.o \
++		pci-dma.o i386_ksyms.o i387.o \
+ 		quirks.o i8237.o topology.o
+ 
+ obj-y				+= cpu/
+@@ -39,6 +39,7 @@
+ obj-$(CONFIG_VM86)		+= vm86.o
+ obj-$(CONFIG_EARLY_PRINTK)	+= early_printk.o
+ obj-$(CONFIG_BOOTFLAG)		+= bootflag.o
++obj-$(CONFIG_DMI_SCAN)		+= dmi_scan.o
+ 
+ 
+ EXTRA_AFLAGS   := -traditional
+Index: linux-2.6.16.19/arch/i386/kernel/setup.c
+===================================================================
+--- linux-2.6.16.19.orig/arch/i386/kernel/setup.c	2006-06-23 17:05:11.000000000 -0700
++++ linux-2.6.16.19/arch/i386/kernel/setup.c	2006-06-23 17:06:28.000000000 -0700
+@@ -1591,7 +1591,9 @@
+ #endif
+ 
+ 
++#ifdef CONFIG_DMI_SCAN
+ 	dmi_scan_machine();
++#endif
+ 
+ #ifdef CONFIG_X86_GENERICARCH
+ 	generic_apic_probe(*cmdline_p);
+Index: linux-2.6.16.19/arch/x86_64/kernel/setup.c
+===================================================================
+--- linux-2.6.16.19.orig/arch/x86_64/kernel/setup.c	2006-06-23 17:05:11.000000000 -0700
++++ linux-2.6.16.19/arch/x86_64/kernel/setup.c	2006-06-23 17:06:28.000000000 -0700
+@@ -1409,7 +1409,9 @@
+ 
+ static int __init run_dmi_scan(void)
+ {
++#ifdef CONFIG_DMI_SCAN
+ 	dmi_scan_machine();
++#endif
+ 	return 0;
+ }
+ core_initcall(run_dmi_scan);
+Index: linux-2.6.16.19/include/linux/dmi.h
+===================================================================
+--- linux-2.6.16.19.orig/include/linux/dmi.h	2006-06-23 17:05:11.000000000 -0700
++++ linux-2.6.16.19/include/linux/dmi.h	2006-06-23 17:06:28.000000000 -0700
+@@ -67,7 +67,9 @@
+ extern char * dmi_get_system_info(int field);
+ extern struct dmi_device * dmi_find_device(int type, const char *name,
+ 	struct dmi_device *from);
++#ifdef CONFIG_DMI_SCAN
+ extern void dmi_scan_machine(void);
++#endif
+ 
+ #else
+ 
+Index: linux-2.6.16.19/init/Kconfig
+===================================================================
+--- linux-2.6.16.19.orig/init/Kconfig	2006-06-23 17:06:26.000000000 -0700
++++ linux-2.6.16.19/init/Kconfig	2006-06-23 17:06:28.000000000 -0700
+@@ -609,6 +609,17 @@
+           this feature if you are only need legacy serial support.
+ 	  Saves about 9K.
+ 
++config DMI_SCAN
++	depends X86
++	default y
++	bool "Enabled DMI scanning" if EMBEDDED
++	help
++          Enabled scanning of DMI to identify machine quirks. Say Y
++          here unless you have verified that your setup is not
++          affected by entries in the DMI blacklist. Required by PNP
++          BIOS code. Disabling scanning saves about 5k of
++          initialization code.
++
+ endmenu		# General setup
+ 
+ config CRC32_CALC
Index: packages/kernel/patches/tiny_m57sli/tiny/audit-bootmem.patch
===================================================================
--- packages/kernel/patches/tiny_m57sli/tiny/audit-bootmem.patch	(revision 0)
+++ packages/kernel/patches/tiny_m57sli/tiny/audit-bootmem.patch	(revision 0)
@@ -0,0 +1,56 @@
+
+Cause bootmem code to print callers and sizes for allocations
+
+
+ tiny-mpm/init/Kconfig |    7 +++++++
+ tiny-mpm/mm/bootmem.c |   25 +++++++++++++++++++++----
+ 2 files changed, 28 insertions(+), 4 deletions(-)
+
+Index: linux-2.6.16.19/init/Kconfig
+===================================================================
+--- linux-2.6.16.19.orig/init/Kconfig	2006-06-23 17:06:18.000000000 -0700
++++ linux-2.6.16.19/init/Kconfig	2006-06-23 17:06:18.000000000 -0700
+@@ -327,6 +327,13 @@
+ 	  This option records kmalloc and kfree activity and reports it via
+ 	  /proc/kmalloc.
+ 
++config AUDIT_BOOTMEM
++	default n
++	bool "Enabled auditing of bootmem allocations" if EMBEDDED
++	help
++	  This option causes the bootmem allocator to print caller and size
++          for each allocation.
++
+ config BASE_FULL
+ 	default y
+ 	bool "Enable full-sized data structures for core" if EMBEDDED
+Index: linux-2.6.16.19/mm/bootmem.c
+===================================================================
+--- linux-2.6.16.19.orig/mm/bootmem.c	2006-06-23 17:05:13.000000000 -0700
++++ linux-2.6.16.19/mm/bootmem.c	2006-06-23 17:06:18.000000000 -0700
+@@ -17,6 +17,7 @@
+ #include <linux/bootmem.h>
+ #include <linux/mmzone.h>
+ #include <linux/module.h>
++#include <linux/kallsyms.h>
+ #include <asm/dma.h>
+ #include <asm/io.h>
+ #include "internal.h"
+@@ -421,8 +422,16 @@
+ 
+ 	for_each_pgdat(pgdat)
+ 		if ((ptr = __alloc_bootmem_core(pgdat->bdata, size,
+-						 align, goal, LOW32LIMIT)))
++						 align, goal, LOW32LIMIT))) {
++#ifdef CONFIG_AUDIT_BOOTMEM
++			printk("bootmem: %ld/%ld by %p", size, align,
++			       __builtin_return_address(0));
++			print_symbol(" (%s)",
++				     (int)__builtin_return_address(0));
++			printk("-> %p\n", ptr);
++#endif
+ 			return(ptr);
++		}
+ 
+ 	/*
+ 	 * Whoops, we cannot satisfy the allocation request.
Index: packages/kernel/patches/tiny_m57sli/tiny/threadinfo-ool.patch
===================================================================
--- packages/kernel/patches/tiny_m57sli/tiny/threadinfo-ool.patch	(revision 0)
+++ packages/kernel/patches/tiny_m57sli/tiny/threadinfo-ool.patch	(revision 0)
@@ -0,0 +1,163 @@
+
+configurable inlining of current() and current_thread_info()
+
+On UP, make get_current and current_thread_info access global
+variables set by switch_to, rather than doing math on the stack
+pointer. Saves about 4k.
+
+
+
+ tiny-mpm/arch/i386/kernel/process.c     |   11 +++++++++++
+ tiny-mpm/include/asm-i386/current.h     |    9 ++++++++-
+ tiny-mpm/include/asm-i386/thread_info.h |   10 ++++++++++
+ tiny-mpm/init/Kconfig                   |    8 ++++++++
+ 4 files changed, 37 insertions(+), 1 deletion(-)
+
+Index: linux-2.6.16.19/arch/i386/kernel/irq.c
+===================================================================
+--- linux-2.6.16.19.orig/arch/i386/kernel/irq.c	2006-06-23 17:06:12.000000000 -0700
++++ linux-2.6.16.19/arch/i386/kernel/irq.c	2006-06-23 17:06:19.000000000 -0700
+@@ -95,6 +95,10 @@
+ 		irqctx->tinfo.task = curctx->tinfo.task;
+ 		irqctx->tinfo.previous_esp = current_stack_pointer;
+ 
++#ifndef CONFIG_INLINE_THREADINFO
++		__current_thread_info = (struct thread_info *)irqctx;
++#endif
++
+ 		asm volatile(
+ 			"       xchgl   %%ebx,%%esp      \n"
+ 			"       call    __do_IRQ         \n"
+@@ -103,6 +107,10 @@
+ 			:  "0" (irq),   "1" (regs),  "2" (isp)
+ 			: "memory", "cc", "ecx"
+ 		);
++
++#ifndef CONFIG_INLINE_THREADINFO
++		__current_thread_info = (struct thread_info *)curctx;
++#endif
+ 	} else
+ #endif
+ 		__do_IRQ(irq, regs);
+@@ -186,6 +194,10 @@
+ 		/* build the stack frame on the softirq stack */
+ 		isp = (u32*) ((char*)irqctx + sizeof(*irqctx));
+ 
++#ifndef CONFIG_INLINE_THREADINFO
++		__current_thread_info = (struct thread_info *)irqctx;
++#endif
++
+ 		asm volatile(
+ 			"       xchgl   %%ebx,%%esp     \n"
+ 			"       call    __do_softirq    \n"
+@@ -194,6 +206,10 @@
+ 			: "0"(isp)
+ 			: "memory", "cc", "edx", "ecx", "eax"
+ 		);
++
++#ifndef CONFIG_INLINE_THREADINFO
++		__current_thread_info = (struct thread_info *)curctx;
++#endif
+ 	}
+ 
+ 	local_irq_restore(flags);
+Index: linux-2.6.16.19/arch/i386/kernel/process.c
+===================================================================
+--- linux-2.6.16.19.orig/arch/i386/kernel/process.c	2006-06-23 17:05:13.000000000 -0700
++++ linux-2.6.16.19/arch/i386/kernel/process.c	2006-06-23 17:06:19.000000000 -0700
+@@ -65,6 +65,13 @@
+ unsigned long boot_option_idle_override = 0;
+ EXPORT_SYMBOL(boot_option_idle_override);
+ 
++#ifndef CONFIG_INLINE_THREADINFO
++struct thread_info *__current_thread_info = &init_thread_info;
++struct task_struct *__current_task = &init_task;
++EXPORT_SYMBOL(__current_thread_info);
++EXPORT_SYMBOL(__current_task);
++#endif
++
+ /*
+  * Return saved PC of a blocked thread.
+  */
+@@ -697,6 +704,11 @@
+ 
+ 	disable_tsc(prev_p, next_p);
+ 
++#ifndef CONFIG_INLINE_THREADINFO
++	__current_thread_info = next_p->thread_info;
++	__current_task = next_p;
++#endif
++
+ 	return prev_p;
+ }
+ 
+Index: linux-2.6.16.19/include/asm-i386/current.h
+===================================================================
+--- linux-2.6.16.19.orig/include/asm-i386/current.h	2006-06-23 17:05:13.000000000 -0700
++++ linux-2.6.16.19/include/asm-i386/current.h	2006-06-23 17:06:19.000000000 -0700
+@@ -5,11 +5,18 @@
+ 
+ struct task_struct;
+ 
++#ifdef CONFIG_INLINE_THREADINFO
+ static __always_inline struct task_struct * get_current(void)
+ {
+ 	return current_thread_info()->task;
+ }
+- 
++#else
++extern struct task_struct *__current_task;
++static __always_inline struct task_struct * get_current(void)
++{
++	return __current_task;
++}
++#endif
+ #define current get_current()
+ 
+ #endif /* !(_I386_CURRENT_H) */
+Index: linux-2.6.16.19/include/asm-i386/thread_info.h
+===================================================================
+--- linux-2.6.16.19.orig/include/asm-i386/thread_info.h	2006-06-23 17:05:13.000000000 -0700
++++ linux-2.6.16.19/include/asm-i386/thread_info.h	2006-06-23 17:06:19.000000000 -0700
+@@ -85,12 +85,22 @@
+ 
+ 
+ /* how to get the thread information struct from C */
++#ifdef CONFIG_INLINE_THREADINFO
+ static inline struct thread_info *current_thread_info(void)
+ {
+ 	struct thread_info *ti;
+ 	__asm__("andl %%esp,%0; ":"=r" (ti) : "0" (~(THREAD_SIZE - 1)));
+ 	return ti;
+ }
++#else
++extern struct thread_info *__current_thread_info;
++
++static inline struct thread_info *current_thread_info(void)
++{
++	return __current_thread_info;
++}
++
++#endif
+ 
+ /* how to get the current stack pointer from C */
+ register unsigned long current_stack_pointer asm("esp") __attribute_used__;
+Index: linux-2.6.16.19/init/Kconfig
+===================================================================
+--- linux-2.6.16.19.orig/init/Kconfig	2006-06-23 17:06:19.000000000 -0700
++++ linux-2.6.16.19/init/Kconfig	2006-06-23 17:06:19.000000000 -0700
+@@ -456,6 +456,14 @@
+           reduces kernel size by about 2K. This may actually result in
+           faster code on machines with limited memory bandwidth.
+ 
++config INLINE_THREADINFO
++	depends X86
++	default y
++	bool "Inline current pointer calculation" if EMBEDDED && !SMP
++	help
++	  Disabling this simplifies the code for finding information
++          about the current thread. Saves about 4k on small kernels.
++
+ endmenu		# General setup
+ 
+ config CRC32_CALC
Index: packages/kernel/patches/tiny_m57sli/tiny/inflate-killmemzero.patch
===================================================================
--- packages/kernel/patches/tiny_m57sli/tiny/inflate-killmemzero.patch	(revision 0)
+++ packages/kernel/patches/tiny_m57sli/tiny/inflate-killmemzero.patch	(revision 0)
@@ -0,0 +1,24 @@
+inflate: replace call to memzero with simple loop
+
+This is the only user of memzero in the inflate code and it's only for
+16 bytes. Removing this lets us drop a copy of memzero from most
+lib/inflate users.
+
+Signed-off-by: Matt Mackall <mpm@selenic.com>
+
+Index: linux-2.6.16.19/lib/inflate.c
+===================================================================
+--- linux-2.6.16.19.orig/lib/inflate.c	2006-06-23 17:06:31.000000000 -0700
++++ linux-2.6.16.19/lib/inflate.c	2006-06-23 17:06:31.000000000 -0700
+@@ -367,8 +367,10 @@
+ 
+ 	DEBG("huft1 ");
+ 
++	for (i = 0; i < BMAX + 1; i++)
++		c[i] = 0;
++
+ 	/* Generate counts for each bit length */
+-	memzero(c, sizeof(c));
+ 	p = b;
+ 	i = n;
+ 	do {
Index: packages/kernel/patches/tiny_m57sli/tiny/tiny-panic.patch
===================================================================
--- packages/kernel/patches/tiny_m57sli/tiny/tiny-panic.patch	(revision 0)
+++ packages/kernel/patches/tiny_m57sli/tiny/tiny-panic.patch	(revision 0)
@@ -0,0 +1,113 @@
+
+configurable panic
+
+
+
+ include/linux/kernel.h |    5 	5 +	0 -	0 !
+ init/Kconfig           |    8 	8 +	0 -	0 !
+ kernel/panic.c         |   24 	22 +	2 -	0 !
+ 3 files changed, 35 insertions(+), 2 deletions(-)
+
+Index: linux-2.6.22.1/include/linux/kernel.h
+===================================================================
+--- linux-2.6.22.1.orig/include/linux/kernel.h
++++ linux-2.6.22.1/include/linux/kernel.h
+@@ -105,8 +105,13 @@ extern int cond_resched(void);
+ 
+ extern struct atomic_notifier_head panic_notifier_list;
+ extern long (*panic_blink)(long time);
++#ifdef CONFIG_FULL_PANIC
+ NORET_TYPE void panic(const char * fmt, ...)
+ 	__attribute__ ((NORET_AND format (printf, 1, 2)));
++#else
++#define panic(fmt, ...) tiny_panic(0, ## __VA_ARGS__)
++NORET_TYPE void tiny_panic(int a, ...) ATTRIB_NORET;
++#endif
+ extern void oops_enter(void);
+ extern void oops_exit(void);
+ extern int oops_may_print(void);
+Index: linux-2.6.22.1/init/Kconfig
+===================================================================
+--- linux-2.6.22.1.orig/init/Kconfig
++++ linux-2.6.22.1/init/Kconfig
+@@ -467,6 +467,14 @@ config ELF_CORE
+ 	help
+ 	  Enable support for generating core dumps. Disabling saves about 4k.
+ 
++config FULL_PANIC
++	default y
++	bool "Full panic reporting data" if EMBEDDED
++	help
++	  This includes text descriptions of panics in addition to stack dumps.
++          Disabling compiles out the explanations for panics, saving
++	  string space. Use with caution.
++
+ config BASE_FULL
+ 	default y
+ 	bool "Enable full-sized data structures for core" if EMBEDDED
+Index: linux-2.6.22.1/kernel/panic.c
+===================================================================
+--- linux-2.6.22.1.orig/kernel/panic.c
++++ linux-2.6.22.1/kernel/panic.c
+@@ -56,12 +56,18 @@ EXPORT_SYMBOL(panic_blink);
+  *
+  *	This function never returns.
+  */
+- 
++
++#ifdef CONFIG_FULL_PANIC
+ NORET_TYPE void panic(const char * fmt, ...)
+ {
+-	long i;
+ 	static char buf[1024];
+ 	va_list args;
++#else
++NORET_TYPE void tiny_panic(int a, ...)
++{
++#endif
++	long i;
++
+ #if defined(CONFIG_S390)
+         unsigned long caller = (unsigned long) __builtin_return_address(0);
+ #endif
+@@ -74,10 +80,16 @@ NORET_TYPE void panic(const char * fmt, 
+ 	preempt_disable();
+ 
+ 	bust_spinlocks(1);
++
++#ifdef CONFIG_FULL_PANIC
+ 	va_start(args, fmt);
+ 	vsnprintf(buf, sizeof(buf), fmt, args);
+ 	va_end(args);
+ 	printk(KERN_EMERG "Kernel panic - not syncing: %s\n",buf);
++#else
++	printk(KERN_EMERG "Kernel panic - not syncing\n");
++#endif
++
+ 	bust_spinlocks(0);
+ 
+ 	/*
+@@ -96,7 +108,11 @@ NORET_TYPE void panic(const char * fmt, 
+ 	smp_send_stop();
+ #endif
+ 
++#ifdef CONFIG_FULL_PANIC
+ 	atomic_notifier_call_chain(&panic_notifier_list, 0, buf);
++#else
++	atomic_notifier_call_chain(&panic_notifier_list, 0, "");
++#endif
+ 
+ 	if (!panic_blink)
+ 		panic_blink = no_blink;
+@@ -139,7 +155,11 @@ NORET_TYPE void panic(const char * fmt, 
+ 	}
+ }
+ 
++#ifdef CONFIG_FULL_PANIC
+ EXPORT_SYMBOL(panic);
++#else
++EXPORT_SYMBOL(tiny_panic);
++#endif
+ 
+ /**
+  *	print_tainted - return a string to represent the kernel taint state.
Index: packages/kernel/patches/tiny_m57sli/tiny/cache_defer_hash.patch
===================================================================
--- packages/kernel/patches/tiny_m57sli/tiny/cache_defer_hash.patch	(revision 0)
+++ packages/kernel/patches/tiny_m57sli/tiny/cache_defer_hash.patch	(revision 0)
@@ -0,0 +1,24 @@
+
+CONFIG_NET_SMALL reduce rpc hash size
+
+
+ net/sunrpc/cache.c |    5 	5 +	0 -	0 !
+ 1 files changed, 5 insertions(+)
+
+Index: linux-2.6.22.1/net/sunrpc/cache.c
+===================================================================
+--- linux-2.6.22.1.orig/net/sunrpc/cache.c
++++ linux-2.6.22.1/net/sunrpc/cache.c
+@@ -516,7 +516,12 @@ void cache_purge(struct cache_detail *de
+  * it to be revisited when cache info is available
+  */
+ 
++#ifdef CONFIG_NET_SMALL
++#define	DFR_HASHSIZE	(512/sizeof(struct list_head))
++#else
+ #define	DFR_HASHSIZE	(PAGE_SIZE/sizeof(struct list_head))
++#endif
++
+ #define	DFR_HASH(item)	((((long)item)>>4 ^ (((long)item)>>13)) % DFR_HASHSIZE)
+ 
+ #define	DFR_MAX	300	/* ??? */
Index: packages/kernel/patches/tiny_m57sli/tiny/inflate-killsillytypes.patch
===================================================================
--- packages/kernel/patches/tiny_m57sli/tiny/inflate-killsillytypes.patch	(revision 0)
+++ packages/kernel/patches/tiny_m57sli/tiny/inflate-killsillytypes.patch	(revision 0)
@@ -0,0 +1,412 @@
+inflate: remove legacy type definitions from callers
+
+This replaces the legacy zlib typedefs and usage with kernel types in
+all the inflate users.
+
+Signed-off-by: Matt Mackall <mpm@selenic.com>
+
+Index: alp-linux/arch/alpha/boot/misc.c
+===================================================================
+--- alp-linux.orig/arch/alpha/boot/misc.c	2006-07-12 15:20:35.000000000 -0700
++++ alp-linux/arch/alpha/boot/misc.c	2006-07-12 15:22:40.000000000 -0700
+@@ -28,20 +28,16 @@
+  * gzip delarations
+  */
+ 
+-typedef unsigned char  uch;
+-typedef unsigned short ush;
+-typedef unsigned long  ulg;
+-
+ static void error(char *m);
+ 
+ static char *input_data;
+ static int  input_data_size;
+ 
+-static uch *output_data;
++static u8 *output_data;
+ 
+ extern int end;
+-static ulg free_mem_ptr;
+-static ulg free_mem_ptr_end;
++static u32 free_mem_ptr;
++static u32 free_mem_ptr_end;
+ 
+ #define HEAP_SIZE 0x2000
+ 
+@@ -70,13 +66,13 @@
+ 		  size_t kzsize)
+ {
+ 	int count;
+-	output_data		= (uch *)output_start;
+-	input_data		= (uch *)input_start;
++	output_data		= (u8 *)output_start;
++	input_data		= (u8 *)input_start;
+ 	input_data_size		= kzsize; /* use compressed size */
+ 
+ 	/* FIXME FIXME FIXME */
+-	free_mem_ptr		= (ulg)output_start + ksize;
+-	free_mem_ptr_end	= (ulg)output_start + ksize + 0x200000;
++	free_mem_ptr		= (u32)output_start + ksize;
++	free_mem_ptr_end	= (u32)output_start + ksize + 0x200000;
+ 	/* FIXME FIXME FIXME */
+ 
+ /*	puts("Uncompressing Linux..."); */
+Index: alp-linux/arch/arm/boot/compressed/misc.c
+===================================================================
+--- alp-linux.orig/arch/arm/boot/compressed/misc.c	2006-07-12 15:22:12.000000000 -0700
++++ alp-linux/arch/arm/boot/compressed/misc.c	2006-07-12 15:23:57.000000000 -0700
+@@ -46,22 +46,18 @@
+  * gzip declarations
+  */
+ 
+-typedef unsigned char  uch;
+-typedef unsigned short ush;
+-typedef unsigned long  ulg;
+-
+ static void error(char *m);
+ 
+ extern char input_data[];
+ extern char input_data_end[];
+ 
+-static uch *output_data;
++static u8 *output_data;
+ 
+ static void putstr(const char *);
+ 
+ extern int end;
+-static ulg free_mem_ptr;
+-static ulg free_mem_ptr_end;
++static u32 free_mem_ptr;
++static u32 free_mem_ptr_end;
+ 
+ #define HEAP_SIZE 0x2000
+ 
+@@ -96,13 +92,13 @@
+ 
+ #ifndef STANDALONE_DEBUG
+ 
+-ulg
+-decompress_kernel(ulg output_start, ulg free_mem_ptr_p, ulg free_mem_ptr_end_p,
++u32
++decompress_kernel(u32 output_start, u32 free_mem_ptr_p, u32 free_mem_ptr_end_p,
+ 		  int arch_id)
+ {
+ 	int count;
+ 
+-	output_data		= (uch *)output_start;	/* Points to kernel start */
++	output_data		= (u8 *)output_start;	/* Points to kernel start */
+ 	free_mem_ptr		= free_mem_ptr_p;
+ 	free_mem_ptr_end	= free_mem_ptr_end_p;
+ 	__machine_arch_type	= arch_id;
+Index: alp-linux/arch/arm26/boot/compressed/misc.c
+===================================================================
+--- alp-linux.orig/arch/arm26/boot/compressed/misc.c	2006-07-12 15:20:35.000000000 -0700
++++ alp-linux/arch/arm26/boot/compressed/misc.c	2006-07-12 15:22:40.000000000 -0700
+@@ -31,21 +31,17 @@
+  * gzip delarations
+  */
+ 
+-typedef unsigned char  uch;
+-typedef unsigned short ush;
+-typedef unsigned long  ulg;
+-
+ static void error(char *m);
+ 
+ extern char input_data[];
+ extern char input_data_end[];
+-static uch *output_data;
++static u8 *output_data;
+ 
+ static void puts(const char *);
+ 
+ extern int end;
+-static ulg free_mem_ptr;
+-static ulg free_mem_ptr_end;
++static u32 free_mem_ptr;
++static u32 free_mem_ptr_end;
+ 
+ #define HEAP_SIZE 0x2000
+ 
+@@ -76,11 +72,11 @@
+ 
+ #ifndef STANDALONE_DEBUG
+ 
+-ulg
+-decompress_kernel(ulg output_start, ulg free_mem_ptr_p, ulg free_mem_ptr_end_p,
++u32
++decompress_kernel(u32 output_start, u32 free_mem_ptr_p, u32 free_mem_ptr_end_p,
+ 		  int arch_id)
+ {
+-	output_data		= (uch *)output_start;	/* Points to kernel start */
++	output_data		= (u8 *)output_start;	/* Points to kernel start */
+ 	free_mem_ptr		= free_mem_ptr_p;
+ 	free_mem_ptr_end	= free_mem_ptr_end_p;
+ 	__machine_arch_type	= arch_id;
+Index: alp-linux/arch/cris/arch-v10/boot/compressed/misc.c
+===================================================================
+--- alp-linux.orig/arch/cris/arch-v10/boot/compressed/misc.c	2006-07-12 15:20:35.000000000 -0700
++++ alp-linux/arch/cris/arch-v10/boot/compressed/misc.c	2006-07-12 15:22:40.000000000 -0700
+@@ -28,16 +28,12 @@
+  * gzip declarations
+  */
+ 
+-typedef unsigned char  uch;
+-typedef unsigned short ush;
+-typedef unsigned long  ulg;
+-
+ unsigned compsize; /* compressed size, used by head.S */
+ 
+ static void error(char *m);
+ 
+ extern char *input_data;  /* lives in head.S */
+-static uch *output_data;
++static u8 *output_data;
+ 
+ static void puts(const char *);
+ 
+Index: alp-linux/arch/cris/arch-v32/boot/compressed/misc.c
+===================================================================
+--- alp-linux.orig/arch/cris/arch-v32/boot/compressed/misc.c	2006-07-12 15:20:35.000000000 -0700
++++ alp-linux/arch/cris/arch-v32/boot/compressed/misc.c	2006-07-12 15:22:40.000000000 -0700
+@@ -30,14 +30,10 @@
+  * gzip declarations
+  */
+ 
+-typedef unsigned char  uch;
+-typedef unsigned short ush;
+-typedef unsigned long  ulg;
+-
+ static void error(char *m);
+ 
+ extern char *input_data;  /* lives in head.S */
+-static uch *output_data;
++static u8 *output_data;
+ 
+ static void puts(const char *);
+ 
+Index: alp-linux/arch/i386/boot/compressed/misc.c
+===================================================================
+--- alp-linux.orig/arch/i386/boot/compressed/misc.c	2006-07-12 15:20:35.000000000 -0700
++++ alp-linux/arch/i386/boot/compressed/misc.c	2006-07-12 15:22:40.000000000 -0700
+@@ -23,9 +23,6 @@
+  *
+  * Incomprehensible are the ways of bootloaders.
+  */
+-typedef unsigned char  uch;
+-typedef unsigned short ush;
+-typedef unsigned long  ulg;
+ 
+ static void error(char *m);
+ 
+@@ -43,7 +40,7 @@
+ extern char input_data[];
+ extern int input_len;
+ static long bytes_out;
+-static uch *output_data;
++static u8 *output_data;
+ 
+ static void putstr(const char *);
+ 
+@@ -57,7 +54,7 @@
+ #define HEAP_SIZE             0x3000
+ static unsigned int low_buffer_end, low_buffer_size;
+ static int high_loaded =0;
+-static uch *high_buffer_start /* = (uch *)(((ulg)&end) + HEAP_SIZE)*/;
++static u8 *high_buffer_start /* = (u8 *)(((u32)&end) + HEAP_SIZE)*/;
+ 
+ static char *vidmem = (char *)0xb8000;
+ static int vidport;
+@@ -158,13 +155,13 @@
+ }
+ 
+ struct moveparams {
+-	uch *low_buffer_start;  int lcount;
+-	uch *high_buffer_start; int hcount;
++	u8 *low_buffer_start;  int lcount;
++	u8 *high_buffer_start; int hcount;
+ };
+ 
+ static void setup_output_buffer_if_we_run_high(struct moveparams *mv)
+ {
+-	high_buffer_start = (uch *)(((ulg)&end) + HEAP_SIZE);
++	high_buffer_start = (u8 *)(((u32)&end) + HEAP_SIZE);
+ #ifdef STANDARD_MEMORY_BIOS_CALL
+ 	if (RM_EXT_MEM_K < (3*1024)) error("Less than 4MB of memory");
+ #else
+@@ -178,8 +175,8 @@
+ 	low_buffer_size = low_buffer_end - LOW_BUFFER_START;
+ 	high_loaded = 1;
+ 	free_mem_end_ptr = (long)high_buffer_start;
+-	if ( (__PHYSICAL_START + low_buffer_size) > ((ulg)high_buffer_start)) {
+-		high_buffer_start = (uch *)(__PHYSICAL_START + low_buffer_size);
++	if ( (__PHYSICAL_START + low_buffer_size) > ((u32)high_buffer_start)) {
++		high_buffer_start = (u8 *)(__PHYSICAL_START + low_buffer_size);
+ 		mv->hcount = 0; /* say: we need not to move high_buffer */
+ 	}
+ 	else mv->hcount = -1;
+Index: alp-linux/arch/m32r/boot/compressed/misc.c
+===================================================================
+--- alp-linux.orig/arch/m32r/boot/compressed/misc.c	2006-07-12 15:20:35.000000000 -0700
++++ alp-linux/arch/m32r/boot/compressed/misc.c	2006-07-12 15:22:40.000000000 -0700
+@@ -17,15 +17,11 @@
+  * gzip declarations
+  */
+ 
+-typedef unsigned char  uch;
+-typedef unsigned short ush;
+-typedef unsigned long  ulg;
+-
+ static void error(char *m);
+ 
+ static unsigned char *input_data;
+ static int input_len;
+-static uch *output_data;
++static u8 *output_data;
+ 
+ #include "m32r_sio.c"
+ 
+Index: alp-linux/arch/sh/boot/compressed/misc.c
+===================================================================
+--- alp-linux.orig/arch/sh/boot/compressed/misc.c	2006-07-12 15:20:35.000000000 -0700
++++ alp-linux/arch/sh/boot/compressed/misc.c	2006-07-12 15:22:40.000000000 -0700
+@@ -19,15 +19,11 @@
+  * gzip declarations
+  */
+ 
+-typedef unsigned char  uch;
+-typedef unsigned short ush;
+-typedef unsigned long  ulg;
+-
+ static void error(char *m);
+ 
+ extern char input_data[];
+ extern int input_len;
+-static uch *output_data;
++static u8 *output_data;
+ 
+ int puts(const char *);
+ 
+Index: alp-linux/arch/sh64/boot/compressed/misc.c
+===================================================================
+--- alp-linux.orig/arch/sh64/boot/compressed/misc.c	2006-07-12 15:20:35.000000000 -0700
++++ alp-linux/arch/sh64/boot/compressed/misc.c	2006-07-12 15:22:40.000000000 -0700
+@@ -19,15 +19,11 @@
+  * gzip declarations
+  */
+ 
+-typedef unsigned char uch;
+-typedef unsigned short ush;
+-typedef unsigned long ulg;
+-
+ static void error(char *m);
+ 
+ extern char input_data[];
+ extern int input_len;
+-static uch *output_data;
++static u8 *output_data;
+ 
+ static void puts(const char *);
+ 
+@@ -68,7 +64,7 @@
+ 
+ void decompress_kernel(void)
+ {
+-	output_data = (uch *) (CONFIG_MEMORY_START + 0x2000);
++	output_data = (u8 *) (CONFIG_MEMORY_START + 0x2000);
+ 	free_mem_ptr = (unsigned long) &_end;
+ 	free_mem_end_ptr = free_mem_ptr + HEAP_SIZE;
+ 
+Index: alp-linux/arch/x86_64/boot/compressed/misc.c
+===================================================================
+--- alp-linux.orig/arch/x86_64/boot/compressed/misc.c	2006-07-12 15:20:35.000000000 -0700
++++ alp-linux/arch/x86_64/boot/compressed/misc.c	2006-07-12 15:22:40.000000000 -0700
+@@ -16,10 +16,6 @@
+  * gzip declarations
+  */
+ 
+-typedef unsigned char  uch;
+-typedef unsigned short ush;
+-typedef unsigned long  ulg;
+-
+ static void error(char *m);
+ 
+ /*
+@@ -37,7 +33,7 @@
+ extern int input_len;
+ 
+ static long bytes_out;
+-static uch *output_data;
++static u8 *output_data;
+ 
+ static void putstr(const char *);
+ 
+@@ -51,7 +47,7 @@
+ #define HEAP_SIZE             0x3000
+ static unsigned int low_buffer_end, low_buffer_size;
+ static int high_loaded =0;
+-static uch *high_buffer_start /* = (uch *)(((ulg)&end) + HEAP_SIZE)*/;
++static u8 *high_buffer_start /* = (u8 *)(((u32)&end) + HEAP_SIZE)*/;
+ 
+ static char *vidmem = (char *)0xb8000;
+ static int vidport;
+@@ -139,13 +135,13 @@
+ }
+ 
+ struct moveparams {
+-	uch *low_buffer_start;  int lcount;
+-	uch *high_buffer_start; int hcount;
++	u8 *low_buffer_start;  int lcount;
++	u8 *high_buffer_start; int hcount;
+ };
+ 
+ void setup_output_buffer_if_we_run_high(struct moveparams *mv)
+ {
+-	high_buffer_start = (uch *)(((ulg)&end) + HEAP_SIZE);
++	high_buffer_start = (u8 *)(((u32)&end) + HEAP_SIZE);
+ #ifdef STANDARD_MEMORY_BIOS_CALL
+ 	if (EXT_MEM_K < (3*1024)) error("Less than 4MB of memory");
+ #else
+@@ -157,8 +153,8 @@
+ 	low_buffer_size = low_buffer_end - LOW_BUFFER_START;
+ 	high_loaded = 1;
+ 	free_mem_end_ptr = (long)high_buffer_start;
+-	if ( (__PHYSICAL_START + low_buffer_size) > ((ulg)high_buffer_start)) {
+-		high_buffer_start = (uch *)(__PHYSICAL_START + low_buffer_size);
++	if ( (__PHYSICAL_START + low_buffer_size) > ((u32)high_buffer_start)) {
++		high_buffer_start = (u8 *)(__PHYSICAL_START + low_buffer_size);
+ 		mv->hcount = 0; /* say: we need not to move high_buffer */
+ 	}
+ 	else mv->hcount = -1;
+Index: alp-linux/init/do_mounts_rd.c
+===================================================================
+--- alp-linux.orig/init/do_mounts_rd.c	2006-07-12 15:20:35.000000000 -0700
++++ alp-linux/init/do_mounts_rd.c	2006-07-12 15:22:40.000000000 -0700
+@@ -286,12 +286,8 @@
+  * gzip declarations
+  */
+ 
+-typedef unsigned char  uch;
+-typedef unsigned short ush;
+-typedef unsigned long  ulg;
+-
+ #define INBUFSIZ 4096
+-static uch *inbuf;
++static u8 *inbuf;
+ static int exit_code;
+ static int unzip_error;
+ static int crd_infd, crd_outfd;
+Index: alp-linux/init/initramfs.c
+===================================================================
+--- alp-linux.orig/init/initramfs.c	2006-07-12 15:20:35.000000000 -0700
++++ alp-linux/init/initramfs.c	2006-07-12 15:22:40.000000000 -0700
+@@ -333,10 +333,6 @@
+  * gzip declarations
+  */
+ 
+-typedef unsigned char  uch;
+-typedef unsigned short ush;
+-typedef unsigned long  ulg;
+-
+ #define INIT __init
+ 
+ static void __init error(char *m);
Index: packages/kernel/patches/tiny_m57sli/tiny/ethtool.patch
===================================================================
--- packages/kernel/patches/tiny_m57sli/tiny/ethtool.patch	(revision 0)
+++ packages/kernel/patches/tiny_m57sli/tiny/ethtool.patch	(revision 0)
@@ -0,0 +1,104 @@
+
+Configurable ethtool support
+
+
+ include/linux/ethtool.h |   18 	18 +	0 -	0 !
+ init/Kconfig            |    8 	8 +	0 -	0 !
+ net/core/Makefile       |    3 	2 +	1 -	0 !
+ net/core/dev.c          |    4 	4 +	0 -	0 !
+ 4 files changed, 32 insertions(+), 1 deletion(-)
+
+Index: linux-2.6.22.1/include/linux/ethtool.h
+===================================================================
+--- linux-2.6.22.1.orig/include/linux/ethtool.h
++++ linux-2.6.22.1/include/linux/ethtool.h
+@@ -260,6 +260,7 @@ struct ethtool_perm_addr {
+ 
+ struct net_device;
+ 
++#ifdef CONFIG_ETHTOOL
+ /* Some generic methods drivers may use in their ethtool_ops */
+ u32 ethtool_op_get_link(struct net_device *dev);
+ u32 ethtool_op_get_tx_csum(struct net_device *dev);
+@@ -273,6 +274,23 @@ int ethtool_op_get_perm_addr(struct net_
+ 			     struct ethtool_perm_addr *addr, u8 *data);
+ u32 ethtool_op_get_ufo(struct net_device *dev);
+ int ethtool_op_set_ufo(struct net_device *dev, u32 data);
++#else
++static inline u32 ethtool_noop1(struct net_device *dev) { return 0; }
++static inline int ethtool_noop2(struct net_device *dev, u32 data) { return 0; }
++static inline int ethtool_noop3(struct net_device *dev,
++				struct ethtool_perm_addr *addr, u32 data) { return 0; }
++#define ethtool_op_get_link ethtool_noop1
++#define ethtool_op_get_tx_csum ethtool_noop1
++#define ethtool_op_set_tx_csum ethtool_noop2
++#define ethtool_op_set_tx_hw_csum ethtool_noop2
++#define ethtool_op_get_sg ethtool_noop1
++#define ethtool_op_set_sg ethtool_noop2
++#define ethtool_op_get_tso ethtool_noop1
++#define ethtool_op_set_tso ethtool_noop2
++#define ethtool_op_get_perm_addr ethtool_noop3
++#define ethtool_op_get_ufo ethtool_noop1
++#define ethtool_op_set_ufo ethtool_noop2
++#endif
+ 
+ /**
+  * &ethtool_ops - Alter and report network device settings
+Index: linux-2.6.22.1/init/Kconfig
+===================================================================
+--- linux-2.6.22.1.orig/init/Kconfig
++++ linux-2.6.22.1/init/Kconfig
+@@ -373,6 +373,14 @@ config FILE_LOCKING
+ 	help
+ 	  This enables standard file locking support.
+ 
++config ETHTOOL
++	depends NET
++	default y
++	bool "Enable ethtool support" if EMBEDDED
++	help
++	  Disabling this option removes support for configuring
++	  ethernet device features via ethtool. Saves about 6k.
++
+ config MEASURE_INLINES
+ 	default n
+ 	bool "Enable inline measurement" if EMBEDDED
+Index: linux-2.6.22.1/net/core/Makefile
+===================================================================
+--- linux-2.6.22.1.orig/net/core/Makefile
++++ linux-2.6.22.1/net/core/Makefile
+@@ -7,10 +7,11 @@ obj-y := sock.o request_sock.o skbuff.o 
+ 
+ obj-$(CONFIG_SYSCTL) += sysctl_net_core.o
+ 
+-obj-y		     += dev.o ethtool.o dev_mcast.o dst.o netevent.o \
++obj-y		     += dev.o dev_mcast.o dst.o netevent.o \
+ 			neighbour.o rtnetlink.o utils.o link_watch.o filter.o
+ 
+ obj-$(CONFIG_XFRM) += flow.o
++obj-$(CONFIG_ETHTOOL) += ethtool.o
+ obj-$(CONFIG_SYSFS) += net-sysfs.o
+ obj-$(CONFIG_NET_PKTGEN) += pktgen.o
+ obj-$(CONFIG_NETPOLL) += netpoll.o
+Index: linux-2.6.22.1/net/core/dev.c
+===================================================================
+--- linux-2.6.22.1.orig/net/core/dev.c
++++ linux-2.6.22.1/net/core/dev.c
+@@ -2903,6 +2903,7 @@ int dev_ioctl(unsigned int cmd, void __u
+ 			return ret;
+ 
+ 		case SIOCETHTOOL:
++#ifdef CONFIG_ETHTOOL
+ 			dev_load(ifr.ifr_name);
+ 			rtnl_lock();
+ 			ret = dev_ethtool(&ifr);
+@@ -2915,6 +2916,9 @@ int dev_ioctl(unsigned int cmd, void __u
+ 					ret = -EFAULT;
+ 			}
+ 			return ret;
++#else
++			return -EINVAL;
++#endif
+ 
+ 		/*
+ 		 *	These ioctl calls:
Index: packages/kernel/patches/tiny_m57sli/tiny/inflate-readbyte.patch
===================================================================
--- packages/kernel/patches/tiny_m57sli/tiny/inflate-readbyte.patch	(revision 0)
+++ packages/kernel/patches/tiny_m57sli/tiny/inflate-readbyte.patch	(revision 0)
@@ -0,0 +1,827 @@
+inflate: replace get_byte with readbyte
+
+Each inflate user was providing a get_byte macro that hid the details
+of tracking the input buffer. This is now handled with new variables
+in the iostate structure and a callback (most users pass NULL as the
+entire input is provided in a single buffer).
+
+Signed-off-by: Matt Mackall <mpm@selenic.com>
+
+Index: linux-2.6.16.19/arch/alpha/boot/misc.c
+===================================================================
+--- linux-2.6.16.19.orig/arch/alpha/boot/misc.c	2006-06-23 17:06:32.000000000 -0700
++++ linux-2.6.16.19/arch/alpha/boot/misc.c	2006-06-23 17:06:32.000000000 -0700
+@@ -35,16 +35,9 @@
+ #define WSIZE 0x8000		/* Window size must be at least 32k, */
+ 				/* and a power of two */
+ 
+-static uch *inbuf;		/* input buffer */
+ static uch *window;		/* Sliding window buffer */
+-
+-static unsigned insize;		/* valid bytes in inbuf */
+-static unsigned inptr;		/* index of next byte to be processed in inbuf */
+ static unsigned outcnt;		/* bytes in output buffer */
+ 
+-#define get_byte()  (inptr < insize ? inbuf[inptr++] : fill_inbuf())
+-
+-static int  fill_inbuf(void);
+ static void flush_window(void);
+ static void error(char *m);
+ 
+@@ -64,22 +57,6 @@
+ #include "../../../lib/inflate.c"
+ 
+ /* ===========================================================================
+- * Fill the input buffer. This is called only when the buffer is empty
+- * and at least one byte is really needed.
+- */
+-int fill_inbuf(void)
+-{
+-	if (insize != 0)
+-		error("ran out of input data");
+-
+-	inbuf = input_data;
+-	insize = input_data_size;
+-
+-	inptr = 1;
+-	return inbuf[0];
+-}
+-
+-/* ===========================================================================
+  * Write the output window window[0..outcnt-1] and update bytes_out.
+  * (Used for the decompressed data only.)
+  */
+@@ -127,7 +104,7 @@
+ 	window = malloc(WSIZE);
+ 
+ /*	puts("Uncompressing Linux..."); */
+-	gunzip();
++	gunzip(input_data, input_data_size, NULL);
+ /*	puts(" done, booting the kernel.\n"); */
+ 	return output_ptr;
+ }
+Index: linux-2.6.16.19/arch/arm/boot/compressed/misc.c
+===================================================================
+--- linux-2.6.16.19.orig/arch/arm/boot/compressed/misc.c	2006-06-23 17:06:32.000000000 -0700
++++ linux-2.6.16.19/arch/arm/boot/compressed/misc.c	2006-06-23 17:06:32.000000000 -0700
+@@ -53,16 +53,9 @@
+ #define WSIZE 0x8000		/* Window size must be at least 32k, */
+ 				/* and a power of two */
+ 
+-static uch *inbuf;		/* input buffer */
+ static uch window[WSIZE];	/* Sliding window buffer */
+-
+-static unsigned insize;		/* valid bytes in inbuf */
+-static unsigned inptr;		/* index of next byte to be processed in inbuf */
+ static unsigned outcnt;		/* bytes in output buffer */
+ 
+-#define get_byte()  (inptr < insize ? inbuf[inptr++] : fill_inbuf())
+-
+-static int  fill_inbuf(void);
+ static void flush_window(void);
+ static void error(char *m);
+ 
+@@ -88,22 +81,6 @@
+ #endif
+ 
+ /* ===========================================================================
+- * Fill the input buffer. This is called only when the buffer is empty
+- * and at least one byte is really needed.
+- */
+-int fill_inbuf(void)
+-{
+-	if (insize != 0)
+-		error("ran out of input data");
+-
+-	inbuf = input_data;
+-	insize = &input_data_end[0] - &input_data[0];
+-
+-	inptr = 1;
+-	return inbuf[0];
+-}
+-
+-/* ===========================================================================
+  * Write the output window window[0..outcnt-1] and update bytes_out.
+  * (Used for the decompressed data only.)
+  */
+@@ -152,7 +129,7 @@
+ 	arch_decomp_setup();
+ 
+ 	putstr("Uncompressing Linux...");
+-	gunzip();
++	gunzip(input_data, input_data_end - input_data, NULL);
+ 	putstr(" done, booting the kernel.\n");
+ 	return output_ptr;
+ }
+@@ -165,7 +142,7 @@
+ 	output_data = output_buffer;
+ 
+ 	putstr("Uncompressing Linux...");
+-	gunzip();
++	gunzip(input_data, input_data_end - input_data, NULL);
+ 	putstr("done.\n");
+ 	return 0;
+ }
+Index: linux-2.6.16.19/arch/arm26/boot/compressed/misc.c
+===================================================================
+--- linux-2.6.16.19.orig/arch/arm26/boot/compressed/misc.c	2006-06-23 17:06:32.000000000 -0700
++++ linux-2.6.16.19/arch/arm26/boot/compressed/misc.c	2006-06-23 17:06:32.000000000 -0700
+@@ -38,16 +38,9 @@
+ #define WSIZE 0x8000		/* Window size must be at least 32k, */
+ 				/* and a power of two */
+ 
+-static uch *inbuf;		/* input buffer */
+ static uch window[WSIZE];	/* Sliding window buffer */
+-
+-static unsigned insize;		/* valid bytes in inbuf */
+-static unsigned inptr;		/* index of next byte to be processed in inbuf */
+ static unsigned outcnt;		/* bytes in output buffer */
+ 
+-#define get_byte()  (inptr < insize ? inbuf[inptr++] : fill_inbuf())
+-
+-static int  fill_inbuf(void);
+ static void flush_window(void);
+ static void error(char *m);
+ 
+@@ -73,22 +66,6 @@
+ #endif
+ 
+ /* ===========================================================================
+- * Fill the input buffer. This is called only when the buffer is empty
+- * and at least one byte is really needed.
+- */
+-int fill_inbuf(void)
+-{
+-	if (insize != 0)
+-		error("ran out of input data");
+-
+-	inbuf = input_data;
+-	insize = &input_data_end[0] - &input_data[0];
+-
+-	inptr = 1;
+-	return inbuf[0];
+-}
+-
+-/* ===========================================================================
+  * Write the output window window[0..outcnt-1] and update bytes_out.
+  * (Used for the decompressed data only.)
+  */
+@@ -133,7 +110,7 @@
+ 	arch_decomp_setup();
+ 
+ 	puts("Uncompressing Linux...");
+-	gunzip();
++	gunzip(input_data, input_data_end - input_data, NULL);
+ 	puts(" done, booting the kernel.\n");
+ 	return output_ptr;
+ }
+@@ -146,7 +123,7 @@
+ 	output_data = output_buffer;
+ 
+ 	puts("Uncompressing Linux...");
+-	gunzip();
++	gunzip(input_data, input_data_end - input_data, NULL);
+ 	puts("done.\n");
+ 	return 0;
+ }
+Index: linux-2.6.16.19/arch/cris/arch-v10/boot/compressed/head.S
+===================================================================
+--- linux-2.6.16.19.orig/arch/cris/arch-v10/boot/compressed/head.S	2006-06-23 17:05:10.000000000 -0700
++++ linux-2.6.16.19/arch/cris/arch-v10/boot/compressed/head.S	2006-06-23 17:06:32.000000000 -0700
+@@ -104,7 +104,7 @@
+ 	;; when mounting from flash
+ 
+ 	move.d	[_input_data], r9	; flash address of compressed kernel
+-	add.d	[_inptr], r9		; size of compressed kernel
++	add.d	[_compsize], r9		; size of compressed kernel
+ 
+ 	;; Restore command line magic and address.
+ 	move.d  _cmd_line_magic, $r10
+Index: linux-2.6.16.19/arch/cris/arch-v10/boot/compressed/misc.c
+===================================================================
+--- linux-2.6.16.19.orig/arch/cris/arch-v10/boot/compressed/misc.c	2006-06-23 17:06:32.000000000 -0700
++++ linux-2.6.16.19/arch/cris/arch-v10/boot/compressed/misc.c	2006-06-23 17:06:32.000000000 -0700
+@@ -35,19 +35,10 @@
+ #define WSIZE 0x8000		/* Window size must be at least 32k, */
+ 				/* and a power of two */
+ 
+-static uch *inbuf;	     /* input buffer */
+ static uch window[WSIZE];    /* Sliding window buffer */
+-
+-unsigned inptr = 0;	/* index of next byte to be processed in inbuf
+-			 * After decompression it will contain the
+-			 * compressed size, and head.S will read it.
+-			 */
+-
++unsigned compsize; /* compressed size, used by head.S */
+ static unsigned outcnt = 0;  /* bytes in output buffer */
+ 
+-#define get_byte() inbuf[inptr++]
+-
+-static int  fill_inbuf(void);
+ static void flush_window(void);
+ static void error(char *m);
+ 
+@@ -172,6 +163,6 @@
+ 	}
+ 
+ 	puts("Uncompressing Linux...\n");
+-	gunzip();
++	compsize = gunzip(input_data, 0x7fffffff, NULL);
+ 	puts("Done. Now booting the kernel.\n");
+ }
+Index: linux-2.6.16.19/arch/cris/arch-v32/boot/compressed/misc.c
+===================================================================
+--- linux-2.6.16.19.orig/arch/cris/arch-v32/boot/compressed/misc.c	2006-06-23 17:06:32.000000000 -0700
++++ linux-2.6.16.19/arch/cris/arch-v32/boot/compressed/misc.c	2006-06-23 17:06:32.000000000 -0700
+@@ -37,19 +37,9 @@
+ #define WSIZE 0x8000		/* Window size must be at least 32k, */
+ 				/* and a power of two */
+ 
+-static uch *inbuf;	     /* input buffer */
+ static uch window[WSIZE];    /* Sliding window buffer */
+-
+-unsigned inptr = 0;	/* index of next byte to be processed in inbuf
+-			 * After decompression it will contain the
+-			 * compressed size, and head.S will read it.
+-			 */
+-
+ static unsigned outcnt = 0;  /* bytes in output buffer */
+ 
+-#define get_byte() inbuf[inptr++]
+-
+-static int  fill_inbuf(void);
+ static void flush_window(void);
+ static void error(char *m);
+ 
+@@ -217,6 +207,6 @@
+ 	}
+ 
+ 	puts("Uncompressing Linux...\n");
+-	gunzip();
++	gunzip(input_data, 0x7fffffff, NULL);
+ 	puts("Done. Now booting the kernel.\n");
+ }
+Index: linux-2.6.16.19/arch/i386/boot/compressed/misc.c
+===================================================================
+--- linux-2.6.16.19.orig/arch/i386/boot/compressed/misc.c	2006-06-23 17:06:32.000000000 -0700
++++ linux-2.6.16.19/arch/i386/boot/compressed/misc.c	2006-06-23 17:06:32.000000000 -0700
+@@ -30,16 +30,9 @@
+ #define WSIZE 0x8000		/* Window size must be at least 32k, */
+ 				/* and a power of two */
+ 
+-static uch *inbuf;	     /* input buffer */
+ static uch window[WSIZE];    /* Sliding window buffer */
+-
+-static unsigned insize = 0;  /* valid bytes in inbuf */
+-static unsigned inptr = 0;   /* index of next byte to be processed in inbuf */
+ static unsigned outcnt = 0;  /* bytes in output buffer */
+ 
+-#define get_byte()  (inptr < insize ? inbuf[inptr++] : fill_inbuf())
+-
+-static int  fill_inbuf(void);
+ static void flush_window(void);
+ static void error(char *m);
+ 
+@@ -134,22 +127,6 @@
+ }
+ 
+ /* ===========================================================================
+- * Fill the input buffer. This is called only when the buffer is empty
+- * and at least one byte is really needed.
+- */
+-static int fill_inbuf(void)
+-{
+-	if (insize != 0) {
+-		error("ran out of input data");
+-	}
+-
+-	inbuf = input_data;
+-	insize = input_len;
+-	inptr = 1;
+-	return inbuf[0];
+-}
+-
+-/* ===========================================================================
+  * Write the output window window[0..outcnt-1] and update bytes_out.
+  * (Used for the decompressed data only.)
+  */
+@@ -277,7 +254,7 @@
+ 	else setup_output_buffer_if_we_run_high(mv);
+ 
+ 	putstr("Uncompressing Linux... ");
+-	gunzip();
++	gunzip(input_data, input_len, NULL);
+ 	putstr("Ok, booting the kernel.\n");
+ 	if (high_loaded) close_output_buffer_if_we_run_high(mv);
+ 	return high_loaded;
+Index: linux-2.6.16.19/arch/m32r/boot/compressed/misc.c
+===================================================================
+--- linux-2.6.16.19.orig/arch/m32r/boot/compressed/misc.c	2006-06-23 17:06:32.000000000 -0700
++++ linux-2.6.16.19/arch/m32r/boot/compressed/misc.c	2006-06-23 17:06:32.000000000 -0700
+@@ -24,16 +24,9 @@
+ #define WSIZE 0x8000		/* Window size must be at least 32k, */
+ 				/* and a power of two */
+ 
+-static uch *inbuf;	     /* input buffer */
+ static uch window[WSIZE];    /* Sliding window buffer */
+-
+-static unsigned insize = 0;  /* valid bytes in inbuf */
+-static unsigned inptr = 0;   /* index of next byte to be processed in inbuf */
+ static unsigned outcnt = 0;  /* bytes in output buffer */
+ 
+-#define get_byte()  (inptr < insize ? inbuf[inptr++] : fill_inbuf())
+-
+-static int  fill_inbuf(void);
+ static void flush_window(void);
+ static void error(char *m);
+ 
+@@ -54,22 +47,6 @@
+ #include "../../../../lib/inflate.c"
+ 
+ /* ===========================================================================
+- * Fill the input buffer. This is called only when the buffer is empty
+- * and at least one byte is really needed.
+- */
+-static int fill_inbuf(void)
+-{
+-	if (insize != 0) {
+-		error("ran out of input data");
+-	}
+-
+-	inbuf = input_data;
+-	insize = input_len;
+-	inptr = 1;
+-	return inbuf[0];
+-}
+-
+-/* ===========================================================================
+  * Write the output window window[0..outcnt-1] and update bytes_out.
+  * (Used for the decompressed data only.)
+  */
+@@ -110,6 +87,6 @@
+ 	input_len = zimage_len;
+ 
+ 	puts("Uncompressing Linux... ");
+-	gunzip();
++	gunzip(input_data, input_len, NULL);
+ 	puts("Ok, booting the kernel.\n");
+ }
+Index: linux-2.6.16.19/arch/sh/boot/compressed/misc.c
+===================================================================
+--- linux-2.6.16.19.orig/arch/sh/boot/compressed/misc.c	2006-06-23 17:06:32.000000000 -0700
++++ linux-2.6.16.19/arch/sh/boot/compressed/misc.c	2006-06-23 17:06:32.000000000 -0700
+@@ -26,16 +26,9 @@
+ #define WSIZE 0x8000		/* Window size must be at least 32k, */
+ 				/* and a power of two */
+ 
+-static uch *inbuf;	     /* input buffer */
+ static uch window[WSIZE];    /* Sliding window buffer */
+-
+-static unsigned insize = 0;  /* valid bytes in inbuf */
+-static unsigned inptr = 0;   /* index of next byte to be processed in inbuf */
+ static unsigned outcnt = 0;  /* bytes in output buffer */
+ 
+-#define get_byte()  (inptr < insize ? inbuf[inptr++] : fill_inbuf())
+-
+-static int  fill_inbuf(void);
+ static void flush_window(void);
+ static void error(char *m);
+ 
+@@ -82,22 +75,6 @@
+ #endif
+ 
+ /* ===========================================================================
+- * Fill the input buffer. This is called only when the buffer is empty
+- * and at least one byte is really needed.
+- */
+-static int fill_inbuf(void)
+-{
+-	if (insize != 0) {
+-		error("ran out of input data");
+-	}
+-
+-	inbuf = input_data;
+-	insize = input_len;
+-	inptr = 1;
+-	return inbuf[0];
+-}
+-
+-/* ===========================================================================
+  * Write the output window window[0..outcnt-1] and update bytes_out.
+  * (Used for the decompressed data only.)
+  */
+@@ -137,6 +114,6 @@
+ 	free_mem_end_ptr = free_mem_ptr + HEAP_SIZE;
+ 
+ 	puts("Uncompressing Linux... ");
+-	gunzip();
++	gunzip(input_data, input_len, NULL);
+ 	puts("Ok, booting the kernel.\n");
+ }
+Index: linux-2.6.16.19/arch/sh64/boot/compressed/misc.c
+===================================================================
+--- linux-2.6.16.19.orig/arch/sh64/boot/compressed/misc.c	2006-06-23 17:06:32.000000000 -0700
++++ linux-2.6.16.19/arch/sh64/boot/compressed/misc.c	2006-06-23 17:06:32.000000000 -0700
+@@ -26,16 +26,9 @@
+ #define WSIZE 0x8000		/* Window size must be at least 32k, */
+ 				/* and a power of two */
+ 
+-static uch *inbuf;		/* input buffer */
+ static uch window[WSIZE];	/* Sliding window buffer */
+-
+-static unsigned insize = 0;	/* valid bytes in inbuf */
+-static unsigned inptr = 0;	/* index of next byte to be processed in inbuf */
+ static unsigned outcnt = 0;	/* bytes in output buffer */
+ 
+-#define get_byte()  (inptr < insize ? inbuf[inptr++] : fill_inbuf())
+-
+-static int fill_inbuf(void);
+ static void flush_window(void);
+ static void error(char *m);
+ 
+@@ -62,22 +55,6 @@
+ }
+ 
+ /* ===========================================================================
+- * Fill the input buffer. This is called only when the buffer is empty
+- * and at least one byte is really needed.
+- */
+-static int fill_inbuf(void)
+-{
+-	if (insize != 0) {
+-		error("ran out of input data\n");
+-	}
+-
+-	inbuf = input_data;
+-	insize = input_len;
+-	inptr = 1;
+-	return inbuf[0];
+-}
+-
+-/* ===========================================================================
+  * Write the output window window[0..outcnt-1] and update bytes_out.
+  * (Used for the decompressed data only.)
+  */
+@@ -118,7 +95,7 @@
+ 
+ 	puts("Uncompressing Linux... ");
+ 	cache_control(CACHE_ENABLE);
+-	gunzip();
++	gunzip(input_data, input_len, NULL);
+ 	puts("\n");
+ 
+ #if 0
+Index: linux-2.6.16.19/arch/x86_64/boot/compressed/misc.c
+===================================================================
+--- linux-2.6.16.19.orig/arch/x86_64/boot/compressed/misc.c	2006-06-23 17:06:32.000000000 -0700
++++ linux-2.6.16.19/arch/x86_64/boot/compressed/misc.c	2006-06-23 17:06:32.000000000 -0700
+@@ -24,16 +24,9 @@
+ #define WSIZE 0x8000		/* Window size must be at least 32k, */
+ 				/* and a power of two */
+ 
+-static uch *inbuf;	     /* input buffer */
+ static uch window[WSIZE];    /* Sliding window buffer */
+-
+-static unsigned insize = 0;  /* valid bytes in inbuf */
+-static unsigned inptr = 0;   /* index of next byte to be processed in inbuf */
+ static unsigned outcnt = 0;  /* bytes in output buffer */
+ 
+-#define get_byte()  (inptr < insize ? inbuf[inptr++] : fill_inbuf())
+-
+-static int  fill_inbuf(void);
+ static void flush_window(void);
+ static void error(char *m);
+ 
+@@ -124,22 +117,6 @@
+ }
+ 
+ /* ===========================================================================
+- * Fill the input buffer. This is called only when the buffer is empty
+- * and at least one byte is really needed.
+- */
+-static int fill_inbuf(void)
+-{
+-	if (insize != 0) {
+-		error("ran out of input data");
+-	}
+-
+-	inbuf = input_data;
+-	insize = input_len;
+-	inptr = 1;
+-	return inbuf[0];
+-}
+-
+-/* ===========================================================================
+  * Write the output window window[0..outcnt-1] and update bytes_out.
+  * (Used for the decompressed data only.)
+  */
+@@ -256,7 +233,7 @@
+ 	else setup_output_buffer_if_we_run_high(mv);
+ 
+ 	putstr(".\nDecompressing Linux...");
+-	gunzip();
++	gunzip(input_data, input_len, NULL);
+ 	putstr("done.\nBooting the kernel.\n");
+ 	if (high_loaded) close_output_buffer_if_we_run_high(mv);
+ 	return high_loaded;
+Index: linux-2.6.16.19/init/do_mounts_rd.c
+===================================================================
+--- linux-2.6.16.19.orig/init/do_mounts_rd.c	2006-06-23 17:06:32.000000000 -0700
++++ linux-2.6.16.19/init/do_mounts_rd.c	2006-06-23 17:06:32.000000000 -0700
+@@ -283,20 +283,14 @@
+ 
+ static uch *inbuf;
+ static uch *window;
+-
+-static unsigned insize;  /* valid bytes in inbuf */
+-static unsigned inptr;   /* index of next byte to be processed in inbuf */
+ static unsigned outcnt;  /* bytes in output buffer */
+ static int exit_code;
+ static int unzip_error;
+ static long bytes_out;
+ static int crd_infd, crd_outfd;
+ 
+-#define get_byte()  (inptr < insize ? inbuf[inptr++] : fill_inbuf())
+-
+ #define INIT __init
+ 
+-static int  __init fill_inbuf(void);
+ static void __init flush_window(void);
+ static void __init error(char *m);
+ 
+@@ -304,24 +298,14 @@
+ 
+ #include "../lib/inflate.c"
+ 
+-/* ===========================================================================
++/*
+  * Fill the input buffer. This is called only when the buffer is empty
+  * and at least one byte is really needed.
+- * Returning -1 does not guarantee that gunzip() will ever return.
+  */
+-static int __init fill_inbuf(void)
++static void __init fill_inbuf(u8 *buf, int len)
+ {
+-	if (exit_code) return -1;
+-	
+-	insize = sys_read(crd_infd, inbuf, INBUFSIZ);
+-	if (insize == 0) {
++	if (!sys_read(crd_infd, buf, len))
+ 		error("RAMDISK: ran out of compressed data");
+-		return -1;
+-	}
+-
+-	inptr = 1;
+-
+-	return inbuf[0];
+ }
+ 
+ /* ===========================================================================
+@@ -358,8 +342,6 @@
+ {
+ 	int result;
+ 
+-	insize = 0;		/* valid bytes in inbuf */
+-	inptr = 0;		/* index of next byte to be processed in inbuf */
+ 	outcnt = 0;		/* bytes in output buffer */
+ 	exit_code = 0;
+ 	bytes_out = 0;
+@@ -377,7 +359,7 @@
+ 		kfree(inbuf);
+ 		return -1;
+ 	}
+-	result = gunzip();
++	result = gunzip(inbuf, INBUFSIZ, fill_inbuf);
+ 	if (unzip_error)
+ 		result = 1;
+ 	kfree(inbuf);
+Index: linux-2.6.16.19/init/initramfs.c
+===================================================================
+--- linux-2.6.16.19.orig/init/initramfs.c	2006-06-23 17:06:32.000000000 -0700
++++ linux-2.6.16.19/init/initramfs.c	2006-06-23 17:06:32.000000000 -0700
+@@ -340,16 +340,11 @@
+ #define WSIZE 0x8000    /* window size--must be a power of two, and */
+ 			/*  at least 32K for zip's deflate method */
+ 
+-static uch *inbuf;
+ static uch *window;
+ 
+-static unsigned insize;  /* valid bytes in inbuf */
+-static unsigned inptr;   /* index of next byte to be processed in inbuf */
+ static unsigned outcnt;  /* bytes in output buffer */
+ static long bytes_out;
+ 
+-#define get_byte()  (inptr < insize ? inbuf[inptr++] : -1)
+-
+ #define INIT __init
+ 
+ static void __init flush_window(void);
+@@ -379,7 +374,7 @@
+ 
+ static char * __init unpack_to_rootfs(char *buf, unsigned len, int check_only)
+ {
+-	int written;
++	int written, cnt;
+ 	dry_run = check_only;
+ 	header_buf = kmalloc(110, GFP_KERNEL);
+ 	symlink_buf = kmalloc(PATH_MAX + N_ALIGN(PATH_MAX) + 1, GFP_KERNEL);
+@@ -406,17 +401,14 @@
+ 			continue;
+ 		}
+ 		this_header = 0;
+-		insize = len;
+-		inbuf = buf;
+-		inptr = 0;
+ 		outcnt = 0;		/* bytes in output buffer */
+ 		bytes_out = 0;
+-		gunzip();
++		cnt = gunzip(buf, len, NULL);
+ 		if (state != Reset)
+ 			error("junk in gzipped archive");
+-		this_header = saved_offset + inptr;
+-		buf += inptr;
+-		len -= inptr;
++		this_header = saved_offset + cnt;
++		buf += cnt;
++		len -= cnt;
+ 	}
+ 	kfree(window);
+ 	kfree(name_buf);
+Index: linux-2.6.16.19/lib/inflate.c
+===================================================================
+--- linux-2.6.16.19.orig/lib/inflate.c	2006-06-23 17:06:32.000000000 -0700
++++ linux-2.6.16.19/lib/inflate.c	2006-06-23 17:06:32.000000000 -0700
+@@ -169,8 +169,9 @@
+ };
+ 
+ struct iostate {
+-	u8 *window;
+-	int opos, osize, bits;
++	u8 *window, *ibuf;
++	int ipos, isize, itotal, opos, osize, bits;
++	void (*fill)(u8 *ibuf, int len);
+ 	u32 buf, crc;
+ };
+ 
+@@ -297,10 +298,21 @@
+    the stream.
+  */
+ 
++static u32 readbyte(struct iostate *io)
++{
++	if (io->ipos == io->isize) {
++		io->fill(io->ibuf, io->isize);
++		io->itotal += io->ipos;
++		io->ipos = 0;
++	}
++
++	return io->ibuf[io->ipos++];
++}
++
+ static inline u32 readbits(struct iostate *io, int n)
+ {
+ 	for( ; io->bits < n; io->bits += 8)
+-		io->buf |= (u32)get_byte() << io->bits;
++		io->buf |= readbyte(io) << io->bits;
+ 	return io->buf & ((1 << n) - 1);
+ }
+ 
+@@ -321,7 +333,7 @@
+ {
+ 	while (io->bits >= 8) {
+ 		io->bits -= 8;
+-		inptr--;
++		io->ipos--;
+ 	}
+ }
+ 
+@@ -713,7 +725,7 @@
+ 
+ 	/* read and output the compressed data */
+ 	while (n--)
+-		put_byte(io, get_byte());
++		put_byte(io, readbyte(io));
+ 
+ 	DEBG(">");
+ 	return 0;
+@@ -977,9 +989,12 @@
+ #define RESERVED     0xC0 /* bit 6,7:   reserved */
+ 
+ /*
+- * Do the uncompression!
++ * gunzip - do the uncompression!
++ * @ibuf: input character pool
++ * @isize: size of pool
++ * @fill: function to fill the input pool
+  */
+-static int INIT gunzip(void)
++static int INIT gunzip(u8 *ibuf, int isize, void (*fill)(u8 *buf, int size))
+ {
+ 	u8 flags;
+ 	unsigned char magic[2];	/* magic header */
+@@ -991,14 +1006,17 @@
+ 
+ 	io.window = window;
+ 	io.osize = WSIZE;
+-	io.opos = io.bits = io.buf = 0;
++	io.opos = io.bits = io.buf = io.ipos = io.itotal = 0;
+ 	io.crc = 0xffffffffUL;
++	io.isize = isize;
++	io.ibuf = ibuf;
++	io.fill = fill;
+ 
+ 	makecrc(); /* initialize the CRC table */
+ 
+-	magic[0] = get_byte();
+-	magic[1] = get_byte();
+-	method = get_byte();
++	magic[0] = readbyte(&io);
++	magic[1] = readbyte(&io);
++	method = readbyte(&io);
+ 
+ 	if (magic[0] != 037 || ((magic[1] != 0213) && (magic[1] != 0236))) {
+ 		error("bad gzip magic numbers");
+@@ -1011,7 +1029,7 @@
+ 		return -1;
+ 	}
+ 
+-	flags = (u8)get_byte();
++	flags = readbyte(&io);
+ 	if (flags & ENCRYPTED) {
+ 		error("Input is encrypted");
+ 		return -1;
+@@ -1024,29 +1042,29 @@
+ 		error("Input has invalid flags");
+ 		return -1;
+ 	}
+-	get_byte();		/* Get timestamp */
+-	get_byte();
+-	get_byte();
+-	get_byte();
++	readbyte(&io); /* skip timestamp */
++	readbyte(&io);
++	readbyte(&io);
++	readbyte(&io);
+ 
+-	get_byte();	/* Ignore extra flags for the moment */
+-	get_byte();	/* Ignore OS type for the moment */
++	readbyte(&io); /* Ignore extra flags */
++	readbyte(&io); /* Ignore OS type */
+ 
+ 	if (flags & EXTRA_FIELD) {
+-		unsigned len = (unsigned)get_byte();
+-		len |= ((unsigned)get_byte()) << 8;
++		unsigned len = readbyte(&io);
++		len |= readbyte(&io) << 8;
+ 		while (len--)
+-			get_byte();
++			readbyte(&io);
+ 	}
+ 
+ 	/* Discard original file name if it was truncated */
+ 	if (flags & ORIG_NAME)
+-		while (get_byte())
++		while (readbyte(&io))
+ 			;
+ 
+ 	/* Discard file comment if any */
+ 	if (flags & COMMENT)
+-		while (get_byte())
++		while (readbyte(&io))
+ 			;
+ 
+ 	/* Decompress */
+@@ -1075,15 +1093,15 @@
+ 	/* Get the crc and original length
+ 	 * uncompressed input size modulo 2^32
+ 	 */
+-	orig_crc = (u32)get_byte();
+-	orig_crc |= (u32)get_byte() << 8;
+-	orig_crc |= (u32)get_byte() << 16;
+-	orig_crc |= (u32)get_byte() << 24;
+-
+-	orig_len = (u32)get_byte();
+-	orig_len |= (u32)get_byte() << 8;
+-	orig_len |= (u32)get_byte() << 16;
+-	orig_len |= (u32)get_byte() << 24;
++	orig_crc = readbyte(&io);
++	orig_crc |= readbyte(&io) << 8;
++	orig_crc |= readbyte(&io) << 16;
++	orig_crc |= readbyte(&io) << 24;
++
++	orig_len = readbyte(&io);
++	orig_len |= readbyte(&io) << 8;
++	orig_len |= readbyte(&io) << 16;
++	orig_len |= readbyte(&io) << 24;
+ 
+ 	/* Validate decompression */
+ 	if (orig_crc != ~io.crc) {
+@@ -1094,5 +1112,6 @@
+ 		error("length error");
+ 		return -1;
+ 	}
+-	return 0;
++
++	return io.itotal + io.ipos;
+ }
Index: packages/kernel/patches/tiny_m57sli/tiny/inflate-legacy.patch
===================================================================
--- packages/kernel/patches/tiny_m57sli/tiny/inflate-legacy.patch	(revision 0)
+++ packages/kernel/patches/tiny_m57sli/tiny/inflate-legacy.patch	(revision 0)
@@ -0,0 +1,734 @@
+inflate  kill legacy bits
+
+Kill RCSID
+Kill old includes and defines
+Kill screwy types
+Kill unused memory usage tracking
+Kill 'register' usage
+Kill unused tracing calls
+
+ tiny-mpm/lib/inflate.c |  276 ++++++++++++++++++++-----------------------------
+ 1 files changed, 116 insertions(+), 160 deletions(-)
+
+Index: linux-2.6.16.19/lib/inflate.c
+===================================================================
+--- linux-2.6.16.19.orig/lib/inflate.c	2006-06-23 17:06:29.000000000 -0700
++++ linux-2.6.16.19/lib/inflate.c	2006-06-23 17:06:29.000000000 -0700
+@@ -103,26 +103,11 @@
+  */
+ #include <linux/compiler.h>
+ 
+-#ifdef RCSID
+-static char rcsid[] = "#Id: inflate.c,v 0.14 1993/06/10 13:27:04 jloup Exp #";
+-#endif
+-
+-#ifndef STATIC
+-
+-#if defined(STDC_HEADERS) || defined(HAVE_STDLIB_H)
+-#  include <sys/types.h>
+-#  include <stdlib.h>
+-#endif
+-
+-#include "gzip.h"
+-#define STATIC
+-#endif /* !STATIC */
+-
+ #ifndef INIT
+ #define INIT
+ #endif
+ 
+-#define slide window
++#include <asm/types.h>
+ 
+ /* Huffman code lookup table entry--this entry is four bytes for machines
+    that have 16-bit pointers (e.g. PC's in the small or medium model).
+@@ -132,36 +117,35 @@
+    an unused code.  If a code with e == 99 is looked up, this implies an
+    error in the data. */
+ struct huft {
+-	uch e;			/* number of extra bits or operation */
+-	uch b;			/* number of bits in this code or subcode */
++	u8 e;			/* number of extra bits or operation */
++	u8 b;			/* number of bits in this code or subcode */
+ 	union {
+-		ush n;		/* literal, length base, or distance base */
++		u16 n;		/* literal, length base, or distance base */
+ 		struct huft *t;	/* pointer to next level of table */
+ 	} v;
+ };
+ 
+ /* Function prototypes */
+-STATIC int INIT huft_build OF((unsigned *, unsigned, unsigned,
+-		const ush *, const ush *, struct huft **, int *));
+-STATIC int INIT huft_free OF((struct huft *));
+-STATIC int INIT inflate_codes OF((struct huft *, struct huft *, int, int));
+-STATIC int INIT inflate_stored OF((void));
+-STATIC int INIT inflate_fixed OF((void));
+-STATIC int INIT inflate_dynamic OF((void));
+-STATIC int INIT inflate_block OF((int *));
+-STATIC int INIT inflate OF((void));
++static int INIT huft_build(unsigned *, unsigned, unsigned,
++			  const u16 *, const u16 *, struct huft **, int *);
++static int INIT huft_free(struct huft *);
++static int INIT inflate_codes(struct huft *, struct huft *, int, int);
++static int INIT inflate_stored(void);
++static int INIT inflate_fixed(void);
++static int INIT inflate_dynamic(void);
++static int INIT inflate_block(int *);
++static int INIT inflate(void);
+ 
+ /* The inflate algorithm uses a sliding 32 K byte window on the uncompressed
+    stream to find repeated byte strings.  This is implemented here as a
+    circular buffer.  The index is updated simply by incrementing and then
+    ANDing with 0x7fff (32K-1). */
+ /* It is left to other modules to supply the 32 K area.  It is assumed
+-   to be usable as if it were declared "uch slide[32768];" or as just
+-   "uch *slide;" and then malloc'ed in the latter case.  The definition
++   to be usable as if it were declared "u8 window[32768];" or as just
++   "u8 *window;" and then malloc'ed in the latter case.  The definition
+    must be in unzip.h, included above. */
+-/* unsigned wp;             current position in slide */
+-#define wp outcnt
+-#define flush_output(w) (wp=(w),flush_window())
++
++#define flush_output(w) (outcnt=(w),flush_window())
+ 
+ /* Tables for deflate from PKZIP's appnote.txt. */
+ 
+@@ -171,7 +155,7 @@
+ };
+ 
+ /* Copy lengths for literal codes 257..285 */
+-static const ush cplens[] = {
++static const u16 cplens[] = {
+ 	3, 4, 5, 6, 7, 8, 9, 10, 11, 13, 15, 17, 19, 23, 27, 31,
+ 	35, 43, 51, 59, 67, 83, 99, 115, 131, 163, 195, 227, 258, 0, 0
+ };
+@@ -180,20 +164,20 @@
+  * note: see note #13 above about the 258 in this list.
+  * 99==invalid
+  */
+-static const ush cplext[] = {
++static const u16 cplext[] = {
+ 	0, 0, 0, 0, 0, 0, 0, 0, 1, 1, 1, 1, 2, 2, 2, 2,
+ 	3, 3, 3, 3, 4, 4, 4, 4, 5, 5, 5, 5, 0, 99, 99
+ };
+ 
+ /* Copy offsets for distance codes 0..29 */
+-static const ush cpdist[] = {
++static const u16 cpdist[] = {
+ 	1, 2, 3, 4, 5, 7, 9, 13, 17, 25, 33, 49, 65, 97, 129, 193,
+ 	257, 385, 513, 769, 1025, 1537, 2049, 3073, 4097, 6145,
+ 	8193, 12289, 16385, 24577
+ };
+ 
+ /* Extra bits for distance codes */
+-static const ush cpdext[] = {
++static const u16 cpdext[] = {
+ 	0, 0, 0, 0, 1, 1, 2, 2, 3, 3, 4, 4, 5, 5, 6, 6,
+ 	7, 7, 8, 8, 9, 9, 10, 10, 11, 11,
+ 	12, 12, 13, 13
+@@ -207,10 +191,10 @@
+         DUMPBITS(j)
+ 
+    where NEEDBITS makes sure that b has at least j bits in it, and
+-   DUMPBITS removes the bits from b.  The macros use the variable k
+-   for the number of bits in b.  Normally, b and k are register
+-   variables for speed, and are initialized at the beginning of a
+-   routine that uses these macros from a global bit buffer and count.
++   DUMPBITS removes the bits from b. The macros use the variable k for
++   the number of bits in b. Normally, b and k are initialized at the
++   beginning of a routine that uses these macros from a global bit
++   buffer and count.
+ 
+    If we assume that EOB will be the longest code, then we will never
+    ask for bits with NEEDBITS that are beyond the end of the stream.
+@@ -229,17 +213,17 @@
+    the stream.
+  */
+ 
+-STATIC ulg bb;			/* bit buffer */
+-STATIC unsigned bk;		/* bits in bit buffer */
++static u32 bb;			/* bit buffer */
++static unsigned bk;		/* bits in bit buffer */
+ 
+-STATIC const ush mask_bits[] = {
++static const u16 mask_bits[] = {
+ 	0x0000,
+ 	0x0001, 0x0003, 0x0007, 0x000f, 0x001f, 0x003f, 0x007f, 0x00ff,
+ 	0x01ff, 0x03ff, 0x07ff, 0x0fff, 0x1fff, 0x3fff, 0x7fff, 0xffff
+ };
+ 
+-#define NEXTBYTE()  ({ int v = get_byte(); if (v < 0) goto underrun; (uch)v; })
+-#define NEEDBITS(n) do {while(k<(n)){b|=((ulg)NEXTBYTE())<<k;k+=8;}} while(0)
++#define NEXTBYTE()  ({ int v = get_byte(); if (v < 0) goto underrun; (u8)v; })
++#define NEEDBITS(n) do {while(k<(n)){b|=((u32)NEXTBYTE())<<k;k+=8;}} while(0)
+ #define DUMPBITS(n) do {b>>=(n);k-=(n);} while(0)
+ 
+ /*
+@@ -274,15 +258,13 @@
+    possibly even between compilers.  Your mileage may vary.
+  */
+ 
+-STATIC const int lbits = 9;	/* bits in base literal/length lookup table */
+-STATIC const int dbits = 6;	/* bits in base distance lookup table */
++static const int lbits = 9;	/* bits in base literal/length lookup table */
++static const int dbits = 6;	/* bits in base distance lookup table */
+ 
+-/* If BMAX needs to be larger than 16, then h and x[] should be ulg. */
++/* If BMAX needs to be larger than 16, then h and x[] should be u32. */
+ #define BMAX 16		/* maximum bit length of any code (16 for explode) */
+ #define N_MAX 288	/* maximum number of codes in any set */
+ 
+-STATIC unsigned hufts;		/* track memory usage */
+-
+ /*
+  * huft-build - build a huffman decoding table
+  * @b: code lengths in bits (all assumed <= BMAX)
+@@ -300,24 +282,24 @@
+  * an oversubscribed set of lengths), and three if not enough
+  * memory.
+  */
+-STATIC int INIT huft_build(unsigned *b, unsigned n, unsigned s, const ush * d,
+-		      const ush * e, struct huft **t, int *m)
++static int INIT huft_build(unsigned *b, unsigned n, unsigned s, const u16 * d,
++		      const u16 * e, struct huft **t, int *m)
+ {
+ 	unsigned a;		/* counter for codes of length k */
+ 	unsigned c[BMAX + 1];	/* bit length count table */
+ 	unsigned f;		/* i repeats in table every f entries */
+ 	int g;			/* maximum code length */
+ 	int h;			/* table level */
+-	register unsigned i;	/* counter, current code */
+-	register unsigned j;	/* counter */
+-	register int k;		/* number of bits in current code */
++	unsigned i;	/* counter, current code */
++	unsigned j;	/* counter */
++	int k;		/* number of bits in current code */
+ 	int l;			/* bits per table (returned in m) */
+-	register unsigned *p;	/* pointer into c[], b[], or v[] */
+-	register struct huft *q;	/* points to current table */
++	unsigned *p;	/* pointer into c[], b[], or v[] */
++	struct huft *q;	/* points to current table */
+ 	struct huft r;		/* table entry for structure assignment */
+ 	struct huft *u[BMAX];	/* table stack */
+ 	unsigned v[N_MAX];	/* values in order of bit length */
+-	register int w;		/* bits before this table == (l * h) */
++	int w;		/* bits before this table == (l * h) */
+ 	unsigned x[BMAX + 1];	/* bit offsets, then code stack */
+ 	unsigned *xp;		/* pointer into x */
+ 	int y;			/* number of dummy codes added */
+@@ -330,14 +312,12 @@
+ 	p = b;
+ 	i = n;
+ 	do {
+-		Tracecv(*p, (stderr, (n - i >= ' ' && n - i <= '~' ?
+-				      "%c %d\n" : "0x%x %d\n"), n - i, *p));
+ 		c[*p]++;	/* assume all entries <= BMAX */
+ 		p++;
+ 	} while (--i);
+ 
+ 	if (c[0] == n) {	/* null input--all zero length codes */
+-		*t = (struct huft *)NULL;
++		*t = 0;
+ 		*m = 0;
+ 		return 2;
+ 	}
+@@ -389,7 +369,7 @@
+ 	p = b;
+ 	i = 0;
+ 	do {
+-		if ((j = *p++) != 0)
++		if ((j = *p++))
+ 			v[x[j]++] = i;
+ 	} while (++i < n);
+ 
+@@ -402,8 +382,8 @@
+ 	p = v; /* grab values in bit order */
+ 	h = -1; /* no tables yet--level -1 */
+ 	w = -l;	/* bits decoded == (l * h) */
+-	u[0] = (struct huft *)NULL; /* just to keep compilers happy */
+-	q = (struct huft *)NULL; /* ditto */
++	u[0] = NULL; /* just to keep compilers happy */
++	q = NULL; /* ditto */
+ 	z = 0; /* ditto */
+ 	DEBG("h6a ");
+ 
+@@ -448,17 +428,15 @@
+ 				z = 1 << j;
+ 
+ 				/* allocate and link in new table */
+-				if ((q = (struct huft *)malloc(
+-					     (z + 1) * sizeof(struct huft)))
+-				    == (struct huft *)NULL) {
++				if (!(q = (struct huft *)malloc(
++					     (z + 1) * sizeof(struct huft)))) {
+ 					if (h)
+ 						huft_free(u[0]);
+ 					return 3;	/* not enough memory */
+ 				}
+ 				DEBG1("4 ");
+-				hufts += z + 1;	/* track memory usage */
+ 				*t = q + 1; /* link to list for huft_free */
+-				*(t = &(q->v.t)) = (struct huft *)NULL;
++				*(t = &(q->v.t)) = 0;
+ 				u[h] = ++q;	/* table starts after link */
+ 
+ 				DEBG1("5 ");
+@@ -467,9 +445,9 @@
+ 					/* save pattern for backing up */
+ 					x[h] = i;
+ 					/* bits to dump before this table */
+-					r.b = (uch)l;
++					r.b = (u8)l;
+ 					/* bits in this table */
+-					r.e = (uch)(16 + j);
++					r.e = (u8)(16 + j);
+ 					/* pointer to this table */
+ 					r.v.t = q;
+ 					/* (get around Turbo C bug) */
+@@ -482,19 +460,19 @@
+ 			DEBG("h6c ");
+ 
+ 			/* set up table entry in r */
+-			r.b = (uch) (k - w);
++			r.b = (u8) (k - w);
+ 			if (p >= v + n)
+ 				r.e = 99; /* out of values--invalid code */
+ 			else if (*p < s) {
+ 				/* 256 is end-of-block code */
+-				r.e = (uch) (*p < 256 ? 16 : 15);
++				r.e = (u8)(*p < 256 ? 16 : 15);
+ 				/* simple code is just the value */
+-				r.v.n = (ush) (*p);
++				r.v.n = (u16)(*p);
+ 				/* one compiler does not like *p++ */
+ 				p++;
+ 			} else {
+ 				/* non-simple--look up in lists */
+-				r.e = (uch)e[*p - s];
++				r.e = (u8)e[*p - s];
+ 				r.v.n = d[*p++ - s];
+ 			}
+ 			DEBG("h6d ");
+@@ -522,7 +500,7 @@
+ 	DEBG("huft7 ");
+ 
+ 	/* Return true (1) if we were given an incomplete table */
+-	return y != 0 && g != 1;
++	return y && g != 1;
+ }
+ 
+ /*
+@@ -535,11 +513,11 @@
+  */
+ STATIC int INIT huft_free(struct huft *t)
+ {
+-	register struct huft *p, *q;
++	struct huft *p, *q;
+ 
+ 	/* Go through list, freeing from the malloced (t[-1]) address. */
+ 	p = t;
+-	while (p != (struct huft *)NULL) {
++	while (p) {
+ 		q = (--p)->v.t;
+ 		free((char *)p);
+ 		p = q;
+@@ -557,20 +535,20 @@
+  * inflate (decompress) the codes in a deflated (compressed) block.
+  * Return an error code or zero if it all goes ok.
+  */
+-STATIC int inflate_codes(struct huft *tl, struct huft *td, int bl, int bd)
++static int inflate_codes(struct huft *tl, struct huft *td, int bl, int bd)
+ {
+-	register unsigned e;	/* table entry flag/number of extra bits */
++	unsigned e;	/* table entry flag/number of extra bits */
+ 	unsigned n, d;		/* length and index for copy */
+ 	unsigned w;		/* current window position */
+ 	struct huft *t;		/* pointer to table entry */
+ 	unsigned ml, md;	/* masks for bl and bd bits */
+-	register ulg b;		/* bit buffer */
+-	register unsigned k;	/* number of bits in bit buffer */
++	u32 b;		/* bit buffer */
++	unsigned k;	/* number of bits in bit buffer */
+ 
+ 	/* make local copies of globals */
+ 	b = bb;			/* initialize bit buffer */
+ 	k = bk;
+-	w = wp;			/* initialize window position */
++	w = outcnt;			/* initialize window position */
+ 
+ 	/* inflate the coded data */
+ 	ml = mask_bits[bl];	/* precompute masks for speed */
+@@ -588,8 +566,7 @@
+ 						     mask_bits[e]))->e) > 16);
+ 		DUMPBITS(t->b);
+ 		if (e == 16) {	/* then it's a literal */
+-			slide[w++] = (uch)t->v.n;
+-			Tracevv((stderr, "%c", slide[w - 1]));
++			window[w++] = (u8)t->v.n;
+ 			if (w == WSIZE) {
+ 				flush_output(w);
+ 				w = 0;
+@@ -619,7 +596,6 @@
+ 			NEEDBITS(e);
+ 			d = w - t->v.n - ((unsigned)b & mask_bits[e]);
+ 			DUMPBITS(e)
+-			    Tracevv((stderr, "\\[%d,%d]", w - d, n));
+ 
+ 			/* do the copy */
+ 			do {
+@@ -628,16 +604,14 @@
+ #if !defined(NOMEMCPY) && !defined(DEBUG)
+ 				/* (this test assumes unsigned comparison) */
+ 				if (w - d >= e) {
+-					memcpy(slide + w, slide + d, e);
++					memcpy(window + w, window + d, e);
+ 					w += e;
+ 					d += e;
+ 				} else
+ #endif				/* !NOMEMCPY */
+ 					/* avoid memcpy() overlap */
+ 					do {
+-						slide[w++] = slide[d++];
+-						Tracevv((stderr, "%c",
+-							 slide[w - 1]));
++						window[w++] = window[d++];
+ 					} while (--e);
+ 				if (w == WSIZE) {
+ 					flush_output(w);
+@@ -648,7 +622,7 @@
+ 	}
+ 
+ 	/* restore the globals from the locals */
+-	wp = w;			/* restore global window pointer */
++	outcnt = w;			/* restore global window pointer */
+ 	bb = b;			/* restore global bit buffer */
+ 	bk = k;
+ 
+@@ -660,19 +634,19 @@
+ }
+ 
+ /* inflate_stored - "decompress" an inflated type 0 (stored) block. */
+-STATIC int INIT inflate_stored(void)
++static int INIT inflate_stored(void)
+ {
+ 	unsigned n;		/* number of bytes in block */
+ 	unsigned w;		/* current window position */
+-	register ulg b;		/* bit buffer */
+-	register unsigned k;	/* number of bits in bit buffer */
++	u32 b;		/* bit buffer */
++	unsigned k;	/* number of bits in bit buffer */
+ 
+ 	DEBG("<stor");
+ 
+ 	/* make local copies of globals */
+ 	b = bb;			/* initialize bit buffer */
+ 	k = bk;
+-	w = wp;			/* initialize window position */
++	w = outcnt;			/* initialize window position */
+ 
+ 	/* go to byte boundary */
+ 	n = k & 7;
+@@ -690,7 +664,7 @@
+ 	/* read and output the compressed data */
+ 	while (n--) {
+ 		NEEDBITS(8);
+-		slide[w++] = (uch)b;
++		window[w++] = (u8)b;
+ 		if (w == WSIZE) {
+ 			flush_output(w);
+ 			w = 0;
+@@ -699,7 +673,7 @@
+ 	}
+ 
+ 	/* restore the globals from the locals */
+-	wp = w;			/* restore global window pointer */
++	outcnt = w;			/* restore global window pointer */
+ 	bb = b;			/* restore global bit buffer */
+ 	bk = k;
+ 
+@@ -719,14 +693,14 @@
+  *
+  * We use `noinline' here to prevent gcc-3.5 from using too much stack space
+  */
+-STATIC int noinline INIT inflate_fixed(void)
++static int noinline INIT inflate_fixed(void)
+ {
+ 	int i;			/* temporary variable */
+ 	struct huft *tl;	/* literal/length code table */
+ 	struct huft *td;	/* distance code table */
+ 	int bl;			/* lookup bits for tl */
+ 	int bd;			/* lookup bits for td */
+-	unsigned l[288];	/* length list for huft_build */
++	unsigned l[N_MAX];	/* length list for huft_build */
+ 
+ 	DEBG("<fix");
+ 
+@@ -737,10 +711,10 @@
+ 		l[i] = 9;
+ 	for (; i < 280; i++)
+ 		l[i] = 7;
+-	for (; i < 288; i++)	/* make a complete, but wrong code set */
++	for (; i < N_MAX; i++)	/* make a complete, but wrong code set */
+ 		l[i] = 8;
+ 	bl = 7;
+-	if ((i = huft_build(l, 288, 257, cplens, cplext, &tl, &bl)) != 0)
++	if ((i = huft_build(l, N_MAX, 257, cplens, cplext, &tl, &bl)))
+ 		return i;
+ 
+ 	/* set up distance table */
+@@ -769,7 +743,7 @@
+  *
+  * We use `noinline' here to prevent gcc-3.5 from using too much stack space
+  */
+-STATIC int noinline INIT inflate_dynamic(void)
++static int noinline INIT inflate_dynamic(void)
+ {
+ 	int i;			/* temporary variables */
+ 	unsigned j;
+@@ -783,13 +757,9 @@
+ 	unsigned nb;		/* number of bit length codes */
+ 	unsigned nl;		/* number of literal/length codes */
+ 	unsigned nd;		/* number of distance codes */
+-#ifdef PKZIP_BUG_WORKAROUND
+-	unsigned ll[288 + 32];	/* literal/length and distance code lengths */
+-#else
+ 	unsigned ll[286 + 30];	/* literal/length and distance code lengths */
+-#endif
+-	register ulg b;		/* bit buffer */
+-	register unsigned k;	/* number of bits in bit buffer */
++	u32 b;		/* bit buffer */
++	unsigned k;	/* number of bits in bit buffer */
+ 
+ 	DEBG("<dyn");
+ 
+@@ -807,11 +777,7 @@
+ 	NEEDBITS(4);
+ 	nb = 4 + ((unsigned)b & 0xf);	/* number of bit length codes */
+ 	DUMPBITS(4);
+-#ifdef PKZIP_BUG_WORKAROUND
+-	if (nl > 288 || nd > 32)
+-#else
+ 	if (nl > 286 || nd > 30)
+-#endif
+ 		return 1;	/* bad lengths */
+ 
+ 	DEBG("dyn1 ");
+@@ -829,7 +795,7 @@
+ 
+ 	/* build decoding table for trees--single level, 7 bit lookup */
+ 	bl = 7;
+-	if ((i = huft_build(ll, 19, 19, NULL, NULL, &tl, &bl)) != 0) {
++	if ((i = huft_build(ll, 19, 19, 0, 0, &tl, &bl))) {
+ 		if (i == 1)
+ 			huft_free(tl);
+ 		return i;	/* incomplete code set */
+@@ -892,7 +858,7 @@
+ 
+ 	/* build the decoding tables for literal/length and distance codes */
+ 	bl = lbits;
+-	if ((i = huft_build(ll, nl, 257, cplens, cplext, &tl, &bl)) != 0) {
++	if ((i = huft_build(ll, nl, 257, cplens, cplext, &tl, &bl))) {
+ 		DEBG("dyn5b ");
+ 		if (i == 1) {
+ 			error("incomplete literal tree");
+@@ -902,19 +868,14 @@
+ 	}
+ 	DEBG("dyn5c ");
+ 	bd = dbits;
+-	if ((i = huft_build(ll + nl, nd, 0, cpdist, cpdext, &td, &bd)) != 0) {
++	if ((i = huft_build(ll + nl, nd, 0, cpdist, cpdext, &td, &bd))) {
+ 		DEBG("dyn5d ");
+ 		if (i == 1) {
+ 			error("incomplete distance tree");
+-#ifdef PKZIP_BUG_WORKAROUND
+-			i = 0;
+-		}
+-#else
+ 			huft_free(td);
+ 		}
+ 		huft_free(tl);
+ 		return i;	/* incomplete code set */
+-#endif
+ 	}
+ 
+ 	DEBG("dyn6 ");
+@@ -939,11 +900,11 @@
+ /* inflate_block - decompress a deflated block
+  * @e: last block flag
+  */
+-STATIC int INIT inflate_block(int *e)
++static int INIT inflate_block(int *e)
+ {
+ 	unsigned t;		/* block type */
+-	register ulg b;		/* bit buffer */
+-	register unsigned k;	/* number of bits in bit buffer */
++	u32 b;		/* bit buffer */
++	unsigned k;	/* number of bits in bit buffer */
+ 
+ 	DEBG("<blk");
+ 
+@@ -983,30 +944,25 @@
+ }
+ 
+ /* inflate - decompress an inflated entry */
+-STATIC int INIT inflate(void)
++static int INIT inflate(void)
+ {
+ 	int e;			/* last block flag */
+ 	int r;			/* result code */
+-	unsigned h;		/* maximum struct huft's malloc'ed */
+ 	void *ptr;
+ 
+ 	/* initialize window, bit buffer */
+-	wp = 0;
++	outcnt = 0;
+ 	bk = 0;
+ 	bb = 0;
+ 
+ 	/* decompress until the last block */
+-	h = 0;
+ 	do {
+-		hufts = 0;
+ 		gzip_mark(&ptr);
+-		if ((r = inflate_block(&e)) != 0) {
++		if ((r = inflate_block(&e))) {
+ 			gzip_release(&ptr);
+ 			return r;
+ 		}
+ 		gzip_release(&ptr);
+-		if (hufts > h)
+-			h = hufts;
+ 	} while (!e);
+ 
+ 	/* Undo too much lookahead. The next read will be byte aligned so we
+@@ -1017,13 +973,10 @@
+ 		inptr--;
+ 	}
+ 
+-	/* flush out slide */
+-	flush_output(wp);
++	/* flush out window */
++	flush_output(outcnt);
+ 
+ 	/* return success */
+-#ifdef DEBUG
+-	fprintf(stderr, "<%u> ", h);
+-#endif				/* DEBUG */
+ 	return 0;
+ }
+ 
+@@ -1033,8 +986,8 @@
+  *
+  **********************************************************************/
+ 
+-static ulg crc_32_tab[256];
+-static ulg crc;		/* initialized in makecrc() so it'll reside in bss */
++static u32 crc_32_tab[256];
++static u32 crc;		/* initialized in makecrc() so it'll reside in bss */
+ #define CRC_VALUE (crc ^ 0xffffffffUL)
+ 
+ /*
+@@ -1073,7 +1026,7 @@
+ 	}
+ 
+ 	/* this is initialized here so this code could reside in ROM */
+-	crc = (ulg)0xffffffffUL;	/* shift register contents */
++	crc = 0xffffffffUL;	/* shift register contents */
+ }
+ 
+ /* gzip flag byte */
+@@ -1090,11 +1043,11 @@
+  */
+ static int INIT gunzip(void)
+ {
+-	uch flags;
++	u8 flags;
+ 	unsigned char magic[2];	/* magic header */
+ 	char method;
+-	ulg orig_crc = 0;	/* original crc */
+-	ulg orig_len = 0;	/* original uncompressed length */
++	u32 orig_crc = 0;	/* original crc */
++	u32 orig_len = 0;	/* original uncompressed length */
+ 	int res;
+ 
+ 	magic[0] = NEXTBYTE();
+@@ -1112,16 +1065,16 @@
+ 		return -1;
+ 	}
+ 
+-	flags = (uch)get_byte();
+-	if ((flags & ENCRYPTED) != 0) {
++	flags = (u8)get_byte();
++	if (flags & ENCRYPTED) {
+ 		error("Input is encrypted");
+ 		return -1;
+ 	}
+-	if ((flags & CONTINUATION) != 0) {
++	if (flags & CONTINUATION) {
+ 		error("Multi part input");
+ 		return -1;
+ 	}
+-	if ((flags & RESERVED) != 0) {
++	if (flags & RESERVED) {
+ 		error("Input has invalid flags");
+ 		return -1;
+ 	}
+@@ -1133,25 +1086,22 @@
+ 	(void)NEXTBYTE();	/* Ignore extra flags for the moment */
+ 	(void)NEXTBYTE();	/* Ignore OS type for the moment */
+ 
+-	if ((flags & EXTRA_FIELD) != 0) {
++	if (flags & EXTRA_FIELD) {
+ 		unsigned len = (unsigned)NEXTBYTE();
+ 		len |= ((unsigned)NEXTBYTE()) << 8;
+ 		while (len--)
+ 			(void)NEXTBYTE();
+ 	}
+ 
+-	/* Get original file name if it was truncated */
+-	if ((flags & ORIG_NAME) != 0) {
+-		/* Discard the old name */
+-		while (NEXTBYTE() != 0)	/* null */
++	/* Discard original file name if it was truncated */
++	if (flags & ORIG_NAME)
++		while (NEXTBYTE())
+ 			;
+-	}
+ 
+ 	/* Discard file comment if any */
+-	if ((flags & COMMENT) != 0) {
+-		while (NEXTBYTE() != 0)	/* null */
++	if (flags & COMMENT)
++		while (NEXTBYTE())
+ 			;
+-	}
+ 
+ 	/* Decompress */
+ 	if ((res = inflate())) {
+@@ -1180,15 +1130,15 @@
+ 	/* crc32  (see algorithm.doc)
+ 	 * uncompressed input size modulo 2^32
+ 	 */
+-	orig_crc = (ulg)NEXTBYTE();
+-	orig_crc |= (ulg)NEXTBYTE() << 8;
+-	orig_crc |= (ulg)NEXTBYTE() << 16;
+-	orig_crc |= (ulg)NEXTBYTE() << 24;
+-
+-	orig_len = (ulg)NEXTBYTE();
+-	orig_len |= (ulg)NEXTBYTE() << 8;
+-	orig_len |= (ulg)NEXTBYTE() << 16;
+-	orig_len |= (ulg)NEXTBYTE() << 24;
++	orig_crc = (u32)NEXTBYTE();
++	orig_crc |= (u32)NEXTBYTE() << 8;
++	orig_crc |= (u32)NEXTBYTE() << 16;
++	orig_crc |= (u32)NEXTBYTE() << 24;
++
++	orig_len = (u32)NEXTBYTE();
++	orig_len |= (u32)NEXTBYTE() << 8;
++	orig_len |= (u32)NEXTBYTE() << 16;
++	orig_len |= (u32)NEXTBYTE() << 24;
+ 
+ 	/* Validate decompression */
+ 	if (orig_crc != CRC_VALUE) {
Index: packages/kernel/patches/tiny_m57sli/tiny/mtrr.patch
===================================================================
--- packages/kernel/patches/tiny_m57sli/tiny/mtrr.patch	(revision 0)
+++ packages/kernel/patches/tiny_m57sli/tiny/mtrr.patch	(revision 0)
@@ -0,0 +1,98 @@
+
+fix mtrr support with x86 cpu types
+
+
+ tiny-mpm/arch/i386/kernel/cpu/mtrr/Makefile |    6 +++---
+ tiny-mpm/arch/i386/kernel/cpu/mtrr/cyrix.c  |    2 +-
+ tiny-mpm/arch/i386/kernel/cpu/mtrr/main.c   |   19 +++++++++++++++++--
+ 3 files changed, 21 insertions(+), 6 deletions(-)
+
+Index: linux-2.6.16.19/arch/i386/kernel/cpu/mtrr/Makefile
+===================================================================
+--- linux-2.6.16.19.orig/arch/i386/kernel/cpu/mtrr/Makefile	2006-06-23 17:05:11.000000000 -0700
++++ linux-2.6.16.19/arch/i386/kernel/cpu/mtrr/Makefile	2006-06-23 17:06:28.000000000 -0700
+@@ -1,5 +1,5 @@
+ obj-y		:= main.o if.o generic.o state.o
+-obj-y		+= amd.o
+-obj-y		+= cyrix.o
+-obj-y		+= centaur.o
++obj-$(CONFIG_CPU_SUP_AMD)	+= amd.o
++obj-$(CONFIG_CPU_SUP_CYRIX)	+= cyrix.o
++obj-$(CONFIG_CPU_SUP_CENTAUR)	+= centaur.o
+ 
+Index: linux-2.6.16.19/arch/i386/kernel/cpu/mtrr/cyrix.c
+===================================================================
+--- linux-2.6.16.19.orig/arch/i386/kernel/cpu/mtrr/cyrix.c	2006-06-23 17:05:11.000000000 -0700
++++ linux-2.6.16.19/arch/i386/kernel/cpu/mtrr/cyrix.c	2006-06-23 17:06:28.000000000 -0700
+@@ -5,7 +5,7 @@
+ #include <asm/io.h>
+ #include "mtrr.h"
+ 
+-int arr3_protected;
++extern int arr3_protected;
+ 
+ static void
+ cyrix_get_arr(unsigned int reg, unsigned long *base,
+Index: linux-2.6.16.19/arch/i386/kernel/cpu/mtrr/main.c
+===================================================================
+--- linux-2.6.16.19.orig/arch/i386/kernel/cpu/mtrr/main.c	2006-06-23 17:05:11.000000000 -0700
++++ linux-2.6.16.19/arch/i386/kernel/cpu/mtrr/main.c	2006-06-23 17:06:28.000000000 -0700
+@@ -58,7 +58,7 @@
+ static void set_mtrr(unsigned int reg, unsigned long base,
+ 		     unsigned long size, mtrr_type type);
+ 
+-extern int arr3_protected;
++int arr3_protected;
+ 
+ void set_mtrr_ops(struct mtrr_ops * ops)
+ {
+@@ -543,9 +543,15 @@
+ 
+ static void __init init_ifs(void)
+ {
++#ifdef CONFIG_CPU_SUP_AMD
+ 	amd_init_mtrr();
++#endif
++#ifdef CONFIG_CPU_SUP_CYRIX
+ 	cyrix_init_mtrr();
++#endif
++#ifdef CONFIG_CPU_SUP_CENTAUR
+ 	centaur_init_mtrr();
++#endif
+ }
+ 
+ /* The suspend/resume methods are only for CPU without MTRR. CPU using generic
+@@ -643,6 +649,7 @@
+ 		}
+ 	} else {
+ 		switch (boot_cpu_data.x86_vendor) {
++#ifdef CONFIG_CPU_SUP_AMD
+ 		case X86_VENDOR_AMD:
+ 			if (cpu_has_k6_mtrr) {
+ 				/* Pre-Athlon (K6) AMD CPU MTRRs */
+@@ -651,6 +658,8 @@
+ 				size_and_mask = 0;
+ 			}
+ 			break;
++#endif
++#ifdef CONFIG_CPU_SUP_CENTAUR
+ 		case X86_VENDOR_CENTAUR:
+ 			if (cpu_has_centaur_mcr) {
+ 				mtrr_if = mtrr_ops[X86_VENDOR_CENTAUR];
+@@ -658,6 +667,8 @@
+ 				size_and_mask = 0;
+ 			}
+ 			break;
++#endif
++#ifdef CONFIG_CPU_SUP_CYRIX
+ 		case X86_VENDOR_CYRIX:
+ 			if (cpu_has_cyrix_arr) {
+ 				mtrr_if = mtrr_ops[X86_VENDOR_CYRIX];
+@@ -665,6 +676,7 @@
+ 				size_and_mask = 0;
+ 			}
+ 			break;
++#endif
+ 		default:
+ 			break;
+ 		}
Index: packages/kernel/patches/tiny_m57sli/tiny/inflate-huft.patch
===================================================================
--- packages/kernel/patches/tiny_m57sli/tiny/inflate-huft.patch	(revision 0)
+++ packages/kernel/patches/tiny_m57sli/tiny/inflate-huft.patch	(revision 0)
@@ -0,0 +1,259 @@
+
+inflate  cleanup huffman table code
+
+sensible names for huft struct members
+get rid of assignment-in-expression usage
+
+
+ tiny-mpm/lib/inflate.c |  113 +++++++++++++++++++++++++------------------------
+ 1 files changed, 59 insertions(+), 54 deletions(-)
+
+Index: linux-2.6.16.19/lib/inflate.c
+===================================================================
+--- linux-2.6.16.19.orig/lib/inflate.c	2006-06-23 17:06:29.000000000 -0700
++++ linux-2.6.16.19/lib/inflate.c	2006-06-23 17:06:29.000000000 -0700
+@@ -117,12 +117,12 @@
+    an unused code.  If a code with e == 99 is looked up, this implies an
+    error in the data. */
+ struct huft {
+-	u8 e;			/* number of extra bits or operation */
+-	u8 b;			/* number of bits in this code or subcode */
+ 	union {
+-		u16 n;		/* literal, length base, or distance base */
+-		struct huft *t;	/* pointer to next level of table */
+-	} v;
++		u16 val; /* literal, length base, or distance base */
++		struct huft *next; /* pointer to next level of table */
++	};
++	u8 extra; /* number of extra bits or operation */
++	u8 bits; /* number of bits in this code or subcode */
+ };
+ 
+ struct iostate {
+@@ -395,11 +395,14 @@
+ 	DEBG("huft3 ");
+ 
+ 	/* Adjust last length count to fill out codes, if needed */
+-	for (y = 1 << j; j < i; j++, y <<= 1)
+-		if ((y -= c[j]) < 0)
++	for (y = 1 << j; j < i; j++, y <<= 1) {
++		y -= c[j];
++		if (y < 0)
+ 			return 2; /* bad input: more codes than bits */
++	}
+ 
+-	if ((y -= c[i]) < 0)
++	y -= c[i];
++	if (y < 0)
+ 		return 2;
+ 	c[i] += y;
+ 
+@@ -410,8 +413,12 @@
+ 	p = c + 1;
+ 	xp = x + 2;
+ 	/* note that i == g from above */
+-	while (--i)
+-		*xp++ = (j += *p++);
++	while (--i) {
++		j += *p;
++		*xp = j;
++		p++;
++		xp++;
++	}
+ 
+ 	DEBG("huft5 ");
+ 
+@@ -419,7 +426,8 @@
+ 	p = b;
+ 	i = 0;
+ 	do {
+-		if ((j = *p++))
++		j = *p++;
++		if (j)
+ 			v[x[j]++] = i;
+ 	} while (++i < n);
+ 
+@@ -452,10 +460,12 @@
+ 
+ 				/* compute min size <= l bits */
+ 				/* upper limit on table size */
+-				z = (z = g - w) > (unsigned)l ? l : z;
++				z = min(l, g - w);
+ 
+ 				/* try a k-w bit table */
+-				if ((f = 1 << (j = k - w)) > a + 1) {
++				j = k - w;
++				f = 1 << j;
++				if (f > a + 1) {
+ 					/* too few codes for k-w bit table */
+ 					DEBG1("2 ");
+ 					/* deduct codes from patterns left */
+@@ -465,7 +475,8 @@
+ 					if (j < z) {
+ 						/* enough codes for j bits? */
+ 						while (++j < z) {
+-							if ((f <<= 1) <= *++xp)
++							f <<= 1;
++							if (f <= *++xp)
+ 								break;
+ 							/* deduct from pats */
+ 							f -= *xp;
+@@ -478,15 +489,17 @@
+ 				z = 1 << j;
+ 
+ 				/* allocate and link in new table */
+-				if (!(q = (struct huft *)malloc(
+-					     (z + 1) * sizeof(struct huft)))) {
++				q = malloc((z + 1) * sizeof(struct huft));
++				if (!q) {
+ 					if (h)
+ 						huft_free(u[0]);
+ 					return 3;	/* not enough memory */
+ 				}
++
+ 				DEBG1("4 ");
+ 				*t = q + 1; /* link to list for huft_free */
+-				*(t = &(q->v.t)) = 0;
++				t = &q->next;
++				*t = NULL;
+ 				u[h] = ++q;	/* table starts after link */
+ 
+ 				DEBG1("5 ");
+@@ -495,11 +508,11 @@
+ 					/* save pattern for backing up */
+ 					x[h] = i;
+ 					/* bits to dump before this table */
+-					r.b = (u8)l;
++					r.bits = (u8)l;
+ 					/* bits in this table */
+-					r.e = (u8)(16 + j);
++					r.extra = (u8)(16 + j);
+ 					/* pointer to this table */
+-					r.v.t = q;
++					r.next = q;
+ 					/* (get around Turbo C bug) */
+ 					j = i >> (w - l);
+ 					/* connect to last table */
+@@ -510,20 +523,20 @@
+ 			DEBG("h6c ");
+ 
+ 			/* set up table entry in r */
+-			r.b = (u8) (k - w);
++			r.bits = (u8)(k - w);
+ 			if (p >= v + n)
+-				r.e = 99; /* out of values--invalid code */
++				r.extra = 99; /* out of values--invalid code */
+ 			else if (*p < s) {
+ 				/* 256 is end-of-block code */
+-				r.e = (u8)(*p < 256 ? 16 : 15);
++				r.extra = (u8)(*p < 256 ? 16 : 15);
+ 				/* simple code is just the value */
+-				r.v.n = (u16)(*p);
++				r.val = (u16)(*p);
+ 				/* one compiler does not like *p++ */
+ 				p++;
+ 			} else {
+ 				/* non-simple--look up in lists */
+-				r.e = (u8)e[*p - s];
+-				r.v.n = d[*p++ - s];
++				r.extra = (u8)e[*p - s];
++				r.val = d[*p++ - s];
+ 			}
+ 			DEBG("h6d ");
+ 
+@@ -561,16 +574,16 @@
+  * linked list of the tables it made, with the links in a dummy first
+  * entry of each table.
+  */
+-STATIC int INIT huft_free(struct huft *t)
++static int INIT huft_free(struct huft *t)
+ {
+-	struct huft *p, *q;
++	struct huft *q;
+ 
+ 	/* Go through list, freeing from the malloced (t[-1]) address. */
+-	p = t;
+-	while (p) {
+-		q = (--p)->v.t;
+-		free((char *)p);
+-		p = q;
++	while (t) {
++		t -= 1;
++		q = t->next;
++		free(t);
++		t = q;
+ 	}
+ 	return 0;
+ }
+@@ -589,45 +602,40 @@
+ static int INIT inflate_codes(struct iostate *io, struct huft *tl, struct huft *td,
+ 			 int bl, int bd)
+ {
+-	unsigned e;	/* table entry flag/number of extra bits */
+ 	unsigned len, dist;
+ 	struct huft *t;		/* pointer to table entry */
+ 
+ 	/* inflate the coded data */
+ 	for (;;) {		/* do until end of block */
+ 		t = tl + readbits(io, bl);
+-		e = t->e;
+-		while (e > 16) {
+-			if (e == 99)
++		while (t->extra > 16) {
++			if (t->extra == 99)
+ 				return 1;
+-			dumpbits(io, t->b);
+-			t = t->v.t + readbits(io, e - 16);
+-			e = t->e;
++			dumpbits(io, t->bits);
++			t = &t->next[readbits(io, t->extra - 16)];
+ 		}
+-		dumpbits(io, t->b);
+-		if (e == 16) {	/* then it's a literal */
+-			put_byte(io, t->v.n);
++		dumpbits(io, t->bits);
++		if (t->extra == 16) {	/* then it's a literal */
++			put_byte(io, t->val);
+ 		} else {	/* it's an EOB or a length */
+ 			/* exit if end of block */
+-			if (e == 15)
++			if (t->extra == 15)
+ 				break;
+ 
+ 			/* get length of block to copy */
+-			len = t->v.n + pullbits(io, e);
++			len = t->val + pullbits(io, t->extra);
+ 
+ 			/* decode distance of block to copy */
+ 			t = td + readbits(io, bd);
+-			e = t->e;
+-			while (e > 16) {
+-				if (e == 99)
++			while (t->extra > 16) {
++				if (t->extra == 99)
+ 					return 1;
+-				dumpbits(io, t->b);
+-				t = t->v.t + readbits(io, e - 16);
+-				e = t->e;
++				dumpbits(io, t->bits);
++				t = &t->next[readbits(io, t->extra - 16)];
+ 			}
+-			dumpbits(io, t->b);
++			dumpbits(io, t->bits);
+ 
+-			dist = t->v.n + pullbits(io, e);
++			dist = t->val + pullbits(io, t->extra);
+ 			copy_bytes(io, len, dist);
+ 		}
+ 	}
+@@ -769,8 +777,8 @@
+ 	i = l = 0;
+ 	while ((unsigned)i < n) {
+ 		td = tl + readbits(io, bl);
+-		dumpbits(io, td->b);
+-		j = td->v.n;
++		dumpbits(io, td->bits);
++		j = td->val;
+ 		if (j < 16)	/* length of code in bits (0..15) */
+ 			ll[i++] = l = j;	/* save last length in l */
+ 		else if (j == 16) {	/* repeat last length 3 to 6 times */
Index: packages/kernel/patches/tiny_m57sli/tiny/elf-no-aout.patch
===================================================================
--- packages/kernel/patches/tiny_m57sli/tiny/elf-no-aout.patch	(revision 0)
+++ packages/kernel/patches/tiny_m57sli/tiny/elf-no-aout.patch	(revision 0)
@@ -0,0 +1,265 @@
+
+configurable support for ELF with a.out loader or libs
+
+
+ tiny-mpm/arch/mips/kernel/irixelf.c |   12 +++++++++++-
+ tiny-mpm/fs/binfmt_elf.c            |   30 +++++++++++++++++++++++++++++-
+ tiny-mpm/fs/exec.c                  |    4 ++++
+ tiny-mpm/init/Kconfig               |    9 +++++++++
+ 4 files changed, 53 insertions(+), 2 deletions(-)
+
+Index: linux-2.6.16.19/arch/mips/kernel/irixelf.c
+===================================================================
+--- linux-2.6.16.19.orig/arch/mips/kernel/irixelf.c	2006-06-23 17:05:12.000000000 -0700
++++ linux-2.6.16.19/arch/mips/kernel/irixelf.c	2006-06-23 17:06:25.000000000 -0700
+@@ -10,6 +10,7 @@
+  * Copyright (C) 1996 - 2004 David S. Miller <dm@engr.sgi.com>
+  * Copyright (C) 2004 - 2005 Steven J. Hill <sjhill@realitydiluted.com>
+  */
++#include <linux/config.h>
+ #include <linux/module.h>
+ #include <linux/fs.h>
+ #include <linux/stat.h>
+@@ -43,12 +44,19 @@
+ #undef DEBUG
+ 
+ static int load_irix_binary(struct linux_binprm * bprm, struct pt_regs * regs);
++#ifdef CONFIG_BINFMT_ELF_AOUT
+ static int load_irix_library(struct file *);
++#endif
+ static int irix_core_dump(long signr, struct pt_regs * regs,
+                           struct file *file);
+ 
+ static struct linux_binfmt irix_format = {
+-	NULL, THIS_MODULE, load_irix_binary, load_irix_library,
++	NULL, THIS_MODULE, load_irix_binary,
++#ifdef CONFIG_BINFMT_ELF_AOUT
++	load_irix_library,
++#else
++	NULL,
++#endif
+ 	irix_core_dump, PAGE_SIZE
+ };
+ 
+@@ -791,6 +799,7 @@
+ 	goto out;
+ }
+ 
++#ifdef CONFIG_BINFMT_ELF_AOUT
+ /* This is really simpleminded and specialized - we are loading an
+  * a.out library that is given an ELF header.
+  */
+@@ -869,6 +878,7 @@
+ 	kfree(elf_phdata);
+ 	return 0;
+ }
++#endif
+ 
+ /* Called through irix_syssgi() to map an elf image given an FD,
+  * a phdr ptr USER_PHDRP in userspace, and a count CNT telling how many
+Index: linux-2.6.16.19/fs/binfmt_elf.c
+===================================================================
+--- linux-2.6.16.19.orig/fs/binfmt_elf.c	2006-06-23 17:05:12.000000000 -0700
++++ linux-2.6.16.19/fs/binfmt_elf.c	2006-06-23 17:06:25.000000000 -0700
+@@ -46,7 +46,9 @@
+ #include <linux/elf.h>
+ 
+ static int load_elf_binary(struct linux_binprm * bprm, struct pt_regs * regs);
++#ifdef CONFIG_BINFMT_ELF_AOUT
+ static int load_elf_library(struct file*);
++#endif
+ static unsigned long elf_map (struct file *, unsigned long, struct elf_phdr *, int, int);
+ extern int dump_fpu (struct pt_regs *, elf_fpregset_t *);
+ 
+@@ -81,7 +83,11 @@
+ static struct linux_binfmt elf_format = {
+ 		.module		= THIS_MODULE,
+ 		.load_binary	= load_elf_binary,
++#ifdef CONFIG_BINFMT_ELF_AOUT
+ 		.load_shlib	= load_elf_library,
++#else
++		.load_shlib	= NULL,
++#endif
+ 		.core_dump	= elf_core_dump,
+ 		.min_coredump	= ELF_EXEC_PAGESIZE
+ };
+@@ -444,6 +450,7 @@
+ 	return error;
+ }
+ 
++#ifdef CONFIG_BINFMT_ELF_AOUT
+ static unsigned long load_aout_interp(struct exec * interp_ex,
+ 			     struct file * interpreter)
+ {
+@@ -490,6 +497,7 @@
+ out:
+ 	return elf_entry;
+ }
++#endif
+ 
+ /*
+  * These are the functions used to load ELF style executables and shared
+@@ -497,7 +505,9 @@
+  */
+ 
+ #define INTERPRETER_NONE 0
++#ifdef CONFIG_BINFMT_ELF_AOUT
+ #define INTERPRETER_AOUT 1
++#endif
+ #define INTERPRETER_ELF 2
+ 
+ 
+@@ -531,7 +541,9 @@
+ 	unsigned long elf_entry, interp_load_addr = 0;
+ 	unsigned long start_code, end_code, start_data, end_data;
+ 	unsigned long reloc_func_desc = 0;
++#ifdef CONFIG_BINFMT_ELF_AOUT
+ 	char passed_fileno[6];
++#endif
+ 	struct files_struct *files;
+ 	int have_pt_gnu_stack, executable_stack = EXSTACK_DEFAULT;
+ 	unsigned long def_flags = 0;
+@@ -701,12 +713,14 @@
+ 
+ 	/* Some simple consistency checks for the interpreter */
+ 	if (elf_interpreter) {
++#ifdef CONFIG_BINFMT_ELF_AOUT
+ 		interpreter_type = INTERPRETER_ELF | INTERPRETER_AOUT;
+ 
+ 		/* Now figure out which format our binary is */
+ 		if ((N_MAGIC(loc->interp_ex) != OMAGIC) &&
+ 		    (N_MAGIC(loc->interp_ex) != ZMAGIC) &&
+ 		    (N_MAGIC(loc->interp_ex) != QMAGIC))
++#endif
+ 			interpreter_type = INTERPRETER_ELF;
+ 
+ 		if (memcmp(loc->interp_elf_ex.e_ident, ELFMAG, SELFMAG) != 0)
+@@ -716,6 +730,7 @@
+ 		if (!interpreter_type)
+ 			goto out_free_dentry;
+ 
++#ifdef CONFIG_BINFMT_ELF_AOUT
+ 		/* Make sure only one type was selected */
+ 		if ((interpreter_type & INTERPRETER_ELF) &&
+ 		     interpreter_type != INTERPRETER_ELF) {
+@@ -723,6 +738,7 @@
+ 			// printk(KERN_WARNING "ELF: Ambiguous type, using ELF\n");
+ 			interpreter_type = INTERPRETER_ELF;
+ 		}
++#endif
+ 		/* Verify the interpreter has a valid arch */
+ 		if ((interpreter_type == INTERPRETER_ELF) &&
+ 		    !elf_check_arch(&loc->interp_elf_ex))
+@@ -735,6 +751,7 @@
+ 	/* OK, we are done with that, now set up the arg stuff,
+ 	   and then start this sucker up */
+ 
++#ifdef CONFIG_BINFMT_ELF_AOUT
+ 	if ((!bprm->sh_bang) && (interpreter_type == INTERPRETER_AOUT)) {
+ 		char *passed_p = passed_fileno;
+ 		sprintf(passed_fileno, "%d", elf_exec_fileno);
+@@ -746,6 +763,7 @@
+ 			bprm->argc++;
+ 		}
+ 	}
++#endif
+ 
+ 	/* Flush all traces of the currently running executable */
+ 	retval = flush_old_exec(bprm);
+@@ -917,10 +935,12 @@
+ 	}
+ 
+ 	if (elf_interpreter) {
++#ifdef CONFIG_BINFMT_ELF_AOUT
+ 		if (interpreter_type == INTERPRETER_AOUT)
+ 			elf_entry = load_aout_interp(&loc->interp_ex,
+ 						     interpreter);
+ 		else
++#endif
+ 			elf_entry = load_elf_interp(&loc->interp_elf_ex,
+ 						    interpreter,
+ 						    &interp_load_addr);
+@@ -947,7 +967,9 @@
+ 
+ 	kfree(elf_phdata);
+ 
++#ifdef CONFIG_BINFMT_ELF_AOUT
+ 	if (interpreter_type != INTERPRETER_AOUT)
++#endif
+ 		sys_close(elf_exec_fileno);
+ 
+ 	set_binfmt(&elf_format);
+@@ -962,11 +984,15 @@
+ 
+ 	compute_creds(bprm);
+ 	current->flags &= ~PF_FORKNOEXEC;
++#ifdef CONFIG_BINFMT_ELF_AOUT
+ 	create_elf_tables(bprm, &loc->elf_ex, (interpreter_type == INTERPRETER_AOUT),
+ 			load_addr, interp_load_addr);
+ 	/* N.B. passed_fileno might not be initialized? */
+ 	if (interpreter_type == INTERPRETER_AOUT)
+ 		current->mm->arg_start += strlen(passed_fileno) + 1;
++#else
++	create_elf_tables(bprm, &loc->elf_ex, 0, load_addr, interp_load_addr);
++#endif
+ 	current->mm->end_code = end_code;
+ 	current->mm->start_code = start_code;
+ 	current->mm->start_data = start_data;
+@@ -1030,6 +1056,7 @@
+ 	goto out;
+ }
+ 
++#ifdef CONFIG_BINFMT_ELF_AOUT
+ /* This is really simpleminded and specialized - we are loading an
+    a.out library that is given an ELF header. */
+ 
+@@ -1113,6 +1140,7 @@
+ out:
+ 	return error;
+ }
++#endif
+ 
+ /*
+  * Note that some platforms still use traditional core dumps and not
+Index: linux-2.6.16.19/fs/exec.c
+===================================================================
+--- linux-2.6.16.19.orig/fs/exec.c	2006-06-23 17:05:12.000000000 -0700
++++ linux-2.6.16.19/fs/exec.c	2006-06-23 17:06:25.000000000 -0700
+@@ -123,6 +123,7 @@
+  */
+ asmlinkage long sys_uselib(const char __user * library)
+ {
++#if defined(CONFIG_BINFMT_AOUT) || defined(CONFIG_BINFMT_AOUT_MODULE) || defined(CONFIG_BINFMT_ELF_AOUT)
+ 	struct file * file;
+ 	struct nameidata nd;
+ 	int error;
+@@ -170,6 +171,9 @@
+ 	release_open_intent(&nd);
+ 	path_release(&nd);
+ 	goto out;
++#else
++	return -ENOSYS;
++#endif
+ }
+ 
+ /*
+Index: linux-2.6.16.19/init/Kconfig
+===================================================================
+--- linux-2.6.16.19.orig/init/Kconfig	2006-06-23 17:06:25.000000000 -0700
++++ linux-2.6.16.19/init/Kconfig	2006-06-23 17:06:25.000000000 -0700
+@@ -553,6 +553,15 @@
+ 	help
+ 	  Enable support for standard #!-style shell scripts.
+ 
++config BINFMT_ELF_AOUT
++	bool "ELF binaries with a.out linkage" if EMBEDDED
++	depends on BINFMT_ELF
++	default BINFMT_AOUT
++	help
++	  The kernel may support ELF executables which use an a.out format
++	  interpreter (dynamic linker) and/or a.out shared libraries, in
++	  addition to the usual ELF-ELF setups. You shouldn't need this.
++
+ endmenu		# General setup
+ 
+ config CRC32_CALC
Index: packages/kernel/patches/tiny_m57sli/tiny/inflate-killdecls.patch
===================================================================
--- packages/kernel/patches/tiny_m57sli/tiny/inflate-killdecls.patch	(revision 0)
+++ packages/kernel/patches/tiny_m57sli/tiny/inflate-killdecls.patch	(revision 0)
@@ -0,0 +1,996 @@
+inflate: remove a bunch of declarations/definitions from callers
+
+This removes:
+
+- memset/memzero/memcpy implementations
+- OF
+- STATIC
+- gzip flag byte defines
+- unused debug defines
+
+and saves an average of 50 lines in each of 12 users.
+
+Signed-off-by: Matt Mackall <mpm@selenic.com>
+
+Index: linux-2.6.16.19/arch/alpha/boot/misc.c
+===================================================================
+--- linux-2.6.16.19.orig/arch/alpha/boot/misc.c	2006-06-23 17:05:11.000000000 -0700
++++ linux-2.6.16.19/arch/alpha/boot/misc.c	2006-06-23 17:06:31.000000000 -0700
+@@ -22,7 +22,6 @@
+ 
+ #include <asm/uaccess.h>
+ 
+-#define memzero(s,n)	memset ((s),0,(n))
+ #define puts		srm_printk
+ extern long srm_printk(const char *, ...)
+      __attribute__ ((format (printf, 1, 2)));
+@@ -30,8 +29,6 @@
+ /*
+  * gzip delarations
+  */
+-#define OF(args)  args
+-#define STATIC static
+ 
+ typedef unsigned char  uch;
+ typedef unsigned short ush;
+@@ -47,34 +44,8 @@
+ static unsigned inptr;		/* index of next byte to be processed in inbuf */
+ static unsigned outcnt;		/* bytes in output buffer */
+ 
+-/* gzip flag byte */
+-#define ASCII_FLAG   0x01 /* bit 0 set: file probably ascii text */
+-#define CONTINUATION 0x02 /* bit 1 set: continuation of multi-part gzip file */
+-#define EXTRA_FIELD  0x04 /* bit 2 set: extra field present */
+-#define ORIG_NAME    0x08 /* bit 3 set: original file name present */
+-#define COMMENT      0x10 /* bit 4 set: file comment present */
+-#define ENCRYPTED    0x20 /* bit 5 set: file is encrypted */
+-#define RESERVED     0xC0 /* bit 6,7:   reserved */
+-
+ #define get_byte()  (inptr < insize ? inbuf[inptr++] : fill_inbuf())
+ 
+-/* Diagnostic functions */
+-#ifdef DEBUG
+-#  define Assert(cond,msg) {if(!(cond)) error(msg);}
+-#  define Trace(x) fprintf x
+-#  define Tracev(x) {if (verbose) fprintf x ;}
+-#  define Tracevv(x) {if (verbose>1) fprintf x ;}
+-#  define Tracec(c,x) {if (verbose && (c)) fprintf x ;}
+-#  define Tracecv(c,x) {if (verbose>1 && (c)) fprintf x ;}
+-#else
+-#  define Assert(cond,msg)
+-#  define Trace(x)
+-#  define Tracev(x)
+-#  define Tracevv(x)
+-#  define Tracec(c,x)
+-#  define Tracecv(c,x)
+-#endif
+-
+ static int  fill_inbuf(void);
+ static void flush_window(void);
+ static void error(char *m);
+Index: linux-2.6.16.19/arch/arm/boot/compressed/misc.c
+===================================================================
+--- linux-2.6.16.19.orig/arch/arm/boot/compressed/misc.c	2006-06-23 17:05:11.000000000 -0700
++++ linux-2.6.16.19/arch/arm/boot/compressed/misc.c	2006-06-23 17:06:31.000000000 -0700
+@@ -45,90 +45,8 @@
+ #define __ptr_t void *
+ 
+ /*
+- * Optimised C version of memzero for the ARM.
++ * gzip declarations
+  */
+-void __memzero (__ptr_t s, size_t n)
+-{
+-	union { void *vp; unsigned long *ulp; unsigned char *ucp; } u;
+-	int i;
+-
+-	u.vp = s;
+-
+-	for (i = n >> 5; i > 0; i--) {
+-		*u.ulp++ = 0;
+-		*u.ulp++ = 0;
+-		*u.ulp++ = 0;
+-		*u.ulp++ = 0;
+-		*u.ulp++ = 0;
+-		*u.ulp++ = 0;
+-		*u.ulp++ = 0;
+-		*u.ulp++ = 0;
+-	}
+-
+-	if (n & 1 << 4) {
+-		*u.ulp++ = 0;
+-		*u.ulp++ = 0;
+-		*u.ulp++ = 0;
+-		*u.ulp++ = 0;
+-	}
+-
+-	if (n & 1 << 3) {
+-		*u.ulp++ = 0;
+-		*u.ulp++ = 0;
+-	}
+-
+-	if (n & 1 << 2)
+-		*u.ulp++ = 0;
+-
+-	if (n & 1 << 1) {
+-		*u.ucp++ = 0;
+-		*u.ucp++ = 0;
+-	}
+-
+-	if (n & 1)
+-		*u.ucp++ = 0;
+-}
+-
+-static inline __ptr_t memcpy(__ptr_t __dest, __const __ptr_t __src,
+-			    size_t __n)
+-{
+-	int i = 0;
+-	unsigned char *d = (unsigned char *)__dest, *s = (unsigned char *)__src;
+-
+-	for (i = __n >> 3; i > 0; i--) {
+-		*d++ = *s++;
+-		*d++ = *s++;
+-		*d++ = *s++;
+-		*d++ = *s++;
+-		*d++ = *s++;
+-		*d++ = *s++;
+-		*d++ = *s++;
+-		*d++ = *s++;
+-	}
+-
+-	if (__n & 1 << 2) {
+-		*d++ = *s++;
+-		*d++ = *s++;
+-		*d++ = *s++;
+-		*d++ = *s++;
+-	}
+-
+-	if (__n & 1 << 1) {
+-		*d++ = *s++;
+-		*d++ = *s++;
+-	}
+-
+-	if (__n & 1)
+-		*d++ = *s++;
+-
+-	return __dest;
+-}
+-
+-/*
+- * gzip delarations
+- */
+-#define OF(args)  args
+-#define STATIC static
+ 
+ typedef unsigned char  uch;
+ typedef unsigned short ush;
+@@ -144,34 +62,8 @@
+ static unsigned inptr;		/* index of next byte to be processed in inbuf */
+ static unsigned outcnt;		/* bytes in output buffer */
+ 
+-/* gzip flag byte */
+-#define ASCII_FLAG   0x01 /* bit 0 set: file probably ascii text */
+-#define CONTINUATION 0x02 /* bit 1 set: continuation of multi-part gzip file */
+-#define EXTRA_FIELD  0x04 /* bit 2 set: extra field present */
+-#define ORIG_NAME    0x08 /* bit 3 set: original file name present */
+-#define COMMENT      0x10 /* bit 4 set: file comment present */
+-#define ENCRYPTED    0x20 /* bit 5 set: file is encrypted */
+-#define RESERVED     0xC0 /* bit 6,7:   reserved */
+-
+ #define get_byte()  (inptr < insize ? inbuf[inptr++] : fill_inbuf())
+ 
+-/* Diagnostic functions */
+-#ifdef DEBUG
+-#  define Assert(cond,msg) {if(!(cond)) error(msg);}
+-#  define Trace(x) fprintf x
+-#  define Tracev(x) {if (verbose) fprintf x ;}
+-#  define Tracevv(x) {if (verbose>1) fprintf x ;}
+-#  define Tracec(c,x) {if (verbose && (c)) fprintf x ;}
+-#  define Tracecv(c,x) {if (verbose>1 && (c)) fprintf x ;}
+-#else
+-#  define Assert(cond,msg)
+-#  define Trace(x)
+-#  define Tracev(x)
+-#  define Tracevv(x)
+-#  define Tracec(c,x)
+-#  define Tracecv(c,x)
+-#endif
+-
+ static int  fill_inbuf(void);
+ static void flush_window(void);
+ static void error(char *m);
+Index: linux-2.6.16.19/arch/arm26/boot/compressed/misc.c
+===================================================================
+--- linux-2.6.16.19.orig/arch/arm26/boot/compressed/misc.c	2006-06-23 17:05:11.000000000 -0700
++++ linux-2.6.16.19/arch/arm26/boot/compressed/misc.c	2006-06-23 17:06:31.000000000 -0700
+@@ -30,90 +30,8 @@
+ #define __ptr_t void *
+ 
+ /*
+- * Optimised C version of memzero for the ARM.
+- */
+-void __memzero (__ptr_t s, size_t n)
+-{
+-	union { void *vp; unsigned long *ulp; unsigned char *ucp; } u;
+-	int i;
+-
+-	u.vp = s;
+-
+-	for (i = n >> 5; i > 0; i--) {
+-		*u.ulp++ = 0;
+-		*u.ulp++ = 0;
+-		*u.ulp++ = 0;
+-		*u.ulp++ = 0;
+-		*u.ulp++ = 0;
+-		*u.ulp++ = 0;
+-		*u.ulp++ = 0;
+-		*u.ulp++ = 0;
+-	}
+-
+-	if (n & 1 << 4) {
+-		*u.ulp++ = 0;
+-		*u.ulp++ = 0;
+-		*u.ulp++ = 0;
+-		*u.ulp++ = 0;
+-	}
+-
+-	if (n & 1 << 3) {
+-		*u.ulp++ = 0;
+-		*u.ulp++ = 0;
+-	}
+-
+-	if (n & 1 << 2)
+-		*u.ulp++ = 0;
+-
+-	if (n & 1 << 1) {
+-		*u.ucp++ = 0;
+-		*u.ucp++ = 0;
+-	}
+-
+-	if (n & 1)
+-		*u.ucp++ = 0;
+-}
+-
+-static inline __ptr_t memcpy(__ptr_t __dest, __const __ptr_t __src,
+-			    size_t __n)
+-{
+-	int i = 0;
+-	unsigned char *d = (unsigned char *)__dest, *s = (unsigned char *)__src;
+-
+-	for (i = __n >> 3; i > 0; i--) {
+-		*d++ = *s++;
+-		*d++ = *s++;
+-		*d++ = *s++;
+-		*d++ = *s++;
+-		*d++ = *s++;
+-		*d++ = *s++;
+-		*d++ = *s++;
+-		*d++ = *s++;
+-	}
+-
+-	if (__n & 1 << 2) {
+-		*d++ = *s++;
+-		*d++ = *s++;
+-		*d++ = *s++;
+-		*d++ = *s++;
+-	}
+-
+-	if (__n & 1 << 1) {
+-		*d++ = *s++;
+-		*d++ = *s++;
+-	}
+-
+-	if (__n & 1)
+-		*d++ = *s++;
+-
+-	return __dest;
+-}
+-
+-/*
+  * gzip delarations
+  */
+-#define OF(args)  args
+-#define STATIC static
+ 
+ typedef unsigned char  uch;
+ typedef unsigned short ush;
+@@ -129,34 +47,8 @@
+ static unsigned inptr;		/* index of next byte to be processed in inbuf */
+ static unsigned outcnt;		/* bytes in output buffer */
+ 
+-/* gzip flag byte */
+-#define ASCII_FLAG   0x01 /* bit 0 set: file probably ascii text */
+-#define CONTINUATION 0x02 /* bit 1 set: continuation of multi-part gzip file */
+-#define EXTRA_FIELD  0x04 /* bit 2 set: extra field present */
+-#define ORIG_NAME    0x08 /* bit 3 set: original file name present */
+-#define COMMENT      0x10 /* bit 4 set: file comment present */
+-#define ENCRYPTED    0x20 /* bit 5 set: file is encrypted */
+-#define RESERVED     0xC0 /* bit 6,7:   reserved */
+-
+ #define get_byte()  (inptr < insize ? inbuf[inptr++] : fill_inbuf())
+ 
+-/* Diagnostic functions */
+-#ifdef DEBUG
+-#  define Assert(cond,msg) {if(!(cond)) error(msg);}
+-#  define Trace(x) fprintf x
+-#  define Tracev(x) {if (verbose) fprintf x ;}
+-#  define Tracevv(x) {if (verbose>1) fprintf x ;}
+-#  define Tracec(c,x) {if (verbose && (c)) fprintf x ;}
+-#  define Tracecv(c,x) {if (verbose>1 && (c)) fprintf x ;}
+-#else
+-#  define Assert(cond,msg)
+-#  define Trace(x)
+-#  define Tracev(x)
+-#  define Tracevv(x)
+-#  define Tracec(c,x)
+-#  define Tracecv(c,x)
+-#endif
+-
+ static int  fill_inbuf(void);
+ static void flush_window(void);
+ static void error(char *m);
+Index: linux-2.6.16.19/arch/cris/arch-v10/boot/compressed/misc.c
+===================================================================
+--- linux-2.6.16.19.orig/arch/cris/arch-v10/boot/compressed/misc.c	2006-06-23 17:05:11.000000000 -0700
++++ linux-2.6.16.19/arch/cris/arch-v10/boot/compressed/misc.c	2006-06-23 17:06:31.000000000 -0700
+@@ -29,16 +29,6 @@
+  * gzip declarations
+  */
+ 
+-#define OF(args)  args
+-#define STATIC static
+-
+-void* memset(void* s, int c, size_t n);
+-void* memcpy(void* __dest, __const void* __src,
+-	     size_t __n);
+-
+-#define memzero(s, n)     memset ((s), 0, (n))
+-
+-
+ typedef unsigned char  uch;
+ typedef unsigned short ush;
+ typedef unsigned long  ulg;
+@@ -56,33 +46,7 @@
+ 
+ static unsigned outcnt = 0;  /* bytes in output buffer */
+ 
+-/* gzip flag byte */
+-#define ASCII_FLAG   0x01 /* bit 0 set: file probably ascii text */
+-#define CONTINUATION 0x02 /* bit 1 set: continuation of multi-part gzip file */
+-#define EXTRA_FIELD  0x04 /* bit 2 set: extra field present */
+-#define ORIG_NAME    0x08 /* bit 3 set: original file name present */
+-#define COMMENT      0x10 /* bit 4 set: file comment present */
+-#define ENCRYPTED    0x20 /* bit 5 set: file is encrypted */
+-#define RESERVED     0xC0 /* bit 6,7:   reserved */
+-
+-#define get_byte() inbuf[inptr++]	
+-	
+-/* Diagnostic functions */
+-#ifdef DEBUG
+-#  define Assert(cond,msg) {if(!(cond)) error(msg);}
+-#  define Trace(x) fprintf x
+-#  define Tracev(x) {if (verbose) fprintf x ;}
+-#  define Tracevv(x) {if (verbose>1) fprintf x ;}
+-#  define Tracec(c,x) {if (verbose && (c)) fprintf x ;}
+-#  define Tracecv(c,x) {if (verbose>1 && (c)) fprintf x ;}
+-#else
+-#  define Assert(cond,msg)
+-#  define Trace(x)
+-#  define Tracev(x)
+-#  define Tracevv(x)
+-#  define Tracec(c,x)
+-#  define Tracecv(c,x)
+-#endif
++#define get_byte() inbuf[inptr++]
+ 
+ static int  fill_inbuf(void);
+ static void flush_window(void);
+@@ -166,25 +130,6 @@
+ #endif
+ }
+ 
+-void*
+-memset(void* s, int c, size_t n)
+-{
+-	int i;
+-	char *ss = (char*)s;
+-
+-	for (i=0;i<n;i++) ss[i] = c;
+-}
+-
+-void*
+-memcpy(void* __dest, __const void* __src,
+-			    size_t __n)
+-{
+-	int i;
+-	char *d = (char *)__dest, *s = (char *)__src;
+-
+-	for (i=0;i<__n;i++) d[i] = s[i];
+-}
+-
+ /* ===========================================================================
+  * Write the output window window[0..outcnt-1] and update crc and bytes_out.
+  * (Used for the decompressed data only.)
+Index: linux-2.6.16.19/arch/cris/arch-v32/boot/compressed/misc.c
+===================================================================
+--- linux-2.6.16.19.orig/arch/cris/arch-v32/boot/compressed/misc.c	2006-06-23 17:05:11.000000000 -0700
++++ linux-2.6.16.19/arch/cris/arch-v32/boot/compressed/misc.c	2006-06-23 17:06:31.000000000 -0700
+@@ -31,16 +31,6 @@
+  * gzip declarations
+  */
+ 
+-#define OF(args)  args
+-#define STATIC static
+-
+-void* memset(void* s, int c, size_t n);
+-void* memcpy(void* __dest, __const void* __src,
+-	     size_t __n);
+-
+-#define memzero(s, n)     memset ((s), 0, (n))
+-
+-
+ typedef unsigned char  uch;
+ typedef unsigned short ush;
+ typedef unsigned long  ulg;
+@@ -58,34 +48,8 @@
+ 
+ static unsigned outcnt = 0;  /* bytes in output buffer */
+ 
+-/* gzip flag byte */
+-#define ASCII_FLAG   0x01 /* bit 0 set: file probably ascii text */
+-#define CONTINUATION 0x02 /* bit 1 set: continuation of multi-part gzip file */
+-#define EXTRA_FIELD  0x04 /* bit 2 set: extra field present */
+-#define ORIG_NAME    0x08 /* bit 3 set: original file name present */
+-#define COMMENT      0x10 /* bit 4 set: file comment present */
+-#define ENCRYPTED    0x20 /* bit 5 set: file is encrypted */
+-#define RESERVED     0xC0 /* bit 6,7:   reserved */
+-
+ #define get_byte() inbuf[inptr++]
+ 
+-/* Diagnostic functions */
+-#ifdef DEBUG
+-#  define Assert(cond,msg) {if(!(cond)) error(msg);}
+-#  define Trace(x) fprintf x
+-#  define Tracev(x) {if (verbose) fprintf x ;}
+-#  define Tracevv(x) {if (verbose>1) fprintf x ;}
+-#  define Tracec(c,x) {if (verbose && (c)) fprintf x ;}
+-#  define Tracecv(c,x) {if (verbose>1 && (c)) fprintf x ;}
+-#else
+-#  define Assert(cond,msg)
+-#  define Trace(x)
+-#  define Tracev(x)
+-#  define Tracevv(x)
+-#  define Tracec(c,x)
+-#  define Tracecv(c,x)
+-#endif
+-
+ static int  fill_inbuf(void);
+ static void flush_window(void);
+ static void error(char *m);
+@@ -180,25 +144,6 @@
+ #endif
+ }
+ 
+-void*
+-memset(void* s, int c, size_t n)
+-{
+-	int i;
+-	char *ss = (char*)s;
+-
+-	for (i=0;i<n;i++) ss[i] = c;
+-}
+-
+-void*
+-memcpy(void* __dest, __const void* __src,
+-			    size_t __n)
+-{
+-	int i;
+-	char *d = (char *)__dest, *s = (char *)__src;
+-
+-	for (i=0;i<__n;i++) d[i] = s[i];
+-}
+-
+ /* ===========================================================================
+  * Write the output window window[0..outcnt-1] and update crc and bytes_out.
+  * (Used for the decompressed data only.)
+Index: linux-2.6.16.19/arch/i386/boot/compressed/misc.c
+===================================================================
+--- linux-2.6.16.19.orig/arch/i386/boot/compressed/misc.c	2006-06-23 17:05:11.000000000 -0700
++++ linux-2.6.16.19/arch/i386/boot/compressed/misc.c	2006-06-23 17:06:31.000000000 -0700
+@@ -19,21 +19,11 @@
+  * gzip declarations
+  */
+ 
+-#define OF(args)  args
+-#define STATIC static
+-
+-#undef memset
+-#undef memcpy
+-
+ /*
+  * Why do we do this? Don't ask me..
+  *
+  * Incomprehensible are the ways of bootloaders.
+  */
+-static void* memset(void *, int, size_t);
+-static void* memcpy(void *, __const void *, size_t);
+-#define memzero(s, n)     memset ((s), 0, (n))
+-
+ typedef unsigned char  uch;
+ typedef unsigned short ush;
+ typedef unsigned long  ulg;
+@@ -48,33 +38,7 @@
+ static unsigned inptr = 0;   /* index of next byte to be processed in inbuf */
+ static unsigned outcnt = 0;  /* bytes in output buffer */
+ 
+-/* gzip flag byte */
+-#define ASCII_FLAG   0x01 /* bit 0 set: file probably ASCII text */
+-#define CONTINUATION 0x02 /* bit 1 set: continuation of multi-part gzip file */
+-#define EXTRA_FIELD  0x04 /* bit 2 set: extra field present */
+-#define ORIG_NAME    0x08 /* bit 3 set: original file name present */
+-#define COMMENT      0x10 /* bit 4 set: file comment present */
+-#define ENCRYPTED    0x20 /* bit 5 set: file is encrypted */
+-#define RESERVED     0xC0 /* bit 6,7:   reserved */
+-
+ #define get_byte()  (inptr < insize ? inbuf[inptr++] : fill_inbuf())
+-		
+-/* Diagnostic functions */
+-#ifdef DEBUG
+-#  define Assert(cond,msg) {if(!(cond)) error(msg);}
+-#  define Trace(x) fprintf x
+-#  define Tracev(x) {if (verbose) fprintf x ;}
+-#  define Tracevv(x) {if (verbose>1) fprintf x ;}
+-#  define Tracec(c,x) {if (verbose && (c)) fprintf x ;}
+-#  define Tracecv(c,x) {if (verbose>1 && (c)) fprintf x ;}
+-#else
+-#  define Assert(cond,msg)
+-#  define Trace(x)
+-#  define Tracev(x)
+-#  define Tracevv(x)
+-#  define Tracec(c,x)
+-#  define Tracecv(c,x)
+-#endif
+ 
+ static int  fill_inbuf(void);
+ static void flush_window(void);
+@@ -163,7 +127,9 @@
+ {
+ 	int i;
+ 
+-	memcpy ( vidmem, vidmem + cols * 2, ( lines - 1 ) * cols * 2 );
++	for (i = 0; i < (lines - 1) * cols * 2; i++)
++		vidmem[i] = vidmem[i + cols * 2];
++
+ 	for ( i = ( lines - 1 ) * cols * 2; i < lines * cols * 2; i += 2 )
+ 		vidmem[i] = ' ';
+ }
+@@ -205,25 +171,6 @@
+ 	outb_p(0xff & (pos >> 1), vidport+1);
+ }
+ 
+-static void* memset(void* s, int c, size_t n)
+-{
+-	int i;
+-	char *ss = (char*)s;
+-
+-	for (i=0;i<n;i++) ss[i] = c;
+-	return s;
+-}
+-
+-static void* memcpy(void* __dest, __const void* __src,
+-			    size_t __n)
+-{
+-	int i;
+-	char *d = (char *)__dest, *s = (char *)__src;
+-
+-	for (i=0;i<__n;i++) d[i] = s[i];
+-	return __dest;
+-}
+-
+ /* ===========================================================================
+  * Fill the input buffer. This is called only when the buffer is empty
+  * and at least one byte is really needed.
+Index: linux-2.6.16.19/arch/m32r/boot/compressed/misc.c
+===================================================================
+--- linux-2.6.16.19.orig/arch/m32r/boot/compressed/misc.c	2006-06-23 17:05:11.000000000 -0700
++++ linux-2.6.16.19/arch/m32r/boot/compressed/misc.c	2006-06-23 17:06:31.000000000 -0700
+@@ -19,13 +19,6 @@
+  * gzip declarations
+  */
+ 
+-#define OF(args)  args
+-#define STATIC static
+-
+-#undef memset
+-#undef memcpy
+-#define memzero(s, n)     memset ((s), 0, (n))
+-
+ typedef unsigned char  uch;
+ typedef unsigned short ush;
+ typedef unsigned long  ulg;
+@@ -40,34 +33,8 @@
+ static unsigned inptr = 0;   /* index of next byte to be processed in inbuf */
+ static unsigned outcnt = 0;  /* bytes in output buffer */
+ 
+-/* gzip flag byte */
+-#define ASCII_FLAG   0x01 /* bit 0 set: file probably ASCII text */
+-#define CONTINUATION 0x02 /* bit 1 set: continuation of multi-part gzip file */
+-#define EXTRA_FIELD  0x04 /* bit 2 set: extra field present */
+-#define ORIG_NAME    0x08 /* bit 3 set: original file name present */
+-#define COMMENT      0x10 /* bit 4 set: file comment present */
+-#define ENCRYPTED    0x20 /* bit 5 set: file is encrypted */
+-#define RESERVED     0xC0 /* bit 6,7:   reserved */
+-
+ #define get_byte()  (inptr < insize ? inbuf[inptr++] : fill_inbuf())
+ 
+-/* Diagnostic functions */
+-#ifdef DEBUG
+-#  define Assert(cond,msg) {if(!(cond)) error(msg);}
+-#  define Trace(x) fprintf x
+-#  define Tracev(x) {if (verbose) fprintf x ;}
+-#  define Tracevv(x) {if (verbose>1) fprintf x ;}
+-#  define Tracec(c,x) {if (verbose && (c)) fprintf x ;}
+-#  define Tracecv(c,x) {if (verbose>1 && (c)) fprintf x ;}
+-#else
+-#  define Assert(cond,msg)
+-#  define Trace(x)
+-#  define Tracev(x)
+-#  define Tracevv(x)
+-#  define Tracec(c,x)
+-#  define Tracecv(c,x)
+-#endif
+-
+ static int  fill_inbuf(void);
+ static void flush_window(void);
+ static void error(char *m);
+@@ -125,25 +92,6 @@
+ 	free_mem_ptr = (long) *ptr;
+ }
+ 
+-void* memset(void* s, int c, size_t n)
+-{
+-	int i;
+-	char *ss = (char*)s;
+-
+-	for (i=0;i<n;i++) ss[i] = c;
+-	return s;
+-}
+-
+-void* memcpy(void* __dest, __const void* __src,
+-			    size_t __n)
+-{
+-	int i;
+-	char *d = (char *)__dest, *s = (char *)__src;
+-
+-	for (i=0;i<__n;i++) d[i] = s[i];
+-	return __dest;
+-}
+-
+ /* ===========================================================================
+  * Fill the input buffer. This is called only when the buffer is empty
+  * and at least one byte is really needed.
+Index: linux-2.6.16.19/arch/sh/boot/compressed/misc.c
+===================================================================
+--- linux-2.6.16.19.orig/arch/sh/boot/compressed/misc.c	2006-06-23 17:05:11.000000000 -0700
++++ linux-2.6.16.19/arch/sh/boot/compressed/misc.c	2006-06-23 17:06:31.000000000 -0700
+@@ -21,13 +21,6 @@
+  * gzip declarations
+  */
+ 
+-#define OF(args)  args
+-#define STATIC static
+-
+-#undef memset
+-#undef memcpy
+-#define memzero(s, n)     memset ((s), 0, (n))
+-
+ typedef unsigned char  uch;
+ typedef unsigned short ush;
+ typedef unsigned long  ulg;
+@@ -42,34 +35,8 @@
+ static unsigned inptr = 0;   /* index of next byte to be processed in inbuf */
+ static unsigned outcnt = 0;  /* bytes in output buffer */
+ 
+-/* gzip flag byte */
+-#define ASCII_FLAG   0x01 /* bit 0 set: file probably ASCII text */
+-#define CONTINUATION 0x02 /* bit 1 set: continuation of multi-part gzip file */
+-#define EXTRA_FIELD  0x04 /* bit 2 set: extra field present */
+-#define ORIG_NAME    0x08 /* bit 3 set: original file name present */
+-#define COMMENT      0x10 /* bit 4 set: file comment present */
+-#define ENCRYPTED    0x20 /* bit 5 set: file is encrypted */
+-#define RESERVED     0xC0 /* bit 6,7:   reserved */
+-
+ #define get_byte()  (inptr < insize ? inbuf[inptr++] : fill_inbuf())
+ 
+-/* Diagnostic functions */
+-#ifdef DEBUG
+-#  define Assert(cond,msg) {if(!(cond)) error(msg);}
+-#  define Trace(x) fprintf x
+-#  define Tracev(x) {if (verbose) fprintf x ;}
+-#  define Tracevv(x) {if (verbose>1) fprintf x ;}
+-#  define Tracec(c,x) {if (verbose && (c)) fprintf x ;}
+-#  define Tracecv(c,x) {if (verbose>1 && (c)) fprintf x ;}
+-#else
+-#  define Assert(cond,msg)
+-#  define Trace(x)
+-#  define Tracev(x)
+-#  define Tracevv(x)
+-#  define Tracec(c,x)
+-#  define Tracecv(c,x)
+-#endif
+-
+ static int  fill_inbuf(void);
+ static void flush_window(void);
+ static void error(char *m);
+@@ -156,25 +123,6 @@
+ }
+ #endif
+ 
+-void* memset(void* s, int c, size_t n)
+-{
+-	int i;
+-	char *ss = (char*)s;
+-
+-	for (i=0;i<n;i++) ss[i] = c;
+-	return s;
+-}
+-
+-void* memcpy(void* __dest, __const void* __src,
+-			    size_t __n)
+-{
+-	int i;
+-	char *d = (char *)__dest, *s = (char *)__src;
+-
+-	for (i=0;i<__n;i++) d[i] = s[i];
+-	return __dest;
+-}
+-
+ /* ===========================================================================
+  * Fill the input buffer. This is called only when the buffer is empty
+  * and at least one byte is really needed.
+Index: linux-2.6.16.19/arch/sh64/boot/compressed/misc.c
+===================================================================
+--- linux-2.6.16.19.orig/arch/sh64/boot/compressed/misc.c	2006-06-23 17:05:11.000000000 -0700
++++ linux-2.6.16.19/arch/sh64/boot/compressed/misc.c	2006-06-23 17:06:31.000000000 -0700
+@@ -21,13 +21,6 @@
+  * gzip declarations
+  */
+ 
+-#define OF(args)  args
+-#define STATIC static
+-
+-#undef memset
+-#undef memcpy
+-#define memzero(s, n)     memset ((s), 0, (n))
+-
+ typedef unsigned char uch;
+ typedef unsigned short ush;
+ typedef unsigned long ulg;
+@@ -42,34 +35,8 @@
+ static unsigned inptr = 0;	/* index of next byte to be processed in inbuf */
+ static unsigned outcnt = 0;	/* bytes in output buffer */
+ 
+-/* gzip flag byte */
+-#define ASCII_FLAG   0x01	/* bit 0 set: file probably ASCII text */
+-#define CONTINUATION 0x02	/* bit 1 set: continuation of multi-part gzip file */
+-#define EXTRA_FIELD  0x04	/* bit 2 set: extra field present */
+-#define ORIG_NAME    0x08	/* bit 3 set: original file name present */
+-#define COMMENT      0x10	/* bit 4 set: file comment present */
+-#define ENCRYPTED    0x20	/* bit 5 set: file is encrypted */
+-#define RESERVED     0xC0	/* bit 6,7:   reserved */
+-
+ #define get_byte()  (inptr < insize ? inbuf[inptr++] : fill_inbuf())
+ 
+-/* Diagnostic functions */
+-#ifdef DEBUG
+-#  define Assert(cond,msg) {if(!(cond)) error(msg);}
+-#  define Trace(x) fprintf x
+-#  define Tracev(x) {if (verbose) fprintf x ;}
+-#  define Tracevv(x) {if (verbose>1) fprintf x ;}
+-#  define Tracec(c,x) {if (verbose && (c)) fprintf x ;}
+-#  define Tracecv(c,x) {if (verbose>1 && (c)) fprintf x ;}
+-#else
+-#  define Assert(cond,msg)
+-#  define Trace(x)
+-#  define Tracev(x)
+-#  define Tracevv(x)
+-#  define Tracec(c,x)
+-#  define Tracecv(c,x)
+-#endif
+-
+ static int fill_inbuf(void);
+ static void flush_window(void);
+ static void error(char *m);
+@@ -138,26 +105,6 @@
+ {
+ }
+ 
+-void *memset(void *s, int c, size_t n)
+-{
+-	int i;
+-	char *ss = (char *) s;
+-
+-	for (i = 0; i < n; i++)
+-		ss[i] = c;
+-	return s;
+-}
+-
+-void *memcpy(void *__dest, __const void *__src, size_t __n)
+-{
+-	int i;
+-	char *d = (char *) __dest, *s = (char *) __src;
+-
+-	for (i = 0; i < __n; i++)
+-		d[i] = s[i];
+-	return __dest;
+-}
+-
+ /* ===========================================================================
+  * Fill the input buffer. This is called only when the buffer is empty
+  * and at least one byte is really needed.
+Index: linux-2.6.16.19/arch/x86_64/boot/compressed/misc.c
+===================================================================
+--- linux-2.6.16.19.orig/arch/x86_64/boot/compressed/misc.c	2006-06-23 17:06:11.000000000 -0700
++++ linux-2.6.16.19/arch/x86_64/boot/compressed/misc.c	2006-06-23 17:06:31.000000000 -0700
+@@ -18,13 +18,6 @@
+  * gzip declarations
+  */
+ 
+-#define OF(args)  args
+-#define STATIC static
+-
+-#undef memset
+-#undef memcpy
+-#define memzero(s, n)     memset ((s), 0, (n))
+-
+ typedef unsigned char  uch;
+ typedef unsigned short ush;
+ typedef unsigned long  ulg;
+@@ -39,33 +32,7 @@
+ static unsigned inptr = 0;   /* index of next byte to be processed in inbuf */
+ static unsigned outcnt = 0;  /* bytes in output buffer */
+ 
+-/* gzip flag byte */
+-#define ASCII_FLAG   0x01 /* bit 0 set: file probably ASCII text */
+-#define CONTINUATION 0x02 /* bit 1 set: continuation of multi-part gzip file */
+-#define EXTRA_FIELD  0x04 /* bit 2 set: extra field present */
+-#define ORIG_NAME    0x08 /* bit 3 set: original file name present */
+-#define COMMENT      0x10 /* bit 4 set: file comment present */
+-#define ENCRYPTED    0x20 /* bit 5 set: file is encrypted */
+-#define RESERVED     0xC0 /* bit 6,7:   reserved */
+-
+ #define get_byte()  (inptr < insize ? inbuf[inptr++] : fill_inbuf())
+-		
+-/* Diagnostic functions */
+-#ifdef DEBUG
+-#  define Assert(cond,msg) {if(!(cond)) error(msg);}
+-#  define Trace(x) fprintf x
+-#  define Tracev(x) {if (verbose) fprintf x ;}
+-#  define Tracevv(x) {if (verbose>1) fprintf x ;}
+-#  define Tracec(c,x) {if (verbose && (c)) fprintf x ;}
+-#  define Tracecv(c,x) {if (verbose>1 && (c)) fprintf x ;}
+-#else
+-#  define Assert(cond,msg)
+-#  define Trace(x)
+-#  define Tracev(x)
+-#  define Tracevv(x)
+-#  define Tracec(c,x)
+-#  define Tracecv(c,x)
+-#endif
+ 
+ static int  fill_inbuf(void);
+ static void flush_window(void);
+@@ -93,9 +60,6 @@
+ 
+ static void *malloc(int size);
+ static void free(void *where);
+- 
+-void* memset(void* s, int c, unsigned n);
+-void* memcpy(void* dest, const void* src, unsigned n);
+ 
+ static void putstr(const char *);
+ 
+@@ -153,7 +117,9 @@
+ {
+ 	int i;
+ 
+-	memcpy ( vidmem, vidmem + cols * 2, ( lines - 1 ) * cols * 2 );
++	for (i = 0; i < (lines - 1) * cols * 2; i++)
++		vidmem[i] = vidmem[i + cols * 2];
++
+ 	for ( i = ( lines - 1 ) * cols * 2; i < lines * cols * 2; i += 2 )
+ 		vidmem[i] = ' ';
+ }
+@@ -195,24 +161,6 @@
+ 	outb_p(0xff & (pos >> 1), vidport+1);
+ }
+ 
+-void* memset(void* s, int c, unsigned n)
+-{
+-	int i;
+-	char *ss = (char*)s;
+-
+-	for (i=0;i<n;i++) ss[i] = c;
+-	return s;
+-}
+-
+-void* memcpy(void* dest, const void* src, unsigned n)
+-{
+-	int i;
+-	char *d = (char *)dest, *s = (char *)src;
+-
+-	for (i=0;i<n;i++) d[i] = s[i];
+-	return dest;
+-}
+-
+ /* ===========================================================================
+  * Fill the input buffer. This is called only when the buffer is empty
+  * and at least one byte is really needed.
+Index: linux-2.6.16.19/init/do_mounts_rd.c
+===================================================================
+--- linux-2.6.16.19.orig/init/do_mounts_rd.c	2006-06-23 17:05:11.000000000 -0700
++++ linux-2.6.16.19/init/do_mounts_rd.c	2006-06-23 17:06:31.000000000 -0700
+@@ -273,12 +273,6 @@
+  * gzip declarations
+  */
+ 
+-#define OF(args)  args
+-
+-#ifndef memzero
+-#define memzero(s, n)     memset ((s), 0, (n))
+-#endif
+-
+ typedef unsigned char  uch;
+ typedef unsigned short ush;
+ typedef unsigned long  ulg;
+@@ -299,16 +293,7 @@
+ static int crd_infd, crd_outfd;
+ 
+ #define get_byte()  (inptr < insize ? inbuf[inptr++] : fill_inbuf())
+-		
+-/* Diagnostic functions (stubbed out) */
+-#define Assert(cond,msg)
+-#define Trace(x)
+-#define Tracev(x)
+-#define Tracevv(x)
+-#define Tracec(c,x)
+-#define Tracecv(c,x)
+ 
+-#define STATIC static
+ #define INIT __init
+ 
+ static int  __init fill_inbuf(void);
+Index: linux-2.6.16.19/init/initramfs.c
+===================================================================
+--- linux-2.6.16.19.orig/init/initramfs.c	2006-06-23 17:05:11.000000000 -0700
++++ linux-2.6.16.19/init/initramfs.c	2006-06-23 17:06:31.000000000 -0700
+@@ -343,12 +343,6 @@
+  * gzip declarations
+  */
+ 
+-#define OF(args)  args
+-
+-#ifndef memzero
+-#define memzero(s, n)     memset ((s), 0, (n))
+-#endif
+-
+ typedef unsigned char  uch;
+ typedef unsigned short ush;
+ typedef unsigned long  ulg;
+@@ -365,16 +359,7 @@
+ static long bytes_out;
+ 
+ #define get_byte()  (inptr < insize ? inbuf[inptr++] : -1)
+-		
+-/* Diagnostic functions (stubbed out) */
+-#define Assert(cond,msg)
+-#define Trace(x)
+-#define Tracev(x)
+-#define Tracevv(x)
+-#define Tracec(c,x)
+-#define Tracecv(c,x)
+ 
+-#define STATIC static
+ #define INIT __init
+ 
+ static void __init flush_window(void);
Index: packages/kernel/patches/tiny_m57sli/tiny/tsc.patch
===================================================================
--- packages/kernel/patches/tiny_m57sli/tiny/tsc.patch	(revision 0)
+++ packages/kernel/patches/tiny_m57sli/tiny/tsc.patch	(revision 0)
@@ -0,0 +1,121 @@
+
+configurable TSC timer support
+
+From: Zwane Mwaikambo <zwane@arm.linux.org.uk>
+Subject: Re: [PATCH][2.6-tiny] optional timer_tsc
+
+
+I'm special;
+
+Index: linux-2.6.1-rc1-tiny2/init/Kconfig===================================================================
+RCS file: /home/cvsroot/linux-2.6.1-rc1-tiny2/init/Kconfig,v
+retrieving revision 1.3
+diff -u -p -B -r1.3 Kconfig
+
+
+ tiny-mpm/arch/i386/kernel/timers/Makefile    |    3 ++-
+ tiny-mpm/arch/i386/kernel/timers/timer.c     |   16 ++++++++++++++--
+ tiny-mpm/arch/i386/kernel/timers/timer_tsc.c |    5 ++---
+ tiny-mpm/init/Kconfig                        |    7 +++++++
+ 4 files changed, 25 insertions(+), 6 deletions(-)
+
+Index: linux-2.6.16.19/arch/i386/kernel/timers/Makefile
+===================================================================
+--- linux-2.6.16.19.orig/arch/i386/kernel/timers/Makefile	2006-06-23 17:05:11.000000000 -0700
++++ linux-2.6.16.19/arch/i386/kernel/timers/Makefile	2006-06-23 17:06:28.000000000 -0700
+@@ -2,8 +2,9 @@
+ # Makefile for x86 timers
+ #
+ 
+-obj-y := timer.o timer_none.o timer_tsc.o timer_pit.o common.o
++obj-y := timer.o timer_none.o timer_pit.o common.o
+ 
++obj-$(CONFIG_X86_TSC_TIMER)	+= timer_tsc.o
+ obj-$(CONFIG_X86_CYCLONE_TIMER)	+= timer_cyclone.o
+ obj-$(CONFIG_HPET_TIMER)	+= timer_hpet.o
+ obj-$(CONFIG_X86_PM_TIMER)	+= timer_pm.o
+Index: linux-2.6.16.19/arch/i386/kernel/timers/timer.c
+===================================================================
+--- linux-2.6.16.19.orig/arch/i386/kernel/timers/timer.c	2006-06-23 17:05:11.000000000 -0700
++++ linux-2.6.16.19/arch/i386/kernel/timers/timer.c	2006-06-23 17:06:28.000000000 -0700
+@@ -1,16 +1,17 @@
+ #include <linux/init.h>
+ #include <linux/kernel.h>
+ #include <linux/string.h>
++#include <linux/jiffies.h>
++#include <linux/timex.h>
+ #include <asm/timer.h>
+ 
+-#ifdef CONFIG_HPET_TIMER
+ /*
+  * HPET memory read is slower than tsc reads, but is more dependable as it
+  * always runs at constant frequency and reduces complexity due to
+  * cpufreq. So, we prefer HPET timer to tsc based one. Also, we cannot use
+  * timer_pit when HPET is active. So, we default to timer_tsc.
+  */
+-#endif
++
+ /* list of timers, ordered by preference, NULL terminated */
+ static struct init_timer_opts* __initdata timers[] = {
+ #ifdef CONFIG_X86_CYCLONE_TIMER
+@@ -22,11 +23,15 @@
+ #ifdef CONFIG_X86_PM_TIMER
+ 	&timer_pmtmr_init,
+ #endif
++#ifdef CONFIG_X86_TSC_TIMER
+ 	&timer_tsc_init,
++#endif
+ 	&timer_pit_init,
+ 	NULL,
+ };
+ 
++int tsc_disable __devinitdata = 0;
++
+ static char clock_override[10] __initdata;
+ 
+ static int __init clock_setup(char* str)
+@@ -73,3 +78,10 @@
+ 	}
+ 	return -1;
+ }
++
++#ifndef CONFIG_X86_TSC_TIMER
++unsigned long long sched_clock(void)
++{
++	return (unsigned long long)jiffies_64 * (1000000000 / HZ);
++}
++#endif
+Index: linux-2.6.16.19/arch/i386/kernel/timers/timer_tsc.c
+===================================================================
+--- linux-2.6.16.19.orig/arch/i386/kernel/timers/timer_tsc.c	2006-06-23 17:05:11.000000000 -0700
++++ linux-2.6.16.19/arch/i386/kernel/timers/timer_tsc.c	2006-06-23 17:06:28.000000000 -0700
+@@ -34,9 +34,9 @@
+ 
+ static inline void cpufreq_delayed_get(void);
+ 
+-int tsc_disable __devinitdata = 0;
++extern int tsc_disable;
+ 
+-static int use_tsc;
++int use_tsc;
+ /* Number of usecs that the last interrupt was delayed */
+ static int delay_at_last_interrupt;
+ 
+Index: linux-2.6.16.19/init/Kconfig
+===================================================================
+--- linux-2.6.16.19.orig/init/Kconfig	2006-06-23 17:06:28.000000000 -0700
++++ linux-2.6.16.19/init/Kconfig	2006-06-23 17:06:28.000000000 -0700
+@@ -628,6 +628,13 @@
+ 	  Disable this only if you know your target machine is unaffected
+ 	  by PCI quirks.
+ 
++config X86_TSC_TIMER
++	depends X86
++	default y
++	bool "Enable TSC timer" if EMBEDDED
++	help
++	  This enables support for the Time Stamp Counter timer.
++
+ endmenu		# General setup
+ 
+ config CRC32_CALC
Index: packages/kernel/patches/tiny_m57sli/tiny/movsl-mask.patch
===================================================================
--- packages/kernel/patches/tiny_m57sli/tiny/movsl-mask.patch	(revision 0)
+++ packages/kernel/patches/tiny_m57sli/tiny/movsl-mask.patch	(revision 0)
@@ -0,0 +1,17 @@
+---
+ arch/i386/lib/usercopy.c |    2 	1 +	1 -	0 !
+ 1 files changed, 1 insertion(+), 1 deletion(-)
+
+Index: linux-2.6.22.1/arch/i386/lib/usercopy.c
+===================================================================
+--- linux-2.6.22.1.orig/arch/i386/lib/usercopy.c
++++ linux-2.6.22.1/arch/i386/lib/usercopy.c
+@@ -16,7 +16,7 @@
+ 
+ static inline int __movsl_is_ok(unsigned long a1, unsigned long a2, unsigned long n)
+ {
+-#ifdef CONFIG_X86_INTEL_USERCOPY
++#if defined(CONFIG_CPU_SUP_INTEL) && defined(CONFIG_X86_INTEL_USERCOPY)
+ 	if (n >= 64 && ((a1 ^ a2) & movsl_mask.mask))
+ 		return 0;
+ #endif
Index: packages/kernel/patches/tiny_m57sli/tiny/inflate-linkage.patch
===================================================================
--- packages/kernel/patches/tiny_m57sli/tiny/inflate-linkage.patch	(revision 0)
+++ packages/kernel/patches/tiny_m57sli/tiny/inflate-linkage.patch	(revision 0)
@@ -0,0 +1,633 @@
+inflate: remove include of lib/inflate.c and use proper linking
+
+- make free_mem_ptr vars nonstatic
+- make gunzip nonstatic
+- add gunzip prototype to new inflate.h
+- add per-arch Makefile bits
+- change inflate.c includes to inflate.h includes
+- change NO_INFLATE_MALLOC to CORE
+- compile core kernel version of inflate with -DCORE
+
+Signed-off-by: Matt Mackall <mpm@selenic.com>
+
+Index: alp-linux/arch/alpha/boot/misc.c
+===================================================================
+--- alp-linux.orig/arch/alpha/boot/misc.c	2006-07-12 15:25:19.000000000 -0700
++++ alp-linux/arch/alpha/boot/misc.c	2006-07-12 15:25:19.000000000 -0700
+@@ -14,6 +14,7 @@
+  */
+ 
+ #include <linux/kernel.h>
++#include <linux/inflate.h>
+ #include <asm/uaccess.h>
+ 
+ #define puts		srm_printk
+@@ -27,11 +28,7 @@
+ 
+ #define HEAP_SIZE 0x2000
+ 
+-/* gzip delarations */
+-static u32 free_mem_ptr;
+-static u32 free_mem_ptr_end;
+-
+-#include "../../../lib/inflate.c"
++static u32 free_mem_ptr, free_mem_ptr_end; /* for gunzip */
+ 
+ /* flush gunzip output window */
+ static void flush_window(const u8 *buf, int len)
+Index: alp-linux/arch/arm/boot/compressed/Makefile
+===================================================================
+--- alp-linux.orig/arch/arm/boot/compressed/Makefile	2006-07-12 15:20:35.000000000 -0700
++++ alp-linux/arch/arm/boot/compressed/Makefile	2006-07-12 15:25:19.000000000 -0700
+@@ -99,8 +99,11 @@
+ # would otherwise mess up our GOT table
+ CFLAGS_misc.o := -Dstatic=
+ 
++$(obj)/inflate.o: lib/inflate.c
++	$(call cmd,cc_o_c)
++
+ $(obj)/vmlinux: $(obj)/vmlinux.lds $(obj)/$(HEAD) $(obj)/piggy.o \
+-	 	$(addprefix $(obj)/, $(OBJS)) FORCE
++	 	$(addprefix $(obj)/, $(OBJS)) $(obj)/inflate.o FORCE
+ 	$(call if_changed,ld)
+ 	@:
+ 
+Index: alp-linux/arch/arm/boot/compressed/misc.c
+===================================================================
+--- alp-linux.orig/arch/arm/boot/compressed/misc.c	2006-07-12 15:25:19.000000000 -0700
++++ alp-linux/arch/arm/boot/compressed/misc.c	2006-07-12 16:26:39.000000000 -0700
+@@ -14,6 +14,7 @@
+ unsigned int __machine_arch_type;
+ 
+ #include <linux/string.h>
++#include <linux/inflate.h>
+ #include <asm/arch/uncompress.h>
+ 
+ #ifdef STANDALONE_DEBUG
+@@ -36,31 +37,17 @@
+ 
+ #endif
+ 
+-#define __ptr_t void *
+-
+-/*
+- * gzip declarations
+- */
+-
+ extern char input_data[];
+ extern char input_data_end[];
++extern int end;
+ 
+ static u8 *output_data;
+-
+ static void putstr(const char *);
+ 
+-extern int end;
+-static u32 free_mem_ptr;
+-static u32 free_mem_ptr_end;
++u32 free_mem_ptr, free_mem_end_ptr; /* for gunzip */
+ 
+ #define HEAP_SIZE 0x2000
+ 
+-#include "../../../../lib/inflate.c"
+-
+-#ifdef STANDALONE_DEBUG
+-#define NO_INFLATE_MALLOC
+-#endif
+-
+ /* flush gunzip output window */
+ static void flush_window(const u8 *buf, int len)
+ {
+@@ -87,14 +74,14 @@
+ #ifndef STANDALONE_DEBUG
+ 
+ u32
+-decompress_kernel(u32 output_start, u32 free_mem_ptr_p, u32 free_mem_ptr_end_p,
++decompress_kernel(u32 output_start, u32 free_mem_ptr_p, u32 free_mem_end_ptr_p,
+ 		  int arch_id)
+ {
+ 	int count;
+ 
+ 	output_data		= (u8 *)output_start;	/* Points to kernel start */
+ 	free_mem_ptr		= free_mem_ptr_p;
+-	free_mem_ptr_end	= free_mem_ptr_end_p;
++	free_mem_end_ptr	= free_mem_end_ptr_p;
+ 	__machine_arch_type	= arch_id;
+ 
+ 	arch_decomp_setup();
+Index: alp-linux/arch/arm26/boot/compressed/Makefile
+===================================================================
+--- alp-linux.orig/arch/arm26/boot/compressed/Makefile	2006-07-12 15:20:35.000000000 -0700
++++ alp-linux/arch/arm26/boot/compressed/Makefile	2006-07-12 15:25:19.000000000 -0700
+@@ -8,7 +8,7 @@
+ #
+ 
+ HEAD	= head.o
+-OBJS	= misc.o
++OBJS	= misc.o inflate.o
+ FONTC	= drivers/video/console/font_acorn_8x8.c
+ 
+ OBJS		+= ll_char_wr.o font.o
+@@ -40,11 +40,14 @@
+ $(obj)/piggy.o:  $(obj)/piggy.gz FORCE
+ 	$(call if_changed,ld)
+ 
++$(obj)/inflate.o: lib/inflate.c
++        $(call cmd,cc_o_c)
++
+ $(obj)/font.o: $(FONTC)
+ 	$(CC) $(CFLAGS) -Dstatic= -c $(FONTC) -o $(obj)/font.o
+ 
+ $(obj)/vmlinux.lds: $(obj)/vmlinux.lds.in Makefile arch/arm26/boot/Makefile .config
+ 	@sed "$(SEDFLAGS)" < $< > $@
+ 
+-$(obj)/misc.o: $(obj)/misc.c $(obj)/uncompress.h lib/inflate.c
++$(obj)/misc.o: $(obj)/misc.c $(obj)/uncompress.h
+ 
+Index: alp-linux/arch/arm26/boot/compressed/misc.c
+===================================================================
+--- alp-linux.orig/arch/arm26/boot/compressed/misc.c	2006-07-12 15:25:19.000000000 -0700
++++ alp-linux/arch/arm26/boot/compressed/misc.c	2006-07-12 15:25:19.000000000 -0700
+@@ -14,6 +14,7 @@
+ unsigned int __machine_arch_type;
+ 
+ #include <linux/kernel.h>
++#include <linux/inflate.h>
+ #include <asm/uaccess.h>
+ #include "uncompress.h"
+ 
+@@ -30,15 +31,7 @@
+ 
+ #define HEAP_SIZE 0x2000
+ 
+-/* gzip delarations */
+-static u32 free_mem_ptr;
+-static u32 free_mem_ptr_end;
+-
+-#include "../../../../lib/inflate.c"
+-
+-#ifdef STANDALONE_DEBUG
+-#define NO_INFLATE_MALLOC
+-#endif
++u32 free_mem_ptr, free_mem_ptr_end; /* for gunzip */
+ 
+ /* flush the gunzip output window */
+ static void flush_window(const u8 *buf, int len)
+Index: alp-linux/arch/cris/arch-v10/boot/compressed/Makefile
+===================================================================
+--- alp-linux.orig/arch/cris/arch-v10/boot/compressed/Makefile	2006-07-12 15:20:35.000000000 -0700
++++ alp-linux/arch/cris/arch-v10/boot/compressed/Makefile	2006-07-12 15:25:19.000000000 -0700
+@@ -9,7 +9,7 @@
+ LD = ld-cris
+ OBJCOPY = objcopy-cris
+ OBJCOPYFLAGS = -O binary --remove-section=.bss
+-OBJECTS = $(target)/head.o $(target)/misc.o
++OBJECTS = $(target)/head.o $(target)/misc.o $(target)/inflate.o
+ 
+ # files to compress
+ SYSTEM = $(objtree)/vmlinux.bin
+@@ -29,6 +29,9 @@
+ $(target)/head.o: $(src)/head.S
+ 	$(CC) -D__ASSEMBLY__ -traditional -c $< -o $@
+ 
++$(target)/inflate.o: lib/inflate.c
++	$(CC) -D__KERNEL__ -c $< -o $@
++
+ $(target)/misc.o: $(src)/misc.c
+ 	$(CC) -D__KERNEL__ -c $< -o $@
+ 
+Index: alp-linux/arch/cris/arch-v10/boot/compressed/misc.c
+===================================================================
+--- alp-linux.orig/arch/cris/arch-v10/boot/compressed/misc.c	2006-07-12 15:25:19.000000000 -0700
++++ alp-linux/arch/cris/arch-v10/boot/compressed/misc.c	2006-07-12 15:25:19.000000000 -0700
+@@ -16,6 +16,7 @@
+ 
+ #include <linux/config.h>
+ #include <linux/types.h>
++#include <linux/inflate.h>
+ #include <asm/arch/svinto.h>
+ 
+ extern int end; /* the "heap" is put directly after the BSS ends, at end */
+@@ -25,11 +26,8 @@
+ 
+ static void puts(const char *);
+ 
+-/* gzip declarations */
+-static long free_mem_ptr = (long)&end;
+-static long free_mem_end_ptr = 0xffffffff;
+-
+-#include "../../../../../lib/inflate.c"
++/* for gunzip */
++long free_mem_ptr = (long)&end, free_mem_end_ptr = 0xffffffff;
+ 
+ /* decompressor info and error messages to serial console */
+ 
+Index: alp-linux/arch/cris/arch-v32/boot/compressed/Makefile
+===================================================================
+--- alp-linux.orig/arch/cris/arch-v32/boot/compressed/Makefile	2006-07-12 15:20:35.000000000 -0700
++++ alp-linux/arch/cris/arch-v32/boot/compressed/Makefile	2006-07-12 15:25:19.000000000 -0700
+@@ -12,7 +12,7 @@
+ LD = gcc-cris -mlinux -march=v32 -nostdlib
+ OBJCOPY = objcopy-cris
+ OBJCOPYFLAGS = -O binary --remove-section=.bss
+-OBJECTS = $(target)/head.o $(target)/misc.o
++OBJECTS = $(target)/head.o $(target)/misc.o $(target)/inflate.o
+ 
+ # files to compress
+ SYSTEM = $(objtree)/vmlinux.bin
+@@ -28,6 +28,9 @@
+ 	rm -f piggy.img
+ 	cp $(objtree)/vmlinuz $(src)
+ 
++$(target)/inflate.o: lib/inflate.c
++	$(call cmd,cc_o_c)
++
+ $(target)/head.o: $(src)/head.S
+ 	$(CC) -D__ASSEMBLY__ -c $< -o $@
+ 
+Index: alp-linux/arch/cris/arch-v32/boot/compressed/misc.c
+===================================================================
+--- alp-linux.orig/arch/cris/arch-v32/boot/compressed/misc.c	2006-07-12 15:25:19.000000000 -0700
++++ alp-linux/arch/cris/arch-v32/boot/compressed/misc.c	2006-07-12 15:25:19.000000000 -0700
+@@ -16,6 +16,7 @@
+ 
+ #include <linux/config.h>
+ #include <linux/types.h>
++#include <linux/inflate.h>
+ #include <asm/arch/hwregs/reg_rdwr.h>
+ #include <asm/arch/hwregs/reg_map.h>
+ #include <asm/arch/hwregs/ser_defs.h>
+@@ -26,11 +27,8 @@
+ 
+ static void puts(const char *);
+ 
+-/* gzip declarations */
+-static long free_mem_ptr = (long)&_end;
+-static long free_mem_end_ptr = 0xffffffff;
+-
+-#include "../../../../../lib/inflate.c"
++/* for gunzip */
++long free_mem_ptr = (long)&_end, free_mem_end_ptr = 0xffffffff;
+ 
+ /* decompressor info and error messages to serial console */
+ 
+Index: alp-linux/arch/i386/boot/compressed/Makefile
+===================================================================
+--- alp-linux.orig/arch/i386/boot/compressed/Makefile	2006-07-12 15:20:35.000000000 -0700
++++ alp-linux/arch/i386/boot/compressed/Makefile	2006-07-12 15:25:19.000000000 -0700
+@@ -12,7 +12,11 @@
+ 
+ LDFLAGS_vmlinux := -Ttext $(IMAGE_OFFSET) -e startup_32
+ 
+-$(obj)/vmlinux: $(obj)/head.o $(obj)/misc.o $(obj)/piggy.o FORCE
++$(obj)/inflate.o: lib/inflate.c
++	$(call cmd,cc_o_c)
++
++$(obj)/vmlinux: $(obj)/head.o $(obj)/misc.o $(obj)/piggy.o $(obj)/inflate.o \
++		FORCE
+ 	$(call if_changed,ld)
+ 	@:
+ 
+Index: alp-linux/arch/i386/boot/compressed/misc.c
+===================================================================
+--- alp-linux.orig/arch/i386/boot/compressed/misc.c	2006-07-12 15:25:19.000000000 -0700
++++ alp-linux/arch/i386/boot/compressed/misc.c	2006-07-12 15:25:19.000000000 -0700
+@@ -8,6 +8,7 @@
+ #include <linux/linkage.h>
+ #include <linux/vmalloc.h>
+ #include <linux/screen_info.h>
++#include <linux/inflate.h>
+ #include <asm/io.h>
+ #include <asm/page.h>
+ 
+@@ -46,11 +47,7 @@
+ static void * xquad_portio = NULL;
+ #endif
+ 
+-/* gzip declarations */
+-static long free_mem_ptr = (long)&end;
+-static long free_mem_end_ptr;
+-
+-#include "../../../../lib/inflate.c"
++long free_mem_ptr = (long)&end, free_mem_end_ptr; /* for gunzip */
+ 
+ static void scroll(void)
+ {
+Index: alp-linux/arch/m32r/boot/compressed/Makefile
+===================================================================
+--- alp-linux.orig/arch/m32r/boot/compressed/Makefile	2006-07-12 15:20:35.000000000 -0700
++++ alp-linux/arch/m32r/boot/compressed/Makefile	2006-07-12 15:25:19.000000000 -0700
+@@ -18,7 +18,11 @@
+ 
+ LDFLAGS_vmlinux := -T
+ 
+-$(obj)/vmlinux: $(obj)/vmlinux.lds $(OBJECTS) $(obj)/piggy.o FORCE
++$(obj)/inflate.o: lib/inflate.c
++	$(call cmd,cc_o_c)
++
++$(obj)/vmlinux: $(obj)/vmlinux.lds $(OBJECTS) $(obj)/piggy.o $(obj)/inflate.o \
++		FORCE
+ 	$(call if_changed,ld)
+ 	@:
+ 
+Index: alp-linux/arch/m32r/boot/compressed/misc.c
+===================================================================
+--- alp-linux.orig/arch/m32r/boot/compressed/misc.c	2006-07-12 15:25:19.000000000 -0700
++++ alp-linux/arch/m32r/boot/compressed/misc.c	2006-07-12 15:25:19.000000000 -0700
+@@ -9,6 +9,7 @@
+ 
+ #include <linux/config.h>
+ #include <linux/string.h>
++#include <linux/inflate.h>
+ 
+ static unsigned char *input_data;
+ static int input_len;
+@@ -18,11 +19,7 @@
+ 
+ #define HEAP_SIZE             0x10000
+ 
+-/* gzip declarations */
+-static unsigned long free_mem_ptr;
+-static unsigned long free_mem_end_ptr;
+-
+-#include "../../../../lib/inflate.c"
++unsigned long free_mem_ptr, free_mem_end_ptr; /* for gunzip */
+ 
+ /* flush the gunzip output buffer */
+ static void flush_window(const u8 *buf, int len)
+Index: alp-linux/arch/sh/boot/compressed/Makefile
+===================================================================
+--- alp-linux.orig/arch/sh/boot/compressed/Makefile	2006-07-12 15:20:35.000000000 -0700
++++ alp-linux/arch/sh/boot/compressed/Makefile	2006-07-12 15:25:19.000000000 -0700
+@@ -27,7 +27,10 @@
+ 
+ LDFLAGS_vmlinux := -Ttext $(IMAGE_OFFSET) -e startup -T $(obj)/../../kernel/vmlinux.lds
+ 
+-$(obj)/vmlinux: $(OBJECTS) $(obj)/piggy.o FORCE
++$(obj)/inflate.o: lib/inflate.c
++	$(call cmd,cc_o_c)
++
++$(obj)/vmlinux: $(OBJECTS) $(obj)/piggy.o $(obj)/inflate.o FORCE
+ 	$(call if_changed,ld)
+ 	@:
+ 
+Index: alp-linux/arch/sh/boot/compressed/misc.c
+===================================================================
+--- alp-linux.orig/arch/sh/boot/compressed/misc.c	2006-07-12 15:25:19.000000000 -0700
++++ alp-linux/arch/sh/boot/compressed/misc.c	2006-07-12 15:25:19.000000000 -0700
+@@ -7,6 +7,7 @@
+  */
+ 
+ #include <linux/config.h>
++#include <linux/inflate.h>
+ #include <asm/uaccess.h>
+ #ifdef CONFIG_SH_STANDARD_BIOS
+ #include <asm/sh_bios.h>
+@@ -22,9 +23,7 @@
+ 
+ #define HEAP_SIZE             0x10000
+ 
+-/* gzip declarations */
+-static unsigned long free_mem_ptr;
+-static unsigned long free_mem_end_ptr;
++unsigned long free_mem_ptr, free_mem_end_ptr; /* for gunzip */
+ 
+ #include "../../../../lib/inflate.c"
+ 
+Index: alp-linux/arch/sh64/boot/compressed/Makefile
+===================================================================
+--- alp-linux.orig/arch/sh64/boot/compressed/Makefile	2006-07-12 15:20:35.000000000 -0700
++++ alp-linux/arch/sh64/boot/compressed/Makefile	2006-07-12 15:25:19.000000000 -0700
+@@ -28,7 +28,10 @@
+ 		    -T $(obj)/../../kernel/vmlinux.lds \
+ 		    --no-warn-mismatch
+ 
+-$(obj)/vmlinux: $(OBJECTS) $(obj)/piggy.o FORCE
++$(obj)/inflate.o: lib/inflate.c
++	$(call cmd,cc_o_c)
++
++$(obj)/vmlinux: $(OBJECTS) $(obj)/piggy.o $(obj)/inflate.o FORCE
+ 	$(call if_changed,ld)
+ 	@:
+ 
+Index: alp-linux/arch/sh64/boot/compressed/misc.c
+===================================================================
+--- alp-linux.orig/arch/sh64/boot/compressed/misc.c	2006-07-12 15:25:19.000000000 -0700
++++ alp-linux/arch/sh64/boot/compressed/misc.c	2006-07-12 15:25:19.000000000 -0700
+@@ -5,6 +5,7 @@
+  */
+ 
+ #include <linux/config.h>
++#include <linux/inflate.h>
+ #include <asm/uaccess.h>
+ 
+ /* cache.c */
+@@ -22,11 +23,7 @@
+ 
+ #define HEAP_SIZE             0x10000
+ 
+-/* gzip declarations */
+-static unsigned long free_mem_ptr;
+-static unsigned long free_mem_end_ptr;
+-
+-#include "../../../../lib/inflate.c"
++unsigned long free_mem_ptr, free_mem_end_ptr; /* for gunzip */
+ 
+ void puts(const char *s)
+ {
+Index: alp-linux/arch/x86_64/boot/compressed/Makefile
+===================================================================
+--- alp-linux.orig/arch/x86_64/boot/compressed/Makefile	2006-07-12 15:20:35.000000000 -0700
++++ alp-linux/arch/x86_64/boot/compressed/Makefile	2006-07-12 15:25:19.000000000 -0700
+@@ -16,7 +16,11 @@
+ 
+ LDFLAGS_vmlinux := -Ttext $(IMAGE_OFFSET) -e startup_32 -m elf_i386
+ 
+-$(obj)/vmlinux: $(obj)/head.o $(obj)/misc.o $(obj)/piggy.o FORCE
++$(obj)/inflate.o: lib/inflate.c
++	$(call cmd,cc_o_c)
++
++$(obj)/vmlinux: $(obj)/head.o $(obj)/misc.o $(obj)/piggy.o $(obj)/inflate.o \
++		FORCE
+ 	$(call if_changed,ld)
+ 	@:
+ 
+Index: alp-linux/arch/x86_64/boot/compressed/misc.c
+===================================================================
+--- alp-linux.orig/arch/x86_64/boot/compressed/misc.c	2006-07-12 15:25:19.000000000 -0700
++++ alp-linux/arch/x86_64/boot/compressed/misc.c	2006-07-12 15:25:19.000000000 -0700
+@@ -6,6 +6,7 @@
+  */
+ 
+ #include <linux/screen_info.h>
++#include <linux/inflate.h>
+ #include <asm/io.h>
+ #include <asm/page.h>
+ 
+@@ -39,11 +40,7 @@
+ static int vidport;
+ static int lines, cols;
+ 
+-/* gzip declarations */
+-static long free_mem_ptr = (long)&end;
+-static long free_mem_end_ptr;
+-
+-#include "../../../../lib/inflate.c"
++long free_mem_ptr = (long)&end, free_mem_end_ptr; /* for gunzip */
+ 
+ static void scroll(void)
+ {
+Index: alp-linux/include/linux/inflate.h
+===================================================================
+--- /dev/null	1970-01-01 00:00:00.000000000 +0000
++++ alp-linux/include/linux/inflate.h	2006-07-12 15:25:19.000000000 -0700
+@@ -0,0 +1,9 @@
++#ifndef _LINUX_INFLATE_H
++#define _LINUX_INFLATE_H
++
++int gunzip(u8 *ibuf, int isize, void (*fill)(u8 *buf, int size),
++	   void (*flush)(const u8 *buf, int size),
++	   void (*error)(const char *msg));
++
++#endif /* _LINUX_INFLATE_H */
++
+Index: alp-linux/init/do_mounts_rd.c
+===================================================================
+--- alp-linux.orig/init/do_mounts_rd.c	2006-07-12 15:25:19.000000000 -0700
++++ alp-linux/init/do_mounts_rd.c	2006-07-12 15:25:19.000000000 -0700
+@@ -1,4 +1,3 @@
+-
+ #include <linux/kernel.h>
+ #include <linux/fs.h>
+ #include <linux/minix_fs.h>
+@@ -8,6 +7,7 @@
+ #include <linux/squashfs_fs.h>
+ #include <linux/initrd.h>
+ #include <linux/string.h>
++#include <linux/inflate.h>
+ 
+ #include "do_mounts.h"
+ 
+@@ -282,13 +282,6 @@
+ 
+ #ifdef BUILD_CRAMDISK
+ 
+-/* gzip declarations */
+-#define INIT __init
+-#define INITDATA __initdata
+-#define NO_INFLATE_MALLOC
+-
+-#include "../lib/inflate.c"
+-
+ #define INBUFSIZ 4096
+ static u8 *inbuf;
+ static int exit_code;
+Index: alp-linux/init/initramfs.c
+===================================================================
+--- alp-linux.orig/init/initramfs.c	2006-07-12 15:25:19.000000000 -0700
++++ alp-linux/init/initramfs.c	2006-07-12 15:25:19.000000000 -0700
+@@ -6,6 +6,7 @@
+ #include <linux/delay.h>
+ #include <linux/string.h>
+ #include <linux/syscalls.h>
++#include <linux/inflate.h>
+ 
+ static __initdata const char *message;
+ static void __init error(const char *x)
+@@ -329,14 +330,6 @@
+ 	}
+ }
+ 
+-/* gzip declarations */
+-
+-#define INIT __init
+-#define INITDATA __initdata
+-#define NO_INFLATE_MALLOC
+-
+-#include "../lib/inflate.c"
+-
+ static const char * __init unpack_to_rootfs(char *buf, unsigned len,
+ 					    int check_only)
+ {
+Index: alp-linux/lib/Makefile
+===================================================================
+--- alp-linux.orig/lib/Makefile	2006-07-12 15:20:35.000000000 -0700
++++ alp-linux/lib/Makefile	2006-07-12 15:25:19.000000000 -0700
+@@ -5,12 +5,14 @@
+ lib-y := errno.o ctype.o string.o vsprintf.o cmdline.o \
+ 	 bust_spinlocks.o rbtree.o radix-tree.o dump_stack.o \
+ 	 idr.o div64.o int_sqrt.o bitmap.o extable.o prio_tree.o \
+-	 sha1.o
++	 sha1.o inflate.o
+ 
+ lib-y	+= kobject.o kref.o kobject_uevent.o klist.o
+ 
+ obj-y += sort.o parser.o halfmd4.o iomap_copy.o
+ 
++CFLAGS_inflate.o += -DCORE
++
+ ifeq ($(CONFIG_DEBUG_KOBJECT),y)
+ CFLAGS_kobject.o += -DDEBUG
+ CFLAGS_kobject_uevent.o += -DDEBUG
+Index: alp-linux/lib/inflate.c
+===================================================================
+--- alp-linux.orig/lib/inflate.c	2006-07-12 15:25:19.000000000 -0700
++++ alp-linux/lib/inflate.c	2006-07-12 16:24:31.000000000 -0700
+@@ -99,22 +99,19 @@
+       a repeat code (16, 17, or 18) to go across the boundary between
+       the two sets of lengths.
+  */
++#include <linux/kernel.h>
+ #include <linux/compiler.h>
++#include <asm/types.h>
+ 
+-#ifndef INIT
++#ifndef CORE
+ #define INIT
+-#endif
+-#ifndef INITDATA
+ #define INITDATA
+-#endif
+-
+-#include <asm/types.h>
+ 
+-#ifndef NO_INFLATE_MALLOC
+ /* A trivial malloc implementation, adapted from
+  *  malloc by Hannu Savolainen 1993 and Matthias Urlichs 1994
+  */
+ 
++extern long free_mem_ptr, free_mem_end_ptr;
+ static unsigned long malloc_ptr;
+ static int malloc_count;
+ 
+@@ -148,10 +145,17 @@
+ }
+ 
+ static u8 INITDATA window[0x8000]; /* use a statically allocated window */
++
+ #else
++
++#include <linux/module.h>
++
+ static u8 *window; /* dynamically allocate */
+ #define malloc(a) kmalloc(a, GFP_KERNEL)
+ #define free(a) kfree(a)
++#define INIT __init
++#define INITDATA __initdata
++
+ #endif
+ 
+ static u32 crc_32_tab[256];
+@@ -939,9 +943,9 @@
+  * @flush: function to flush the output pool
+  * @error: function to report an error
+  */
+-static int INIT gunzip(u8 *ibuf, int isize, void (*fill)(u8 *buf, int size),
+-		       void (*flush)(const u8 *buf, int size),
+-		       void (*error)(const char *msg))
++int INIT gunzip(u8 *ibuf, int isize, void (*fill)(u8 *buf, int size),
++		void (*flush)(const u8 *buf, int size),
++		void (*error)(const char *msg))
+ {
+ 	u8 flags;
+ 	unsigned char magic[2];	/* magic header */
Index: packages/kernel/patches/tiny_m57sli/tiny/ide-hwif.patch
===================================================================
--- packages/kernel/patches/tiny_m57sli/tiny/ide-hwif.patch	(revision 0)
+++ packages/kernel/patches/tiny_m57sli/tiny/ide-hwif.patch	(revision 0)
@@ -0,0 +1,58 @@
+
+configurable number of supported IDE interfaces
+
+
+ tiny-mpm/drivers/ide/setup-pci.c |    2 +-
+ tiny-mpm/include/linux/ide.h     |    5 +++++
+ tiny-mpm/init/Kconfig            |    9 +++++++++
+ 3 files changed, 15 insertions(+), 1 deletion(-)
+
+Index: linux-2.6.16.19/drivers/ide/setup-pci.c
+===================================================================
+--- linux-2.6.16.19.orig/drivers/ide/setup-pci.c	2006-06-23 17:05:12.000000000 -0700
++++ linux-2.6.16.19/drivers/ide/setup-pci.c	2006-06-23 17:06:26.000000000 -0700
+@@ -102,7 +102,7 @@
+ 				return hwif;	/* pick an unused entry */
+ 		}
+ 	}
+-	for (h = 0; h < 2; ++h) {
++	for (h = 0; h < 2 && h < MAX_HWIFS; ++h) {
+ 		hwif = ide_hwifs + h;
+ 		if (hwif->chipset == ide_unknown)
+ 			return hwif;	/* pick an unused entry */
+Index: linux-2.6.16.19/include/linux/ide.h
+===================================================================
+--- linux-2.6.16.19.orig/include/linux/ide.h	2006-06-23 17:05:12.000000000 -0700
++++ linux-2.6.16.19/include/linux/ide.h	2006-06-23 17:06:26.000000000 -0700
+@@ -299,6 +299,11 @@
+ }
+ #endif /* IDE_ARCH_OBSOLETE_INIT */
+ 
++#if defined(CONFIG_IDE_HWIFS) && CONFIG_IDE_HWIFS > 0
++#undef MAX_HWIFS
++#define MAX_HWIFS CONFIG_IDE_HWIFS
++#endif
++
+ /* Currently only m68k, apus and m8xx need it */
+ #ifndef IDE_ARCH_ACK_INTR
+ # define ide_ack_intr(hwif) (1)
+Index: linux-2.6.16.19/init/Kconfig
+===================================================================
+--- linux-2.6.16.19.orig/init/Kconfig	2006-06-23 17:06:25.000000000 -0700
++++ linux-2.6.16.19/init/Kconfig	2006-06-23 17:06:26.000000000 -0700
+@@ -584,6 +584,15 @@
+ 	help
+ 	  This option allows reducing the size of the O(1) scheduler arrays.
+ 
++config IDE_HWIFS
++	depends IDE
++	int "Number of IDE hardware interfaces (0 for default)" if EMBEDDED
++	range 0 20
++	default 0
++	help
++	  Select the maximum number of IDE interfaces (0 for default).
++          Saves up to 14k.
++
+ endmenu		# General setup
+ 
+ config CRC32_CALC
Index: packages/kernel/patches/tiny_m57sli/tiny/igmp.patch
===================================================================
--- packages/kernel/patches/tiny_m57sli/tiny/igmp.patch	(revision 0)
+++ packages/kernel/patches/tiny_m57sli/tiny/igmp.patch	(revision 0)
@@ -0,0 +1,172 @@
+
+configurable IGMP support
+
+
+ tiny-mpm/include/linux/igmp.h       |   20 ++++++++++++++++++++
+ tiny-mpm/include/linux/mroute.h     |    5 ++++-
+ tiny-mpm/init/Kconfig               |    8 ++++++++
+ tiny-mpm/net/ipv4/Makefile          |    3 ++-
+ tiny-mpm/net/ipv4/af_inet.c         |    2 --
+ tiny-mpm/net/ipv4/ip_sockglue.c     |    4 ++++
+ tiny-mpm/net/ipv4/sysctl_net_ipv4.c |    2 ++
+ 7 files changed, 40 insertions(+), 4 deletions(-)
+
+Index: linux-2.6.16.19/include/linux/igmp.h
+===================================================================
+--- linux-2.6.16.19.orig/include/linux/igmp.h	2006-06-23 17:05:12.000000000 -0700
++++ linux-2.6.16.19/include/linux/igmp.h	2006-06-23 17:06:25.000000000 -0700
+@@ -197,6 +197,7 @@
+ #define IGMPV3_QQIC(value) IGMPV3_EXP(0x80, 4, 3, value)
+ #define IGMPV3_MRC(value) IGMPV3_EXP(0x80, 4, 3, value)
+ 
++#ifdef CONFIG_IGMP
+ extern int ip_check_mc(struct in_device *dev, u32 mc_addr, u32 src_addr, u16 proto);
+ extern int igmp_rcv(struct sk_buff *);
+ extern int ip_mc_join_group(struct sock *sk, struct ip_mreqn *imr);
+@@ -217,5 +218,24 @@
+ extern void ip_mc_down(struct in_device *);
+ extern void ip_mc_dec_group(struct in_device *in_dev, u32 addr);
+ extern void ip_mc_inc_group(struct in_device *in_dev, u32 addr);
++#else /* !CONFIG_IGMP */
++#define ip_check_mc(a, b, c, d) (0)
++#define igmp_rcv(a) (0)
++#define ip_mc_join_group(a, b) (0)
++#define ip_mc_leave_group(a, b) (0)
++#define ip_mc_drop_socket(a)
++#define ip_mc_source(a, b, c, d, e) (0)
++#define ip_mc_msfilter(a, b, c) (0)
++#define ip_mc_msfget(a, b, c, d) (0)
++#define ip_mc_gsfget(a, b, c, d) (0)
++#define ip_mc_sf_allow(a, b, c, d) (0)
++#define ip_mc_init_dev(a)
++#define ip_mc_destroy_dev(a)
++#define ip_mc_up(a)
++#define ip_mc_down(a)
++#define ip_mc_dec_group(a, b)
++#define ip_mc_inc_group(a, b)
++#endif /* !CONFIG_IGMP */
++
+ #endif
+ #endif
+Index: linux-2.6.16.19/include/linux/mroute.h
+===================================================================
+--- linux-2.6.16.19.orig/include/linux/mroute.h	2006-06-23 17:05:12.000000000 -0700
++++ linux-2.6.16.19/include/linux/mroute.h	2006-06-23 17:06:25.000000000 -0700
+@@ -131,7 +131,11 @@
+ extern int ip_mroute_setsockopt(struct sock *, int, char __user *, int);
+ extern int ip_mroute_getsockopt(struct sock *, int, char __user *, int __user *);
+ extern int ipmr_ioctl(struct sock *sk, int cmd, void __user *arg);
++#ifdef CONFIG_IGMP
+ extern void ip_mr_init(void);
++#else
++static inline void ip_mr_init(void) {}
++#endif
+ 
+ 
+ struct vif_device
+Index: linux-2.6.16.19/init/Kconfig
+===================================================================
+--- linux-2.6.16.19.orig/init/Kconfig	2006-06-23 17:06:25.000000000 -0700
++++ linux-2.6.16.19/init/Kconfig	2006-06-23 17:06:25.000000000 -0700
+@@ -291,6 +291,14 @@
+ 	help
+ 	  This enables the core device multicast list management code.
+ 
++config IGMP
++	depends NET
++	default y
++	bool "Enable IGMP support" if EMBEDDED && !IP_MULTICAST
++	help
++	  This includes support for the Internet group management
++	  protocol, used for multicast.
++
+ config MEASURE_INLINES
+ 	default n
+ 	bool "Enable inline measurement" if EMBEDDED
+Index: linux-2.6.16.19/net/ipv4/Makefile
+===================================================================
+--- linux-2.6.16.19.orig/net/ipv4/Makefile	2006-06-23 17:06:22.000000000 -0700
++++ linux-2.6.16.19/net/ipv4/Makefile	2006-06-23 17:06:25.000000000 -0700
+@@ -8,11 +8,12 @@
+ 	     inet_timewait_sock.o inet_connection_sock.o \
+ 	     tcp.o tcp_input.o tcp_output.o tcp_timer.o tcp_ipv4.o \
+ 	     tcp_minisocks.o tcp_cong.o \
+-	     datagram.o raw.o udp.o arp.o icmp.o devinet.o af_inet.o igmp.o \
++	     datagram.o raw.o udp.o arp.o icmp.o devinet.o af_inet.o \
+ 	     sysctl_net_ipv4.o fib_frontend.o fib_semantics.o
+ 
+ obj-$(CONFIG_IP_FIB_HASH) += fib_hash.o
+ obj-$(CONFIG_IP_FIB_TRIE) += fib_trie.o
++obj-$(CONFIG_IGMP) += igmp.o
+ obj-$(CONFIG_INETPEER) += inetpeer.o
+ obj-$(CONFIG_PROC_FS) += proc.o
+ obj-$(CONFIG_IP_MULTIPLE_TABLES) += fib_rules.o
+Index: linux-2.6.16.19/net/ipv4/af_inet.c
+===================================================================
+--- linux-2.6.16.19.orig/net/ipv4/af_inet.c	2006-06-23 17:05:12.000000000 -0700
++++ linux-2.6.16.19/net/ipv4/af_inet.c	2006-06-23 17:06:25.000000000 -0700
+@@ -117,8 +117,6 @@
+ 
+ DEFINE_SNMP_STAT(struct linux_mib, net_statistics) __read_mostly;
+ 
+-extern void ip_mc_drop_socket(struct sock *sk);
+-
+ /* The inetsw table contains everything that inet_create needs to
+  * build a new socket.
+  */
+Index: linux-2.6.16.19/net/ipv4/ip_sockglue.c
+===================================================================
+--- linux-2.6.16.19.orig/net/ipv4/ip_sockglue.c	2006-06-23 17:05:12.000000000 -0700
++++ linux-2.6.16.19/net/ipv4/ip_sockglue.c	2006-06-23 17:06:25.000000000 -0700
+@@ -610,6 +610,7 @@
+ 				err = ip_mc_leave_group(sk, &mreq);
+ 			break;
+ 		}
++#ifdef CONFIG_IGMP
+ 		case IP_MSFILTER:
+ 		{
+ 			extern int sysctl_igmp_max_msf;
+@@ -647,6 +648,7 @@
+ 			kfree(msf);
+ 			break;
+ 		}
++#endif
+ 		case IP_BLOCK_SOURCE:
+ 		case IP_UNBLOCK_SOURCE:
+ 		case IP_ADD_SOURCE_MEMBERSHIP:
+@@ -764,6 +766,7 @@
+ 				greqs.gsr_interface);
+ 			break;
+ 		}
++#ifdef CONFIG_IGMP
+ 		case MCAST_MSFILTER:
+ 		{
+ 			extern int sysctl_igmp_max_msf;
+@@ -830,6 +833,7 @@
+ 			kfree(gsf);
+ 			break;
+ 		}
++#endif
+ 		case IP_ROUTER_ALERT:	
+ 			err = ip_ra_control(sk, val ? 1 : 0, NULL);
+ 			break;
+Index: linux-2.6.16.19/net/ipv4/sysctl_net_ipv4.c
+===================================================================
+--- linux-2.6.16.19.orig/net/ipv4/sysctl_net_ipv4.c	2006-06-23 17:06:22.000000000 -0700
++++ linux-2.6.16.19/net/ipv4/sysctl_net_ipv4.c	2006-06-23 17:06:25.000000000 -0700
+@@ -435,6 +435,7 @@
+ 
+ #endif
+ #ifdef CONFIG_INETPEER
++#ifdef CONFIG_IGMP
+ 	{
+ 		.ctl_name	= NET_IPV4_IGMP_MAX_MSF,
+ 		.procname	= "igmp_max_msf",
+@@ -443,6 +444,7 @@
+ 		.mode		= 0644,
+ 		.proc_handler	= &proc_dointvec
+ 	},
++#endif
+ 	{
+ 		.ctl_name	= NET_IPV4_INET_PEER_THRESHOLD,
+ 		.procname	= "inet_peer_threshold",
Index: packages/kernel/patches/tiny_m57sli/tiny/inflate-roll-crc.patch
===================================================================
--- packages/kernel/patches/tiny_m57sli/tiny/inflate-roll-crc.patch	(revision 0)
+++ packages/kernel/patches/tiny_m57sli/tiny/inflate-roll-crc.patch	(revision 0)
@@ -0,0 +1,46 @@
+inflate: roll CRC calculation
+
+If CONFIG_CRC32_TABLES is set, do tableless CRC calculation
+
+ tiny-mpm/lib/inflate.c |   55 +++++++++++++++++++++++++++++++++----------------
+ 1 files changed, 38 insertions(+), 17 deletions(-)
+
+Index: linux-2.6.16.19/lib/inflate.c
+===================================================================
+--- linux-2.6.16.19.orig/lib/inflate.c	2006-06-23 17:06:37.000000000 -0700
++++ linux-2.6.16.19/lib/inflate.c	2006-06-23 17:06:37.000000000 -0700
+@@ -147,8 +147,9 @@
+ static u8 window[0x8000]; /* use a statically allocated window */
+ 
+ /* simple CRC calculation */
+-static u32 crc_32_tab[256];
+ #define CRCPOLY_LE 0xedb88320
++#ifdef CONFIG_CRC32_TABLES
++static u32 crc_32_tab[256];
+ 
+ static void makecrc(void)
+ {
+@@ -169,6 +170,23 @@
+ 
+ 	return crc;
+ }
++#else
++
++#define makecrc()
++
++static u32 crc32_le(u32 crc, unsigned char const *p, size_t len)
++{
++	int i;
++
++	while (len--) {
++		crc ^= *p++;
++		for (i = 0; i < 8; i++)
++			crc = (crc >> 1) ^ ((crc & 1) ? CRCPOLY_LE : 0);
++	}
++
++	return crc;
++}
++#endif /* crc tables */
+ 
+ #else
+ 
Index: packages/kernel/patches/tiny_m57sli/tiny/config-net-small.patch
===================================================================
--- packages/kernel/patches/tiny_m57sli/tiny/config-net-small.patch	(revision 0)
+++ packages/kernel/patches/tiny_m57sli/tiny/config-net-small.patch	(revision 0)
@@ -0,0 +1,26 @@
+
+Add CONFIG_NET_SMALL for miscellaneous networking tweaks
+
+
+
+ init/Kconfig |    7 	7 +	0 -	0 !
+ 1 files changed, 7 insertions(+)
+
+Index: linux-2.6.22.1/init/Kconfig
+===================================================================
+--- linux-2.6.22.1.orig/init/Kconfig
++++ linux-2.6.22.1/init/Kconfig
+@@ -466,6 +466,13 @@ config BASE_FULL
+ 	  kernel data structures. This saves memory on small machines,
+ 	  but may reduce performance.
+ 
++config NET_SMALL
++	default n
++	bool "Enable various size reductions for networking" if EMBEDDED
++	help
++	  This reduces the size of miscellaneous networking data structures.
++          Currently a placeholder.
++
+ config FUTEX
+ 	bool "Enable futex support" if EMBEDDED
+ 	default y
Index: packages/kernel/patches/tiny_m57sli/tiny/unix_socket_table.patch
===================================================================
--- packages/kernel/patches/tiny_m57sli/tiny/unix_socket_table.patch	(revision 0)
+++ packages/kernel/patches/tiny_m57sli/tiny/unix_socket_table.patch	(revision 0)
@@ -0,0 +1,23 @@
+
+CONFIG_NET_SMALL shrink AF_UNIX socket hash
+
+
+ tiny-mpm/include/net/af_unix.h |    4 ++++
+ 1 files changed, 4 insertions(+)
+
+Index: linux-2.6.16.19/include/net/af_unix.h
+===================================================================
+--- linux-2.6.16.19.orig/include/net/af_unix.h	2006-06-23 17:05:14.000000000 -0700
++++ linux-2.6.16.19/include/net/af_unix.h	2006-06-23 17:06:12.000000000 -0700
+@@ -10,7 +10,11 @@
+ extern void unix_notinflight(struct file *fp);
+ extern void unix_gc(void);
+ 
++#ifdef CONFIG_NET_SMALL
++#define UNIX_HASH_SIZE	16
++#else
+ #define UNIX_HASH_SIZE	256
++#endif
+ 
+ extern struct hlist_head unix_socket_table[UNIX_HASH_SIZE + 1];
+ extern spinlock_t unix_table_lock;
Index: packages/kernel/patches/tiny_m57sli/tiny/mempool-shrink.patch
===================================================================
--- packages/kernel/patches/tiny_m57sli/tiny/mempool-shrink.patch	(revision 0)
+++ packages/kernel/patches/tiny_m57sli/tiny/mempool-shrink.patch	(revision 0)
@@ -0,0 +1,143 @@
+
+configurable tiny mempool
+
+Reduces mempools to a single element managed with xchg
+
+
+ tiny-mpm/include/linux/mempool.h |   19 ++++++++++++++
+ tiny-mpm/init/Kconfig            |   10 +++++++
+ tiny-mpm/mm/mempool.c            |   51 +++++++++++++++++++++++++++++++++++++++
+ 3 files changed, 80 insertions(+)
+
+Index: linux-2.6.16.19/include/linux/mempool.h
+===================================================================
+--- linux-2.6.16.19.orig/include/linux/mempool.h	2006-06-23 17:05:13.000000000 -0700
++++ linux-2.6.16.19/include/linux/mempool.h	2006-06-23 17:06:21.000000000 -0700
+@@ -9,6 +9,7 @@
+ typedef void * (mempool_alloc_t)(gfp_t gfp_mask, void *pool_data);
+ typedef void (mempool_free_t)(void *element, void *pool_data);
+ 
++#ifdef CONFIG_MEMPOOL
+ typedef struct mempool_s {
+ 	spinlock_t lock;
+ 	int min_nr;		/* nr of elements at *elements */
+@@ -31,6 +32,25 @@
+ extern void * mempool_alloc(mempool_t *pool, gfp_t gfp_mask);
+ extern void mempool_free(void *element, mempool_t *pool);
+ 
++#else
++
++typedef struct mempool_s {
++	void *pool_data;
++	void *cache;
++	mempool_alloc_t *alloc;
++	mempool_free_t *free;
++} mempool_t;
++
++extern mempool_t * mempool_create(int min_nr, mempool_alloc_t *alloc_fn,
++				 mempool_free_t *free_fn, void *pool_data);
++extern void * mempool_alloc(mempool_t *pool, int gfp_mask);
++extern void mempool_free(void *element, mempool_t *pool);
++extern void mempool_destroy(mempool_t *pool);
++#define mempool_resize(a, b, c) (0)
++#define mempool_create_node(m, a, f, p, n) mempool_create(m, a, f, p)
++
++#endif
++
+ /*
+  * A mempool_alloc_t and mempool_free_t that get the memory from
+  * a slab that is passed in through pool_data.
+Index: linux-2.6.16.19/init/Kconfig
+===================================================================
+--- linux-2.6.16.19.orig/init/Kconfig	2006-06-23 17:06:19.000000000 -0700
++++ linux-2.6.16.19/init/Kconfig	2006-06-23 17:06:21.000000000 -0700
+@@ -464,6 +464,16 @@
+ 	  Disabling this simplifies the code for finding information
+           about the current thread. Saves about 4k on small kernels.
+ 
++config MEMPOOL
++	default y
++	bool "Use mempool allocator" if EMBEDDED
++	help
++	  The mempool allocator attempts keep a reserve pool of
++	  various objects available for use in low-memory situations.
++	  Disabling this feature may make more memory available and
++	  will somewhat decrease code size, but with a higher risk of
++	  deadlock under some loads.
++
+ endmenu		# General setup
+ 
+ config CRC32_CALC
+Index: linux-2.6.16.19/mm/mempool.c
+===================================================================
+--- linux-2.6.16.19.orig/mm/mempool.c	2006-06-23 17:05:13.000000000 -0700
++++ linux-2.6.16.19/mm/mempool.c	2006-06-23 17:06:21.000000000 -0700
+@@ -15,6 +15,8 @@
+ #include <linux/blkdev.h>
+ #include <linux/writeback.h>
+ 
++#ifdef CONFIG_MEMPOOL
++
+ static void add_element(mempool_t *pool, void *element)
+ {
+ 	BUG_ON(pool->curr_nr >= pool->min_nr);
+@@ -273,6 +275,54 @@
+ }
+ EXPORT_SYMBOL(mempool_free);
+ 
++#else /* !CONFIG_MEMPOOL */
++
++mempool_t *mempool_create(int min_nr, mempool_alloc_t *alloc_fn,
++				 mempool_free_t *free_fn, void *pool_data)
++{
++	mempool_t *m;
++
++	m = (mempool_t *)kmalloc(sizeof(mempool_t), GFP_KERNEL);
++
++	if (m) {
++		m->pool_data = pool_data;
++		m->alloc = alloc_fn;
++		m->free = free_fn;
++		m->cache = alloc_fn(GFP_KERNEL, pool_data);
++	}
++
++	return m;
++}
++EXPORT_SYMBOL(mempool_create);
++
++extern void mempool_destroy(mempool_t *pool)
++{
++	if (pool->cache)
++		pool->free(pool->cache, pool->pool_data);
++	kfree(pool);
++}
++EXPORT_SYMBOL(mempool_destroy);
++
++void * mempool_alloc(mempool_t *pool, int gfp_mask)
++{
++	void *p = pool->alloc(gfp_mask, pool->pool_data);
++	if (!p)
++		p = xchg(&pool->cache, p);
++
++	return p;
++}
++EXPORT_SYMBOL(mempool_alloc);
++
++void mempool_free(void *element, mempool_t *pool)
++{
++	element = xchg(&pool->cache, element);
++	if (element)
++		pool->free(element, pool->pool_data);
++}
++EXPORT_SYMBOL(mempool_free);
++
++#endif /* !CONFIG_MEMPOOL */
++
+ /*
+  * A commonly used alloc and free fn.
+  */
+@@ -289,3 +339,4 @@
+ 	kmem_cache_free(mem, element);
+ }
+ EXPORT_SYMBOL(mempool_free_slab);
++
Index: packages/kernel/patches/tiny_m57sli/tiny/dev_mcast.patch
===================================================================
--- packages/kernel/patches/tiny_m57sli/tiny/dev_mcast.patch	(revision 0)
+++ packages/kernel/patches/tiny_m57sli/tiny/dev_mcast.patch	(revision 0)
@@ -0,0 +1,79 @@
+
+configurable netdev multicast support code
+
+
+ include/linux/netdevice.h |   13 	12 +	1 -	0 !
+ init/Kconfig              |    7 	7 +	0 -	0 !
+ net/core/Makefile         |    3 	2 +	1 -	0 !
+ 3 files changed, 21 insertions(+), 2 deletions(-)
+
+Index: linux-2.6.22.1/include/linux/netdevice.h
+===================================================================
+--- linux-2.6.22.1.orig/include/linux/netdevice.h
++++ linux-2.6.22.1/include/linux/netdevice.h
+@@ -997,17 +997,28 @@ extern struct net_device *alloc_netdev(i
+ extern int		register_netdev(struct net_device *dev);
+ extern void		unregister_netdev(struct net_device *dev);
+ /* Functions used for multicast support */
++#ifdef CONFIG_NET_DEV_MULTICAST
+ extern void		dev_mc_upload(struct net_device *dev);
+ extern int 		dev_mc_delete(struct net_device *dev, void *addr, int alen, int all);
+ extern int		dev_mc_add(struct net_device *dev, void *addr, int alen, int newonly);
+ extern void		dev_mc_discard(struct net_device *dev);
++extern void		dev_mcast_init(void);
++#else
++#define	dev_mc_upload(a)
++static inline int dev_mc_delete(struct net_device *dev,
++				void *addr, int alen, int all) { return 0; }
++static inline int dev_mc_add(struct net_device *dev,
++			     void *addr, int alen, int newonly) { return 0; }
++#define dev_mc_discard(a)
++#define dev_mcast_init()
++#endif
++
+ extern void		dev_set_promiscuity(struct net_device *dev, int inc);
+ extern void		dev_set_allmulti(struct net_device *dev, int inc);
+ extern void		netdev_state_change(struct net_device *dev);
+ extern void		netdev_features_change(struct net_device *dev);
+ /* Load a device via the kmod */
+ extern void		dev_load(const char *name);
+-extern void		dev_mcast_init(void);
+ extern int		netdev_max_backlog;
+ extern int		weight_p;
+ extern int		netdev_set_master(struct net_device *dev, struct net_device *master);
+Index: linux-2.6.22.1/init/Kconfig
+===================================================================
+--- linux-2.6.22.1.orig/init/Kconfig
++++ linux-2.6.22.1/init/Kconfig
+@@ -388,6 +388,13 @@ config INETPEER
+ 	  This allows storing various information about network peers.
+ 	  Disabling saves about 2k.
+ 
++config NET_DEV_MULTICAST
++	default y
++	depends NET
++	bool "Enable device multicast support" if EMBEDDED
++	help
++	  This enables the core device multicast list management code.
++
+ config MEASURE_INLINES
+ 	default n
+ 	bool "Enable inline measurement" if EMBEDDED
+Index: linux-2.6.22.1/net/core/Makefile
+===================================================================
+--- linux-2.6.22.1.orig/net/core/Makefile
++++ linux-2.6.22.1/net/core/Makefile
+@@ -7,7 +7,7 @@ obj-y := sock.o request_sock.o skbuff.o 
+ 
+ obj-$(CONFIG_SYSCTL) += sysctl_net_core.o
+ 
+-obj-y		     += dev.o dev_mcast.o dst.o netevent.o \
++obj-y		     += dev.o dst.o netevent.o \
+ 			neighbour.o rtnetlink.o utils.o link_watch.o filter.o
+ 
+ obj-$(CONFIG_XFRM) += flow.o
+@@ -17,3 +17,4 @@ obj-$(CONFIG_NET_PKTGEN) += pktgen.o
+ obj-$(CONFIG_NETPOLL) += netpoll.o
+ obj-$(CONFIG_NET_DMA) += user_dma.o
+ obj-$(CONFIG_FIB_RULES) += fib_rules.o
++obj-$(CONFIG_NET_DEV_MULTICAST) += dev_mcast.o
Index: packages/kernel/patches/tiny_m57sli/tiny/kmalloc-accounting.patch
===================================================================
--- packages/kernel/patches/tiny_m57sli/tiny/kmalloc-accounting.patch	(revision 0)
+++ packages/kernel/patches/tiny_m57sli/tiny/kmalloc-accounting.patch	(revision 0)
@@ -0,0 +1,371 @@
+
+/proc/kmalloc allocation tracing
+
+
+ tiny-mpm/fs/proc/proc_misc.c  |   21 ++++
+ tiny-mpm/include/linux/slab.h |   19 ++++
+ tiny-mpm/init/Kconfig         |    7 +
+ tiny-mpm/mm/Makefile          |    2 
+ tiny-mpm/mm/kmallocacct.c     |  182 ++++++++++++++++++++++++++++++++++++++++++
+ tiny-mpm/mm/slab.c            |    7 +
+ 6 files changed, 237 insertions(+), 1 deletion(-)
+
+Index: alp-linux/fs/proc/proc_misc.c
+===================================================================
+--- alp-linux.orig/fs/proc/proc_misc.c	2007-01-12 09:30:31.000000000 -0800
++++ alp-linux/fs/proc/proc_misc.c	2007-01-12 17:19:11.000000000 -0800
+@@ -396,6 +396,24 @@
+ };
+ #endif
+ 
++#ifdef CONFIG_KMALLOC_ACCOUNTING
++
++extern struct seq_operations kmalloc_account_op;
++
++static int kmalloc_account_open(struct inode *inode, struct file *file)
++{
++	return seq_open(file, &kmalloc_account_op);
++}
++
++static struct file_operations proc_kmalloc_account_operations = {
++	.open		= kmalloc_account_open,
++	.read		= seq_read,
++	.llseek		= seq_lseek,
++	.release	= seq_release,
++};
++
++#endif
++
+ static int show_stat(struct seq_file *p, void *v)
+ {
+ 	int i;
+@@ -669,6 +687,9 @@
+ #ifdef CONFIG_SLAB
+ 	create_seq_entry("slabinfo",S_IWUSR|S_IRUGO,&proc_slabinfo_operations);
+ #endif
++#ifdef CONFIG_KMALLOC_ACCOUNTING
++	create_seq_entry("kmalloc",S_IRUGO,&proc_kmalloc_account_operations);
++#endif
+ 	create_seq_entry("buddyinfo",S_IRUGO, &fragmentation_file_operations);
+ 	create_seq_entry("vmstat",S_IRUGO, &proc_vmstat_file_operations);
+ 	create_seq_entry("zoneinfo",S_IRUGO, &proc_zoneinfo_file_operations);
+Index: alp-linux/include/linux/slab.h
+===================================================================
+--- alp-linux.orig/include/linux/slab.h	2007-01-12 09:30:31.000000000 -0800
++++ alp-linux/include/linux/slab.h	2007-01-12 09:34:13.000000000 -0800
+@@ -55,6 +55,23 @@
+ 
+ #ifndef CONFIG_SLOB
+ 
++#ifdef CONFIG_KMALLOC_ACCOUNTING
++void __kmalloc_account(const void *, const void *, int, int);
++
++static void inline kmalloc_account(const void *addr, int size, int req)
++{
++	__kmalloc_account(__builtin_return_address(0), addr, size, req);
++}
++
++static void inline kfree_account(const void *addr, int size)
++{
++	__kmalloc_account(__builtin_return_address(0), addr, size, -1);
++}
++#else
++#define kmalloc_account(a, b, c)
++#define kfree_account(a, b)
++#endif
++
+ /* prototypes */
+ extern void __init kmem_cache_init(void);
+ 
+@@ -87,6 +104,7 @@
+ 
+ static inline void *kmalloc(size_t size, gfp_t flags)
+ {
++#ifndef CONFIG_KMALLOC_ACCOUNTING
+ 	if (__builtin_constant_p(size)) {
+ 		int i = 0;
+ #define CACHE(x) \
+@@ -105,6 +123,7 @@
+ 			malloc_sizes[i].cs_dmacachep :
+ 			malloc_sizes[i].cs_cachep, flags);
+ 	}
++#endif
+ 	return __kmalloc(size, flags);
+ }
+ 
+Index: alp-linux/init/Kconfig
+===================================================================
+--- alp-linux.orig/init/Kconfig	2007-01-12 09:30:31.000000000 -0800
++++ alp-linux/init/Kconfig	2007-01-12 17:19:12.000000000 -0800
+@@ -348,6 +348,13 @@
+ 	help
+ 	  Enable support for generating core dumps. Disabling saves about 4k.
+ 
++config KMALLOC_ACCOUNTING
++	default n
++	bool "Enabled accounting of kmalloc/kfree allocations" if EMBEDDED
++	help
++	  This option records kmalloc and kfree activity and reports it via
++	  /proc/kmalloc.
++
+ config BASE_FULL
+ 	default y
+ 	bool "Enable full-sized data structures for core" if EMBEDDED
+Index: alp-linux/mm/Makefile
+===================================================================
+--- alp-linux.orig/mm/Makefile	2007-01-12 09:30:31.000000000 -0800
++++ alp-linux/mm/Makefile	2007-01-12 09:34:22.000000000 -0800
+@@ -12,6 +12,7 @@
+ 			   readahead.o swap.o truncate.o vmscan.o \
+ 			   prio_tree.o util.o $(mmu-y)
+ 
++obj-$(CONFIG_KMALLOC_ACCOUNTING) += kmallocacct.o
+ obj-$(CONFIG_SWAP)	+= page_io.o swap_state.o swapfile.o thrash.o
+ obj-$(CONFIG_HUGETLBFS)	+= hugetlb.o
+ obj-$(CONFIG_NUMA) 	+= mempolicy.o
+Index: alp-linux/mm/kmallocacct.c
+===================================================================
+--- /dev/null	1970-01-01 00:00:00.000000000 +0000
++++ alp-linux/mm/kmallocacct.c	2007-01-12 17:19:12.000000000 -0800
+@@ -0,0 +1,182 @@
++#include	<linux/config.h>
++#include	<linux/seq_file.h>
++#include	<linux/kallsyms.h>
++
++struct kma_caller {
++	const void *caller;
++	int total, net, slack, allocs, frees;
++};
++
++struct kma_list {
++	int callerhash;
++	const void *address;
++};
++
++#define MAX_CALLER_TABLE 512
++#define MAX_ALLOC_TRACK 4096
++
++#define kma_hash(address, size) (((u32)address / (u32)size) % size)
++
++static struct kma_list kma_alloc[MAX_ALLOC_TRACK];
++static struct kma_caller kma_caller[MAX_CALLER_TABLE];
++
++static int kma_callers;
++static int kma_lost_callers, kma_lost_allocs, kma_unknown_frees;
++static int kma_total, kma_net, kma_slack, kma_allocs, kma_frees;
++static spinlock_t kma_lock = SPIN_LOCK_UNLOCKED;
++
++void __kmalloc_account(const void *caller, const void *addr, int size, int req)
++{
++	int i, hasha, hashc;
++	unsigned long flags;
++
++	spin_lock_irqsave(&kma_lock, flags);
++	if(req >= 0) /* kmalloc */
++	{
++		/* find callers slot */
++		hashc = kma_hash(caller, MAX_CALLER_TABLE);
++		for (i = 0; i < MAX_CALLER_TABLE; i++) {
++			if (!kma_caller[hashc].caller ||
++			    kma_caller[hashc].caller == caller)
++				break;
++			hashc = (hashc + 1) % MAX_CALLER_TABLE;
++		}
++
++		if (!kma_caller[hashc].caller)
++			kma_callers++;
++
++		if (i < MAX_CALLER_TABLE) {
++			/* update callers stats */
++			kma_caller[hashc].caller = caller;
++			kma_caller[hashc].total += size;
++			kma_caller[hashc].net += size;
++			kma_caller[hashc].slack += size - req;
++			kma_caller[hashc].allocs++;
++
++			/* add malloc to list */
++			hasha = kma_hash(addr, MAX_ALLOC_TRACK);
++			for (i = 0; i < MAX_ALLOC_TRACK; i++) {
++				if (!kma_alloc[hasha].callerhash)
++					break;
++				hasha = (hasha + 1) % MAX_ALLOC_TRACK;
++			}
++
++			if(i < MAX_ALLOC_TRACK) {
++				kma_alloc[hasha].callerhash = hashc;
++				kma_alloc[hasha].address = addr;
++			}
++			else
++				kma_lost_allocs++;
++		}
++		else {
++			kma_lost_callers++;
++			kma_lost_allocs++;
++		}
++
++		kma_total += size;
++		kma_net += size;
++		kma_slack += size - req;
++		kma_allocs++;
++	}
++	else { /* kfree */
++		hasha = kma_hash(addr, MAX_ALLOC_TRACK);
++		for (i = 0; i < MAX_ALLOC_TRACK ; i++) {
++			if (kma_alloc[hasha].address == addr)
++				break;
++			hasha = (hasha + 1) % MAX_ALLOC_TRACK;
++		}
++
++		if (i < MAX_ALLOC_TRACK) {
++			hashc = kma_alloc[hasha].callerhash;
++			kma_alloc[hasha].callerhash = 0;
++			kma_caller[hashc].net -= size;
++			kma_caller[hashc].frees++;
++		}
++		else
++			kma_unknown_frees++;
++
++		kma_net -= size;
++		kma_frees++;
++	}
++	spin_unlock_irqrestore(&kma_lock, flags);
++}
++
++static void *as_start(struct seq_file *m, loff_t *pos)
++{
++	int i;
++	loff_t n = *pos;
++
++	if (!n) {
++		seq_printf(m, "total bytes allocated: %8d\n", kma_total);
++		seq_printf(m, "slack bytes allocated: %8d\n", kma_slack);
++		seq_printf(m, "net bytes allocated:   %8d\n", kma_net);
++		seq_printf(m, "number of allocs:      %8d\n", kma_allocs);
++		seq_printf(m, "number of frees:       %8d\n", kma_frees);
++		seq_printf(m, "number of callers:     %8d\n", kma_callers);
++		seq_printf(m, "lost callers:          %8d\n",
++			   kma_lost_callers);
++		seq_printf(m, "lost allocs:           %8d\n",
++			   kma_lost_allocs);
++		seq_printf(m, "unknown frees:         %8d\n",
++			   kma_unknown_frees);
++		seq_puts(m, "\n   total    slack      net alloc/free  caller\n");
++	}
++
++	for (i = 0; i < MAX_CALLER_TABLE; i++) {
++		if(kma_caller[i].caller)
++			n--;
++		if(n < 0)
++			return (void *)(i+1);
++	}
++
++	return 0;
++}
++
++static void *as_next(struct seq_file *m, void *p, loff_t *pos)
++{
++	int n = (int)p-1, i;
++	++*pos;
++
++	for (i = n + 1; i < MAX_CALLER_TABLE; i++)
++		if(kma_caller[i].caller)
++			return (void *)(i+1);
++
++	return 0;
++}
++
++static void as_stop(struct seq_file *m, void *p)
++{
++}
++
++static int as_show(struct seq_file *m, void *p)
++{
++	int n = (int)p-1;
++	struct kma_caller *c;
++#ifdef CONFIG_KALLSYMS
++	char *modname;
++	const char *name;
++	unsigned long offset = 0, size;
++	char namebuf[128];
++
++	c = &kma_caller[n];
++	name = kallsyms_lookup((int)c->caller, &size, &offset, &modname,
++			       namebuf);
++	seq_printf(m, "%8d %8d %8d %5d/%-5d %s+0x%lx\n",
++		   c->total, c->slack, c->net, c->allocs, c->frees,
++		   name, offset);
++#else
++	c = &kma_caller[n];
++	seq_printf(m, "%8d %8d %8d %5d/%-5d %p\n",
++		   c->total, c->slack, c->net, c->allocs, c->frees, c->caller);
++#endif
++
++	return 0;
++}
++
++struct seq_operations kmalloc_account_op = {
++	.start	= as_start,
++	.next	= as_next,
++	.stop	= as_stop,
++	.show	= as_show,
++};
++
+Index: alp-linux/mm/slab.c
+===================================================================
+--- alp-linux.orig/mm/slab.c	2007-01-12 09:30:31.000000000 -0800
++++ alp-linux/mm/slab.c	2007-01-12 17:14:31.000000000 -0800
+@@ -3169,16 +3169,43 @@
+ 					  void *caller)
+ {
+ 	struct kmem_cache *cachep;
++	void *a;
+ 
+ 	/* If you want to save a few bytes .text space: replace
+ 	 * __ with kmem_.
+ 	 * Then kmalloc uses the uninlined functions instead of the inline
+ 	 * functions.
+ 	 */
+-	cachep = __find_general_cachep(size, flags);
++
++	/* cachep = __find_general_cachep(size, flags); */
++/* TRB - jan 12, 2007 - move __find_general_cachep from inline to internal for kmalloc accounting */
++	struct cache_sizes *csizep = malloc_sizes;
++
++#if DEBUG
++	/* This happens if someone tries to call
++	 * kmem_cache_create(), or __kmalloc(), before
++	 * the generic caches are initialized.
++	 */
++	BUG_ON(malloc_sizes[INDEX_AC].cs_cachep == NULL);
++#endif
++	while (size > csizep->cs_size)
++		csizep++;
++
++	/*
++	 * Really subtle: The last entry with cs->cs_size==ULONG_MAX
++	 * has cs_{dma,}cachep==NULL. Thus no special case
++	 * for large kmalloc calls required.
++	 */
++	if (unlikely(flags & GFP_DMA)) {
++		cachep = csizep->cs_dmacachep;
++	} else {
++		cachep = csizep->cs_cachep;
++	}
+ 	if (unlikely(cachep == NULL))
+ 		return NULL;
+-	return __cache_alloc(cachep, flags, caller);
++	a = __cache_alloc(cachep, flags, caller);
++	kmalloc_account(a, csizep->cs_size, size);
++	return a;
+ }
+ 
+ #ifndef CONFIG_DEBUG_SLAB
+@@ -3280,6 +3307,8 @@
+ 	struct kmem_cache *c;
+ 	unsigned long flags;
+ 
++	kfree_account(objp, ksize(objp));
++
+ 	if (unlikely(!objp))
+ 		return;
+ 	local_irq_save(flags);
Index: packages/kernel/patches/tiny_m57sli/tiny/inflate-tidyusers.patch
===================================================================
--- packages/kernel/patches/tiny_m57sli/tiny/inflate-tidyusers.patch	(revision 0)
+++ packages/kernel/patches/tiny_m57sli/tiny/inflate-tidyusers.patch	(revision 0)
@@ -0,0 +1,504 @@
+inflate: tidy user declarations
+
+Regroup the few remaining gzip-related declarations in users. As the
+bulk of the users are now _not_ a collection of routines copied from
+gzip, that comment is removed.
+
+Signed-off-by: Matt Mackall <mpm@selenic.com>
+
+Index: linux-2.6.16.19/arch/alpha/boot/misc.c
+===================================================================
+--- linux-2.6.16.19.orig/arch/alpha/boot/misc.c	2006-06-23 17:06:33.000000000 -0700
++++ linux-2.6.16.19/arch/alpha/boot/misc.c	2006-06-23 17:06:33.000000000 -0700
+@@ -1,8 +1,5 @@
+ /*
+  * misc.c
+- * 
+- * This is a collection of several routines from gzip-1.0.3 
+- * adapted for Linux.
+  *
+  * Modified for ARM Linux by Russell King
+  *
+@@ -17,28 +14,23 @@
+  */
+ 
+ #include <linux/kernel.h>
+-
+ #include <asm/uaccess.h>
+ 
+ #define puts		srm_printk
+ extern long srm_printk(const char *, ...)
+      __attribute__ ((format (printf, 1, 2)));
+ 
+-/*
+- * gzip delarations
+- */
+-
++extern int end;
+ static char *input_data;
+ static int  input_data_size;
+-
+ static u8 *output_data;
+ 
+-extern int end;
++#define HEAP_SIZE 0x2000
++
++/* gzip delarations */
+ static u32 free_mem_ptr;
+ static u32 free_mem_ptr_end;
+ 
+-#define HEAP_SIZE 0x2000
+-
+ #include "../../../lib/inflate.c"
+ 
+ /* flush gunzip output window */
+Index: linux-2.6.16.19/arch/arm/boot/compressed/misc.c
+===================================================================
+--- linux-2.6.16.19.orig/arch/arm/boot/compressed/misc.c	2006-06-23 17:06:33.000000000 -0700
++++ linux-2.6.16.19/arch/arm/boot/compressed/misc.c	2006-06-23 17:06:33.000000000 -0700
+@@ -1,8 +1,5 @@
+ /*
+  * misc.c
+- * 
+- * This is a collection of several routines from gzip-1.0.3 
+- * adapted for Linux.
+  *
+  * Modified for ARM Linux by Russell King
+  *
+@@ -17,7 +14,6 @@
+ unsigned int __machine_arch_type;
+ 
+ #include <linux/string.h>
+-
+ #include <asm/arch/uncompress.h>
+ 
+ #ifdef STANDALONE_DEBUG
+Index: linux-2.6.16.19/arch/arm26/boot/compressed/misc.c
+===================================================================
+--- linux-2.6.16.19.orig/arch/arm26/boot/compressed/misc.c	2006-06-23 17:06:33.000000000 -0700
++++ linux-2.6.16.19/arch/arm26/boot/compressed/misc.c	2006-06-23 17:06:33.000000000 -0700
+@@ -1,8 +1,5 @@
+ /*
+  * misc.c
+- * 
+- * This is a collection of several routines from gzip-1.0.3 
+- * adapted for Linux.
+  *
+  * Modified for ARM Linux by Russell King
+  *
+@@ -17,7 +14,6 @@
+ unsigned int __machine_arch_type;
+ 
+ #include <linux/kernel.h>
+-
+ #include <asm/uaccess.h>
+ #include "uncompress.h"
+ 
+@@ -25,24 +21,19 @@
+ #define puts printf
+ #endif
+ 
+-#define __ptr_t void *
+-
+-/*
+- * gzip delarations
+- */
+-
++extern int end;
+ extern char input_data[];
+ extern char input_data_end[];
+ static u8 *output_data;
+ 
+ static void puts(const char *);
+ 
+-extern int end;
++#define HEAP_SIZE 0x2000
++
++/* gzip delarations */
+ static u32 free_mem_ptr;
+ static u32 free_mem_ptr_end;
+ 
+-#define HEAP_SIZE 0x2000
+-
+ #include "../../../../lib/inflate.c"
+ 
+ #ifdef STANDALONE_DEBUG
+Index: linux-2.6.16.19/arch/cris/arch-v10/boot/compressed/misc.c
+===================================================================
+--- linux-2.6.16.19.orig/arch/cris/arch-v10/boot/compressed/misc.c	2006-06-23 17:06:33.000000000 -0700
++++ linux-2.6.16.19/arch/cris/arch-v10/boot/compressed/misc.c	2006-06-23 17:06:33.000000000 -0700
+@@ -1,14 +1,9 @@
+ /*
+  * misc.c
+  *
+- * $Id: misc.c,v 1.6 2003/10/27 08:04:31 starvik Exp $
+- * 
+- * This is a collection of several routines from gzip-1.0.3 
+- * adapted for Linux.
+- *
+  * puts by Nick Holloway 1993, better puts by Martin Mares 1995
+  * adoptation for Linux/CRIS Axis Communications AB, 1999
+- * 
++ *
+  */
+ 
+ /* where the piggybacked kernel image expects itself to live.
+@@ -20,24 +15,17 @@
+ #define KERNEL_LOAD_ADR 0x40004000
+ 
+ #include <linux/config.h>
+-
+ #include <linux/types.h>
+ #include <asm/arch/svinto.h>
+ 
+-/*
+- * gzip declarations
+- */
+-
+-unsigned compsize; /* compressed size, used by head.S */
+-
++extern int end; /* the "heap" is put directly after the BSS ends, at end */
++extern unsigned compsize; /* compressed size, used by head.S */
+ extern char *input_data;  /* lives in head.S */
+ static u8 *output_data;
+ 
+ static void puts(const char *);
+ 
+-/* the "heap" is put directly after the BSS ends, at end */
+-  
+-extern int end;
++/* gzip declarations */
+ static long free_mem_ptr = (long)&end;
+ static long free_mem_end_ptr = 0xffffffff;
+ 
+Index: linux-2.6.16.19/arch/cris/arch-v32/boot/compressed/misc.c
+===================================================================
+--- linux-2.6.16.19.orig/arch/cris/arch-v32/boot/compressed/misc.c	2006-06-23 17:06:33.000000000 -0700
++++ linux-2.6.16.19/arch/cris/arch-v32/boot/compressed/misc.c	2006-06-23 17:06:33.000000000 -0700
+@@ -1,11 +1,6 @@
+ /*
+  * misc.c
+  *
+- * $Id: misc.c,v 1.8 2005/04/24 18:34:29 starvik Exp $
+- *
+- * This is a collection of several routines from gzip-1.0.3
+- * adapted for Linux.
+- *
+  * puts by Nick Holloway 1993, better puts by Martin Mares 1995
+  * adoptation for Linux/CRIS Axis Communications AB, 1999
+  *
+@@ -20,24 +15,18 @@
+ #define KERNEL_LOAD_ADR 0x40004000
+ 
+ #include <linux/config.h>
+-
+ #include <linux/types.h>
+ #include <asm/arch/hwregs/reg_rdwr.h>
+ #include <asm/arch/hwregs/reg_map.h>
+ #include <asm/arch/hwregs/ser_defs.h>
+ 
+-/*
+- * gzip declarations
+- */
+-
++extern int _end; /* the "heap" is put directly after the BSS ends, at end */
+ extern char *input_data;  /* lives in head.S */
+ static u8 *output_data;
+ 
+ static void puts(const char *);
+ 
+-/* the "heap" is put directly after the BSS ends, at end */
+-
+-extern int _end;
++/* gzip declarations */
+ static long free_mem_ptr = (long)&_end;
+ static long free_mem_end_ptr = 0xffffffff;
+ 
+Index: linux-2.6.16.19/arch/i386/boot/compressed/misc.c
+===================================================================
+--- linux-2.6.16.19.orig/arch/i386/boot/compressed/misc.c	2006-06-23 17:06:33.000000000 -0700
++++ linux-2.6.16.19/arch/i386/boot/compressed/misc.c	2006-06-23 17:06:33.000000000 -0700
+@@ -1,8 +1,5 @@
+ /*
+  * misc.c
+- * 
+- * This is a collection of several routines from gzip-1.0.3 
+- * adapted for Linux.
+  *
+  * puts by Nick Holloway 1993, better puts by Martin Mares 1995
+  * High loaded stuff by Hans Lermen & Werner Almesberger, Feb. 1996
+@@ -15,16 +12,6 @@
+ #include <asm/page.h>
+ 
+ /*
+- * gzip declarations
+- */
+-
+-/*
+- * Why do we do this? Don't ask me..
+- *
+- * Incomprehensible are the ways of bootloaders.
+- */
+-
+-/*
+  * This is set up by the setup-routine at boot-time
+  */
+ static unsigned char *real_mode; /* Pointer to real-mode data */
+@@ -35,6 +22,7 @@
+ #endif
+ #define RM_SCREEN_INFO (*(struct screen_info *)(real_mode+0))
+ 
++extern int end;
+ extern char input_data[];
+ extern int input_len;
+ static long bytes_out;
+@@ -42,10 +30,6 @@
+ 
+ static void putstr(const char *);
+ 
+-extern int end;
+-static long free_mem_ptr = (long)&end;
+-static long free_mem_end_ptr;
+-
+ #define INPLACE_MOVE_ROUTINE  0x1000
+ #define LOW_BUFFER_START      0x2000
+ #define LOW_BUFFER_MAX       0x90000
+@@ -62,6 +46,10 @@
+ static void * xquad_portio = NULL;
+ #endif
+ 
++/* gzip declarations */
++static long free_mem_ptr = (long)&end;
++static long free_mem_end_ptr;
++
+ #include "../../../../lib/inflate.c"
+ 
+ static void scroll(void)
+Index: linux-2.6.16.19/arch/m32r/boot/compressed/misc.c
+===================================================================
+--- linux-2.6.16.19.orig/arch/m32r/boot/compressed/misc.c	2006-06-23 17:06:33.000000000 -0700
++++ linux-2.6.16.19/arch/m32r/boot/compressed/misc.c	2006-06-23 17:06:33.000000000 -0700
+@@ -1,9 +1,6 @@
+ /*
+  * arch/m32r/boot/compressed/misc.c
+  *
+- * This is a collection of several routines from gzip-1.0.3
+- * adapted for Linux.
+- *
+  * Adapted for SH by Stuart Menefy, Aug 1999
+  *
+  * 2003-02-12:	Support M32R by Takeo Takahashi
+@@ -13,21 +10,18 @@
+ #include <linux/config.h>
+ #include <linux/string.h>
+ 
+-/*
+- * gzip declarations
+- */
+-
+ static unsigned char *input_data;
+ static int input_len;
+ static u8 *output_data;
+ 
+ #include "m32r_sio.c"
+ 
++#define HEAP_SIZE             0x10000
++
++/* gzip declarations */
+ static unsigned long free_mem_ptr;
+ static unsigned long free_mem_end_ptr;
+ 
+-#define HEAP_SIZE             0x10000
+-
+ #include "../../../../lib/inflate.c"
+ 
+ /* flush the gunzip output buffer */
+Index: linux-2.6.16.19/arch/sh/boot/compressed/misc.c
+===================================================================
+--- linux-2.6.16.19.orig/arch/sh/boot/compressed/misc.c	2006-06-23 17:06:33.000000000 -0700
++++ linux-2.6.16.19/arch/sh/boot/compressed/misc.c	2006-06-23 17:06:33.000000000 -0700
+@@ -1,9 +1,6 @@
+ /*
+  * arch/sh/boot/compressed/misc.c
+  *
+- * This is a collection of several routines from gzip-1.0.3
+- * adapted for Linux.
+- *
+  * Adapted for SH by Stuart Menefy, Aug 1999
+  *
+  * Modified to use standard LinuxSH BIOS by Greg Banks 7Jul2000
+@@ -15,23 +12,20 @@
+ #include <asm/sh_bios.h>
+ #endif
+ 
+-/*
+- * gzip declarations
+- */
+-
++extern int _text;		/* Defined in vmlinux.lds.S */
++extern int _end;
+ extern char input_data[];
+ extern int input_len;
+ static u8 *output_data;
+ 
+ int puts(const char *);
+ 
+-extern int _text;		/* Defined in vmlinux.lds.S */
+-extern int _end;
++#define HEAP_SIZE             0x10000
++
++/* gzip declarations */
+ static unsigned long free_mem_ptr;
+ static unsigned long free_mem_end_ptr;
+ 
+-#define HEAP_SIZE             0x10000
+-
+ #include "../../../../lib/inflate.c"
+ 
+ #ifdef CONFIG_SH_STANDARD_BIOS
+Index: linux-2.6.16.19/arch/sh64/boot/compressed/misc.c
+===================================================================
+--- linux-2.6.16.19.orig/arch/sh64/boot/compressed/misc.c	2006-06-23 17:06:33.000000000 -0700
++++ linux-2.6.16.19/arch/sh64/boot/compressed/misc.c	2006-06-23 17:06:33.000000000 -0700
+@@ -1,9 +1,6 @@
+ /*
+  * arch/shmedia/boot/compressed/misc.c
+  *
+- * This is a collection of several routines from gzip-1.0.3
+- * adapted for Linux.
+- *
+  * Adapted for SHmedia from sh by Stuart Menefy, May 2002
+  */
+ 
+@@ -15,23 +12,20 @@
+ #define CACHE_DISABLE     1
+ int cache_control(unsigned int command);
+ 
+-/*
+- * gzip declarations
+- */
+-
++extern int _text;		/* Defined in vmlinux.lds.S */
++extern int _end;
+ extern char input_data[];
+ extern int input_len;
+ static u8 *output_data;
+ 
+ static void puts(const char *);
+ 
+-extern int _text;		/* Defined in vmlinux.lds.S */
+-extern int _end;
++#define HEAP_SIZE             0x10000
++
++/* gzip declarations */
+ static unsigned long free_mem_ptr;
+ static unsigned long free_mem_end_ptr;
+ 
+-#define HEAP_SIZE             0x10000
+-
+ #include "../../../../lib/inflate.c"
+ 
+ void puts(const char *s)
+Index: linux-2.6.16.19/arch/x86_64/boot/compressed/misc.c
+===================================================================
+--- linux-2.6.16.19.orig/arch/x86_64/boot/compressed/misc.c	2006-06-23 17:06:33.000000000 -0700
++++ linux-2.6.16.19/arch/x86_64/boot/compressed/misc.c	2006-06-23 17:06:33.000000000 -0700
+@@ -1,8 +1,5 @@
+ /*
+  * misc.c
+- * 
+- * This is a collection of several routines from gzip-1.0.3 
+- * adapted for Linux.
+  *
+  * puts by Nick Holloway 1993, better puts by Martin Mares 1995
+  * High loaded stuff by Hans Lermen & Werner Almesberger, Feb. 1996
+@@ -14,10 +11,6 @@
+ #include <asm/page.h>
+ 
+ /*
+- * gzip declarations
+- */
+-
+-/*
+  * This is set up by the setup-routine at boot-time
+  */
+ static unsigned char *real_mode; /* Pointer to real-mode data */
+@@ -30,16 +23,11 @@
+ 
+ extern unsigned char input_data[];
+ extern int input_len;
+-
++extern int end;
+ static long bytes_out;
+ static u8 *output_data;
+-
+ static void putstr(const char *);
+ 
+-extern int end;
+-static long free_mem_ptr = (long)&end;
+-static long free_mem_end_ptr;
+-
+ #define INPLACE_MOVE_ROUTINE  0x1000
+ #define LOW_BUFFER_START      0x2000
+ #define LOW_BUFFER_MAX       0x90000
+@@ -52,6 +40,10 @@
+ static int vidport;
+ static int lines, cols;
+ 
++/* gzip declarations */
++static long free_mem_ptr = (long)&end;
++static long free_mem_end_ptr;
++
+ #include "../../../../lib/inflate.c"
+ 
+ static void scroll(void)
+Index: linux-2.6.16.19/init/do_mounts_rd.c
+===================================================================
+--- linux-2.6.16.19.orig/init/do_mounts_rd.c	2006-06-23 17:06:33.000000000 -0700
++++ linux-2.6.16.19/init/do_mounts_rd.c	2006-06-23 17:06:33.000000000 -0700
+@@ -269,9 +269,11 @@
+ 
+ #ifdef BUILD_CRAMDISK
+ 
+-/*
+- * gzip declarations
+- */
++/* gzip declarations */
++#define INIT __init
++#define NO_INFLATE_MALLOC
++
++#include "../lib/inflate.c"
+ 
+ #define INBUFSIZ 4096
+ static u8 *inbuf;
+@@ -279,12 +281,6 @@
+ static int unzip_error;
+ static int crd_infd, crd_outfd;
+ 
+-#define INIT __init
+-
+-#define NO_INFLATE_MALLOC
+-
+-#include "../lib/inflate.c"
+-
+ static void __init error(const char *x)
+ {
+ 	printk(KERN_ERR "%s\n", x);
+Index: linux-2.6.16.19/init/initramfs.c
+===================================================================
+--- linux-2.6.16.19.orig/init/initramfs.c	2006-06-23 17:06:33.000000000 -0700
++++ linux-2.6.16.19/init/initramfs.c	2006-06-23 17:06:33.000000000 -0700
+@@ -329,12 +329,9 @@
+ 	}
+ }
+ 
+-/*
+- * gzip declarations
+- */
++/* gzip declarations */
+ 
+ #define INIT __init
+-
+ #define NO_INFLATE_MALLOC
+ 
+ #include "../lib/inflate.c"
Index: packages/kernel/patches/tiny_m57sli/tiny/inetpeer.patch
===================================================================
--- packages/kernel/patches/tiny_m57sli/tiny/inetpeer.patch	(revision 0)
+++ packages/kernel/patches/tiny_m57sli/tiny/inetpeer.patch	(revision 0)
@@ -0,0 +1,96 @@
+
+configurable support for INET peer data tracking
+
+
+ include/net/inetpeer.h     |   10 	10 +	0 -	0 !
+ init/Kconfig               |    7 	7 +	0 -	0 !
+ net/ipv4/Makefile          |    3 	2 +	1 -	0 !
+ net/ipv4/sysctl_net_ipv4.c |    2 	2 +	0 -	0 !
+ 4 files changed, 21 insertions(+), 1 deletion(-)
+
+Index: linux-2.6.22.1/include/net/inetpeer.h
+===================================================================
+--- linux-2.6.22.1.orig/include/net/inetpeer.h
++++ linux-2.6.22.1/include/net/inetpeer.h
+@@ -31,6 +31,7 @@ struct inet_peer
+ 	unsigned long		tcp_ts_stamp;
+ };
+ 
++#ifdef CONFIG_INETPEER
+ void			inet_initpeers(void) __init;
+ 
+ /* can be called with or without local BH being disabled */
+@@ -51,5 +52,14 @@ static inline __u16	inet_getid(struct in
+ 	spin_unlock_bh(&inet_peer_idlock);
+ 	return id;
+ }
++#else
++
++#define inet_getpeer(a, b) (0)
++#define inet_putpeer(b)
++static void inline inet_initpeers(void) { }
++#define inet_getid(a, b) (0)
++
++#endif
++
+ 
+ #endif /* _NET_INETPEER_H */
+Index: linux-2.6.22.1/init/Kconfig
+===================================================================
+--- linux-2.6.22.1.orig/init/Kconfig
++++ linux-2.6.22.1/init/Kconfig
+@@ -381,6 +381,13 @@ config ETHTOOL
+ 	  Disabling this option removes support for configuring
+ 	  ethernet device features via ethtool. Saves about 6k.
+ 
++config INETPEER
++	default y
++	bool "Enable INET peer information storage" if EMBEDDED
++	help
++	  This allows storing various information about network peers.
++	  Disabling saves about 2k.
++
+ config MEASURE_INLINES
+ 	default n
+ 	bool "Enable inline measurement" if EMBEDDED
+Index: linux-2.6.22.1/net/ipv4/Makefile
+===================================================================
+--- linux-2.6.22.1.orig/net/ipv4/Makefile
++++ linux-2.6.22.1/net/ipv4/Makefile
+@@ -2,7 +2,7 @@
+ # Makefile for the Linux TCP/IP (INET) layer.
+ #
+ 
+-obj-y     := route.o inetpeer.o protocol.o \
++obj-y     := route.o protocol.o \
+ 	     ip_input.o ip_fragment.o ip_forward.o ip_options.o \
+ 	     ip_output.o ip_sockglue.o inet_hashtables.o \
+ 	     inet_timewait_sock.o inet_connection_sock.o \
+@@ -14,6 +14,7 @@ obj-y     := route.o inetpeer.o protocol
+ 
+ obj-$(CONFIG_IP_FIB_HASH) += fib_hash.o
+ obj-$(CONFIG_IP_FIB_TRIE) += fib_trie.o
++obj-$(CONFIG_INETPEER) += inetpeer.o
+ obj-$(CONFIG_PROC_FS) += proc.o
+ obj-$(CONFIG_IP_MULTIPLE_TABLES) += fib_rules.o
+ obj-$(CONFIG_IP_MROUTE) += ipmr.o
+Index: linux-2.6.22.1/net/ipv4/sysctl_net_ipv4.c
+===================================================================
+--- linux-2.6.22.1.orig/net/ipv4/sysctl_net_ipv4.c
++++ linux-2.6.22.1/net/ipv4/sysctl_net_ipv4.c
+@@ -482,6 +482,7 @@ ctl_table ipv4_table[] = {
+ 	},
+ 
+ #endif
++#ifdef CONFIG_INETPEER
+ 	{
+ 		.ctl_name	= NET_IPV4_IGMP_MAX_MSF,
+ 		.procname	= "igmp_max_msf",
+@@ -534,6 +535,7 @@ ctl_table ipv4_table[] = {
+ 		.proc_handler	= &proc_dointvec_jiffies,
+ 		.strategy	= &sysctl_jiffies
+ 	},
++#endif
+ 	{
+ 		.ctl_name	= NET_TCP_ORPHAN_RETRIES,
+ 		.procname	= "tcp_orphan_retries",
Index: packages/kernel/patches/tiny_m57sli/tiny/inflate-initdata.patch
===================================================================
--- packages/kernel/patches/tiny_m57sli/tiny/inflate-initdata.patch	(revision 0)
+++ packages/kernel/patches/tiny_m57sli/tiny/inflate-initdata.patch	(revision 0)
@@ -0,0 +1,90 @@
+inflate: mark some arrays as INITDATA and define it in in-core callers
+
+Signed-off-by: Matt Mackall <mpm@selenic.com>
+
+Index: linux-2.6.16.19/init/do_mounts_rd.c
+===================================================================
+--- linux-2.6.16.19.orig/init/do_mounts_rd.c	2006-06-23 17:06:33.000000000 -0700
++++ linux-2.6.16.19/init/do_mounts_rd.c	2006-06-23 17:06:36.000000000 -0700
+@@ -271,6 +271,7 @@
+ 
+ /* gzip declarations */
+ #define INIT __init
++#define INITDATA __initdata
+ #define NO_INFLATE_MALLOC
+ 
+ #include "../lib/inflate.c"
+Index: linux-2.6.16.19/init/initramfs.c
+===================================================================
+--- linux-2.6.16.19.orig/init/initramfs.c	2006-06-23 17:06:33.000000000 -0700
++++ linux-2.6.16.19/init/initramfs.c	2006-06-23 17:06:36.000000000 -0700
+@@ -332,6 +332,7 @@
+ /* gzip declarations */
+ 
+ #define INIT __init
++#define INITDATA __initdata
+ #define NO_INFLATE_MALLOC
+ 
+ #include "../lib/inflate.c"
+Index: linux-2.6.16.19/lib/inflate.c
+===================================================================
+--- linux-2.6.16.19.orig/lib/inflate.c	2006-06-23 17:06:36.000000000 -0700
++++ linux-2.6.16.19/lib/inflate.c	2006-06-23 17:06:36.000000000 -0700
+@@ -104,6 +104,9 @@
+ #ifndef INIT
+ #define INIT
+ #endif
++#ifndef INITDATA
++#define INITDATA
++#endif
+ 
+ #include <asm/types.h>
+ 
+@@ -144,7 +147,7 @@
+ 		malloc_ptr = free_mem_ptr;
+ }
+ 
+-static u8 window[0x8000]; /* use a statically allocated window */
++static u8 INITDATA window[0x8000]; /* use a statically allocated window */
+ #else
+ static u8 *window; /* dynamically allocate */
+ #define malloc(a) kmalloc(a, GFP_KERNEL)
+@@ -236,12 +239,12 @@
+ /* Tables for deflate from PKZIP's appnote.txt. */
+ 
+ /* Order of the bit length code lengths */
+-static const unsigned border[] = {
++static INITDATA unsigned border[] = {
+ 	16, 17, 18, 0, 8, 7, 9, 6, 10, 5, 11, 4, 12, 3, 13, 2, 14, 1, 15
+ };
+ 
+ /* Copy lengths for literal codes 257..285 */
+-static const u16 cplens[] = {
++static INITDATA u16 cplens[] = {
+ 	3, 4, 5, 6, 7, 8, 9, 10, 11, 13, 15, 17, 19, 23, 27, 31,
+ 	35, 43, 51, 59, 67, 83, 99, 115, 131, 163, 195, 227, 258, 0, 0
+ };
+@@ -250,20 +253,20 @@
+  * note: see note #13 above about the 258 in this list.
+  * 99==invalid
+  */
+-static const u16 cplext[] = {
++static INITDATA u16 cplext[] = {
+ 	0, 0, 0, 0, 0, 0, 0, 0, 1, 1, 1, 1, 2, 2, 2, 2,
+ 	3, 3, 3, 3, 4, 4, 4, 4, 5, 5, 5, 5, 0, 99, 99
+ };
+ 
+ /* Copy offsets for distance codes 0..29 */
+-static const u16 cpdist[] = {
++static INITDATA u16 cpdist[] = {
+ 	1, 2, 3, 4, 5, 7, 9, 13, 17, 25, 33, 49, 65, 97, 129, 193,
+ 	257, 385, 513, 769, 1025, 1537, 2049, 3073, 4097, 6145,
+ 	8193, 12289, 16385, 24577
+ };
+ 
+ /* Extra bits for distance codes */
+-static const u16 cpdext[] = {
++static INITDATA u16 cpdext[] = {
+ 	0, 0, 0, 0, 1, 1, 2, 2, 3, 3, 4, 4, 5, 5, 6, 6,
+ 	7, 7, 8, 8, 9, 9, 10, 10, 11, 11,
+ 	12, 12, 13, 13
Index: packages/kernel/patches/tiny_m57sli/tiny/flow-cache-small.patch
===================================================================
--- packages/kernel/patches/tiny_m57sli/tiny/flow-cache-small.patch	(revision 0)
+++ packages/kernel/patches/tiny_m57sli/tiny/flow-cache-small.patch	(revision 0)
@@ -0,0 +1,23 @@
+
+CONFIG_NET_SMALL flow cache hash sizing
+
+
+ tiny-mpm/net/core/flow.c |    4 ++++
+ 1 files changed, 4 insertions(+)
+
+Index: new-linux/net/core/flow.c
+===================================================================
+--- new-linux.orig/net/core/flow.c	2007-04-27 10:51:05.000000000 -0700
++++ new-linux/net/core/flow.c	2007-04-30 16:48:22.000000000 -0700
+@@ -351,7 +351,11 @@
+ 					sizeof(struct flow_cache_entry),
+ 					0, SLAB_HWCACHE_ALIGN|SLAB_PANIC,
+ 					NULL, NULL);
++#ifdef CONFIG_NET_SMALL
++	flow_hash_shift = 3;
++#else
+ 	flow_hash_shift = 10;
++#endif
+ 	flow_lwm = 2 * flow_hash_size;
+ 	flow_hwm = 4 * flow_hash_size;
+ 
Index: packages/kernel/patches/tiny_m57sli/tiny/inflate-input.patch
===================================================================
--- packages/kernel/patches/tiny_m57sli/tiny/inflate-input.patch	(revision 0)
+++ packages/kernel/patches/tiny_m57sli/tiny/inflate-input.patch	(revision 0)
@@ -0,0 +1,439 @@
+
+inflate  cleanup input logic
+
+Transform ugly macros to inlines
+Kill mask_bits table
+Eliminate magic underrun handling (dealt with by getbyte())
+
+
+ tiny-mpm/lib/inflate.c |  249 ++++++++++++++++++-------------------------------
+ 1 files changed, 96 insertions(+), 153 deletions(-)
+
+Index: linux-2.6.16.19/lib/inflate.c
+===================================================================
+--- linux-2.6.16.19.orig/lib/inflate.c	2006-06-23 17:06:29.000000000 -0700
++++ linux-2.6.16.19/lib/inflate.c	2006-06-23 17:06:29.000000000 -0700
+@@ -183,24 +183,23 @@
+ 	12, 12, 13, 13
+ };
+ 
+-/* Macros for inflate() bit peeking and grabbing.
++/* Inlines for inflate() bit peeking and grabbing.
+    The usage is:
+ 
+-        NEEDBITS(j)
+-        x = b & mask_bits[j];
+-        DUMPBITS(j)
+-
+-   where NEEDBITS makes sure that b has at least j bits in it, and
+-   DUMPBITS removes the bits from b. The macros use the variable k for
+-   the number of bits in b. Normally, b and k are initialized at the
+-   beginning of a routine that uses these macros from a global bit
+-   buffer and count.
++        x = readbits(&b, &k, j);
++	dumpbits(&b, &k, j);
++
++	x = pullbits(&b, &k, j);
++
++   where readbits makes sure that b has at least j bits in it, and
++   dumpbits removes the bits from b, while k tracks the number of bits
++   in b.
+ 
+    If we assume that EOB will be the longest code, then we will never
+-   ask for bits with NEEDBITS that are beyond the end of the stream.
+-   So, NEEDBITS should not read any more bytes than are needed to
+-   meet the request.  Then no bytes need to be "returned" to the buffer
+-   at the end of the last block.
++   ask for bits that are beyond the end of the stream. So, readbits
++   should not read any more bytes than are needed to meet the request.
++   Then no bytes need to be "returned" to the buffer at the end of the
++   last block.
+ 
+    However, this assumption is not true for fixed blocks--the EOB code
+    is 7 bits, but the other literal/length codes can be 8 or 9 bits.
+@@ -216,15 +215,25 @@
+ static u32 bb;			/* bit buffer */
+ static unsigned bk;		/* bits in bit buffer */
+ 
+-static const u16 mask_bits[] = {
+-	0x0000,
+-	0x0001, 0x0003, 0x0007, 0x000f, 0x001f, 0x003f, 0x007f, 0x00ff,
+-	0x01ff, 0x03ff, 0x07ff, 0x0fff, 0x1fff, 0x3fff, 0x7fff, 0xffff
+-};
++static inline u32 readbits(u32 *b, u32 *k, int n)
++{
++	for( ; *k < n; *k += 8)
++		*b |= (u32)get_byte() << *k;
++	return *b & ((1 << n) - 1);
++}
+ 
+-#define NEXTBYTE()  ({ int v = get_byte(); if (v < 0) goto underrun; (u8)v; })
+-#define NEEDBITS(n) do {while(k<(n)){b|=((u32)NEXTBYTE())<<k;k+=8;}} while(0)
+-#define DUMPBITS(n) do {b>>=(n);k-=(n);} while(0)
++static inline void dumpbits(u32 *b, u32 *k, int n)
++{
++	*b >>= n;
++	*k -= n;
++}
++
++static inline u32 pullbits(u32 *b, u32 *k, int n)
++{
++	u32 r = readbits(b, k, n);
++	dumpbits(b, k, n);
++	return r;
++}
+ 
+ /*
+    Huffman code decoding is performed using a multi-level table lookup.
+@@ -541,7 +550,6 @@
+ 	unsigned n, d;		/* length and index for copy */
+ 	unsigned w;		/* current window position */
+ 	struct huft *t;		/* pointer to table entry */
+-	unsigned ml, md;	/* masks for bl and bd bits */
+ 	u32 b;		/* bit buffer */
+ 	unsigned k;	/* number of bits in bit buffer */
+ 
+@@ -551,20 +559,17 @@
+ 	w = outcnt;			/* initialize window position */
+ 
+ 	/* inflate the coded data */
+-	ml = mask_bits[bl];	/* precompute masks for speed */
+-	md = mask_bits[bd];
+ 	for (;;) {		/* do until end of block */
+-		NEEDBITS((unsigned)bl);
+-		if ((e = (t = tl + ((unsigned)b & ml))->e) > 16)
+-			do {
+-				if (e == 99)
+-					return 1;
+-				DUMPBITS(t->b);
+-				e -= 16;
+-				NEEDBITS(e);
+-			} while ((e = (t = t->v.t + ((unsigned)b &
+-						     mask_bits[e]))->e) > 16);
+-		DUMPBITS(t->b);
++		t = tl + readbits(&b, &k, bl);
++		e = t->e;
++		while (e > 16) {
++			if (e == 99)
++				return 1;
++			dumpbits(&b, &k, t->b);
++			t = t->v.t + readbits(&b, &k, e - 16);
++			e = t->e;
++		}
++		dumpbits(&b, &k, t->b);
+ 		if (e == 16) {	/* then it's a literal */
+ 			window[w++] = (u8)t->v.n;
+ 			if (w == WSIZE) {
+@@ -577,25 +582,21 @@
+ 				break;
+ 
+ 			/* get length of block to copy */
+-			NEEDBITS(e);
+-			n = t->v.n + ((unsigned)b & mask_bits[e]);
+-			DUMPBITS(e);
++			n = t->v.n + pullbits(&b, &k, e);
+ 
+ 			/* decode distance of block to copy */
+-			NEEDBITS((unsigned)bd);
+-			if ((e = (t = td + ((unsigned)b & md))->e) > 16)
+-				do {
+-					if (e == 99)
+-						return 1;
+-					DUMPBITS(t->b);
+-					e -= 16;
+-					NEEDBITS(e);
+-				} while ((e = (t = t->v.t + ((unsigned)b
+-					   & mask_bits[e]))->e) > 16);
+-			DUMPBITS(t->b);
+-			NEEDBITS(e);
+-			d = w - t->v.n - ((unsigned)b & mask_bits[e]);
+-			DUMPBITS(e)
++			t = td + readbits(&b, &k, bd);
++			e = t->e;
++			while (e > 16) {
++				if (e == 99)
++					return 1;
++				dumpbits(&b, &k, t->b);
++				t = t->v.t + readbits(&b, &k, e - 16);
++				e = t->e;
++			}
++			dumpbits(&b, &k, t->b);
++
++			d = w - t->v.n - pullbits(&b, &k, e);
+ 
+ 			/* do the copy */
+ 			do {
+@@ -628,9 +629,6 @@
+ 
+ 	/* done */
+ 	return 0;
+-
+-      underrun:
+-	return 4;		/* Input underrun */
+ }
+ 
+ /* inflate_stored - "decompress" an inflated type 0 (stored) block. */
+@@ -649,27 +647,20 @@
+ 	w = outcnt;			/* initialize window position */
+ 
+ 	/* go to byte boundary */
+-	n = k & 7;
+-	DUMPBITS(n);
++	dumpbits(&b, &k, k & 7);
+ 
+ 	/* get the length and its complement */
+-	NEEDBITS(16);
+-	n = ((unsigned)b & 0xffff);
+-	DUMPBITS(16);
+-	NEEDBITS(16);
+-	if (n != (unsigned)((~b) & 0xffff))
++	n = pullbits(&b, &k, 16);
++	if (n != (~pullbits(&b, &k, 16) & 0xffff))
+ 		return 1;	/* error in compressed data */
+-	DUMPBITS(16);
+ 
+ 	/* read and output the compressed data */
+ 	while (n--) {
+-		NEEDBITS(8);
+-		window[w++] = (u8)b;
++		window[w++] = (u8)get_byte();
+ 		if (w == WSIZE) {
+ 			flush_output(w);
+ 			w = 0;
+ 		}
+-		DUMPBITS(8);
+ 	}
+ 
+ 	/* restore the globals from the locals */
+@@ -679,9 +670,6 @@
+ 
+ 	DEBG(">");
+ 	return 0;
+-
+-      underrun:
+-	return 4;		/* Input underrun */
+ }
+ 
+ 
+@@ -748,7 +736,6 @@
+ 	int i;			/* temporary variables */
+ 	unsigned j;
+ 	unsigned l;		/* last length */
+-	unsigned m;		/* mask for bit lengths table */
+ 	unsigned n;		/* number of lengths to get */
+ 	struct huft *tl;	/* literal/length code table */
+ 	struct huft *td;	/* distance code table */
+@@ -768,26 +755,17 @@
+ 	k = bk;
+ 
+ 	/* read in table lengths */
+-	NEEDBITS(5);
+-	nl = 257 + ((unsigned)b & 0x1f); /* number of literal/length codes */
+-	DUMPBITS(5);
+-	NEEDBITS(5);
+-	nd = 1 + ((unsigned)b & 0x1f);	/* number of distance codes */
+-	DUMPBITS(5);
+-	NEEDBITS(4);
+-	nb = 4 + ((unsigned)b & 0xf);	/* number of bit length codes */
+-	DUMPBITS(4);
++	nl = 257 + pullbits(&b, &k, 5); /* number of literal/length codes */
++	nd = 1 + pullbits(&b, &k, 5); /* number of distance codes */
++	nb = 4 + pullbits(&b, &k, 4); /* number of bit length codes */
+ 	if (nl > 286 || nd > 30)
+ 		return 1;	/* bad lengths */
+ 
+ 	DEBG("dyn1 ");
+ 
+ 	/* read in bit-length-code lengths */
+-	for (j = 0; j < nb; j++) {
+-		NEEDBITS(3);
+-		ll[border[j]] = (unsigned)b & 7;
+-		DUMPBITS(3);
+-	}
++	for (j = 0; j < nb; j++)
++		ll[border[j]] = pullbits(&b, &k, 3);
+ 	for (; j < 19; j++)
+ 		ll[border[j]] = 0;
+ 
+@@ -805,36 +783,28 @@
+ 
+ 	/* read in literal and distance code lengths */
+ 	n = nl + nd;
+-	m = mask_bits[bl];
+ 	i = l = 0;
+ 	while ((unsigned)i < n) {
+-		NEEDBITS((unsigned)bl);
+-		j = (td = tl + ((unsigned)b & m))->b;
+-		DUMPBITS(j);
++		td = tl + readbits(&b, &k, bl);
++		dumpbits(&b, &k, td->b);
+ 		j = td->v.n;
+ 		if (j < 16)	/* length of code in bits (0..15) */
+ 			ll[i++] = l = j;	/* save last length in l */
+ 		else if (j == 16) {	/* repeat last length 3 to 6 times */
+-			NEEDBITS(2);
+-			j = 3 + ((unsigned)b & 3);
+-			DUMPBITS(2);
+-			    if ((unsigned)i + j > n)
++			j = 3 + pullbits(&b, &k, 2);
++			if ((unsigned)i + j > n)
+ 				return 1;
+ 			while (j--)
+ 				ll[i++] = l;
+ 		} else if (j == 17) {	/* 3 to 10 zero length codes */
+-			NEEDBITS(3);
+-			j = 3 + ((unsigned)b & 7);
+-			DUMPBITS(3);
++			j = 3 + pullbits(&b, &k, 3);
+ 			if ((unsigned)i + j > n)
+ 				return 1;
+ 			while (j--)
+ 				ll[i++] = 0;
+ 			l = 0;
+ 		} else {	/* j == 18: 11 to 138 zero length codes */
+-			NEEDBITS(7);
+-			j = 11 + ((unsigned)b & 0x7f);
+-			DUMPBITS(7);
++			j = 11 + pullbits(&b, &k, 7);
+ 			if ((unsigned)i + j > n)
+ 				return 1;
+ 			while (j--)
+@@ -892,9 +862,6 @@
+ 
+ 	DEBG(">");
+ 	return 0;
+-
+-      underrun:
+-	return 4;		/* Input underrun */
+ }
+ 
+ /* inflate_block - decompress a deflated block
+@@ -903,28 +870,11 @@
+ static int INIT inflate_block(int *e)
+ {
+ 	unsigned t;		/* block type */
+-	u32 b;		/* bit buffer */
+-	unsigned k;	/* number of bits in bit buffer */
+ 
+ 	DEBG("<blk");
+ 
+-	/* make local bit buffer */
+-	b = bb;
+-	k = bk;
+-
+-	/* read in last block bit */
+-	NEEDBITS(1);
+-	*e = (int)b & 1;
+-	DUMPBITS(1);
+-
+-	/* read in block type */
+-	NEEDBITS(2);
+-	t = (unsigned)b & 3;
+-	DUMPBITS(2);
+-
+-	/* restore the global bit buffer */
+-	bb = b;
+-	bk = k;
++	*e = pullbits(&bb, &bk, 1); /* read in last block bit */
++	t = pullbits(&bb, &bk, 2); /* read in block type */
+ 
+ 	/* inflate that block type */
+ 	if (t == 2)
+@@ -938,9 +888,6 @@
+ 
+ 	/* bad block type */
+ 	return 2;
+-
+-      underrun:
+-	return 4;		/* Input underrun */
+ }
+ 
+ /* inflate - decompress an inflated entry */
+@@ -1050,9 +997,9 @@
+ 	u32 orig_len = 0;	/* original uncompressed length */
+ 	int res;
+ 
+-	magic[0] = NEXTBYTE();
+-	magic[1] = NEXTBYTE();
+-	method = NEXTBYTE();
++	magic[0] = get_byte();
++	magic[1] = get_byte();
++	method = get_byte();
+ 
+ 	if (magic[0] != 037 || ((magic[1] != 0213) && (magic[1] != 0236))) {
+ 		error("bad gzip magic numbers");
+@@ -1078,29 +1025,29 @@
+ 		error("Input has invalid flags");
+ 		return -1;
+ 	}
+-	NEXTBYTE();		/* Get timestamp */
+-	NEXTBYTE();
+-	NEXTBYTE();
+-	NEXTBYTE();
++	get_byte();		/* Get timestamp */
++	get_byte();
++	get_byte();
++	get_byte();
+ 
+-	(void)NEXTBYTE();	/* Ignore extra flags for the moment */
+-	(void)NEXTBYTE();	/* Ignore OS type for the moment */
++	get_byte();	/* Ignore extra flags for the moment */
++	get_byte();	/* Ignore OS type for the moment */
+ 
+ 	if (flags & EXTRA_FIELD) {
+-		unsigned len = (unsigned)NEXTBYTE();
+-		len |= ((unsigned)NEXTBYTE()) << 8;
++		unsigned len = (unsigned)get_byte();
++		len |= ((unsigned)get_byte()) << 8;
+ 		while (len--)
+-			(void)NEXTBYTE();
++			get_byte();
+ 	}
+ 
+ 	/* Discard original file name if it was truncated */
+ 	if (flags & ORIG_NAME)
+-		while (NEXTBYTE())
++		while (get_byte())
+ 			;
+ 
+ 	/* Discard file comment if any */
+ 	if (flags & COMMENT)
+-		while (NEXTBYTE())
++		while (get_byte())
+ 			;
+ 
+ 	/* Decompress */
+@@ -1130,15 +1077,15 @@
+ 	/* crc32  (see algorithm.doc)
+ 	 * uncompressed input size modulo 2^32
+ 	 */
+-	orig_crc = (u32)NEXTBYTE();
+-	orig_crc |= (u32)NEXTBYTE() << 8;
+-	orig_crc |= (u32)NEXTBYTE() << 16;
+-	orig_crc |= (u32)NEXTBYTE() << 24;
+-
+-	orig_len = (u32)NEXTBYTE();
+-	orig_len |= (u32)NEXTBYTE() << 8;
+-	orig_len |= (u32)NEXTBYTE() << 16;
+-	orig_len |= (u32)NEXTBYTE() << 24;
++	orig_crc = (u32)get_byte();
++	orig_crc |= (u32)get_byte() << 8;
++	orig_crc |= (u32)get_byte() << 16;
++	orig_crc |= (u32)get_byte() << 24;
++
++	orig_len = (u32)get_byte();
++	orig_len |= (u32)get_byte() << 8;
++	orig_len |= (u32)get_byte() << 16;
++	orig_len |= (u32)get_byte() << 24;
+ 
+ 	/* Validate decompression */
+ 	if (orig_crc != CRC_VALUE) {
+@@ -1150,8 +1097,4 @@
+ 		return -1;
+ 	}
+ 	return 0;
+-
+-      underrun:		/* NEXTBYTE() goto's here if needed */
+-	error("out of input data");
+-	return -1;
+ }
Index: packages/kernel/patches/tiny_m57sli/series
===================================================================
--- packages/kernel/patches/tiny_m57sli/series	(revision 0)
+++ packages/kernel/patches/tiny_m57sli/series	(revision 0)
@@ -0,0 +1,144 @@
+###################################
+## Linux-tiny
+##
+## Following are patches for linux-tiny
+## 
+## stuff already in -mm ??
+##
+tiny/use-funit-at-a-time.patch 
+# add a kconfig option for the above
+tiny/use-funit-at-a-time-kconfig.patch
+####devlist.patch - too much is changed, bitrotted?
+#
+# tiny stuff
+# 
+# net
+tiny/config-net-small.patch 
+tiny/cache_defer_hash.patch 
+tiny/unix_socket_table.patch 
+tiny/inet_protos.patch 
+tiny/flow-cache-small.patch 
+#
+# Unconditional bits
+#
+tiny/namei-inlines.patch 
+tiny/buffer-inlines.patch 
+tiny/ext2namei-inlines.patch 
+tiny/inline-kconfig.patch
+# still have about 115 inlines in fs/*.c, in 2.6.22.1
+
+#
+# build options
+#
+####tiny-cflags.patch - changes too big, doesn't apply
+#
+# measurement bits
+#
+# omit next 2 for now
+#tiny/kmalloc-accounting.patch 
+#tiny/audit-bootmem.patch 
+tiny/deprecate-inline.patch 
+# this next patch is ALP-specific, and just moves some config entries
+# around in init/Kconfig (from O'hara-san feb 7, 2007)
+# already applied in 2.6.21
+#tiny/deprecate-inline-kconfig-move.patch
+tiny/func-size.patch 
+#
+# trimmed debugging support
+#
+# omit next 3 for now
+tiny/tiny-panic.patch 
+tiny/nopanic.patch 
+# the following fixes a compile bug on MIPS when PANIC is configured off
+tiny/nopanic-mips-fix.patch
+
+#
+# trimmed code
+#
+tiny/tiny-crc.patch 
+# omit next for now
+#tiny/threadinfo-ool.patch 
+### semaphore-inline.patch  bitrotted (16)
+# next patch conflicts with rt-preempt, omit for now
+#tiny/slob-accounting.patch 
+###tiny/slob-accounting-fix.patch - no longer needed (21)
+# omit next for now
+#tiny/mempool-shrink.patch 
+#tiny/no-translations.patch 
+###tinyvt.patch - omitted (16)
+#
+# trimmed apis
+#
+# ptrace.patch # broken by unified ptrace?
+tiny/sysenter.patch
+tiny/no-aio.patch 
+tiny/no-xattr.patch 
+tiny/fslock.patch 
+#direct-io-core.patch - too hard (16)
+# posix-timers.patch  bitrotted (16)
+tiny/ethtool.patch 
+tiny/inetpeer.patch 
+# TRB 2007.07.06 - disable next for 2.6.22-rc7
+#tiny/net-filter.patch 
+tiny/dev_mcast.patch 
+# omit next for now
+#tiny/igmp.patch 
+tiny/binfmt-script.patch 
+# omit next for now
+#tiny/elf-no-aout.patch 
+#
+# trimmed constants
+#
+tiny/max-swapfiles.patch 
+tiny/ldiscs.patch 
+tiny/max_user_rt_prio.patch 
+#
+# trimmed hw support
+#
+# omit next 8 for now
+#tiny/ide-hwif.patch 
+#tiny/sbf.patch 
+#tiny/serial-pci.patch 
+#tiny/dmi_blacklist.patch 
+#tiny/pci-quirks.patch 
+#tiny/tsc.patch 
+#tiny/cpu-support.patch 
+#tiny/mtrr.patch 
+#
+# inflate cleanups
+#
+# TRB 2006-07-12 - disable these inflate patches
+# Even though I fixed a few problems with these patches, they still don't work 100%
+# These patches caused problems compiling with MODVERSIONS=y on i386 and ARM
+# Also, I could not boot the kernel with these patches applied on ARM (OSK)
+#tiny/inflate-formatting.patch 
+#tiny/inflate-legacy.patch 
+#tiny/inflate-input.patch 
+#tiny/inflate-iostate.patch 
+#tiny/inflate-huft.patch 
+#tiny/inflate-crc.patch 
+#tiny/inflate-killmemzero.patch 
+#tiny/inflate-killdecls.patch 
+#tiny/inflate-killmarkrelease.patch 
+#tiny/inflate-makecrc.patch 
+#tiny/inflate-readbyte.patch 
+#tiny/inflate-flush.patch 
+#tiny/inflate-killsillytypes.patch 
+#tiny/inflate-error.patch 
+#tiny/inflate-tidyusers.patch 
+#tiny/inflate-killdebug.patch 
+#tiny/inflate-initdata.patch 
+#tiny/inflate-constdef.patch 
+#tiny/inflate-linkage.patch 
+#tiny/inflate-share-crc.patch 
+#
+# unsorted
+#
+#tiny/inflate-roll-crc.patch 
+tiny/movsl-mask.patch 
+#
+###tiny-extraversion.patch 
+#
+# omit next for now
+tiny/do-printk.patch
+
